//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as
//  published by the Free Software Foundation; either version 2.1
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//  MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//

#include <config.h>

#if HAVE_OMNIORB

#ifdef __cplusplus

#include "g2mpls_corba_utils.h"

using namespace std;

#include <omniORB4/CORBA.h>

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <sstream>
#include <iomanip>
#include <stdexcept>
#include <arpa/inet.h>

//
// nodeId
//
g2mpls_addr_t &
operator << (g2mpls_addr_t &                 dst,
	     const g2mplsTypes::nodeId     & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.type              = IPv4;
	dst.preflen           = 32;
	dst.value.ipv4.s_addr = htonl(src);

	return dst;
}

g2mplsTypes::nodeId &
operator << (g2mplsTypes::nodeId &     dst,
	     const g2mpls_addr_t &     src)
{
	switch (src.type) {
		case IPv4: {
			dst = ntohl(src.value.ipv4.s_addr);
			break;
		}
		case IPv6:
		case UNNUMBERED:
		case NSAP:
		default:
			throw out_of_range("nodeId G2.ADDR type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                   dst,
	     const g2mplsTypes::nodeId &     src)
{
	std::ostringstream id;

	ipv4_t addr;
	addr.s_addr = htonl(src);

	id << "0x" << std::hex << std::setw(8) << std::setfill('0') << src;
	id << std::hex << " (IPv4 " << inet_ntoa(addr) << ")";

	dst = id.str();

	return dst;
}

//
// linkId
//
g2mpls_addr_t &
operator << (g2mpls_addr_t &                 dst,
	     const g2mplsTypes::linkId_var & src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes::LINKIDTYPE_IPV4: {
			dst.type              = IPv4;
			dst.preflen           = 32;
			dst.value.ipv4.s_addr = htonl(src->ipv4());
			break;
		}
		case g2mplsTypes::LINKIDTYPE_IPV6: {
			int    i;

			dst.type    = IPv6;
			dst.preflen = 128;

			for (i = 0; i < 4; i++) {
				dst.value.ipv6.s6_addr32[i] =
					htonl(src->ipv6()[i]);
			}
			break;
		}
		case g2mplsTypes::LINKIDTYPE_UNNUM: {
			dst.type                = UNNUMBERED;
			dst.preflen             = 0;
			//dst.value.unnum.addr    = src.unnum().addr;
			//dst.value.unnum.node_id = src.unnum().node;
			//dst.value.unnum.addr    = src->unnum();
			//dst.value.unnum.node_id = 0; // meaningless
			dst.value.unnum         = src->unnum();
			break;
		}
		default:
			throw out_of_range("linkId type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::linkId_var &
operator << (g2mplsTypes::linkId_var & dst,
	     const g2mpls_addr_t &     src)
{
	dst = new g2mplsTypes::linkId;

	switch (src.type) {
		case IPv4:
			dst->ipv4(ntohl(src.value.ipv4.s_addr));
			break;
		case IPv6: {
			g2mplsTypes::addrIPv6 parts;
			int i;

			for (i = 0; i < 4; i++) {
				parts[i] =
					ntohl(src.value.ipv6.s6_addr32[i]);
			}

			dst->ipv6(parts);
			break;
		}
		case UNNUMBERED: {
			//g2mplsTypes::addrUnnum unnId;
			//
			//unnId.node = src.value.unnum.node_id;
			//unnId.addr = src.value.unnum.addr;
			//dst.unnum(unnId);
			//dst->unnum(src.value.unnum.addr);
			dst->unnum(src.value.unnum);
			break;
		}
		default:
			throw out_of_range("link G2.ADDR type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                   dst,
	     const g2mplsTypes::linkId_var & src)
{
	std::ostringstream id;

	switch (src->_d()) {
		case g2mplsTypes::LINKIDTYPE_IPV4: {
			id << "(LINK-IPv4) " << numb_htoa(src->ipv4());
			break;
		}
		case g2mplsTypes::LINKIDTYPE_IPV6: {
			int    i;

			id << "(LINK-IPv6) " << std::hex;
			for (i = 0; i < 4; i++) {
				id << std::setw(8)
				   << std::setfill('0')
				   << src->ipv6()[i];
			}
			id << std::dec;
			break;
		}
		case g2mplsTypes::LINKIDTYPE_UNNUM: {
			id << "(LINK-UNN) ";
			//  id << "Node:0x"
			//     << std::hex
			//     << std::setw(8)
			//     << std::setfill('0')
			//     << src.unnum().node
			//     << std::dec;
			//  id << " / Id:0x"
			//     << std::hex
			//     << std::setw(8)
			//     << std::setfill('0')
			//     << src.unnum().addr
			//     << std::dec;
			id << "Id:0x"
			   << std::hex
			   << std::setw(8)
			   << std::setfill('0')
			   << src->unnum()
			   << std::dec;
			break;
		}
		default:
			throw out_of_range("linkId type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

bool
operator == (const g2mpls_addr_t &           src,
	     const g2mplsTypes::linkId_var & dst)
{
	switch (dst->_d()) {
		case g2mplsTypes::LINKIDTYPE_IPV4: {
			if (src.type != IPv4 ||
			    ntohl(src.value.ipv4.s_addr) != dst->ipv4()) {
				return false;
			}
			break;
		}
		case g2mplsTypes::LINKIDTYPE_IPV6: {
			int i;

			if (src.type != IPv6) {
				return false;
			}

			for (i = 0; i < 4; i++) {
				if (htonl(src.value.ipv6.s6_addr32[i]) !=
				    dst->ipv6()[i]) {
					return false;
				}
			}
			break;
		}
		case g2mplsTypes::LINKIDTYPE_UNNUM: {
			//  if (src.type                != UNNUMBERED       ||
			//      src.value.unnum.node_id != dst.unnum().node ||
			//      src.value.unnum.addr    != dst.unnum().addr) {
			//      return false;
			//  }
			// if (src.type                != UNNUMBERED ||
			//     src.value.unnum.addr    != dst->unnum()) {
			//     return false;
			// }
			if (src.type        != UNNUMBERED ||
			    src.value.unnum != dst->unnum()) {
				return false;
			}
			break;
		}
		default:
			return false;
	}
	return true;
}

//
// labelId
//
label_id_t &
operator<< (label_id_t &                     dst,
	    const g2mplsTypes::labelId_var & src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes:: LABELTYPE_L32: {
			dst.type = LABEL_32BIT;
			dst.value.label32.id = src->label32();
			break;
		}
		case g2mplsTypes:: LABELTYPE_L60: {
			uint64_t l60v, l60m;
			int      i;

			l60v = src->label60() & 0x0FFFFFFFFFFFFFFFull;

			dst.type = LABEL_60BIT;
			for (i = 0; i < 6; i++) {
				int disp = ((5 - i) * 8 + 12);
				l60m = (0xFFull) << disp;
				dst.value.label60.mac[i] =
					(uint8_t) ((l60v & l60m) >> disp);
			}
			dst.value.label60.vlan_id =
				(l60v & 0x0000000000000FFFull);
			break;
		}
		default:
			throw out_of_range("LABEL ID type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::labelId_var &
operator<< (g2mplsTypes::labelId_var & dst,
	    const label_id_t &         src)
{
	dst = new g2mplsTypes::labelId;

	switch (src.type) {
		case LABEL_32BIT: {
			dst->label32(src.value.label32.id);
			break;
		}
		case LABEL_60BIT: {
			uint64_t l60v;
			int i;

			l60v = 0x0ull;
			for (i = 0; i < 6; i++) {
				int disp = ((5 - i) * 8 + 12);
				l60v |= src.value.label60.mac[i] << disp;
			}
			l60v |= src.value.label60.vlan_id;

			dst->label60(l60v);
			break;
		}
		default:
			throw out_of_range("G2.LABEL type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator<< (std::string &                    dst,
	    const g2mplsTypes::labelId_var & src)
{
	std::ostringstream id;

	switch (src->_d()) {
		case g2mplsTypes:: LABELTYPE_L32: {
			id << "(LABEL-32bit) - 0x"
			   << std::hex
			   << std::setw(8)
			   << std::setfill('0')
			   << src->label32()
			   << std::dec;
			break;
		}
		case g2mplsTypes:: LABELTYPE_L60: {
			id << "(LABEL-60bit) 0x"
			   << std::hex
			   << std::setw(16)
			   << std::setfill('0')
			   << src->label60()
			   << std::dec;
			break;
		}
		default:
			throw out_of_range("LABEL ID type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// xcDirection
//
xcdirection_t &
operator<< (xcdirection_t &                      dst,
	    const g2mplsTypes::xcDirection &     src)
{
	switch (src) {
		case g2mplsTypes::XCDIR_UNIDIRECTIONAL: {
			dst = XCDIRECTION_UNIDIR;
			break;
		}
		case g2mplsTypes::XCDIR_BIDIRECTIONAL: {
			dst = XCDIRECTION_BIDIR;
			break;
		}
		case g2mplsTypes::XCDIR_BCAST: {
			dst = XCDIRECTION_BCAST;
			break;
		}
		default:
			throw out_of_range("xcDirection type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::xcDirection &
operator<< (g2mplsTypes::xcDirection &           dst,
	    const xcdirection_t &                src)
{
	switch (src) {
		case XCDIRECTION_UNIDIR: {
			dst = g2mplsTypes::XCDIR_UNIDIRECTIONAL;
			break;
		}
		case XCDIRECTION_BIDIR: {
			dst = g2mplsTypes::XCDIR_BIDIRECTIONAL;
			break;
		}
		case XCDIRECTION_BCAST: {
			dst = g2mplsTypes::XCDIR_BCAST;
			break;
		}
		default:
			throw out_of_range("xcDirection type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::xcDirection &    src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::XCDIR_UNIDIRECTIONAL: {
			id << "XCDIR_UNIDIRECTIONAL";
			break;
		}
		case g2mplsTypes::XCDIR_BIDIRECTIONAL: {
			id << "XCDIR_BIDIRECTIONAL";
			break;
		}
		case g2mplsTypes::XCDIR_BCAST: {
			id << "XCDIR_BCAST";
			break;
		}
		default:
			throw out_of_range("xcDirection type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// tnrcResult
//
std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::tnrcResult &     src)
{
	std::ostringstream id;



	switch (src) {
		case g2mplsTypes::TNRC_RESULT_MAKEXC_NOERROR: {
			id << "MAKE XC NO ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_EQPTDOWN: {
			id << "MAKE XC EQPT DOWN";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_PARAMERROR: {
			id << "MAKE XC PARAM ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_NOTCAPABLE: {
			id << "MAKE XC NOT CAPABLE";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_BUSYRESOURCES: {
			id << "MAKE XC BUSY RESOURCES";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_INTERNALERROR: {
			id << "MAKE XC INTERNAL ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_GENERICERROR: {
			id << "MAKE XC GENERIC ERROR";
			break;
		}

		case g2mplsTypes::TNRC_RESULT_DESTROYXC_NOERROR: {
			id << "DESTROY XC NO ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_EQPTDOWN: {
			id << "DESTROY XC EQPT DOWN";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_PARAMERROR: {
			id << "DESTROY XC PARAM ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_NOTCAPABLE: {
			id << "DESTROY XC NOT CAPABLE";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_BUSYRESOURCES: {
			id << "DESTROY XC BUSY RESOURCES";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_INTERNALERROR: {
			id << "DESTROY XC INTERNAL ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_GENERICERROR: {
			id << "DESTROY XC GENERIC ERROR";
			break;
		}

		case g2mplsTypes::TNRC_RESULT_RESERVEXC_NOERROR: {
			id << "RESERVE XC NO ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_EQPTDOWN: {
			id << "RESERVE XC EQPT DOWN";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_PARAMERROR: {
			id << "RESERVE XC PARAM ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_NOTCAPABLE: {
			id << "RESERVE XC NOT CAPABLE";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_BUSYRESOURCES: {
			id << "RESERVE XC BUSY RESOURCES";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_INTERNALERROR: {
			id << "RESERVE XC INTERNAL ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_GENERICERROR: {
			id << "RESERVE XC GENERIC ERROR";
			break;
		}

		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_NOERROR: {
			id << "UNRESERVE XC NO ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_EQPTDOWN: {
			id << "UNRESERVE XC EQPT DOWN";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_PARAMERROR: {
			id << "UNRESERVE XC PARAM ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_NOTCAPABLE: {
			id << "UNRESERVE XC NOT CAPABLE";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_BUSYRESOURCES: {
			id << "UNRESERVE XC BUSY RESOURCES";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_INTERNALERROR: {
			id << "UNRESERVE XC INTERNAL ERROR";
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_GENERICERROR: {
			id << "UNRESERVE XC GENERIC ERROR";
			break;
		}
		default:
			throw out_of_range("tnrcResult type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// adjType
//

adj_type_t &
operator << (adj_type_t &                        dst,
	     const g2mplsTypes::adjType &        src)
{
	switch (src) {
		case g2mplsTypes::ADJTYPE_UNI: {
			dst = UNI;
			break;
		}
		case g2mplsTypes::ADJTYPE_INNI: {
			dst = INNI;
			break;
		}
		case g2mplsTypes::ADJTYPE_ENNI: {
		        dst = ENNI;
			break;
		}
		default:
			throw out_of_range("adjType type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::adjType &
operator << (g2mplsTypes::adjType &              dst,
	     const adj_type_t &                  src)
{
	switch (src) {
		case UNI: {
			dst = g2mplsTypes::ADJTYPE_UNI;
			break;
		}
		case INNI: {
			dst = g2mplsTypes::ADJTYPE_INNI;
			break;
		}
		case ENNI: {
		        dst = g2mplsTypes::ADJTYPE_ENNI;
			break;
		}
		default:
			throw out_of_range("adj_type_t type out-of-range");
			break;
	}

	return dst;
}

bool
operator == (const adj_type_t &                  dst,
	     const g2mplsTypes::adjType &        src)
{
	switch (src) {
		case g2mplsTypes::ADJTYPE_UNI: {
			if (dst != UNI) {
				return false;
			}
			break;
		}
		case g2mplsTypes::ADJTYPE_INNI: {
			if (dst != INNI) {
				return false;
			}
			break;
		}
		case g2mplsTypes::ADJTYPE_ENNI: {
			if (dst != ENNI) {
				return false;
			}
			break;
		}
		default:
			return false;
	}
	return true;
}

std::string &
operator << (std::string &                      dst,
	     const g2mplsTypes::adjType &       src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::ADJTYPE_INNI: {
			id << "INNI";
			break;
		}
		case g2mplsTypes::ADJTYPE_UNI: {
			id << "UNI";
			break;
		}
		case g2mplsTypes::ADJTYPE_ENNI: {
			id << "ENNI";
			break;
		}
		default:
			throw out_of_range("adjType type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// operState
//
opstate_t &
operator << (opstate_t &                        dst,
	     const g2mplsTypes::operState &     src)
{
	switch (src) {
		case g2mplsTypes::OPERSTATE_UP: {
			dst = UP;
			break;
		}
		case g2mplsTypes::OPERSTATE_DOWN: {
			dst = DOWN;
			break;
		}
		default:
			throw out_of_range("operState type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::operState &
operator << (g2mplsTypes::operState &     dst,
	     const opstate_t &            src)
{
	switch (src) {
		case UP: {
			dst = g2mplsTypes::OPERSTATE_UP;
			break;
		}
		case DOWN: {
			dst = g2mplsTypes::OPERSTATE_DOWN;
			break;
		}
		default:
			throw out_of_range("opstate_t type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                      dst,
	     const g2mplsTypes::operState &     src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::OPERSTATE_UP: {
			id << "UP";
			break;
		}
		case g2mplsTypes::OPERSTATE_DOWN: {
			id << "DOWN";
			break;
		}
		default:
			throw out_of_range("operState type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// adminState
//
admstate_t &
operator << (admstate_t &                        dst,
	     const g2mplsTypes::adminState &     src)
{
	switch (src) {
		case g2mplsTypes::ADMINSTATE_ENABLED: {
			dst = ENABLED;
			break;
		}
		case g2mplsTypes::ADMINSTATE_DISABLED: {
			dst = DISABLED;
			break;
		}
		default:
			throw out_of_range("adminState type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::adminState &
operator << (g2mplsTypes::adminState &     dst,
	     const admstate_t &            src)
{
	switch (src) {
		case ENABLED: {
			dst = g2mplsTypes::ADMINSTATE_ENABLED;
			break;
		}
		case DISABLED: {
			dst = g2mplsTypes::ADMINSTATE_DISABLED;
			break;
		}
		default:
			throw out_of_range("admstate_t type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::adminState &     src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::ADMINSTATE_ENABLED: {
			id << "ENABLED";
			break;
		}
		case g2mplsTypes::ADMINSTATE_DISABLED: {
			id << "DISABLED";
			break;
		}
		default:
			throw out_of_range("adminState type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// switchingCap
//

sw_cap_t &
operator << (sw_cap_t &                          dst,
	     const g2mplsTypes::switchingCap &   src)
{
	switch (src) {
		case g2mplsTypes::SWITCHINGCAP_PSC_1: {
			dst = SWCAP_PSC_1;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_PSC_2: {
			dst = SWCAP_PSC_2;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_PSC_3: {
			dst = SWCAP_PSC_3;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_PSC_4: {
			dst = SWCAP_PSC_4;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_L2SC: {
			dst = SWCAP_L2SC;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_TDM: {
			dst = SWCAP_TDM;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_LSC: {
			dst = SWCAP_LSC;
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_FSC: {
			dst = SWCAP_FSC;
			break;
		}
		default:
			throw out_of_range("switchingCap type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::switchingCap &
operator << (g2mplsTypes::switchingCap &         dst,
	     const sw_cap_t  &                   src)
{
	switch (src) {
		case SWCAP_PSC_1: {
			dst = g2mplsTypes::SWITCHINGCAP_PSC_1;
			break;
		}
		case SWCAP_PSC_2: {
			dst = g2mplsTypes::SWITCHINGCAP_PSC_2;
			break;
		}
		case SWCAP_PSC_3: {
			dst = g2mplsTypes::SWITCHINGCAP_PSC_3;
			break;
		}
		case SWCAP_PSC_4: {
			dst =g2mplsTypes::SWITCHINGCAP_PSC_4;
			break;
		}
		case SWCAP_L2SC: {
			dst = g2mplsTypes::SWITCHINGCAP_L2SC;
			break;
		}
		case SWCAP_TDM: {
			dst = g2mplsTypes::SWITCHINGCAP_TDM;
			break;
		}
		case SWCAP_LSC: {
			dst = g2mplsTypes::SWITCHINGCAP_LSC;
			break;
		}
		case SWCAP_FSC: {
			dst = g2mplsTypes::SWITCHINGCAP_FSC;
			break;
		}
		default:
			throw out_of_range("switchingCap type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::switchingCap &   src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::SWITCHINGCAP_PSC_1: {
			id << "PSC_1";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_PSC_2: {
			id << "PSC_2";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_PSC_3: {
			id << "PSC_3";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_PSC_4: {
			id << "PSC_4";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_L2SC: {
			id << "L2SC";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_TDM: {
			id << "TDM";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_LSC: {
			id << "LSC";
			break;
		}
		case g2mplsTypes::SWITCHINGCAP_FSC: {
			id << "FSC";
			break;
		}
		default:
			throw out_of_range("switchingCap type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// encodingType
//

enc_type_t &
operator << (enc_type_t &                        dst,
	     const g2mplsTypes::encodingType &   src)
{
	switch (src) {
		case g2mplsTypes::ENCODINGTYPE_PACKET: {
			dst = ENCT_PACKET;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_ETHERNET: {
			dst = ENCT_ETHERNET;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH: {
			dst = ENCT_ANSI_ETSI_PDH;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_RESERVED_1: {
			dst = ENCT_RESERVED_1;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_SDH_SONET: {
			dst = ENCT_SDH_SONET;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_RESERVED_2: {
			dst = ENCT_RESERVED_2;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER: {
			dst = ENCT_DIGITAL_WRAPPER;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_LAMBDA: {
			dst = ENCT_LAMBDA;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_FIBER: {
			dst = ENCT_FIBER;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_RESERVED_3: {
			dst = ENCT_RESERVED_3;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL: {
			dst = ENCT_FIBERCHANNEL;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_G709_ODU: {
			dst = ENCT_G709_ODU;
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_G709_OC: {
			dst = ENCT_G709_OC;
			break;
		}
	}

	return dst;
}

g2mplsTypes::encodingType &
operator << (g2mplsTypes::encodingType &         dst,
	     const enc_type_t &                  src)
{
	switch (src) {
		case ENCT_PACKET: {
			dst = g2mplsTypes::ENCODINGTYPE_PACKET;
			break;
		}
		case ENCT_ETHERNET: {
			dst = g2mplsTypes::ENCODINGTYPE_ETHERNET;
			break;
		}
		case ENCT_ANSI_ETSI_PDH: {
			dst = g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH;
			break;
		}
		case ENCT_RESERVED_1: {
			dst = g2mplsTypes::ENCODINGTYPE_RESERVED_1;
			break;
		}
		case ENCT_SDH_SONET: {
			dst = g2mplsTypes::ENCODINGTYPE_SDH_SONET;
			break;
		}
		case ENCT_RESERVED_2: {
			dst = g2mplsTypes::ENCODINGTYPE_RESERVED_2;
			break;
		}
		case ENCT_DIGITAL_WRAPPER: {
			dst = g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER;
			break;
		}
		case ENCT_LAMBDA: {
			dst = g2mplsTypes::ENCODINGTYPE_LAMBDA;
			break;
		}
		case ENCT_FIBER: {
			dst = g2mplsTypes::ENCODINGTYPE_FIBER;
			break;
		}
		case ENCT_RESERVED_3: {
			dst = g2mplsTypes::ENCODINGTYPE_RESERVED_3;
			break;
		}
		case ENCT_FIBERCHANNEL: {
			dst = g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL;
			break;
		}
		case ENCT_G709_ODU: {
			dst = g2mplsTypes::ENCODINGTYPE_G709_ODU;
			break;
		}
		case ENCT_G709_OC: {
			dst = g2mplsTypes::ENCODINGTYPE_G709_OC;
			break;
		}
	}

	return dst;
}

std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::encodingType &   src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::ENCODINGTYPE_PACKET: {
			id << "PACKET";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_ETHERNET: {
			id << "ETHERNET";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH: {
			id << "ANSI_ETSI_PDH";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_RESERVED_1: {
			id << "RESERVED_1";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_SDH_SONET: {
			id << "SDH_SONET";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_RESERVED_2: {
			id << "RESERVED_2";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER: {
			id << "DIGITAL_WRAPPER";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_LAMBDA: {
			id << "LAMBDA";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_FIBER: {
			id << "FIBER";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_RESERVED_3: {
			id << "RESERVED_3";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL: {
			id << "FIBERCHANNEL";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_G709_ODU: {
			id << "G709_ODU";
			break;
		}
		case g2mplsTypes::ENCODINGTYPE_G709_OC: {
			id << "G709_OC";
			break;
		}
	}

	dst = id.str();

	return dst;
}

//
// labelState
//
label_state_t &
operator << (label_state_t &                     dst,
	     const g2mplsTypes::labelState &     src)
{
	switch (src) {
		case g2mplsTypes::LABELSTATE_FREE: {
			dst = LABEL_FREE;
			break;
		}
		case g2mplsTypes::LABELSTATE_BOOKED: {
			dst = LABEL_BOOKED;
			break;
		}
		case g2mplsTypes::LABELSTATE_XCONNECTED: {
			dst = LABEL_XCONNECTED;
			break;
		}
		case g2mplsTypes::LABELSTATE_BUSY: {
			dst = LABEL_BUSY;
			break;
		}
		default:
			throw out_of_range("labelState type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::labelState &
operator << (g2mplsTypes::labelState &     dst,
	     const label_state_t &         src)
{
	switch (src) {
		case LABEL_FREE: {
			dst = g2mplsTypes::LABELSTATE_FREE;
			break;
		}
		case LABEL_BOOKED: {
			dst = g2mplsTypes::LABELSTATE_BOOKED;
			break;
		}
		case LABEL_XCONNECTED: {
			dst = g2mplsTypes::LABELSTATE_XCONNECTED;
			break;
		}
		case LABEL_BUSY: {
			dst = g2mplsTypes::LABELSTATE_BUSY;
			break;
		}
		default:
			throw out_of_range("label_state_t type out-of-range");
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::labelState &     src)
{
	std::ostringstream id;

	switch (src) {
		case g2mplsTypes::LABELSTATE_FREE: {
			id << "LABEL_FREE";
			break;
		}
		case g2mplsTypes::LABELSTATE_BOOKED: {
			id << "LABEL_BOOKED";
			break;
		}
		case g2mplsTypes::LABELSTATE_XCONNECTED: {
			id << "LABEL_XCONNECTED";
			break;
		}
		case g2mplsTypes::LABELSTATE_BUSY: {
			id << "LABEL_BUSY";
			break;
		}
		default:
			throw out_of_range("labelState type out-of-range");
			break;
	}

	dst = id.str();

	return dst;
}

//
// sourceId
//
g2mpls_addr_t &  operator<< (g2mpls_addr_t &                   dst,
			     const g2mplsTypes::sourceId_var & src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes::SOURCEIDTYPE_IPV4: {
			dst.type    = IPv4;
			dst.preflen = 32;
			dst.value.ipv4.s_addr = htonl(src->ipv4());
			break;
		}
		case g2mplsTypes::SOURCEIDTYPE_IPV6: {
			int    i;

			dst.type    = IPv6;
			dst.preflen = 128;

			for (i = 0; i < 4; i++) {
				dst.value.ipv6.s6_addr32[i] =
					htonl(src->ipv6()[i]);
			}
			break;
		}
		case g2mplsTypes::SOURCEIDTYPE_NSAP: {
			int    i;

			dst.type    = NSAP;
			dst.preflen = 160;

			for (i = 0; i < 20; i++) {
				dst.value.nsap.nsap_addr8[i] =
					src->nsap()[i];
			}
			break;
		}
		case g2mplsTypes::SOURCEIDTYPE_MAC: {
			// not supported
			break;
		}
		default:
			throw out_of_range("SOURCE ID type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::sourceId_var & operator<< (g2mplsTypes::sourceId_var &  dst,
					const g2mpls_addr_t &        src)
{
	dst = new g2mplsTypes::sourceId;

	switch (src.type) {
		case IPv4:
			dst->ipv4(ntohl(src.value.ipv4.s_addr));
			break;
		case IPv6: {
			g2mplsTypes::addrIPv6 parts;
			int i;

			for (i = 0; i < 4; i++) {
				parts[i] =
					ntohl(src.value.ipv6.s6_addr32[i]);
			}
			dst->ipv6(parts);
		}
			break;
		case NSAP: {
			g2mplsTypes::addrNSAP parts;
			int i;

			for (i = 0; i < 20; i++) {
				parts[i] = src.value.nsap.nsap_addr8[i];
			}
			dst->nsap(parts);
		}
			break;
		default:
			throw out_of_range("sourceId G2.ADDR type "
					   "out-of-range");
			break;
	}

	return dst;
}

std::string & operator<< (std::string &                     dst,
			  const g2mplsTypes::sourceId_var & src)
{
	std::ostringstream id;

	dst = std::string("");

	switch (src->_d()) {
		case g2mplsTypes::SOURCEIDTYPE_IPV4: {
			ipv4_t addr;

			addr.s_addr = htonl(src->ipv4());
			id << "(IPv4) " << inet_ntoa(addr);
			break;
		}
		case g2mplsTypes::SOURCEIDTYPE_IPV6: {
			int    i;

			id << "(IPv6) " ;
			for (i = 0; i < 4; i++) {
				id << std::hex
				   << std::setw(8)
				   << std::setfill('0')
				   << src->ipv6()[i];
			}
			break;
		}
		case g2mplsTypes::SOURCEIDTYPE_NSAP: {
			int    i;

			id << "(NSAP) " ;
			for (i = 0; i < 20; i++) {

				id << std::hex
				   << std::setw(2)
				   << std::setfill('0')
				   << (uint32_t) src->nsap()[i];
				if ((i == 0)  | (i == 2)  | (i == 3)  |
				    (i == 6)  | (i == 8)  | (i == 10) |
				    (i == 12) | (i == 18)) {
					id << ".";
				}
			}
			break;
		}
		case g2mplsTypes::SOURCEIDTYPE_MAC: {
			int    i;

			id << "(MAC) " ;
			for (i = 0; i < 6; i++) {

				id << std::hex
				   << std::setw(2)
				   << std::setfill('0')
				   << (uint32_t) src->mac()[i];
				if (i != 5) {
					id << ":";
				}
			}
			break;
		}
		default:
			throw out_of_range("SOURCE ID type out-of-range");
			break;
	}

	dst += id.str();

	return dst;
}

//
// tnaId
//
g2mpls_addr_t &  operator<< (g2mpls_addr_t &                dst,
			     const g2mplsTypes::tnaId_var & src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes::TNAIDTYPE_IPV4: {
			dst.type    = IPv4;
			dst.preflen = 32;
			dst.value.ipv4.s_addr = htonl(src->ipv4());
			break;
		}
		case g2mplsTypes::TNAIDTYPE_IPV6: {
			int    i;

			dst.type    = IPv6;
			dst.preflen = 128;

			for (i = 0; i < 4; i++) {
				dst.value.ipv6.s6_addr32[i] =
					htonl(src->ipv6()[i]);
			}
			break;
		}
		case g2mplsTypes::TNAIDTYPE_NSAP: {
			int    i;

			dst.type    = NSAP;
			dst.preflen = 160;

			for (i = 0; i < 20; i++) {
				dst.value.nsap.nsap_addr8[i] =
					src->nsap()[i];
			}
			break;
		}
		default:
			throw out_of_range("TNA ID type out-of-range");
	}

	return dst;
}

g2mplsTypes::tnaId_var & operator<< (g2mplsTypes::tnaId_var &   dst,
				     const g2mpls_addr_t &      src)
{
	dst = new g2mplsTypes::tnaId;

	switch (src.type) {
		case IPv4:
			dst->ipv4(ntohl(src.value.ipv4.s_addr));
			break;
		case IPv6: {
			g2mplsTypes::addrIPv6 parts;
			int i;

			for (i = 0; i < 4; i++) {
				parts[i] =
					ntohl(src.value.ipv6.s6_addr32[i]);
			}
			dst->ipv6(parts);
		}
			break;
		case NSAP: {
			g2mplsTypes::addrNSAP parts;
			int i;

			for (i = 0; i < 20; i++) {
				parts[i] = src.value.nsap.nsap_addr8[i];
			}
			dst->nsap(parts);
		}
			break;
		default:
			throw out_of_range("tnaId G2.ADDR type out-of-range");
			break;
	}

	return dst;
}

std::string & operator<< (std::string &                  dst,
			  const g2mplsTypes::tnaId_var & src)
{
	std::ostringstream id;

	dst = std::string("");

	switch (src->_d()) {
		case g2mplsTypes::TNAIDTYPE_IPV4: {
			ipv4_t addr;

			addr.s_addr = htonl(src->ipv4());
			id << "(TNA-IPv4) " << inet_ntoa(addr);
			break;
		}
		case g2mplsTypes::TNAIDTYPE_IPV6: {
			int    i;

			id << "(TNA-IPv6) " ;
			for (i = 0; i < 4; i++) {
				id << std::hex
				   << std::setw(8)
				   << std::setfill('0')
				   << src->ipv6()[i];
			}
			break;
		}
		case g2mplsTypes::TNAIDTYPE_NSAP: {
			int    i;

			id << "(TNA-NSAP) " ;
			for (i = 0; i < 20; i++) {

				id << std::hex
				   << std::setw(2)
				   << std::setfill('0')
				   << (uint32_t) src->nsap()[i];
				if ((i == 0)  | (i == 2)  | (i == 3)  |
				    (i == 6)  | (i == 8)  | (i == 10) |
				    (i == 12) | (i == 18)) {
					id << ".";
				}
			}
			break;
		}
		default:
			throw out_of_range("TNA ID type out-of-range");
			break;
	}

	dst += id.str();

	return dst;
}

//
// gridHostId
//
g2mpls_addr_t &  operator<< (g2mpls_addr_t &                         dst,
			     const g2mplsTypes::gridHostId_var &     src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes::GRIDHOSTTYPE_IPV4: {
			dst.type    = IPv4;
			dst.preflen = 32;
			dst.value.ipv4.s_addr = htonl(src->ipv4());
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_IPV6: {
			int    i;

			dst.type    = IPv6;
			dst.preflen = 128;

			for (i = 0; i < 4; i++) {
				dst.value.ipv6.s6_addr32[i] =
					htonl(src->ipv6()[i]);
			}
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_NSAP: {
			int    i;

			dst.type    = NSAP;
			dst.preflen = 160;

			for (i = 0; i < 20; i++) {
				dst.value.nsap.nsap_addr8[i] = src->nsap()[i];
			}
			break;
		}
		default:
			throw out_of_range("GRIDHOST type out-of-range");
	}

	return dst;
}

g2mplsTypes::gridHostId_var &
operator<< (g2mplsTypes::gridHostId_var &       dst,
	    const g2mpls_addr_t &               src)
{
	dst = new g2mplsTypes::gridHostId;

	switch (src.type) {
		case IPv4:
			dst->ipv4(ntohl(src.value.ipv4.s_addr));
			break;
		case IPv6: {
			g2mplsTypes::addrIPv6 parts;
			int i;

			for (i = 0; i < 4; i++) {
				parts[i] =
					ntohl(src.value.ipv6.s6_addr32[i]);
			}
			dst->ipv6(parts);
		}
			break;
		case NSAP: {
			g2mplsTypes::addrNSAP parts;
			int i;

			for (i = 0; i < 20; i++) {
				parts[i] = src.value.nsap.nsap_addr8[i];
			}
			dst->nsap(parts);
		}
			break;
		default:
			throw out_of_range("CHost G2.ADDR type out-of-range");
			break;
	}

	return dst;
}

std::string & operator<< (std::string &                           dst,
			  const g2mplsTypes::gridHostId_var &     src)
{
	std::ostringstream id;

	dst = std::string("");

	switch (src->_d()) {
		case g2mplsTypes::GRIDHOSTTYPE_IPV4: {
			ipv4_t addr;

			addr.s_addr = htonl(src->ipv4());
			id << "(GRID-HOST-IPv4) " << inet_ntoa(addr);
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_IPV6: {
			int    i;

			id << "(GRID-HOST-IPv6) " ;
			for (i = 0; i < 4; i++) {
				id << std::hex
				   << std::setw(8)
				   << std::setfill('0')
				   << src->ipv6()[i];
			}
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_NSAP: {
			int    i;

			id << "(GRID-HOST-NSAP) " ;
			for (i = 0; i < 20; i++) {

				id << std::hex
				   << std::setw(2)
				   << std::setfill('0')
				   << (uint32_t) src->nsap()[i];
				if ((i == 0)  | (i == 2)  | (i == 3)  |
				    (i == 6)  | (i == 8)  | (i == 10) |
				    (i == 12) | (i == 18)) {
					id << ".";
				}
			}
			break;
		}
		default:
			throw out_of_range("GRIDHOST type out-of-range");
			break;
	}

	dst += id.str();

	return dst;
}



//
//
// Call attrs operators
//
//
call_ident_t & operator<< (call_ident_t &                     dst,
			   const g2mplsTypes::callIdent_var & src)
{
	switch (src->idType) {
		case g2mplsTypes::CALLIDTYPE_NULL:
			dst.type = CALLID_NULL;
			break;
		case g2mplsTypes::CALLIDTYPE_OPSPEC:
			dst.type = CALLID_OPSPEC;
			break;
		case g2mplsTypes::CALLIDTYPE_GLOBUNIQ:
			dst.type = CALLID_GLOBUNIQ;
			break;
		default:
			throw out_of_range("Call type out-of-range");
			break;
	}

	dst.local_id  = src->localId;

	g2mplsTypes::sourceId_var lclAddr;
	lclAddr      = src->srcId;
	dst.src_addr << lclAddr;

#if 0
	if (dst.type == CALLID_GLOBUNIQ) {
		dst.itu_country_code |=
			(((uint32_t) src->segs.intlSeg[0]) << 16);
		dst.itu_country_code |=
			(((uint32_t) src->segs.intlSeg[1]) <<  8);
		dst.itu_country_code |=
			(((uint32_t) src->segs.intlSeg[2]) <<  0);
		dst.itu_country_code &= 0x00FFFFFF;

		dst.itu_carrier_code |=
			(((uint64_t) src->segs.natlSeg[0]) << 16);
		dst.itu_carrier_code |=
			(((uint64_t) ((src->segs.natlSeg[1] &
				       0xFFFF0000)) >> 16) << 0);
		dst.itu_carrier_code &= 0x0000FFFFFFFFFFFFull;

		dst.unique_ap |=
			(((uint64_t) ((src->segs.natlSeg[1] &
				       0x0000FFFF)) >> 0) << 32);
		dst.unique_ap |=
			(((uint64_t) src->segs.natlSeg[2]) <<  0);
		dst.unique_ap &= 0x0000FFFFFFFFFFFFull;
	}
#endif // 0

	return dst;
}

g2mplsTypes::callIdent_var & operator<<(g2mplsTypes::callIdent_var &   dst,
					const call_ident_t &           src)
{
	dst = new g2mplsTypes::callIdent;

	switch (src.type) {
		case CALLID_NULL:
			dst->idType = g2mplsTypes::CALLIDTYPE_NULL;
			break;
		case CALLID_OPSPEC:
			dst->idType = g2mplsTypes::CALLIDTYPE_OPSPEC;
			break;
		case CALLID_GLOBUNIQ:
			dst->idType = g2mplsTypes::CALLIDTYPE_GLOBUNIQ;
			break;
		default:
			throw out_of_range("G2.Call type out-of-range");
			break;
	}

	dst->localId  = src.local_id;

	g2mplsTypes::sourceId_var lclAddr;
	lclAddr    << src.src_addr;
	dst->srcId  = lclAddr;

#if 0
	if (src.type == CALLID_GLOBUNIQ) {
		dst->segs.intlSeg[0] |=
			(uint8_t) ((src.itu_country_code & 0x00FF0000) >> 16);
		dst->segs.intlSeg[1] |=
			(uint8_t) ((src.itu_country_code & 0x0000FF00) >>  8);
		dst->segs.intlSeg[2] |=
			(uint8_t) ((src.itu_country_code & 0x000000FF) >>  0);

		dst->segs.natlSeg[0] |=
			(uint32_t) ((src.itu_carrier_code &
				     0x0000FFFFFFFF0000ull) >>  16);
		dst->segs.natlSeg[1] |=
			(uint32_t) (((src.itu_carrier_code &
				      0x000000000000FFFFull) >>   0) << 16);
		dst->segs.natlSeg[1] |=
			(uint32_t) (((src.unique_ap &
				      0x0000FFFF00000000ull) >>  32) <<  0);
		dst->segs.natlSeg[2] |=
			(uint32_t) (((src.unique_ap &
				      0x00000000FFFFFFFFull) >>   0) <<  0);
	}
#endif // 0

	return dst;
}

//
//
// Call attrs operators
//
//
call_info_t & operator<<(call_info_t &                        dst,
			 const g2mplsTypes::callParams_var &  src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->type) {
		case g2mplsTypes::CALLTYPE_SPC:
			dst.call_type = CALL_TYPE_SPC;
			break;
		case g2mplsTypes::CALLTYPE_PC:
			dst.call_type = CALL_TYPE_PC;
			break;
		case g2mplsTypes::CALLTYPE_SC:
			dst.call_type = CALL_TYPE_SC;
			break;
		case g2mplsTypes::CALLTYPE_AUTO:
			dst.call_type = CALL_TYPE_AUTO;
			break;
		case g2mplsTypes::CALLTYPE_aUGWzUGW:
			dst.call_type = CALL_TYPE_aUGWzUGW;
			break;
		case g2mplsTypes::CALLTYPE_aMGTzEGW:
			dst.call_type = CALL_TYPE_aMGTzEGW;
			break;
		case g2mplsTypes::CALLTYPE_aUGWzEGW:
			dst.call_type = CALL_TYPE_aUGWzEGW;
			break;
		case g2mplsTypes::CALLTYPE_aEGWzMGT:
			dst.call_type = CALL_TYPE_aEGWzMGT;
			break;
		case g2mplsTypes::CALLTYPE_aEGWzUGW:
			dst.call_type = CALL_TYPE_aEGWzUGW;
			break;
		case g2mplsTypes::CALLTYPE_aEGWzEGW:
			dst.call_type = CALL_TYPE_aEGWzEGW;
			break;
		default:
			throw out_of_range("CALL TYPE out-of-range");
			break;
	}

	dst.call_name    = new std::string(src->name);

	// GNS attributes
	dst.times.start_time = src->times.startTime;
	dst.times.end_time   = src->times.endTime;

	dst.job_name     = new std::string(src->jobName);
	dst.job_project  = new std::string(src->jobProject);

	BITMASK_BITSET(dst.mask_,   call_type);
	BITMASK_BITSET(dst.mask_,   call_name);
	BITMASK_BITSET(dst.mask_,   times);
	BITMASK_BITSET(dst.mask_,   job_name);
	BITMASK_BITSET(dst.mask_,   job_project);
	BITMASK_BITRESET(dst.mask_, iTNA_res);
	BITMASK_BITRESET(dst.mask_, eTNA_res);

	return dst;
}

g2mplsTypes::callParams_var & operator<<(g2mplsTypes::callParams_var &  dst,
					 const call_info_t &            src)
{
	dst = new g2mplsTypes::callParams;

	switch (src.call_type) {
		case CALL_TYPE_SPC:
			dst->type = g2mplsTypes::CALLTYPE_SPC;
			break;
		case CALL_TYPE_PC:
			dst->type = g2mplsTypes::CALLTYPE_PC;
			break;
		case CALL_TYPE_SC:
			dst->type = g2mplsTypes::CALLTYPE_SC;
			break;
		case CALL_TYPE_AUTO:
			dst->type = g2mplsTypes::CALLTYPE_AUTO;
			break;
		case CALL_TYPE_aUGWzUGW:
			dst->type = g2mplsTypes::CALLTYPE_aUGWzUGW;
			break;
		case CALL_TYPE_aMGTzEGW:
			dst->type = g2mplsTypes::CALLTYPE_aMGTzEGW;
			break;
		case CALL_TYPE_aUGWzEGW:
			dst->type = g2mplsTypes::CALLTYPE_aUGWzEGW;
			break;
		case CALL_TYPE_aEGWzMGT:
			dst->type = g2mplsTypes::CALLTYPE_aEGWzMGT;
			break;
		case CALL_TYPE_aEGWzUGW:
			dst->type = g2mplsTypes::CALLTYPE_aEGWzUGW;
			break;
		case CALL_TYPE_aEGWzEGW:
			dst->type = g2mplsTypes::CALLTYPE_aEGWzEGW;
			break;
		default:
			throw out_of_range("CALL TYPE out-of-range");
			break;
	}

	if (src.call_name) {
		dst->name = CORBA::string_dup(src.call_name->c_str());
	}

	// GNS Attributes
	dst->times.startTime = src.times.start_time;
	dst->times.endTime   = src.times.end_time;
	if (src.job_name) {
		dst->jobName = CORBA::string_dup(src.job_name->c_str());
	}
	if (src.job_project) {
		dst->jobProject = CORBA::string_dup(src.job_project->c_str());
	}

	return dst;
}

recovery_info_t & operator<<(recovery_info_t &                        dst,
			     const g2mplsTypes::recoveryParams_var &  src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->recType) {
		case g2mplsTypes::RECOVERYTYPE_UNPROTECTED:
			dst.rec_type = RECOVERY_NONE;
			break;
		case g2mplsTypes::RECOVERYTYPE_PROTECTION:
			dst.rec_type = RECOVERY_PROTECTION;
			break;
		case g2mplsTypes::RECOVERYTYPE_PREPLANNED:
			dst.rec_type = RECOVERY_PREPLANNED;
			break;
		case g2mplsTypes::RECOVERYTYPE_OTF:
			dst.rec_type = RECOVERY_ONTHEFLY;
			break;
		case g2mplsTypes::RECOVERYTYPE_OTF_REVERTIVE:
			dst.rec_type = RECOVERY_REVERTIVEONTHEFLY;
			break;
		default:
			throw out_of_range("RECOVERY TYPE out-of-range");
			break;
	}

	switch (src->disjType) {
		case g2mplsTypes::DISJOINTNESS_NONE:
			dst.disj_type = DISJOINTNESS_NONE;
			break;
		case g2mplsTypes::DISJOINTNESS_LINK:
			dst.disj_type = DISJOINTNESS_LINK;
			break;
		case g2mplsTypes::DISJOINTNESS_NODE:
			dst.disj_type = DISJOINTNESS_NODE;
			break;
		case g2mplsTypes::DISJOINTNESS_SRLG:
			dst.disj_type = DISJOINTNESS_SRLG;
			break;
		default:
			throw out_of_range("DISJOINTNESS type out-of-range");
			break;
	}

	BITMASK_BITSET(dst.mask_, rec_type);
	BITMASK_BITSET(dst.mask_, disj_type);

	return dst;
}



g2mplsTypes::recoveryParams_var &
operator<<(g2mplsTypes::recoveryParams_var &  dst,
	   const recovery_info_t &            src)
{
	dst = new g2mplsTypes::recoveryParams;

	switch (src.rec_type) {
		case RECOVERY_NONE:
			dst->recType = g2mplsTypes::RECOVERYTYPE_UNPROTECTED;
			break;
		case RECOVERY_PROTECTION:
			dst->recType = g2mplsTypes::RECOVERYTYPE_PROTECTION;
			break;
		case RECOVERY_PREPLANNED:
			dst->recType = g2mplsTypes::RECOVERYTYPE_PREPLANNED;
			break;
		case RECOVERY_ONTHEFLY:
			dst->recType = g2mplsTypes::RECOVERYTYPE_OTF;
			break;
		case RECOVERY_REVERTIVEONTHEFLY:
			dst->recType = g2mplsTypes::RECOVERYTYPE_OTF_REVERTIVE;
			break;
		default:
			throw out_of_range("RECOVERY TYPE out-of-range");
			break;
	}

	switch (src.disj_type) {
		case DISJOINTNESS_NONE:
			dst->disjType = g2mplsTypes::DISJOINTNESS_NONE;
			break;
		case DISJOINTNESS_LINK:
			dst->disjType = g2mplsTypes::DISJOINTNESS_LINK;
			break;
		case DISJOINTNESS_NODE:
			dst->disjType = g2mplsTypes::DISJOINTNESS_NODE;
			break;
		case DISJOINTNESS_SRLG:
			dst->disjType = g2mplsTypes::DISJOINTNESS_SRLG;
			break;
		default:
			throw out_of_range("DISJOINTNESS type out-of-range");
			break;
	}

	return dst;
}

//
// lspIdent
//
lsp_ident_t & operator<<(lsp_ident_t &                     dst,
			 const g2mplsTypes::lspIdent_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.dst_nid    = src->dstNodeId;
	dst.src_nid    = src->srcNodeId;
	dst.tun_id     = src->tunId;
	dst.ext_tun_id = src->extTid;
	dst.lsp_id     = src->lspId;

	return dst;
}

g2mplsTypes::lspIdent_var & operator<<(g2mplsTypes::lspIdent_var & dst,
				       const lsp_ident_t &         src)
{
	dst = new g2mplsTypes::lspIdent;

	dst->dstNodeId   = src.dst_nid;
	dst->srcNodeId   = src.src_nid;
	dst->tunId       = src.tun_id;
	dst->extTid      = src.ext_tun_id;
	dst->lspId       = src.lsp_id;

	return dst;
}

//
// lspParams
//

lsp_info_t & operator<<(lsp_info_t &                       dst,
			const g2mplsTypes::lspParams_var & src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->swCap) {
		case g2mplsTypes::SWITCHINGCAP_PSC_1:
			dst.sw_cap = SWCAP_PSC_1;
			break;
		case g2mplsTypes::SWITCHINGCAP_PSC_2:
			dst.sw_cap = SWCAP_PSC_2;
			break;
		case g2mplsTypes::SWITCHINGCAP_PSC_3:
			dst.sw_cap = SWCAP_PSC_3;
			break;
		case g2mplsTypes::SWITCHINGCAP_PSC_4:
			dst.sw_cap = SWCAP_PSC_4;
			break;
		case g2mplsTypes::SWITCHINGCAP_L2SC:
			dst.sw_cap = SWCAP_L2SC;
			break;
		case g2mplsTypes::SWITCHINGCAP_TDM:
			dst.sw_cap = SWCAP_TDM;
			break;
		case g2mplsTypes::SWITCHINGCAP_LSC:
			dst.sw_cap = SWCAP_LSC;
			break;
		case g2mplsTypes::SWITCHINGCAP_FSC:
			dst.sw_cap = SWCAP_FSC;
			break;
		default:
			throw out_of_range("SWITCHING CAP out-of-range");
			break;
	}

	switch (src->encType) {
		case g2mplsTypes::ENCODINGTYPE_PACKET:
			dst.enc_type = ENCT_PACKET;
			break;
		case g2mplsTypes::ENCODINGTYPE_ETHERNET:
			dst.enc_type =ENCT_ETHERNET ;
			break;
		case g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH:
			dst.enc_type = ENCT_ANSI_ETSI_PDH;
			break;
		case g2mplsTypes::ENCODINGTYPE_SDH_SONET:
			dst.enc_type = ENCT_SDH_SONET;
			break;
		case g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER:
			dst.enc_type = ENCT_DIGITAL_WRAPPER;
			break;
		case g2mplsTypes::ENCODINGTYPE_LAMBDA:
			dst.enc_type = ENCT_LAMBDA;
			break;
		case g2mplsTypes::ENCODINGTYPE_FIBER:
			dst.enc_type = ENCT_FIBER;
			break;
		case g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL:
			dst.enc_type = ENCT_FIBERCHANNEL;
			break;
		case g2mplsTypes::ENCODINGTYPE_G709_ODU:
			dst.enc_type = ENCT_G709_ODU;
			break;
		case g2mplsTypes::ENCODINGTYPE_G709_OC:
			dst.enc_type = ENCT_G709_OC;
			break;
		case g2mplsTypes::ENCODINGTYPE_RESERVED_1:
		case g2mplsTypes::ENCODINGTYPE_RESERVED_2:
		case g2mplsTypes::ENCODINGTYPE_RESERVED_3:
		default:
			throw out_of_range("ENCODING TYPE out-of-range");
			break;
	}

	switch (src->gpid) {
		case g2mplsTypes::GPID_ASYNCH_E4:
			dst.gpid = GPID_ASYNCH_E4;
			break;
		case g2mplsTypes::GPID_ASYNCH_DS3_T3:
			dst.gpid = GPID_ASYNCH_DS3_T3;
			break;
		case g2mplsTypes::GPID_ASYNCH_E3:
			dst.gpid = GPID_ASYNCH_E3;
			break;
		case g2mplsTypes::GPID_BIT_SYNCH_E3:
			dst.gpid = GPID_BIT_SYNCH_E3;
			break;
		case g2mplsTypes::GPID_BYTE_SYNCH_E3:
			dst.gpid = GPID_BYTE_SYNCH_E3;
			break;
		case g2mplsTypes::GPID_ASYNCH_DS2_T2:
			dst.gpid = GPID_ASYNCH_DS2_T2;
			break;
		case g2mplsTypes::GPID_BIT_SYNCH_DS2_T2:
			dst.gpid = GPID_BIT_SYNCH_DS2_T2;
			break;
		case g2mplsTypes::GPID_ASYNCH_E1:
			dst.gpid = GPID_ASYNCH_E1;
			break;
		case g2mplsTypes::GPID_BYTE_SYNCH_E1:
			dst.gpid = GPID_BYTE_SYNCH_E1;
			break;
		case g2mplsTypes::GPID_BYTE_SYNCH_31DS0:
			dst.gpid = GPID_BYTE_SYNCH_31DS0;
			break;
		case g2mplsTypes::GPID_ASYNCH_DS1_T1:
			dst.gpid = GPID_ASYNCH_DS1_T1;
			break;
		case g2mplsTypes::GPID_BIT_SYNCH_DS1_T1:
			dst.gpid = GPID_BIT_SYNCH_DS1_T1;
			break;
		case g2mplsTypes::GPID_BYTE_SYNCH_DS1_T1:
			dst.gpid = GPID_BYTE_SYNCH_DS1_T1;
			break;
		case g2mplsTypes::GPID_VC_11_IN_VC_12:
			dst.gpid = GPID_VC_11_IN_VC_12;
			break;
		case g2mplsTypes::GPID_DS1_SF_ASYNCH:
			dst.gpid = GPID_DS1_SF_ASYNCH;
			break;
		case g2mplsTypes::GPID_DS1_ESF_ASYNCH:
			dst.gpid = GPID_DS1_ESF_ASYNCH;
			break;
		case g2mplsTypes::GPID_DS3_M23_ASYNCH:
			dst.gpid = GPID_DS3_M23_ASYNCH;
			break;
		case g2mplsTypes::GPID_DS3_C_PARITY_ASYNCH:
			dst.gpid = GPID_DS3_C_PARITY_ASYNCH;
			break;
		case g2mplsTypes::GPID_VT_LOVC:
			dst.gpid = GPID_VT_LOVC;
			break;
		case g2mplsTypes::GPID_STSSPE_HOVC:
			dst.gpid = GPID_STSSPE_HOVC;
			break;
		case g2mplsTypes::GPID_POS_NOSCRAMBLING_16CRC:
			dst.gpid = GPID_POS_NOSCRAMBLING_16CRC;
			break;
		case g2mplsTypes::GPID_POS_NOSCRAMBLING_32CRC:
			dst.gpid = GPID_POS_NOSCRAMBLING_32CRC;
			break;
		case g2mplsTypes::GPID_POS_SCRAMBLING_16CRC:
			dst.gpid = GPID_POS_SCRAMBLING_16CRC;
			break;
		case g2mplsTypes::GPID_POS_SCRAMBLING_32CRC:
			dst.gpid = GPID_POS_SCRAMBLING_32CRC;
			break;
		case g2mplsTypes::GPID_ATM_MAPPING:
			dst.gpid = GPID_ATM_MAPPING;
			break;
		case g2mplsTypes::GPID_ETHERNET:
			dst.gpid = GPID_ETHERNET;
			break;
		case g2mplsTypes::GPID_SONET_SDH:
			dst.gpid = GPID_SONET_SDH;
			break;
		case g2mplsTypes::GPID_DIGITAL_WRAPPER:
			dst.gpid = GPID_DIGITAL_WRAPPER;
			break;
		case g2mplsTypes::GPID_LAMBDA:
			dst.gpid = GPID_LAMBDA;
			break;
		case g2mplsTypes::GPID_ANSI_ETSI_PDH:
			dst.gpid = GPID_ANSI_ETSI_PDH;
			break;
		case g2mplsTypes::GPID_LAPS_X85_X86:
			dst.gpid = GPID_LAPS_X85_X86;
			break;
		case g2mplsTypes::GPID_FDDI:
			dst.gpid = GPID_FDDI;
			break;
		case g2mplsTypes::GPID_DQDB:
			dst.gpid = GPID_DQDB;
			break;
		case g2mplsTypes::GPID_FIBERCHANNEL_3:
			dst.gpid = GPID_FIBERCHANNEL_3;
			break;
		case g2mplsTypes::GPID_HDLC:
			dst.gpid = GPID_HDLC;
			break;
		case g2mplsTypes::GPID_ETH_V2_DIX:
			dst.gpid = GPID_ETH_V2_DIX;
			break;
		case g2mplsTypes::GPID_ETH_802_3:
			dst.gpid = GPID_ETH_802_3;
			break;
		case g2mplsTypes::GPID_G709_ODUJ:
			dst.gpid = GPID_G709_ODUJ;
			break;
		case g2mplsTypes::GPID_G709_OTUK:
			dst.gpid = GPID_G709_OTUK;
			break;
		case g2mplsTypes::GPID_CBR_CBRA:
			dst.gpid = GPID_CBR_CBRA;
			break;
		case g2mplsTypes::GPID_CBRB:
			dst.gpid = GPID_CBRB;
			break;
		case g2mplsTypes::GPID_BSOT:
			dst.gpid = GPID_BSOT;
			break;
		case g2mplsTypes::GPID_BSNT:
			dst.gpid = GPID_BSNT;
			break;
		case g2mplsTypes::GPID_IP_PPP_GFP:
			dst.gpid = GPID_IP_PPP_GFP;
			break;
		case g2mplsTypes::GPID_ETHMAC_GFP:
			dst.gpid = GPID_ETHMAC_GFP;
			break;
		case g2mplsTypes::GPID_ETHPHY_GFP:
			dst.gpid = GPID_ETHPHY_GFP;
			break;
		case g2mplsTypes::GPID_ESCON:
			dst.gpid = GPID_ESCON;
			break;
		case g2mplsTypes::GPID_FICON:
			dst.gpid = GPID_FICON;
			break;
		default:
			throw out_of_range("GPID out-of-range");
			break;
	}

	dst.bw           = src->bw;
	dst.setup_prio   = src->setupPrio;
	dst.holding_prio = src->holdingPrio;
	dst.exclude_any  = src->excludeAny;
	dst.include_any  = src->includeAny;
	dst.include_all  = src->includeAll;

	switch (src->linkProtMask) {
		case g2mplsTypes::PROTTYPE_NONE:
			dst.link_prot_mask = PROTTYPE_NONE;
			break;
		case g2mplsTypes::PROTTYPE_EXTRA:
			dst.link_prot_mask = PROTTYPE_EXTRA;
			break;
		case g2mplsTypes::PROTTYPE_UNPROTECTED:
			dst.link_prot_mask = PROTTYPE_UNPROTECTED;
			break;
		case g2mplsTypes::PROTTYPE_SHARED:
			dst.link_prot_mask = PROTTYPE_SHARED;
			break;
		case g2mplsTypes::PROTTYPE_DEDICATED_1TO1:
			dst.link_prot_mask = PROTTYPE_DEDICATED_1TO1;
			break;
		case g2mplsTypes::PROTTYPE_DEDICATED_1PLUS1:
			dst.link_prot_mask = PROTTYPE_DEDICATED_1PLUS1;
			break;
		case g2mplsTypes::PROTTYPE_ENHANCED:
			dst.link_prot_mask = PROTTYPE_ENHANCED;
			break;
		default:
			throw out_of_range("PROT TYPE out-of-range");
			break;
	}


	switch (src->crankback) {
		case g2mplsTypes::CRANCKBACKSCOPE_NONE:
			dst.crankback = CB_NONE;
			break;
		case g2mplsTypes::CRANCKBACKSCOPE_E2E:
			dst.crankback = CB_E2E;
			break;
		case g2mplsTypes::CRANCKBACKSCOPE_BOUNDARY:
			dst.crankback = CB_BOUNDARY;
			break;
		case g2mplsTypes::CRANCKBACKSCOPE_SEGMENTBASED:
			dst.crankback = CB_SEGMENT;
			break;
		default:
			throw out_of_range("CRANCKBACK SCOPE out-of-range");
			break;
	}

	dst.max_cback_retries_src    = src->maxCbackRetriesSrc;
	dst.max_cback_retries_intmd  = src->maxCbackRetriesIntmd;

	switch (src->type) {
		case g2mplsTypes::LSPTYPE_SPC:
			dst.type = LSP_TYPE_SPC;
			break;
		case g2mplsTypes::LSPTYPE_PC:
			dst.type = LSP_TYPE_PC;
			break;
		case g2mplsTypes::LSPTYPE_SC:
			dst.type = LSP_TYPE_SC;
			break;
		default:
			throw out_of_range("LSP TYPE out-of-range");
			break;
	}

	switch (src->role) {
		case g2mplsTypes::LSPROLE_UNDEFINED:
		case g2mplsTypes::LSPROLE_WORKER:
			dst.role = LSP_ROLE_PRIMARY;
			break;
		case g2mplsTypes::LSPROLE_BACKUP:
			dst.role = LSP_ROLE_SECONDARY;
			break;
		default:
			throw out_of_range("LSP ROLE out-of-range");
			break;
	}

	switch (src->action) {
		case g2mplsTypes::LSPRESOURCEACTION_XCONNECT:
			dst.action = RESOURCE_XCONNECT;
			break;
		case g2mplsTypes::LSPRESOURCEACTION_BOOK:
			dst.action = RESOURCE_BOOK;
			break;
		default:
			throw out_of_range("LSP RESOURCE ACTION type "
					   "out-of-range");
			break;
	}

	switch (src->rroMode) {
		case g2mplsTypes::LSPRROMODE_OFF:
			dst.rro_mode = LSP_RRO_OFF;
			break;
		case g2mplsTypes::LSPRROMODE_TEL_DETAIL:
			dst.rro_mode = LSP_RRO_TEL_DETAIL;
			break;
		case g2mplsTypes::LSPRROMODE_DL_DETAIL:
			dst.rro_mode = LSP_RRO_DL_DETAIL;
			break;
		case g2mplsTypes::LSPRROMODE_ALL:
			dst.rro_mode = LSP_RRO_ALL;
			break;
		default:
			throw out_of_range("LSP RRO MODE type out-of-range");
			break;
	}

	dst.refresh_interval        = src->refreshInterval;
	dst.activate_ack            = src->activateAck ? 1 : 0;
	dst.rapid_retransm_interval = src->rapidRetransmInterval;
	dst.rapid_retry_limit       = src->rapidRetryLimit;
	dst.increment_value_delta   = src->incrementValueDelta;

	dst.times.start_time        = src->times.startTime;
	dst.times.end_time          = src->times.endTime;

	BITMASK_BITSET(dst.mask_, sw_cap);
	BITMASK_BITSET(dst.mask_, enc_type);
	BITMASK_BITSET(dst.mask_, gpid);
	BITMASK_BITSET(dst.mask_, bw);
	BITMASK_BITSET(dst.mask_, setup_prio);
	BITMASK_BITSET(dst.mask_, holding_prio);
	BITMASK_BITSET(dst.mask_, exclude_any);
	BITMASK_BITSET(dst.mask_, include_any);
	BITMASK_BITSET(dst.mask_, include_all);
	BITMASK_BITSET(dst.mask_, link_prot_mask);
	BITMASK_BITSET(dst.mask_, crankback);
	BITMASK_BITSET(dst.mask_, max_cback_retries_src);
	BITMASK_BITSET(dst.mask_, max_cback_retries_intmd);
	BITMASK_BITSET(dst.mask_, type);
	BITMASK_BITSET(dst.mask_, role);
	BITMASK_BITSET(dst.mask_, action);
	BITMASK_BITSET(dst.mask_, rro_mode);
	BITMASK_BITSET(dst.mask_, refresh_interval);
	BITMASK_BITSET(dst.mask_, activate_ack);
	BITMASK_BITSET(dst.mask_, rapid_retransm_interval);
	BITMASK_BITSET(dst.mask_, rapid_retry_limit);
	BITMASK_BITSET(dst.mask_, increment_value_delta);
	BITMASK_BITSET(dst.mask_, times);

	return dst;
}



g2mplsTypes::lspParams_var & operator<<(g2mplsTypes::lspParams_var &   dst,
					const lsp_info_t &             src)
{
	dst = new g2mplsTypes::lspParams;

	switch (src.sw_cap) {
		case SWCAP_PSC_1:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_PSC_1;
			break;
		case SWCAP_PSC_2:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_PSC_2;
			break;
		case SWCAP_PSC_3:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_PSC_3;
			break;
		case SWCAP_PSC_4:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_PSC_4;
			break;
		case SWCAP_L2SC:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_L2SC;
			break;
		case SWCAP_TDM:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_TDM;
			break;
		case SWCAP_LSC:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_LSC;
			break;
		case SWCAP_FSC:
			dst->swCap = g2mplsTypes::SWITCHINGCAP_FSC;
			break;
		default:
			throw out_of_range("SWITCHING CAP out-of-range");
			break;
	}

	switch (src.enc_type) {
		case ENCT_PACKET:
			dst->encType = g2mplsTypes::ENCODINGTYPE_PACKET;
			break;
		case ENCT_ETHERNET:
			dst->encType = g2mplsTypes::ENCODINGTYPE_ETHERNET ;
			break;
		case ENCT_ANSI_ETSI_PDH:
			dst->encType = g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH;
			break;
		case ENCT_SDH_SONET:
			dst->encType = g2mplsTypes::ENCODINGTYPE_SDH_SONET;
			break;
		case ENCT_DIGITAL_WRAPPER:
			dst->encType =
				g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER;
			break;
		case ENCT_LAMBDA:
			dst->encType = g2mplsTypes::ENCODINGTYPE_LAMBDA;
			break;
		case ENCT_FIBER:
			dst->encType = g2mplsTypes::ENCODINGTYPE_FIBER;
			break;
		case ENCT_FIBERCHANNEL:
			dst->encType = g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL;
			break;
		case ENCT_G709_ODU:
			dst->encType = g2mplsTypes::ENCODINGTYPE_G709_ODU;
			break;
		case ENCT_G709_OC:
			dst->encType = g2mplsTypes::ENCODINGTYPE_G709_OC;
			break;
		case ENCT_RESERVED_1:
		case ENCT_RESERVED_2:
		case ENCT_RESERVED_3:
		default:
			throw out_of_range("ENCODING TYPE out-of-range");
			break;
	}

	switch (src.gpid) {
		case GPID_ASYNCH_E4:
			dst->gpid = g2mplsTypes::GPID_ASYNCH_E4;
			break;
		case GPID_ASYNCH_DS3_T3:
			dst->gpid = g2mplsTypes::GPID_ASYNCH_DS3_T3;
			break;
		case GPID_ASYNCH_E3:
			dst->gpid = g2mplsTypes::GPID_ASYNCH_E3;
			break;
		case GPID_BIT_SYNCH_E3:
			dst->gpid = g2mplsTypes::GPID_BIT_SYNCH_E3;
			break;
		case GPID_BYTE_SYNCH_E3:
			dst->gpid = g2mplsTypes::GPID_BYTE_SYNCH_E3;
			break;
		case GPID_ASYNCH_DS2_T2:
			dst->gpid = g2mplsTypes::GPID_ASYNCH_DS2_T2;
			break;
		case GPID_BIT_SYNCH_DS2_T2:
			dst->gpid = g2mplsTypes::GPID_BIT_SYNCH_DS2_T2;
			break;
		case GPID_ASYNCH_E1:
			dst->gpid = g2mplsTypes::GPID_ASYNCH_E1;
			break;
		case GPID_BYTE_SYNCH_E1:
			dst->gpid = g2mplsTypes::GPID_BYTE_SYNCH_E1;
			break;
		case GPID_BYTE_SYNCH_31DS0:
			dst->gpid = g2mplsTypes::GPID_BYTE_SYNCH_31DS0;
			break;
		case GPID_ASYNCH_DS1_T1:
			dst->gpid = g2mplsTypes::GPID_ASYNCH_DS1_T1;
			break;
		case GPID_BIT_SYNCH_DS1_T1:
			dst->gpid = g2mplsTypes::GPID_BIT_SYNCH_DS1_T1;
			break;
		case GPID_BYTE_SYNCH_DS1_T1:
			dst->gpid = g2mplsTypes::GPID_BYTE_SYNCH_DS1_T1;
			break;
		case GPID_VC_11_IN_VC_12:
			dst->gpid = g2mplsTypes::GPID_VC_11_IN_VC_12;
			break;
		case GPID_DS1_SF_ASYNCH:
			dst->gpid = g2mplsTypes::GPID_DS1_SF_ASYNCH;
			break;
		case GPID_DS1_ESF_ASYNCH:
			dst->gpid = g2mplsTypes::GPID_DS1_ESF_ASYNCH;
			break;
		case GPID_DS3_M23_ASYNCH:
			dst->gpid = g2mplsTypes::GPID_DS3_M23_ASYNCH;
			break;
		case GPID_DS3_C_PARITY_ASYNCH:
			dst->gpid = g2mplsTypes::GPID_DS3_C_PARITY_ASYNCH;
			break;
		case GPID_VT_LOVC:
			dst->gpid = g2mplsTypes::GPID_VT_LOVC;
			break;
		case GPID_STSSPE_HOVC:
			dst->gpid = g2mplsTypes::GPID_STSSPE_HOVC;
			break;
		case GPID_POS_NOSCRAMBLING_16CRC:
			dst->gpid = g2mplsTypes::GPID_POS_NOSCRAMBLING_16CRC;
			break;
		case GPID_POS_NOSCRAMBLING_32CRC:
			dst->gpid = g2mplsTypes::GPID_POS_NOSCRAMBLING_32CRC;
			break;
		case GPID_POS_SCRAMBLING_16CRC:
			dst->gpid = g2mplsTypes::GPID_POS_SCRAMBLING_16CRC;
			break;
		case GPID_POS_SCRAMBLING_32CRC:
			dst->gpid = g2mplsTypes::GPID_POS_SCRAMBLING_32CRC;
			break;
		case GPID_ATM_MAPPING:
			dst->gpid = g2mplsTypes::GPID_ATM_MAPPING;
			break;
		case GPID_ETHERNET:
			dst->gpid = g2mplsTypes::GPID_ETHERNET;
			break;
		case GPID_SONET_SDH:
			dst->gpid = g2mplsTypes::GPID_SONET_SDH;
			break;
		case GPID_DIGITAL_WRAPPER:
			dst->gpid = g2mplsTypes::GPID_DIGITAL_WRAPPER;
			break;
		case GPID_LAMBDA:
			dst->gpid = g2mplsTypes::GPID_LAMBDA;
			break;
		case GPID_ANSI_ETSI_PDH:
			dst->gpid = g2mplsTypes::GPID_ANSI_ETSI_PDH;
			break;
		case GPID_LAPS_X85_X86:
			dst->gpid = g2mplsTypes::GPID_LAPS_X85_X86;
			break;
		case GPID_FDDI:
			dst->gpid = g2mplsTypes::GPID_FDDI;
			break;
		case GPID_DQDB:
			dst->gpid = g2mplsTypes::GPID_DQDB;
			break;
		case GPID_FIBERCHANNEL_3:
			dst->gpid = g2mplsTypes::GPID_FIBERCHANNEL_3;
			break;
		case GPID_HDLC:
			dst->gpid = g2mplsTypes::GPID_HDLC;
			break;
		case GPID_ETH_V2_DIX:
			dst->gpid = g2mplsTypes::GPID_ETH_V2_DIX;
			break;
		case GPID_ETH_802_3:
			dst->gpid = g2mplsTypes::GPID_ETH_802_3;
			break;
		case GPID_G709_ODUJ:
			dst->gpid = g2mplsTypes::GPID_G709_ODUJ;
			break;
		case GPID_G709_OTUK:
			dst->gpid = g2mplsTypes::GPID_G709_OTUK;
			break;
		case GPID_CBR_CBRA:
			dst->gpid = g2mplsTypes::GPID_CBR_CBRA;
			break;
		case GPID_CBRB:
			dst->gpid = g2mplsTypes::GPID_CBRB;
			break;
		case GPID_BSOT:
			dst->gpid = g2mplsTypes::GPID_BSOT;
			break;
		case GPID_BSNT:
			dst->gpid = g2mplsTypes::GPID_BSNT;
			break;
		case GPID_IP_PPP_GFP:
			dst->gpid = g2mplsTypes::GPID_IP_PPP_GFP;
			break;
		case GPID_ETHMAC_GFP:
			dst->gpid = g2mplsTypes::GPID_ETHMAC_GFP;
			break;
		case GPID_ETHPHY_GFP:
			dst->gpid = g2mplsTypes::GPID_ETHPHY_GFP;
			break;
		case GPID_ESCON:
			dst->gpid = g2mplsTypes::GPID_ESCON;
			break;
		case GPID_FICON:
			dst->gpid = g2mplsTypes::GPID_FICON;
			break;
		default:
			throw out_of_range("GPID out-of-range");
			break;
	}

	dst->bw                   = src.bw;
	dst->setupPrio            = src.setup_prio;
	dst->holdingPrio          = src.holding_prio;
	dst->excludeAny           = src.exclude_any;
	dst->includeAny           = src.include_any;
	dst->includeAll           = src.include_all;

	switch (src.link_prot_mask) {
		case PROTTYPE_NONE:
			dst->linkProtMask = g2mplsTypes::PROTTYPE_NONE;
			break;
		case PROTTYPE_EXTRA:
			dst->linkProtMask = g2mplsTypes::PROTTYPE_EXTRA;
			break;
		case PROTTYPE_UNPROTECTED:
			dst->linkProtMask = g2mplsTypes::PROTTYPE_UNPROTECTED;
			break;
		case PROTTYPE_SHARED:
			dst->linkProtMask = g2mplsTypes::PROTTYPE_SHARED;
			break;
		case PROTTYPE_DEDICATED_1TO1:
			dst->linkProtMask =
				g2mplsTypes::PROTTYPE_DEDICATED_1TO1;
			break;
		case PROTTYPE_DEDICATED_1PLUS1:
			dst->linkProtMask =
				g2mplsTypes::PROTTYPE_DEDICATED_1PLUS1;
			break;
		case PROTTYPE_ENHANCED:
			dst->linkProtMask = g2mplsTypes::PROTTYPE_ENHANCED;
			break;
		default:
			throw out_of_range("PROT TYPE out-of-range");
			break;
	}


	switch (src.crankback) {
		case CB_NONE:
			dst->crankback = g2mplsTypes::CRANCKBACKSCOPE_NONE;
			break;
		case CB_E2E:
			dst->crankback = g2mplsTypes::CRANCKBACKSCOPE_E2E;
			break;
		case CB_BOUNDARY:
			dst->crankback = g2mplsTypes::CRANCKBACKSCOPE_BOUNDARY;
			break;
		case CB_SEGMENT:
			dst->crankback =
				g2mplsTypes::CRANCKBACKSCOPE_SEGMENTBASED;
			break;
		default:
			throw out_of_range("CRANCKBACK SCOPE out-of-range");
			break;
	}

	dst->maxCbackRetriesSrc    = src.max_cback_retries_src;
	dst->maxCbackRetriesIntmd  = src.max_cback_retries_intmd;

	switch (src.type) {
		case LSP_TYPE_SPC:
			dst->type = g2mplsTypes::LSPTYPE_SPC;
			break;
		case LSP_TYPE_PC:
			dst->type = g2mplsTypes::LSPTYPE_PC;
			break;
		case LSP_TYPE_SC:
			dst->type = g2mplsTypes::LSPTYPE_SC;
			break;
		default:
			throw out_of_range("LSP TYPE out-of-range");
			break;
	}

	switch (src.role) {
		case LSP_ROLE_PRIMARY:
			dst->role = g2mplsTypes::LSPROLE_WORKER;
			break;
		case LSP_ROLE_SECONDARY:
			dst->role = g2mplsTypes::LSPROLE_BACKUP;
			break;
		default:
			throw out_of_range("LSP ROLE out-of-range");
			break;
	}

	switch (src.action) {
		case RESOURCE_XCONNECT:
			dst->action = g2mplsTypes::LSPRESOURCEACTION_XCONNECT;
			break;
		case RESOURCE_BOOK:
			dst->action = g2mplsTypes::LSPRESOURCEACTION_BOOK;
			break;
		default:
			throw out_of_range("LSP RESOURCE ACTION type "
					   "out-of-range");
			break;
	}

	switch (src.rro_mode) {
		case LSP_RRO_OFF:
			dst->rroMode = g2mplsTypes::LSPRROMODE_OFF;
			break;
		case LSP_RRO_TEL_DETAIL:
			dst->rroMode = g2mplsTypes::LSPRROMODE_TEL_DETAIL;
			break;
		case LSP_RRO_DL_DETAIL:
			dst->rroMode = g2mplsTypes::LSPRROMODE_DL_DETAIL;
			break;
		case LSP_RRO_ALL:
			dst->rroMode = g2mplsTypes::LSPRROMODE_ALL;
			break;
		default:
			throw out_of_range("LSP RRO MODE type out-of-range");
			break;
	}

	dst->refreshInterval       = src.refresh_interval;
	dst->activateAck           = src.activate_ack ? true : false;
	dst->rapidRetransmInterval = src.rapid_retransm_interval;
	dst->rapidRetryLimit       = src.rapid_retry_limit;
	dst->incrementValueDelta   = src.increment_value_delta;

	dst->times.startTime       = src.times.start_time;
	dst->times.endTime         = src.times.end_time;

	return dst;
}

// tnResource
net_res_spec_t &
operator<<(net_res_spec_t &                        dst,
	   const g2mplsTypes::tnResource_var &     src)
{
	memset(&dst, 0, sizeof(dst));

	g2mplsTypes::TELinkId_var lclTel;
	lclTel   = src->teLink;
	dst.tna << lclTel;
	if (!is_addr_null(dst.tna)) {
		BITMASK_BITSET(dst.mask_, tna);
	}

	g2mplsTypes::DLinkId_var lclDl;
	lclDl          = src->dataLink;
	dst.data_link << lclDl;
	if (!is_addr_null(dst.data_link)) {
		BITMASK_BITSET(dst.mask_, data_link);
	}

	g2mplsTypes::labelId_var lclLbl;
	lclLbl      = src->label;
	dst.label  << lclLbl;
	if (!is_label_null(dst.label)) {
		BITMASK_BITSET(dst.mask_, label);
	}

	return dst;
}

g2mplsTypes::tnResource_var &
operator<<(g2mplsTypes::tnResource_var &           dst,
	   const net_res_spec_t &                  src)
{
	dst = new g2mplsTypes::tnResource;

	g2mplsTypes::TELinkId_var  lclTel;
	lclTel << src.tna;
	dst->teLink = lclTel;

	g2mplsTypes::DLinkId_var  lclDl;
	lclDl << src.data_link;
	dst->dataLink = lclDl;

	g2mplsTypes::labelId_var  lclLbl;
	lclLbl << src.label;
	dst->label = lclLbl;

	return dst;
}


// tnaResource
net_res_spec_t &
operator<<(net_res_spec_t &                         dst,
	   const g2mplsTypes::tnaResource_var &     src)
{
	memset(&dst, 0, sizeof(dst));

	g2mplsTypes::tnaId_var lclTna;
	lclTna   = src->tna;
	dst.tna << lclTna;
	if (!is_addr_null(dst.tna)) {
		BITMASK_BITSET(dst.mask_, tna);
	}

	g2mplsTypes::DLinkId_var lclDl;
	lclDl          = src->dataLink;
	dst.data_link << lclDl;
	if (!is_addr_null(dst.data_link)) {
		BITMASK_BITSET(dst.mask_, data_link);
	}

	g2mplsTypes::labelId_var lclLbl;
	lclLbl      = src->label;
	dst.label  << lclLbl;
	if (!is_label_null(dst.label)) {
		BITMASK_BITSET(dst.mask_, label);
	}

	return dst;
}

g2mplsTypes::tnaResource_var &
operator<<(g2mplsTypes::tnaResource_var &           dst,
	   const net_res_spec_t &                   src)
{
	dst = new g2mplsTypes::tnaResource;

	g2mplsTypes::tnaId_var  lclTna;
	lclTna << src.tna;
	dst->tna = lclTna;

	g2mplsTypes::DLinkId_var  lclDl;
	lclDl << src.data_link;
	dst->dataLink = lclDl;

	g2mplsTypes::labelId_var  lclLbl;
	lclLbl << src.label;
	dst->label = lclLbl;

	return dst;
}

#define RANGE_COPY_C2G(DST, SRC)		\
	DST.lower_bound  = SRC.lowerBound;	\
	DST.upper_bound  = SRC.upperBound;	\
	DST.lb_included  = SRC.lbIncluded;	\
	DST.ub_included  = SRC.ubIncluded;

#define RANGE_CONVERT_C2G(DST,GNAME,SRC,CNAME)		\
	if (SRC->CNAME.valid) {				\
		BITMASK_BITSET(DST.mask_, GNAME);	\
		RANGE_COPY_C2G(DST.GNAME, SRC->CNAME);	\
	}

#define RANGE_COPY_G2C(DST, SRC)			\
	DST.lowerBound  = SRC.lower_bound;		\
	DST.upperBound  = SRC.upper_bound;		\
	DST.lbIncluded  = SRC.lb_included;		\
	DST.ubIncluded  = SRC.ub_included;

#define RANGE_COPY_ZERO2C(DST)			        \
	DST.lowerBound  = 0;		                \
	DST.upperBound  = 0;		                \
	DST.lbIncluded  = 0;		                \
	DST.ubIncluded  = 0;


#define RANGE_CONVERT_G2C(DST,CNAME,SRC,GNAME)			\
	if (BITMASK_BITTEST(SRC.mask_, GNAME)) {		\
		DST->CNAME.valid = true;			\
		RANGE_COPY_G2C(DST->CNAME, SRC.GNAME);		\
	} else {						\
	        DST->CNAME.valid = false;                       \
		RANGE_COPY_ZERO2C(DST->CNAME);                  \
	}

// gridParams
grid_res_spec_t &
operator<<(grid_res_spec_t &                        dst,
	   const g2mplsTypes::gridParams_var &      src)
{
	memset(&dst, 0, sizeof(dst));

	if (src->application.valid) {
		BITMASK_BITSET(dst.mask_, application);

		switch (src->application.type) {
			case g2mplsTypes::GRIDAPPLICATIONTYPE_UNKNOWN:
				dst.application.type = GRID_APPNAME_UNKNOWN;
				break;
			case g2mplsTypes::GRIDAPPLICATIONTYPE_WISDOM :
				dst.application.type = GRID_APPNAME_WISDOM;
				break;
			case g2mplsTypes::GRIDAPPLICATIONTYPE_KODAVIS:
				dst.application.type = GRID_APPNAME_KODAVIS;
				break;
			case g2mplsTypes::GRIDAPPLICATIONTYPE_TOPS:
				dst.application.type = GRID_APPNAME_TOPS;
				break;
			case g2mplsTypes::GRIDAPPLICATIONTYPE_DDSS:
				dst.application.type = GRID_APPNAME_DDSS;
				break;
			case g2mplsTypes::GRIDAPPLICATIONTYPE_INCA:
				dst.application.type = GRID_APPNAME_INCA;
				break;
			case g2mplsTypes::GRIDAPPLICATIONTYPE_OTHER:
				dst.application.type = GRID_APPNAME_OTHER;
				break;
			default:
				throw out_of_range("GRID APPLICATION "
						   "out-of-range");
				break;
		}

		dst.application.version.major_revision =
			src->application.mjrRev;
		dst.application.version.minor_revision =
			src->application.mnrRev;
		dst.application.version.build_fix      =
			src->application.bldFix;
	}

	if (src->candHost._d() != g2mplsTypes::GRIDHOSTTYPE_UNDEFINED) {
		BITMASK_BITSET(dst.mask_, cand_host);
		g2mplsTypes::gridHostId_var  cHost;
		cHost = src->candHost;
		dst.cand_host << cHost;
	}

	if (src->fileSystemRes.valid) {
		BITMASK_BITSET(dst.mask_, fs_resources);

		switch (src->fileSystemRes.fsName) {
			case g2mplsTypes::GRIDFSNAME_UNKNOWN:
				dst.fs_resources.fs_name = FS_NAME_UNKNOWN;
				break;
			case g2mplsTypes::GRIDFSNAME_HOME:
				dst.fs_resources.fs_name = FS_NAME_HOME;
				break;
			case g2mplsTypes::GRIDFSNAME_ROOT:
				dst.fs_resources.fs_name = FS_NAME_ROOT;
				break;
			case g2mplsTypes::GRIDFSNAME_SCRATCH:
				dst.fs_resources.fs_name = FS_NAME_SCRATCH;
				break;
			case g2mplsTypes::GRIDFSNAME_TMP:
				dst.fs_resources.fs_name = FS_NAME_TMP;
				break;
			case g2mplsTypes::GRIDFSNAME_OTHER:
				dst.fs_resources.fs_name = FS_NAME_OTHER;
				break;
			default:
				throw out_of_range("GRID FS NAME "
						   "out-of-range");
				break;
		}

		switch (src->fileSystemRes.fsType) {
			case g2mplsTypes::GRIDFSTYPE_UNKNOWN:
				dst.fs_resources.fs_type = FS_TYPE_UNKNOWN;
				break;
			case g2mplsTypes::GRIDFSTYPE_SWAP:
				dst.fs_resources.fs_type = FS_TYPE_SWAP;
				break;
			case g2mplsTypes::GRIDFSTYPE_TEMPORARY:
				dst.fs_resources.fs_type = FS_TYPE_TEMPORARY;
				break;
			case g2mplsTypes::GRIDFSTYPE_SPOOL:
				dst.fs_resources.fs_type = FS_TYPE_SPOOL;
				break;
			case g2mplsTypes::GRIDFSTYPE_NORMAL:
				dst.fs_resources.fs_type = FS_TYPE_NORMAL;
				break;
			case g2mplsTypes::GRIDFSTYPE_OTHER:
				dst.fs_resources.fs_type = FS_TYPE_OTHER;
				break;
			default:
				throw out_of_range("GRID FS TYPE "
						   "out-of-range");
				break;
		}

		RANGE_COPY_C2G(dst.fs_resources.disk_space,
			       src->fileSystemRes.diskSpace);

		dst.fs_resources.mount_point  =
			new std::string(src->fileSystemRes.mountPoint);
		dst.fs_resources.mount_source =
			new std::string(src->fileSystemRes.mountSource);
	}

	if (src->systemCaps.valid) {
		BITMASK_BITSET(dst.mask_, sys_caps);

		switch (src->systemCaps.os.type) {
			case g2mplsTypes::GRIDOSTYPE_UNKNOWN:
				dst.sys_caps.os_type = OS_UNKNOWN;
				break;
			case g2mplsTypes::GRIDOSTYPE_MACOS:
				dst.sys_caps.os_type = OS_MACOS;
				break;
			case g2mplsTypes::GRIDOSTYPE_ATTUNIX:
				dst.sys_caps.os_type = OS_ATTUNIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_DGUX:
				dst.sys_caps.os_type = OS_DGUX;
				break;
			case g2mplsTypes::GRIDOSTYPE_DECNT:
				dst.sys_caps.os_type = OS_DECNT;
				break;
			case g2mplsTypes::GRIDOSTYPE_TRU64_UNIX:
				dst.sys_caps.os_type = OS_TRU64_UNIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_OPENVMS:
				dst.sys_caps.os_type = OS_OPENVMS;
				break;
			case g2mplsTypes::GRIDOSTYPE_HPUX:
				dst.sys_caps.os_type = OS_HPUX;
				break;
			case g2mplsTypes::GRIDOSTYPE_AIX:
				dst.sys_caps.os_type = OS_AIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_MVS:
				dst.sys_caps.os_type = OS_MVS;
				break;
			case g2mplsTypes::GRIDOSTYPE_OS400:
				dst.sys_caps.os_type = OS_OS400;
				break;
			case g2mplsTypes::GRIDOSTYPE_OS_2:
				dst.sys_caps.os_type = OS_OS_2;
				break;
			case g2mplsTypes::GRIDOSTYPE_JAVAVM:
				dst.sys_caps.os_type = OS_JAVAVM;
				break;
			case g2mplsTypes::GRIDOSTYPE_MSDOS:
				dst.sys_caps.os_type = OS_MSDOS;
				break;
			case g2mplsTypes::GRIDOSTYPE_WIN3X:
				dst.sys_caps.os_type = OS_WIN3X;
				break;
			case g2mplsTypes::GRIDOSTYPE_WIN95:
				dst.sys_caps.os_type = OS_WIN95;
				break;
			case g2mplsTypes::GRIDOSTYPE_WIN98:
				dst.sys_caps.os_type = OS_WIN98;
				break;
			case g2mplsTypes::GRIDOSTYPE_WINNT:
				dst.sys_caps.os_type = OS_WINNT;
				break;
			case g2mplsTypes::GRIDOSTYPE_WINCE:
				dst.sys_caps.os_type = OS_WINCE;
				break;
			case g2mplsTypes::GRIDOSTYPE_NCR3000:
				dst.sys_caps.os_type = OS_NCR3000;
				break;
			case g2mplsTypes::GRIDOSTYPE_NETWARE:
				dst.sys_caps.os_type = OS_NETWARE;
				break;
			case g2mplsTypes::GRIDOSTYPE_OSF:
				dst.sys_caps.os_type = OS_OSF;
				break;
			case g2mplsTypes::GRIDOSTYPE_DC_OS:
				dst.sys_caps.os_type = OS_DC_OS;
				break;
			case g2mplsTypes::GRIDOSTYPE_RELIANT_UNIX:
				dst.sys_caps.os_type = OS_RELIANT_UNIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_SCO_UNIXWARE:
				dst.sys_caps.os_type = OS_SCO_UNIXWARE;
				break;
			case g2mplsTypes::GRIDOSTYPE_SCO_OPENSERVER:
				dst.sys_caps.os_type = OS_SCO_OPENSERVER;
				break;
			case g2mplsTypes::GRIDOSTYPE_SEQUENT:
				dst.sys_caps.os_type = OS_SEQUENT;
				break;
			case g2mplsTypes::GRIDOSTYPE_IRIX:
				dst.sys_caps.os_type = OS_IRIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_SOLARIS:
				dst.sys_caps.os_type = OS_SOLARIS;
				break;
			case g2mplsTypes::GRIDOSTYPE_SUNOS:
				dst.sys_caps.os_type = OS_SUNOS;
				break;
			case g2mplsTypes::GRIDOSTYPE_U6000:
				dst.sys_caps.os_type = OS_U6000;
				break;
			case g2mplsTypes::GRIDOSTYPE_ASERIES:
				dst.sys_caps.os_type = OS_ASERIES;
				break;
			case g2mplsTypes::GRIDOSTYPE_TANDEMNSK:
				dst.sys_caps.os_type = OS_TANDEMNSK;
				break;
			case g2mplsTypes::GRIDOSTYPE_TANDEMNT:
				dst.sys_caps.os_type = OS_TANDEMNT;
				break;
			case g2mplsTypes::GRIDOSTYPE_BS2000:
				dst.sys_caps.os_type = OS_BS2000;
				break;
			case g2mplsTypes::GRIDOSTYPE_LINUX:
				dst.sys_caps.os_type = OS_LINUX;
				break;
			case g2mplsTypes::GRIDOSTYPE_LYNX:
				dst.sys_caps.os_type = OS_LYNX;
				break;
			case g2mplsTypes::GRIDOSTYPE_XENIX:
				dst.sys_caps.os_type = OS_XENIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_VM:
				dst.sys_caps.os_type = OS_VM;
				break;
			case g2mplsTypes::GRIDOSTYPE_INTERACTIVE_UNIX:
				dst.sys_caps.os_type = OS_INTERACTIVE_UNIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_BSDUNIX:
				dst.sys_caps.os_type = OS_BSDUNIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_FREEBSD:
				dst.sys_caps.os_type = OS_FREEBSD;
				break;
			case g2mplsTypes::GRIDOSTYPE_NETBSD:
				dst.sys_caps.os_type = OS_NETBSD;
				break;
			case g2mplsTypes::GRIDOSTYPE_GNU_HURD:
				dst.sys_caps.os_type = OS_GNU_HURD;
				break;
			case g2mplsTypes::GRIDOSTYPE_OS9:
				dst.sys_caps.os_type = OS_OS9;
				break;
			case g2mplsTypes::GRIDOSTYPE_MACH_KERNEL:
				dst.sys_caps.os_type = OS_MACH_KERNEL;
				break;
			case g2mplsTypes::GRIDOSTYPE_INFERNO:
				dst.sys_caps.os_type = OS_INFERNO;
				break;
			case g2mplsTypes::GRIDOSTYPE_QNX:
				dst.sys_caps.os_type = OS_QNX;
				break;
			case g2mplsTypes::GRIDOSTYPE_EPOC:
				dst.sys_caps.os_type = OS_EPOC;
				break;
			case g2mplsTypes::GRIDOSTYPE_IXWORKS:
				dst.sys_caps.os_type = OS_IXWORKS;
				break;
			case g2mplsTypes::GRIDOSTYPE_VXWORKS:
				dst.sys_caps.os_type = OS_VXWORKS;
				break;
			case g2mplsTypes::GRIDOSTYPE_MINT:
				dst.sys_caps.os_type = OS_MINT;
				break;
			case g2mplsTypes::GRIDOSTYPE_BEOS:
				dst.sys_caps.os_type = OS_BEOS;
				break;
			case g2mplsTypes::GRIDOSTYPE_HP_MPE:
				dst.sys_caps.os_type = OS_HP_MPE;
				break;
			case g2mplsTypes::GRIDOSTYPE_NEXTSTEP:
				dst.sys_caps.os_type = OS_NEXTSTEP;
				break;
			case g2mplsTypes::GRIDOSTYPE_PALMPILOT:
				dst.sys_caps.os_type = OS_PALMPILOT;
				break;
			case g2mplsTypes::GRIDOSTYPE_RHAPSODY:
				dst.sys_caps.os_type = OS_RHAPSODY;
				break;
			case g2mplsTypes::GRIDOSTYPE_WINDOWS_2000:
				dst.sys_caps.os_type = OS_WINDOWS_2000;
				break;
			case g2mplsTypes::GRIDOSTYPE_DEDICATED:
				dst.sys_caps.os_type = OS_DEDICATED;
				break;
			case g2mplsTypes::GRIDOSTYPE_OS_390:
				dst.sys_caps.os_type = OS_OS_390;
				break;
			case g2mplsTypes::GRIDOSTYPE_VSE:
				dst.sys_caps.os_type = OS_VSE;
				break;
			case g2mplsTypes::GRIDOSTYPE_TPF:
				dst.sys_caps.os_type = OS_TPF;
				break;
			case g2mplsTypes::GRIDOSTYPE_WINDOWS_R_ME:
				dst.sys_caps.os_type = OS_WINDOWS_R_ME;
				break;
			case g2mplsTypes::GRIDOSTYPE_CALDERA_OPEN_UNIX:
				dst.sys_caps.os_type = OS_CALDERA_OPEN_UNIX;
				break;
			case g2mplsTypes::GRIDOSTYPE_OPENBSD:
				dst.sys_caps.os_type = OS_OPENBSD;
				break;
			case g2mplsTypes::GRIDOSTYPE_WINDOWS_XP:
				dst.sys_caps.os_type = OS_WINDOWS_XP;
				break;
			case g2mplsTypes::GRIDOSTYPE_Z_OS:
				dst.sys_caps.os_type = OS_Z_OS;
				break;
			case g2mplsTypes::GRIDOSTYPE_OTHER:
				dst.sys_caps.os_type = OS_OTHER;
				break;
			default:
				throw out_of_range("GRID OS TYPE "
						   "out-of-range");
				break;
		}

		dst.sys_caps.os_version.major_revision =
			src->systemCaps.os.mjrRev;
		dst.sys_caps.os_version.minor_revision =
			src->systemCaps.os.mnrRev;
		dst.sys_caps.os_version.build_fix      =
			src->systemCaps.os.bldFix;

		switch (src->systemCaps.cpuArch) {
			case g2mplsTypes::GRIDCPUARCH_UNKNOWN:
				dst.sys_caps.cpu_arch = CPU_UNKNOWN;
				break;
			case g2mplsTypes::GRIDCPUARCH_SPARC:
				dst.sys_caps.cpu_arch = CPU_SPARC;
				break;
			case g2mplsTypes::GRIDCPUARCH_POWERPC:
				dst.sys_caps.cpu_arch = CPU_POWERPC;
				break;
			case g2mplsTypes::GRIDCPUARCH_X86:
				dst.sys_caps.cpu_arch = CPU_X86;
				break;
			case g2mplsTypes::GRIDCPUARCH_X86_32:
				dst.sys_caps.cpu_arch = CPU_X86_32;
				break;
			case g2mplsTypes::GRIDCPUARCH_X86_64:
				dst.sys_caps.cpu_arch = CPU_X86_64;
				break;
			case g2mplsTypes::GRIDCPUARCH_PARISC:
				dst.sys_caps.cpu_arch = CPU_PARISC;
				break;
			case g2mplsTypes::GRIDCPUARCH_MIPS:
				dst.sys_caps.cpu_arch = CPU_MIPS;
				break;
			case g2mplsTypes::GRIDCPUARCH_IA64:
				dst.sys_caps.cpu_arch = CPU_IA64;
				break;
			case g2mplsTypes::GRIDCPUARCH_ARM:
				dst.sys_caps.cpu_arch = CPU_ARM;
				break;
			case g2mplsTypes::GRIDCPUARCH_OTHER:
				dst.sys_caps.cpu_arch = CPU_OTHER;
				break;
			default:
				throw out_of_range("GRID CPU ARCH "
						   "out-of-range");
				break;
		}

		dst.sys_caps.exclusive_access =
			src->systemCaps.exclusiveAccess ? 1 : 0;
	}

	RANGE_CONVERT_C2G(dst, ind_cpu_speed , src, indCpuSpeed );
	RANGE_CONVERT_C2G(dst, ind_cpu_time  , src, indCpuTime  );
	RANGE_CONVERT_C2G(dst, ind_cpu_count , src, indCpuCount );
	RANGE_CONVERT_C2G(dst, ind_net_bw    , src, indNetBw    );
	RANGE_CONVERT_C2G(dst, ind_phy_mem   , src, indPhyMem   );
	RANGE_CONVERT_C2G(dst, ind_vir_mem   , src, indVirMem   );
	RANGE_CONVERT_C2G(dst, ind_disk_space, src, indDiskSpace);
	RANGE_CONVERT_C2G(dst, tot_cpu_time  , src, totCpuTime  );
	RANGE_CONVERT_C2G(dst, tot_cpu_count , src, totCpuCount );
	RANGE_CONVERT_C2G(dst, tot_phy_mem   , src, totPhyMem   );
	RANGE_CONVERT_C2G(dst, tot_vir_mem   , src, totVirMem   );
	RANGE_CONVERT_C2G(dst, tot_disk_space, src, totDiskSpace);
	RANGE_CONVERT_C2G(dst, tot_res_count , src, totResCount );

	// Grid Data Staging
	if (src->dataStaging.valid) {
		BITMASK_BITSET(dst.mask_, data_staging);

		switch (src->dataStaging.fsName) {
			case g2mplsTypes::GRIDFSNAME_UNKNOWN:
				dst.data_staging.fs_name = FS_NAME_UNKNOWN;
				break;
			case g2mplsTypes::GRIDFSNAME_HOME:
				dst.data_staging.fs_name = FS_NAME_HOME;
				break;
			case g2mplsTypes::GRIDFSNAME_ROOT:
				dst.data_staging.fs_name = FS_NAME_ROOT;
				break;
			case g2mplsTypes::GRIDFSNAME_SCRATCH:
				dst.data_staging.fs_name = FS_NAME_SCRATCH;
				break;
			case g2mplsTypes::GRIDFSNAME_TMP:
				dst.data_staging.fs_name = FS_NAME_TMP;
				break;
			case g2mplsTypes::GRIDFSNAME_OTHER:
				dst.data_staging.fs_name = FS_NAME_OTHER;
				break;
			default:
				throw out_of_range("GRID DS FS NAME "
						   "out-of-range");
				break;
		}

		switch (src->dataStaging.cf) {
			case g2mplsTypes::GRIDSTAGINGCF_UNKNOWN:
				dst.data_staging.creation_flag = CF_UNKNOWN;
				break;
			case g2mplsTypes::GRIDSTAGINGCF_OVERWRITE:
				dst.data_staging.creation_flag = CF_OVERWRITE;
				break;
			case g2mplsTypes::GRIDSTAGINGCF_APPEND:
				dst.data_staging.creation_flag = CF_APPEND;
				break;
			case g2mplsTypes::GRIDSTAGINGCF_DONTOVERWRITE:
				dst.data_staging.creation_flag =
					CF_DONTOVERWRITE;
				break;
			default:
				throw out_of_range("GRID DS CREAT. FLAG "
						   "out-of-range");
				break;
		}

		dst.data_staging.del_on_termination =
			src->dataStaging.delOnTermination ? 1 : 0;

		dst.data_staging.filename     =
			new std::string(src->dataStaging.fileName);
		dst.data_staging.source       =
			new std::string(src->dataStaging.source);
		dst.data_staging.target       =
			new std::string(src->dataStaging.target);
	}

	if (src->gridSite != 0) {
		BITMASK_BITSET(dst.mask_, grid_site_id);
		dst.grid_site_id = src->gridSite;
	}

	return dst;
}

g2mplsTypes::gridParams_var &
operator<<(g2mplsTypes::gridParams_var &            dst,
	   const grid_res_spec_t &                  src)
{
	dst = new g2mplsTypes::gridParams;

	if (BITMASK_BITTEST(src.mask_, application)) {
		dst->application.valid = true;

		switch (src.application.type) {
			case GRID_APPNAME_UNKNOWN:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_UNKNOWN;
				break;
			case GRID_APPNAME_WISDOM:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_WISDOM;
				break;
			case GRID_APPNAME_KODAVIS:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_KODAVIS;
				break;
			case GRID_APPNAME_TOPS:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_TOPS;
				break;
			case GRID_APPNAME_DDSS:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_DDSS;
				break;
			case GRID_APPNAME_INCA:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_INCA;
				break;
			case GRID_APPNAME_OTHER:
				dst->application.type =
					g2mplsTypes::
					GRIDAPPLICATIONTYPE_OTHER;
				break;
			default:
				throw out_of_range("GRID APPLICATION "
						   "out-of-range");
				break;
		}

		dst->application.mjrRev =
			src.application.version.major_revision;
		dst->application.mnrRev =
			src.application.version.minor_revision;
		dst->application.bldFix =
			src.application.version.build_fix;
	} else {
	        dst->application.valid  = false;
	        dst->application.type   = g2mplsTypes::GRIDAPPLICATIONTYPE_UNKNOWN;
		dst->application.mjrRev = 0;
		dst->application.mnrRev = 0;
		dst->application.mnrRev = 0;
	}

	if (BITMASK_BITTEST(src.mask_, cand_host)) {
		g2mplsTypes::gridHostId_var  cHost;
		cHost << src.cand_host;
		dst->candHost = cHost;
	} else {
	        dst->candHost.value(0);
	}

	if (BITMASK_BITTEST(src.mask_, fs_resources)) {
		dst->fileSystemRes.valid = true;

		switch (src.fs_resources.fs_name) {
			case FS_NAME_UNKNOWN:
				dst->fileSystemRes.fsName =
					g2mplsTypes::GRIDFSNAME_UNKNOWN;
				break;
			case FS_NAME_HOME:
				dst->fileSystemRes.fsName  =
					g2mplsTypes::GRIDFSNAME_HOME;
				break;
			case FS_NAME_ROOT:
				dst->fileSystemRes.fsName  =
					g2mplsTypes::GRIDFSNAME_ROOT;
				break;
			case FS_NAME_SCRATCH:
				dst->fileSystemRes.fsName  =
					g2mplsTypes::GRIDFSNAME_SCRATCH;
				break;
			case FS_NAME_TMP:
				dst->fileSystemRes.fsName  =
					g2mplsTypes::GRIDFSNAME_TMP;
				break;
			case FS_NAME_OTHER:
				dst->fileSystemRes.fsName  =
					g2mplsTypes::GRIDFSNAME_OTHER;
				break;
			default:
				throw out_of_range("GRID FS NAME "
						   "out-of-range");
				break;
		}

		switch (src.fs_resources.fs_type) {
			case FS_TYPE_UNKNOWN:
				dst->fileSystemRes.fsType =
					g2mplsTypes::GRIDFSTYPE_UNKNOWN;
				break;
			case FS_TYPE_SWAP:
				dst->fileSystemRes.fsType =
					g2mplsTypes::GRIDFSTYPE_SWAP;
				break;
			case FS_TYPE_TEMPORARY:
				dst->fileSystemRes.fsType =
					g2mplsTypes::GRIDFSTYPE_TEMPORARY;
				break;
			case FS_TYPE_SPOOL:
				dst->fileSystemRes.fsType =
					g2mplsTypes::GRIDFSTYPE_SPOOL;
				break;
			case FS_TYPE_NORMAL:
				dst->fileSystemRes.fsType =
					g2mplsTypes::GRIDFSTYPE_NORMAL;
				break;
			case FS_TYPE_OTHER:
				dst->fileSystemRes.fsType =
					g2mplsTypes::GRIDFSTYPE_OTHER;
				break;
			default:
				throw out_of_range("GRID FS TYPE "
						   "out-of-range");
				break;
		}

		RANGE_COPY_G2C(dst->fileSystemRes.diskSpace,
			       src.fs_resources.disk_space);

		if (src.fs_resources.mount_point) {
			std::string * tmp;

			tmp = src.fs_resources.mount_point;

			dst->fileSystemRes.mountPoint =
				CORBA::string_dup(tmp->c_str());
		}

		if (src.fs_resources.mount_source) {
			std::string * tmp;

			tmp = src.fs_resources.mount_source;

			dst->fileSystemRes.mountSource =
				CORBA::string_dup(tmp->c_str());
		}
	} else {
	        dst->fileSystemRes.valid       = false;
	        dst->fileSystemRes.fsName      = g2mplsTypes::GRIDFSNAME_UNKNOWN;
	        dst->fileSystemRes.fsType      = g2mplsTypes::GRIDFSTYPE_UNKNOWN;
		RANGE_COPY_ZERO2C(dst->fileSystemRes.diskSpace);
		dst->fileSystemRes.mountPoint  = CORBA::string_dup("");
		dst->fileSystemRes.mountSource = CORBA::string_dup("");
	}

	if (BITMASK_BITTEST(src.mask_, sys_caps)) {
		dst->systemCaps.valid    = true;
		dst->systemCaps.os.valid = true;

		switch (src.sys_caps.os_type) {
			case OS_UNKNOWN:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_UNKNOWN;
				break;
			case OS_MACOS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_MACOS;
				break;
			case OS_ATTUNIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_ATTUNIX;
				break;
			case OS_DGUX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_DGUX;
				break;
			case OS_DECNT:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_DECNT;
				break;
			case OS_TRU64_UNIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_TRU64_UNIX;
				break;
			case OS_OPENVMS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OPENVMS;
				break;
			case OS_HPUX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_HPUX;
				break;
			case OS_AIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_AIX;
				break;
			case OS_MVS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_MVS;
				break;
			case OS_OS400:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OS400;
				break;
			case OS_OS_2:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OS_2;
				break;
			case OS_JAVAVM:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_JAVAVM;
				break;
			case OS_MSDOS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_MSDOS;
				break;
			case OS_WIN3X:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WIN3X;
				break;
			case OS_WIN95:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WIN95;
				break;
			case OS_WIN98:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WIN98;
				break;
			case OS_WINNT:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WINNT;
				break;
			case OS_WINCE:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WINCE;
				break;
			case OS_NCR3000:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_NCR3000;
				break;
			case OS_NETWARE:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_NETWARE;
				break;
			case OS_OSF:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OSF;
				break;
			case OS_DC_OS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_DC_OS;
				break;
			case OS_RELIANT_UNIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_RELIANT_UNIX;
				break;
			case OS_SCO_UNIXWARE:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_SCO_UNIXWARE;
				break;
			case OS_SCO_OPENSERVER:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_SCO_OPENSERVER;
				break;
			case OS_SEQUENT:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_SEQUENT;
				break;
			case OS_IRIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_IRIX;
				break;
			case OS_SOLARIS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_SOLARIS;
				break;
			case OS_SUNOS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_SUNOS;
				break;
			case OS_U6000:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_U6000;
				break;
			case OS_ASERIES:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_ASERIES;
				break;
			case OS_TANDEMNSK:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_TANDEMNSK;
				break;
			case OS_TANDEMNT:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_TANDEMNT;
				break;
			case OS_BS2000:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_BS2000;
				break;
			case OS_LINUX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_LINUX;
				break;
			case OS_LYNX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_LYNX;
				break;
			case OS_XENIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_XENIX;
				break;
			case OS_VM:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_VM;
				break;
			case OS_INTERACTIVE_UNIX:
				dst->systemCaps.os.type =
					g2mplsTypes::
					GRIDOSTYPE_INTERACTIVE_UNIX;
				break;
			case OS_BSDUNIX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_BSDUNIX;
				break;
			case OS_FREEBSD:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_FREEBSD;
				break;
			case OS_NETBSD:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_NETBSD;
				break;
			case OS_GNU_HURD:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_GNU_HURD;
				break;
			case OS_OS9:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OS9;
				break;
			case OS_MACH_KERNEL:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_MACH_KERNEL;
				break;
			case OS_INFERNO:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_INFERNO;
				break;
			case OS_QNX:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_QNX;
				break;
			case OS_EPOC:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_EPOC;
				break;
			case OS_IXWORKS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_IXWORKS;
				break;
			case OS_VXWORKS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_VXWORKS;
				break;
			case OS_MINT:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_MINT;
				break;
			case OS_BEOS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_BEOS;
				break;
			case OS_HP_MPE:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_HP_MPE;
				break;
			case OS_NEXTSTEP:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_NEXTSTEP;
				break;
			case OS_PALMPILOT:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_PALMPILOT;
				break;
			case OS_RHAPSODY:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_RHAPSODY;
				break;
			case OS_WINDOWS_2000:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WINDOWS_2000;
				break;
			case OS_DEDICATED:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_DEDICATED;
				break;
			case OS_OS_390:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OS_390;
				break;
			case OS_VSE:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_VSE;
				break;
			case OS_TPF:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_TPF;
				break;
			case OS_WINDOWS_R_ME:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WINDOWS_R_ME;
				break;
			case OS_CALDERA_OPEN_UNIX:
				dst->systemCaps.os.type =
					g2mplsTypes::
					GRIDOSTYPE_CALDERA_OPEN_UNIX;
				break;
			case OS_OPENBSD:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OPENBSD;
				break;
			case OS_WINDOWS_XP:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_WINDOWS_XP;
				break;
			case OS_Z_OS:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_Z_OS;
				break;
			case OS_OTHER:
				dst->systemCaps.os.type =
					g2mplsTypes::GRIDOSTYPE_OTHER;
				break;
			default:
				throw out_of_range("GRID OS TYPE "
						   "out-of-range");
				break;
		}

		dst->systemCaps.os.mjrRev =
			src.sys_caps.os_version.major_revision;
		dst->systemCaps.os.mnrRev =
			src.sys_caps.os_version.minor_revision;
		dst->systemCaps.os.bldFix =
			src.sys_caps.os_version.build_fix;

		switch (src.sys_caps.cpu_arch) {
			case CPU_UNKNOWN:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_UNKNOWN;
				break;
			case CPU_SPARC:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_SPARC;
				break;
			case CPU_POWERPC:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_POWERPC;
				break;
			case CPU_X86:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_X86;
				break;
			case CPU_X86_32:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_X86_32;
				break;
			case CPU_X86_64:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_X86_64;
				break;
			case CPU_PARISC:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_PARISC;
				break;
			case CPU_MIPS:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_MIPS;
				break;
			case CPU_IA64:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_IA64;
				break;
			case CPU_ARM:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_ARM;
				break;
			case CPU_OTHER:
				dst->systemCaps.cpuArch =
					g2mplsTypes::GRIDCPUARCH_OTHER;
				break;
			default:
				throw out_of_range("GRID CPU ARCH "
						   "out-of-range");
				break;
		}

		dst->systemCaps.exclusiveAccess =
			src.sys_caps.exclusive_access ? true : false ;
	} else {
	        dst->systemCaps.valid           = false;
	        dst->systemCaps.os.valid        = false;
		dst->systemCaps.os.type         = g2mplsTypes::GRIDOSTYPE_UNKNOWN;
		dst->systemCaps.cpuArch         = g2mplsTypes::GRIDCPUARCH_UNKNOWN;
		dst->systemCaps.os.mjrRev       = 0;
		dst->systemCaps.os.mnrRev       = 0;
		dst->systemCaps.os.bldFix       = 0;
		dst->systemCaps.exclusiveAccess = false;
	}

	RANGE_CONVERT_G2C(dst, indCpuSpeed , src, ind_cpu_speed );
	RANGE_CONVERT_G2C(dst, indCpuTime  , src, ind_cpu_time  );
	RANGE_CONVERT_G2C(dst, indCpuCount , src, ind_cpu_count );
	RANGE_CONVERT_G2C(dst, indNetBw    , src, ind_net_bw    );
	RANGE_CONVERT_G2C(dst, indPhyMem   , src, ind_phy_mem   );
	RANGE_CONVERT_G2C(dst, indVirMem   , src, ind_vir_mem   );
	RANGE_CONVERT_G2C(dst, indDiskSpace, src, ind_disk_space);
	RANGE_CONVERT_G2C(dst, totCpuTime  , src, tot_cpu_time  );
	RANGE_CONVERT_G2C(dst, totCpuCount , src, tot_cpu_count );
	RANGE_CONVERT_G2C(dst, totPhyMem   , src, tot_phy_mem   );
	RANGE_CONVERT_G2C(dst, totVirMem   , src, tot_vir_mem   );
	RANGE_CONVERT_G2C(dst, totDiskSpace, src, tot_disk_space);
	RANGE_CONVERT_G2C(dst, totResCount , src, tot_res_count );

	// Grid Data Staging
	if (BITMASK_BITTEST(src.mask_, data_staging)) {
		dst->dataStaging.valid = true;

		switch (src.data_staging.fs_name) {
			case FS_NAME_UNKNOWN:
				dst->dataStaging.fsName =
					g2mplsTypes::GRIDFSNAME_UNKNOWN;
				break;
			case FS_NAME_HOME:
				dst->dataStaging.fsName =
					g2mplsTypes::GRIDFSNAME_HOME;
				break;
			case FS_NAME_ROOT:
				dst->dataStaging.fsName =
					g2mplsTypes::GRIDFSNAME_ROOT;
				break;
			case FS_NAME_SCRATCH:
				dst->dataStaging.fsName =
					g2mplsTypes::GRIDFSNAME_SCRATCH;
				break;
			case FS_NAME_TMP:
				dst->dataStaging.fsName =
					g2mplsTypes::GRIDFSNAME_TMP;
				break;
			case FS_NAME_OTHER:
				dst->dataStaging.fsName =
					g2mplsTypes::GRIDFSNAME_OTHER;
				break;
			default:
				throw out_of_range("GRID DS FS NAME "
						   "out-of-range");
				break;
		}

		switch (src.data_staging.creation_flag) {
			case CF_UNKNOWN:
				dst->dataStaging.cf =
					g2mplsTypes::GRIDSTAGINGCF_UNKNOWN;
				break;
			case CF_OVERWRITE:
				dst->dataStaging.cf =
					g2mplsTypes::GRIDSTAGINGCF_OVERWRITE;
				break;
			case CF_APPEND:
				dst->dataStaging.cf =
					g2mplsTypes::GRIDSTAGINGCF_APPEND;
				break;
			case CF_DONTOVERWRITE:
				dst->dataStaging.cf =
					g2mplsTypes::
					GRIDSTAGINGCF_DONTOVERWRITE;
				break;
			default:
				throw out_of_range("GRID DS CREAT. FLAG "
						   "out-of-range");
				break;
		}

		dst->dataStaging.delOnTermination =
			src.data_staging.del_on_termination ? true : false;

		if (src.data_staging.filename) {
			std::string * tmp;

			tmp = src.data_staging.filename;

			dst->dataStaging.fileName =
				CORBA::string_dup(tmp->c_str());
		}
		if (src.data_staging.source) {
			std::string * tmp;

			tmp = src.data_staging.source;

			dst->dataStaging.source =
				CORBA::string_dup(tmp->c_str());
		}
		if (src.data_staging.target) {
			std::string * tmp;

			tmp = src.data_staging.target;

			dst->dataStaging.target =
				CORBA::string_dup(tmp->c_str());
		}
	} else {
	        dst->dataStaging.valid            = false;
		dst->dataStaging.fsName           = g2mplsTypes::GRIDFSNAME_UNKNOWN;
		dst->dataStaging.cf               = g2mplsTypes::GRIDSTAGINGCF_UNKNOWN;
		dst->dataStaging.delOnTermination = false;
		dst->dataStaging.fileName         = CORBA::string_dup("");
		dst->dataStaging.source           = CORBA::string_dup("");
		dst->dataStaging.target           = CORBA::string_dup("");
	}

	if (BITMASK_BITTEST(src.mask_, grid_site_id)) {
		dst->gridSite = src.grid_site_id;
	} else {
	        dst->gridSite = 0;
	}

	return dst;
}

//
// wdmLambdasBitmap
//
wdm_link_lambdas_bitmap_t &
operator<<(wdm_link_lambdas_bitmap_t &               dst,
	   const g2mplsTypes::wdmLambdasBitmap_var & src)
{
	size_t    size;
	uint8_t * mask;

	memset(&dst, 0, sizeof(dst));

	dst.base_lambda_label = src->baseLambda;
	dst.num_wavelengths   = src->numLambdas;

	dst.bitmap_size = (src->numLambdas % 32 ? (src->numLambdas / 32) + 1 :
			   src->numLambdas / 32);

	size = (size_t) (dst.bitmap_size * 4);
	dst.bitmap_word = (uint32_t *) malloc(size);

	mask = (uint8_t *) dst.bitmap_word;

	for (size_t i = 0; i < src->bitmap.length(); i++) {
		*mask = src->bitmap[i];
		mask++;
	}

	return dst;
}

g2mplsTypes::wdmLambdasBitmap_var &
operator<<(g2mplsTypes::wdmLambdasBitmap_var & dst,
	   const wdm_link_lambdas_bitmap_t &   src)
{
	size_t    size;
	uint8_t * byte;

	dst = new g2mplsTypes::wdmLambdasBitmap;

	dst->baseLambda = src.base_lambda_label;
	dst->numLambdas = src.num_wavelengths;

	size = src.bitmap_size * 4;

	g2mplsTypes::bitmapSeq_var lambdaBitmap;
	{
		g2mplsTypes::bitmapSeq * tmp;
		tmp = new g2mplsTypes::bitmapSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::bitmapSeq");
		}
		lambdaBitmap = tmp;
	}
	lambdaBitmap->length(size);

	byte = (uint8_t *) src.bitmap_word;
	if (byte == NULL) {
		dst->bitmap = lambdaBitmap;
		return dst;
	}

	for (size_t i = 0; i < size; i++) {
		lambdaBitmap[i] = *byte;
		byte++;
	}
	dst->bitmap = lambdaBitmap;

	return dst;
}

//
// errorInfo
//

error_info_t &
operator<<(error_info_t &                           dst,
	   const g2mplsTypes::errorInfo_var &       src)
{
	memset(&dst, 0, sizeof(dst));

	dst.flags    = src->stateRemoved ? 0x04 : 0x00;
	dst.err_code = 24;
	dst.err_val  = 1;
	//dst.err_code = src->errorCode;
	//dst.err_val  = src->errorValue;
	dst.node_id  = src->erroredNode;
}

g2mplsTypes::errorInfo_var &
operator<<(g2mplsTypes::errorInfo_var &             dst,
	   const error_info_t &                     src)
{
	dst = new g2mplsTypes::errorInfo;

	//dst->errorCode    = src.err_code;
	//dst->errorValue   = src.err_val;
	//TMP - FIXME
	dst->errorCode  = g2mplsTypes::G2RSVPTEERRORCODE_X;
	dst->errorValue = g2mplsTypes::G2RSVPTEERRORVALUE_X;
	//
	dst->erroredNode  = src.node_id;
	dst->stateRemoved = src.flags & 0x04 ? true : false;
}


#endif // __cpluscplus

#endif // HAVE_OMNIORB
