//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#ifndef __G2PCERA_ALGOVIS_HH__
#define __G2PCERA_ALGOVIS_HH__

#include <list>
#include <map>
#include <iterator>
#include <string>
#include <assert.h>

#include "g2pcera_trace.hh"
#include "g2pcera_common.hh"
#include "g2pcera_dm.hh"

namespace PCERA {

	class topologyVisitor {
	public:
		virtual ~topologyVisitor(void) {}

		virtual void	visitTopology(Topology *		p) = 0;

		virtual void	visitNode(Node *			p) = 0;

		virtual void	visitNetNode(NetNode *			p) = 0;
		virtual void	visitTna(Tna *				p) = 0;
		virtual void	visitTeLink(TeLink *			p) = 0;

		virtual void	visitGridNode(GridNode *		p) = 0;
		virtual void	visitGridService(GridService *		p) = 0;
		virtual void	visitGridCompElem(GridCompElem *	p) = 0;
		virtual void	visitGridSubCluster(GridSubCluster *	p) = 0;
		virtual void	visitGridStorageElem(GridStorageElem *	p) = 0;

	protected:
		topologyVisitor(void) {}
		topologyVisitor(const topologyVisitor &vis);
		const topologyVisitor &operator=(const topologyVisitor & vis);
	};


	class algoStubVisitor: public topologyVisitor {
	public:
		algoStubVisitor(void);
		virtual ~algoStubVisitor(void);

		bool		computeOneRoute(Topology &		topo,
						const ero_hop_t &	srcHop,
						const ero_hop_t &	dstHop,
						const cspf_constr_t &	constr,
						std::list<ero_hop_t> &	pEro);

		virtual void	visitTopology(Topology *		ptr);

		virtual void	visitNode(Node *			ptr);

		virtual void	visitNetNode(NetNode *			ptr);
		virtual void	visitTna(Tna *				ptr);
		virtual void	visitTeLink(TeLink *			ptr);

		virtual void	visitGridNode(GridNode *		ptr);
		virtual void	visitGridService(GridService *		ptr);
		virtual void	visitGridCompElem(GridCompElem *	ptr);
		virtual void	visitGridSubCluster(GridSubCluster *	ptr);
		virtual void	visitGridStorageElem(GridStorageElem *	ptr);

	private:
		uint32_t	pathCost_;
	};
}

std::list<ero_hop_t>	stubEroComp(uint32_t			rootLsr,
				    g2mpls_addr_t &		eTna,
				    uint32_t			dstNode);
std::list<ero_hop_t>	stubGnsEroComp(uint32_t			rootLsr,
				       g2mpls_addr_t &		eTna,
				       std::list<ero_hop_t> &	ero);
g2mpls_addr_t		stubTnaFromGridSE(PCERA::Topology & topo);

g2mpls_addr_t		stubTnaFromGridCE(PCERA::Topology & topo,
					  uint32_t          reqCPUCount);

#endif // __G2PCERA_ALGOVIS_HH__
