//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#ifndef __G2PCERA_COMMON_HH__
#define __G2PCERA_COMMON_HH__

#include <g2mpls_types.h>
#include <g2mpls_addr.h>
#include <list>
#include <map>
#include <assert.h>

//
//   PCERA common types
//

typedef enum {
	PCERA_ERROR_NONE = 0,
	PCERA_ERROR_INVALID_PARAMETER,
	PCERA_ERROR_INSTANCE_NOT_FOUND,
	PCERA_ERROR_INSTANCE_ALREADY_EXISTS,
	PCERA_ERROR_TOPOLOGY_NOT_FOUND,
	PCERA_ERROR_TOPOLOGY_ALREADY_EXISTS,
	PCERA_ERROR_NODE_NOT_FOUND,
	PCERA_ERROR_REMOTE_NODE_NOT_FOUND,
	PCERA_ERROR_NODE_ALREADY_EXISTS,
	PCERA_ERROR_NODE_PARAMS_MISMATCH,
	PCERA_ERROR_SUBNODE_NOT_FOUND,
	PCERA_ERROR_SUBNODE_ALREADY_EXISTS,
	PCERA_ERROR_SUBNODE_PARAMS_MISMATCH,
	PCERA_ERROR_TNA_NOT_FOUND,
	PCERA_ERROR_TNA_ALREADY_EXISTS,
	PCERA_ERROR_LINK_NOT_FOUND,
	PCERA_ERROR_LINK_ALREADY_EXISTS,
	PCERA_ERROR_LINK_PARAMS_MISMATCH,
	PCERA_ERROR_CALL_NOT_FOUND,
	PCERA_ERROR_CALL_PARAMS_MISMATCH,
	PCERA_ERROR_CONN_NOT_FOUND,
	PCERA_ERROR_CONN_PARAMS_MISMATCH,
	PCERA_ERROR_NO_ROUTE,
	PCERA_ERROR_ERO_MISMATCH,
	PCERA_ERROR_NO_MEMORY,
	PCERA_ERROR_GENERIC
} pceraErrorCode_t;

#define PCERA_ERR_TO_STRING(X)                                                \
  (((X)== PCERA_ERROR_NONE                   ) ? "NO ERROR"                  :\
  (((X)== PCERA_ERROR_INVALID_PARAMETER      ) ? "INVALID PARAMETER"         :\
  (((X)== PCERA_ERROR_INSTANCE_NOT_FOUND     ) ? "INSTANCE NOT FOUND"        :\
  (((X)== PCERA_ERROR_INSTANCE_ALREADY_EXISTS) ? "INSTANCE ALREADY EXISTS"   :\
  (((X)== PCERA_ERROR_TOPOLOGY_NOT_FOUND     ) ? "TOPOLOGY NOT FOUND"        :\
  (((X)== PCERA_ERROR_TOPOLOGY_ALREADY_EXISTS) ? "TOPOLOGY ALREADY EXISTS"   :\
  (((X)== PCERA_ERROR_NODE_NOT_FOUND         ) ? "NODE NOT FOUND"            :\
  (((X)== PCERA_ERROR_REMOTE_NODE_NOT_FOUND  ) ? "REMOTE NODE NOT FOUND"     :\
  (((X)== PCERA_ERROR_NODE_ALREADY_EXISTS    ) ? "NODE ALREADY EXISTS"       :\
  (((X)== PCERA_ERROR_NODE_PARAMS_MISMATCH   ) ? "NODE PARAMS MISMATCH"      :\
  (((X)== PCERA_ERROR_SUBNODE_NOT_FOUND      ) ? "SUBNODE NOT FOUND"         :\
  (((X)== PCERA_ERROR_SUBNODE_ALREADY_EXISTS ) ? "SUBNODE ALREADY EXISTS"    :\
  (((X)== PCERA_ERROR_SUBNODE_PARAMS_MISMATCH) ? "SUBNODE PARAMS MISMATCH"   :\
  (((X)== PCERA_ERROR_TNA_NOT_FOUND          ) ? "TNA NOT FOUND"             :\
  (((X)== PCERA_ERROR_TNA_ALREADY_EXISTS     ) ? "TNA ALREADY EXISTS"        :\
  (((X)== PCERA_ERROR_LINK_NOT_FOUND         ) ? "LINK NOT FOUND"            :\
  (((X)== PCERA_ERROR_LINK_ALREADY_EXISTS    ) ? "LINK ALREADY EXISTS"       :\
  (((X)== PCERA_ERROR_LINK_PARAMS_MISMATCH   ) ? "LINK PARAMS MISMATCH"      :\
  (((X)== PCERA_ERROR_CALL_NOT_FOUND         ) ? "CALL NOT FOUND"            :\
  (((X)== PCERA_ERROR_CALL_PARAMS_MISMATCH   ) ? "CALL PARAMS MISMATCH"      :\
  (((X)== PCERA_ERROR_CONN_NOT_FOUND         ) ? "CONNECTION NOT FOUND"      :\
  (((X)== PCERA_ERROR_CONN_PARAMS_MISMATCH   ) ? "CONNECTION PARAMS MISMATCH":\
  (((X)== PCERA_ERROR_NO_ROUTE               ) ? "NO ROUTE"                  :\
  (((X)== PCERA_ERROR_ERO_MISMATCH           ) ? "ERO MISMATCH"              :\
  (((X)== PCERA_ERROR_NO_MEMORY              ) ? "NO MEMORY"                 :\
  (((X)== PCERA_ERROR_GENERIC                ) ? "GENERIC ERROR"             :\
  "==UNKNOWN=="))))))))))))))))))))))))))


//
//   PCERA internal types
//

typedef struct nodeFlags {
	bool inCandidateList;
	bool inTree;
	bool inTopoTransform;
	bool inBhandariTransform;
} nodeFlags_t;


//
//   PCERA internal types
//

typedef enum spfType {
	spfUnknown  = 0x0,
	spfDijkstra = 0x1,
} spfType_t;


#define SPF_TYPE_TO_STRING(X)                   \
   (((X) == spfDijkstra) ? "Dijkstra" :         \
			   "==UNKNOWN==")


//
//   Topology common types -- not to be used by any OSPF(er)
//
typedef enum topology_node_type {
	TOPO_NODE_UNKNOWN,
	TOPO_NODE_NETWORK,
	TOPO_NODE_GRID
} topo_node_type_t ;

#define NODE_TYPE_TO_STRING(X)				\
  (((X) == TOPO_NODE_NETWORK) ? "Network Router" :	\
  (((X) == TOPO_NODE_GRID   ) ? "Grid Site" :		\
				"==UNKNOWN==   "))

typedef struct node_ident {
	uint32_t				id;
	topo_node_type_t			type;

	bool	operator==(const struct node_ident & other) const;
	bool	operator!=(const struct node_ident & other) const;
} node_ident_t;

typedef struct net_node_data_mask {
	uint32_t				is_domain:1;
	uint32_t				op_state:1;
	uint32_t				adm_state:1;
	uint32_t				te_colors:1;
	uint32_t				areas:1;
} net_node_data_mask_t;

typedef struct net_node_data {
	net_node_data_mask_t			mask_;
	uint8_t					is_domain:1;
	opstate_t				op_state;
	admstate_t				adm_state;
	uint32_t				te_colors;
	std::list<uint16_t>			areas;
} net_node_data_t;

#ifdef G2MPLS
typedef struct grid_site_data_mask {
	uint32_t				name:1;
	uint32_t				location:1;
	uint32_t				pe_router_id:1;
} grid_site_data_mask_t;

typedef struct grid_site_data {
	grid_site_data_mask_t			mask_;
	std::string *				name;
	geo_coords_t				location;
	uint32_t				pe_router_id;

#ifdef __cplusplus
	grid_site_data(void) {
		name = 0;
	}

	~grid_site_data(void) {
		if (name && name->size()) delete name;
	}

	grid_site_data & operator= (const grid_site_data & src) {
		mask_        =  src.mask_;
		if (name && name->size())
			delete name;
		name         =  (src.name ? new std::string(*(src.name)) : 0);
		location     =  src.location;
		pe_router_id =  src.pe_router_id;
	}
#endif // __cplusplus
} grid_site_data_t;

typedef enum grid_sub_node_type {
	GRIDSUBNODE_UNKNOWN,
	GRIDSUBNODE_SERVICE,
	GRIDSUBNODE_COMPUTINGELEMENT,
	GRIDSUBNODE_SUBCLUSTER,
	GRIDSUBNODE_STORAGEELEMENT
} grid_sub_node_type_t;

#define GRID_SUBNODE_TO_STRING(X)					\
   (((X) == GRIDSUBNODE_SERVICE         ) ? "GRID SERVICE"           :	\
   (((X) == GRIDSUBNODE_COMPUTINGELEMENT) ? "GRID COMPUTING ELEMENT" :	\
   (((X) == GRIDSUBNODE_SUBCLUSTER      ) ? "GRID SUB-CLUSTER"       :	\
   (((X) == GRIDSUBNODE_STORAGEELEMENT  ) ? "GRID STORAGE ELEMENT"   :	\
    "==UNKNOWN=="))))


typedef struct grid_subnode_ident {
	uint32_t				id;
	grid_sub_node_type_t			type;

	bool	operator==(const struct grid_subnode_ident & other) const;
	bool	operator!=(const struct grid_subnode_ident & other) const;
} grid_subnode_ident_t;


typedef struct grid_subnodes {
	std::list<grid_subnode_ident_t>		services;
	std::list<grid_subnode_ident_t>		comp_elems;
	std::list<grid_subnode_ident_t>		sub_clusters;
	std::list<grid_subnode_ident_t>		storage_elems;
} grid_subnodes_t ;

typedef struct grid_service_data_mask {
	uint32_t				name:1;
	uint32_t				data:1;
	uint32_t				state:1;
	uint32_t				endpoint_addr:1;
} grid_service_data_mask_t;

typedef struct grid_service_data {
	grid_service_data_mask_t		mask_;
	std::string *				name;
	grid_service_info_t			data;
	grid_service_state_t			state;
	g2mpls_addr_t				endpoint_addr;
#ifdef __cplusplus
	grid_service_data(void) {
		memset(&mask_, 0, sizeof(mask_));

		name = 0;
	};

	~grid_service_data(void) {
		if (name && name->size()) delete name;
	}

	grid_service_data & operator= (const grid_service_data & src) {
		mask_         = src.mask_;
		if (name && name->size())
			delete name;
		name          = (src.name ? new std::string(*(src.name)) : 0);
		data          = src.data;
		state         = src.state;
		endpoint_addr = src.endpoint_addr;
	}
#endif // __cplusplus
} grid_service_data_t;

typedef struct grid_ce_data_mask {
	uint32_t				name:1;
	uint32_t				lrms_info:1;
	uint32_t				host_addr:1;
	uint32_t				gatekeeper_port:1;
	uint32_t				job_manager:1;
	uint32_t				data_dir:1;
	uint32_t				default_storage_elem_id:1;
	uint32_t				jobs_state:1;
	uint32_t				jobs_stats:1;
	uint32_t				jobs_timeperf:1;
	uint32_t				jobs_timepolicy:1;
	uint32_t				jobs_loadpolicy:1;
	uint32_t				free_job_slots_calendar:1;
} grid_ce_data_mask_t;

typedef struct grid_ce_data {
	grid_ce_data_mask_t			mask_;
	std::string *				name;
	grid_lrms_info_t			lrms_info;
	g2mpls_addr_t				host_addr;
	uint32_t				gatekeeper_port;
	std::string *				job_manager;
	std::string *				data_dir;
	uint32_t				default_storage_elem_id;
	grid_jobs_state_t			jobs_state;
	grid_jobs_stats_t			jobs_stats;
	grid_jobs_time_perf_t			jobs_timeperf;
	grid_jobs_time_policy_t			jobs_timepolicy;
	grid_jobs_load_policy_t			jobs_loadpolicy;
	std::map<uint32_t, uint16_t>		free_job_slots_calendar;

#ifdef __cplusplus
	grid_ce_data(void) {
		memset(&mask_, 0, sizeof(mask_));

		name        = 0;
		job_manager = 0;
		data_dir    = 0;
		free_job_slots_calendar.clear();
	};

	~grid_ce_data(void) {
		if (name && name->size())               delete name;
		if (job_manager && job_manager->size()) delete job_manager;
		if (data_dir    && data_dir->size())    delete data_dir;
	};

	grid_ce_data & operator= (const grid_ce_data & src) {
		mask_                   = src.mask_;
		if (name && name->size())
			delete name;
		name                    = (src.name ?
					   new std::string(*(src.name)):
					   0);
		lrms_info               = src.lrms_info;
		host_addr               = src.host_addr;
		gatekeeper_port         = src.gatekeeper_port;
		default_storage_elem_id = src.default_storage_elem_id ;
		jobs_state              = src.jobs_state;
		jobs_stats              = src.jobs_stats;
		jobs_timeperf           = src.jobs_timeperf;
		jobs_timepolicy         = src.jobs_timepolicy;
		jobs_loadpolicy         = src.jobs_loadpolicy;
		free_job_slots_calendar = src.free_job_slots_calendar;

		if (job_manager && job_manager->size())
			delete job_manager;
		job_manager             = (src.job_manager ?
					   new std::string(*(src.job_manager)):
					   0);
		if (data_dir && data_dir->size())
			delete data_dir;
		data_dir                = (src.data_dir ?
					   new std::string(*(src.data_dir)) :
					   0);
	}
#endif // __cplusplus
} grid_ce_data_t;

typedef struct grid_software_package {
	grid_application_t			software;
	std::string *				software_env_setup;
#ifdef __cplusplus
	grid_software_package(void) {
		memset(&software, 0, sizeof(software));
		software_env_setup = 0;
	}

	~grid_software_package(void) {
		if (software_env_setup &&
		    software_env_setup->size())
			delete software_env_setup;
	}

	grid_software_package & operator= (const grid_software_package& src){
		software            = src.software;
		if (software_env_setup && software_env_setup->size())
			delete software_env_setup;
		software_env_setup  =
			(src.software_env_setup ?
			 new std::string(*(src.software_env_setup)) : 0);
	}
#endif // __cplusplus
} grid_software_package_t;

typedef struct grid_subcluster_data_mask {
	uint32_t				name:1;
	uint32_t				cpu:1;
	uint32_t				os:1;
	uint32_t				memory:1;
	uint32_t				software_packages:1;
	uint32_t				subcluster_calendar:1;
} grid_subcluster_data_mask_t;

typedef struct grid_subcluster_data {
	grid_subcluster_data_mask_t		mask_;
	std::string *				name;
	grid_cpu_info_t				cpu;
	grid_os_info_t				os;
	grid_memory_info_t			memory;
	std::list<grid_software_package_t *>	software_packages;
	std::map<uint32_t, grid_cpu_count_t>	subcluster_calendar;

#ifdef __cplusplus
	grid_subcluster_data(void) {
		memset(&mask_, 0, sizeof(mask_));

		name = 0;
		software_packages.clear();
		subcluster_calendar.clear();
	}

	~grid_subcluster_data(void) {
		if (name && name->size()) {
			delete name;
		}

		std::list<grid_software_package_t *>::const_iterator iter;
		for (iter  = software_packages.begin();
		     iter != software_packages.end();
		     iter++) {
			grid_software_package_t * tmp;
			tmp = *iter;
			delete (tmp);
		}
		software_packages.clear();
	}

	grid_subcluster_data & operator= (const grid_subcluster_data & src) {
		mask_               = src.mask_;
		if (name && name->size())
			delete name;
		name                = (src.name ?
				       new std::string(*(src.name)) :
				       0);
		cpu                 = src.cpu;
		os                  = src.os;
		memory              = src.memory;

		std::list<grid_software_package_t *>::const_iterator iter;
		for (iter  = software_packages.begin();
		     iter != software_packages.end();
		     iter++) {
			grid_software_package_t * tmp;
			tmp = *iter;
			delete (tmp);
		}
		software_packages.clear();
		for (iter  = src.software_packages.begin();
		     iter != src.software_packages.end();
		     iter++) {
			grid_software_package_t * tmp;
			tmp = new grid_software_package_t;
			if (!tmp) {
				assert(0);
			}
			*tmp = *(*iter);
			software_packages.push_back(tmp);
		}

		subcluster_calendar = src.subcluster_calendar;
	}
#endif // __cplusplus
} grid_subcluster_data_t;

typedef struct grid_storage_area {
	std::string *					storage_area_name;
	std::string *					storage_area_path;
	grid_storage_area_info_t			storage_area_info;
#ifdef __cplusplus
	grid_storage_area(void) {
		storage_area_name = 0;
		storage_area_path = 0;
	}

	~grid_storage_area(void) {
		if (storage_area_name &&
		    storage_area_name->size())
			delete storage_area_name;
		if (storage_area_path &&
		    storage_area_path->size())
			delete storage_area_path;
	}

	grid_storage_area & operator= (const grid_storage_area & src) {
		if (storage_area_name && storage_area_name->size())
			delete storage_area_name;
		storage_area_name  =
			(src.storage_area_name ?
			 new std::string(*(src.storage_area_name)) : 0);
		if (storage_area_path && storage_area_path->size())
			delete storage_area_path;
		storage_area_path  =
			(src.storage_area_path ?
			 new std::string(*(src.storage_area_path)) : 0);
		storage_area_info  = src.storage_area_info;
	}
#endif // __cplusplus

} grid_storage_area_t;


typedef struct grid_se_data_mask {
	uint32_t				name:1;
	uint32_t				storage_info:1;
	uint32_t				online_size:1;
	uint32_t				nearline_size:1;
	uint32_t				storage_areas:1;
	uint32_t				se_calendar:1;
} grid_se_data_mask_t;

typedef struct grid_se_data {
	grid_se_data_mask_t				mask_;
	std::string *					name;
	grid_storage_info_t				storage_info;
	grid_storage_size_t				online_size;
	grid_storage_size_t				nearline_size;
	std::list<grid_storage_area_t *>		storage_areas;
	std::map<uint32_t, grid_storage_count_t>	se_calendar;

#ifdef __cplusplus
	grid_se_data(void) {
		memset(&mask_, 0, sizeof(mask_));

		name              = 0;
		storage_areas.clear();
		se_calendar.clear();
	}

	~grid_se_data(void) {
		if (name && name->size()) {
			delete name;
		}

		std::list<grid_storage_area_t *>::const_iterator iter;
		for (iter  = storage_areas.begin();
		     iter != storage_areas.end();
		     iter++) {
			grid_storage_area_t * tmp;
			tmp = *iter;
			delete tmp;
		}
		storage_areas.clear();
	}

	grid_se_data & operator= (const grid_se_data & src) {
		mask_              = src.mask_;
		if (name && name->size()) {
			delete name;
		}
		name               = (src.name ?
				      new std::string(*(src.name)) :
				      0);
		storage_info       = src.storage_info;
		online_size        = src.online_size;
		nearline_size      = src.nearline_size;

		std::list<grid_storage_area_t *>::const_iterator iter;
		for (iter  = storage_areas.begin();
		     iter != storage_areas.end();
		     iter++) {
			grid_storage_area_t * tmp;
			tmp = *iter;
			delete tmp;
		}
		storage_areas.clear();

		for (iter  = src.storage_areas.begin();
		     iter != src.storage_areas.end();
		     iter++) {
			grid_storage_area_t * tmp;
			tmp = new grid_storage_area_t;
			if (!tmp) {
				assert(0);
			}
			*tmp = *(*iter);
			storage_areas.push_back(tmp);
		}

		se_calendar        = src.se_calendar;
	}
#endif // __cplusplus
} grid_se_data_t;

#endif //G2MPLS

typedef enum topology_link_mode {
	LINK_MODE_UNKNOWN         ,
	LINK_MODE_P2P_UNNUMBERED  ,
	LINK_MODE_P2P_NUMBERED    ,
	LINK_MODE_MULTIACCESS     ,
	LINK_MODE_ENNI_INTERDOMAIN,
	LINK_MODE_ENNI_INTRADOMAIN
} topo_link_mode_t;

#define LINK_MODE_TO_STRING(X)					\
  (((X) == LINK_MODE_P2P_UNNUMBERED   ) ? "P2P Unnumbered"    :	\
  (((X) == LINK_MODE_P2P_NUMBERED     ) ? "P2P Numbered"      :	\
  (((X) == LINK_MODE_MULTIACCESS      ) ? "Multiaccess"       :	\
  (((X) == LINK_MODE_ENNI_INTERDOMAIN ) ? "ENNI Inter-Domain" :	\
  (((X) == LINK_MODE_ENNI_INTRADOMAIN ) ? "ENNI Intra-Domain" :	\
					  "==UNKNOWN==")))))

typedef struct telink_ident {
	uint32_t					lcl_rid;
	g2mpls_addr_t					lcl_id;
	uint32_t					rem_rid;
	g2mpls_addr_t					rem_id;
	topo_link_mode_t				mode;
	uint32_t					lcl_rc_id;
	uint32_t					rem_rc_id;

	bool	operator==(const struct telink_ident & other) const;
	bool	operator!=(const struct telink_ident & other) const;
} telink_ident_t;

typedef struct telink_com_data_mask {
	uint32_t					admin_metric:1;
	uint32_t					te_metric:1;
	uint32_t					te_color_mask:1;
	uint32_t					te_prot_type_mask:1;
	uint32_t					te_max_bw:1;
	uint32_t					te_max_resv_bw:1;
} telink_com_data_mask_t;

typedef struct telink_com_data {
	telink_com_data_mask_t				mask_;
	uint32_t					admin_metric;
	uint32_t					te_metric;
	uint32_t					te_color_mask;
	uint8_t						te_prot_type_mask;
	uint32_t					te_max_bw;
	uint32_t					te_max_resv_bw;
} telink_com_data_t;

typedef struct telink_psc_data_mask {
	uint32_t					min_lsp_bw:1;
	uint32_t					if_mtu:1;
} telink_psc_data_mask_t;

typedef struct telink_psc_data {
	telink_psc_data_mask_t				mask_;
	uint32_t					min_lsp_bw;
	uint16_t					if_mtu;
} telink_psc_data_t;

typedef struct telink_tdm_data_mask {
	uint32_t					min_lsp_bw:1;
	uint32_t					stdarb_cc_indication:1;
	uint32_t					ho_muxcap_mask:1;
	uint32_t					lo_muxcap_mask:1;
	uint32_t					transparency_mask:1;
	uint32_t					blsr_ring_id:1;
} telink_tdm_data_mask_t;

typedef struct telink_tdm_data {
	telink_tdm_data_mask_t				mask_;
	uint32_t					min_lsp_bw;
	gmpls_sdhsonet_stdarbcap_t			stdarb_cc_indication;
	uint8_t						ho_muxcap_mask;
	uint8_t						lo_muxcap_mask;
	uint32_t					transparency_mask;
	uint32_t					blsr_ring_id;
} telink_tdm_data_t;

typedef struct telink_lscg709_data_mask {
	uint32_t					oduk_muxcap_mask:1;
} telink_lscg709_data_mask_t;

typedef struct telink_lscg709_data {
	telink_lscg709_data_mask_t			mask_;
	uint8_t						oduk_muxcap_mask;
} telink_lscg709_data_t;

typedef struct telink_lscwdm_data_mask {
	uint32_t					dispersion_PMD:1;
	uint32_t					span_length:1;
	uint32_t					amplifiers:1;
} telink_lscwdm_data_mask_t;

typedef struct telink_lscwdm_data {
	telink_lscwdm_data_mask_t			mask_;
	uint32_t					dispersion_PMD;
	uint32_t					span_length;
	std::list<wdm_amplifier_data_t>			amplifiers;
} telink_lscwdm_data_t;


#define GMPLS_ISC_INDICATION_STD_SDH_SONET	0x00
#define GMPLS_ISC_INDICATION_ARB_SDH_SONET	0x01

typedef struct isc {
	sw_cap_t			switching;
	enc_type_t			encoding;
	bw_per_prio_t			max_lsp_bw;
	union {
		struct {
			uint32_t		min_lsp_bw;
			uint16_t		if_mtu;
		} psc;

		struct {
			uint32_t		min_lsp_bw;
			uint8_t			indication;
		} tdm;
		uint64_t raw;
	} spec;

	bool	operator==(const struct isc & other) const;
	bool	operator!=(const struct isc & other) const;
} isc_t;

typedef struct ero_hop {
	uint32_t				node;
	g2mpls_addr_t				te_link;
	g2mpls_addr_t				upstream_datalink;
	g2mpls_addr_t				downstream_datalink;
	label_id_t				upstream_label;
	label_id_t				downstream_label;
	uint8_t					loose:1;

	bool					idmn;
	bool	is_null(void) const;
	bool	operator==(const struct ero_hop & other) const;
	bool	operator!=(const struct ero_hop & other) const;
} ero_hop_t;


typedef struct topology_summary_data {
	uint32_t				root_rid;
	uint32_t				tot_nodes;
	uint32_t				tot_links;
	uint32_t				tot_tnas;
	uint32_t				tot_gns_calls;
	uint32_t				max_link_cost;
	spfType_t				spf_selector;
	uint32_t				spf_revision;
} topology_summary_data_t;

typedef struct cspf_constraints {
	call_info_t				call_info;
	recovery_info_t				rec_info;
	lsp_info_t				lsp_info;
	std::list<ero_hop_t>			exclude_ero;
	std::list<uint32_t>			exclude_srlgs;
} cspf_constr_t;


typedef struct tna_ident {
	uint32_t	rc_id;
	uint32_t	rid;
	g2mpls_addr_t	id;

	bool	operator==(const struct tna_ident & other) const;
	bool	operator!=(const struct tna_ident & other) const;
	bool	operator>=(const struct tna_ident & other) const;
} tna_ident_t;


size_t		myrand(size_t min, size_t max);

std::string	bool2string(bool key);
std::string	uint8string(const uint8_t & key);
std::string	uint16string(const uint16_t & key);
std::string	uint32string(const uint32_t & key);
std::string	key2string(const uint64_t & key);
std::string	version2string(const version_t & ver);
std::string	range2string(const range_set_t & range);
std::string	nodeIdent2string(const node_ident_t & ident);
std::string	subnodeIdent2string(const grid_subnode_ident_t & ident);
std::string	teLinkIdent2string(const telink_ident_t & ident);
std::string	tnaIdent2string(const tna_ident_t & ident);
std::string	rid2string(const uint32_t & rid);
std::string	geoCoord2string(const geo_coords_t & geo);
std::string	unixTime2string(const uint32_t & t);
std::string	wdmLabel2string(const uint32_t & t);
std::string	nodeFlags2string(const nodeFlags_t & flags);

bool		isValid(const net_res_spec_t & res, std::string & err);
bool		isValid(const grid_res_spec_t & res, std::string & err);
bool		isValid(const call_ident_t & id, std::string & err);
bool		isValid(const call_info_t & callInfo, std::string & err);
bool		isValid(const recovery_info_t & recInfo, std::string & err);
bool		isValid(const lsp_info_t & lspInfo, std::string & err);
bool		isValid(const ero_hop_t & hop, std::string & err);
bool		isValid(const std::list<ero_hop_t> & ero, std::string & err);

#endif // __G2PCERA_COMMON_HH__
