//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "log.h"
#include "zebra.h"

#include "lib/corba.h"
#include "g2pcera_trace.hh"
#include "g2pcera_corba.hh"
#include "g2pcera_corba_servant.hh"

#include "../idl/lrm.hh"

#include <iostream>
#include <string>
#include <stdlib.h>

using namespace std;

#if HAVE_OMNIORB

#include "g2mpls_corba_utils.h"


//
// Corba Clients
//

bool corba_client_setup(LRM::Info_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();

	if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		G2PCERA_ERROR("Cannot get ORB");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_LRM, ior)) {
		throw string("Cannot fetch IOR");
		G2PCERA_ERROR("Cannot fetch IOR");
		return false;
	}

	CORBA::Object_var obj;
	obj = orb->string_to_object(ior.c_str());
	if (CORBA::is_nil(obj)) {
		throw string("Cannot get object");
		G2PCERA_ERROR("Cannot get object");
		return false;
	}

	l = LRM::Info::_narrow(obj);
	if (CORBA::is_nil(l)) {
		throw string("Cannot invoke on a nil object reference");
		G2PCERA_ERROR("cannot invoke on a nil object reference");
		return false;
	}

	return true;
}

bool corba_client_shutdown(corba_servant_t s)
{
	return 1;
}

G2PCERA_i*     servantG2pcera  = 0;
void corba_server_setup_g2pcera(void)
{
	G2PCERA_DEBUG("Setting up CORBA server side");

	try {
		servantG2pcera = new G2PCERA_i();
		if (!servantG2pcera) {
			throw string("Cannot create servantG2pcera");
		}

		PortableServer::POA_var poa;
		poa = corba_poa();
		if (CORBA::is_nil(poa)) {
			throw string("Cannot get POA");
		}

		PortableServer::ObjectId_var servantG2pcera_id;
		servantG2pcera_id = poa->activate_object(servantG2pcera);

		CORBA::Object_var obj;
		obj = servantG2pcera->_this();
		if (CORBA::is_nil(obj)) {
			throw string("Cannot get object");
		}

		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			throw string("Cannot get ORB");
		}

		CORBA::String_var sior(orb->object_to_string(obj));
		G2PCERA_DEBUG("IOR = '%s'", (char*) sior);

		if (!corba_dump_ior(CORBA_SERVANT_G2PCERA,
				    string(sior))) {
			throw string("Cannot dump IOR");
		}

		servantG2pcera->_remove_ref();

		PortableServer::POAManager_var poa_manager;
		poa_manager = corba_poa_manager();
		if (CORBA::is_nil(poa_manager)) {
			throw string("Cannot get POA Manager");
		}

		poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		throw string("Caught CORBA::SystemException");
	} catch (CORBA::Exception & e) {
		throw string("Caught CORBA::Exception");
	} catch (omniORB::fatalException & e) {
		throw string("Caught omniORB::fatalException:");
		throw string("  file: %s", e.file());
		throw string("  line: %d", e.line());
		throw string("  mesg: %s", e.errmsg());
	} catch (string & e) {
		throw string("Caught exception: %s", e.c_str());
	} catch (...) {
		throw string("Caught unknown exception");
	}
}

g2mplsTopology_i*     servantG2topo = 0;
void corba_server_setup_g2topo(void)
{
	G2PCERA_DEBUG("Setting up CORBA server side");

	try {
		servantG2topo = new g2mplsTopology_i();
		if (!servantG2topo) {
			throw string("Cannot create servantG2topo");
		}

		PortableServer::POA_var poa;
		poa = corba_poa();
		if (CORBA::is_nil(poa)) {
			throw string("Cannot get POA");
		}

		PortableServer::ObjectId_var servantG2topo_id;
		servantG2topo_id = poa->activate_object(servantG2topo);

		CORBA::Object_var obj;
		obj = servantG2topo->_this();
		if (CORBA::is_nil(obj)) {
			throw string("Cannot get object");
		}

		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			throw string("Cannot get ORB");
		}

		CORBA::String_var sior(orb->object_to_string(obj));
		G2PCERA_DEBUG("IOR = '%s'", (char*) sior);

		if (!corba_dump_ior(CORBA_SERVANT_G2MPLS_TOPOLOGY,
				    string(sior))) {
			throw string("Cannot dump IOR");
		}

		servantG2topo->_remove_ref();

		PortableServer::POAManager_var poa_manager;
		poa_manager = corba_poa_manager();
		if (CORBA::is_nil(poa_manager)) {
			throw string("Cannot get POA Manager");
		}

		poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		throw string("Caught CORBA::SystemException");
	} catch (CORBA::Exception & e) {
		throw string("Caught CORBA::Exception");
	} catch (omniORB::fatalException & e) {
		throw string("Caught omniORB::fatalException:");
		throw string("  file: %s", e.file());
		throw string("  line: %d", e.line());
		throw string("  mesg: %s", e.errmsg());
	} catch (string & e) {
		throw string("Caught exception: %s", e.c_str());
	} catch (...) {
		throw string("Caught unknown exception");
	}
}

int corba_server_setup(corba_servant_t s)
{
	try {
		switch (s) {
			case CORBA_SERVANT_G2PCERA:
				corba_server_setup_g2pcera();
				break;
			case CORBA_SERVANT_G2MPLS_TOPOLOGY:
				corba_server_setup_g2topo();
				break;
			default:
				G2PCERA_ERROR("Wrong servant to setup (%d)",s);
				return 0;
		}
	} catch (...) {
		G2PCERA_ERROR("Caught unknown exception");
		return 0;
	}

	return 1;
}

int corba_server_shutdown(corba_servant_t s)
{
	try {
		G2PCERA_DEBUG("Shutting down CORBA server side");

		if (!corba_remove_ior(s)) {
			throw string("Cannot remove IOR");
		}
	} catch (...) {
		G2PCERA_ERROR("Caught unknown exception");
		return 0;
	}

	return 1;
}




#endif // HAVE_OMNIORB
