//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed const the hope that it will be useful,
//  but WITHANY WARRANTY; witheven the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklconst Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#ifndef __G2PCERA_CORBA_SERVANT_HH__
#define __G2PCERA_CORBA_SERVANT_HH__

#if HAVE_OMNIORB

#include "g2pcera_config.hh"
#include "g2pcera_corba.hh"

//
// Topology servant implementation
//

class g2mplsTopology_i : public POA_g2mplsTopology::Info,
			 public PortableServer::RefCountServantBase
{
public:
	inline g2mplsTopology_i() {};
	virtual ~g2mplsTopology_i();

	CORBA::Boolean
	nodeAdd(const g2mplsTypes::nodeIdent& id);

	CORBA::Boolean
	nodeDel(const g2mplsTypes::nodeIdent& id);

	g2mplsTypes::nodeIdentSeq*
	nodeGetAll();

	CORBA::Boolean
	netNodeUpdate(g2mplsTypes::nodeId id,
		      const g2mplsTypes::netNodeParams& info);

	CORBA::Boolean
	netNodeGet(g2mplsTypes::nodeId id,
		   g2mplsTypes::netNodeParams_out info);

	CORBA::Boolean
	gridSiteUpdate(g2mplsTypes::nodeId id,
		       const g2mplsTypes::gridSiteParams& info);

	CORBA::Boolean
	gridSiteGet(g2mplsTypes::nodeId id,
		    g2mplsTypes::gridSiteParams_out info,
		    g2mplsTypes::gridSubNodes_out snodes);

	CORBA::Boolean
	gridSubNodeDel(g2mplsTypes::nodeId siteId,
		       const g2mplsTypes::gridSubNodeIdent& id);

	CORBA::Boolean
	gridServiceUpdate(g2mplsTypes::nodeId siteId,
			  g2mplsTypes::gridSubNodeId id,
			  const g2mplsTypes::gridServiceParams& info);

	CORBA::Boolean
	gridServiceGet(g2mplsTypes::nodeId siteId,
		       g2mplsTypes::gridSubNodeId id,
		       g2mplsTypes::gridServiceParams& info);

	CORBA::Boolean
	gridCompElemUpdate(g2mplsTypes::nodeId siteId,
			   g2mplsTypes::gridSubNodeId id,
			   const g2mplsTypes::gridCEParams& info);

	CORBA::Boolean
	gridCompElemGet(g2mplsTypes::nodeId siteId,
			g2mplsTypes::gridSubNodeId id,
			g2mplsTypes::gridCEParams_out info);

	CORBA::Boolean
	gridSubClusterUpdate(g2mplsTypes::nodeId siteId,
			     g2mplsTypes::gridSubNodeId id,
			     const g2mplsTypes::gridSubClusterParams& info);

	CORBA::Boolean
	gridSubClusterGet(g2mplsTypes::nodeId siteId,
			  g2mplsTypes::gridSubNodeId id,
			  g2mplsTypes::gridSubClusterParams_out info);

	CORBA::Boolean
	gridStorageElemUpdate(g2mplsTypes::nodeId siteId,
			      g2mplsTypes::gridSubNodeId id,
			      const g2mplsTypes::gridSEParams& info);

	CORBA::Boolean
	gridStorageElemGet(g2mplsTypes::nodeId siteId,
			   g2mplsTypes::gridSubNodeId id,
			   g2mplsTypes::gridSEParams_out info);

	CORBA::Boolean
	tnaIdAdd(const g2mplsTypes::tnaIdent& ident);

	CORBA::Boolean
	tnaIdDel(const g2mplsTypes::tnaIdent& ident);

	g2mplsTypes::tnaIdentSeq*
	tnaIdsGetAllFromNode(g2mplsTypes::nodeId node,
			     CORBA::Boolean isDomain);

	CORBA::Boolean
	linkAdd(const g2mplsTypes::teLinkIdent& ident);

	CORBA::Boolean
	linkDel(const g2mplsTypes::teLinkIdent& ident);

	g2mplsTypes::teLinkIdentSeq*
	teLinkGetAllFromNode(const g2mplsTypes::nodeIdent& ident);

	CORBA::Boolean
	teLinkUpdateCom(const g2mplsTypes::teLinkIdent& ident,
			const g2mplsTypes::teLinkComParams& info);

	CORBA::Boolean
	teLinkGetCom(const g2mplsTypes::teLinkIdent& ident,
		     g2mplsTypes::teLinkComParams& info);

	CORBA::Boolean
	teLinkUpdateTdm(const g2mplsTypes::teLinkIdent& ident,
			const g2mplsTypes::teLinkTdmParams& info);

	CORBA::Boolean
	teLinkGetTdm(const g2mplsTypes::teLinkIdent& ident,
		     g2mplsTypes::teLinkTdmParams& info);

	CORBA::Boolean
	teLinkUpdateLscG709(const g2mplsTypes::teLinkIdent& ident,
			    const g2mplsTypes::teLinkLscG709Params& info);

	CORBA::Boolean
	teLinkGetLscG709(const g2mplsTypes::teLinkIdent& ident,
			 g2mplsTypes::teLinkLscG709Params& info);

	CORBA::Boolean
	teLinkUpdateLscWdm(const g2mplsTypes::teLinkIdent& ident,
			   const g2mplsTypes::teLinkLscWdmParams& info);

	CORBA::Boolean
	teLinkGetLscWdm(const g2mplsTypes::teLinkIdent& ident,
			g2mplsTypes::teLinkLscWdmParams_out info);

	CORBA::Boolean
	teLinkUpdateStates(const g2mplsTypes::teLinkIdent& ident,
			   const g2mplsTypes::statesBundle& states);

	CORBA::Boolean
	teLinkGetStates(const g2mplsTypes::teLinkIdent& ident,
			g2mplsTypes::statesBundle& states);

	CORBA::Boolean
	teLinkUpdateGenBw(const g2mplsTypes::teLinkIdent& ident,
			  const g2mplsTypes::availBwPerPrio bw);

	CORBA::Boolean
	teLinkGetGenBw(const g2mplsTypes::teLinkIdent& ident,
		       g2mplsTypes::availBwPerPrio bw);

	CORBA::Boolean
	teLinkUpdateTdmBw(const g2mplsTypes::teLinkIdent& ident,
			  const g2mplsTypes::freeCTPSeq& freeTS);

	CORBA::Boolean
	teLinkGetTdmBw(const g2mplsTypes::teLinkIdent& ident,
		       g2mplsTypes::freeCTPSeq_out freeTS);

	CORBA::Boolean
	teLinkUpdateLscG709Bw(const g2mplsTypes::teLinkIdent& ident,
			      const g2mplsTypes::freeCTPSeq& freeODUk,
			      const g2mplsTypes::freeCTPSeq& freeOCh);

	CORBA::Boolean
	teLinkGetLscG709Bw(const g2mplsTypes::teLinkIdent& ident,
			   g2mplsTypes::freeCTPSeq_out freeODUk,
			   g2mplsTypes::freeCTPSeq_out freeOCh);

	CORBA::Boolean
	teLinkUpdateLscWdmBw(const g2mplsTypes::teLinkIdent& ident,
			     const g2mplsTypes::teLinkWdmLambdasBitmap& bm);

	CORBA::Boolean
	teLinkGetLscWdmBw(const g2mplsTypes::teLinkIdent& ident,
			  g2mplsTypes::teLinkWdmLambdasBitmap_out bm);

	CORBA::Boolean
	teLinkAppendSrlgs(const g2mplsTypes::teLinkIdent& ident,
			  const g2mplsTypes::srlgSeq& srlgs);

	CORBA::Boolean
	teLinkGetSrlgs(const g2mplsTypes::teLinkIdent& ident,
		       g2mplsTypes::srlgSeq_out srlgs);

	CORBA::Boolean
	teLinkAppendCalendar(const g2mplsTypes::teLinkIdent& ident,
			     const g2mplsTypes::teLinkCalendarSeq& cal);

	CORBA::Boolean
	teLinkGetCalendar(const g2mplsTypes::teLinkIdent& ident,
			  g2mplsTypes::teLinkCalendarSeq_out cal);

	CORBA::Boolean
	teLinkAppendIsc(const g2mplsTypes::teLinkIdent& ident,
			const g2mplsTypes::iscSeq& iscs);

	CORBA::Boolean
	teLinkGetIsc(const g2mplsTypes::teLinkIdent& ident,
		     g2mplsTypes::iscSeq_out iscs);
};


//
// Computation servant implementation
//

class G2PCERA_i : public POA_G2PCERA::RoutingServices,
		  public PortableServer::RefCountServantBase
{
public:
	inline G2PCERA_i() {};
	virtual ~G2PCERA_i();

	CORBA::Boolean
	nodeGetFromTnaRes(const g2mplsTypes::tnaResource& tnaRes,
			  g2mplsTypes::nodeId& node);

	g2mplsTypes::nodeId
	nodeGetFromTna(const g2mplsTypes::tnaId& tna);

	CORBA::Boolean
	nodeGetFromGnsTna(const g2mplsTypes::gridParams& tnaGnsRes,
			  const g2mplsTypes::nodeIdentSeq& excludeSet,
			  g2mplsTypes::nodeId& netNodeId,
			  g2mplsTypes::nodeId& gridSiteId);

	CORBA::Boolean
	nodeGetFromTELink(const g2mplsTypes::TELinkId& teLink,
			  g2mplsTypes::nodeId& node,
			  g2mplsTypes::nodeId& adjNode);

	CORBA::Boolean
	callRoute(const g2mplsTypes::tnaResource& src,
		  const g2mplsTypes::tnaResource& dst,
		  const g2mplsTypes::callIdent& callId,
		  const g2mplsTypes::callParams& callInfo,
		  const g2mplsTypes::recoveryParams& recInfo,
		  const g2mplsTypes::lspParams& lspInfo,
		  g2mplsTypes::eroSeq_out wEro);

	CORBA::Boolean
	callRouteAny(const g2mplsTypes::tnaResource& src,
		     const g2mplsTypes::gridParams& dst,
		     const g2mplsTypes::callIdent& callId,
		     const g2mplsTypes::callParams& callInfo,
		     const g2mplsTypes::recoveryParams& recInfo,
		     const g2mplsTypes::lspParams& lspInfo,
		     g2mplsTypes::eroSeq_out wEro,
		     g2mplsTypes::tnaResource& eTna);

	CORBA::Boolean
	callFlush(const g2mplsTypes::callIdent& callId);

	CORBA::Boolean
	callConfirm(const g2mplsTypes::callIdent& callId);

	CORBA::Boolean
	connectionRoute(const g2mplsTypes::connEndPoint& src,
			const g2mplsTypes::connEndPoint& dst,
			const g2mplsTypes::callIdent& callId,
			const g2mplsTypes::lspParams& lspInfo,
			g2mplsTypes::eroSeq_out wEro,
			g2mplsTypes::eroSeq_out pEro);

	CORBA::Boolean
	connectionRouteDisjoint(const g2mplsTypes::connEndPoint& src,
				const g2mplsTypes::connEndPoint& dst,
				const g2mplsTypes::callIdent& callId,
				const g2mplsTypes::lspParams& lspInfo,
				const g2mplsTypes::multiEroSeq& multiXro,
				g2mplsTypes::eroSeq_out wEro,
				g2mplsTypes::eroSeq_out pEro);

};


#endif // HAVE_OMNIORB

#endif // __G2PCERA_CORBA_SERVANT_HH__
