//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed const the hope that it will be useful,
//  but WITHANY WARRANTY; witheven the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklconst Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//


#include "log.h"
#include "zebra.h"

#if HAVE_OMNIORB

#ifdef  __cplusplus

#include "g2pcera_corba_utils.hh"

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <stdexcept>
#include <sstream>
#include <iomanip>

//
// nodeIdent
//
node_ident_t &
operator << (node_ident_t &                     dst,
	     const g2mplsTypes::nodeIdent_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.id = src->id;
	switch (src->type) {
		case g2mplsTypes::NODETYPE_UNKNOWN: {
			dst.type = TOPO_NODE_UNKNOWN;
		}
			break;
		case g2mplsTypes::NODETYPE_NETWORK: {
			dst.type = TOPO_NODE_NETWORK;
		}
			break;
		case g2mplsTypes::NODETYPE_GRID: {
			dst.type = TOPO_NODE_GRID;
		}
			break;
		default: {
			throw out_of_range("g2mplsTypes::NODETYPE "
					   "out-of-range");
		}
			break;
	}

	return dst;
}

g2mplsTypes::nodeIdent_var &
operator << (g2mplsTypes::nodeIdent_var &       dst,
	     const node_ident_t &               src)
{
	dst = new g2mplsTypes::nodeIdent;

	dst->id = src.id;

	switch (src.type) {
		case TOPO_NODE_UNKNOWN: {
			dst->type = g2mplsTypes::NODETYPE_UNKNOWN;
		}
			break;
		case TOPO_NODE_NETWORK: {
			dst->type = g2mplsTypes::NODETYPE_NETWORK;
		}
			break;
		case TOPO_NODE_GRID: {
			dst->type = g2mplsTypes::NODETYPE_GRID;
		}
			break;
		default: {
			throw out_of_range("node_ident_t type out-of-range");
		}
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                       dst,
	     const g2mplsTypes::nodeIdent_var &  src)
{
	node_ident_t tmp;

	dst = std::string("");

	try {
		tmp << src;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "g2mplsTypes::nodeIdent_var "
				       "to convert");
	}

	dst = nodeIdent2string(tmp);

	return dst;
}


//
// gridSubNodeIdent
//
grid_subnode_ident_t &
operator << (grid_subnode_ident_t &                    dst,
	     const g2mplsTypes::gridSubNodeIdent_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.id = src->id;

	switch (src->type) {
		case g2mplsTypes::GRIDSUBNODETYPE_UNKNOWN: {
			dst.type = GRIDSUBNODE_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDSUBNODETYPE_SERVICE: {
			dst.type = GRIDSUBNODE_SERVICE;
		}
			break;
		case g2mplsTypes::GRIDSUBNODETYPE_COMPUTINGELEMENT: {
			dst.type = GRIDSUBNODE_COMPUTINGELEMENT;
		}
			break;
		case g2mplsTypes::GRIDSUBNODETYPE_SUBCLUSTER: {
			dst.type = GRIDSUBNODE_SUBCLUSTER;
		}
			break;
		case g2mplsTypes::GRIDSUBNODETYPE_STORAGEELEMENT: {
			dst.type = GRIDSUBNODE_STORAGEELEMENT;
		}
			break;
		default: {
			throw out_of_range("g2mplsTypes::gridSubNodeIdent "
					   "type out-of-range");
		}
			break;
	}

	return dst;
}

g2mplsTypes::gridSubNodeIdent_var &
operator << (g2mplsTypes::gridSubNodeIdent_var &       dst,
	     const grid_subnode_ident_t &              src)
{
	dst = new g2mplsTypes::gridSubNodeIdent;

	dst->id = src.id;

	switch (src.type) {
		case GRIDSUBNODE_UNKNOWN: {
			dst->type = g2mplsTypes::GRIDSUBNODETYPE_UNKNOWN;
		}
			break;
		case GRIDSUBNODE_SERVICE: {
			dst->type = g2mplsTypes::GRIDSUBNODETYPE_SERVICE;
		}
			break;
		case GRIDSUBNODE_COMPUTINGELEMENT: {
			dst->type =
				g2mplsTypes::GRIDSUBNODETYPE_COMPUTINGELEMENT;
		}
			break;
		case GRIDSUBNODE_SUBCLUSTER: {
			dst->type = g2mplsTypes::GRIDSUBNODETYPE_SUBCLUSTER;
		}
			break;
		case GRIDSUBNODE_STORAGEELEMENT: {
			dst->type =
				g2mplsTypes::GRIDSUBNODETYPE_STORAGEELEMENT;
		}
			break;
		default: {
			throw out_of_range("grid_subnode_ident_t type "
					   "out-of-range");
		}
			break;
	}

	return dst;
}

std::string &
operator << (std::string &                             dst,
	     const g2mplsTypes::gridSubNodeIdent_var & src)
{
	grid_subnode_ident_t tmp;

	dst = std::string("");

	try {
		tmp << src;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "g2mplsTypes::gridSubNodeIdent_var "
				       "to convert");
	}

	dst = subnodeIdent2string(tmp);

	return dst;
}


//
// teLinkIdent
//
telink_ident_t &
operator << (telink_ident_t &                     dst,
	     const g2mplsTypes::teLinkIdent_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.lcl_rid = src->localNodeId;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp = src->localId;
		dst.lcl_id << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed localId in "
				       "g2mplsTypes::teLinkIdent_var "
				       "to convert");
	}

	dst.rem_rid = src->remoteNodeId;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp = src->remoteId;
		dst.rem_id << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed remoteId in "
				       "g2mplsTypes::teLinkIdent_var "
				       "to convert");
	}

	switch (src->mode) {
		case g2mplsTypes::LINKMODE_UNKNOWN: {
			dst.mode = LINK_MODE_UNKNOWN;
		}
			break;
		case g2mplsTypes::LINKMODE_P2P_UNNUMBERED: {
			dst.mode = LINK_MODE_P2P_UNNUMBERED;
		}
			break;
		case g2mplsTypes::LINKMODE_P2P_NUMBERED: {
			dst.mode = LINK_MODE_P2P_NUMBERED;
		}
			break;
		case g2mplsTypes::LINKMODE_MULTIACCESS: {
			dst.mode = LINK_MODE_MULTIACCESS;
		}
			break;
		case g2mplsTypes::LINKMODE_ENNI_INTERDOMAIN: {
			dst.mode = LINK_MODE_ENNI_INTERDOMAIN;
		}
			break;
		case g2mplsTypes::LINKMODE_ENNI_INTRADOMAIN: {
			dst.mode = LINK_MODE_ENNI_INTRADOMAIN;
		}
			break;
		default: {
		}
			break;
	}

	dst.lcl_rc_id = src->localRcId;
	dst.rem_rc_id = src->remoteRcId;

	return dst;
}

g2mplsTypes::teLinkIdent_var &
operator << (g2mplsTypes::teLinkIdent_var &       dst,
	     const telink_ident_t &               src)
{
	dst = new g2mplsTypes::teLinkIdent;

	dst->localNodeId = src.lcl_rid;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp << src.lcl_id;
		dst->localId = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed lcl_id in "
				       "telink_ident_t "
				       "to convert");
	}

	dst->remoteNodeId = src.rem_rid;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp << src.rem_id;
		dst->remoteId = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed rem_id in "
				       "telink_ident_t "
				       "to convert");
	}

	switch (src.mode) {
		case LINK_MODE_UNKNOWN: {
			dst->mode = g2mplsTypes::LINKMODE_UNKNOWN;
		}
			break;
		case LINK_MODE_P2P_UNNUMBERED: {
			dst->mode = g2mplsTypes::LINKMODE_P2P_UNNUMBERED;
		}
			break;
		case LINK_MODE_P2P_NUMBERED: {
			dst->mode = g2mplsTypes::LINKMODE_P2P_NUMBERED;
		}
			break;
		case LINK_MODE_MULTIACCESS: {
			dst->mode = g2mplsTypes::LINKMODE_MULTIACCESS;
		}
			break;
		case LINK_MODE_ENNI_INTERDOMAIN: {
			dst->mode = g2mplsTypes::LINKMODE_ENNI_INTERDOMAIN;
		}
			break;
		case LINK_MODE_ENNI_INTRADOMAIN: {
			dst->mode = g2mplsTypes::LINKMODE_ENNI_INTRADOMAIN;
		}
			break;
		default: {
		}
			break;
	}

	dst->localRcId  = src.lcl_rc_id;
	dst->remoteRcId = src.rem_rc_id;

	return dst;
}

std::string &
operator << (std::string &                        dst,
	     const g2mplsTypes::teLinkIdent_var & src)
{
	telink_ident_t tmp;

	dst = std::string("");

	try {
		tmp << src;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "g2mplsTypes::teLinkIdent_var "
				       "to convert");
	}

	dst = teLinkIdent2string(tmp);

	return dst;
}


//
// tnaIdent
//
tna_ident_t &
operator << (tna_ident_t &                     dst,
	     const g2mplsTypes::tnaIdent_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.rc_id = src->rc;
	dst.rid   = src->node;

	try {
		g2mplsTypes::tnaId_var tmp;
		tmp = src->tna;
		dst.id << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed tna in "
				       "g2mplsTypes::tnaIdent_var "
				       "to convert");
	}

	dst.id.preflen = src->prefix;

	return dst;
}

g2mplsTypes::tnaIdent_var &
operator << (g2mplsTypes::tnaIdent_var &       dst,
	     const tna_ident_t &               src)
{
	dst = new g2mplsTypes::tnaIdent;

	dst->rc   = src.rc_id;
	dst->node = src.rid;

	try {
		g2mplsTypes::tnaId_var tmp;
		tmp << src.id;
		dst->tna = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed id in "
				       "tna_ident_t "
				       "to convert");
	}

	dst->prefix = src.id.preflen;

	return dst;
}

std::string &
operator << (std::string &                     dst,
	     const g2mplsTypes::tnaIdent_var & src)
{
	tna_ident_t tmp;

	dst = std::string("");

	try {
		tmp << src;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "g2mplsTypes::tnaIdent_var "
				       "to convert");
	}

	dst = tnaIdent2string(tmp);

	return dst;
}

//
//netNodeParams
//
net_node_data_t &
operator << (net_node_data_t &                      dst,
	     const g2mplsTypes::netNodeParams_var & src)
{
	memset(&dst.mask_, 0, sizeof(dst.mask_));

	dst.is_domain = (src->isDomain) ? 1 : 0;

	try {
		dst.op_state  << src->state.opState;
		dst.adm_state << src->state.admState;
	} catch(...) {
		throw invalid_argument("Invalid or malformed "
				       "g2mplsTypes::statesBundle "
				       "to convert");
	}

	dst.te_colors = src->colors;

	dst.areas.clear();

	size_t i;

	for (i = 0; i < src->areas.length(); i++) {
		dst.areas.push_back(src->areas[i]);
	}

	BITMASK_BITSET(dst.mask_, is_domain);
	BITMASK_BITSET(dst.mask_, op_state);
	BITMASK_BITSET(dst.mask_, adm_state);
	BITMASK_BITSET(dst.mask_, te_colors);
	BITMASK_BITSET(dst.mask_, areas);

	return dst;
}

g2mplsTypes::netNodeParams_var &
operator << (g2mplsTypes::netNodeParams_var & dst,
	     const net_node_data_t &          src)
{
	dst = new g2mplsTypes::netNodeParams;

	dst->isDomain       = false;
	dst->state.opState  = g2mplsTypes::OPERSTATE_DOWN;
	dst->state.admState = g2mplsTypes::ADMINSTATE_DISABLED;
	dst->colors         = 0;

	if (BITMASK_BITTEST(src.mask_, is_domain)) {
		dst->isDomain = (src.is_domain) ? true : false;
	}

	if (BITMASK_BITTEST(src.mask_, op_state)) {
		try {
			dst->state.opState  << src.op_state;
		} catch(...) {
			throw invalid_argument("Invalid or malformed "
					       "operative state "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, adm_state)) {
		try {
			dst->state.admState << src.adm_state;
		} catch(...) {
			throw invalid_argument("Invalid or malformed "
					       "administrative state "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, te_colors)) {
		dst->colors = src.te_colors;
	}

	g2mplsTypes::areaSeq_var areas;
	g2mplsTypes::areaSeq *   tmp;
	size_t                   size;

	size = 0;

	if (BITMASK_BITTEST(src.mask_, areas)) {
		size = src.areas.size();
	}

	tmp = new g2mplsTypes::areaSeq(size);
	if (!tmp) {
		throw out_of_range("Cannot allocate "
				   "g2mplsTypes::areaSeq");
	}
	areas = tmp;
	areas->length(size);

	if (BITMASK_BITTEST(src.mask_, areas) && size) {
		std::list<uint16_t>::const_iterator iter;
		size_t                              i;

		for (iter = src.areas.begin(), i = 0;
		     iter != src.areas.end();
		     iter++, i++) {
			areas[i] = *iter;
		}
	}

	dst->areas = areas;

	return dst;
}


//
//gridSiteParams
//
grid_site_data_t &
operator << (grid_site_data_t &                      dst,
	     const g2mplsTypes::gridSiteParams_var & src)
{
	memset(&dst.mask_, 0, sizeof(dst.mask_));


	dst.name = new std::string(src->name);

	if (src->location.valid) {
		dst.location.lat_resolution =
			(uint8_t) (src->location.latResolution & 0x000000FF);
		dst.location.latitude =
			src->location.latitute;
		dst.location.lon_resolution =
			(uint8_t) (src->location.lonResolution & 0x000000FF);
		dst.location.longitude =
			src->location.longitude;
		BITMASK_BITSET(dst.mask_, location);
	} else {
		memset(&dst.location, 0, sizeof(dst.location));
	}

	dst.pe_router_id = src->peRouterId;

	BITMASK_BITSET(dst.mask_, name);
	BITMASK_BITSET(dst.mask_, pe_router_id);

	return dst;
}

g2mplsTypes::gridSiteParams_var &
operator << (g2mplsTypes::gridSiteParams_var & dst,
	     const grid_site_data_t &          src)
{
	dst = new g2mplsTypes::gridSiteParams;

	dst->name                   = CORBA::string_dup("");
	dst->location.valid         = false;
	dst->location.latResolution = 0;
	dst->location.latitute      = 0;
	dst->location.lonResolution = 0;
	dst->location.longitude     = 0;
	dst->peRouterId             = 0;

	if (BITMASK_BITTEST(src.mask_, name) && src.name) {
		dst->name = CORBA::string_dup(src.name->c_str());
	}

	if (BITMASK_BITTEST(src.mask_, location)) {
		dst->location.valid = true;
		dst->location.latResolution =
			(uint32_t) src.location.lat_resolution;
		dst->location.latitute =
			src.location.latitude;
		dst->location.lonResolution =
			(uint32_t) src.location.lon_resolution;
		dst->location.longitude =
			src.location.longitude;
	}

	if (BITMASK_BITTEST(src.mask_, pe_router_id)) {
		dst->peRouterId = src.pe_router_id;
	}

	return dst;
}

//
//gridSubNodes
//
grid_subnodes_t &
operator << (grid_subnodes_t &                     dst,
	     const g2mplsTypes::gridSubNodes_var & src)
{
	dst.services.clear();
	dst.comp_elems.clear();
	dst.sub_clusters.clear();
	dst.storage_elems.clear();

	size_t i;

	for (i = 0; i < src->services.length(); i++) {
		grid_subnode_ident_t id;
		try {
			g2mplsTypes::gridSubNodeIdent_var tmp;
			tmp = src->services[i];
			id << tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "gridSubNodeIdent "
					       "to convert");
		}
		dst.services.push_back(id);
	}
	for (i = 0; i < src->compElems.length(); i++) {
		grid_subnode_ident_t id;
		try {
			g2mplsTypes::gridSubNodeIdent_var tmp;
			tmp = src->compElems[i];
			id << tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "gridSubNodeIdent "
					       "to convert");
		}
		dst.comp_elems.push_back(id);
	}
	for (i = 0; i < src->subClusters.length(); i++) {
		grid_subnode_ident_t id;
		try {
			g2mplsTypes::gridSubNodeIdent_var tmp;
			tmp = src->subClusters[i];
			id << tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "gridSubNodeIdent "
					       "to convert");
		}
		dst.sub_clusters.push_back(id);
	}
	for (i = 0; i < src->storageElems.length(); i++) {
		grid_subnode_ident_t id;
		try {
			g2mplsTypes::gridSubNodeIdent_var tmp;
			tmp = src->storageElems[i];
			id << tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "gridSubNodeIdent "
					       "to convert");
		}
		dst.storage_elems.push_back(id);
	}

	return dst;
}

g2mplsTypes::gridSubNodes_var &
operator << (g2mplsTypes::gridSubNodes_var & dst,
	     const grid_subnodes_t &         src)
{
	dst = new g2mplsTypes::gridSubNodes;

	// services
	{
		g2mplsTypes::gridSubNodeIdentSeq_var idents;
		g2mplsTypes::gridSubNodeIdentSeq *   tmp;
		size_t                               size;

		size = src.services.size();
		tmp = new g2mplsTypes::gridSubNodeIdentSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::gridSubNodeIdentSeq");
		}
		idents = tmp;
		idents->length(size);

		std::list<grid_subnode_ident_t>::const_iterator iter;
		size_t                                          i;

		for (iter  = src.services.begin(), i = 0;
		     iter != src.services.end();
		     iter++, i++) {
			try {
				g2mplsTypes::gridSubNodeIdent_var tmp;
				tmp << (*iter);
				idents[i] = tmp;
			} catch (...) {
				throw invalid_argument("Invalid or malformed "
						       "grid_subnode_ident_t "
						       "to convert");
			}
		}

		dst->services = idents;
		idents._retn();
	}

	// compElems
	{
		g2mplsTypes::gridSubNodeIdentSeq_var idents;
		g2mplsTypes::gridSubNodeIdentSeq *   tmp;
		size_t                               size;

		size = src.comp_elems.size();
		tmp = new g2mplsTypes::gridSubNodeIdentSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::gridSubNodeIdentSeq");
		}
		idents = tmp;
		idents->length(size);

		std::list<grid_subnode_ident_t>::const_iterator iter;
		size_t                                          i;

		for (iter  = src.comp_elems.begin(), i = 0;
		     iter != src.comp_elems.end();
		     iter++, i++) {
			try {
				g2mplsTypes::gridSubNodeIdent_var tmp;
				tmp << (*iter);
				idents[i] = tmp;
			} catch (...) {
				throw invalid_argument("Invalid or malformed "
						       "grid_subnode_ident_t "
						       "to convert");
			}
		}

		dst->compElems = idents;
		idents._retn();
	}

	// subClusters
	{
		g2mplsTypes::gridSubNodeIdentSeq_var idents;
		g2mplsTypes::gridSubNodeIdentSeq *   tmp;
		size_t                               size;

		size = src.sub_clusters.size();
		tmp = new g2mplsTypes::gridSubNodeIdentSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::gridSubNodeIdentSeq");
		}
		idents = tmp;
		idents->length(size);

		std::list<grid_subnode_ident_t>::const_iterator iter;
		size_t                                          i;

		for (iter  = src.sub_clusters.begin(), i = 0;
		     iter != src.sub_clusters.end();
		     iter++, i++) {
			try {
				g2mplsTypes::gridSubNodeIdent_var tmp;
				tmp << (*iter);
				idents[i] = tmp;
			} catch (...) {
				throw invalid_argument("Invalid or malformed "
						       "grid_subnode_ident_t "
						       "to convert");
			}

		}

		dst->subClusters = idents;
		idents._retn();
	}

	// storageElems
	{
		g2mplsTypes::gridSubNodeIdentSeq_var idents;
		g2mplsTypes::gridSubNodeIdentSeq *   tmp;
		size_t                               size;

		size = src.storage_elems.size();
		tmp = new g2mplsTypes::gridSubNodeIdentSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::gridSubNodeIdentSeq");
		}
		idents = tmp;
		idents->length(size);

		std::list<grid_subnode_ident_t>::const_iterator iter;
		size_t                                          i;

		for (iter  = src.storage_elems.begin(), i = 0;
		     iter != src.storage_elems.end();
		     iter++, i++) {
			try {
				g2mplsTypes::gridSubNodeIdent_var tmp;
				tmp << (*iter);
				idents[i] = tmp;
			} catch (...) {
				throw invalid_argument("Invalid or malformed "
						       "grid_subnode_ident_t "
						       "to convert");
			}
		}

		dst->storageElems = idents;
		idents._retn();
	}

	return dst;
}


//
// gridHostId
//
g2mpls_addr_t &
operator<<(g2mpls_addr_t &                          dst,
	   const g2mplsTypes::gridHostId_var &      src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes::GRIDHOSTTYPE_IPV4: {
			dst.type    = IPv4;
			dst.preflen = 32;
			dst.value.ipv4.s_addr = htonl(src->ipv4());
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_IPV6: {
			int    i;

			dst.type    = IPv6;
			dst.preflen = 128;

			for (i = 0; i < 4; i++) {
				dst.value.ipv6.s6_addr32[i] =
					htonl(src->ipv6()[i]);
			}
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_NSAP: {
			int    i;

			dst.type    = NSAP;
			dst.preflen = 160;

			for (i = 0; i < 20; i++) {
				dst.value.nsap.nsap_addr8[i] =
					src->nsap()[i];
			}
			break;
		}
		default:
			throw out_of_range("gridHostId type out-of-range");
			break;
	}

	return dst;
}

g2mplsTypes::gridHostId_var &
operator<<(g2mplsTypes::gridHostId_var &            dst,
	   const g2mpls_addr_t &                    src)
{
	dst = new g2mplsTypes::gridHostId;

	switch (src.type) {
		case IPv4:
			dst->ipv4(ntohl(src.value.ipv4.s_addr));
			break;
		case IPv6: {
			g2mplsTypes::addrIPv6 parts;
			int i;

			for (i = 0; i < 4; i++) {
				parts[i] =
					ntohl(src.value.ipv6.s6_addr32[i]);
			}
			dst->ipv6(parts);
		}
			break;
		case NSAP: {
			g2mplsTypes::addrNSAP parts;
			int i;

			for (i = 0; i < 20; i++) {
				parts[i] = src.value.nsap.nsap_addr8[i];
			}
			dst->nsap(parts);
		}
			break;
		default:
			throw out_of_range("gridHostId G2.ADDR type "
					   "out-of-range");
			break;
	}

	return dst;
}

std::string &
operator<<(std::string &                            dst,
	   const g2mplsTypes::gridHostId_var &      src)
{
	std::ostringstream id;

	dst = std::string("");

	switch (src->_d()) {
		case g2mplsTypes::GRIDHOSTTYPE_IPV4: {
			ipv4_t addr;

			addr.s_addr = htonl(src->ipv4());
			id << "(GRID-HOST--IPv4) " << inet_ntoa(addr);
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_IPV6: {
			int    i;

			id << "(GRID-HOST--IPv6) " ;
			for (i = 0; i < 4; i++) {
				id << std::hex
				   << std::setw(8)
				   << std::setfill('0')
				   << src->ipv6()[i];
			}
			break;
		}
		case g2mplsTypes::GRIDHOSTTYPE_NSAP: {
			int    i;

			id << "(GRID-HOST--NSAP) " ;
			for (i = 0; i < 20; i++) {

				id << std::hex
				   << std::setw(2)
				   << std::setfill('0')
				   << (uint32_t) src->nsap()[i];
				if ((i == 0)  | (i == 2)  | (i == 3)  |
				    (i == 6)  | (i == 8)  | (i == 10) |
				    (i == 12) | (i == 18)) {
					id << ".";
				}
			}
			break;
		}
		default:
			throw out_of_range("gridHostId type out-of-range");
			break;
	}

	dst += id.str();

	return dst;
}

//
//gridServiceState
//
grid_service_state_t &
operator<<(grid_service_state_t &                   dst,
	   const g2mplsTypes::gridServiceState &    src)
{
	switch (src) {
		case g2mplsTypes::GRIDSERVICESTATE_UNKNOWN: {
			dst = SERVICE_STATE_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDSERVICESTATE_OK: {
			dst = SERVICE_STATE_OK;
		}
			break;
		case g2mplsTypes::GRIDSERVICESTATE_WARNING: {
			dst = SERVICE_STATE_WARNING;
		}
			break;
		case g2mplsTypes::GRIDSERVICESTATE_CRITICAL: {
			dst = SERVICE_STATE_CRITICAL;
		}
			break;
		case g2mplsTypes::GRIDSERVICESTATE_OTHER: {
			dst = SERVICE_STATE_OTHER;
		}
			break;
		default: {
			throw out_of_range("gridServiceState out-of-range");
		}
			break;
	}

	return dst;
}

g2mplsTypes::gridServiceState &
operator<<(g2mplsTypes::gridServiceState &          dst,
	   const grid_service_state_t &             src)
{
	switch (src) {
		case SERVICE_STATE_UNKNOWN: {
			dst = g2mplsTypes::GRIDSERVICESTATE_UNKNOWN;
		}
			break;
		case SERVICE_STATE_OK: {
			dst = g2mplsTypes::GRIDSERVICESTATE_OK;
		}
			break;
		case SERVICE_STATE_WARNING: {
			dst = g2mplsTypes::GRIDSERVICESTATE_WARNING;
		}
			break;
		case SERVICE_STATE_CRITICAL: {
			dst = g2mplsTypes::GRIDSERVICESTATE_CRITICAL;
		}
			break;
		case SERVICE_STATE_OTHER: {
			dst = g2mplsTypes::GRIDSERVICESTATE_OTHER;
		}
			break;
		default: {
			throw out_of_range("grid_service_state out-of-range");
		}
			break;
	}

	return dst;
}

//
// gridServiceInfo
//
grid_service_info_t &
operator<<(grid_service_info_t &                    dst,
	   const g2mplsTypes::gridServiceInfo_var & src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	switch (src->type) {
		case g2mplsTypes::SERVICE_UNKNOWN: {
			dst.type = SERVICE_UNKNOWN;
		}
			break;
		case g2mplsTypes::ORG_GLITE_WMS: {
			dst.type = ORG_GLITE_WMS;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_LATESTPRODUCER: {
			dst.type = ORG_GLITE_RGMA_LATESTPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_STREAMPRODUCER: {
			dst.type = ORG_GLITE_RGMA_STREAMPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_DBPRODUCER: {
			dst.type = ORG_GLITE_RGMA_DBPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_CANONICALPRODUCER: {
			dst.type = ORG_GLITE_RGMA_CANONICALPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_ARCHIVER: {
			dst.type = ORG_GLITE_RGMA_ARCHIVER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_CONSUMER: {
			dst.type = ORG_GLITE_RGMA_CONSUMER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_REGISTRY: {
			dst.type = ORG_GLITE_RGMA_REGISTRY;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_SCHEMA: {
			dst.type = ORG_GLITE_RGMA_SCHEMA;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_BROWSER: {
			dst.type = ORG_GLITE_RGMA_BROWSER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_PRIMARYPRODUCER: {
			dst.type = ORG_GLITE_RGMA_PRIMARYPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_SECONDARYPRODUCER: {
			dst.type = ORG_GLITE_RGMA_SECONDARYPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_RGMA_ONDEMANDPRODUCER: {
			dst.type = ORG_GLITE_RGMA_ONDEMANDPRODUCER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_VOMS: {
			dst.type = ORG_GLITE_VOMS;
		}
			break;
		case g2mplsTypes::ORG_GLITE_FIREMANCATALOG: {
			dst.type = ORG_GLITE_FIREMANCATALOG;
		}
			break;
		case g2mplsTypes::ORG_GLITE_SEINDEX: {
			dst.type = ORG_GLITE_SEINDEX;
		}
			break;
		case g2mplsTypes::ORG_GLITE_METADATA: {
			dst.type = ORG_GLITE_METADATA;
		}
			break;
		case g2mplsTypes::ORG_GLITE_CHANNELMANAGEMENT: {
			dst.type = ORG_GLITE_CHANNELMANAGEMENT;
		}
			break;
		case g2mplsTypes::ORG_GLITE_FILETRANSFER: {
			dst.type = ORG_GLITE_FILETRANSFER;
		}
			break;
		case g2mplsTypes::ORG_GLITE_FILETRANSFERSTATS: {
			dst.type = ORG_GLITE_FILETRANSFERSTATS;
		}
			break;
		case g2mplsTypes::ORG_GLITE_CHANNELAGENT: {
			dst.type = ORG_GLITE_CHANNELAGENT;
		}
			break;
		case g2mplsTypes::ORG_GLITE_KEYSTORE: {
			dst.type = ORG_GLITE_KEYSTORE;
		}
			break;
		case g2mplsTypes::ORG_GLITE_FAS: {
			dst.type = ORG_GLITE_FAS;
		}
			break;
		case g2mplsTypes::ORG_GLITE_GLITEIO: {
			dst.type = ORG_GLITE_GLITEIO;
		}
			break;
		case g2mplsTypes::SRM: {
			dst.type = SRM;
		}
			break;
		case g2mplsTypes::GSIFTP: {
			dst.type = GSIFTP;
		}
			break;
		case g2mplsTypes::ORG_EDG_LOCAL_REPLICA_CATALOG: {
			dst.type = ORG_EDG_LOCAL_REPLICA_CATALOG;
		}
			break;
		case g2mplsTypes::ORG_EDG_REPLICA_METADATA_CATALOG: {
			dst.type = ORG_EDG_REPLICA_METADATA_CATALOG;
		}
			break;
		case g2mplsTypes::ORG_EDG_SE: {
			dst.type = ORG_EDG_SE;
		}
			break;
		case g2mplsTypes::IT_INFN_GRIDICE: {
			dst.type = IT_INFN_GRIDICE;
		}
			break;
		case g2mplsTypes::MYPROXY: {
			dst.type = MYPROXY;
		}
			break;
		case g2mplsTypes::GUMS: {
			dst.type = GUMS;
		}
			break;
		case g2mplsTypes::GRIDCAT: {
			dst.type = GRIDCAT;
		}
			break;
		case g2mplsTypes::EDU_CALTECH_CACR_MONALISA: {
			dst.type = EDU_CALTECH_CACR_MONALISA;
		}
			break;
		case g2mplsTypes::OPENSSH: {
			dst.type = OPENSSH;
		}
			break;
		case g2mplsTypes::MDS_GIIS: {
			dst.type = MDS_GIIS;
		}
			break;
		case g2mplsTypes::BDII: {
			dst.type = BDII;
		}
			break;
		case g2mplsTypes::RLS: {
			dst.type = RLS;
		}
			break;
		case g2mplsTypes::DATA_LOCATION_INTERFACE: {
			dst.type = DATA_LOCATION_INTERFACE;
		}
			break;
		case g2mplsTypes::PBS_TORQUE_SERVER: {
			dst.type = PBS_TORQUE_SERVER;
		}
			break;
		case g2mplsTypes::PBS_TORQUE_MAUI: {
			dst.type = PBS_TORQUE_MAUI;
		}
			break;
		case g2mplsTypes::UNICORE_CORE_TARGETSYSTEMFACTORY: {
			dst.type = UNICORE_CORE_TARGETSYSTEMFACTORY;
		}
			break;
		case g2mplsTypes::UNICORE_CORE_TARGETSYSTEM: {
			dst.type = UNICORE_CORE_TARGETSYSTEM;
		}
			break;
		case g2mplsTypes::UNICORE_CORE_STORAGEMANAGEMENT: {
			dst.type = UNICORE_CORE_STORAGEMANAGEMENT;
		}
			break;
		case g2mplsTypes::UNICORE_CORE_FILETRANSFER: {
			dst.type = UNICORE_CORE_FILETRANSFER;
		}
			break;
		case g2mplsTypes::UNICORE_CORE_JOBMANAGEMENT: {
			dst.type = UNICORE_CORE_JOBMANAGEMENT;
		}
			break;
		case g2mplsTypes::UNICORE_CORE_REGISTRY: {
			dst.type = UNICORE_CORE_REGISTRY;
		}
			break;
		case g2mplsTypes::UNICORE_WORKFLOW_WORKFLOWFACTORY: {
			dst.type = UNICORE_WORKFLOW_WORKFLOWFACTORY;
		}
			break;
		case g2mplsTypes::UNICORE_WORKFLOW_WORKFLOWMANAGEMENT: {
			dst.type = UNICORE_WORKFLOW_WORKFLOWMANAGEMENT;
		}
			break;
		case g2mplsTypes::UNICORE_WORKFLOW_SERVICEORCHESTRATOR: {
			dst.type = UNICORE_WORKFLOW_SERVICEORCHESTRATOR;
		}
			break;
		case g2mplsTypes::UNICORE_WORKFLOW_GRIDRESOURCEINFORMATIONSERVICE: {
			dst.type = UNICORE_WORKFLOW_GRIDRESOURCEINFORMATIONSERVICE;
		}
			break;
		case g2mplsTypes::UNICORE_CISINFORMATIONPROVIDER: {
			dst.type = UNICORE_CISINFORMATIONPROVIDER;
		}
			break;
		case g2mplsTypes::SERVICE_OTHER: {
			dst.type = SERVICE_OTHER;
		}
			break;
		default: {
			throw out_of_range("gridServiceType out-of-range");
		}
			break;
	}


	dst.version.major_revision = (uint16_t) (src->mjrRev & 0x0F);
	dst.version.minor_revision = (uint16_t) (src->mnrRev & 0x2F);
	dst.version.build_fix      = (uint16_t) (src->bldFix & 0x2F);

	return dst;
}

g2mplsTypes::gridServiceInfo_var &
operator<<(g2mplsTypes::gridServiceInfo_var &       dst,
	   const grid_service_info_t &              src)
{
	dst = new g2mplsTypes::gridServiceInfo;

	dst->valid = true;

	switch (src.type) {
		case SERVICE_UNKNOWN: {
			dst->type = g2mplsTypes::SERVICE_UNKNOWN;
		}
			break;
		case ORG_GLITE_WMS: {
			dst->type = g2mplsTypes::ORG_GLITE_WMS;
		}
			break;
		case ORG_GLITE_RGMA_LATESTPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_LATESTPRODUCER;
		}
			break;
		case ORG_GLITE_RGMA_STREAMPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_STREAMPRODUCER;
		}
			break;
		case ORG_GLITE_RGMA_DBPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_DBPRODUCER;
		}
			break;
		case ORG_GLITE_RGMA_CANONICALPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_CANONICALPRODUCER;
		}
			break;
		case ORG_GLITE_RGMA_ARCHIVER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_ARCHIVER;
		}
			break;
		case ORG_GLITE_RGMA_CONSUMER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_CONSUMER;
		}
			break;
		case ORG_GLITE_RGMA_REGISTRY: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_REGISTRY;
		}
			break;
		case ORG_GLITE_RGMA_SCHEMA: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_SCHEMA;
		}
			break;
		case ORG_GLITE_RGMA_BROWSER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_BROWSER;
		}
			break;
		case ORG_GLITE_RGMA_PRIMARYPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_PRIMARYPRODUCER;
		}
			break;
		case ORG_GLITE_RGMA_SECONDARYPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_SECONDARYPRODUCER;
		}
			break;
		case ORG_GLITE_RGMA_ONDEMANDPRODUCER: {
			dst->type = g2mplsTypes::ORG_GLITE_RGMA_ONDEMANDPRODUCER;
		}
			break;
		case ORG_GLITE_VOMS: {
			dst->type = g2mplsTypes::ORG_GLITE_VOMS;
		}
			break;
		case ORG_GLITE_FIREMANCATALOG: {
			dst->type = g2mplsTypes::ORG_GLITE_FIREMANCATALOG;
		}
			break;
		case ORG_GLITE_SEINDEX: {
			dst->type = g2mplsTypes::ORG_GLITE_SEINDEX;
		}
			break;
		case ORG_GLITE_METADATA: {
			dst->type = g2mplsTypes::ORG_GLITE_METADATA;
		}
			break;
		case ORG_GLITE_CHANNELMANAGEMENT: {
			dst->type = g2mplsTypes::ORG_GLITE_CHANNELMANAGEMENT;
		}
			break;
		case ORG_GLITE_FILETRANSFER: {
			dst->type = g2mplsTypes::ORG_GLITE_FILETRANSFER;
		}
			break;
		case ORG_GLITE_FILETRANSFERSTATS: {
			dst->type = g2mplsTypes::ORG_GLITE_FILETRANSFERSTATS;
		}
			break;
		case ORG_GLITE_CHANNELAGENT: {
			dst->type = g2mplsTypes::ORG_GLITE_CHANNELAGENT;
		}
			break;
		case ORG_GLITE_KEYSTORE: {
			dst->type = g2mplsTypes::ORG_GLITE_KEYSTORE;
		}
			break;
		case ORG_GLITE_FAS: {
			dst->type = g2mplsTypes::ORG_GLITE_FAS;
		}
			break;
		case ORG_GLITE_GLITEIO: {
			dst->type = g2mplsTypes::ORG_GLITE_GLITEIO;
		}
			break;
		case SRM: {
			dst->type = g2mplsTypes::SRM;
		}
			break;
		case GSIFTP: {
			dst->type = g2mplsTypes::GSIFTP;
		}
			break;
		case ORG_EDG_LOCAL_REPLICA_CATALOG: {
			dst->type = g2mplsTypes::ORG_EDG_LOCAL_REPLICA_CATALOG;
		}
			break;
		case ORG_EDG_REPLICA_METADATA_CATALOG: {
			dst->type = g2mplsTypes::ORG_EDG_REPLICA_METADATA_CATALOG;
		}
			break;
		case ORG_EDG_SE: {
			dst->type = g2mplsTypes::ORG_EDG_SE;
		}
			break;
		case IT_INFN_GRIDICE: {
			dst->type = g2mplsTypes::IT_INFN_GRIDICE;
		}
			break;
		case MYPROXY: {
			dst->type = g2mplsTypes::MYPROXY;
		}
			break;
		case GUMS: {
			dst->type = g2mplsTypes::GUMS;
		}
			break;
		case GRIDCAT: {
			dst->type = g2mplsTypes::GRIDCAT;
		}
			break;
		case EDU_CALTECH_CACR_MONALISA: {
			dst->type = g2mplsTypes::EDU_CALTECH_CACR_MONALISA;
		}
			break;
		case OPENSSH: {
			dst->type = g2mplsTypes::OPENSSH;
		}
			break;
		case MDS_GIIS: {
			dst->type = g2mplsTypes::MDS_GIIS;
		}
			break;
		case BDII: {
			dst->type = g2mplsTypes::BDII;
		}
			break;
		case RLS: {
			dst->type = g2mplsTypes::RLS;
		}
			break;
		case DATA_LOCATION_INTERFACE: {
			dst->type = g2mplsTypes::DATA_LOCATION_INTERFACE;
		}
			break;
		case PBS_TORQUE_SERVER: {
			dst->type = g2mplsTypes::PBS_TORQUE_SERVER;
		}
			break;
		case PBS_TORQUE_MAUI: {
			dst->type = g2mplsTypes::PBS_TORQUE_MAUI;
		}
			break;
		case UNICORE_CORE_TARGETSYSTEMFACTORY: {
			dst->type = g2mplsTypes::UNICORE_CORE_TARGETSYSTEMFACTORY;
		}
			break;
		case UNICORE_CORE_TARGETSYSTEM: {
			dst->type = g2mplsTypes::UNICORE_CORE_TARGETSYSTEM;
		}
			break;
		case UNICORE_CORE_STORAGEMANAGEMENT: {
			dst->type = g2mplsTypes::UNICORE_CORE_STORAGEMANAGEMENT;
		}
			break;
		case UNICORE_CORE_FILETRANSFER: {
			dst->type = g2mplsTypes::UNICORE_CORE_FILETRANSFER;
		}
			break;
		case UNICORE_CORE_JOBMANAGEMENT: {
			dst->type = g2mplsTypes::UNICORE_CORE_JOBMANAGEMENT;
		}
			break;
		case UNICORE_CORE_REGISTRY: {
			dst->type = g2mplsTypes::UNICORE_CORE_REGISTRY;
		}
			break;
		case UNICORE_WORKFLOW_WORKFLOWFACTORY: {
			dst->type = g2mplsTypes::UNICORE_WORKFLOW_WORKFLOWFACTORY;
		}
			break;
		case UNICORE_WORKFLOW_WORKFLOWMANAGEMENT: {
			dst->type = g2mplsTypes::UNICORE_WORKFLOW_WORKFLOWMANAGEMENT;
		}
			break;
		case UNICORE_WORKFLOW_SERVICEORCHESTRATOR: {
			dst->type = g2mplsTypes::UNICORE_WORKFLOW_SERVICEORCHESTRATOR;
		}
			break;
		case UNICORE_WORKFLOW_GRIDRESOURCEINFORMATIONSERVICE: {
			dst->type = g2mplsTypes::UNICORE_WORKFLOW_GRIDRESOURCEINFORMATIONSERVICE;
		}
			break;
		case UNICORE_CISINFORMATIONPROVIDER: {
			dst->type = g2mplsTypes::UNICORE_CISINFORMATIONPROVIDER;
		}
			break;
		case SERVICE_OTHER: {
			dst->type = g2mplsTypes::SERVICE_OTHER;
		}
			break;
		default: {
			throw out_of_range("grid_service_type out-of-range");
		}
			break;
	}


	dst->mjrRev = src.version.major_revision;
	dst->mnrRev = src.version.minor_revision;
	dst->bldFix = src.version.build_fix;

	return dst;
}


//
//gridServiceParams
//
grid_service_data_t &
operator << (grid_service_data_t &                      dst,
	     const g2mplsTypes::gridServiceParams_var & src)
{
	memset(&dst.mask_, 0, sizeof(dst.mask_));

	try {
		g2mplsTypes::gridServiceInfo_var tmp;
		tmp = src->data;
		dst.data << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed data in "
				       "g2mplsTypes::gridServiceParams_var "
				       "to convert");
	}

	try {
		dst.state << src->state;
	} catch (...) {
		throw invalid_argument("Invalid or malformed state in "
				       "g2mplsTypes::gridServiceParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridHostId_var tmp;
		tmp = src->endPointAddr;
		dst.endpoint_addr << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed endPointAddr in "
				       "g2mplsTypes::gridServiceParams_var "
				       "to convert");
	}

	BITMASK_BITSET(dst.mask_, data);
	BITMASK_BITSET(dst.mask_, state);
	BITMASK_BITSET(dst.mask_, endpoint_addr);

	return dst;
}

g2mplsTypes::gridServiceParams_var &
operator << (g2mplsTypes::gridServiceParams_var & dst,
	     const grid_service_data_t &          src)
{
	dst = new g2mplsTypes::gridServiceParams;

	dst->data.valid   = false;
	dst->data.type    = g2mplsTypes::SERVICE_UNKNOWN;
	dst->data.mjrRev  = 0;
	dst->data.mnrRev  = 0;
	dst->data.bldFix  = 0;
	dst->state        = g2mplsTypes::GRIDSERVICESTATE_UNKNOWN;
	dst->endPointAddr.value(0);

	if (BITMASK_BITTEST(src.mask_, data)) {
		try {
			g2mplsTypes::gridServiceInfo_var tmp;
			tmp << src.data;
			dst->data = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed data in "
					       "grid_service_data_t "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, state)) {
		try {
			dst->state << src.state;
		} catch (...) {
			throw invalid_argument("Invalid or malformed state in "
					       "grid_service_data_t "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, endpoint_addr)) {
		try {
			g2mplsTypes::gridHostId_var tmp;
			tmp << src.endpoint_addr;
			dst->endPointAddr = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "endpoint_addr_t in "
					       "grid_service_data_t "
					       "to convert");
		}
	}

	return dst;
}

//
// gridLrmsInfo
//
grid_lrms_info_t &
operator<<(grid_lrms_info_t &                      dst,
	   const g2mplsTypes::gridLrmsInfo_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	switch (src->type) {
		case g2mplsTypes::GRIDLRMSTYPE_UNKNOWN: {
			dst.type = LRMS_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_OPENPBS: {
			dst.type = LRMS_OPENPBS;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_LSF    : {
			dst.type = LRMS_LSF;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_CONDOR : {
			dst.type = LRMS_CONDOR;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_BQS    : {
			dst.type = LRMS_BQS;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_CONDORG: {
			dst.type = LRMS_CONDORG;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_FBSNG  : {
			dst.type = LRMS_FBSNG;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_TORQUE : {
			dst.type = LRMS_TORQUE;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_PBSPRO : {
			dst.type = LRMS_PBSPRO;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_SGE    : {
			dst.type = LRMS_SGE;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_NQE    : {
			dst.type = LRMS_NQE;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_FORK   : {
			dst.type = LRMS_FORK;
		}
			break;
		case g2mplsTypes::GRIDLRMSTYPE_OTHER  : {
			dst.type = LRMS_OTHER;
		}
			break;
		default: {
			throw out_of_range("gridLrmsType out-of-range");
		}
			break;
	}

	dst.version.major_revision = (uint16_t) (src->mjrRev & 0x0F);
	dst.version.minor_revision = (uint16_t) (src->mnrRev & 0x2F);
	dst.version.build_fix      = (uint16_t) (src->bldFix & 0x2F);

	return dst;
}

g2mplsTypes::gridLrmsInfo_var &
operator<<(g2mplsTypes::gridLrmsInfo_var &         dst,
	   const grid_lrms_info_t &                src)
{
	dst = new g2mplsTypes::gridLrmsInfo;

	dst->valid = true;

	switch (src.type) {
		case LRMS_UNKNOWN: {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_UNKNOWN;
		}
			break;
		case LRMS_OPENPBS: {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_OPENPBS;
		}
			break;
		case LRMS_LSF    : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_LSF;
		}
			break;
		case LRMS_CONDOR : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_CONDOR;
		}
			break;
		case LRMS_BQS    : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_BQS;
		}
			break;
		case LRMS_CONDORG: {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_CONDORG;
		}
			break;
		case LRMS_FBSNG  : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_FBSNG;
		}
			break;
		case LRMS_TORQUE : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_TORQUE;
		}
			break;
		case LRMS_PBSPRO : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_PBSPRO;
		}
			break;
		case LRMS_SGE    : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_SGE;
		}
			break;
		case LRMS_NQE    : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_NQE;
		}
			break;
		case LRMS_FORK   : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_FORK;
		}
			break;
		case LRMS_OTHER  : {
			dst->type = g2mplsTypes::GRIDLRMSTYPE_OTHER;
		}
			break;
		default: {
			throw out_of_range("grid_lrms_type_t out-of-range");
		}
			break;
	}

	dst->mjrRev = src.version.major_revision;
	dst->mnrRev = src.version.minor_revision;
	dst->bldFix = src.version.build_fix;

	return dst;
}

//
// gridJobsstate
//
grid_jobs_state_t &
operator<<(grid_jobs_state_t &                      dst,
	   const g2mplsTypes::gridJobsState_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.free_job_slots = src->freeJobSlots;

	switch(src->state) {
		case g2mplsTypes::GRIDCESESTATE_UNKNOWN: {
			dst.state = CESE_STATE_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_QUEUING: {
			dst.state = CESE_STATE_QUEUING;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_PRODUCTION: {
			dst.state = CESE_STATE_PRODUCTION;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_CLOSED: {
			dst.state = CESE_STATE_CLOSED;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_DRAINING: {
			dst.state = CESE_STATE_DRAINING;
		}
			break;
		default: {
			throw out_of_range("gridCeSeState out-of-range");
		}
			break;
	}

	return dst;
}

g2mplsTypes::gridJobsState_var &
operator<<(g2mplsTypes::gridJobsState_var & dst,
	   const grid_jobs_state_t &        src)
{
	dst = new g2mplsTypes::gridJobsState;

	dst->valid = true;

	dst->freeJobSlots = src.free_job_slots;

	switch(src.state) {
		case CESE_STATE_UNKNOWN: {
			dst->state = g2mplsTypes::GRIDCESESTATE_UNKNOWN;
		}
			break;
		case CESE_STATE_QUEUING: {
			dst->state = g2mplsTypes::GRIDCESESTATE_QUEUING;
		}
			break;
		case CESE_STATE_PRODUCTION: {
			dst->state = g2mplsTypes::GRIDCESESTATE_PRODUCTION;
		}
			break;
		case CESE_STATE_CLOSED: {
			dst->state = g2mplsTypes::GRIDCESESTATE_CLOSED;
		}
			break;
		case CESE_STATE_DRAINING: {
			dst->state = g2mplsTypes::GRIDCESESTATE_DRAINING;
		}
			break;
		default: {
			throw out_of_range("grid_cese_state_t out-of-range");
		}
			break;
	}

	return dst;
}

//
// gridJobsStats
//
grid_jobs_stats_t &
operator<<(grid_jobs_stats_t &                      dst,
	   const g2mplsTypes::gridJobsStats_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.running_jobs = src->runningJobs;
	dst.waiting_jobs = src->waitingJobs;
	dst.total_jobs   = src->totalJobs;

	return dst;
}

g2mplsTypes::gridJobsStats_var &
operator<<(g2mplsTypes::gridJobsStats_var & dst,
	   const grid_jobs_stats_t &        src)
{
	dst = new g2mplsTypes::gridJobsStats;

	dst->valid = true;

	dst->runningJobs = src.running_jobs;
	dst->waitingJobs = src.waiting_jobs;
	dst->totalJobs   = src.total_jobs;

	return dst;
}

//
// gridJobsTimePerf
//
grid_jobs_time_perf_t &
operator<<(grid_jobs_time_perf_t &                      dst,
	   const g2mplsTypes::gridJobsTimePerf_var &    src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.estimated_response_time = src->estimatedResponseTime;
	dst.worst_response_time     = src->worstResponseTime;

	return dst;
}

g2mplsTypes::gridJobsTimePerf_var &
operator<<(g2mplsTypes::gridJobsTimePerf_var &  dst,
	   const grid_jobs_time_perf_t &        src)
{
	dst = new g2mplsTypes::gridJobsTimePerf;

	dst->valid = true;

	dst->estimatedResponseTime = src.estimated_response_time;
	dst->worstResponseTime     = src.worst_response_time;

	return dst;
}

//
// gridJobsTimePolicy
//
grid_jobs_time_policy_t &
operator<<(grid_jobs_time_policy_t &                     dst,
	   const g2mplsTypes::gridJobsTimePolicy_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.max_wallclock_time            = src->maxWallclockTime;
	dst.max_obtainable_wallclock_time = src->maxObtainableWallclockTime;
	dst.max_cpu_time                  = src->maxCpuTime;
	dst.max_obtainable_cpu_time       = src->maxObtainableCpuTime;

	return dst;
}

g2mplsTypes::gridJobsTimePolicy_var &
operator<<(g2mplsTypes::gridJobsTimePolicy_var &  dst,
	   const grid_jobs_time_policy_t &        src)
{
	dst = new g2mplsTypes::gridJobsTimePolicy;

	dst->valid = true;

	dst->maxWallclockTime           = src.max_wallclock_time;
	dst->maxObtainableWallclockTime = src.max_obtainable_wallclock_time;
	dst->maxCpuTime                 = src.max_cpu_time;
	dst->maxObtainableCpuTime       = src.max_obtainable_cpu_time;

	return dst;
}

//
// gridJobsLoadpolicy
//
grid_jobs_load_policy_t &
operator<<(grid_jobs_load_policy_t &                     dst,
	   const g2mplsTypes::gridJobsLoadPolicy_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.max_total_jobs     = src->maxTotalJobs;
	dst.max_running_jobs   = src->maxRunningJobs;
	dst.max_waiting_jobs   = src->maxWaitingJobs;
	dst.assigned_job_slots = (uint16_t) (src->assignedJobSlots &
					     0x0000FFFF);
	dst.max_slots_per_job  = (uint16_t) (src->maxSlotsPerJob &
					     0x0000FFFF);
	dst.priority           = src->priority;
	dst.preemption_flag    = src->preemptionFlag ? 1 : 0;

	return dst;
}

g2mplsTypes::gridJobsLoadPolicy_var &
operator<<(g2mplsTypes::gridJobsLoadPolicy_var &  dst,
	   const grid_jobs_load_policy_t &        src)
{
	dst = new g2mplsTypes::gridJobsLoadPolicy;

	dst->valid = true;

	dst->maxTotalJobs     = src.max_total_jobs;
	dst->maxRunningJobs   = src.max_running_jobs;
	dst->maxWaitingJobs   = src.max_waiting_jobs;
	dst->assignedJobSlots = src.assigned_job_slots;
	dst->maxSlotsPerJob   = src.max_slots_per_job;
	dst->priority         = src.priority;
	dst->preemptionFlag   = (src.preemption_flag == 1) ? true : false;

	return dst;
}

//
//gridCEParams
//
grid_ce_data_t &
operator << (grid_ce_data_t &                      dst,
	     const g2mplsTypes::gridCEParams_var & src)
{
	memset(&dst.mask_, 0, sizeof(dst.mask_));

	dst.name = new std::string(src->name);

	try {
		g2mplsTypes::gridLrmsInfo_var tmp;
		tmp = src->lrmsInfo;
		dst.lrms_info << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "lrmsInfo in "
				       "gridCEParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridHostId_var tmp;
		tmp = src->hostAddr;
		dst.host_addr << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed hostAddr in "
				       "g2mplsTypes::gridCEParams_var "
				       "to convert");
	}

	dst.gatekeeper_port = src->gatekeeperPort;

	dst.job_manager = new std::string(src->jobManager);
	dst.data_dir    = new std::string(src->dataDir);

	dst.default_storage_elem_id = src->defaultStorageElemId;

	try {
		g2mplsTypes::gridJobsState_var tmp;
		tmp = src->jobsState;
		dst.jobs_state << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "jobState in "
				       "gridCEParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridJobsStats_var tmp;
		tmp = src->jobsStats;
		dst.jobs_stats << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "jobStats in "
				       "gridCEParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridJobsTimePerf_var tmp;
		tmp = src->jobsTimePerf;
		dst.jobs_timeperf << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "jobTimePerf in "
				       "gridCEParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridJobsTimePolicy_var tmp;
		tmp = src->jobsTimePolicy;
		dst.jobs_timepolicy << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "jobTimePolicy in "
				       "gridCEParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridJobsLoadPolicy_var tmp;
		tmp = src->jobsLoadPolicy;
		dst.jobs_loadpolicy << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "jobLoadPolicy in "
				       "gridCEParams_var "
				       "to convert");
	}

	dst.free_job_slots_calendar.clear();

	size_t i;

	for (i = 0; i < src->freeJobSlotsCalendar.length(); i++) {
		uint32_t	unixTime;
		uint16_t	free_job_slots;

		unixTime =
			src->freeJobSlotsCalendar[i].unixTime;
		free_job_slots =
			(uint16_t)
			(src->freeJobSlotsCalendar[i].JobSlots & 0x0000FFFF);

		dst.free_job_slots_calendar[unixTime] = free_job_slots;
	}

	BITMASK_BITSET(dst.mask_, name);
	BITMASK_BITSET(dst.mask_, lrms_info);
	BITMASK_BITSET(dst.mask_, host_addr);
	BITMASK_BITSET(dst.mask_, gatekeeper_port);
	BITMASK_BITSET(dst.mask_, job_manager);
	BITMASK_BITSET(dst.mask_, data_dir);
	BITMASK_BITSET(dst.mask_, default_storage_elem_id);
	BITMASK_BITSET(dst.mask_, jobs_state);
	BITMASK_BITSET(dst.mask_, jobs_stats);
	BITMASK_BITSET(dst.mask_, jobs_timeperf);
	BITMASK_BITSET(dst.mask_, jobs_timepolicy);
	BITMASK_BITSET(dst.mask_, jobs_loadpolicy);
	BITMASK_BITSET(dst.mask_, free_job_slots_calendar);

	return dst;
}

g2mplsTypes::gridCEParams_var &
operator << (g2mplsTypes::gridCEParams_var & dst,
	     const grid_ce_data_t &          src)
{
	dst = new g2mplsTypes::gridCEParams;

	dst->name = CORBA::string_dup("");

	dst->lrmsInfo.valid  = false;
	dst->lrmsInfo.type   = g2mplsTypes::GRIDLRMSTYPE_UNKNOWN;
	dst->lrmsInfo.mjrRev = 0;
	dst->lrmsInfo.mnrRev = 0;
	dst->lrmsInfo.bldFix = 0;

	dst->hostAddr.value(0);
	dst->gatekeeperPort         = 0;
	dst->jobManager             = CORBA::string_dup("");
	dst->dataDir                = CORBA::string_dup("");
	dst->defaultStorageElemId   = 0;

	dst->jobsState.valid        = false;
	dst->jobsState.freeJobSlots = 0;
	dst->jobsState.state        = g2mplsTypes::GRIDCESESTATE_UNKNOWN;

	dst->jobsStats.valid        = false;
	dst->jobsStats.runningJobs  = 0;
	dst->jobsStats.waitingJobs  = 0;
	dst->jobsStats.totalJobs    = 0;

	dst->jobsTimePerf.valid                 = false;
	dst->jobsTimePerf.estimatedResponseTime = false;
	dst->jobsTimePerf.worstResponseTime     = false;

	dst->jobsTimePolicy.valid                     = false;
	dst->jobsTimePolicy.maxWallclockTime          = 0;
	dst->jobsTimePolicy.maxObtainableWallclockTime= 0;
	dst->jobsTimePolicy.maxCpuTime                = 0;
	dst->jobsTimePolicy.maxObtainableCpuTime      = 0;

	dst->jobsLoadPolicy.valid            = false;
	dst->jobsLoadPolicy.maxTotalJobs     = 0;
	dst->jobsLoadPolicy.maxRunningJobs   = 0;
	dst->jobsLoadPolicy.maxWaitingJobs   = 0;
	dst->jobsLoadPolicy.assignedJobSlots = 0;
	dst->jobsLoadPolicy.maxSlotsPerJob   = 0;
	dst->jobsLoadPolicy.priority         = 0;
	dst->jobsLoadPolicy.preemptionFlag   = false;

	if (BITMASK_BITTEST(src.mask_, name) && src.name) {
		dst->name = CORBA::string_dup(src.name->c_str());
	}
	if (BITMASK_BITTEST(src.mask_, lrms_info)) {
		try {
			g2mplsTypes::gridLrmsInfo_var tmp;
			tmp << src.lrms_info;
			dst->lrmsInfo = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "lrms_info in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}
	if (BITMASK_BITTEST(src.mask_, host_addr)) {
		try {
			g2mplsTypes::gridHostId_var tmp;
			tmp << src.host_addr;
			dst->hostAddr = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "host_addr in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}
	if (BITMASK_BITTEST(src.mask_, gatekeeper_port)) {
		dst->gatekeeperPort = src.gatekeeper_port;
	}
	if (BITMASK_BITTEST(src.mask_, job_manager) && src.job_manager) {
		dst->jobManager = CORBA::string_dup(src.job_manager->c_str());
	}
	if (BITMASK_BITTEST(src.mask_, data_dir) && src.data_dir) {
		dst->dataDir = CORBA::string_dup(src.data_dir->c_str());
	}
	if (BITMASK_BITTEST(src.mask_, default_storage_elem_id)) {
		dst->defaultStorageElemId = src.default_storage_elem_id;
	}
	if (BITMASK_BITTEST(src.mask_, jobs_state)) {
		try {
			g2mplsTypes::gridJobsState_var tmp;
			tmp << src.jobs_state;
			dst->jobsState = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "job_state in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}
	if (BITMASK_BITTEST(src.mask_, jobs_stats)) {
		try {
			g2mplsTypes::gridJobsStats_var tmp;
			tmp << src.jobs_stats;
			dst->jobsStats = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "job_stats in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}
	if (BITMASK_BITTEST(src.mask_, jobs_timeperf)) {
		try {
			g2mplsTypes::gridJobsTimePerf_var tmp;
			tmp << src.jobs_timeperf;
			dst->jobsTimePerf = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "job_timeperf in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}
	if (BITMASK_BITTEST(src.mask_, jobs_timepolicy)) {
		try {
			g2mplsTypes::gridJobsTimePolicy_var tmp;
			tmp << src.jobs_timepolicy;
			dst->jobsTimePolicy = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "job_timepolicy in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}
	if (BITMASK_BITTEST(src.mask_, jobs_loadpolicy)) {
		try {
			g2mplsTypes::gridJobsLoadPolicy_var tmp;
			tmp << src.jobs_loadpolicy;
			dst->jobsLoadPolicy = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "job_loadpolicy in "
					       "grid_ce_data_t "
					       "to convert");
		}
	}

	// JobSlotsCalendar
	{
		g2mplsTypes::JobSlotsCalendarSeq_var cal;
		g2mplsTypes::JobSlotsCalendarSeq *   tmp;
		size_t                               size;

		size = 0;

		if (BITMASK_BITTEST(src.mask_, free_job_slots_calendar)) {
			size = src.free_job_slots_calendar.size();
		}

		tmp = new g2mplsTypes::JobSlotsCalendarSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::JobSlotsCalendarSeq");
		}
		cal = tmp;
		cal->length(size);

		if (BITMASK_BITTEST(src.mask_, free_job_slots_calendar) &&
		    size) {
			std::map<uint32_t, uint16_t>::const_iterator iter;
			size_t                                       i;

			for (iter = src.free_job_slots_calendar.begin(), i = 0;
			     iter != src.free_job_slots_calendar.end();
			     iter++, i++) {
				g2mplsTypes::JobSlotsCalendarEvent_var ev;

				ev->unixTime = (*iter).first;
				ev->JobSlots = (*iter).second;
				cal[i] = ev;
			}
		}

		dst->freeJobSlotsCalendar = cal;
	}

	return dst;
}

//
// gridCpuInfo
//
grid_cpu_info_t &
operator<<(grid_cpu_info_t &                      dst,
	   const g2mplsTypes::gridCpuInfo_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.count.physical = src->cpuCounts.physical;
	dst.count.logical  = src->cpuCounts.logical;

	switch (src->cpuArch) {
		case g2mplsTypes::GRIDCPUARCH_UNKNOWN: {
			dst.arch = CPU_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_SPARC: {
			dst.arch = CPU_SPARC;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_POWERPC: {
			dst.arch = CPU_POWERPC;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_X86: {
			dst.arch = CPU_X86;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_X86_32: {
			dst.arch = CPU_X86_32;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_X86_64: {
			dst.arch = CPU_X86_64;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_PARISC: {
			dst.arch = CPU_PARISC;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_MIPS: {
			dst.arch = CPU_MIPS;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_IA64: {
			dst.arch = CPU_IA64;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_ARM: {
			dst.arch = CPU_ARM;
		}
			break;
		case g2mplsTypes::GRIDCPUARCH_OTHER: {
			dst.arch = CPU_OTHER;
		}
			break;

		default: {
			throw out_of_range("gridCpuArch out-of-range");
		}
			break;
	}

	return dst;
}

g2mplsTypes::gridCpuInfo_var &
operator<<(g2mplsTypes::gridCpuInfo_var &         dst,
	   const grid_cpu_info_t &                src)
{
	dst = new g2mplsTypes::gridCpuInfo;

	dst->valid = true;

	dst->cpuCounts.physical = src.count.physical;
	dst->cpuCounts.logical  = src.count.logical;

	switch (src.arch) {
		case CPU_UNKNOWN: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_UNKNOWN;
		}
			break;
		case CPU_SPARC: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_SPARC;
		}
			break;
		case CPU_POWERPC: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_POWERPC;
		}
			break;
		case CPU_X86: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_X86;
		}
			break;
		case CPU_X86_32: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_X86_32;
		}
			break;
		case CPU_X86_64: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_X86_64;
		}
			break;
		case CPU_PARISC: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_PARISC;
		}
			break;
		case CPU_MIPS: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_MIPS;
		}
			break;
		case CPU_IA64: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_IA64;
		}
			break;
		case CPU_ARM: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_ARM;
		}
			break;
		case CPU_OTHER: {
			dst->cpuArch = g2mplsTypes::GRIDCPUARCH_OTHER;
		}
			break;

		default: {
			throw out_of_range("grid_cpu_arch_t out-of-range");
		}
			break;
	}

	return dst;
}

//
// gridOsInfo
//
grid_os_info_t &
operator<<(grid_os_info_t &                      dst,
	   const g2mplsTypes::gridOsInfo_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->type) {
		case g2mplsTypes::GRIDOSTYPE_UNKNOWN: {
			dst.type = OS_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_MACOS: {
			dst.type = OS_MACOS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_ATTUNIX: {
			dst.type = OS_ATTUNIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_DGUX: {
			dst.type = OS_DGUX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_DECNT: {
			dst.type = OS_DECNT;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_TRU64_UNIX: {
			dst.type = OS_TRU64_UNIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OPENVMS: {
			dst.type = OS_OPENVMS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_HPUX: {
			dst.type = OS_HPUX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_AIX: {
			dst.type = OS_AIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_MVS: {
			dst.type = OS_MVS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OS400: {
			dst.type = OS_OS400;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OS_2: {
			dst.type = OS_OS_2;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_JAVAVM: {
			dst.type = OS_JAVAVM;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_MSDOS: {
			dst.type = OS_MSDOS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WIN3X: {
			dst.type = OS_WIN3X;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WIN95: {
			dst.type = OS_WIN95;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WIN98: {
			dst.type = OS_WIN98;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WINNT: {
			dst.type = OS_WINNT;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WINCE: {
			dst.type = OS_WINCE;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_NCR3000: {
			dst.type = OS_NCR3000;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_NETWARE: {
			dst.type = OS_NETWARE;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OSF: {
			dst.type = OS_OSF;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_DC_OS: {
			dst.type = OS_DC_OS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_RELIANT_UNIX: {
			dst.type = OS_RELIANT_UNIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_SCO_UNIXWARE: {
			dst.type = OS_SCO_UNIXWARE;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_SCO_OPENSERVER: {
			dst.type = OS_SCO_OPENSERVER;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_SEQUENT: {
			dst.type = OS_SEQUENT;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_IRIX: {
			dst.type = OS_IRIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_SOLARIS: {
			dst.type = OS_SOLARIS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_SUNOS: {
			dst.type = OS_SUNOS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_U6000: {
			dst.type = OS_U6000;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_ASERIES: {
			dst.type = OS_ASERIES;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_TANDEMNSK: {
			dst.type = OS_TANDEMNSK  ;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_TANDEMNT: {
			dst.type = OS_TANDEMNT;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_BS2000: {
			dst.type = OS_BS2000;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_LINUX: {
			dst.type = OS_LINUX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_LYNX: {
			dst.type = OS_LYNX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_XENIX: {
			dst.type = OS_XENIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_VM: {
			dst.type = OS_VM;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_INTERACTIVE_UNIX: {
			dst.type = OS_INTERACTIVE_UNIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_BSDUNIX: {
			dst.type = OS_BSDUNIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_FREEBSD: {
			dst.type = OS_FREEBSD;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_NETBSD: {
			dst.type = OS_NETBSD;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_GNU_HURD: {
			dst.type = OS_GNU_HURD;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OS9: {
			dst.type = OS_OS9;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_MACH_KERNEL: {
			dst.type = OS_MACH_KERNEL;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_INFERNO: {
			dst.type = OS_INFERNO;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_QNX: {
			dst.type = OS_QNX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_EPOC: {
			dst.type = OS_EPOC;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_IXWORKS: {
			dst.type = OS_IXWORKS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_VXWORKS: {
			dst.type = OS_VXWORKS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_MINT: {
			dst.type = OS_MINT ;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_BEOS: {
			dst.type = OS_BEOS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_HP_MPE: {
			dst.type = OS_HP_MPE;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_NEXTSTEP: {
			dst.type = OS_NEXTSTEP;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_PALMPILOT: {
			dst.type = OS_PALMPILOT;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_RHAPSODY: {
			dst.type = OS_RHAPSODY;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WINDOWS_2000: {
			dst.type = OS_WINDOWS_2000;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_DEDICATED: {
			dst.type = OS_DEDICATED;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OS_390: {
			dst.type = OS_OS_390;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_VSE: {
			dst.type = OS_VSE;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_TPF: {
			dst.type = OS_TPF;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WINDOWS_R_ME: {
			dst.type = OS_WINDOWS_R_ME;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_CALDERA_OPEN_UNIX: {
			dst.type = OS_CALDERA_OPEN_UNIX;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OPENBSD: {
			dst.type = OS_OPENBSD;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_WINDOWS_XP: {
			dst.type = OS_WINDOWS_XP;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_Z_OS: {
			dst.type = OS_Z_OS;
		}
			break;
		case g2mplsTypes::GRIDOSTYPE_OTHER: {
			dst.type = OS_OTHER;
		}
			break;
		default: {
			throw out_of_range("gridOsType out-of-range");
		}
			break;
	}

	dst.version.major_revision = (uint16_t) (src->mjrRev & 0x0F);
	dst.version.minor_revision = (uint16_t) (src->mnrRev & 0x2F);
	dst.version.build_fix= (uint16_t) (src->bldFix & 0x2F);

	return dst;
}

g2mplsTypes::gridOsInfo_var &
operator<<(g2mplsTypes::gridOsInfo_var &dst,
	   const grid_os_info_t & src)
{
	dst = new g2mplsTypes::gridOsInfo;

	dst->valid = true;

	switch (src.type) {
		case OS_UNKNOWN: {
			dst->type = g2mplsTypes::GRIDOSTYPE_UNKNOWN;
		}
			break;
		case OS_MACOS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_MACOS;
		}
			break;
		case OS_ATTUNIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_ATTUNIX;
		}
			break;
		case OS_DGUX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_DGUX;
		}
			break;
		case OS_DECNT: {
			dst->type = g2mplsTypes::GRIDOSTYPE_DECNT;
		}
			break;
		case OS_TRU64_UNIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_TRU64_UNIX;
		}
			break;
		case OS_OPENVMS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OPENVMS;
		}
			break;
		case OS_HPUX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_HPUX;
		}
			break;
		case OS_AIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_AIX;
		}
			break;
		case OS_MVS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_MVS;
		}
			break;
		case OS_OS400: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OS400;
		}
			break;
		case OS_OS_2: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OS_2;
		}
			break;
		case OS_JAVAVM: {
			dst->type = g2mplsTypes::GRIDOSTYPE_JAVAVM;
		}
			break;
		case OS_MSDOS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_MSDOS;
		}
			break;
		case OS_WIN3X: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WIN3X;
		}
			break;
		case OS_WIN95: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WIN95;
		}
			break;
		case OS_WIN98: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WIN98;
		}
			break;
		case OS_WINNT: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WINNT;
		}
			break;
		case OS_WINCE: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WINCE;
		}
			break;
		case OS_NCR3000: {
			dst->type = g2mplsTypes::GRIDOSTYPE_NCR3000;
		}
			break;
		case OS_NETWARE: {
			dst->type = g2mplsTypes::GRIDOSTYPE_NETWARE;
		}
			break;
		case OS_OSF: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OSF;
		}
			break;
		case OS_DC_OS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_DC_OS;
		}
			break;
		case OS_RELIANT_UNIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_RELIANT_UNIX;
		}
			break;
		case OS_SCO_UNIXWARE: {
			dst->type = g2mplsTypes::GRIDOSTYPE_SCO_UNIXWARE;
		}
			break;
		case OS_SCO_OPENSERVER: {
			dst->type = g2mplsTypes::GRIDOSTYPE_SCO_OPENSERVER;
		}
			break;
		case OS_SEQUENT: {
			dst->type = g2mplsTypes::GRIDOSTYPE_SEQUENT;
		}
			break;
		case OS_IRIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_IRIX;
		}
			break;
		case OS_SOLARIS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_SOLARIS;
		}
			break;
		case OS_SUNOS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_SUNOS;
		}
			break;
		case OS_U6000: {
			dst->type = g2mplsTypes::GRIDOSTYPE_U6000;
		}
			break;
		case OS_ASERIES: {
			dst->type = g2mplsTypes::GRIDOSTYPE_ASERIES;
		}
			break;
		case OS_TANDEMNSK: {
			dst->type = g2mplsTypes::GRIDOSTYPE_TANDEMNSK  ;
		}
			break;
		case OS_TANDEMNT: {
			dst->type = g2mplsTypes::GRIDOSTYPE_TANDEMNT;
		}
			break;
		case OS_BS2000: {
			dst->type = g2mplsTypes::GRIDOSTYPE_BS2000;
		}
			break;
		case OS_LINUX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_LINUX;
		}
			break;
		case OS_LYNX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_LYNX;
		}
			break;
		case OS_XENIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_XENIX;
		}
			break;
		case OS_VM: {
			dst->type = g2mplsTypes::GRIDOSTYPE_VM;
		}
			break;
		case OS_INTERACTIVE_UNIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_INTERACTIVE_UNIX;
		}
			break;
		case OS_BSDUNIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_BSDUNIX;
		}
			break;
		case OS_FREEBSD: {
			dst->type = g2mplsTypes::GRIDOSTYPE_FREEBSD;
		}
			break;
		case OS_NETBSD: {
			dst->type = g2mplsTypes::GRIDOSTYPE_NETBSD;
		}
			break;
		case OS_GNU_HURD: {
			dst->type = g2mplsTypes::GRIDOSTYPE_GNU_HURD;
		}
			break;
		case OS_OS9: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OS9;
		}
			break;
		case OS_MACH_KERNEL: {
			dst->type = g2mplsTypes::GRIDOSTYPE_MACH_KERNEL;
		}
			break;
		case OS_INFERNO: {
			dst->type = g2mplsTypes::GRIDOSTYPE_INFERNO;
		}
			break;
		case OS_QNX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_QNX;
		}
			break;
		case OS_EPOC: {
			dst->type = g2mplsTypes::GRIDOSTYPE_EPOC;
		}
			break;
		case OS_IXWORKS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_IXWORKS;
		}
			break;
		case OS_VXWORKS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_VXWORKS;
		}
			break;
		case OS_MINT: {
			dst->type = g2mplsTypes::GRIDOSTYPE_MINT ;
		}
			break;
		case OS_BEOS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_BEOS;
		}
			break;
		case OS_HP_MPE: {
			dst->type = g2mplsTypes::GRIDOSTYPE_HP_MPE;
		}
			break;
		case OS_NEXTSTEP: {
			dst->type = g2mplsTypes::GRIDOSTYPE_NEXTSTEP;
		}
			break;
		case OS_PALMPILOT: {
			dst->type = g2mplsTypes::GRIDOSTYPE_PALMPILOT;
		}
			break;
		case OS_RHAPSODY: {
			dst->type = g2mplsTypes::GRIDOSTYPE_RHAPSODY;
		}
			break;
		case OS_WINDOWS_2000: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WINDOWS_2000;
		}
			break;
		case OS_DEDICATED: {
			dst->type = g2mplsTypes::GRIDOSTYPE_DEDICATED;
		}
			break;
		case OS_OS_390: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OS_390;
		}
			break;
		case OS_VSE: {
			dst->type = g2mplsTypes::GRIDOSTYPE_VSE;
		}
			break;
		case OS_TPF: {
			dst->type = g2mplsTypes::GRIDOSTYPE_TPF;
		}
			break;
		case OS_WINDOWS_R_ME: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WINDOWS_R_ME;
		}
			break;
		case OS_CALDERA_OPEN_UNIX: {
			dst->type = g2mplsTypes::GRIDOSTYPE_CALDERA_OPEN_UNIX;
		}
			break;
		case OS_OPENBSD: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OPENBSD;
		}
			break;
		case OS_WINDOWS_XP: {
			dst->type = g2mplsTypes::GRIDOSTYPE_WINDOWS_XP;
		}
			break;
		case OS_Z_OS: {
			dst->type = g2mplsTypes::GRIDOSTYPE_Z_OS;
		}
			break;
		case OS_OTHER: {
			dst->type = g2mplsTypes::GRIDOSTYPE_OTHER;
		}
			break;
		default: {
			throw out_of_range("grid_os_type_t out-of-range");
		}
			break;
	}

	dst->mjrRev = src.version.major_revision;
	dst->mnrRev = src.version.minor_revision;
	dst->bldFix = src.version.build_fix;

	return dst;
}

//
// gridMemoryInfo
//
grid_memory_info_t &
operator<<(grid_memory_info_t &                      dst,
	   const g2mplsTypes::gridMemoryInfo_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.ram_size            = src->ramSize;
	dst.virtual_memory_size = src->virtualMemorySize;

	return dst;
}

g2mplsTypes::gridMemoryInfo_var &
operator<<(g2mplsTypes::gridMemoryInfo_var &         dst,
	   const grid_memory_info_t &                src)
{
	dst = new g2mplsTypes::gridMemoryInfo;

	dst->valid = true;

	dst->ramSize           = src.ram_size;
	dst->virtualMemorySize = src.virtual_memory_size;

	return dst;
}

//
// gridApplication
//
grid_application_t &
operator<<(grid_application_t &                      dst,
	   const g2mplsTypes::gridApplication_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	switch (src->type) {
		case g2mplsTypes::GRIDAPPLICATIONTYPE_UNKNOWN: {
			dst.type = GRID_APPNAME_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDAPPLICATIONTYPE_WISDOM: {
			dst.type = GRID_APPNAME_WISDOM;
		}
			break;
		case g2mplsTypes::GRIDAPPLICATIONTYPE_KODAVIS: {
			dst.type = GRID_APPNAME_KODAVIS;
		}
			break;
		case g2mplsTypes::GRIDAPPLICATIONTYPE_TOPS: {
			dst.type = GRID_APPNAME_TOPS;
		}
			break;
		case g2mplsTypes::GRIDAPPLICATIONTYPE_DDSS: {
			dst.type = GRID_APPNAME_DDSS;
		}
			break;
		case g2mplsTypes::GRIDAPPLICATIONTYPE_INCA: {
			dst.type = GRID_APPNAME_INCA;
		}
			break;
		case g2mplsTypes::GRIDAPPLICATIONTYPE_OTHER: {
			dst.type = GRID_APPNAME_OTHER;
		}
			break;
		default: {
			throw out_of_range("gridApplicationType out-of-range");
		}
			break;
	}

	dst.version.major_revision = (uint16_t) (src->mjrRev & 0x0F);
	dst.version.minor_revision = (uint16_t) (src->mnrRev & 0x2F);
	dst.version.build_fix      = (uint16_t) (src->bldFix & 0x2F);

	return dst;
}

g2mplsTypes::gridApplication_var &
operator<<(g2mplsTypes::gridApplication_var &         dst,
	   const grid_application_t &                src)
{
	dst = new g2mplsTypes::gridApplication;

	dst->valid = true;

	switch (src.type) {
		case GRID_APPNAME_UNKNOWN: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_UNKNOWN;
		}
			break;
		case GRID_APPNAME_WISDOM: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_WISDOM;
		}
			break;
		case GRID_APPNAME_KODAVIS: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_KODAVIS;
		}
			break;
		case GRID_APPNAME_TOPS: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_TOPS;
		}
			break;
		case GRID_APPNAME_DDSS: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_DDSS;
		}
			break;
		case GRID_APPNAME_INCA: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_INCA;
		}
			break;
		case GRID_APPNAME_OTHER: {
			dst->type = g2mplsTypes::GRIDAPPLICATIONTYPE_OTHER;
		}
			break;
		default: {
			throw out_of_range("gridApplicationType out-of-range");
		}
			break;
	}

	dst->mjrRev = src.version.major_revision;
	dst->mnrRev = src.version.minor_revision;
	dst->bldFix = src.version.build_fix;

	return dst;
}

//
//gridSubClusterParams
//
grid_subcluster_data_t &
operator << (grid_subcluster_data_t &                      dst,
	     const g2mplsTypes::gridSubClusterParams_var & src)
{
	memset(&dst.mask_, 0, sizeof(dst.mask_));

	dst.name = new std::string(src->name);

	try {
		g2mplsTypes::gridCpuInfo_var tmp;
		tmp = src->cpu;
		dst.cpu << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "cpuInfo in "
				       "gridSubClusterParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridOsInfo_var tmp;
		tmp = src->os;
		dst.os << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "osInfo in "
				       "gridSubClusterParams_var "
				       "to convert");
	}

	try {
		g2mplsTypes::gridMemoryInfo_var tmp;
		tmp = src->memory;
		dst.memory << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "memoryInfo in "
				       "gridSubClusterParams_var "
				       "to convert");
	}

	size_t i;

	dst.software_packages.clear();
	for (i = 0; i < src->softwarePackages.length(); i++) {
		grid_software_package_t * tmp;

		tmp = new grid_software_package_t;
		if (!tmp) {
			throw invalid_argument("Out of memory for a "
					       "grid_software_package_t...");
		}

		try {
			g2mplsTypes::gridApplication_var tmp1;
			tmp1 = new g2mplsTypes::gridApplication;
			tmp1 = src->softwarePackages[i].software;
			tmp->software << tmp1;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "software in "
					       "gridSubClusterParams_var "
					       "to convert");
		}

		tmp->software_env_setup =
			new std::string(src->softwarePackages[i].
					softwareEnvironmentSetup);

		dst.software_packages.push_back(tmp);
	}

	dst.subcluster_calendar.clear();
	for (i = 0; i < src->subClusterCalendar.length(); i++) {
		uint32_t		unixTime;
		grid_cpu_count_t	cpu;

		unixTime     = src->subClusterCalendar[i].unixTime;
		cpu.physical = src->subClusterCalendar[i].cpuCount.physical;
		cpu.logical  = src->subClusterCalendar[i].cpuCount.logical;

		dst.subcluster_calendar[unixTime] = cpu;
	}

	BITMASK_BITSET(dst.mask_, name);
	BITMASK_BITSET(dst.mask_, cpu);
	BITMASK_BITSET(dst.mask_, os);
	BITMASK_BITSET(dst.mask_, memory);
	BITMASK_BITSET(dst.mask_, software_packages);
	BITMASK_BITSET(dst.mask_, subcluster_calendar);

	return dst;
}

g2mplsTypes::gridSubClusterParams_var &
operator << (g2mplsTypes::gridSubClusterParams_var & dst,
	     const grid_subcluster_data_t &          src)
{
	dst = new g2mplsTypes::gridSubClusterParams;

	dst->name = CORBA::string_dup("");

	dst->cpu.valid              = false;
	dst->cpu.cpuCounts.physical = 0;
	dst->cpu.cpuCounts.logical  = 0;
	dst->cpu.cpuArch            = g2mplsTypes::GRIDCPUARCH_UNKNOWN;

	dst->os.valid  = false;
	dst->os.type   = g2mplsTypes::GRIDOSTYPE_UNKNOWN;
	dst->os.mjrRev = 0;
	dst->os.mnrRev = 0;
	dst->os.bldFix = 0;

	dst->memory.valid             = false;
	dst->memory.ramSize           = 0;
	dst->memory.virtualMemorySize = 0;

	if (BITMASK_BITTEST(src.mask_, name) && src.name) {
		dst->name = CORBA::string_dup(src.name->c_str());
	}

	if (BITMASK_BITTEST(src.mask_, cpu)) {
		try {
			g2mplsTypes::gridCpuInfo_var tmp;
			tmp << src.cpu;
			dst->cpu = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "cpu_info in "
					       "grid_subcluster_data_t "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, os)) {
		try {
			g2mplsTypes::gridOsInfo_var tmp;
			tmp << src.os;
			dst->os = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "os_info in "
					       "grid_subcluster_data_t "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, memory)) {
		try {
			g2mplsTypes::gridMemoryInfo_var tmp;
			tmp << src.memory;
			dst->memory = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "memory_info in "
					       "grid_subcluster_data_t "
					       "to convert");
		}
	}

	// softwarePackages
	{
		g2mplsTypes::softwarePackageSeq_var swps;
		g2mplsTypes::softwarePackageSeq *   tmp;
		size_t                              size;

		size = 0;

		if (BITMASK_BITTEST(src.mask_, software_packages)) {
			size = src.software_packages.size();
		}

		tmp = new g2mplsTypes::softwarePackageSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::"
					   "softwarePackagesSeq");
		}

		swps = tmp;
		swps->length(size);

		if (BITMASK_BITTEST(src.mask_, software_packages) &&
		    size) {
			std::list<grid_software_package_t *>::
				const_iterator			iter;
			size_t					i;

			for (iter  = src.software_packages.begin(), i = 0;
			     iter != src.software_packages.end();
			     iter++, i++) {
				g2mplsTypes::gridSoftwarePackage_var swp;
				swp = new g2mplsTypes::gridSoftwarePackage;
				try {
					g2mplsTypes::gridApplication_var tmp1;
					tmp1 << (*iter)->software;
					swp->software = tmp1;
				} catch (...) {
					throw invalid_argument("Invalid or "
							       "malformed "
							       "software in "
							       "SwPackages "
							       "to convert");
				}

				if ((*iter)->software_env_setup) {
					swp->softwareEnvironmentSetup =
						CORBA::string_dup(((*iter)->software_env_setup)->c_str());
				}

				swps[i] = swp;
			}
		}

		dst->softwarePackages = swps;
	}

	// subClusterCalendar
	{
		g2mplsTypes::subClusterCalendarSeq_var cal;
		g2mplsTypes::subClusterCalendarSeq *   tmp;
		size_t                                 size;

		size = 0;

		if (BITMASK_BITTEST(src.mask_, subcluster_calendar)) {
			size = src.subcluster_calendar.size();
		}

		tmp = new g2mplsTypes::subClusterCalendarSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::"
					   "subClusterCalendarSeq");
		}

		cal = tmp;
		cal->length(size);

		if (BITMASK_BITTEST(src.mask_, subcluster_calendar) &&
		    size) {
			std::map<uint32_t,
				grid_cpu_count_t>::const_iterator iter;
			size_t                                       i;

			for (iter  = src.subcluster_calendar.begin(), i = 0;
			     iter != src.subcluster_calendar.end();
			     iter++, i++) {
				g2mplsTypes::subClusterCalendarEvent_var ev;
				ev = new g2mplsTypes::subClusterCalendarEvent;
				ev->unixTime = (*iter).first;
				ev->cpuCount.physical =
					(*iter).second.physical;
				ev->cpuCount.logical  =
					(*iter).second.logical;
				cal[i] = ev;
			}
		}

		dst->subClusterCalendar = cal;
	}

	return dst;
}

// gridStorageInfo
grid_storage_info_t &
operator<<(grid_storage_info_t &                      dst,
	   const g2mplsTypes::gridStorageInfo_var &   src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	switch (src->arch) {
		case g2mplsTypes::GRIDSTORAGEARCH_UNKNOWN: {
			dst.arch = STORAGE_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEARCH_DISK: {
			dst.arch = STORAGE_DISK;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEARCH_TAPE: {
			dst.arch = STORAGE_TAPE;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEARCH_MULTIDISK: {
			dst.arch = STORAGE_MULTIDISK;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEARCH_OTHER: {
			dst.arch = STORAGE_OTHER;}
			break;
		default: {
			throw out_of_range("gridStorageArch out-of-range");
		}
			break;
	}

	switch(src->state) {
		case g2mplsTypes::GRIDCESESTATE_UNKNOWN: {
			dst.state = CESE_STATE_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_QUEUING: {
			dst.state = CESE_STATE_QUEUING;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_PRODUCTION: {
			dst.state = CESE_STATE_PRODUCTION;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_CLOSED: {
			dst.state = CESE_STATE_CLOSED;
		}
			break;
		case g2mplsTypes::GRIDCESESTATE_DRAINING: {
			dst.state = CESE_STATE_DRAINING;
		}
			break;
		default: {
			throw out_of_range("gridCeSeState out-of-range");
		}
			break;
	}

	uint16_t ap1, cp1;
	grid_storage_access_protocol_t  ap;
	grid_storage_control_protocol_t cp;

	ap1 = (uint16_t)(src->accessProtocolsMask  & 0x0000FFFF);
	ap  = *((grid_storage_access_protocol_t *) &(ap1));
	cp1 = (uint16_t)(src->controlProtocolsMask & 0x0000FFFF);
	cp  = *((grid_storage_control_protocol_t *)&(cp1));

	dst.access_protocols  = ap;
	dst.control_protocols = cp;

	return dst;
}

g2mplsTypes::gridStorageInfo_var &
operator<<(g2mplsTypes::gridStorageInfo_var &         dst,
	   const grid_storage_info_t &                src)
{
	dst = new g2mplsTypes::gridStorageInfo;

	dst->valid = true;

	switch (src.arch) {
		case STORAGE_UNKNOWN: {
			dst->arch = g2mplsTypes::GRIDSTORAGEARCH_UNKNOWN;
		}
			break;
		case STORAGE_DISK: {
			dst->arch = g2mplsTypes::GRIDSTORAGEARCH_DISK;
		}
			break;
		case STORAGE_TAPE: {
			dst->arch = g2mplsTypes::GRIDSTORAGEARCH_TAPE;
		}
			break;
		case STORAGE_MULTIDISK: {
			dst->arch = g2mplsTypes::GRIDSTORAGEARCH_MULTIDISK;
		}
			break;
		case STORAGE_OTHER: {
			dst->arch = g2mplsTypes::GRIDSTORAGEARCH_OTHER;}
			break;
		default: {
			throw out_of_range("gridStorageArch out-of-range");
		}
			break;
	}

	switch(src.state) {
		case CESE_STATE_UNKNOWN: {
			dst->state = g2mplsTypes::GRIDCESESTATE_UNKNOWN;
		}
			break;
		case CESE_STATE_QUEUING: {
			dst->state = g2mplsTypes::GRIDCESESTATE_QUEUING;
		}
			break;
		case CESE_STATE_PRODUCTION: {
			dst->state = g2mplsTypes::GRIDCESESTATE_PRODUCTION;
		}
			break;
		case CESE_STATE_CLOSED: {
			dst->state = g2mplsTypes::GRIDCESESTATE_CLOSED;
		}
			break;
		case CESE_STATE_DRAINING: {
			dst->state = g2mplsTypes::GRIDCESESTATE_DRAINING;
		}
			break;
		default: {
			throw out_of_range("grid_cese_state_t out-of-range");
		}
			break;
	}

	uint32_t ap, cp;
	ap = *((uint16_t *) &(src.access_protocols));
	cp = *((uint16_t *) &(src.control_protocols));

	dst->accessProtocolsMask  = ap;
	dst->controlProtocolsMask = cp;

	return dst;
}

//
// gridStorageAreaInfo
//
grid_storage_area_info_t &
operator<<(grid_storage_area_info_t &                      dst,
	   const g2mplsTypes::gridStorageAreaInfo_var &    src)
{
	memset(&dst, 0, sizeof(dst));

	if (!src->valid) {
		return dst;
	}

	dst.total_online_size          = src->totalOnlineSize;
	dst.free_online_size           = src->freeOnlineSize;
	dst.reserved_total_online_size = src->reservedTotalOnlineSize;
	dst.total_nearline_size        = src->totalNearlineSize;
	dst.free_nearline_size         = src->freeNearlineSize;
	dst.reserved_nearline_size     = src->reservedNearlineSize;

	switch (src->retentionPolicy) {
		case g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_UNKNOWN: {
			dst.retention_policy =
				RETPOL_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_CUSTODIAL: {
			dst.retention_policy =
				RETPOL_CUSTODIAL;
		}
			break;
		case g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_OUTPUT: {
			dst.retention_policy =
				RETPOL_OUTPUT;
		}
			break;
		case g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_REPLICA: {
			dst.retention_policy =
				RETPOL_REPLICA;
		}
			break;
		default: {
			throw out_of_range("retentionPolicy out-of-range");
		}
			break;
	}

	switch (src->accessLatency) {
		case g2mplsTypes::GRIDSTORAGEACCESSLATENCY_UNKNOWN: {
			dst.access_latency =
				ACCLAT_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEACCESSLATENCY_ONLINE: {
			dst.access_latency =
				ACCLAT_ONLINE;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEACCESSLATENCY_NEARLINE: {
			dst.access_latency =
				ACCLAT_NEARLINE;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEACCESSLATENCY_OFFLINE: {
			dst.access_latency =
				ACCLAT_OFFLINE;
		}
			break;
		default: {
			throw out_of_range("accessLatency out-of-range");
		}
			break;
	}

	switch (src->expirationMode) {
		case g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_UNKNOWN: {
			dst.expiration_mode =
				EXP_UNKNOWN;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_NEVER_EXPIRE: {
			dst.expiration_mode =
				EXP_NEVER_EXPIRE;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_WARN_WHEN_EXPIRED:{
			dst.expiration_mode =
				EXP_WARN_WHEN_EXPIRED;
		}
			break;
		case g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_RELEASE_WHEN_EXPIRED: {
			dst.expiration_mode =
				EXP_RELEASE_WHEN_EXPIRED;
		}
			break;
		default: {
			throw out_of_range("expirationMode out-of-range");
		}
			break;
	}

	return dst;
}

g2mplsTypes::gridStorageAreaInfo_var &
operator<<(g2mplsTypes::gridStorageAreaInfo_var &         dst,
	   const grid_storage_area_info_t &               src)
{
	dst = new g2mplsTypes::gridStorageAreaInfo;

	dst->valid = true;

	dst->totalOnlineSize =
		src.total_online_size;
	dst->freeOnlineSize =
		src.free_online_size;
	dst->reservedTotalOnlineSize =
		src.reserved_total_online_size;
	dst->totalNearlineSize =
		src.total_nearline_size;
	dst->freeNearlineSize =
		src.free_nearline_size;
	dst->reservedNearlineSize =
		src.reserved_nearline_size;

	switch (src.retention_policy) {
		case RETPOL_UNKNOWN: {
			dst->retentionPolicy =
				g2mplsTypes::
				GRIDSTORAGERETENTIONPOLICY_UNKNOWN;
		}
			break;
		case RETPOL_CUSTODIAL: {
			dst->retentionPolicy =
				g2mplsTypes::
				GRIDSTORAGERETENTIONPOLICY_CUSTODIAL;
		}
			break;
		case RETPOL_OUTPUT: {
			dst->retentionPolicy =
				g2mplsTypes::
				GRIDSTORAGERETENTIONPOLICY_OUTPUT;
		}
			break;
		case RETPOL_REPLICA: {
			dst->retentionPolicy =
				g2mplsTypes::
				GRIDSTORAGERETENTIONPOLICY_REPLICA;
		}
			break;
		default: {
			throw out_of_range("retention_policy out-of-range");
		}
			break;
	}

	switch (src.access_latency) {
		case ACCLAT_UNKNOWN: {
			dst->accessLatency =
				g2mplsTypes::GRIDSTORAGEACCESSLATENCY_UNKNOWN;
		}
			break;
		case ACCLAT_ONLINE: {
			dst->accessLatency =
				g2mplsTypes::GRIDSTORAGEACCESSLATENCY_ONLINE;
		}
			break;
		case ACCLAT_NEARLINE: {
			dst->accessLatency =
				g2mplsTypes::GRIDSTORAGEACCESSLATENCY_NEARLINE;
		}
			break;
		case ACCLAT_OFFLINE: {
			dst->accessLatency =
				g2mplsTypes::GRIDSTORAGEACCESSLATENCY_OFFLINE;
		}
			break;
		default: {
			throw out_of_range("access_latency out-of-range");
		}
			break;
	}

	switch (src.expiration_mode) {
		case EXP_UNKNOWN: {
			dst->expirationMode =
				g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_UNKNOWN;
		}
			break;
		case EXP_NEVER_EXPIRE: {
			dst->expirationMode =
				g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_NEVER_EXPIRE;
		}
			break;
		case EXP_WARN_WHEN_EXPIRED:{
			dst->expirationMode =
				g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_WARN_WHEN_EXPIRED;
		}
			break;
		case EXP_RELEASE_WHEN_EXPIRED: {
			dst->expirationMode =
				g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_RELEASE_WHEN_EXPIRED;
		}
			break;
		default: {
			throw out_of_range("expirationMode out-of-range");
		}
			break;
	}

	return dst;
}

//
//gridSEParams
//
grid_se_data_t &
operator << (grid_se_data_t &                      dst,
	     const g2mplsTypes::gridSEParams_var & src)
{
	memset(&(dst.mask_), 0, sizeof(dst.mask_));

	dst.name = new std::string(src->name);

	try {
		g2mplsTypes::gridStorageInfo_var tmp;
		tmp = src->storageInfo;
		dst.storage_info << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "gridStorageInfo in "
				       "gridSEParams_var "
				       "to convert");
	}

	if (src->onlineSize.valid) {
		dst.online_size.total = src->onlineSize.total;
		dst.online_size.used  = src->onlineSize.used;
	}

	if (src->nearlineSize.valid) {
		dst.nearline_size.total = src->nearlineSize.total;
		dst.nearline_size.used  = src->nearlineSize.used;
	}

	size_t i;

	dst.storage_areas.clear();
	for (i = 0; i < src->storageAreas.length(); i++) {
		grid_storage_area_t * sc;

		sc = new grid_storage_area_t;
		if (!sc) {
			throw invalid_argument("Out of memory for a "
					       "grid_storage_area_t...");
		}

		memset(sc, 0, sizeof(*sc));

		sc->storage_area_name =
			new std::string(src->storageAreas[i].storageAreaName);

		sc->storage_area_path =
			new std::string(src->storageAreas[i].storageAreaPath);

		try {
			g2mplsTypes::gridStorageAreaInfo_var tmp;
			tmp = src->storageAreas[i].storageAreaInfo;
			sc->storage_area_info << tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "gridStorageAreaInfo element "
					       "in gridSEParams_var "
					       "to convert");
		}

		dst.storage_areas.push_back(sc);
	}

	dst.se_calendar.clear();
	for (i = 0; i < src->seCalendar.length(); i++) {
		uint32_t		unixTime;
		grid_storage_count_t	sc;

		unixTime = src->seCalendar[i].unixTime;
		sc.free_online_size =
			src->seCalendar[i].storageCount.freeOnlineSize;
		sc.logical_cpus =
			src->seCalendar[i].storageCount.logicalCpus;

		dst.se_calendar[unixTime] = sc;
	}

	BITMASK_BITSET(dst.mask_, name);
	BITMASK_BITSET(dst.mask_, storage_info);
	BITMASK_BITSET(dst.mask_, online_size);
	BITMASK_BITSET(dst.mask_, nearline_size);
	BITMASK_BITSET(dst.mask_, storage_areas);
	BITMASK_BITSET(dst.mask_, se_calendar);

	return dst;
}

g2mplsTypes::gridSEParams_var &
operator << (g2mplsTypes::gridSEParams_var & dst,
	     const grid_se_data_t &          src)
{
	dst = new g2mplsTypes::gridSEParams;

	dst->name = CORBA::string_dup("");

	dst->storageInfo.valid = false;
	dst->storageInfo.arch  = g2mplsTypes::GRIDSTORAGEARCH_UNKNOWN;
	dst->storageInfo.state = g2mplsTypes::GRIDCESESTATE_UNKNOWN;
	dst->storageInfo.accessProtocolsMask  = 0;
	dst->storageInfo.controlProtocolsMask = 0;

	dst->onlineSize.valid = false;
	dst->onlineSize.total = 0;
	dst->onlineSize.used  = 0;

	dst->nearlineSize.valid = false;
	dst->nearlineSize.total = 0;
	dst->nearlineSize.used  = 0;

	if (BITMASK_BITTEST(src.mask_, name) && src.name) {
		dst->name = CORBA::string_dup(src.name->c_str());
	}

	if (BITMASK_BITTEST(src.mask_, storage_info)) {
		try {
			g2mplsTypes::gridStorageInfo_var tmp;
			tmp << src.storage_info;
			dst->storageInfo = tmp;
		} catch (...) {
			throw invalid_argument("Invalid or malformed "
					       "grid_storage_info in "
					       "grid_se_data_t "
					       "to convert");
		}
	}

	if (BITMASK_BITTEST(src.mask_, online_size)) {
		dst->onlineSize.valid = true;
		dst->onlineSize.total = src.online_size.total;
		dst->onlineSize.used  = src.online_size.used;
	}

	if (BITMASK_BITTEST(src.mask_, nearline_size)) {
		dst->nearlineSize.valid = true;
		dst->nearlineSize.total = src.nearline_size.total;
		dst->nearlineSize.used  = src.nearline_size.used;
	}

	// StorageAreas
	{
		g2mplsTypes::StorageAreaSeq_var sas;
		g2mplsTypes::StorageAreaSeq *    tmp;
		size_t                           size;

		size = 0;

		if (BITMASK_BITTEST(src.mask_, storage_areas)) {
			size = src.storage_areas.size();
		}

		tmp = new g2mplsTypes::StorageAreaSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::StorageAreaSeq");
		}

		sas = tmp;
		sas->length(size);

		if (BITMASK_BITTEST(src.mask_, storage_areas) && size) {
			std::list<grid_storage_area_t *>::const_iterator iter;
			size_t						 i;

			for (iter  = src.storage_areas.begin(), i = 0;
			     iter != src.storage_areas.end();
			     iter++, i++) {
				g2mplsTypes::gridStorageArea_var sa;
				sa = new g2mplsTypes::gridStorageArea;
				try {
					g2mplsTypes::gridStorageAreaInfo_var tmp1;
					tmp1 << (*iter)->storage_area_info;
					sa->storageAreaInfo = tmp1;
				} catch (...) {
					throw invalid_argument("Invalid or "
							       "malformed "
							       "storage_area_info in "
							       "storage_areas "
							       "to convert");
				}

				if ((*iter)->storage_area_name) {
					sa->storageAreaName =
						CORBA::string_dup(((*iter)->storage_area_name)->c_str());
				}

				if ((*iter)->storage_area_path) {
					sa->storageAreaPath =
						CORBA::string_dup(((*iter)->storage_area_path)->c_str());
				}

				sas[i] = sa;
			}
		}

		dst->storageAreas = sas;
	}

	// seCalendar
	{
		g2mplsTypes::seCalendarSeq_var cal;
		g2mplsTypes::seCalendarSeq *   tmp;
		size_t                         size;

		size = 0;

		if (BITMASK_BITTEST(src.mask_, se_calendar)) {
			size = src.se_calendar.size();
		}

		tmp = new g2mplsTypes::seCalendarSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::"
					   "seCalendarSeq");
		}

		cal = tmp;
		cal->length(size);

		if (BITMASK_BITTEST(src.mask_, se_calendar) &&
		    size) {
			std::map<uint32_t,
				grid_storage_count_t>::const_iterator iter;
			size_t                                        i;

			for (iter  = src.se_calendar.begin(), i = 0;
			     iter != src.se_calendar.end();
			     iter++, i++) {
				g2mplsTypes::seCalendarEvent_var ev;

				ev->unixTime = (*iter).first;
				ev->storageCount.freeOnlineSize =
					(*iter).second.free_online_size;
				ev->storageCount.logicalCpus =
					(*iter).second.logical_cpus;
				cal[i] = ev;
			}
		}

		dst->seCalendar = cal;
	}

	return dst;
}

//
// teLinkComParams
//
telink_com_data_t &
operator << (telink_com_data_t &                      dst,
	     const g2mplsTypes::teLinkComParams_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.admin_metric      = src->adminMetric;
	dst.te_metric         = src->teMetric;
	dst.te_color_mask     = src->teColorMask;
	dst.te_prot_type_mask = src->teProtectionTypeMask;
	dst.te_max_bw         = src->teMaxBw;
	dst.te_max_resv_bw    = src->teMaxResvBw;

	BITMASK_BITSET(dst.mask_, admin_metric);
	BITMASK_BITSET(dst.mask_, te_metric);
	BITMASK_BITSET(dst.mask_, te_color_mask);
	BITMASK_BITSET(dst.mask_, te_prot_type_mask);
	BITMASK_BITSET(dst.mask_, te_max_bw);
	BITMASK_BITSET(dst.mask_, te_max_resv_bw);

	return dst;
}

g2mplsTypes::teLinkComParams_var &
operator << (g2mplsTypes::teLinkComParams_var &       dst,
	     const telink_com_data_t &                  src)
{
	dst = new g2mplsTypes::teLinkComParams;

	dst->adminMetric          = 0;
	dst->teMetric             = 0;
	dst->teColorMask          = 0;
	dst->teProtectionTypeMask = 0;
	dst->teMaxBw              = 0;
	dst->teMaxResvBw          = 0;

	if (BITMASK_BITTEST(src.mask_, admin_metric)) {
		dst->adminMetric = src.admin_metric;
	}

	if (BITMASK_BITTEST(src.mask_, te_metric)) {
		dst->teMetric = src.te_metric;
	}

	if (BITMASK_BITTEST(src.mask_, te_color_mask)) {
		dst->teColorMask = src.te_color_mask;
	}

	if (BITMASK_BITTEST(src.mask_, te_prot_type_mask)) {
		dst->teProtectionTypeMask = src.te_prot_type_mask;
	}

	if (BITMASK_BITTEST(src.mask_, te_max_bw)) {
		dst->teMaxBw = src.te_max_bw;
	}
	if (BITMASK_BITTEST(src.mask_, te_max_resv_bw)) {
		dst->teMaxResvBw = src.te_max_resv_bw;
	}

	return dst;
}

// teLinkTdmParams
telink_tdm_data &
operator << (telink_tdm_data_t &                      dst,
	     const g2mplsTypes::teLinkTdmParams_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.min_lsp_bw           = 0; // fixed by ISC
	dst.stdarb_cc_indication = GMPLS_SDH_StdArbCap_Unknown; // fixed by ISC

	dst.ho_muxcap_mask       = src->hoMuxCapMask;
	dst.lo_muxcap_mask       = src->loMuxCapMask;
	dst.transparency_mask    = src->transparencyMask;
	dst.blsr_ring_id         = src->blsrRingId;

	BITMASK_BITRESET(dst.mask_,min_lsp_bw);
	BITMASK_BITRESET(dst.mask_,stdarb_cc_indication);
	BITMASK_BITSET(dst.mask_,ho_muxcap_mask);
	BITMASK_BITSET(dst.mask_,lo_muxcap_mask);
	BITMASK_BITSET(dst.mask_,transparency_mask);
	BITMASK_BITSET(dst.mask_,blsr_ring_id);

	return dst;
}

g2mplsTypes::teLinkTdmParams_var &
operator << (g2mplsTypes::teLinkTdmParams_var &       dst,
	     const telink_tdm_data_t &                src)
{
	dst = new g2mplsTypes::teLinkTdmParams;

	dst->hoMuxCapMask     = 0;
	dst->loMuxCapMask     = 0;
	dst->transparencyMask = 0;
	dst->blsrRingId       = 0;

	if (BITMASK_BITTEST(src.mask_,ho_muxcap_mask)) {
		dst->hoMuxCapMask = src.ho_muxcap_mask;
	}
	if (BITMASK_BITTEST(src.mask_,lo_muxcap_mask)) {
		dst->loMuxCapMask = src.lo_muxcap_mask;
	}
	if (BITMASK_BITTEST(src.mask_,transparency_mask)) {
		dst->transparencyMask = src.transparency_mask;
	}
	if (BITMASK_BITTEST(src.mask_,blsr_ring_id)) {
		dst->blsrRingId = src.blsr_ring_id;
	}

	return dst;
}

//
// teLinkLscG709Params
//
telink_lscg709_data &
operator << (telink_lscg709_data_t &                      dst,
	     const g2mplsTypes::teLinkLscG709Params_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.oduk_muxcap_mask = src->odukMuxCapMask;

	BITMASK_BITSET(dst.mask_, oduk_muxcap_mask);

	return dst;
}

g2mplsTypes::teLinkLscG709Params_var &
operator << (g2mplsTypes::teLinkLscG709Params_var &       dst,
	     const telink_lscg709_data_t &                src)
{
	dst = new g2mplsTypes::teLinkLscG709Params;

	dst->odukMuxCapMask = 0;

	if (BITMASK_BITTEST(src.mask_, oduk_muxcap_mask)) {
		dst->odukMuxCapMask = src.oduk_muxcap_mask;
	}

	return dst;
}

// teLinkLscWdmParams
telink_lscwdm_data &
operator << (telink_lscwdm_data_t &                      dst,
	     const g2mplsTypes::teLinkLscWdmParams_var & src)
{
	memset(&dst.mask_, 0, sizeof(dst.mask_));

	dst.dispersion_PMD = src->dispersionPMD;
	dst.span_length    = src->spanLength;

	dst.amplifiers.clear();
	size_t i;

	for (i = 0; i < src->amplifiers.length(); i++) {
		wdm_amplifier_data_t tmp;

		tmp.gain         = src->amplifiers[i].gain;
		tmp.noise_figure = src->amplifiers[i].noiseFigure;

		dst.amplifiers.push_back(tmp);
	}

	BITMASK_BITSET(dst.mask_,dispersion_PMD);
	BITMASK_BITSET(dst.mask_,span_length);
	BITMASK_BITSET(dst.mask_,amplifiers);

	return dst;
}

g2mplsTypes::teLinkLscWdmParams_var &
operator << (g2mplsTypes::teLinkLscWdmParams_var & dst,
	     const telink_lscwdm_data_t &          src)
{
	dst = new g2mplsTypes::teLinkLscWdmParams;

	dst->dispersionPMD = 0;
	dst->spanLength    = 0;

	if (BITMASK_BITTEST(src.mask_,dispersion_PMD)) {
		dst->dispersionPMD = src.dispersion_PMD;
	}
	if (BITMASK_BITTEST(src.mask_,span_length)) {
		dst->spanLength = src.span_length;
	}

	g2mplsTypes::amplifiersSeq_var amplifiers;
	g2mplsTypes::amplifiersSeq *   tmp;
	size_t                         size;

	size = 0;

	if (BITMASK_BITTEST(src.mask_, amplifiers)) {
		size = src.amplifiers.size();
	}

	tmp = new g2mplsTypes::amplifiersSeq(size);
	if (!tmp) {
		throw out_of_range("Cannot allocate "
				   "g2mplsTypes::amplifiersSeq");
	}
	amplifiers = tmp;
	amplifiers->length(size);

	if (BITMASK_BITTEST(src.mask_, amplifiers) && size) {
		std::list<wdm_amplifier_data_t>::const_iterator iter;
		size_t                                          i;

		for (iter = src.amplifiers.begin(), i = 0;
		     iter != src.amplifiers.end();
		     iter++, i++) {

			g2mplsTypes::teLinkWdmAmplifierEntry_var  amp;
			amp->gain        = (*iter).gain;
			amp->noiseFigure = (*iter).noise_figure;

			amplifiers[i] = amp;
		}
	}

	dst->amplifiers = amplifiers;

	return dst;
}


// isc
isc_t &
operator << (isc_t &                      dst,
	     const g2mplsTypes::isc_var & src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src->_d()) {
		case g2mplsTypes::SWITCHINGCAP_PSC_1:
		case g2mplsTypes::SWITCHINGCAP_PSC_2:
		case g2mplsTypes::SWITCHINGCAP_PSC_3:
		case g2mplsTypes::SWITCHINGCAP_PSC_4: {
			g2mplsTypes::iscParamsPsc_var tmp;
			tmp = src->psc();

			try {
				dst.switching << tmp->swCap;
			} catch (...) {
				throw out_of_range("switchingCap "
						   "out-of-range");
			}
			try {
				dst.encoding << tmp->encType;
			} catch (...) {
				throw out_of_range("encodingType "
						   "out-of-range");
			}

			size_t i;
			for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
				dst.max_lsp_bw.bw[i] = tmp->maxLSPbandwidth[i];
			}

			dst.spec.psc.min_lsp_bw = tmp->minLSPbandwidth;
			dst.spec.psc.if_mtu     = tmp->interfaceMTU;
		}
			break;
		case g2mplsTypes::SWITCHINGCAP_TDM  : {
			g2mplsTypes::iscParamsTdm_var tmp;
			tmp = src->tdm();

			try {
				dst.switching << tmp->swCap;
			} catch (...) {
				throw out_of_range("switchingCap "
						   "out-of-range");
			}
			try {
				dst.encoding << tmp->encType;
			} catch (...) {
				throw out_of_range("encodingType "
						   "out-of-range");
			}

			size_t i;
			for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
				dst.max_lsp_bw.bw[i] = tmp->maxLSPbandwidth[i];
			}

			dst.spec.tdm.min_lsp_bw = tmp->minLSPbandwidth;
			dst.spec.tdm.indication = tmp->indication;
		}
			break;
		case g2mplsTypes::SWITCHINGCAP_L2SC :
		case g2mplsTypes::SWITCHINGCAP_LSC  :
		case g2mplsTypes::SWITCHINGCAP_FSC  : {
			g2mplsTypes::iscParamsGen_var tmp;
			tmp = src->gen();

			try {
				dst.switching << tmp->swCap;
			} catch (...) {
				throw out_of_range("switchingCap "
						   "out-of-range");
			}
			try {
				dst.encoding << tmp->encType;
			} catch (...) {
				throw out_of_range("encodingType "
						   "out-of-range");
			}

			size_t i;
			for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
				dst.max_lsp_bw.bw[i] = tmp->maxLSPbandwidth[i];
			}
		}
			break;
		default: {
			throw out_of_range("isc type out-of-range");
		}
			break;

	}

	return dst;
}

g2mplsTypes::isc_var &
operator << (g2mplsTypes::isc_var & dst,
	     const isc_t &          src)
{
	dst = new g2mplsTypes::isc;

	switch (src.switching) {
		case SWCAP_PSC_1:
		case SWCAP_PSC_2:
		case SWCAP_PSC_3:
		case SWCAP_PSC_4: {
			g2mplsTypes::iscParamsPsc_var tmp;
			tmp = new g2mplsTypes::iscParamsPsc;

			try {
				tmp->swCap << src.switching;
			} catch (...) {
				throw out_of_range("sw_cap_t "
						   "out-of-range");
			}
			try {
				tmp->encType << src.encoding;
			} catch (...) {
				throw out_of_range("enc_type_t "
						   "out-of-range");
			}

			size_t i;
			for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
				tmp->maxLSPbandwidth[i] = src.max_lsp_bw.bw[i];
			}

			tmp->minLSPbandwidth = src.spec.psc.min_lsp_bw;
			tmp->interfaceMTU    = src.spec.psc.if_mtu;

			dst->psc(tmp._retn());
		}
			break;
		case SWCAP_TDM: {
			g2mplsTypes::iscParamsTdm_var tmp;
			tmp = new g2mplsTypes::iscParamsTdm;

			try {
				tmp->swCap << src.switching;
			} catch (...) {
				throw out_of_range("sw_cap_t "
						   "out-of-range");
			}
			try {
				tmp->encType << src.encoding;
			} catch (...) {
				throw out_of_range("enc_type_t "
						   "out-of-range");
			}

			size_t i;
			for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
				tmp->maxLSPbandwidth[i] = src.max_lsp_bw.bw[i];
			}

			tmp->minLSPbandwidth = src.spec.tdm.min_lsp_bw;
			tmp->indication      = src.spec.tdm.indication;

			dst->tdm(tmp._retn());
		}
			break;
		case SWCAP_L2SC:
		case SWCAP_LSC:
		case SWCAP_FSC: {
			g2mplsTypes::iscParamsGen_var tmp;
			tmp = new g2mplsTypes::iscParamsGen;

			try {
				tmp->swCap << src.switching;
			} catch (...) {
				throw out_of_range("sw_cap_t "
						   "out-of-range");
			}
			try {
				tmp->encType << src.encoding;
			} catch (...) {
				throw out_of_range("enc_type_t "
						   "out-of-range");
			}

			size_t i;
			for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
				tmp->maxLSPbandwidth[i] = src.max_lsp_bw.bw[i];
			}

			dst->gen(tmp._retn());
		}
			break;
		default: {
			throw out_of_range("sw_cap_t type out-of-range");
		}
			break;
	}

	return dst;
}

//
// eroItem
//
ero_hop_t &
operator << (ero_hop_t &                      dst,
	     const g2mplsTypes::eroItem_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.node = src->node;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp = src->teLink;
		dst.te_link << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "teLink "
				       "in eroItem_var");
	}
	try {
		g2mplsTypes::DLinkId_var tmp;
		tmp = src->upstreamDataLink;
		dst.upstream_datalink << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "upstreamDataLink "
				       "in eroItem_var");
	}
	try {
		g2mplsTypes::DLinkId_var tmp;
		tmp = src->downstreamDataLink;
		dst.downstream_datalink << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "downstreamDataLink "
				       "in eroItem_var");
	}
	try {
		g2mplsTypes::labelId_var tmp;
		tmp = src->upstreamLabel;
		dst.upstream_label << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "upstreamLabel "
				       "in eroItem_var");
	}
	try {
		g2mplsTypes::labelId_var tmp;
		tmp = src->downstreamLabel;
		dst.downstream_label << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "downstreamLabel "
				       "in eroItem_var");
	}

	dst.loose = src->loose ? 1 : 0;

	return dst;
}

g2mplsTypes::eroItem_var &
operator << (g2mplsTypes::eroItem_var &       dst,
	     const ero_hop_t &                src)
{
	dst = new g2mplsTypes::eroItem;

	dst->node = src.node;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp << src.te_link;
		dst->teLink = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "te_link "
				       "in ero_hop_t");
	}
	try {
		g2mplsTypes::DLinkId_var tmp;
		tmp << src.upstream_datalink;
		dst->upstreamDataLink = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "upstream_datalink "
				       "in ero_hop_t");
	}
	try {
		g2mplsTypes::DLinkId_var tmp;
		tmp << src.downstream_datalink;
		dst->downstreamDataLink = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "downstream_datalink "
				       "in ero_hop_t");
	}
	try {
		g2mplsTypes::labelId_var tmp;
		tmp << src.upstream_label;
		dst->upstreamLabel = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "upstream_label "
				       "in ero_hop_t");
	}
	try {
		g2mplsTypes::labelId_var tmp;
		tmp << src.downstream_label;
		dst->downstreamLabel = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "downstream_label "
				       "in ero_hop_t");
	}

	dst->loose = (src.loose == 1) ? true : false;

	return dst;
}

//
// connEndPoint
//
ero_hop_t &
operator << (ero_hop_t &                           dst,
	     const g2mplsTypes::connEndPoint_var & src)
{
	memset(&dst, 0, sizeof(dst));

	dst.node = src->node;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp = src->teLink;
		dst.te_link << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "teLink "
				       "in connEndPoint_var");
	}
	try {
		g2mplsTypes::DLinkId_var tmp;
		tmp = src->dataLink;
		dst.downstream_datalink << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "dataLink "
				       "in connEndPoint_var");
	}
	try {
		g2mplsTypes::labelId_var tmp;
		tmp = src->label;
		dst.downstream_label << tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "label "
				       "in connEndPoint_var");
	}

	return dst;
}

g2mplsTypes::connEndPoint_var &
operator << (g2mplsTypes::connEndPoint_var &       dst,
	     const ero_hop_t &                     src)
{
	dst = new g2mplsTypes::connEndPoint;

	dst->node = src.node;

	try {
		g2mplsTypes::TELinkId_var tmp;
		tmp << src.te_link;
		dst->teLink = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "te_link "
				       "in ero_hop_t");
	}
	try {
		g2mplsTypes::DLinkId_var tmp;
		tmp << src.downstream_datalink;
		dst->dataLink = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "downstream_datalink "
				       "in ero_hop_t");
	}
	try {
		g2mplsTypes::labelId_var tmp;
		tmp << src.downstream_label;
		dst->label = tmp;
	} catch (...) {
		throw invalid_argument("Invalid or malformed "
				       "downstream_label "
				       "in ero_hop_t");
	}

	if (src.loose == 1) {
		throw invalid_argument("Invalid loose "
				       "ero_hop_t to convert to a "
				       "connEndPoint");
	}

	return dst;
}

//
// wdmLambdasBitmap
//
wdm_link_lambdas_bitmap_t &
operator<<(wdm_link_lambdas_bitmap_t &           dst,
	   const g2mplsTypes::wdmLambdasBitmap & src)
{
	size_t    size;
	uint8_t * mask;

	memset(&dst, 0, sizeof(dst));

	dst.base_lambda_label = src.baseLambda;
	dst.num_wavelengths   = src.numLambdas;

	dst.bitmap_size = (src.numLambdas % 32 ? (src.numLambdas / 32) + 1 :
			   src.numLambdas / 32);

	size = (size_t) (dst.bitmap_size * 4);
	dst.bitmap_word = (uint32_t *) malloc(size);

	mask = (uint8_t *) dst.bitmap_word;

	for (size_t i = 0; i < src.bitmap.length(); i++) {
		*mask = src.bitmap[i];
		mask++;
	}

	return dst;
}

g2mplsTypes::wdmLambdasBitmap_var &
operator<<(g2mplsTypes::wdmLambdasBitmap_var & dst,
	   const wdm_link_lambdas_bitmap_t &   src)
{
	size_t    size;
	uint8_t * byte;

	dst = new g2mplsTypes::wdmLambdasBitmap;

	dst->baseLambda = src.base_lambda_label;
	dst->numLambdas = src.num_wavelengths;

	size = src.bitmap_size * 4;

	g2mplsTypes::bitmapSeq_var lambdaBitmap;
	{
		g2mplsTypes::bitmapSeq * tmp;
		tmp = new g2mplsTypes::bitmapSeq(size);
		if (!tmp) {
			throw out_of_range("Cannot allocate "
					   "g2mplsTypes::bitmapSeq");
		}
		lambdaBitmap = tmp;
	}
	lambdaBitmap->length(size);

	byte = (uint8_t *) src.bitmap_word;
	if (byte == NULL) {
		dst->bitmap = lambdaBitmap;
		return dst;
	}

	for (size_t i = 0; i < size; i++) {
		lambdaBitmap[i] = *byte;
		byte++;
	}
	dst->bitmap = lambdaBitmap;

	return dst;
}

#define RANGE_COPY(DST, SRC)			\
	(DST).lower_bound  = (SRC).lowerBound;	\
	(DST).upper_bound  = (SRC).upperBound;	\
	(DST).lb_included  = (SRC).lbIncluded;	\
	(DST).ub_included  = (SRC).ubIncluded;

#define RANGE_CONVERT(DST,CNAME,SRC,GNAME)		\
	if (SRC.GNAME.valid) {				\
		BITMASK_BITSET(DST.mask_, CNAME);	\
		RANGE_COPY(DST.CNAME, SRC.GNAME);	\
	}

//
// gridParams
//
grid_res_spec_t &
operator<<(grid_res_spec_t &               dst,
	   const g2mplsTypes::gridParams & src)
{
	memset(&dst, 0, sizeof(dst));

	if (src.application.valid) {
		BITMASK_BITSET(dst.mask_, application);
	}
	try {
		g2mplsTypes::gridApplication_var tmp1;

		tmp1 = src.application;
		dst.application << tmp1;
	} catch (...) {
		throw invalid_argument("Invalid or malformed application in "
				       "gridParams to convert");
	}

	if (src.candHost._d() != g2mplsTypes::GRIDHOSTTYPE_UNDEFINED) {
		BITMASK_BITSET(dst.mask_, cand_host);
	        try {
		        g2mplsTypes::gridHostId_var tmp;
		        tmp = src.candHost;
		        dst.cand_host << tmp;
	        } catch (...) {
		        throw invalid_argument("Invalid or malformed candHost in "
				               "gridParams to convert");
	        }
        }

	if (src.fileSystemRes.valid) {
		BITMASK_BITSET(dst.mask_, fs_resources);

		switch (src.fileSystemRes.fsName) {
			case g2mplsTypes::GRIDFSNAME_UNKNOWN:
				dst.fs_resources.fs_name = FS_NAME_UNKNOWN;
				break;
			case g2mplsTypes::GRIDFSNAME_HOME:
				dst.fs_resources.fs_name = FS_NAME_HOME;
				break;
			case g2mplsTypes::GRIDFSNAME_ROOT:
				dst.fs_resources.fs_name = FS_NAME_ROOT;
				break;
			case g2mplsTypes::GRIDFSNAME_SCRATCH:
				dst.fs_resources.fs_name = FS_NAME_SCRATCH;
				break;
			case g2mplsTypes::GRIDFSNAME_TMP:
				dst.fs_resources.fs_name = FS_NAME_TMP;
				break;
			case g2mplsTypes::GRIDFSNAME_OTHER:
				dst.fs_resources.fs_name = FS_NAME_OTHER;
				break;
			default:
				throw out_of_range("GRID FS NAME "
						   "out-of-range");
				break;
		}

		switch (src.fileSystemRes.fsType) {
			case g2mplsTypes::GRIDFSTYPE_UNKNOWN:
				dst.fs_resources.fs_type = FS_TYPE_UNKNOWN;
				break;
			case g2mplsTypes::GRIDFSTYPE_SWAP:
				dst.fs_resources.fs_type = FS_TYPE_SWAP;
				break;
			case g2mplsTypes::GRIDFSTYPE_TEMPORARY:
				dst.fs_resources.fs_type = FS_TYPE_TEMPORARY;
				break;
			case g2mplsTypes::GRIDFSTYPE_SPOOL:
				dst.fs_resources.fs_type = FS_TYPE_SPOOL;
				break;
			case g2mplsTypes::GRIDFSTYPE_NORMAL:
				dst.fs_resources.fs_type = FS_TYPE_NORMAL;
				break;
			case g2mplsTypes::GRIDFSTYPE_OTHER:
				dst.fs_resources.fs_type = FS_TYPE_OTHER;
				break;
			default:
				throw out_of_range("GRID FS TYPE "
						   "out-of-range");
				break;
		}

		RANGE_COPY(dst.fs_resources.disk_space,
			   src.fileSystemRes.diskSpace);

		dst.fs_resources.mount_point  =
			new std::string(src.fileSystemRes.mountPoint);
		dst.fs_resources.mount_source =
			new std::string(src.fileSystemRes.mountSource);
	}

	if (src.systemCaps.valid) {
		BITMASK_BITSET(dst.mask_, sys_caps);

		switch (src.systemCaps.cpuArch) {
			case g2mplsTypes::GRIDCPUARCH_UNKNOWN:
				dst.sys_caps.cpu_arch = CPU_UNKNOWN;
				break;
			case g2mplsTypes::GRIDCPUARCH_SPARC:
				dst.sys_caps.cpu_arch = CPU_SPARC;
				break;
			case g2mplsTypes::GRIDCPUARCH_POWERPC:
				dst.sys_caps.cpu_arch = CPU_POWERPC;
				break;
			case g2mplsTypes::GRIDCPUARCH_X86:
				dst.sys_caps.cpu_arch = CPU_X86;
				break;
			case g2mplsTypes::GRIDCPUARCH_X86_32:
				dst.sys_caps.cpu_arch = CPU_X86_32;
				break;
			case g2mplsTypes::GRIDCPUARCH_X86_64:
				dst.sys_caps.cpu_arch = CPU_X86_64;
				break;
			case g2mplsTypes::GRIDCPUARCH_PARISC:
				dst.sys_caps.cpu_arch = CPU_PARISC;
				break;
			case g2mplsTypes::GRIDCPUARCH_MIPS:
				dst.sys_caps.cpu_arch = CPU_MIPS;
				break;
			case g2mplsTypes::GRIDCPUARCH_IA64:
				dst.sys_caps.cpu_arch = CPU_IA64;
				break;
			case g2mplsTypes::GRIDCPUARCH_ARM:
				dst.sys_caps.cpu_arch = CPU_ARM;
				break;
			case g2mplsTypes::GRIDCPUARCH_OTHER:
				dst.sys_caps.cpu_arch = CPU_OTHER;
				break;
			default:
				throw out_of_range("GRID CPU ARCH "
						   "out-of-range");
				break;
		}

		dst.sys_caps.exclusive_access =
			src.systemCaps.exclusiveAccess ? 1 : 0;
	}
	try {
		g2mplsTypes::gridOsInfo_var tmp;
		grid_os_info_t tmp1;

		tmp = src.systemCaps.os;
		tmp1 << tmp;

		dst.sys_caps.os_type    = tmp1.type;
		dst.sys_caps.os_version = tmp1.version;

	} catch (...) {
		throw invalid_argument("Invalid or malformed osInfo in "
				       "gridParams to convert");
	}

	RANGE_CONVERT(dst, ind_cpu_speed , src, indCpuSpeed );
	RANGE_CONVERT(dst, ind_cpu_time  , src, indCpuTime  );
	RANGE_CONVERT(dst, ind_cpu_count , src, indCpuCount );
	RANGE_CONVERT(dst, ind_net_bw    , src, indNetBw    );
	RANGE_CONVERT(dst, ind_phy_mem   , src, indPhyMem   );
	RANGE_CONVERT(dst, ind_vir_mem   , src, indVirMem   );
	RANGE_CONVERT(dst, ind_disk_space, src, indDiskSpace);
	RANGE_CONVERT(dst, tot_cpu_time  , src, totCpuTime  );
	RANGE_CONVERT(dst, tot_cpu_count , src, totCpuCount );
	RANGE_CONVERT(dst, tot_phy_mem   , src, totPhyMem   );
	RANGE_CONVERT(dst, tot_vir_mem   , src, totVirMem   );
	RANGE_CONVERT(dst, tot_disk_space, src, totDiskSpace);
	RANGE_CONVERT(dst, tot_res_count , src, totResCount );

	// Grid Data Staging
	if (src.dataStaging.valid) {
		BITMASK_BITSET(dst.mask_, data_staging);

		switch (src.dataStaging.fsName) {
			case g2mplsTypes::GRIDFSNAME_UNKNOWN:
				dst.data_staging.fs_name = FS_NAME_UNKNOWN;
				break;
			case g2mplsTypes::GRIDFSNAME_HOME:
				dst.data_staging.fs_name = FS_NAME_HOME;
				break;
			case g2mplsTypes::GRIDFSNAME_ROOT:
				dst.data_staging.fs_name = FS_NAME_ROOT;
				break;
			case g2mplsTypes::GRIDFSNAME_SCRATCH:
				dst.data_staging.fs_name = FS_NAME_SCRATCH;
				break;
			case g2mplsTypes::GRIDFSNAME_TMP:
				dst.data_staging.fs_name = FS_NAME_TMP;
				break;
			case g2mplsTypes::GRIDFSNAME_OTHER:
				dst.data_staging.fs_name = FS_NAME_OTHER;
				break;
			default:
				throw out_of_range("GRID DS FS NAME "
						   "out-of-range");
				break;
		}

		switch (src.dataStaging.cf) {
			case g2mplsTypes::GRIDSTAGINGCF_UNKNOWN:
				dst.data_staging.creation_flag = CF_UNKNOWN;
				break;
			case g2mplsTypes::GRIDSTAGINGCF_OVERWRITE:
				dst.data_staging.creation_flag = CF_OVERWRITE;
				break;
			case g2mplsTypes::GRIDSTAGINGCF_APPEND:
				dst.data_staging.creation_flag = CF_APPEND;
				break;
			case g2mplsTypes::GRIDSTAGINGCF_DONTOVERWRITE:
				dst.data_staging.creation_flag =
					CF_DONTOVERWRITE;
				break;
			default:
				throw out_of_range("GRID DS CREAT. FLAG "
						   "out-of-range");
				break;
		}

		dst.data_staging.del_on_termination =
			src.dataStaging.delOnTermination ? 1 : 0;

		dst.data_staging.filename     =
			new std::string(src.dataStaging.fileName);
		dst.data_staging.source       =
			new std::string(src.dataStaging.source);
		dst.data_staging.target       =
			new std::string(src.dataStaging.target);
	}

	if (src.gridSite != 0) {
		BITMASK_BITSET(dst.mask_, grid_site_id);
		dst.grid_site_id = src.gridSite;
	}

	return dst;
}

//
// callPArams
//
call_info_t & operator<<(call_info_t &                    dst,
			 const g2mplsTypes::callParams &  src)
{
	memset(&dst, 0, sizeof(dst));

	switch (src.type) {
		case g2mplsTypes::CALLTYPE_SPC:
			dst.call_type = CALL_TYPE_SPC;
			break;
		case g2mplsTypes::CALLTYPE_PC:
			dst.call_type = CALL_TYPE_PC;
			break;
		case g2mplsTypes::CALLTYPE_SC:
			dst.call_type = CALL_TYPE_SC;
			break;
		case g2mplsTypes::CALLTYPE_AUTO:
			dst.call_type = CALL_TYPE_AUTO;
			break;
		case g2mplsTypes::CALLTYPE_aUGWzUGW:
			dst.call_type = CALL_TYPE_aUGWzUGW;
			break;
		case g2mplsTypes::CALLTYPE_aMGTzEGW:
			dst.call_type = CALL_TYPE_aMGTzEGW;
			break;
		case g2mplsTypes::CALLTYPE_aUGWzEGW:
			dst.call_type = CALL_TYPE_aUGWzEGW;
			break;
		case g2mplsTypes::CALLTYPE_aEGWzMGT:
			dst.call_type = CALL_TYPE_aEGWzMGT;
			break;
		case g2mplsTypes::CALLTYPE_aEGWzUGW:
			dst.call_type = CALL_TYPE_aEGWzUGW;
			break;
		case g2mplsTypes::CALLTYPE_aEGWzEGW:
			dst.call_type = CALL_TYPE_aEGWzEGW;
			break;
		default:
			throw out_of_range("CALL TYPE out-of-range");
			break;
	}

	dst.call_name    = new std::string(src.name);

	// GNS attributes
	dst.times.start_time = src.times.startTime;
	dst.times.end_time   = src.times.endTime;

	dst.job_name     = new std::string(src.jobName);
	dst.job_project  = new std::string(src.jobProject);

	BITMASK_BITSET(dst.mask_,   call_type);
	BITMASK_BITSET(dst.mask_,   call_name);
	BITMASK_BITSET(dst.mask_,   times);
	BITMASK_BITSET(dst.mask_,   job_name);
	BITMASK_BITSET(dst.mask_,   job_project);
	BITMASK_BITRESET(dst.mask_, iTNA_res);
	BITMASK_BITRESET(dst.mask_, eTNA_res);

	return dst;
}

#endif // __cplusplus

#endif // HAVE_OMNIORB
