//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#ifndef __G2PCERA_TRACE_HH__
#define __G2PCERA_TRACE_HH__

#include <assert.h>
#include <string>
#include <iostream>

#include "log.h"
#include "g2pcera_config.hh"

#ifdef VERBOSE_DEBUG
#define G2PCERA_ERROR(FMT, ...) {		\
	zlog_err("%s:%d [%s] " FMT,		\
	         __FILE__,			\
		 __LINE__,			\
	         __PRETTY_FUNCTION__,		\
                 ##__VA_ARGS__);		\
}

#define G2PCERA_WARNING(FMT, ...) {		\
	zlog_warn("%s:%d [%s] " FMT,		\
	          __FILE__,			\
		  __LINE__,			\
	          __PRETTY_FUNCTION__,		\
                  ##__VA_ARGS__);		\
}

#define G2PCERA_INFO(FMT, ...) {		\
	zlog_info("%s:%d [%s] " FMT,		\
	          __FILE__,			\
		  __LINE__,			\
	          __PRETTY_FUNCTION__,		\
                  ##__VA_ARGS__);		\
}

#define G2PCERA_NOTICE(FMT, ...) {		\
	zlog_notice("%s:%d [%s] " FMT,		\
	            __FILE__,			\
		    __LINE__,			\
	            __PRETTY_FUNCTION__,	\
                    ##__VA_ARGS__);		\
}

#define G2PCERA_DEBUG(FMT, ...) {		\
	zlog_debug("%s:%d [%s] " FMT,		\
	           __FILE__,			\
		   __LINE__,			\
	           __PRETTY_FUNCTION__,		\
                   ##__VA_ARGS__);		\
}

#else // VERBOSE_DEBUG

#define G2PCERA_ERROR(FMT, ...) {			\
	zlog_err("[ERROR] " FMT, ##__VA_ARGS__);	\
}

#define G2PCERA_WARNING(FMT, ...) {			\
	zlog_warn("[WARNING] " FMT, ##__VA_ARGS__);	\
}

#define G2PCERA_INFO(FMT, ...) {			\
	zlog_info("[INFO] " FMT, ##__VA_ARGS__);	\
}

#define G2PCERA_NOTICE(FMT, ...) {			\
	zlog_notice("[NOTICE] " FMT, ##__VA_ARGS__);	\
}

#define G2PCERA_DEBUG(FMT, ...) {			\
	zlog_debug("[DEBUG] " FMT, ##__VA_ARGS__);	\
}

#endif // VERBOSE_DEBUG



#ifdef VERBOSE_DEBUG_METHODS
#define METIN() {					\
	zlog_debug("Entering %s", __PRETTY_FUNCTION__);	\
}


#define METOUT() {					\
	zlog_debug("Exiting %s",  __PRETTY_FUNCTION__);	\
}
#else
#define METIN()
#define METOUT()
#endif //  VERBOSE_DEBUG_METHODS

#define HBEAT()					\
	zlog_debug("[HBEAT]> %s:%d (%s)\n",	\
	           __FILE__,			\
	           __LINE__,			\
	           __PRETTY_FUNCTION__)

#define BUG()					\
	zlog_err("Bug hit in %s:%d (%s)\n",	\
	         __FILE__,			\
	         __LINE__,			\
	         __PRETTY_FUNCTION__);

#define DUMP_BAR1	"=================================================="
#define DUMP_BAR2	"--------------------------------------------------"
#define DUMP_TAB	"   "


#endif // __G2PCERA_TRACE_HH__
