//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include <config.h>
#include <map>
#include <list>
#include <string>
#include <sstream>
#include <iomanip>
#include <iostream>
#include <stdexcept>

#include "vty.h"

#include "g2pcerad.hh"
#include "g2pcera_api.hh"
#include "g2pcera_dumper.hh"
#include "g2pcera_vty_utils.hh"

std::map<mapKey_t, call_data_t *>	vtyCallContainer;


#define CALLMAP_FIND(MAP, TYPE, KEY, RES) {		\
	std::map<mapKey_t, TYPE >::iterator itmp;	\
							\
	(RES) = 0;					\
	itmp = (MAP).find(KEY);				\
	if (itmp != (MAP).end()) {			\
		(RES) = (*itmp).second;			\
	}						\
}

#define CALLMAP_COND_CREATE_AND_BIND(MAP, TYPE, KEY, RES, ERR) {	\
	if (!RES) {							\
		(RES) = new (TYPE);					\
		if (!RES) {						\
			(ERR) += "Cannot allocate a VTY CALL container";\
			(MAP).erase(KEY);				\
			return false;					\
		}							\
									\
		memset((RES), 0, sizeof(*(RES)));			\
		(MAP)[KEY] = (RES);					\
	}								\
}




#define SET_RANGE(DST, LB, UB, IS_L, IS_U)	\
	DST.lower_bound = LB;			\
	DST.upper_bound = UB;			\
	DST.lb_included = IS_L ? 1 : 0;		\
	DST.ub_included = IS_U ? 1 : 0;

bool
defaultGridResSpec(res_spec_t & res)
{

	res.grid.application.type                   = GRID_APPNAME_KODAVIS;
	res.grid.application.version.major_revision = 1;
	res.grid.application.version.minor_revision = 2;
	res.grid.application.version.build_fix      = 0;

	res.grid.cand_host.type    = IPv4;
	res.grid.cand_host.preflen = 32;
	inet_aton("192.168.1.1", &(res.grid.cand_host.value.ipv4));

	res.grid.fs_resources.fs_name = FS_NAME_HOME;
	res.grid.fs_resources.fs_type = FS_TYPE_NORMAL;
	SET_RANGE(res.grid.fs_resources.disk_space, 10, 100, true, false);

	delete res.grid.fs_resources.mount_point;
	res.grid.fs_resources.mount_point =
		new std::string("/dev/remote");
	delete res.grid.fs_resources.mount_source;
	res.grid.fs_resources.mount_source =
		new std::string("nfs://cluster.man.poznan.pl");

	res.grid.sys_caps.os_type                   = OS_LINUX;
	res.grid.sys_caps.os_version.major_revision = 2;
	res.grid.sys_caps.os_version.minor_revision = 6;
	res.grid.sys_caps.os_version.build_fix      = 20;
	res.grid.sys_caps.cpu_arch                  = CPU_X86_64;
	res.grid.sys_caps.exclusive_access          = 1;

	SET_RANGE(res.grid.ind_cpu_speed , 1 ,  100, false, true);
	SET_RANGE(res.grid.ind_cpu_time  , 2 ,  200, true,  true);
	SET_RANGE(res.grid.ind_cpu_count , 3 ,  300, false, true);
	SET_RANGE(res.grid.ind_net_bw    , 4 ,  400, false, true);
	SET_RANGE(res.grid.ind_phy_mem   , 5 ,  500, false, true);
	SET_RANGE(res.grid.ind_vir_mem   , 6 ,  600, false, true);
	SET_RANGE(res.grid.ind_disk_space, 7 ,  700, false, true);
	SET_RANGE(res.grid.tot_cpu_time  , 8 ,  800, false, true);
	SET_RANGE(res.grid.tot_cpu_count , 9 ,  900, false, true);
	SET_RANGE(res.grid.tot_phy_mem   , 10, 1000, false, true);
	SET_RANGE(res.grid.tot_vir_mem   , 11, 1100, false, true);
	SET_RANGE(res.grid.tot_disk_space, 12, 1200, false, true);
	SET_RANGE(res.grid.tot_res_count , 13, 1300, false, true);


	res.grid.data_staging.fs_name            = FS_NAME_TMP;
	res.grid.data_staging.creation_flag      = CF_DONTOVERWRITE;
	res.grid.data_staging.del_on_termination = 0;
	delete res.grid.data_staging.filename;
	res.grid.data_staging.filename =
		new std::string("phosphorus.staging.log");
	delete res.grid.data_staging.source;
	res.grid.data_staging.source =
		new std::string("nfs://cluster.man.poznan.pl");
	delete res.grid.data_staging.target;
	res.grid.data_staging.target =
		new std::string("nfs://storage.man.poznan.pl");

	res.grid.grid_site_id = 0XFACAAAAA;
	BITMASK_BITSET(res.mask_     , grid);		// never unset
	BITMASK_BITSET(res.grid.mask_, grid_site_id);	// never unset


	BITMASK_BITSET(res.grid.mask_, application);
	BITMASK_BITSET(res.grid.mask_, cand_host);
	BITMASK_BITSET(res.grid.mask_, fs_resources);
	BITMASK_BITSET(res.grid.mask_, sys_caps);
	BITMASK_BITSET(res.grid.mask_, ind_cpu_speed);
	BITMASK_BITSET(res.grid.mask_, ind_cpu_time);
	BITMASK_BITSET(res.grid.mask_, ind_cpu_count);
	BITMASK_BITSET(res.grid.mask_, ind_net_bw);
	BITMASK_BITSET(res.grid.mask_, ind_phy_mem);
	BITMASK_BITSET(res.grid.mask_, ind_vir_mem);
	BITMASK_BITSET(res.grid.mask_, ind_disk_space);
	BITMASK_BITSET(res.grid.mask_, tot_cpu_time);
	BITMASK_BITSET(res.grid.mask_, tot_cpu_count);
	BITMASK_BITSET(res.grid.mask_, tot_phy_mem);
	BITMASK_BITSET(res.grid.mask_, tot_vir_mem);
	BITMASK_BITSET(res.grid.mask_, tot_disk_space);
	BITMASK_BITSET(res.grid.mask_, tot_res_count);
	BITMASK_BITSET(res.grid.mask_, data_staging);

	return true;
}

bool
vtyCallRoutable(const mapKey_t & callKey)
{
	call_data_t *	data;

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	return (data &&
		isResSpecValid(data->cdata.iTNA_res) &&
		isResSpecValid(data->cdata.eTNA_res) &&
		(data->ldata.sw_cap   != SWCAP_UNKNOWN) &&
		(data->ldata.enc_type != ENCT_UNKNOWN) &&
		(data->ldata.bw       != BwEnc_Unknown));
}


void
vtyCallDumpLog(const mapKey_t & callKey)
{
	call_data_t * data;

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	if (!data) {
		G2PCERA_ERROR("CALL with %s not present in VTY containers",
			      key2string(callKey).c_str());
		vtyCallContainer.erase(callKey);
		return;
	}

	G2PCERA_DEBUG("%s", BAR1);
	G2PCERA_DEBUG("VTY CALL container:");
	G2PCERA_DEBUG("\t - localId             : %s",
		  key2string(callKey).c_str());
	G2PCERA_DEBUG("\t - routable            : %s",
		  vtyCallRoutable(callKey) ? "YES" : "NO");

	logDump("", data->cid);
	logDump("", data->cdata);
	logDump("", data->rdata);
	logDump("", data->ldata);

	G2PCERA_DEBUG("%s", BAR2);
	G2PCERA_DEBUG("");
}

bool
vtyCallUpdate(const mapKey_t &		callKey,
	      const g2mpls_addr_t &	lsr,
	      const uint32_t &		ituCountryCode,
	      const uint64_t &		ituCarrierCode,
	      const uint64_t &		uniqueAP,
	      std::string &		err)
{
	call_data_t * data;

	err = "";


	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	data->cid.type                       = CALLID_OPSPEC;
	data->cid.local_id                   = callKey;
	memcpy(&(data->cid.src_addr), &lsr, sizeof(lsr));

	if (ituCountryCode) {
		data->cid.type             = CALLID_GLOBUNIQ;
		data->cid.itu_country_code = ituCountryCode;
		data->cid.itu_carrier_code = ituCarrierCode;
		data->cid.unique_ap        = uniqueAP;
	}

	// vtyCallDumpLog(callKey);
	return true;
}

bool
vtyCallUpdate(const mapKey_t &		callKey,
	      const call_info_t &	cinfo,
	      std::string &		err)
{
	call_data_t * data;

	err = "";
	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);
	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	data->cdata.call_type = CALL_TYPE_AUTO;
	BITMASK_BITSET(data->cdata.mask_, call_type);

	if (BITMASK_BITTEST(cinfo.mask_, call_name)) {
		if (data->cdata.call_name) {

			delete data->cdata.call_name;
		}
		BITMASK_BITRESET(data->cdata.mask_, call_name);
		data->cdata.call_name = cinfo.call_name;
		BITMASK_BITSET(data->cdata.mask_, call_name);
	}

	if (BITMASK_BITTEST(cinfo.mask_, times)) {
		BITMASK_BITRESET(data->cdata.mask_, times);
		if (cinfo.times.start_time) {
			data->cdata.times.start_time = cinfo.times.start_time;
		}
		if (cinfo.times.end_time) {
			data->cdata.times.end_time = cinfo.times.end_time;
		}
		BITMASK_BITSET(data->cdata.mask_, times);
	}

	if (BITMASK_BITTEST(cinfo.mask_, job_name)) {
		if (data->cdata.job_name) {
			delete data->cdata.job_name;
		}
		BITMASK_BITRESET(data->cdata.mask_, job_name);
		data->cdata.job_name = cinfo.job_name;
		BITMASK_BITSET(data->cdata.mask_, job_name);
	}

	if (BITMASK_BITTEST(cinfo.mask_, job_project)) {
		if (data->cdata.job_project) {
			delete data->cdata.job_project;
		}
		BITMASK_BITRESET(data->cdata.mask_, job_project);
		data->cdata.job_project = cinfo.job_project;
		BITMASK_BITSET(data->cdata.mask_, job_project);
	}

	if (BITMASK_BITTEST(cinfo.mask_, iTNA_res)) {
		BITMASK_BITRESET(data->cdata.mask_, iTNA_res);

		if (!ResSpecUpdate(data->cdata.iTNA_res, cinfo.iTNA_res)) {
			err = "Cannot copy iTNA resource specifications";
			return false;
		}

		BITMASK_BITSET(data->cdata.mask_, iTNA_res);
	}

	if (BITMASK_BITTEST(cinfo.mask_, eTNA_res)) {
		BITMASK_BITRESET(data->cdata.mask_, eTNA_res);

		if (!ResSpecUpdate(data->cdata.eTNA_res, cinfo.eTNA_res)) {
			err = "Cannot copy eTNA resource specifications";
			return false;
		}

		BITMASK_BITSET(data->cdata.mask_, eTNA_res);
	}
	//vtyCallDumpLog(callKey);

	return true;
}

bool
vtyCallUpdate(const mapKey_t &		callKey,
	      const recovery_info_t &	rinfo,
	      std::string &		err)
{
	call_data_t * data;

	err = "";
	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);
	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	SELECTIVE_UPDATE2(data->rdata, rinfo, rec_type);
	SELECTIVE_UPDATE2(data->rdata, rinfo, disj_type);

	//	vtyCallDumpLog(callKey);
	return true;
}

bool
vtyCallUpdate(const mapKey_t &		callKey,
	      const lsp_info_t &	linfo,
	      std::string &		err)
{
	call_data_t * data;

	err = "";
	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);
	CALLMAP_COND_CREATE_AND_BIND(vtyCallContainer, call_data_t,
				     callKey, data, err);

	SELECTIVE_UPDATE2(data->ldata, linfo, sw_cap);
	SELECTIVE_UPDATE2(data->ldata, linfo, enc_type);
	SELECTIVE_UPDATE2(data->ldata, linfo, gpid);
	SELECTIVE_UPDATE2(data->ldata, linfo, bw);
	SELECTIVE_UPDATE2(data->ldata, linfo, setup_prio);
	SELECTIVE_UPDATE2(data->ldata, linfo, holding_prio);
	SELECTIVE_UPDATE2(data->ldata, linfo, exclude_any);
	SELECTIVE_UPDATE2(data->ldata, linfo, include_any);
	SELECTIVE_UPDATE2(data->ldata, linfo, include_all);
	SELECTIVE_UPDATE2(data->ldata, linfo, link_prot_mask);
	SELECTIVE_UPDATE2(data->ldata, linfo, crankback);
	SELECTIVE_UPDATE2(data->ldata, linfo, max_cback_retries_src);
	SELECTIVE_UPDATE2(data->ldata, linfo, max_cback_retries_intmd);
	SELECTIVE_UPDATE2(data->ldata, linfo, type);
	SELECTIVE_UPDATE2(data->ldata, linfo, role);
	SELECTIVE_UPDATE2(data->ldata, linfo, action);
	SELECTIVE_UPDATE2(data->ldata, linfo, rro_mode);
	SELECTIVE_UPDATE2(data->ldata, linfo, refresh_interval);

	if (BITMASK_BITTEST(linfo.mask_, activate_ack)) {
		BITMASK_BITRESET(data->ldata.mask_, activate_ack);
		data->ldata.activate_ack = linfo.activate_ack;
		BITMASK_BITRESET(data->ldata.mask_, activate_ack);
	}

	SELECTIVE_UPDATE2(data->ldata, linfo, rapid_retransm_interval);
	SELECTIVE_UPDATE2(data->ldata, linfo, rapid_retry_limit);
	SELECTIVE_UPDATE2(data->ldata, linfo, increment_value_delta);
	SELECTIVE_UPDATE2(data->ldata, linfo, times);

	if (!BITMASK_BITTEST(data->ldata.mask_, times)) {
		data->ldata.mask_.times = data->cdata.mask_.times;
		data->ldata.times       = data->cdata.times;
	}

	// vtyCallDumpLog(callKey);
	return true;
}

bool
vtyCallRoute(uint32_t			g2pceraId,
	     const mapKey_t &		callKey,
	     std::list<ero_hop_t> &	ero,
	     net_res_spec_t &		eTna,
	     std::string &		err)
{
	call_data_t * data;

	err = "";

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	vtyCallDumpLog(callKey);

	if (!data) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not present in VTY containers";
		vtyCallContainer.erase(callKey);
		return false;
	}

	if (!vtyCallRoutable(callKey)) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not yet routable: missing parameters";
		return false;
	}

	pceraErrorCode_t		ret;

	if (BITMASK_BITTEST(data->cdata.eTNA_res.mask_, net)) {
		ret = PCERA_CALC_API::callRoute(g2pceraId,
						data->cdata.iTNA_res.net,
						data->cdata.eTNA_res.net,
						data->cid,
						data->cdata,
						data->rdata,
						data->ldata,
						ero,
						err);
		eTna = data->cdata.eTNA_res.net;
	} else {
		ret = PCERA_CALC_API::callRouteAny(g2pceraId,
						   data->cdata.iTNA_res.net,
						   data->cdata.eTNA_res.grid,
						   data->cid,
						   data->cdata,
						   data->rdata,
						   data->ldata,
						   ero,
						   eTna,
						   err);
	}

	if (ret != PCERA_ERROR_NONE) {
		G2PCERA_ERROR("Cannot route call into topology "
			      "for G2.PCE-RA %s "
			      "(err: code=%s, reason='%s')",
			      numb_htoa(g2pceraId),
			      PCERA_ERR_TO_STRING(ret), err.c_str());
		return false;
	}

	return true;
}

bool
vtyCallFlush(uint32_t		g2pceraId,
	     const mapKey_t &	callKey,
	     std::string &	err)
{
	call_data_t * data;

	err = "";

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	//vtyCallDumpLog(callKey);

	if (!data) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not present in VTY containers";
		vtyCallContainer.erase(callKey);
		return false;
	}

	if (!vtyCallRoutable(callKey)) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not yet routable: missing parameters";
		return false;
	}

	pceraErrorCode_t		ret;

	ret = PCERA_CALC_API::callFlush(g2pceraId,
					data->cid,
					err);

	if (ret != PCERA_ERROR_NONE) {
		G2PCERA_ERROR("Cannot flush call from topology "
			      "for G2.PCE-RA %s "
			      "(err: code=%s, reason='%s')",
			      numb_htoa(g2pceraId),
			      PCERA_ERR_TO_STRING(ret), err.c_str());
		return false;
	}

	// remove item from container
	if (data->cdata.call_name) {
		delete data->cdata.call_name;
	}
	if (data->cdata.job_name) {
		delete data->cdata.job_name;
	}
	if (data->cdata.job_project) {
		delete data->cdata.job_project;
	}
	if (data->cdata.iTNA_res.grid.fs_resources.mount_point) {
		delete data->cdata.iTNA_res.grid.fs_resources.mount_point;
	}
	if (data->cdata.iTNA_res.grid.fs_resources.mount_source) {
		delete data->cdata.iTNA_res.grid.fs_resources.mount_source;
	}
	if (data->cdata.iTNA_res.grid.data_staging.filename) {
		delete data->cdata.iTNA_res.grid.data_staging.filename;
	}
	if (data->cdata.iTNA_res.grid.data_staging.source) {
		delete data->cdata.iTNA_res.grid.data_staging.source;
	}
	if (data->cdata.iTNA_res.grid.data_staging.target) {
		delete data->cdata.iTNA_res.grid.data_staging.target;
	}
	delete data;
	vtyCallContainer.erase(callKey);

	return true;
}

bool
vtyCallConfirm(uint32_t		g2pceraId,
	       const mapKey_t &	callKey,
	       std::string &	err)
{
	call_data_t * data;

	err = "";

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	//vtyCallDumpLog(callKey);

	if (!data) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not present in VTY containers";
		vtyCallContainer.erase(callKey);
		return false;
	}

	if (!vtyCallRoutable(callKey)) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not yet routable: missing parameters";
		return false;
	}

	pceraErrorCode_t		ret;

	ret = PCERA_CALC_API::callConfirm(g2pceraId,
					  data->cid,
					  err);

	if (ret != PCERA_ERROR_NONE) {
		G2PCERA_ERROR("Cannot confirm call into topology "
			      "for G2.PCE-RA %s "
			      "(err: code=%s, reason='%s')",
			      numb_htoa(g2pceraId),
			      PCERA_ERR_TO_STRING(ret), err.c_str());
		return false;
	}

	return true;
}

bool
vtyGetConnections(uint32_t			g2pceraId,
		  const mapKey_t &		callKey,
		  std::list<ero_hop_t> &	pEro,
		  std::list<ero_hop_t> &	sEro,
		  std::string &			err)
{
	call_data_t * data;

	err = "";

	CALLMAP_FIND(vtyCallContainer, call_data_t *, callKey, data);

	//vtyCallDumpLog(callKey);

	if (!data) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not present in VTY containers";
		vtyCallContainer.erase(callKey);
		return false;
	}

	if (!vtyCallRoutable(callKey)) {
		err  = "CALL with ";
		err += key2string(callKey);
		err += " not yet routable: missing parameters";
		return false;
	}

	pceraErrorCode_t		ret;
	ero_hop_t			srcHop;
	ero_hop_t			dstHop;

	memset(&srcHop, 0, sizeof(srcHop));
	memset(&dstHop, 0, sizeof(dstHop));

	std::list<ero_hop_t> wEro;
	std::list< std::list<ero_hop_t> > iMXRO;

	wEro.empty();
	iMXRO.empty();

#ifdef TMP_TEST_MDMN
	ipv4_t ridAddr;
	inet_aton("192.168.100.9", &ridAddr);
	srcHop.node = ntohl(ridAddr.s_addr);

	inet_aton("192.168.100.8", &ridAddr);
	dstHop.node = ntohl(ridAddr.s_addr);
#endif

	ret = PCERA_CALC_API::connectionRoute(g2pceraId,
					      srcHop,
					      dstHop,
					      data->cid,
					      data->ldata,
					      wEro,
					      sEro,
					      err);

	if (ret != PCERA_ERROR_NONE) {
		G2PCERA_ERROR("Cannot route call into topology "
			      "for G2.PCE-RA %s "
			      "(err: code=%s, reason='%s')",
			      numb_htoa(g2pceraId),
			      PCERA_ERR_TO_STRING(ret), err.c_str());
		return false;
	}

#ifdef TMP_TEST_CBACK
	iMXRO.push_front(wEro);
	iMXRO.push_front(wEro);

	ret = PCERA_CALC_API::connectionRouteDisjoint(g2pceraId,
						      srcHop,
						      dstHop,
						      data->cid,
						      data->ldata,
						      iMXRO,
						      pEro,
						      sEro,
						      err);

	if (ret != PCERA_ERROR_NONE) {
		G2PCERA_ERROR("Cannot route call into topology "
			      "for G2.PCE-RA %s "
			      "(err: code=%s, reason='%s')",
			      numb_htoa(g2pceraId),
			      PCERA_ERR_TO_STRING(ret), err.c_str());
		return false;
	}

	iMXRO.clear();
	iMXRO.push_front(pEro);
	pEro.clear();
	ret = PCERA_CALC_API::connectionRouteDisjoint(g2pceraId,
						      srcHop,
						      dstHop,
						      data->cid,
						      data->ldata,
						      iMXRO,
						      pEro,
						      sEro,
						      err);

	if (ret != PCERA_ERROR_NONE) {
		G2PCERA_ERROR("Cannot route call into topology "
			      "for G2.PCE-RA %s "
			      "(err: code=%s, reason='%s')",
			      numb_htoa(g2pceraId),
			      PCERA_ERR_TO_STRING(ret), err.c_str());
		return false;
	}
#else
	pEro = wEro;
#endif
	return true;
}

void
vtyCallDump(struct vty *	vty,
	    const mapKey_t &	key)
{
	call_data_t *	   data;

	CALLMAP_FIND(vtyCallContainer, call_data_t *, key, data);

	if (!data) {
		vty_out(vty,
			"CALL with %s not present in VTY containers%s",
			key2string(key).c_str(),
			VTY_NEWLINE);
		vtyCallContainer.erase(key);
		return;
	}

	vty_out(vty, "%s%s", BAR1, VTY_NEWLINE);
	vty_out(vty, "VTY CALL container%s", VTY_NEWLINE);
	vty_out(vty, "\t - routable	   : %s%s",
		vtyCallRoutable(key) ? "YES" : "NO", VTY_NEWLINE);

	assert(data);

	vtyDump(vty, "", data->cid);
	vtyDump(vty, "", data->cdata);
	vtyDump(vty, "", data->rdata);
	vtyDump(vty, "", data->ldata);

	vty_out(vty, "%s%s", BAR2, VTY_NEWLINE);
	vty_out(vty, "%s", VTY_NEWLINE);

}

void
vtyCallDump(struct vty * vty)
{
	int i;

	vty_out(vty, "G2PCERA GNS-CALL(s) pending in VTY%s", VTY_NEWLINE);

	if (vtyCallContainer.empty()) {
		vty_out(vty, "   ***EMPTY***%s", VTY_NEWLINE);
		vty_out(vty, "%s", VTY_NEWLINE);
		return;
	}

	std::map<mapKey_t, call_data_t * >::const_iterator iter;
	for (iter = vtyCallContainer.begin(), i = 0;
	     iter != vtyCallContainer.end();
	     iter++, i++) {
		assert((*iter).second);

		vty_out(vty, "   - [p] #%02d: src-LSR = %s / local-ID = %s%s",
			i,
			addr_ntoa(((*iter).second)->cid.src_addr),
			key2string((*iter).first).c_str(),
			VTY_NEWLINE);
		if (i == SHOWLIST_DEPTH) {
			break;
		}
	}
	vty_out(vty, "%s", VTY_NEWLINE);
	vty_out(vty, "   printed %d VTY pending CALL(s)%s", i, VTY_NEWLINE);
}
