//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



//
//

#include "g2rsvpte_config.h"

#include "g2rsvpte_apis.h"
#include "g2rsvpte_scngw_api.h"
#include "g2rsvpte_lrm_api.h"
#include "g2rsvpte_rc_api.h"
#include "g2rsvpte_tnrc_api.h"
#include "g2rsvpte_lsp.h"
#ifdef CRANKBACK_ON
#include "g2rsvpte_g2pcera_api.h"
#endif // CRANKBACK_ON
#ifdef PERSISTENCY_ON
#include "g2rsvpte_persistency_py.h"
#endif // PERSISTENCY_ON

#include "g2rsvpte_utils.h"
#include "g2rsvpted.h"
#include "thread.h"

#include <string>
#include <sstream>
#include <iomanip>

//
//
//    RSVP G2RAPI
//
//
grapiErrorCode_t
RSVP_G2RAPI::lspCreate(const lsp_ident_t &      key,
		       const std::string &      sessionName,
		       const call_ident_t &     callIdent,
		       const net_res_spec_t &   iRes,
		       const net_res_spec_t &   eRes,
		       const lsp_info_t &       lspInfo,
		       const recovery_info_t &  recoveryInfo,
		       std::string &            resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	if (!lspInfo.times.start_time) {
		if (lspInfo.times.end_time) {
			resp = "TIMES PARAMETER UNCORRECT: START TIME NULL AND END TIME NON NULL";
			return GRAPI_ERROR_LSP_NOT_CREATED;
		}
	}
	if (!lspInfo.times.end_time) {
		if (lspInfo.times.start_time) {
			resp = "TIMES PARAMETER UNCORRECT: END TIME NULL AND START TIME NON NULL";
			return GRAPI_ERROR_LSP_NOT_CREATED;
		}
	}
	if (lspInfo.times.start_time && lspInfo.times.end_time) {
		if (lspInfo.times.end_time <= lspInfo.times.start_time) {
			resp = "END TIME <= START TIME";
			return GRAPI_ERROR_LSP_NOT_CREATED;
		}
	}

	if (!proto->createLSP(key,
			      sessionName,
			      callIdent,
			      iRes,
			      eRes,
			      lspInfo,
			      recoveryInfo)) {
		resp = "IMPOSSIBLE TO CREATE LSP";
		return GRAPI_ERROR_LSP_NOT_CREATED;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspDestroy(const lsp_ident_t &     key,
			std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP JUST DELETED OR NEVER CREATED";
		return  GRAPI_ERROR_NONE;
	}

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	//	if (!proto->destroyLSP(key, false)) {
	//		resp = "IMPOSSIBLE TO DESTROY LSP";
	//	return GRAPI_ERROR_LSP_NOT_DESTROYED;
	//}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspEnable(const lsp_ident_t &     key,
		       std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUNT";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->enableLSP()) {
		resp = "IMPOSSIBLE TO ENABLE LSP";
		return GRAPI_ERROR_LSP_NOT_ENABLED;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspDisable(const lsp_ident_t &     key,
			std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUNT";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->disableLSP()) {
		resp = "IMPOSSIBLE TO DISABLE LSP";
		return GRAPI_ERROR_LSP_NOT_DISABLED;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspEroAttach(const lsp_ident_t &             key,
			  const std::list<lsp_ero_sobj_t> ero,
			  std::string &                   resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	std::list<lsp_ero_sobj_t>::const_iterator it;
	for (it = ero.begin(); it != ero.end(); it++) {
		RSVP::EroSubObject * subObj = 0;

		subObj = RSVP::createEroSubObj((*it));
		if (!subObj) {
			resp = "IMPOSSIBLE TO CREATE ERO SUBOBJ";
			return GRAPI_ERROR_ERO_NOT_ADDED;
		}
		if (!lsp->attachEroSubObj(subObj)) {
			resp = "IMPOSSIBLE TO ADD ERO SUBOBJ TO LSP";
			return GRAPI_ERROR_ERO_NOT_ADDED;
		}
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspEroDetach(const lsp_ident_t &             key,
			  const std::list<lsp_ero_sobj_t> ero,
			  std::string &                   resp)
{
	METIN();

	resp = std::string(__PRETTY_FUNCTION__) +
		std::string(" --> NOT YET UNSTUBBED");

	METOUT();
	return GRAPI_ERROR_GENERIC;
}

grapiErrorCode_t
RSVP_G2RAPI::lspSendResv(const lsp_ident_t &        key,
			 std::string &              resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->sendResv()) {
		resp = "IMPOSSIBLE TO SEND A RESV";
		return GRAPI_ERROR_RESV_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspSendConfirm(const lsp_ident_t &     key,
			    std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->sendConfirm()) {
		resp = "IMPOSSIBLE TO SEND A CONFIRM";
		return GRAPI_ERROR_CONFIRM_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspForceUp(const lsp_ident_t &    key,
			std::string &          resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->signalUpLSP()) {
		resp = "IMPOSSIBLE TO FORCE UP LSP";
		return GRAPI_ERROR_LSP_NOT_FORCEDUP;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspForceDown(const lsp_ident_t &    key,
			  std::string &          resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }


	if (!lsp->signalDownLSP()) {
		resp = "IMPOSSIBLE TO FORCE DOWN LSP";
		return GRAPI_ERROR_LSP_NOT_FORCEDDOWN;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspXConnCompleted(const lsp_ident_t &  key,
			       std::string &        resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->xConnCompleted()) {
		resp = "IMPOSSIBLE TO SEND A XC COMPLETED EVENT";
		return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::lspXConnFailed(const lsp_ident_t &  key,
			    std::string &        resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->xConnFailed()) {
		resp = "IMPOSSIBLE TO SEND A XC FAILED EVENT";
		return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::getLsps(std::list<lsp_ident_t> & lsps,
		     std::string &            resp)
{
	METIN();

	RSVP::G2RSVPTE *                  proto;
	RSVP::G2RSVPTE::iterator_sessions sessionIter;
	RSVP::Session *                   session;
	RSVP::Session::iterator_lsps      lspIter;
	RSVP::LSP *                       lsp;
	lsp_ident_t                       lspIdent;
	adj_type_t                        adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	for (sessionIter  = proto->begin_sessions();
	     sessionIter != proto->end_sessions();
	     sessionIter++) {
		session = (*sessionIter).second;
		assert(session);
		for (lspIter  = session->begin_lsps();
		     lspIter != session->end_lsps();
		     lspIter++) {
			lsp = (*lspIter).second;
			assert(lsp);
			lspIdent = lsp->getIdent();
			//Add LSP ident to the list
			lsps.push_back(lspIdent);
		}
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::getLspDetails(const lsp_ident_t &      key,
			   call_ident_t &           callIdent,
			   lsp_info_t &             lspInfo,
			   recovery_info_t &        recoveryInfo,
			   std::string &            resp)
{
	METIN();

	RSVP::G2RSVPTE *                  proto;
	adj_type_t                        adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	callIdent = lsp->callId();

	if (!lsp->getInfo(lspInfo)) {
		resp = "IMPOSSIBLE TO GET LSP INFO";
		return GRAPI_ERROR_GENERIC;
	}

	//TMP-FIXME
	memset(&recoveryInfo, 0, sizeof(recoveryInfo));
	//

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::getLspEro(const lsp_ident_t &         key,
		       std::list<lsp_ero_sobj_t> & ero,
		       std::string &               resp)
{

	METIN();

	RSVP::G2RSVPTE *                  proto;
	adj_type_t                        adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	//get ERO
	if (!lsp->psb()) {
		resp = "NULL PSB POINTER";
		return GRAPI_ERROR_GENERIC;
	}
	if (!lsp->psb()->pathIn()) {
		resp = "NULL PATH IN POINTER";
		return GRAPI_ERROR_GENERIC;
	}
	if (!lsp->psb()->pathIn()->getEro(ero)) {
		resp = "IMPOSSIBLE TO GET ERO";
		return GRAPI_ERROR_GENERIC;
	}
	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::getLspRro(const lsp_ident_t &         key,
		       std::list<lsp_ero_sobj_t> & rro,
		       std::string &               resp)
{

	METIN();

	RSVP::G2RSVPTE *                  proto;
	adj_type_t                        adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	//get RRO
	if (!lsp->psb()) {
		resp = "NULL PSB POINTER";
		return GRAPI_ERROR_GENERIC;
	}
	if (!lsp->rsb()) {
		resp = "NULL RSB POINTER";
		return GRAPI_ERROR_GENERIC;
	}
	if (lsp->rsb()->resvIn()) {
		if(!lsp->rsb()->resvIn()->getRro(rro)) {
			resp = "IMPOSSIBLE TO GET RRO FROM RESV IN";
			return GRAPI_ERROR_GENERIC;
		}
	} else if (lsp->psb()->pathErr()) {
		if (lsp->psb()->pathErr()->getRro(rro)) {
			resp = "IMPOSSIBLE TO GET RRO FROM PATH ERR";
			return GRAPI_ERROR_GENERIC;
		}
	} else {
		resp = "IMPOSSIBLE TO GET RRO: NULL RESV IN and PATH ERR POINTERS";
			return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_G2RAPI::getLspXro(const lsp_ident_t &         key,
		       std::list<lsp_ero_sobj_t> & xro,
		       std::string &               resp)
{

	METIN();

	RSVP::G2RSVPTE *                  proto;
	adj_type_t                        adjType;

	adjType = INNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (lsp->isHead()) {
		resp = "THIS ISN'T THE HEAD NODE";
		return GRAPI_ERROR_GENERIC;
	}
#ifdef CRANKBACK_ON
	if (lsp->cbackScope() == CB_NONE) {
		resp = "CRANKBACK SCOPE IS 'CB_NONE'";
		return GRAPI_ERROR_GENERIC;
	}

	//get XRO
	xro = lsp->getExcludeEro();
	if (xro.size() == 0) {
		resp = "EXCLUDE ERO IS EMPTY";
		return GRAPI_ERROR_GENERIC;
	}
#else
	resp = "CRANKBACK IS DISABLED";
	return GRAPI_ERROR_GENERIC;
#endif
	METOUT();

	return GRAPI_ERROR_NONE;
}

//
//
//    RSVP GXRAPI
//
//

grapiErrorCode_t
RSVP_GXRAPI::lspCreate(const lsp_ident_t &      key,
		       const std::string &      sessionName,
		       const call_ident_t &     callIdent,
		       const call_info_t &      callInfo,
		       const lsp_info_t &       lspInfo,
		       const recovery_info_t &  recoveryInfo,
		       const std::string &      aaaInfo,
		       std::string &            resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	if (!lspInfo.times.start_time) {
		if (lspInfo.times.end_time) {
			resp = "TIMES PARAMETER UNCORRECT: END TIME NULL";
			return GRAPI_ERROR_LSP_NOT_CREATED;
		}
	}
	if (!lspInfo.times.end_time) {
		if (lspInfo.times.start_time) {
			resp = "TIMES PARAMETER UNCORRECT: START TIME NULL";
			return GRAPI_ERROR_LSP_NOT_CREATED;
		}
	}
	if (lspInfo.times.start_time && lspInfo.times.end_time) {
		if (lspInfo.times.end_time <= lspInfo.times.start_time) {
			resp = "END TIME <= START TIME";
			return GRAPI_ERROR_LSP_NOT_CREATED;
		}
	}
	if (lspInfo.crankback != CB_NONE &&
	    lspInfo.crankback != CB_E2E) {
		resp += "INVALID CRANKBACK SCOPE (";
		resp += SHOW_CBACK_SCOPE(lspInfo.crankback);
		resp += "): ";
		resp += "ONLY E2E CRANKBACK IS ALLOWED";

		return GRAPI_ERROR_INVALID_PARAMETER;
	}

	if (!proto->createLSP(key,
			      sessionName,
			      callIdent,
			      callInfo,
			      lspInfo,
			      recoveryInfo,
			      aaaInfo)) {
		resp = "IMPOSSIBLE TO CREATE LSP";
		return GRAPI_ERROR_LSP_NOT_CREATED;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspDestroy(const lsp_ident_t &     key,
			std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP JUST DELETED OR NEVER CREATED";
		return  GRAPI_ERROR_NONE;
	}

	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	//if (!proto->destroyLSP(key, false)) {
	//	resp = "IMPOSSIBLE TO DESTROY LSP";
	//	return GRAPI_ERROR_LSP_NOT_DESTROYED;
	//}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSetTna(const lsp_ident_t &      key,
		       const net_res_spec_t &   tnaRes,
		       bool                     ingress,
		       std::string &            resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (lsp->isHead()) {
		if (!lsp->setTna(tnaRes, ingress)) {
			resp = "IMPOSSIBLE TO SET TNA";
			return GRAPI_ERROR_GENERIC;
		}
	} else {
		if (!lsp->updateTna(tnaRes, ingress)) {
			resp = "IMPOSSIBLE TO UPDATE TNA";
			return GRAPI_ERROR_GENERIC;
		}
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSetGnsTna(const lsp_ident_t &     key,
			  const grid_res_spec_t & gnsTna,
			  bool                    ingress,
			  std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->setGnsTna(gnsTna, ingress)) {
		resp = "IMPOSSIBLE TO SET GNS-TNA";
		return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspGetTna(const lsp_ident_t &      key,
		       bool                     ingress,
		       net_res_spec_t &         tnaRes,
		       std::string &            resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->getTna(ingress, tnaRes)) {
		resp = "IMPOSSIBLE TO GET TNA";
		return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspGetGnsTna(const lsp_ident_t &   key,
			  bool                  ingress,
			  grid_res_spec_t &     gnsTna,
			  std::string &         resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->getGnsTna(ingress, gnsTna)) {
		resp = "IMPOSSIBLE TO GET GNS-TNA";
		return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspGetTnResource(const lsp_ident_t & key,
			      net_res_spec_t &    tnRes,
			      std::string &       resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	if (!lsp->getTnResource(tnRes)) {
		resp = "IMPOSSIBLE TO GET TN RESOURCE";
		return GRAPI_ERROR_GENERIC;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspEroAttach(const lsp_ident_t &             key,
			  const std::list<lsp_ero_sobj_t> ero,
			  std::string &                   resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = ENNI;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(key);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

	std::list<lsp_ero_sobj_t>::const_iterator it;
	for (it = ero.begin(); it != ero.end(); it++) {
		RSVP::EroSubObject * subObj = 0;

		subObj = RSVP::createEroSubObj((*it));
		if (!subObj) {
			resp = "IMPOSSIBLE TO CREATE ERO SUBOBJ";
			return GRAPI_ERROR_ERO_NOT_ADDED;
		}
		if (!lsp->attachEroSubObj(subObj)) {
			resp = "IMPOSSIBLE TO ADD ERO SUBOBJ TO LSP";
			return GRAPI_ERROR_ERO_NOT_ADDED;
		}
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSendPath(const lsp_ident_t &        key,
			 std::string &              resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

        if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }

	if (!lsp->sendPath()) {
		resp = "IMPOSSIBLE TO SEND A PATH";
		return GRAPI_ERROR_PATH_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSendResv(const lsp_ident_t &        key,
			 std::string &              resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

        if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }

	if (!lsp->sendResv()) {
		resp = "IMPOSSIBLE TO SEND A RESV";
		return GRAPI_ERROR_RESV_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSendConfirm(const lsp_ident_t &     key,
			    std::string &           resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

        if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }
	
	if (!lsp->sendConfirm()) {
		resp = "IMPOSSIBLE TO SEND A CONFIRM";
		return GRAPI_ERROR_CONFIRM_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSendDown(const lsp_ident_t &    key,
			 std::string &          resp)
{
	METIN();


	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

        if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }

	if (!lsp->sendDown()) {
		resp = "IMPOSSIBLE TO SEND A DOWN";
		return GRAPI_ERROR_DOWN_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSendTear(const lsp_ident_t &    key,
			 std::string &          resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

        if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }

	if (!lsp->sendTear()) {
		resp = "IMPOSSIBLE TO SEND A TEAR";
		return GRAPI_ERROR_TEAR_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_GXRAPI::lspSendError(const lsp_ident_t &    key,
			  const error_info_t &   error,
			  std::string &          resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;

	resp = "";

	proto = RSVP_MASTER.getProtoInstance(UNI);

	if (!proto) {
		proto = RSVP_MASTER.getProtoInstance(ENNI);
		if (!proto) {
			resp = "INSTANCE NOT FOUND";
			return GRAPI_ERROR_INSTANCE_NOT_FOUND;
		}
	}

	lsp_ident_t tmpKey;
	tmpKey = key;

	if (tmpKey.src_nid == 0) {
		tmpKey.src_nid = proto->nodeId();
	}

	RSVP::LSP * lsp;
	lsp = proto->findLSP(tmpKey);

	if (!lsp) {
		resp = "LSP NOT FOUND";
		return GRAPI_ERROR_LSP_NOT_FOUND;
	}

        if (lsp->toBeDestroyed()) {
	        resp = "LSP HAS TO BE DESTROYED";
	        return GRAPI_ERROR_LSP_NOT_FOUND;
        }

	if (!lsp->sendError(error)) {
		resp = "IMPOSSIBLE TO SEND AN ERROR";
		return GRAPI_ERROR_TEAR_NOT_SENT;
	}

	METOUT();

	return GRAPI_ERROR_NONE;
}


//
//
//    RSVP API
//
//
RSVP::G2RSVPTE *
RSVP_API::g2rsvpteGet(std::string & resp)
{
	METIN();

	RSVP::G2RSVPTE * rsvp;

	rsvp = NULL; //	= RSVP_MASTER.getProtoInstance();

	resp = std::string(__PRETTY_FUNCTION__) +
		std::string(" --> NOT YET UNSTUBBED");
#if 0
	rsvp = rsvp_lookup();
	if (rsvp == NULL) {
		rsvp = rsvp_new();
		rsvp_add(rsvp);
	}
#endif
	return rsvp;

	METOUT();
}


grapiErrorCode_t
RSVP_API::g2rsvpteStart(adj_type_t adjType, std::string & resp, bool isClient, bool isCore)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	std::string      protoType;
	bool             tunneledPackets;

	resp = "";

	switch (adjType) {
		case INNI:
			proto = RSVP_MASTER.getProtoInstance(UNI);
			if (proto && proto->isUniClient()) {
				resp = "AN_UNI_CLIENT_INSTANCE_ALREADY_EXISTS";
				METOUT();
				return GRAPI_ERROR_INSTANCE_UNI_EXISTS;
			}
			//TMP - FIXME: add in configuration?
			tunneledPackets = true;
			//
			protoType = "G2.RSVP-TE" ;
			break;
		case ENNI:
			proto = RSVP_MASTER.getProtoInstance(UNI);
			if (proto) {
				resp = "AN_UNI_INSTANCE_ALREADY_EXISTS";
				METOUT();
				return GRAPI_ERROR_INSTANCE_UNI_EXISTS;
			}
			//TMP - FIXME: add in configuration?
			tunneledPackets = false;
			//
			if (RSVP_MASTER.getProtoInstanceNoTun()) {
				resp = "NO_TUNNELED_INSTANCE_ALREADY_EXISTS";
				METOUT();
				return GRAPI_ERROR_INSTANCE_ALREADY_EXISTS;
			}
			protoType = "G.ENNI-RSVP";
			break;
		case UNI:
			if (isClient && RSVP_MASTER.getProtoListSize()) {
				resp = "AN_INSTANCE_ALREADY_EXISTS";
				METOUT();
				return GRAPI_ERROR_INSTANCE_EXISTS;
			}
			proto = RSVP_MASTER.getProtoInstance(ENNI);
			if (proto) {
				resp = "AN_ENNI_INSTANCE_ALREADY_EXISTS";
				METOUT();
				return GRAPI_ERROR_INSTANCE_ENNI_EXISTS;
			}
			//TMP - FIXME: add in configuration?
			tunneledPackets = false;
			//
			if (RSVP_MASTER.getProtoInstanceNoTun()) {
				resp = "NO_TUNNELED_INSTANCE_ALREADY_EXISTS";
				METOUT();
				return GRAPI_ERROR_INSTANCE_ALREADY_EXISTS;
			}
			protoType = (isClient ? "G.UNI/C-RSVP" : "G.UNI/N-RSVP");
			break;
		default:
			resp = "WRONG_TYPE_OF_INSTANCE";
			METOUT();
			return GRAPI_ERROR_GENERIC;
	}

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (proto) {
		resp = "INSTANCE_ALREADY_EXISTS";
		METOUT();
		return GRAPI_ERROR_INSTANCE_ALREADY_EXISTS;
	}

	G2RSVPTE_DBG("Creating %s instance...", protoType.c_str());

	try {
		proto = new RSVP::G2RSVPTE(adjType, tunneledPackets);
		if (!proto) {
			G2RSVPTE_ERR("Cannot allocate enough memory "
				     "for a %s instance", protoType.c_str());
			METOUT();
			return GRAPI_ERROR_NO_MEMORY;
		}
	} catch (...) {
		G2RSVPTE_ERR("Cannot init connection with SCNgw ... Exit !");
		METOUT();
		exit(-1);
	}

	proto->isUniClient(isClient);
	proto->isCore(isCore);

	uint32_t nodeId;
	if (!getRouterId(nodeId)) {
		G2RSVPTE_WRN("Starting %s instance with NULL "
			     "router ID!",
			     protoType.c_str());
	} else {
		G2RSVPTE_DBG("Starting %s instance with router "
			     "ID %s",
			     protoType.c_str(), numb_htoa(nodeId));
		//set nodeId
		proto->nodeId(nodeId);
	}

	if (!proto->loadInterfaces()) {
		G2RSVPTE_ERR("Cannot load the interfaces for this "
			     "%s instance: delete it.", protoType.c_str());
		delete proto;
		resp = "CANNOT_LOAD_INTERFACES";
		METOUT();
		return GRAPI_ERROR_GENERIC;
	}

	if (!RSVP_MASTER.attach(proto)) {
		G2RSVPTE_ERR("Cannot attach %s instance "
			     "to its master manager: delete it.", protoType.c_str());
		delete proto;
		resp = "CANNOT_ATTACH_INSTANCE";
		METOUT();
		return GRAPI_ERROR_GENERIC;
	}

#ifdef PERSISTENCY_ON
	if (!proto->persistency_.initDatabase(adjType)) {
		G2RSVPTE_ERR("Cannot initialize database in Persistency Controller"
			     "for %s instance: delete it.", protoType.c_str());
		delete proto;
		resp = "CANNOT_INIT_DB";
		METOUT();
		return GRAPI_ERROR_GENERIC;
	}
	proto->databaseSync();
#endif // PERSISTENCY_ON

	// Starting listen from SCNgw
	G2RSVPTE_DBG("Listening %s packets ...", protoType.c_str());

#ifdef G2RSVPTE2SCNGW_DEBUG
	server_start();
#else
	struct thread * thr;
	thr = thread_add_read(RSVP_MASTER.getMaster(),
			      listen_from_scngw,
			      proto,
			      proto->scngwFd());
	if (thr) {
		proto->readThread(thr);
	}
#endif

	G2RSVPTE_DBG("%s instance started", protoType.c_str());

	METOUT();

	return GRAPI_ERROR_NONE;
}

grapiErrorCode_t
RSVP_API::g2rsvpteStop(adj_type_t adjType, std::string & resp)
{
	METIN();

	RSVP::G2RSVPTE * proto;
	std::string      protoType;

	resp = "";

	protoType = (( adjType == INNI) ? "G2.RSVP-TE" :
		     ((adjType == UNI ) ? "G.UNI-RSVP" : "G.ENNI-RSVP"));

	G2RSVPTE_DBG("Stopping %s instance...", protoType.c_str());

	proto = RSVP_MASTER.getProtoInstance(adjType);

	if (!proto) {
		resp = "INSTANCE NOT FOUND";
		METOUT();
		return GRAPI_ERROR_INSTANCE_NOT_FOUND;
	}

	if (!RSVP_MASTER.detach(proto)) {
		G2RSVPTE_ERR("Cannot detach %s instance "
			     "to its master manager",
			     protoType.c_str());
		resp = "CANNOT_DETACH_INSTANCE";
		METOUT();
		return GRAPI_ERROR_GENERIC;
	}

#ifdef PERSISTENCY_ON
	if (!proto->persistency_.closeDatabase(adjType)) {
		G2RSVPTE_ERR("Cannot close database in Persistency Controller"
			     "for %s instance" , protoType.c_str());
		resp = "CANNOT_CLOSE_INIT_DB";
		METOUT();
		return GRAPI_ERROR_GENERIC;
	}
#endif // PERSISTENCY_ON

	delete proto;
	proto = 0;

	G2RSVPTE_DBG("%s instance stopped", protoType.c_str());

	METOUT();

	return GRAPI_ERROR_NONE;
}

int
destroyAdvRsrvLsp(struct thread * thr)
{
	RSVP::LSP *      lsp;
	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	G2RSVPTE_DBG("Advance reservation TIMED OUT: destroy LSP.");

	lsp   = (RSVP::LSP *) THREAD_ARG(thr);
	proto = RSVP_MASTER.getProtoInstance(adjType);
	if (!proto) {
		G2RSVPTE_ERR("ERROR: instance not found");
		return 0;
	}

	if (!proto->checkLSP(lsp)) {
		G2RSVPTE_WRN("LSP already deleted.");
		return 0;
	}

	//Destroy LSP
	if (!lsp->signalDownLSP()) {
		G2RSVPTE_ERR("Impossible to delete LSP.");
		return 0;
	}

	return 1;
}

int
lspFSMtimerElapsed(struct thread * thr)
{
	RSVP::LSP *                    lsp;
	RSVP::G2RSVPTE *               proto;
	G2RSVPTE_LSP_TIMER_context_t * timerCtxt;
	adj_type_t                     adjType;

	timerCtxt = (G2RSVPTE_LSP_TIMER_context_t *) THREAD_ARG(thr);
	assert(timerCtxt);

	lsp     = timerCtxt->lsp;
	adjType = timerCtxt->adjType;
	assert(lsp);

	G2RSVPTE_WRN("FSM %s state timer elapsed", lsp->fsmInst_->currentState().c_str());

	proto = RSVP_MASTER.getProtoInstance(adjType);
	if (!proto) {
		G2RSVPTE_ERR("Instance not found. Cannot destroy LSP.");
		return 0;
	}

	if (!proto->checkLSP(lsp)) {
		G2RSVPTE_WRN("LSP already destroyed.");
		return 0;
	}

	if (!lsp->isErrored()) {
		// send a PathTear downstream if is not tail
		if (!lsp->isTail()) {
			// Build the PathTear msg from the PSB PathOut
			if (!lsp->psb()->pathOut()) {
				G2RSVPTE_ERR("Invalid PathOut msg. Cannot send PathTear");
				return 0;
			}

			if (lsp->psb()->pathTear()) {
				G2RSVPTE_WRN("Freeing PathTear msg before allocating");
				delete lsp->psb()->pathTear();
				lsp->psb()->pathTear(0);
			}

			lsp->psb()->pathTear(new RSVP::PathTearMessage(*lsp->psb()->pathOut()));

			if (!lsp->psb()->pathTear()) {
				G2RSVPTE_ERR("Could not allocate outgoing PathTear msg");
				return 0;
			}

			G2RSVPTE_INF("sending(DS) PathTear msg to NE %s ...",
				     numb_htoa(lsp->rsb()->nodeId()));
			G2RSVPTE_INF("... through TE link %s",
				     addr_ntoa(lsp->rsb()->interface()->localId()));

			if (!lsp->dsAckNack() ||
			    !lsp->dsAckNack()->sendMsg(lsp->psb()->pathTear())) {
				G2RSVPTE_ERR("Could not deliver PathTear msg to the DS ACK FSM");
				return 0;
			}
		}
		// Send a PathErr with PSR upstream
		if (!lsp->isHead()) {
			// Build the PathErr msg from the PSB PathOut
			if (!lsp->psb()->pathOut()) {
				G2RSVPTE_ERR("Invalid PathOut msg. Cannot send PathErr upstream");
				return 0;
			}

			if (lsp->psb()->pathErr()) {
				G2RSVPTE_WRN("PathErr already exists ... deleting it before creating again");
				delete lsp->psb()->pathErr();
				lsp->psb()->pathErr(0);
			}

			RSVP::PathErrMessage * pathErr;
			// Set NOT_ERASE flags for this pkg
			pathErr = new RSVP::PathErrMessage(*lsp->psb()->pathOut());

			if (!pathErr) {
				G2RSVPTE_ERR("Could not allocate PathErr msg");
				return 0;
			}

			//
			// Create Error Spec, for now IPv4 but with cb IF_ID_IPv4 ?????
			//
			RSVP::IFIDIPv4ErrorSpec_Object * tmpErrSpec;
			tmpErrSpec = new RSVP::IFIDIPv4ErrorSpec_Object(lsp->getInstance()->nodeId(),
									G2RSVPTE_ERROR_SPEC_FLAG_PathStateRemoved,
									G2RSVPTE_ERROR_SPEC_SYSTEM,
									0);
			if (!tmpErrSpec) {
				G2RSVPTE_ERR("Impossible to allocate ErrorSpec subObj for PathErr msg");
				delete pathErr;
				return 0;
			}
			// Set error spec into PathErr msg
			pathErr->error_spec_ = tmpErrSpec;

			//
			// Set PathErr msg into PSB
			//
			lsp->psb()->pathErr(pathErr);

			G2RSVPTE_INF("sending(US) PathErr msg (errCode = %d, errVal = %d)",
				     G2RSVPTE_ERROR_SPEC_SYSTEM, 0);

			if (!lsp->usAckNack() ||
			    !lsp->usAckNack()->sendMsg(lsp->psb()->pathErr())) {
				G2RSVPTE_ERR("could not deliver PathErr msg (errCode = %d, "
					     "errVal = %d) to upstream ACK/NACK",
					     G2RSVPTE_ERROR_SPEC_SYSTEM, 0);
				return 0;
			}
		}
	}
	if ((lsp->getInstance()->adjType() == INNI) &&
	    lsp->tnrcCookie()) {
		// Request a resource release at the TNRC
		if (!releaseResource(lsp)) {
			G2RSVPTE_ERR("Impossible to release resource");
		}
		//set tnrcCookie to 0
		lsp->tnrcCookie(0);
	}
	lsp->toBeDestroyed(true);
	postLspDeletion(lsp->getIdent(), adjType);

	return 0;
}

#ifdef CRANKBACK_ON
int
lspNextCbackAttemptStart(struct thread * thr)
{
	RSVP::LSP *                    lsp;

	lsp = (RSVP::LSP *) THREAD_ARG(thr);
	assert(lsp);
	assert(lsp->getInstance());

	lsp->currentCbackAttemptIncr();

	G2RSVPTE_DBG("Crankback: attempt number %d",
		     lsp->currentCbackAttempt());

	//clean PSB and RSB
	if (!lsp->cleanSBs()) {
		G2RSVPTE_ERR("Impossible to clean state blocks. "
			     "Going to notify RC about failure.");
		postLspErrEvent(lsp, true);
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return -1;
	}
	//clean PSB and RSB
	if (!lsp->cleanLabelObjs()) {
		G2RSVPTE_ERR("Impossible to clean old Label Objects. "
			     "Going to notify RC about failure.");
		postLspErrEvent(lsp, true);
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return -1;
	}
	//clean old ero
	if (!lsp->cleanEro()) {
		G2RSVPTE_ERR("Impossible to clean old ERO. "
			     "Going to notify RC about failure.");
		postLspErrEvent(lsp, true);
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return -1;
	}
	//clean old rsvphop
	if (!lsp->cleanRsvpHop()) {
		G2RSVPTE_ERR("Impossible to clean old RsvpHop Obj. "
			     "Going to notify RC about failure.");
		postLspErrEvent(lsp, true);
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return -1;
	}

	// call G2PCERA
	std::list<lsp_ero_sobj_t> ero;
	ero.clear();
	if (!connectionRouteDisjoint(lsp, ero)) {
		G2RSVPTE_ERR("Impossible to get new ERO from G2PCERA. "
			     "Going to notify RC about failure.");
		postLspErrEvent(lsp, true);
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
		return -1;
	}
	// attach ERO to LSP
	std::list<lsp_ero_sobj_t>::const_iterator it;
	for (it = ero.begin(); it != ero.end(); it++) {
		RSVP::EroSubObject * subObj = 0;

		subObj = RSVP::createEroSubObj((*it));
		if (!subObj) {
			G2RSVPTE_ERR("Impossible to create ERO subobj. "
			     "Going to notify RC about failure.");
			postLspErrEvent(lsp, true);
			lsp->toBeDestroyed(true);
			postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
			return -1;
		}
		if (!lsp->attachEroSubObj(subObj)) {
			G2RSVPTE_ERR("Impossible to add ERO subobj to LSP. "
			     "Going to notify RC about failure.");
			postLspErrEvent(lsp, true);
			lsp->toBeDestroyed(true);
			postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
			return -1;
		}
	}

	if (!lsp->signalUpLSP(true)) {
		G2RSVPTE_ERR("Impossible to signal UP the LSP. "
			     "Going to notify RC about failure.");
		postLspErrEvent(lsp, true);
		lsp->toBeDestroyed(true);
		postLspDeletion(lsp->getIdent(), lsp->getInstance()->adjType());
	}

	return 0;
}
#endif // CRANKBACK_ON
