/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#ifndef __G2RSVP_APIS_H__
#define __G2RSVP_APIS_H__

#include "g2rsvpte_dm.h"
#include "g2rsvpte_config.h"
#include <g2mpls_types.h>
#include <g2mpls_addr.h>
#include <string>
#include "thread.h"

#include "g2rsvpte_common_types.h"

typedef enum {
	GRAPI_ERROR_NONE = 0,
	GRAPI_ERROR_INVALID_PARAMETER,
	GRAPI_ERROR_INSTANCE_NOT_FOUND,
	GRAPI_ERROR_INSTANCE_ALREADY_EXISTS,
	GRAPI_ERROR_INSTANCE_UNI_EXISTS,
	GRAPI_ERROR_INSTANCE_ENNI_EXISTS,
	GRAPI_ERROR_INSTANCE_EXISTS,
	GRAPI_ERROR_LSP_NOT_FOUND,
	GRAPI_ERROR_LSP_NOT_CREATED,
	GRAPI_ERROR_LSP_NOT_DESTROYED,
	GRAPI_ERROR_LSP_NOT_ENABLED,
	GRAPI_ERROR_LSP_NOT_DISABLED,
	GRAPI_ERROR_LSP_NOT_FORCEDUP,
	GRAPI_ERROR_LSP_NOT_FORCEDDOWN,
	GRAPI_ERROR_LSP_NOT_CONFIGURED,
	GRAPI_ERROR_LSP_ALREADY_EXISTS,
	GRAPI_ERROR_LSP_ERO_SOBJ_NOT_FOUND,
	GRAPI_ERROR_LSP_ERO_SOBJ_ALREADY_EXISTS,
	GRAPI_ERROR_LSP_TNA_NOT_SET,
	GRAPI_ERROR_NO_MEMORY,
	GRAPI_ERROR_RESV_NOT_SENT,
	GRAPI_ERROR_PATH_NOT_SENT,
	GRAPI_ERROR_DOWN_NOT_SENT,
	GRAPI_ERROR_TEAR_NOT_SENT,
	GRAPI_ERROR_ERROR_NOT_SENT,
	GRAPI_ERROR_CONFIRM_NOT_SENT,
	GRAPI_ERROR_ERO_NOT_ADDED,
	GRAPI_ERROR_GENERIC
} grapiErrorCode_t;

#define GRAPIERR2STRING(X)                              \
  (((X) == GRAPI_ERROR_NONE                        ) ?  \
   "NONE"                                            :  \
  (((X) == GRAPI_ERROR_INVALID_PARAMETER           ) ?  \
   "INVALID_PARAMETER"                               :  \
  (((X) == GRAPI_ERROR_INSTANCE_NOT_FOUND          ) ?  \
   "INSTANCE_NOT_FOUND"                              :  \
  (((X) == GRAPI_ERROR_INSTANCE_ALREADY_EXISTS     ) ?  \
   "INSTANCE_ALREADY_EXISTS"                         :  \
  (((X) == GRAPI_ERROR_INSTANCE_UNI_EXISTS         ) ?  \
   "INSTANCE_UNI_EXISTS"                             :  \
  (((X) == GRAPI_ERROR_INSTANCE_ENNI_EXISTS        ) ?  \
   "INSTANCE_ENNI_EXISTS"                            :  \
  (((X) == GRAPI_ERROR_INSTANCE_EXISTS             ) ?  \
   "INSTANCE_EXISTS"                                 :	\
  (((X) == GRAPI_ERROR_LSP_NOT_FOUND               ) ?  \
   "LSP_NOT_FOUND"                                   :  \
  (((X) == GRAPI_ERROR_LSP_NOT_CREATED             ) ?  \
   "LSP_NOT_CREATED"                                 :  \
  (((X) == GRAPI_ERROR_LSP_NOT_DESTROYED           ) ?  \
   "LSP_NOT_DESTROYED"                               :  \
  (((X) == GRAPI_ERROR_LSP_NOT_ENABLED             ) ?  \
   "LSP_NOT_ENABLED"                                 :  \
  (((X) == GRAPI_ERROR_LSP_NOT_DISABLED            ) ?  \
   "LSP_NOT_DISABLED"                                :  \
  (((X) == GRAPI_ERROR_LSP_NOT_FORCEDUP            ) ?  \
   "LSP_NOT_FORCED_UP"                               :  \
  (((X) == GRAPI_ERROR_LSP_NOT_FORCEDDOWN          ) ?  \
   "LSP_NOT_FORCED_DOWN"                             :  \
  (((X) == GRAPI_ERROR_LSP_NOT_CONFIGURED          ) ?  \
   "LSP_NOT_CONFIGURED"                              :  \
  (((X) == GRAPI_ERROR_LSP_ALREADY_EXISTS          ) ?  \
   "LSP_ALREADY_EXISTS"                              :  \
  (((X) == GRAPI_ERROR_LSP_ERO_SOBJ_NOT_FOUND      ) ?  \
   "ERO_SUBOBJECT_NOT_FOUND"                         :  \
  (((X) == GRAPI_ERROR_LSP_ERO_SOBJ_ALREADY_EXISTS ) ?  \
   "ERO_SUBOBJECT_ALREADY_EXISTS"                    :  \
  (((X) == GRAPI_ERROR_NO_MEMORY                   ) ?  \
   "INSUFFICIENT_MEMORY"                             :  \
  (((X) == GRAPI_ERROR_RESV_NOT_SENT               ) ?  \
   "RESV_NOT_SENT"                                   :  \
  (((X) == GRAPI_ERROR_PATH_NOT_SENT               ) ?  \
   "PATH_NOT_SENT"                                   :  \
  (((X) == GRAPI_ERROR_DOWN_NOT_SENT               ) ?  \
   "DOWN_NOT_SENT"                                   :  \
  (((X) == GRAPI_ERROR_TEAR_NOT_SENT               ) ?  \
   "TEAR_NOT_SENT"                                   :  \
  (((X) == GRAPI_ERROR_ERROR_NOT_SENT              ) ?  \
   "ERROR_NOT_SENT"                                  :  \
  (((X) == GRAPI_ERROR_CONFIRM_NOT_SENT            ) ?  \
   "CONFIRM_NOT_SENT"                                :  \
  (((X) == GRAPI_ERROR_ERO_NOT_ADDED               ) ?  \
   "ERO_NOT_ADDED"                                   :  \
  (((X) == GRAPI_ERROR_GENERIC                     ) ?  \
   "GENERIC"                                         :  \
   "<unrecognized>")))))))))))))))))))))))))))

namespace RSVP_G2RAPI {

	grapiErrorCode_t  lspCreate(const lsp_ident_t &      key,
				    const std::string &      sessionName,
				    const call_ident_t &     callIdent,
				    const net_res_spec_t &   iRes,
				    const net_res_spec_t &   eRes,
				    const lsp_info_t &       lspInfo,
				    const recovery_info_t &  recoveryInfo,
				    std::string &            resp);

	grapiErrorCode_t  lspDestroy(const lsp_ident_t &    key,
				     std::string &          resp);

	grapiErrorCode_t  lspEnable(const lsp_ident_t &     key,
				    std::string &           resp);

	grapiErrorCode_t  lspDisable(const lsp_ident_t &    key,
				     std::string &          resp);

	grapiErrorCode_t  lspEroAttach(const lsp_ident_t &              key,
				       const std::list<lsp_ero_sobj_t>  ero,
				       std::string &                    resp);

	grapiErrorCode_t  lspEroDetach(const lsp_ident_t &              key,
				       const std::list<lsp_ero_sobj_t>  ero,
				       std::string &                    resp);

	grapiErrorCode_t  lspSendResv(const lsp_ident_t &    key,
				      std::string &          resp);

	grapiErrorCode_t  lspSendConfirm(const lsp_ident_t & key,
					 std::string &       resp);


	grapiErrorCode_t  lspForceUp(const lsp_ident_t &         key,
				     std::string &               resp);

	grapiErrorCode_t  lspForceDown(const lsp_ident_t &       key,
				       std::string &             resp);

	grapiErrorCode_t  lspXConnCompleted(const lsp_ident_t &  key,
					    std::string &        resp);

	grapiErrorCode_t  lspXConnFailed(const lsp_ident_t &     key,
					 std::string &           resp);

	grapiErrorCode_t  getLsps(std::list<lsp_ident_t> &       lsps,
				  std::string &                  resp);

	grapiErrorCode_t  getLspDetails(const lsp_ident_t &      key,
					call_ident_t &           callIdent,
					lsp_info_t &             lspInfo,
					recovery_info_t &        recoveryInfo,
					std::string &            resp);

	grapiErrorCode_t getLspEro(const lsp_ident_t &         key,
				   std::list<lsp_ero_sobj_t> & ero,
				   std::string &               resp);

	grapiErrorCode_t getLspRro(const lsp_ident_t &         key,
				   std::list<lsp_ero_sobj_t> & rro,
				   std::string &               resp);

	grapiErrorCode_t getLspXro(const lsp_ident_t &         key,
				   std::list<lsp_ero_sobj_t> & xro,
				   std::string &               resp);
};

namespace RSVP_GXRAPI {

	grapiErrorCode_t  lspCreate(const lsp_ident_t &      key,
				    const std::string &      sessionName,
				    const call_ident_t &     callIdent,
				    const call_info_t &      callInfo,
				    const lsp_info_t &       lspInfo,
				    const recovery_info_t &  recoveryInfo,
				    const std::string &      aaaInfo,
				    std::string &            resp);

	grapiErrorCode_t  lspDestroy(const lsp_ident_t &     key,
				     std::string &           resp);

	grapiErrorCode_t  lspSetTna(const lsp_ident_t &      key,
				    const net_res_spec_t &   tnaRes,
				    bool                     ingress,
				    std::string &            resp);

	grapiErrorCode_t  lspSetGnsTna(const lsp_ident_t &     key,
				       const grid_res_spec_t & gnsTna,
				       bool                    ingress,
				       std::string &           resp);

	grapiErrorCode_t  lspGetTna(const lsp_ident_t &      key,
				    bool                     ingress,
				    net_res_spec_t &         tnaRes,
				    std::string &            resp);

	grapiErrorCode_t  lspGetGnsTna(const lsp_ident_t &   key,
				       bool                  ingress,
				       grid_res_spec_t &     gnsTna,
				       std::string &         resp);

	grapiErrorCode_t  lspGetTnResource(const lsp_ident_t & key,
					   net_res_spec_t &    tnRes,
					   std::string &       resp);

	grapiErrorCode_t  lspEroAttach(const lsp_ident_t &              key,
				       const std::list<lsp_ero_sobj_t>  ero,
				       std::string &                    resp);

	grapiErrorCode_t  lspSendPath(const lsp_ident_t &    key,
				      std::string &          resp);

	grapiErrorCode_t  lspSendResv(const lsp_ident_t &    key,
				      std::string &          resp);

	grapiErrorCode_t  lspSendConfirm(const lsp_ident_t & key,
					 std::string &       resp);

	grapiErrorCode_t  lspSendDown(const lsp_ident_t &    key,
				      std::string &          resp);

	grapiErrorCode_t  lspSendTear(const lsp_ident_t &    key,
				      std::string &          resp);

	grapiErrorCode_t  lspSendError(const lsp_ident_t &    key,
				       const error_info_t &   error,
				       std::string &          resp);


// 	grapiErrorCode_t  lspForceUp(const lsp_ident_t &     key,
// 				     std::string &           resp);

// 	grapiErrorCode_t  lspForceDown(const lsp_ident_t &   key,
// 				       std::string &         resp);
};

namespace RSVP_API {

	RSVP::G2RSVPTE *  g2rsvpteGet(std::string & resp);
	grapiErrorCode_t  g2rsvpteStart(adj_type_t    adjType,
					std::string & resp,
					bool          isClient = false,
					bool          isCore = false);
	grapiErrorCode_t  g2rsvpteStop(adj_type_t    adjType,
				       std::string & resp);
};

extern "C" {
	int destroyAdvRsrvLsp(struct thread * thr);
	int lspFSMtimerElapsed(struct thread * thr);
#ifdef CRANKBACK_ON
	int lspNextCbackAttemptStart(struct thread * thr);
#endif // CRANKBACK_ON
}

#endif // __G2RSVP_APIS_H__
