//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "zebra.h"

#if HAVE_OMNIORB

#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "log.h"
#include "g2rsvpte_trace.h"
#include "g2rsvpted.h"
#include "g2rsvpte_dm.h"
#include "g2rsvpte_apis.h"
#include "g2rsvpte_utils.h"
#include "g2rsvpte_config.h"
#include "idl/g2rsvpte.hh"
#include "idl/lrm.hh"
#include "idl/tnrc.hh"
#include "idl/RecoveryController.hh"
#include "idl/CallController.hh"
#include "idl/PersistencyController.hh"
#include "idl/g2pcera.hh"

#include <iostream>
#include <string>
#include <stdlib.h>
#include <stdexcept>

using namespace std;

///////////////////////////////////////////////////////////////////////////////
//                                TnrControl_i                               //
///////////////////////////////////////////////////////////////////////////////

class TnrControl_i : public POA_g2rsvpte::TnrControl,
		     public PortableServer::RefCountServantBase
{
public:
	inline TnrControl_i();
	virtual ~TnrControl_i();

	void
	actionResponse(Types::uint32           cookie,
		       g2mplsTypes::tnrcResult result,
		       CORBA::Long             responseCtxt);

	void
	actionNotify(Types::uint32             cookie,
		     //some event parameter
		     CORBA::Long               notifyCtxt);
};

TnrControl_i::TnrControl_i()
{
}

TnrControl_i::~TnrControl_i()
{
}

void
TnrControl_i::actionResponse(Types::uint32           cookie,
			     g2mplsTypes::tnrcResult result,
			     CORBA::Long             responseCtxt)
{
	STACK_LOCK();

	uint32_t         tnrcCookie;
	std::string      responseString;
	RSVP::LSP *      lsp;
	lsp_ident_t      ident;
	grapiErrorCode_t ret;
	std::string      err;
	RSVP::G2RSVPTE * proto;
	adj_type_t       adjType;

	adjType = INNI;

	tnrcCookie      = cookie;
	responseString << result;

	G2RSVPTE_DBG("TNRC pseudo-synch actionResponse for action "
		     "with cookie %d: %s",
		     tnrcCookie, responseString.c_str());

	//	lsp   = (RSVP::LSP *) responseCtxt;
	proto = RSVP_MASTER.getProtoInstance(adjType);
	if (!proto) {
		G2RSVPTE_ERR("ERROR: instance not found");
		STACK_UNLOCK();
		throw(Types::InternalProblems());
	}

//	if (!proto->checkLSP(lsp)) {
//		G2RSVPTE_WRN("LSP already deleted. No event will be posted");
//		STACK_UNLOCK();
//		throw(Types::InternalProblems());
//	}

	lsp = proto->findLSP((uint32_t) responseCtxt);

	if (!lsp) {
		G2RSVPTE_ERR("LSP not found for context 0x%08x (%d)..."
			     "No event will be posted",
			     (uint32_t)responseCtxt, (uint32_t)responseCtxt);
		STACK_UNLOCK();
		throw(Types::InternalProblems());
	}
	assert(lsp);

	ident = lsp->getIdent();

	switch (result) {
		case g2mplsTypes::TNRC_RESULT_MAKEXC_NOERROR:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_NOERROR: {

			ret = RSVP_G2RAPI::lspXConnCompleted(ident, err);
			if (ret != GRAPI_ERROR_NONE) {
				G2RSVPTE_WRN("Cannot post xc_completed event "
					     "to G2.RSVP-TE LSP with "
					     "id %d to destination router %s "
					     "(err: code=%s, reason='%s')",
					     ident.lsp_id,
					     numb_htoa(ident.dst_nid),
					     GRAPIERR2STRING(ret), err.c_str());
				STACK_UNLOCK();
				throw(Types::InternalProblems());
			}
			G2RSVPTE_DBG("Post xc_completed event to LSP "
				     "with id %d to destination "
				     "router %s successfully sent",
				     ident.lsp_id,
				     numb_htoa(ident.dst_nid));
			break;
		}
		case g2mplsTypes::TNRC_RESULT_MAKEXC_EQPTDOWN:
		case g2mplsTypes::TNRC_RESULT_MAKEXC_PARAMERROR:
		case g2mplsTypes::TNRC_RESULT_MAKEXC_NOTCAPABLE:
		case g2mplsTypes::TNRC_RESULT_MAKEXC_BUSYRESOURCES:
		case g2mplsTypes::TNRC_RESULT_MAKEXC_INTERNALERROR:
		case g2mplsTypes::TNRC_RESULT_MAKEXC_GENERICERROR:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_EQPTDOWN:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_PARAMERROR:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_NOTCAPABLE:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_BUSYRESOURCES:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_INTERNALERROR:
		case g2mplsTypes::TNRC_RESULT_RESERVEXC_GENERICERROR: {

			ret = RSVP_G2RAPI::lspXConnFailed(ident, err);
			if (ret != GRAPI_ERROR_NONE) {
				G2RSVPTE_WRN("Cannot post xc_failed event "
					     "to G2.RSVP-TE LSP with "
					     "id %d to destination router %s "
					     "(err: code=%s, reason='%s')",
					     ident.lsp_id,
					     numb_htoa(ident.dst_nid),
					     GRAPIERR2STRING(ret), err.c_str());
				STACK_UNLOCK();
				throw(Types::InternalProblems());
			}
			G2RSVPTE_DBG("Post xc_failed event to LSP "
				     "with id %d to destination "
				     "router %s successfully sent",
				     ident.lsp_id,
				     numb_htoa(ident.dst_nid));
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_NOERROR: {
#ifdef CRANKBACK_ON
			if (lsp->cbackWaitReleaseRes()) {
				// post an event to start next crankback
				// attempt signalling
				thread_add_event(RSVP_MASTER.getMaster(),
						 lspNextCbackAttemptStart,
						 lsp,
						 0);
				G2RSVPTE_DBG("Posted event to start "
					     "next crankback attempt "
					     "after release resource");
				lsp->cbackWaitReleaseRes(false);
			}
#endif // CRANKBACK_ON
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_NOERROR: {
			//TMP - DON't DO ANYTHING - FIX
			break;
		}
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_EQPTDOWN:
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_PARAMERROR:
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_NOTCAPABLE:
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_BUSYRESOURCES:
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_INTERNALERROR:
		case g2mplsTypes::TNRC_RESULT_DESTROYXC_GENERICERROR: {
#ifdef CRANKBACK_ON
			if (lsp->cbackWaitReleaseRes()) {
				G2RSVPTE_WRN("Release resource failed. "
					     "Next crankback attempt could fail");
			}
			thread_add_event(RSVP_MASTER.getMaster(),
					 lspNextCbackAttemptStart,
					 lsp,
					 0);
			G2RSVPTE_DBG("Posted event to start "
				     "next crankback attempt "
				     "after release resource");
			lsp->cbackWaitReleaseRes(false);
#endif // CRANKBACK_ON
			break;
		}
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_EQPTDOWN:
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_PARAMERROR:
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_NOTCAPABLE:
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_BUSYRESOURCES:
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_INTERNALERROR:
		case g2mplsTypes::TNRC_RESULT_UNRESERVEXC_GENERICERROR: {
			//TMP - DON't DO ANYTHING - FIX
			break;
		}
		default:
			break;
	}

	STACK_UNLOCK();
}

void
TnrControl_i::actionNotify(Types::uint32             cookie,
			   //some event parameter
			   CORBA::Long               notifyCtxt)
{
	STACK_LOCK();

	uint32_t  tnrcCookie;

	tnrcCookie = cookie;

	G2RSVPTE_DBG("Received TNRC async actionNotify for action "
		     "with cookie %d", tnrcCookie);

	STACK_UNLOCK();
}

///////////////////////////////////////////////////////////////////////////////
//                                NorthBound_i                               //
///////////////////////////////////////////////////////////////////////////////

class NorthBound_i : public POA_g2rsvpte::NorthBound,
		     public PortableServer::RefCountServantBase
{
public:
	inline NorthBound_i();
	virtual ~NorthBound_i();

	CORBA::Boolean
	lspCreate(const g2mplsTypes::lspIdent &       lspId,
		  const g2mplsTypes::callIdent &      callId,
		  const g2mplsTypes::lspParams &      lspInfo,
		  const g2mplsTypes::recoveryParams & recoveryInfo,
		  const g2mplsTypes::tnResource &     iTnr,
		  const g2mplsTypes::tnResource &     eTnr,
		  CORBA::Boolean                      setup);

	CORBA::Boolean
	lspAddEroPart(const g2mplsTypes::lspIdent &   lspId,
		      const g2mplsTypes::eroSeq &     eroItem);

	CORBA::Boolean
	lspEnable(const g2mplsTypes::lspIdent &       lspId);

	CORBA::Boolean
	lspDisable(const g2mplsTypes::lspIdent &      lspId);

	CORBA::Boolean
	lspDestroy(const g2mplsTypes::lspIdent &      lspId);

	CORBA::Boolean
	lspSetUp(const g2mplsTypes::lspIdent &        lspId);

	CORBA::Boolean
	lspSetDown(const g2mplsTypes::lspIdent &      lspId);

	g2rsvpte::NorthBound::lspIdentSeq * getLsps();

	CORBA::Boolean
	lspGetDetails(const g2mplsTypes::lspIdent &   lspId,
		      g2mplsTypes::callIdent &        callId,
		      g2mplsTypes::lspParams &        lspInfo,
		      g2mplsTypes::recoveryParams &   recoveryInfo,
		      g2mplsTypes::statesBundle &     states);

	g2mplsTypes::eroSeq *
	lspGetEro(const g2mplsTypes::lspIdent &       lspId);

	g2mplsTypes::eroSeq *
	lspGetRro(const g2mplsTypes::lspIdent &       lspId);

	g2mplsTypes::eroSeq *
	lspGetXro(const g2mplsTypes::lspIdent &       lspId);

};

NorthBound_i::NorthBound_i()
{
}

NorthBound_i::~NorthBound_i()
{
}

CORBA::Boolean
NorthBound_i::lspCreate(const g2mplsTypes::lspIdent &       lspId,
			const g2mplsTypes::callIdent &      callId,
			const g2mplsTypes::lspParams &      lspInfo,
			const g2mplsTypes::recoveryParams & recoveryInfo,
			const g2mplsTypes::tnResource &     iTnr,
			const g2mplsTypes::tnResource &     eTnr,
			CORBA::Boolean                      setup)
{
	STACK_LOCK();

	grapiErrorCode_t                res;
	lsp_ident_t                     key;
	std::string                     sessionName;
	net_res_spec_t                  iRes;
	net_res_spec_t                  eRes;
	std::string                     resp;
	call_ident_t                    callIdent;
	lsp_info_t                      lspParams;
	recovery_info_t                 recoveryParams;
	g2mplsTypes::lspIdent_var       lspIdTmp;
	g2mplsTypes::callIdent_var      callIdTmp;
	g2mplsTypes::lspParams_var      lspInfoTmp;
	g2mplsTypes::recoveryParams_var recoveryInfoTmp;
	g2mplsTypes::tnResource_var     iTnrTmp;
	g2mplsTypes::tnResource_var     eTnrTmp;

	G2RSVPTE_DBG("RC is requesting to create an LSP");

	try {
		lspIdTmp = lspId;
		key << lspIdTmp;

		callIdTmp  = callId;
		callIdent << callIdTmp;

		lspInfoTmp = lspInfo;
		lspParams << lspInfoTmp;

		recoveryInfoTmp = recoveryInfo;
		recoveryParams << recoveryInfoTmp;

		sessionName = "";

		iTnrTmp  = iTnr;
		iRes    << iTnrTmp;
		eTnrTmp  = eTnr;
		eRes    << eTnrTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not created: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not created: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_G2RAPI::lspCreate(key,
				     sessionName,
				     callIdent,
				     iRes,
				     eRes,
				     lspParams,
				     recoveryParams,
				     resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("LSP successfully created");
		STACK_UNLOCK();
		return true;
	} else {
		G2RSVPTE_ERR("LSP not created: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not created: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}


CORBA::Boolean
NorthBound_i::lspAddEroPart(const g2mplsTypes::lspIdent & lspId,
			    const g2mplsTypes::eroSeq &   eroItem)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;
	std::list<lsp_ero_sobj_t> ero;
	g2mplsTypes::eroItem_var  eroItTmp;

	G2RSVPTE_DBG("RC is requesting to add ERO to LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("ERO not added: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("ERO not added: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	ero.clear();

	for (size_t i = 0; i < eroItem.length(); i++) {
		eroItTmp   = eroItem[i];
		eroItemProcess(eroItTmp, ero);
	}

	res = RSVP_G2RAPI::lspEroAttach(key, ero, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("ERO successfully added");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("ERO not added to LSP: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("ERO not added to LSP: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("ERO not added to LSP: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
NorthBound_i::lspEnable(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("RC is requesting to enable LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not enabled: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not enabled: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_G2RAPI::lspEnable(key, resp);

	if (res == GRAPI_ERROR_NONE) {
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("LSP not enabled: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("LSP not enabled: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not enabled: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
NorthBound_i::lspDisable(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("RC is requesting to disable LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not disabled: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not disabled: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_G2RAPI::lspDisable(key, resp);

	if (res == GRAPI_ERROR_NONE) {
		STACK_UNLOCK();
		G2RSVPTE_DBG("LSP successfully disabled");
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("LSP not disabled: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("LSP not disabled: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not disabled: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
NorthBound_i::lspDestroy(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("RC is requesting to destroy LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not destroyed: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not destroyed: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_G2RAPI::lspDestroy(key, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("LSP destroy request successfully got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("LSP not destroyed: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("LSP not destroyed: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not destroyed: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
NorthBound_i::lspSetUp(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("RC is requesting to set up LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not set up: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not set up: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_G2RAPI::lspForceUp(key, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("LSP set up request successfully got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("LSP not set up: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("LSP not set up: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not set up: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
NorthBound_i::lspSetDown(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("RC is requesting to set down LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not set down: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not set down: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_G2RAPI::lspForceDown(key, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("LSP set down request successfully got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("LSP not set down: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("LSP not set down: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not set down: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

g2rsvpte::NorthBound::lspIdentSeq *
NorthBound_i::getLsps(void)
{
	STACK_LOCK();

	grapiErrorCode_t                       res;
	std::string                            resp;
	std::list<lsp_ident_t>                 lsps;
	std::list<lsp_ident_t>::const_iterator iter;
	lsp_ident_t                            lspIdent;
	g2mplsTypes::lspIdent_var              lspId;

	G2RSVPTE_DBG("RC is requesting to get LSPs");

	lsps.clear();

	res = RSVP_G2RAPI::getLsps(lsps, resp);

	if (res == GRAPI_ERROR_NONE) {
		// build the sequence
		g2rsvpte::NorthBound::lspIdentSeq_var lspSeq;
		{
			g2rsvpte::NorthBound::lspIdentSeq * tmp;

			tmp = new g2rsvpte::NorthBound::lspIdentSeq(lsps.size());
			if (!tmp) {
				STACK_UNLOCK();
				throw(Types::InternalProblems());
			}
			lspSeq = tmp;
		}
		lspSeq->length(lsps.size());

		int i = 0;
		for (iter = lsps.begin(); iter != lsps.end(); iter++) {
			lspIdent = *iter;
			lspId    << lspIdent;

			lspSeq[i] = lspId;

			i++;
		}
		G2RSVPTE_DBG("LSPs got");

		STACK_UNLOCK();
		return lspSeq._retn();
	} else {
		G2RSVPTE_ERR("Cannot get LSPs: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::InternalProblems());
	}

	G2RSVPTE_ERR("Cannot get LSPs: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
NorthBound_i::lspGetDetails(const g2mplsTypes::lspIdent & lspId,
			    g2mplsTypes::callIdent &      callId,
			    g2mplsTypes::lspParams &      lspInfo,
			    g2mplsTypes::recoveryParams & recoveryInfo,
			    g2mplsTypes::statesBundle &   states)
{
	STACK_LOCK();

	grapiErrorCode_t                 res;
	std::string                      resp;
	lsp_ident_t                      key;
	g2mplsTypes::lspIdent_var        lspIdTmp;
	g2mplsTypes::callIdent_var       callIdTmp;
	g2mplsTypes::lspParams_var       lspInfoTmp;
	g2mplsTypes::recoveryParams_var  recoveryInfoTmp;
	call_ident_t                     callIdent;
	recovery_info_t                  recoveryParams;
	lsp_info_t                       lspParams;

	G2RSVPTE_DBG("RC is requesting to get LSP details");

	try {
		lspIdTmp = lspId;
	        key     << lspIdTmp;

		res = RSVP_G2RAPI::getLspDetails(key,
						 callIdent,
						 lspParams,
						 recoveryParams,
						 resp);

		if (res == GRAPI_ERROR_NONE) {

			callIdTmp << callIdent;
			callId     = callIdTmp;

			lspInfoTmp << lspParams;
			lspInfo     = lspInfoTmp;

			recoveryInfoTmp << recoveryParams;
			recoveryInfo     = recoveryInfoTmp;

			//TMP-FIXME
			states.opState  = g2mplsTypes::OPERSTATE_UP;
			states.admState = g2mplsTypes::ADMINSTATE_ENABLED;
			//
			G2RSVPTE_DBG("LSP details got");
			STACK_UNLOCK();
			return true;
		} else {
			G2RSVPTE_ERR("Cannot get LSP details: %s", resp.c_str());
			STACK_UNLOCK();
			throw(Types::InternalProblems());
		}
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Cannot get LSP details: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get LSP details: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get LSP details: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

g2mplsTypes::eroSeq *
NorthBound_i::lspGetEro(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t                    res;
	std::string                         resp;
	lsp_ident_t                         key;
	g2mplsTypes::lspIdent_var           lspIdTmp;
	std::list<lsp_ero_sobj_t>           ero;
	std::list<lsp_ero_sobj_t>::iterator it;

	G2RSVPTE_DBG("Someone is requesting to get LSP ERO details");

	try {
		lspIdTmp = lspId;
	        key     << lspIdTmp;

		ero.clear();
		res = RSVP_G2RAPI::getLspEro(key,
					     ero,
					     resp);

		if (res == GRAPI_ERROR_NONE) {
			g2mplsTypes::eroSeq_var eroItem;
			{
				g2mplsTypes::eroSeq * tmp;

				tmp = new g2mplsTypes::eroSeq(ero.size());
				if (!tmp) {
					G2RSVPTE_ERR("Impossible to allocate "
						     "a new eroSeq sequence");
					return false;
				}
				eroItem = tmp;
			}

			eroItem->length(ero.size());
			int i = 0;
			lsp_ero_sobj_t sObj;
			for (it = ero.begin(); it != ero.end(); it++) {
				memset(&sObj, 0, sizeof(sObj));
				sObj = *it;
				eroItem[i] = getEroItem(sObj);
				i++;
			}

			G2RSVPTE_DBG("LSP ERO details got");
			STACK_UNLOCK();
			return eroItem._retn();
		} else {
			G2RSVPTE_ERR("Cannot get LSP ERO details: %s", resp.c_str());
			STACK_UNLOCK();
			throw(Types::CannotFetch());
		}
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Cannot get LSP ERO details: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get LSP ERO details: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get LSP ERO details: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}


g2mplsTypes::eroSeq *
NorthBound_i::lspGetRro(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t                    res;
	std::string                         resp;
	lsp_ident_t                         key;
	g2mplsTypes::lspIdent_var           lspIdTmp;
	std::list<lsp_ero_sobj_t>           rro;
	std::list<lsp_ero_sobj_t>::iterator it;

	G2RSVPTE_DBG("Someone is requesting to get LSP RRO details");

	try {
		lspIdTmp = lspId;
	        key     << lspIdTmp;

		rro.clear();
		res = RSVP_G2RAPI::getLspRro(key,
					     rro,
					     resp);

		if (res == GRAPI_ERROR_NONE) {
			g2mplsTypes::eroSeq_var rroItem;
			{
				g2mplsTypes::eroSeq * tmp;

				tmp = new g2mplsTypes::eroSeq(rro.size());
				if (!tmp) {
					G2RSVPTE_ERR("Impossible to allocate "
						     "a new eroSeq sequence");
					STACK_UNLOCK();
					return false;
				}
				rroItem = tmp;
			}

			rroItem->length(rro.size());
			int i = 0;
			lsp_ero_sobj_t sObj;
			for (it = rro.begin(); it != rro.end(); it++) {
				memset(&sObj, 0, sizeof(sObj));
				sObj = *it;
				rroItem[i] = getEroItem(sObj);
				i++;
			}

			G2RSVPTE_DBG("LSP RRO details got");
			STACK_UNLOCK();
			return rroItem._retn();
		} else {
			G2RSVPTE_ERR("Cannot get LSP RRO details: %s", resp.c_str());
			STACK_UNLOCK();
			throw(Types::CannotFetch());
		}
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Cannot get LSP RRO details: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get LSP RRO details: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get LSP RRO details: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

g2mplsTypes::eroSeq *
NorthBound_i::lspGetXro(const g2mplsTypes::lspIdent & lspId)
{
	STACK_LOCK();

	grapiErrorCode_t                    res;
	std::string                         resp;
	lsp_ident_t                         key;
	g2mplsTypes::lspIdent_var           lspIdTmp;
	std::list<lsp_ero_sobj_t>           xro;
	std::list<lsp_ero_sobj_t>::iterator it;

	G2RSVPTE_DBG("Someone is requesting to get LSP exclude ERO details");

#ifdef CRANKBACK_ON
	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;

		xro.clear();
		res = RSVP_G2RAPI::getLspXro(key,
					     xro,
					     resp);

		if (res == GRAPI_ERROR_NONE) {
			g2mplsTypes::eroSeq_var   exclEro;

			exclEro = getExclEroSeq(xro);

			G2RSVPTE_DBG("LSP exclude ERO details got");
			STACK_UNLOCK();
			return exclEro._retn();
		} else {
			G2RSVPTE_ERR("Cannot get LSP exclude ERO details: %s", resp.c_str());
			STACK_UNLOCK();
			throw(Types::CannotFetch());
		}
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Cannot get LSP exclude ERO details: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get LSP exclude ERO details: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get LSP exclude ERO details: error=%d err-msg: %s",
		     res, resp.c_str());
	STACK_UNLOCK();
	throw(Types::InternalProblems());
#else
	G2RSVPTE_ERR("Cannot get LSP exclude ERO details: CRANKBACK is disabled");
	STACK_UNLOCK();
	return false;
#endif // CRANKBACK_ON
}

///////////////////////////////////////////////////////////////////////////////
//                                LspSigNorth_i                              //
///////////////////////////////////////////////////////////////////////////////

class LspSigNorth_i : public POA_g2rsvpte::LspSigNorth,
		      public PortableServer::RefCountServantBase
{
public:
	inline LspSigNorth_i();
	virtual ~LspSigNorth_i();

	CORBA::Boolean
	lspCreate(const g2mplsTypes::lspIdent &		        lspId,
		  const g2mplsTypes::callIdent &		callId,
		  const g2mplsTypes::callParams	&	        callInfo,
		  const g2mplsTypes::lspParams &		lspInfo,
		  const g2mplsTypes::recoveryParams &	        recoveryInfo,
		  const g2mplsTypes::aaaParams &		aaaInfo,
		  const g2mplsTypes::actorInfo &                actor);

	CORBA::Boolean
	lspDestroy(const g2mplsTypes::lspIdent &		lspId);

	CORBA::Boolean
	lspSetTna(const g2mplsTypes::lspIdent &		        lspId,
		  const g2mplsTypes::resourcePosition	        pos,
		  const g2mplsTypes::tnaResource &	tnaRes);

	CORBA::Boolean
	lspSetGnsTna(const g2mplsTypes::lspIdent &		lspId,
		     const g2mplsTypes::resourcePosition	pos,
		     const g2mplsTypes::gridParams &		gnsTna);

	CORBA::Boolean
	lspGetTna(const g2mplsTypes::lspIdent &			lspId,
		  const  g2mplsTypes::resourcePosition	pos,
		  g2mplsTypes::tnaResource &		        tnaRes);

	CORBA::Boolean
	lspGetGnsTna(const g2mplsTypes::lspIdent &		lspId,
		     const  g2mplsTypes::resourcePosition	pos,
		     g2mplsTypes::gridParams_out                gnsTna);

	CORBA::Boolean
	lspAddEroPart(const g2mplsTypes::lspIdent &		lspId,
		      const g2mplsTypes::eroSeq	&		eroItem);

	CORBA::Boolean
	lspGetTnResource(const g2mplsTypes::lspIdent &		lspId,
			 g2mplsTypes::tnResource &		tnRes);

	CORBA::Boolean
	lspSendMessage(const g2mplsTypes::lspIdent &		lspId,
		       const g2mplsTypes::callMessageType	msgType);

	CORBA::Boolean
	lspSendError(const g2mplsTypes::lspIdent &              lspId,
		     const g2mplsTypes::errorInfo &             error);

};

LspSigNorth_i::LspSigNorth_i()
{
}

LspSigNorth_i::~LspSigNorth_i()
{
}

CORBA::Boolean
LspSigNorth_i::lspCreate(const g2mplsTypes::lspIdent &	     lspId,
			 const g2mplsTypes::callIdent &	     callId,
			 const g2mplsTypes::callParams	&    callInfo,
			 const g2mplsTypes::lspParams &	     lspInfo,
			 const g2mplsTypes::recoveryParams & recoveryInfo,
			 const g2mplsTypes::aaaParams &	     aaaInfo,
			 const g2mplsTypes::actorInfo &      actor)
{
	STACK_LOCK();

	grapiErrorCode_t                res;
	lsp_ident_t                     key;
	std::string                     sessionName;
	std::string                     resp;
	std::string                     aaaParams;
	call_ident_t                    callIdent;
	call_info_t                     callParams;
	lsp_info_t                      lspParams;
	recovery_info_t                 recoveryParams;
	g2mplsTypes::lspIdent_var       lspIdTmp;
	g2mplsTypes::callIdent_var      callIdTmp;
	g2mplsTypes::callParams_var     callInfoTmp;
	g2mplsTypes::lspParams_var      lspInfoTmp;
	g2mplsTypes::recoveryParams_var recoveryInfoTmp;

	G2RSVPTE_DBG("xCC is requesting to create an LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;

		callIdTmp  = callId;
		callIdent << callIdTmp;

		callInfoTmp  = new g2mplsTypes::callParams(callInfo);
		callParams  << callInfoTmp;

		lspInfoTmp = lspInfo;
		lspParams << lspInfoTmp;

		recoveryInfoTmp = recoveryInfo;
		recoveryParams << recoveryInfoTmp;

		aaaParams = aaaInfo.authzToken;

		sessionName = "";
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not created: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not created: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_GXRAPI::lspCreate(key,
				     sessionName,
				     callIdent,
				     callParams,
				     lspParams,
				     recoveryParams,
				     aaaParams,
				     resp);

	if (res == GRAPI_ERROR_NONE) {
		STACK_UNLOCK();
		G2RSVPTE_DBG("LSP successfully created");
		return true;
	} else {
		G2RSVPTE_ERR("LSP not created: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not created: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspDestroy(const g2mplsTypes::lspIdent &	lspId)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("xCC is requesting to destroy an LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("LSP not destroyed: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("LSP not destroyed: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_GXRAPI::lspDestroy(key, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("LSP destroy request successfully got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("LSP not destroyed: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("LSP not destroyed: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("LSP not destroyed: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspSetTna(const g2mplsTypes::lspIdent &	      lspId,
			 const g2mplsTypes::resourcePosition  pos,
			 const g2mplsTypes::tnaResource &     tnaRes)
{
	STACK_LOCK();

	grapiErrorCode_t             res;
	lsp_ident_t                  key;
	std::string                  resp;
	g2mplsTypes::lspIdent_var    lspIdTmp;
	g2mplsTypes::tnaResource_var tnaResTmp;
	bool                         ingress;
	net_res_spec_t               tna;

	G2RSVPTE_DBG("xCC is requesting to set TNA for an LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;

		tnaResTmp = tnaRes;
		tna      << tnaResTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("TNA not set: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("TNA not set: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	if (pos == g2mplsTypes::RESOURCEPOSITION_INGRESS) {
		ingress = true;
	} else {
		ingress = false;
	}

	res = RSVP_GXRAPI::lspSetTna(key, tna, ingress, resp);
	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("TNA successfully set");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("TNA not set: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("TNA not set: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("TNA not set: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspSetGnsTna(const g2mplsTypes::lspIdent &        lspId,
			    const g2mplsTypes::resourcePosition  pos,
			    const g2mplsTypes::gridParams &	 gnsTna)
{
	STACK_LOCK();

	grapiErrorCode_t             res;
	lsp_ident_t                  key;
	std::string                  resp;
	g2mplsTypes::lspIdent_var    lspIdTmp;
	g2mplsTypes::gridParams_var  gnsTnaTmp;
	bool                         ingress;
	grid_res_spec_t              gridRes;

	G2RSVPTE_DBG("xCC is requesting to set Gns-TNA for an LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;

		gnsTnaTmp = new g2mplsTypes::gridParams(gnsTna);
		gridRes  << gnsTnaTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("GNS-TNA not set: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("GNS-TNA not set: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	if (pos == g2mplsTypes::RESOURCEPOSITION_INGRESS) {
		ingress = true;
	} else {
		ingress = false;
	}

	res = RSVP_GXRAPI::lspSetGnsTna(key, gridRes, ingress, resp);
	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("GNS-TNA successfully set");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("GNS-TNA not set: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("GNS-TNA not set: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("GNS-TNA not set: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}


CORBA::Boolean
LspSigNorth_i::lspGetTna(const g2mplsTypes::lspIdent &          lspId,
			 const  g2mplsTypes::resourcePosition   pos,
			 g2mplsTypes::tnaResource &		tnaRes)
{
	STACK_LOCK();

	grapiErrorCode_t             res;
	lsp_ident_t                  key;
	std::string                  resp;
	g2mplsTypes::lspIdent_var    lspIdTmp;
	g2mplsTypes::tnaResource_var tnaResTmp;
	bool                         ingress;
	net_res_spec_t               tna;

	G2RSVPTE_DBG("xCC is requesting to get TNA for an LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Impossible to get TNA: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Impossible to get TNA: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	if (pos == g2mplsTypes::RESOURCEPOSITION_INGRESS) {
		ingress = true;
	} else {
		ingress = false;
	}

	memset(&tna, 0, sizeof(tna));

	res = RSVP_GXRAPI::lspGetTna(key, ingress, tna, resp);
	if (res == GRAPI_ERROR_NONE) {
		//set out parameter
		tnaResTmp << tna;
		tnaRes     = tnaResTmp;
		G2RSVPTE_DBG("TNA successfully got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("Cannot get TNA: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("Cannot get TNA: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get TNA: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspGetGnsTna(const g2mplsTypes::lspIdent &         lspId,
			    const  g2mplsTypes::resourcePosition  pos,
			    g2mplsTypes::gridParams_out           gnsTna)
{
	STACK_LOCK();

	grapiErrorCode_t             res;
	lsp_ident_t                  key;
	std::string                  resp;
	g2mplsTypes::lspIdent_var    lspIdTmp;
	g2mplsTypes::gridParams_var  gnsTnaTmp;
	bool                         ingress;
	grid_res_spec_t              gridRes;

	G2RSVPTE_DBG("xCC is requesting to get Gns-TNA for an LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Impossible to get Gns-TNA: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Impossible to get Gns-TNA: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	if (pos == g2mplsTypes::RESOURCEPOSITION_INGRESS) {
		ingress = true;
	} else {
		ingress = false;
	}

	memset(&gridRes, 0, sizeof(gridRes));

	res = RSVP_GXRAPI::lspGetGnsTna(key, ingress, gridRes, resp);
	if (res == GRAPI_ERROR_NONE) {
		//set out parameter
		gnsTnaTmp << gridRes;
		gnsTna     = gnsTnaTmp._retn();
		G2RSVPTE_DBG("GNS-TNA successfully got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("Cannot get GNS-TNA: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("Cannot get GNS-TNA: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get GNS-TNA: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspAddEroPart(const g2mplsTypes::lspIdent & lspId,
			     const g2mplsTypes::eroSeq	&  eroItem)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;
	std::list<lsp_ero_sobj_t> ero;
	g2mplsTypes::eroItem_var  eroItTmp;

	G2RSVPTE_DBG("xCC is requesting to add ERO to LSP");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Impossible to add ERO: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Impossible to add ERO: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	ero.clear();

	for (size_t i = 0; i < eroItem.length(); i++) {
		eroItTmp   = eroItem[i];
		eroItemProcess(eroItTmp, ero);
	}

	res = RSVP_GXRAPI::lspEroAttach(key, ero, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("ERO successfully added");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("ERO not added to LSP: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("ERO not added to LSP: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("ERO not added to LSP: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());

}

CORBA::Boolean
LspSigNorth_i::lspGetTnResource(const g2mplsTypes::lspIdent & lspId,
			        g2mplsTypes::tnResource &     tnRes)
{
	STACK_LOCK();

	grapiErrorCode_t             res;
	lsp_ident_t                  key;
	std::string                  resp;
	g2mplsTypes::lspIdent_var    lspIdTmp;
	g2mplsTypes::tnResource_var  tnResTmp;
	net_res_spec_t               tnResource;

	G2RSVPTE_DBG("xCC is requesting to get TnResource");

	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Impossible to get TnResource: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Impossible to get TnResource: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_GXRAPI::lspGetTnResource(key, tnResource, resp);
	if (res == GRAPI_ERROR_NONE) {
		//set out parameter
		try {
			tnResTmp << tnResource;
			tnRes     = tnResTmp;
		} catch (...) {
			G2RSVPTE_DBG("Impossible to get TnResource");
			STACK_UNLOCK();
			return false;
		}
		G2RSVPTE_DBG("TN Resource successfully got");
		STACK_UNLOCK();
		return true;

	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("Cannot get TN Resource: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("Cannot get TN Resource: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot get TN Resource: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspSendMessage(const g2mplsTypes::lspIdent & lspId,
			      g2mplsTypes::callMessageType  msgType)
{
	STACK_LOCK();

	grapiErrorCode_t          res;
	lsp_ident_t               key;
	std::string               resp;
	g2mplsTypes::lspIdent_var lspIdTmp;

	G2RSVPTE_DBG("xCC is requesting to send message");
	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Impossible to send message: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Impossible to send message: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	switch (msgType) {
		case g2mplsTypes::CALLMESSAGETYPE_SETUPREQUEST:
			G2RSVPTE_DBG("Trying to send Path");
			res = RSVP_GXRAPI::lspSendPath(key, resp);
			break;
		case g2mplsTypes::CALLMESSAGETYPE_SETUPINDICATION:
			G2RSVPTE_DBG("Trying to send Resv");
			res = RSVP_GXRAPI::lspSendResv(key, resp);
			break;
		case g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM:
			G2RSVPTE_DBG("Trying to send ResvConf");
			res = RSVP_GXRAPI::lspSendConfirm(key, resp);
			break;
		case g2mplsTypes::CALLMESSAGETYPE_RELEASEREQUEST:
			G2RSVPTE_DBG("Trying to send a ReleaseRequest");
			res = RSVP_GXRAPI::lspSendDown(key, resp);
			break;
		case g2mplsTypes::CALLMESSAGETYPE_RELEASEINDICATION:
			G2RSVPTE_DBG("Trying to send a ReleaseIndication");
			res = RSVP_GXRAPI::lspSendTear(key, resp);
			break;
		default:
			G2RSVPTE_ERR("Message Type not supported (0x%x)", msgType);
			STACK_UNLOCK();
			return false;
	}

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("Send message request got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("Message not sent: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("Message not sent: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}

	G2RSVPTE_ERR("Cannot send message: error=%d err-msg: %s",
		     res, resp.c_str());

	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

CORBA::Boolean
LspSigNorth_i::lspSendError(const g2mplsTypes::lspIdent	 & lspId,
			    const g2mplsTypes::errorInfo & error)
{
	STACK_LOCK();

	grapiErrorCode_t           res;
	lsp_ident_t                key;
	std::string                resp;
	g2mplsTypes::lspIdent_var  lspIdTmp;
	g2mplsTypes::errorInfo_var errorTmp;
	error_info_t               errorInfo;

	G2RSVPTE_DBG("xCC is requesting to send error");
	try {
		lspIdTmp = lspId;
		key     << lspIdTmp;

		errorTmp  = error;
		errorInfo << errorTmp;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Impossible to send error: parameters error (%s)",
			     e.what());
		STACK_UNLOCK();
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Impossible to send error: unknown parameters error");
		STACK_UNLOCK();
		return false;
	}

	res = RSVP_GXRAPI::lspSendError(key, errorInfo, resp);

	if (res == GRAPI_ERROR_NONE) {
		G2RSVPTE_DBG("Send error request got");
		STACK_UNLOCK();
		return true;
	} else if (res == GRAPI_ERROR_LSP_NOT_FOUND) {
		G2RSVPTE_ERR("Cannot send error: %s", resp.c_str());
		STACK_UNLOCK();
		throw(Types::CannotFetch());
	} else {
		G2RSVPTE_ERR("Cannot send error: %s", resp.c_str());
		STACK_UNLOCK();
		return false;
	}


	G2RSVPTE_ERR("Cannot send error: error=%d err-msg: %s",
		     res, resp.c_str());
	STACK_UNLOCK();

	throw(Types::InternalProblems());
}

///////////////////////////////////////////////////////////////////////////////

TnrControl_i *                                tnrc_servant = 0;
NorthBound_i *                                nb_servant   = 0;
LspSigNorth_i *                               unirsvp_lspsignorth_servant = 0;
LspSigNorth_i *                               ennirsvp_lspsignorth_servant = 0;
LRM::Info_var                                 lrm_proxy;
TNRC::NorthBound_var                          tnrc_proxy;
RecoveryController::SouthBound_var            rc_proxy;
CallController::LspSigSouth_var               cc_ls_proxy;
CallController::Mgmt_var                      cc_mgmt_proxy;
PersistencyController::StorageMgmt_var        pc_mgmt_proxy;
PersistencyController::StorageInniRsvpLsp_var pc_innirsvplsp_proxy;
PersistencyController::StorageUniRsvpLsp_var  pc_unirsvplsp_proxy;
PersistencyController::StorageEnniRsvpLsp_var pc_ennirsvplsp_proxy;
G2PCERA::RoutingServices_var                  g2pcera_proxy;

#endif // HAVE_OMNIORB

#if HAVE_OMNIORB
int tnrc_server_setup(void)
{
	printf("Setting up TnrController CORBA server side\n");

	try {
		tnrc_servant = new TnrControl_i();
		if (!tnrc_servant) {
			throw std::string("Cannot create servant");
		}

		PortableServer::POA_var poa;
		poa = corba_poa();
		if (CORBA::is_nil(poa)) {
			throw string("Cannot get POA");
		}

		PortableServer::ObjectId_var servant_id;
		servant_id = poa->activate_object(tnrc_servant);

		CORBA::Object_var obj;
		obj = tnrc_servant->_this();
		if (CORBA::is_nil(obj)) {
			throw string("Cannot get object");
		}

		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			throw string("Cannot get ORB");
		}

		CORBA::String_var sior(orb->object_to_string(obj));
		printf("IOR = '%s'\n", (char*) sior);

		if (!corba_dump_ior(CORBA_SERVANT_TNRCONTROLLER_G2RSVPTE,
				    string(sior))) {
			throw string("Cannot dump IOR");
		}

		tnrc_servant->_remove_ref();

		PortableServer::POAManager_var poa_manager;
		poa_manager = corba_poa_manager();
		if (CORBA::is_nil(poa_manager)) {
			throw string("Cannot get POA Manager");
		}

		poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return 0;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return 0;
	} catch (omniORB::fatalException & e) {
		fprintf(stderr, "Caught omniORB::fatalException:\n");
		fprintf(stderr, "  file: %s\n", e.file());
		fprintf(stderr, "  line: %d\n", e.line());
		fprintf(stderr, "  mesg: %s\n", e.errmsg());
		return 0;
	} catch (string & e) {
		fprintf(stderr, "Caught exception: %s\n", e.c_str());
		return 0;
	} catch (...) {
		fprintf(stderr, "Caught unknown exception\n");
		return 0;
	}

	return 1;
}

int nb_server_setup(void)
{
	printf("Setting up NorthBound CORBA server side\n");

	try {
		nb_servant = new NorthBound_i();
		if (!nb_servant) {
			throw std::string("Cannot create servant");
		}

		PortableServer::POA_var poa;
		poa = corba_poa();
		if (CORBA::is_nil(poa)) {
			throw string("Cannot get POA");
		}

		PortableServer::ObjectId_var servant_id;
		servant_id = poa->activate_object(nb_servant);

		CORBA::Object_var obj;
		obj = nb_servant->_this();
		if (CORBA::is_nil(obj)) {
			throw string("Cannot get object");
		}

		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			throw string("Cannot get ORB");
		}

		CORBA::String_var sior(orb->object_to_string(obj));
		printf("IOR = '%s'\n", (char*) sior);

		if (!corba_dump_ior(CORBA_SERVANT_NORTHBOUND_G2RSVPTE,
				    string(sior))) {
			throw string("Cannot dump IOR");
		}

		nb_servant->_remove_ref();

		PortableServer::POAManager_var poa_manager;
		poa_manager = corba_poa_manager();
		if (CORBA::is_nil(poa_manager)) {
			throw string("Cannot get POA Manager");
		}

		poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return 0;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return 0;
	} catch (omniORB::fatalException & e) {
		fprintf(stderr, "Caught omniORB::fatalException:\n");
		fprintf(stderr, "  file: %s\n", e.file());
		fprintf(stderr, "  line: %d\n", e.line());
		fprintf(stderr, "  mesg: %s\n", e.errmsg());
		return 0;
	} catch (string & e) {
		fprintf(stderr, "Caught exception: %s\n", e.c_str());
		return 0;
	} catch (...) {
		fprintf(stderr, "Caught unknown exception\n");
		return 0;
	}

	return 1;
}

int unirsvp_lspsignorth_server_setup(void)
{
	printf("Setting up LspSigNorth CORBA server side\n");

	try {
		unirsvp_lspsignorth_servant = new LspSigNorth_i();
		if (!unirsvp_lspsignorth_servant) {
			throw std::string("Cannot create servant");
		}

		PortableServer::POA_var poa;
		poa = corba_poa();
		if (CORBA::is_nil(poa)) {
			throw string("Cannot get POA");
		}

		PortableServer::ObjectId_var servant_id;
		servant_id = poa->activate_object(unirsvp_lspsignorth_servant);

		CORBA::Object_var obj;
		obj = unirsvp_lspsignorth_servant->_this();
		if (CORBA::is_nil(obj)) {
			throw string("Cannot get object");
		}

		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			throw string("Cannot get ORB");
		}

		CORBA::String_var sior(orb->object_to_string(obj));
		printf("IOR = '%s'\n", (char*) sior);

		if (!corba_dump_ior(CORBA_SERVANT_LSPSIGNORTH_UNIRSVP,
				    string(sior))) {
			throw string("Cannot dump IOR");
		}

		unirsvp_lspsignorth_servant->_remove_ref();

		PortableServer::POAManager_var poa_manager;
		poa_manager = corba_poa_manager();
		if (CORBA::is_nil(poa_manager)) {
			throw string("Cannot get POA Manager");
		}

		poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return 0;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return 0;
	} catch (omniORB::fatalException & e) {
		fprintf(stderr, "Caught omniORB::fatalException:\n");
		fprintf(stderr, "  file: %s\n", e.file());
		fprintf(stderr, "  line: %d\n", e.line());
		fprintf(stderr, "  mesg: %s\n", e.errmsg());
		return 0;
	} catch (string & e) {
		fprintf(stderr, "Caught exception: %s\n", e.c_str());
		return 0;
	} catch (...) {
		fprintf(stderr, "Caught unknown exception\n");
		return 0;
	}

	return 1;
}

int ennirsvp_lspsignorth_server_setup(void)
{
	printf("Setting up LspSigNorth CORBA server side\n");

	try {
		ennirsvp_lspsignorth_servant = new LspSigNorth_i();
		if (!ennirsvp_lspsignorth_servant) {
			throw std::string("Cannot create servant");
		}

		PortableServer::POA_var poa;
		poa = corba_poa();
		if (CORBA::is_nil(poa)) {
			throw string("Cannot get POA");
		}

		PortableServer::ObjectId_var servant_id;
		servant_id = poa->activate_object(ennirsvp_lspsignorth_servant);

		CORBA::Object_var obj;
		obj = ennirsvp_lspsignorth_servant->_this();
		if (CORBA::is_nil(obj)) {
			throw string("Cannot get object");
		}

		CORBA::ORB_var orb;
		orb = corba_orb();
		if (CORBA::is_nil(orb)) {
			throw string("Cannot get ORB");
		}

		CORBA::String_var sior(orb->object_to_string(obj));
		printf("IOR = '%s'\n", (char*) sior);

		if (!corba_dump_ior(CORBA_SERVANT_LSPSIGNORTH_ENNIRSVP,
				    string(sior))) {
			throw string("Cannot dump IOR");
		}

		ennirsvp_lspsignorth_servant->_remove_ref();

		PortableServer::POAManager_var poa_manager;
		poa_manager = corba_poa_manager();
		if (CORBA::is_nil(poa_manager)) {
			throw string("Cannot get POA Manager");
		}

		poa_manager->activate();
	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return 0;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return 0;
	} catch (omniORB::fatalException & e) {
		fprintf(stderr, "Caught omniORB::fatalException:\n");
		fprintf(stderr, "  file: %s\n", e.file());
		fprintf(stderr, "  line: %d\n", e.line());
		fprintf(stderr, "  mesg: %s\n", e.errmsg());
		return 0;
	} catch (string & e) {
		fprintf(stderr, "Caught exception: %s\n", e.c_str());
		return 0;
	} catch (...) {
		fprintf(stderr, "Caught unknown exception\n");
		return 0;
	}

	return 1;
}
#endif // HAVE_OMNIORB

int corba_server_setup(void)
{
#if HAVE_OMNIORB
	return tnrc_server_setup() && nb_server_setup() &&
		unirsvp_lspsignorth_server_setup() && ennirsvp_lspsignorth_server_setup();
#endif // HAVE_OMNIORB

	return 1;
}

#if HAVE_OMNIORB
bool corba_client_setup(LRM::Info_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "ERR:Cannot get ORB\n");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_LRM, ior)) {
		fprintf(stderr, "ERR:Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "ERR:Cannot get object\n");
			return false;
		}

		l = LRM::Info::_narrow(obj);
		if (CORBA::is_nil(l)) {
			fprintf(stderr, "ERR:cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "ERR:Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "ERR:Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "ERR:CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(TNRC::NorthBound_var & t)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "ERR:Cannot get ORB\n");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_TNRC, ior)) {
		fprintf(stderr, "ERR:Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "ERR:Cannot get object\n");
			return false;
		}

		t = TNRC::NorthBound::_narrow(obj);
		if (CORBA::is_nil(t)) {
			fprintf(stderr, "ERR:cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "ERR:Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "ERR:Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "ERR:CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(RecoveryController::SouthBound_var & rc)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_RC_SBI, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		rc = RecoveryController::SouthBound::_narrow(obj);
		if (CORBA::is_nil(rc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
	        fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(CallController::LspSigSouth_var & cc, bool isClient)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string          ior;
	corba_servant_t s;
	if (isClient) {
		s = CORBA_SERVANT_LSPSIGSOUTH_CLIENT_CALLCONTROLLER;
	} else {
		s = CORBA_SERVANT_LSPSIGSOUTH_NETWORK_CALLCONTROLLER;
	}

	if (!corba_fetch_ior(s, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		cc = CallController::LspSigSouth::_narrow(obj);
		if (CORBA::is_nil(cc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
	        fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(CallController::Mgmt_var & cc, bool isClient)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string          ior;
	corba_servant_t s;
	if (isClient) {
		s = CORBA_SERVANT_CLIENT_CALLCONTROLLER;
	} else {
		s = CORBA_SERVANT_NETWORK_CALLCONTROLLER;
	}

	if (!corba_fetch_ior(s, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		cc = CallController::Mgmt::_narrow(obj);
		if (CORBA::is_nil(cc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(PersistencyController::StorageMgmt_var & pc)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string ior;

	if (!corba_fetch_ior(CORBA_SERVANT_MGMT_PERSISTENCYCONTROLLER, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		pc = PersistencyController::StorageMgmt::_narrow(obj);
		if (CORBA::is_nil(pc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(PersistencyController::StorageInniRsvpLsp_var & pc)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string ior;

	if (!corba_fetch_ior(CORBA_SERVANT_INNIRSVPLSP_PERSISTENCYCONTROLLER, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		pc = PersistencyController::StorageInniRsvpLsp::_narrow(obj);
		if (CORBA::is_nil(pc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(PersistencyController::StorageUniRsvpLsp_var & pc)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string ior;

	if (!corba_fetch_ior(CORBA_SERVANT_UNIRSVPLSP_PERSISTENCYCONTROLLER, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		pc = PersistencyController::StorageUniRsvpLsp::_narrow(obj);
		if (CORBA::is_nil(pc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(PersistencyController::StorageEnniRsvpLsp_var & pc)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string ior;

	if (!corba_fetch_ior(CORBA_SERVANT_ENNIRSVPLSP_PERSISTENCYCONTROLLER, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

		pc = PersistencyController::StorageEnniRsvpLsp::_narrow(obj);
		if (CORBA::is_nil(pc)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}

bool corba_client_setup(G2PCERA::RoutingServices_var & g2pcera)
{
	CORBA::ORB_var orb;
	orb = corba_orb();
	if (CORBA::is_nil(orb)) {
		fprintf(stderr, "Cannot get ORB\n");
		return false;
	}

	string ior;

	if (!corba_fetch_ior(CORBA_SERVANT_G2PCERA, ior)) {
		fprintf(stderr, "Cannot fetch IOR\n");
		return false;
	}

	try {
		CORBA::Object_var obj;
		obj = orb->string_to_object(ior.c_str());
		if (CORBA::is_nil(obj)) {
			fprintf(stderr, "Cannot get object\n");
			return false;
		}

	        g2pcera = G2PCERA::RoutingServices::_narrow(obj);
		if (CORBA::is_nil(g2pcera)) {
			fprintf(stderr, "cannot invoke on a nil object reference\n");
			return false;
		}

	} catch (CORBA::SystemException & e) {
		fprintf(stderr, "Caught CORBA::SystemException\n");
		return false;
	} catch (CORBA::Exception & e) {
		fprintf(stderr, "Caught CORBA::Exception\n");
		return false;
	} catch (...) {
		fprintf(stderr, "CLIENT_SETUP: caught unknown exception\n");
		return false;
	}

	return true;
}
#endif // HAVE_OMNIORB

int corba_client_shutdown(corba_servant_t s)
{
#if HAVE_OMNIORB
#endif // HAVE_OMNIORB
	return 1;
}

#if HAVE_OMNIORB
int tnrc_server_shutdown(void)
{
	try {
		G2RSVPTE_DBG("Shutting down TnrController CORBA server side");

		if (!corba_remove_ior(CORBA_SERVANT_TNRCONTROLLER_G2RSVPTE)) {
			throw string("Cannot remove IOR");
		}
	} catch (...) {
		G2RSVPTE_ERR("Caught unknown exception");
		return 0;
	}
	return 1;
}

int nb_server_shutdown(void)
{
	try {
		G2RSVPTE_DBG("Shutting down NorthBound CORBA server side");

		if (!corba_remove_ior(CORBA_SERVANT_NORTHBOUND_G2RSVPTE)) {
			throw string("Cannot remove IOR");
		}
	} catch (...) {
		G2RSVPTE_ERR("Caught unknown exception");
		return 0;
	}
	return 1;
}

int unirsvp_lspsignorth_server_shutdown()
{
	try {
		G2RSVPTE_DBG("Shutting down UNI LspSigNorth CORBA server side");

		if (!corba_remove_ior(CORBA_SERVANT_LSPSIGNORTH_UNIRSVP)) {
			throw string("Cannot remove IOR");
		}
	} catch (...) {
		G2RSVPTE_ERR("Caught unknown exception");
		return 0;
	}
	return 1;
}

int ennirsvp_lspsignorth_server_shutdown()
{
	try {
		G2RSVPTE_DBG("Shutting down ENNI LspSigNorth CORBA server side");

		if (!corba_remove_ior(CORBA_SERVANT_LSPSIGNORTH_ENNIRSVP)) {
			throw string("Cannot remove IOR");
		}
	} catch (...) {
		G2RSVPTE_ERR("Caught unknown exception");
		return 0;
	}
	return 1;
}
#endif // HAVE_OMNIORB

int corba_server_shutdown(void)
{
#if HAVE_OMNIORB
	return tnrc_server_shutdown() && nb_server_shutdown() &&
		unirsvp_lspsignorth_server_shutdown() && ennirsvp_lspsignorth_server_shutdown();
#endif // HAVE_OMNIORB

	return 1;
}
