//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "g2rsvpte_corba.h"

#include "g2rsvpted/g2rsvpte_lrm_api.h"
#include "g2rsvpted/g2rsvpte_trace.h"


bool getLocDLid_on_remDLid(uint32_t        nodeId,
			   g2mpls_addr_t   remoteDL,
			   g2mpls_addr_t & localDL,
			   admstate_t &    adminStatus,
			   opstate_t &     operStatus)
{
#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("GetLocal DL on remote DL ... debugging");

	ipv4_t tmp;

	if (!inet_aton(ROUTERID1, &tmp)) {
		G2RSVPTE_ERR("Invalid IP address!");
		return false;
	}

	localDL.type                = UNNUMBERED;
	localDL.preflen             = 0;
	//localDL.value.unnum.node_id = ntohl(tmp.s_addr);
	//localDL.value.unnum.addr    = 12345;
	localDL.value.unnum         = 12345;
	adminStatus                 = ENABLED;
	operStatus                  = UP;

	return true;
#endif

#if HAVE_OMNIORB
	try {
		g2mplsTypes::DLinkId_var remDL;
		remDL << remoteDL;

		g2mplsTypes::DLinkId_var locDL;
		g2mplsTypes::operState   operState;
		g2mplsTypes::adminState  adminState;

		lrm_proxy->localDLinkIdFromRemoteDLinkId(nodeId,
							 remDL,
							 locDL,
							 operState,
							 adminState);

		localDL     << locDL;
		adminStatus << adminState;
		operStatus  << operState;

	//} catch (LRM::Info::UnknownNodeId) {
	} catch (LRM::Info::UnknownDLink) {
		G2RSVPTE_ERR("UnknownDLink... cannot get local DLink "
			     "from remote DL (LRM)");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get local DLink from remote DL (LRM)");
		return false;
	}
#endif
	return true;
}


RSVP::Interface * getTELink_on_DLid(RSVP::G2RSVPTE * instance,
				    g2mpls_addr_t    localDL)
{
	if (!instance) {
		G2RSVPTE_ERR("Invalid G2RSVPTE instance pointer!");
		return 0;
	}

#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Get TELink on DL ... debugging");

	ipv4_t        localId;
	g2mpls_addr_t tmp;
	memset(&localId, 0, sizeof(localId));

	if (strchr(LOCALID2, '.') != 0) {
		// matched "A.B.C.D"
		if (!inet_aton(LOCALID2, &(localId))) {
			G2RSVPTE_ERR("Invalid IP address (%s) for local id",
				     LOCALID2);
			return 0;
		}
		tmp.type        = IPv4;
		tmp.preflen     = 32;
		tmp.value.ipv4  = localId;
	} else {
		G2RSVPTE_ERR("Invalid IP address (%s) for local id",
			     LOCALID2);
		return 0;
	}

	return instance->findInterface(tmp, false);
#endif

#if HAVE_OMNIORB
	try {
		//g2mplsTypes::DLinkId_var id;
		//{
		//	g2mplsTypes::DLinkId * tmp;
		//
		//	tmp = new g2mplsTypes::DLinkId;
		//	if (!tmp) {
		//		G2RSVPTE_ERR("Impossible to allocate DLinkId");
		//		return false;
		//	}
		//	id = tmp;
		//}
		//id << DLink;
		//
		//NOTE: the above code, is MemLeak free like the following:
		//
		//g2mplsTypes::DLinkId id;
		//id << localDL;

		g2mplsTypes::DLinkId_var id;
		id << localDL;

		g2mplsTypes::TELinkId_var data;
		data = lrm_proxy->TELinkFromDLink(id);

		g2mpls_addr_t TELink;
		TELink << data;

		return instance->findInterface(TELink, false);

	} catch (LRM::Info::UnknownDLink) {
		G2RSVPTE_ERR("UnknownDLink... cannot get TELink from DL (LRM)");
		return 0;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get TELink from DL (LRM)");
		return 0;
	}
#endif
}

bool getDL_checkLabel_on_TELink(g2mpls_addr_t   TELink,
				uint32_t        lbl,
				g2mpls_addr_t & dl)
{
	bool res = false;
#if HAVE_OMNIORB
	try {
		g2mplsTypes::TELinkId_var id;
		id << TELink;

		g2mplsTypes::DLinkId_var dataLink;
		dataLink = lrm_proxy->DLinkFromTELink(id);

		dl << dataLink;

		//START-TMP
		label_id_t lab;
		lab.type             = LABEL_32BIT;
		lab.value.label32.id = lbl;
		//END-TMP
		g2mplsTypes::labelId_var label;
		label << lab;

		g2mplsTypes::operState  opState;
		g2mplsTypes::labelState lblState;

		res = tnrc_proxy->getLabelStatus(dataLink,
						 label,
						 lblState,
						 opState);

		if (res) {
			label_state_t  labelStatus;
			opstate_t      operStatus;

			labelStatus << lblState;
			operStatus  << opState;

			if (operStatus != UP) {
				G2RSVPTE_ERR("Label is disabled/down "
					     "(failure)");
			return false;
			}
			if (labelStatus != LABEL_FREE) {
				G2RSVPTE_ERR("Label is already busy");
				return false;
			}
		}
	} catch (LRM::Info::UnknownTELink) {
		G2RSVPTE_ERR("UnknownTELink...cannot get DL from TELink (LRM)");
		return false;
	} catch (TNRC::CannotFetch) {
		G2RSVPTE_ERR("CannotFetch ... cannot get LABEL from DL (TNRC)");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get DL & Label from TE Link (LRM/TNRC)");
		return false;
	}
#endif
	return res;
}

bool getDL_Label_on_TELink(g2mpls_addr_t   TELink,
			   // .... , TODO add params
			   g2mpls_addr_t & dl,
			   uint32_t &      lbl)
{
#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Get DL & label on TE Link ... debugging");

	ipv4_t tmp;

	if (!inet_aton(ROUTERID1, &tmp)) {
		G2RSVPTE_ERR("Invalid IP address!");
		return false;
	}

	dl.type                = UNNUMBERED;
	dl.preflen             = 0;
	//dl.value.unnum.node_id = ntohl(tmp.s_addr);
	//dl.value.unnum.addr    = 12345;
	dl.value.unnum         = 12345;
	lbl                    = 0x666;

	return true;
#endif

	bool res = false;
#if HAVE_OMNIORB
	try {
		g2mplsTypes::TELinkId_var id;
		id << TELink;

		g2mplsTypes::DLinkId_var dataLink;
		dataLink = lrm_proxy->DLinkFromTELink(id);

		dl << dataLink;

		g2mplsTypes::labelId_var label;
		res = tnrc_proxy->getLabelFromDLink(dataLink, label);

		if (res) {
			label_id_t lab;
			lab << label;

			//START-TMP
			if (lab.type != LABEL_32BIT) {
				G2RSVPTE_ERR("Only 32-bit label are supported");
				return false;
			}
			lbl = lab.value.label32.id;
			//END-TMP
		}

	} catch (LRM::Info::UnknownTELink) {
		G2RSVPTE_ERR("UnknownTELink...cannot get DL from TELink (LRM)");
		return false;
	} catch (LRM::Info::NoDLinks) {
		G2RSVPTE_ERR("No DLinks in UP state...cannot get DL from TELink (LRM)");
		return false;
	} catch (TNRC::CannotFetch) {
		G2RSVPTE_ERR("CannotFetch ... cannot get LABEL from DL (TNRC)");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get DL & Label from TE Link (LRM/TNRC)");
		return false;
	}
#endif
	return res;
}

bool checkTELink(g2mpls_addr_t teLink)
{
#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Check TE Link ... debugging");
	return true;
#endif

	bool res = false;

#if HAVE_OMNIORB
	try {
		g2mplsTypes::TELinkId_var link;
		link << teLink;

		res = lrm_proxy->checkTELinkId(link);

	}  catch (LRM::Info::TELinkDown) {
		G2RSVPTE_ERR("TELink is Down");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot check TELink from LRM");
		return false;
	}

	G2RSVPTE_DBG("TE link %s %s one of my TELink",
		     addr_ntoa(teLink), res ? "IS" : "is NOT");
#endif
	return res;
}

bool getTELinkfromTna(g2mpls_addr_t tna, g2mpls_addr_t & teLink)
{
#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Get TE Link from TNA ... debugging");
	return false;
#endif

#if HAVE_OMNIORB
	try {
		g2mplsTypes::TELinkId_var telId;
		g2mplsTypes::tnaId_var    tnaId;

		tnaId << tna;

		telId = lrm_proxy->TELinkFromTna(tnaId);

		teLink << telId;

	}  catch (LRM::Info::UnknownTna) {
		G2RSVPTE_ERR("Tna not found");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get TELink from Tna");
		return false;
	}
#endif
	return true;
}

bool getTELinkfromRemNodeId(uint32_t node, g2mpls_addr_t & teLink)
{
#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Get TE Link from NodeId ... debugging");
	return false;
#endif

#if HAVE_OMNIORB
	try {
		g2mplsTypes::TELinkId_var telId;
		g2mplsTypes::nodeId       nodeId;

		nodeId = node;

		telId = lrm_proxy->TELinkFromRemNodeId(nodeId);

		teLink << telId;

	}  catch (LRM::Info::UnknownNodeId) {
		G2RSVPTE_ERR("NodeId not found");
		return false;
	}  catch (LRM::Info::NoTELinks) {
		G2RSVPTE_ERR("No TeLinks to Node %s", numb_htoa(node));
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get TELink from Tna");
		return false;
	}
#endif
	return true;
}

bool getRouterId(uint32_t & routerId)
{
#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Get routerID from TE Link ... debugging");

	ipv4_t tmp;
	int    ret;
	routerId = 0;

	ret = inet_aton(ROUTERID1, &tmp);

	if (!ret) {
		G2RSVPTE_ERR("Invalid IP address!");
		return false;
	}

	routerId = ntohl(tmp.s_addr);
	return true;
#endif

#if HAVE_OMNIORB
	try {
		g2mplsTypes::nodeId id;

		id = lrm_proxy->nodeId();
		routerId = id;

		G2RSVPTE_INF("Node ID: %s", numb_htoa(routerId));

	} catch (...) {
		G2RSVPTE_ERR("Cannot get node ID from LRM");
		return false;
	}
#endif
	return true;
}

bool getInterfaces(uint32_t routerId,
		   std::list<Interface_data_t> & intfs,
		   adj_type_t adjType)
{
	if (!intfs.empty()) {
		G2RSVPTE_WRN("Interface list is not empty!");
		return false;
	}

#ifdef G2RSVPTE2LRM_DEBUG
	G2RSVPTE_DBG("Get Interfaces (TE Links) ... debugging");

	Interface_data_t tmp;
	ipv4_t           localId;
	ipv4_t           remoteId;
	ipv4_t           nId;

	memset(&localId, 0, sizeof(localId));
	memset(&remoteId, 0, sizeof(remoteId));

	// First interface
	if (strchr(LOCALID2, '.') != 0) {
		// matched "A.B.C.D"
		if (!inet_aton(LOCALID2, &(localId))) {
			G2RSVPTE_ERR("Invalid IP address (%s) for local id",
				     LOCALID2);
			return false;
		}
		tmp.localId.type        = IPv4;
		tmp.localId.preflen     = 32;
		tmp.localId.value.ipv4  = localId;
	} else {
		G2RSVPTE_ERR("Invalid IP address (%s) for local id",
			     LOCALID2);
		return false;
	}

	if (strchr(REMOTEID2, '.') != 0) {
		// matched "A.B.C.D"
		if (!inet_aton(REMOTEID2, &(remoteId))) {
			G2RSVPTE_ERR("Invalid IP address (%s) for remote id",
				     REMOTEID2);
			return false;
		}
		tmp.remoteId.type        = IPv4;
		tmp.remoteId.preflen     = 32;
		tmp.remoteId.value.ipv4  = remoteId;
	} else {
		G2RSVPTE_ERR("Invalid IP address (%s) for remote id",
			     REMOTEID2);
		return false;
	}

	if (!inet_aton(ROUTERID2, &nId)) {
		G2RSVPTE_ERR("Invalid IP address!");
		return false;
	}
	tmp.neighbourId = ntohl(nId.s_addr);


	tmp.op_state  = UP;
	tmp.adm_state = ENABLED;

	intfs.push_front(tmp);


	// Second interface (iTNA)
	memset(&localId, 0, sizeof(localId));
	memset(&remoteId, 0, sizeof(remoteId));

	if (strchr(ITNA, '.') != 0) {
		// matched "A.B.C.D"
		if (!inet_aton(ITNA, &(localId))) {
			G2RSVPTE_ERR("Invalid IP address (%s) for local id",
				     ITNA);
			return false;
		}
		tmp.localId.type        = IPv4;
		tmp.localId.preflen     = 32;
		tmp.localId.value.ipv4  = localId;
	} else {
		G2RSVPTE_ERR("Invalid IP address (%s) for local id",
			     ITNA);
		return false;
	}

	tmp.remoteId.type                = UNNUMBERED;
	tmp.remoteId.preflen             = 0;
	//tmp.remoteId.value.unnum.node_id = 0;
	//tmp.remoteId.value.unnum.addr    = 0;
	tmp.remoteId.value.unnum         = 0;

	tmp.neighbourId = 0;

	tmp.op_state  = UP;
	tmp.adm_state = ENABLED;

	intfs.push_front(tmp);

	return true;
#endif

#if HAVE_OMNIORB
	try {
		g2mplsTypes::TELinkDataSeq_var TELinks;
		g2mplsTypes::adjType           adjacencyType;

		adjacencyType << adjType;

		TELinks = lrm_proxy->allTELinks(adjacencyType);

		G2RSVPTE_DBG("Number of %s TE Links: %ld",
			     SHOW_ADJTYPE(adjType), TELinks->length());

		for (size_t i = 0; i < TELinks->length(); i++) {

			g2mplsTypes::TELinkData_var data;
			data = TELinks[i];

			Interface_data_t tmp;
			g2mplsTypes::TELinkId_var te;
			te = data->localId;
			tmp.localId   << te;
			te = data->remoteId;
			tmp.remoteId  << te;
			tmp.op_state  << data->parms.states.opState;
			tmp.adm_state << data->parms.states.admState;
			tmp.neighbourId = data->neighbour;

			intfs.push_front(tmp);
		}
	} catch (LRM::Info::NoTELinks) {
		G2RSVPTE_ERR("There aren't %s TE Links", SHOW_ADJTYPE(adjType));
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get TE Links from LRM");
		return false;
	}
#endif
	return true;
}
