//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2rsvpte_message_objects.h"
#include "g2rsvpte_trace.h"
#include "g2rsvpte_common_types.h"

// Label Utils
#define GLABEL_OBJ_OPERATOR_IMP(OBJ_NAME)				      \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj)  \
	{								      \
		os << #OBJ_NAME << std::endl;				      \
		for (uint32_t i=0; i<obj.value_.size;  i+=sizeof(uint32_t)) { \
			os << " label[" << i / sizeof(uint32_t) << "]: "      \
			   << obj.value_.label[i / sizeof(uint32_t)]	      \
			   << std::endl;				      \
		}							      \
		return os;						      \
	}								      \
									      \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj)  \
	{								      \
		for (uint32_t i=0; i<obj.value_.size; i+=sizeof(uint32_t)) {  \
			mc << obj.value_.label[i / sizeof(uint32_t)];	      \
		}							      \
									      \
		assert(mc.remainingSize() == 0);			      \
									      \
		return mc;						      \
	}								      \
									      \
	MemoryCursor & operator >> (MemoryCursor &  mc, OBJ_NAME & obj)	      \
	{								      \
		obj.value_.size = 0;					      \
		uint32_t len = mc.remainingSize();			      \
									      \
		obj.value_.label = new uint32_t[len / sizeof(uint32_t)];      \
		assert(obj.value_.label);				      \
									      \
		for (uint32_t i = 0; i < len; i += sizeof(uint32_t)) {	      \
			mc >> obj.value_.label[i / sizeof(uint32_t)];	      \
			obj.value_.size += sizeof(uint32_t);		      \
		}							      \
									      \
		assert(mc.remainingSize() == 0);			      \
									      \
		return mc;						      \
	}

// TLV Utils
#define TLVGEN_OBJ_OPERATOR_IMP(TLV_NAME)				      \
	std::ostream & operator << (std::ostream & os, const TLV_NAME & obj)  \
	{								      \
		os << "  " #TLV_NAME                             << std::endl \
/*TODO numb_hton*/ << "   IP addr:  " << obj.value_.ip_address   << std::endl \
		   << "   InterfID: " << obj.value_.interface_id << std::endl;\
		return os;						      \
	}								      \
									      \
	MemoryCursor & operator << (MemoryCursor & mc, const TLV_NAME & obj)  \
	{								      \
		mc << obj.value_.ip_address << obj.value_.interface_id;	      \
		assert(mc.remainingSize() == 0);			      \
		return mc;						      \
	}								      \
									      \
	MemoryCursor & operator >> (MemoryCursor & mc, TLV_NAME & obj)	      \
	{								      \
		mc >> obj.value_.ip_address >> obj.value_.interface_id;	      \
		assert(mc.remainingSize() == 0);			      \
		return mc;						      \
	}

// Label32 utils
#define LABEL32_TLV_OPERATOR_IMP(TLV_NAME)                                    \
	std::ostream & operator << (std::ostream & os, const TLV_NAME & obj)  \
	{								      \
		os << "  " #TLV_NAME                    << std::endl          \
		   << "   label:  " << obj.value_.label << std::endl;         \
		return os;						      \
	}								      \
									      \
	MemoryCursor & operator << (MemoryCursor & mc, const TLV_NAME & obj)  \
	{								      \
		mc << obj.value_.label;	                                      \
		assert(mc.remainingSize() == 0);			      \
		return mc;						      \
	}								      \
									      \
	MemoryCursor & operator >> (MemoryCursor & mc, TLV_NAME & obj)	      \
	{								      \
		mc >> obj.value_.label;					      \
		assert(mc.remainingSize() == 0);			      \
		return mc;						      \
	}

// Address Obj Utils
#define IPV4_ADDR_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << "  " #OBJ_NAME                         << std::endl    \
		   << "   IPv4:  "   << obj.value_.ipv4_addr << std::endl;   \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		mc << obj.value_.ipv4_addr;				     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		mc >> obj.value_.ipv4_addr;				     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}

#define IPV6_ADDR_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << "  " #OBJ_NAME                         << std::endl    \
		   << "   IPv6:  "   << obj.value_.ipv6_addr << std::endl;   \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		mc << obj.value_.ipv6_addr;				     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		mc >> obj.value_.ipv6_addr;				     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}

#define NSAP_ADDR_OPERATOR_IMP(OBJ_NAME)				      \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj)  \
	{								      \
		os << "  " #OBJ_NAME                           << std::endl   \
		   << "   nsap len:  " << obj.value_.nsap_len  << std::endl   \
		   << "   nsap addr: " << obj.value_.nsap_addr << std::endl;  \
		return os;						      \
	}								      \
									      \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj)  \
	{								      \
		mc << obj.value_.nsap_len				      \
		   << (uint8_t) 0   /* reserved */			      \
		   << (uint16_t)0;  /* reserved */			      \
									      \
		uint32_t i;						      \
		for (i = 0; i < obj.value_.nsap_len; i++) {		      \
			mc << ((uint8_t *)obj.value_.nsap_addr)[i];	      \
		}							      \
									      \
		assert(mc.remainingSize() == 0);			      \
		return mc;						      \
	}								      \
									      \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	      \
	{								      \
		uint8_t  resvd8;					      \
		uint16_t resvd16;					      \
									      \
		mc >> obj.value_.nsap_len				      \
		   >> resvd8        /* reserved */			      \
		   >> resvd16;      /* reserved */			      \
									      \
		obj.value_.nsap_addr = new char[obj.value_.nsap_len];/*+1];*/ \
		assert(obj.value_.nsap_addr);				      \
									      \
		uint32_t i;						      \
		for (i = 0; i < obj.value_.nsap_len; i++) {		      \
			mc >> ((uint8_t *)obj.value_.nsap_addr)[i];	      \
		}							      \
		/*((uint8_t *)obj.value_.nsap_addr)[i] = '\0'; needed ???*/   \
									      \
		assert(mc.remainingSize() == 0);			      \
		return mc;						      \
	}

// Utils for OBJ with a list of SubObjs (ERO/RRO/GUNI/GNS ...)
#define _LIST_SUBOBJ_OUT_OPERATOR_IMP(OBJ_NAME, SUBOBJ_NAME)	              \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj)  \
	{								      \
		os << #OBJ_NAME       << std::endl			      \
		   << " subObjects: " << std::endl;			      \
									      \
		std::list<SUBOBJ_NAME *>::const_iterator it;		      \
		for (it = obj.value_.subObjs.begin();			      \
		     it != obj.value_.subObjs.end();			      \
		     it++) {						      \
			if (*it) {					      \
				os << *(*it);				      \
			}						      \
		}							      \
		return os;						      \
	}								      \
									      \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj)  \
	{								      \
		MemoryCursor mcObj(mc);					      \
		std::list<SUBOBJ_NAME *>::const_iterator it;		      \
		for (it = obj.value_.subObjs.begin();			      \
		     it != obj.value_.subObjs.end();			      \
		     it++) {						      \
			if (*it) {					      \
				if (it == obj.value_.subObjs.begin()) {	      \
					mcObj.resize(mc.start(), mc.start() + \
						     ((*it)->length()-1));    \
				} else {				      \
					mcObj = mcObj.			      \
						neighbor((*it)->length());    \
				}					      \
				mcObj << *(*it);			      \
			}						      \
		}							      \
		return mc;						      \
	}

#define _LIST_SUBOBJ_IN_OPERATOR_IMP1(OBJ_NAME, SUBOBJ_NAME, HDR_LEN)	   \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	   \
	{								   \
		int len = mc.remainingSize();				   \
									   \
		uint8_t subObj_type;					   \
		uint8_t subObj_len;					   \
									   \
		MemoryCursor mcObjHdr(mc);				   \
		mcObjHdr.resize(mc.start(), mc.start() + (HDR_LEN - 1));   \
									   \
		while (len > 0) {					   \
									   \
			mcObjHdr >> subObj_type >> subObj_len;		   \
			assert(mcObjHdr.remainingSize() == 0);		   \
									   \
			MemoryCursor mcObj(mc);				   \
			mcObj.resize(mcObjHdr.start(),			   \
				     mcObjHdr.start() + (subObj_len - 1)); \
									   \
			SUBOBJ_NAME * subObj = 0;			   \
			mcObj >> subObj;				   \
									   \
			obj.value_.subObjs.push_back(subObj);		   \
			len -= subObj->length();			   \
									   \
			if (len > 0) {					   \
				mcObjHdr = mcObj.neighbor(HDR_LEN);	   \
			}						   \
		}							   \
		return mc;						   \
	}

#define _LIST_SUBOBJ_IN_OPERATOR_IMP2(OBJ_NAME, SUBOBJ_NAME, HDR_LEN)	   \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	   \
	{								   \
		int len = mc.remainingSize();				   \
									   \
		uint16_t sObj_len;					   \
		uint8_t  sObj_type;					   \
		uint8_t  sObj_subType;					   \
									   \
		MemoryCursor mcObjHdr(mc);				   \
		mcObjHdr.resize(mc.start(), mc.start() + (HDR_LEN - 1));   \
									   \
		while (len > 0) {					   \
									   \
			mcObjHdr >> sObj_len >> sObj_type >> sObj_subType; \
			assert(mcObjHdr.remainingSize() == 0);		   \
									   \
			MemoryCursor mcObj(mc);				   \
			mcObj.resize(mcObjHdr.start(),			   \
				     mcObjHdr.start() + (sObj_len - 1));   \
									   \
			SUBOBJ_NAME * subObj = 0;			   \
			mcObj >> subObj;				   \
									   \
			obj.value_.subObjs.push_back(subObj);		   \
			len -= subObj->length();			   \
									   \
			if (len > 0) {					   \
				mcObjHdr = mcObj.neighbor(HDR_LEN);	   \
			}						   \
		}							   \
		return mc;						   \
	}

// ERO/RRO Utils
#define ERO_RRO_OBJ_OPERATOR_IMP(OBJ_NAME, SUBOBJ_NAME, HDR_LEN)	\
  _LIST_SUBOBJ_OUT_OPERATOR_IMP(OBJ_NAME, SUBOBJ_NAME)			\
  _LIST_SUBOBJ_IN_OPERATOR_IMP1(OBJ_NAME, SUBOBJ_NAME, HDR_LEN)

// GNS/GUNI Utils
#define GUNI_GNS_OBJ_OPERATOR_IMP(OBJ_NAME, SUBOBJ_NAME, HDR_LEN)	\
  _LIST_SUBOBJ_OUT_OPERATOR_IMP(OBJ_NAME, SUBOBJ_NAME)			\
  _LIST_SUBOBJ_IN_OPERATOR_IMP2(OBJ_NAME, SUBOBJ_NAME, HDR_LEN)

// Address List Utils
#define ADDR_LIST_OPERATOR_IMP(OBJ_NAME, SUBOBJ_NAME)			     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << #OBJ_NAME       << std::endl			     \
		   << " Addresses: "  << std::endl;			     \
									     \
		std::list<SUBOBJ_NAME *>::const_iterator it;		     \
		for (it = obj.value_.subObjs.begin();			     \
		     it != obj.value_.subObjs.end();			     \
		     it++) {						     \
			if (*it) {					     \
				os << "   IPv4: " << *(*it) << std::endl;    \
			}						     \
		}							     \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		std::list<SUBOBJ_NAME *>::const_iterator it;		     \
		for (it = obj.value_.subObjs.begin();			     \
		     it != obj.value_.subObjs.end();			     \
		     it++) {						     \
			if (*it) {					     \
				mc << *(*it);				     \
			}						     \
		}							     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		while (mc.remainingSize() > 0) {			     \
									     \
			SUBOBJ_NAME * subObj = new SUBOBJ_NAME();	     \
			assert(subObj);					     \
			mc >> *subObj;					     \
									     \
			obj.value_.subObjs.push_back(subObj);		     \
		}							     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}


// FilterSpec/SenderTemplate utils
#define IPV6FLOW_OBJ_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << #OBJ_NAME                             << std::endl     \
		   << " IPv6:    " << obj.value_.ipv6_addr  << std::endl     \
		   << " FlowLbl: " << obj.value_.flow_label << std::endl;    \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		uint32_t res_fLabel;					     \
		res_fLabel  = obj.value_.flow_label & 0x00ffffff;	     \
		/* useless! res_fLabel |= ((uint8_t)0 << 24); //reserved */  \
		mc << obj.value_.ipv6_addr << res_fLabel;		     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		uint32_t res_fLabel;					     \
		mc >> obj.value_.ipv6_addr >> res_fLabel;		     \
		assert(mc.remainingSize() == 0);			     \
		obj.value_.flow_label = res_fLabel & 0x00ffffff;	     \
		/* reserved must be ignored */				     \
		obj.value_.reserved   = (res_fLabel & 0xff000000) >> 24;     \
		return mc;						     \
	}

#define IPV4_PORT_OBJ_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << #OBJ_NAME                              << std::endl    \
		   << " IPv4:     " << obj.value_.ipv4_addr  << std::endl    \
		   << " src port: " << obj.value_.src_port   << std::endl;   \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		mc << obj.value_.ipv4_addr				     \
		   << obj.value_.reserved				     \
		   << obj.value_.src_port;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		mc >> obj.value_.ipv4_addr				     \
		   >> obj.value_.reserved				     \
		   >> obj.value_.src_port;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}

#define IPV6_PORT_OBJ_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << #OBJ_NAME                              << std::endl    \
		   << " IPv6:     " << obj.value_.ipv6_addr  << std::endl    \
		   << " src port: " << obj.value_.src_port   << std::endl;   \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		mc << obj.value_.ipv6_addr				     \
		   << obj.value_.reserved				     \
		   << obj.value_.src_port;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		mc >> obj.value_.ipv6_addr				     \
		   >> obj.value_.reserved				     \
		   >> obj.value_.src_port;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}

#define IPV4LSP_OBJ_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << #OBJ_NAME                           << std::endl	     \
		   << " IPv4:   " << obj.value_.ipv4_addr << std::endl	     \
		   << " LSP ID: " << obj.value_.lsp_id    << std::endl;	     \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		mc << obj.value_.ipv4_addr				     \
		   << obj.value_.reserved				     \
		   << obj.value_.lsp_id;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		mc >> obj.value_.ipv4_addr				     \
		   >> obj.value_.reserved				     \
		   >> obj.value_.lsp_id;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}

#define IPV6LSP_OBJ_OPERATOR_IMP(OBJ_NAME)				     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << #OBJ_NAME                           << std::endl	     \
		   << " IPv6:   " << obj.value_.ipv6_addr << std::endl	     \
		   << " LSP ID: " << obj.value_.lsp_id    << std::endl;	     \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		mc << obj.value_.ipv6_addr				     \
		   << obj.value_.reserved				     \
		   << obj.value_.lsp_id;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		mc >> obj.value_.ipv6_addr				     \
		   >> obj.value_.reserved				     \
		   >> obj.value_.lsp_id;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
									     \
		return mc;						     \
	}


///////////////////////////////////////////////////////////////////////////////
//                              TLV Classes                                  //
///////////////////////////////////////////////////////////////////////////////

namespace RSVP {

	//
	// Base (hdr) TLV
	//
	std::ostream & operator << (std::ostream & os, const TLV & tlv)
	{
		os << " TLV Object"                << std::endl
		   << "  type:   " << tlv.type()   << std::endl
		   << "  length: " << tlv.length() << std::endl;
		TLV_OBJ2STREAM(os, tlv);
		return os;
	}

	MemoryCursor & operator << (MemoryCursor & mc, const TLV & tlv)
	{
		// Writes Object Header
		MemoryCursor mcTlvHdr(mc);
		mcTlvHdr.resize(mc.start(),
				mc.start() + (TLV::tlvHdrLength - 1));
		mcTlvHdr << tlv.type() << tlv.length();
		assert(mcTlvHdr.remainingSize() == 0);

		MemoryCursor mcTlv = mcTlvHdr.neighbor(mc.stop() - mc.start() -
						      (TLV::tlvHdrLength - 1));
		// Writes Object
		TLV_OBJ2STREAM(mcTlv, tlv);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, TLV * & tlv)
	{
		// Check Object Header
		MemoryCursor mcTlvHdr(mc);
		mcTlvHdr.resize(mc.start(),
				mc.start() + (TLV::tlvHdrLength - 1));
		assert(mcTlvHdr.remainingSize() >= TLV::tlvHdrLength);

		uint16_t type;
		uint16_t len;

		mcTlvHdr >> type >> len;
		assert(mcTlvHdr.remainingSize() == 0);

		MemoryCursor mcTlv = mcTlvHdr.neighbor(len -TLV::tlvHdrLength);

		TLV_STREAM2OBJ(mcTlv, tlv, type);
		return mc;
	}

	//
	// TLVs
	//

	// IPv4_TLV
	IPV4_ADDR_OPERATOR_IMP(IPv4_TLV);

	// IPv6_TLV
	IPV6_ADDR_OPERATOR_IMP(IPv6_TLV);

	// IF_IDX_TLV
	TLVGEN_OBJ_OPERATOR_IMP(IF_IDX_TLV);

	// COMP_IF_DOWN_TLV
	TLVGEN_OBJ_OPERATOR_IMP(COMP_IF_DOWN_TLV);

	// COMP_IF_UP_TLV
	TLVGEN_OBJ_OPERATOR_IMP(COMP_IF_UP_TLV);

	// DOWN_LABEL_TLV
	LABEL32_TLV_OPERATOR_IMP(DOWN_LABEL_TLV);

	// UP_LABEL_TLV
	LABEL32_TLV_OPERATOR_IMP(UP_LABEL_TLV);

#ifdef CRANKBACK_ON
	// LSP_ATTR_TLV
	std::ostream & operator << (std::ostream & os, const LSP_ATTR_TLV  & tlv)
	{
		os << "  TLV Object"                << std::endl
		   << "   type:   " << tlv.type()   << std::endl
		   << "   length: " << tlv.length() << std::endl;

		os << "   LSP_ATTR_TLV"                                << std::endl
		   << "    e2e_cb     :  "   << tlv.value_.e2e_cb      << std::endl
		   << "    boundary_cb:  "   << tlv.value_.boundary_cb << std::endl
		   << "    segment_cb :  "   << tlv.value_.segment_cb  << std::endl;

		return os;
	}

	MemoryCursor & operator << (MemoryCursor & mc, const LSP_ATTR_TLV & tlv)
	{
		// Writes Object Header
		MemoryCursor mcTlvHdr(mc);
		mcTlvHdr.resize(mc.start(),
				mc.start() + (TLV::tlvHdrLength - 1));
		mcTlvHdr << tlv.type() << tlv.length();
		assert(mcTlvHdr.remainingSize() == 0);

		MemoryCursor mcTlv = mcTlvHdr.neighbor(mc.stop() - mc.start() -
						      (TLV::tlvHdrLength - 1));

		uint32_t lsp_attr_flags = 0x0;
		lsp_attr_flags |= tlv.value_.flags        & 0x1fffffff;
		lsp_attr_flags |= (tlv.value_.segment_cb  & 0x1) << 29;
		lsp_attr_flags |= (tlv.value_.boundary_cb & 0x1) << 30;
		lsp_attr_flags |= (tlv.value_.e2e_cb      & 0x1) << 31;

		mcTlv << lsp_attr_flags;

		assert(mcTlv.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc,  LSP_ATTR_TLV * & tlv)
	{
		// Check Object Header
		MemoryCursor mcTlvHdr(mc);
		mcTlvHdr.resize(mc.start(),
				mc.start() + (TLV::tlvHdrLength - 1));
		assert(mcTlvHdr.remainingSize() >= TLV::tlvHdrLength);

		uint16_t type;
		uint16_t len;

		mcTlvHdr >> type >> len;
		assert(mcTlvHdr.remainingSize() == 0);

		MemoryCursor mcTlv = mcTlvHdr.neighbor(len -TLV::tlvHdrLength);

		uint32_t lsp_attr_flags;

		mcTlv >> lsp_attr_flags;

		tlv = new LSP_ATTR_TLV();
		assert(tlv);

		tlv->get().flags       = lsp_attr_flags  & 0x1fffffff;
		tlv->get().segment_cb  = (lsp_attr_flags & 0x20000000) >> 29;
		tlv->get().boundary_cb = (lsp_attr_flags & 0x40000000) >> 30;
		tlv->get().e2e_cb      = (lsp_attr_flags & 0x80000000) >> 31;

		assert(mcTlv.remainingSize() == 0);

		return mc;
	}
#endif // CRANKBACK_ON
}


///////////////////////////////////////////////////////////////////////////////
//                         ERO SubObject Classes                             //
///////////////////////////////////////////////////////////////////////////////

namespace RSVP {

	//
	// (hdr) ERO SubObject
	//
	std::ostream & operator << (std::ostream &       os,
				    const EroSubObject & obj)
	{
		os << " ERO subObject"                << std::endl
		   << "  loose:  " << obj.looseFlag() << std::endl
		   << "  type:   " << obj.type()      << std::endl
		   << "  length: " << obj.length()    << std::endl;
		EROSUBOBJ_OBJ2STREAM(os, obj);
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &       mc,
				    const EroSubObject & obj)
	{
		// Writes Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(EroSubObject::eroSubObjHdrLength - 1));

		uint8_t flag_type;
		flag_type  = obj.looseFlag() ? 0x80 : 0x00;
		flag_type |= obj.type() & 0x7f;
		mcObjHdr << flag_type << obj.length();
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(mc.stop() - mc.start() -
				 (EroSubObject::eroSubObjHdrLength - 1));
		// Writes Object
		EROSUBOBJ_OBJ2STREAM(mcObj, obj);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, EroSubObject * & obj)
	{
		// Check Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(EroSubObject::eroSubObjHdrLength - 1));
		assert(mcObjHdr.remainingSize() >=
		       EroSubObject::eroSubObjHdrLength);

		uint8_t type;
		uint8_t len;
		bool    loose;

		mcObjHdr >> type >> len;
		assert(mcObjHdr.remainingSize() == 0);

		loose = (type & 0x80) >> 7;  // equivalent to (type >> 7)
		type  = type & 0x7f;

		MemoryCursor mcObj = mcObjHdr.
			neighbor(len - EroSubObject::eroSubObjHdrLength);

		EROSUBOBJ_STREAM2OBJ(mcObj, obj, type);

		// Setting loose flag
		obj->looseFlag(loose);

		return mc;
	}


	//
	// ERO SubObjects
	//

	EroSubObject * createEroSubObj(const lsp_ero_sobj_t & sObj)
	{
		RSVP::EroSubObject * tmp = 0;
		switch (sObj.type) {
			case ERO_SUBOBJ_IPV4: {
				RSVP::IPv4Pref_SubObj * ipv4;
				ipv4 = new RSVP::IPv4Pref_SubObj(sObj.hop.
								 ipv4.addr,
								 sObj.hop.
								 ipv4.prefix);
				if (!ipv4) {
					return 0;
				}
				tmp = ipv4;
				break;
			}
			case ERO_SUBOBJ_IPV6: {
				RSVP::IPv6Pref_SubObj * ipv6;
				ipv6 = new RSVP::IPv6Pref_SubObj(sObj.hop.
								 ipv6.addr,
								 sObj.hop.
								 ipv6.prefix);
				if (!ipv6) {

					return 0;
				}
				tmp = ipv6;
				break;
			}
			case ERO_SUBOBJ_LABEL: {
				RSVP::LabelERO_SubObj * label;
				label = new RSVP::LabelERO_SubObj(sObj.hop.
								  label.value,
								  sObj.hop.
								  label.upstream);
				if (!label) {
					return 0;
				}
				tmp = label;
				break;

				break;
			}
			case ERO_SUBOBJ_UNNUM: {
				RSVP::Unnum_SubObj * unnum;
				unnum = new RSVP::Unnum_SubObj(sObj.hop.
							       unnum.router_id,
							       sObj.hop.
							       unnum.intf_id);
				if (!unnum) {
					return 0;
				}
				tmp = unnum;
				break;
			}
			case ERO_SUBOBJ_NODE: {
				RSVP::Unnum_SubObj * node;
				node = new RSVP::Unnum_SubObj(sObj.hop.
							      node.node_id,
							      0);
				if (!node) {
					return 0;
				}
				tmp = node;
				break;
			}
			case ERO_SUBOBJ_AS: {
				if (sObj.loose == ERO_SUBOBJ_STRICT) {
					return 0;
				}

				RSVP::ASNum_SubObj * as;
				as = new RSVP::ASNum_SubObj(sObj.hop.
							    as.number);
				if (!as) {
					return 0;
				}
				tmp = as;
				break;
			}
			case ERO_SUBOBJ_UNKNOWN:
			default:
				return 0;
		}

		tmp->looseFlag(sObj.loose == ERO_SUBOBJ_LOOSE ? true:false);

		return tmp;
	}

	// IPv4Pref_SubObj
	std::ostream & operator << (std::ostream &          os,
				    const IPv4Pref_SubObj & obj)
	{
		os << "  IPv4Pref_SubObj"                      << std::endl
		   << "   IPv4:    " << obj.content_.ipv4_addr << std::endl
		   << "   PrefLen: " << obj.content_.preflen   << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const IPv4Pref_SubObj & obj)
	{
		mc << obj.content_.ipv4_addr
		   << obj.content_.preflen
		   << (uint8_t)0;           // reserved
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, IPv4Pref_SubObj & obj)
	{
		mc >> obj.content_.ipv4_addr
		   >> obj.content_.preflen
		   >> obj.content_.reserved;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// IPv6Pref_SubObj
	std::ostream & operator << (std::ostream &          os,
				    const IPv6Pref_SubObj & obj)
	{
		os << "  IPv6Pref_SubObj"                      << std::endl
		   << "   IPv6:    " << obj.content_.ipv6_addr << std::endl
		   << "   PrefLen: " << obj.content_.preflen   << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const IPv6Pref_SubObj & obj)
	{
		mc << obj.content_.ipv6_addr
		   << obj.content_.preflen
		   << (uint8_t)0;           // reserved
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, IPv6Pref_SubObj & obj)
	{
		mc >> obj.content_.ipv6_addr
		   >> obj.content_.preflen
		   >> obj.content_.reserved;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// ASNum_SubObj
	std::ostream & operator << (std::ostream & os,const ASNum_SubObj & obj)
	{
		os << "  ASNum_SubObj"                           << std::endl
		   << "   ASNumber:  " << obj.content_.as_number << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor & mc,const ASNum_SubObj & obj)
	{
		mc << obj.content_.as_number;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, ASNum_SubObj & obj)
	{
		mc >> obj.content_.as_number;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// Unnum_SubObj
	std::ostream & operator << (std::ostream &       os,
				    const Unnum_SubObj & obj)
	{
		os << "  Unnum_SubObj"                             << std::endl
		   << "   RouterId: " << obj.content_.router_id    << std::endl
		   << "   InterfId: " << obj.content_.interface_id << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &       mc,
				    const Unnum_SubObj & obj)
	{
		mc << (uint16_t)0  // reserved
		   << obj.content_.router_id
		   << obj.content_.interface_id;

		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Unnum_SubObj & obj)
	{
		mc >> obj.content_.reserved
		   >> obj.content_.router_id
		   >> obj.content_.interface_id;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// LabelERO_SubObj
	std::ostream & operator << (std::ostream &          os,
				    const LabelERO_SubObj & obj)
	{
		os << "  LabelERO_SubObj"                      << std::endl
		   << "   Upstream: " << obj.content_.upstream << std::endl
		   << "   c-type:   " << obj.content_.c_type   << std::endl;

		if (obj.content_.label) {
			Label_Class_OBJ2STREAM(os, (*obj.content_.label));
		}

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const LabelERO_SubObj & obj)
	{
		int len = sizeof(uint8_t) +         // upstream_reserved
			sizeof(obj.content_.c_type);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint8_t up_res;
		up_res  = (uint8_t)0 & 0x7f;  // reserved
		up_res |= (obj.content_.upstream & 0x01) << 7;
		mcHdr << up_res << obj.content_.c_type;
		assert(mcHdr.remainingSize() == 0);

		if (obj.content_.label) {
			//mcObj= mcObj.neighbor(obj.content_.label->length() -
			//		      objHeader::objHdrLength);
			mcObj= mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop()); // ???

			Label_Class_OBJ2STREAM(mcObj, (*obj.content_.label));
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, LabelERO_SubObj & obj)
	{
		int len = sizeof(uint8_t) +   // upstream_reserved
			sizeof(obj.content_.c_type);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint8_t up_res;
		mcHdr >> up_res >> obj.content_.c_type;
		assert(mcHdr.remainingSize() == 0);

		obj.content_.upstream = (up_res & 0x80) >> 7;
		obj.content_.reserved = up_res & 0x7f; // must be ignored

		len = mc.remainingSize() - len;

		if (len > 0) {
			//PUT the length of label
			mcObj = mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop()); // ???

			Label_Class * lbl = 0;
			Label_Class_STREAM2OBJ(mcObj, lbl,obj.content_.c_type);
			obj.content_.label = lbl;

			len -= (lbl->length() - objHeader::objHdrLength);
		}
		assert(len == 0);

		return mc;
	}
}


///////////////////////////////////////////////////////////////////////////////
//                         RRO SubObject Classes                             //
///////////////////////////////////////////////////////////////////////////////

namespace RSVP {

	//
	// (hdr) RRO SubObject
	//
	std::ostream & operator << (std::ostream &       os,
				    const RroSubObject & obj)
	{
		os << " RRO subObject"                << std::endl
		   << "  type:   " << obj.type()      << std::endl
		   << "  length: " << obj.length()    << std::endl;
		RROSUBOBJ_OBJ2STREAM(os, obj);
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &       mc,
				    const RroSubObject & obj)
	{
		// Writes Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(RroSubObject::rroSubObjHdrLength - 1));

		mcObjHdr << obj.type() << obj.length();
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(mc.stop() - mc.start() -
				 (RroSubObject::rroSubObjHdrLength - 1));
		// Writes Object
		RROSUBOBJ_OBJ2STREAM(mcObj, obj);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, RroSubObject * & obj)
	{
		// Check Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(RroSubObject::rroSubObjHdrLength - 1));
		assert(mcObjHdr.remainingSize() >=
		       RroSubObject::rroSubObjHdrLength);

		uint8_t type;
		uint8_t len;

		mcObjHdr >> type >> len;
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(len - RroSubObject::rroSubObjHdrLength);

		RROSUBOBJ_STREAM2OBJ(mcObj, obj, type);

		assert(len == obj->length());
		return mc;
	}


	//
	// RRO SubObjects
	//

	// IPv4Pref_SubObj
	std::ostream & operator << (std::ostream &          os,
				    const IPv4Addr_SubObj & obj)
	{
		os << "  IPv4Addr_SubObj"                      << std::endl
		   << "   IPv4:    " << obj.content_.ipv4_addr << std::endl
		   << "   PrefLen: " << obj.content_.preflen   << std::endl
		   << "   Flags:   " << obj.content_.flags     << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const IPv4Addr_SubObj & obj)
	{
		mc << obj.content_.ipv4_addr
		   << obj.content_.preflen
		   << obj.content_.flags;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, IPv4Addr_SubObj & obj)
	{
		mc >> obj.content_.ipv4_addr
		   >> obj.content_.preflen
		   >> obj.content_.flags;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// IPv6Addr_SubObj
	std::ostream & operator << (std::ostream &          os,
				    const IPv6Addr_SubObj & obj)
	{
		os << "  IPv6Addr_SubObj"                      << std::endl
		   << "   IPv6:    " << obj.content_.ipv6_addr << std::endl
		   << "   PrefLen: " << obj.content_.preflen   << std::endl
		   << "   Flags:   " << obj.content_.flags     << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const IPv6Addr_SubObj & obj)
	{
		mc << obj.content_.ipv6_addr
		   << obj.content_.preflen
		   << obj.content_.flags;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, IPv6Addr_SubObj & obj)
	{
		mc >> obj.content_.ipv6_addr
		   >> obj.content_.preflen
		   >> obj.content_.flags;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// Unnumbered_SubObj
	std::ostream & operator << (std::ostream &            os,
				    const Unnumbered_SubObj & obj)
	{
		os << "  Unnumbered_SubObj"                       << std::endl
		   << "   Flags:    " << obj.content_.flags       << std::endl
		   << "   RouterId: " << obj.content_.router_id   << std::endl
		   << "   InterfId: " << obj.content_.interface_id<< std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const Unnumbered_SubObj & obj)
	{
		mc << obj.content_.flags
		   << (uint8_t)0   // reserved
		   << obj.content_.router_id
		   << obj.content_.interface_id;

		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Unnumbered_SubObj & obj)
	{
		mc >> obj.content_.flags
		   >> obj.content_.reserved
		   >> obj.content_.router_id
		   >> obj.content_.interface_id;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// Label_SubObj
	std::ostream & operator << (std::ostream &       os,
				    const Label_SubObj & obj)
	{
		os << "  Label_SubObj"                     << std::endl
		   << "   Flags:  " << obj.content_.flags  << std::endl
		   << "   c-type: " << obj.content_.c_type << std::endl;

		if (obj.content_.label) {
			Label_Class_OBJ2STREAM(os, (*obj.content_.label));
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &       mc,
				    const Label_SubObj & obj)
	{
		int len = sizeof(obj.content_.flags) +
			sizeof(obj.content_.c_type);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		mcHdr << obj.content_.flags << obj.content_.c_type;
		assert(mcHdr.remainingSize() == 0);

		if (obj.content_.label) {
			//mcObj= mcObj.neighbor(obj.content_.label->length() -
			//		      objHeader::objHdrLength);
			mcObj= mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop()); // ???

			Label_Class_OBJ2STREAM(mcObj, (*obj.content_.label));
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Label_SubObj & obj)
	{
		int len = sizeof(obj.content_.flags) +
			sizeof(obj.content_.c_type);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		mcHdr >> obj.content_.flags >> obj.content_.c_type;
		assert(mcHdr.remainingSize() == 0);

		len = mc.remainingSize() - len;

		if (len > 0) {
			//PUT the length of label
			mcObj = mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop()); // ???

			Label_Class * lbl = 0;
			Label_Class_STREAM2OBJ(mcObj, lbl,obj.content_.c_type);
			obj.content_.label = lbl;

			len -= (lbl->length() - objHeader::objHdrLength);
		}
		assert(len == 0);

		return mc;
	}
}

///////////////////////////////////////////////////////////////////////////////
//                           GUNI SubObject Class                            //
///////////////////////////////////////////////////////////////////////////////

namespace RSVP {

	//
	// (hdr) GuniSubObj
	//
	std::ostream & operator << (std::ostream & os, const GuniSubObj & obj)
	{
		os << " GuniSubObj"                   << std::endl
		   << "  length:   " << obj.length()  << std::endl
		   << "  type:     " << obj.type()    << std::endl
		   << "  sub type: " << obj.subType() << std::endl;
		GUNI_SUBOBJ_OBJ2STREAM(os, obj);
		return os;
	}

	MemoryCursor & operator << (MemoryCursor & mc, const GuniSubObj & obj)
	{
		// Writes Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(SubObject::subObjHdrLength - 1));

		mcObjHdr << obj.length() << obj.type() << obj.subType();
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(mc.stop() - mc.start() -
				 (SubObject::subObjHdrLength - 1));

		// Writes Object
		GUNI_SUBOBJ_OBJ2STREAM(mcObj, obj);
		return mc;
	}

	GuniSubObj * createGuniSrcSubObj(const g2mpls_addr_t & addr)
	{
		switch (addr.type) {
			case IPv4: {
				IPv4SourceTnaAddr_SubObj * subObj;
				subObj = new IPv4SourceTnaAddr_SubObj
					(addr.value.ipv4);
				return subObj;
			}
			case IPv6: {
				IPv6SourceTnaAddr_SubObj * subObj;
				subObj = new IPv6SourceTnaAddr_SubObj
					(addr.value.ipv6);
				return subObj;
			}
			case NSAP: {
				NSAPSourceTnaAddr_SubObj * subObj;
				subObj = new NSAPSourceTnaAddr_SubObj
					(addr.value.nsap);
				return subObj;
			}
			case UNNUMBERED:
			default:
				throw(std::string("G2.ADDR type not valid for "
						  "Source Tna Address"));
				break;
		}

		return 0;
	}

	GuniSubObj * createGuniDstSubObj(const g2mpls_addr_t & addr)
	{
		switch (addr.type) {
			case IPv4: {
				IPv4DestTnaAddr_SubObj * subObj;
				subObj = new IPv4DestTnaAddr_SubObj
					(addr.value.ipv4);
				return subObj;
			}
			case IPv6: {
				IPv6DestTnaAddr_SubObj * subObj;
				subObj = new IPv6DestTnaAddr_SubObj
					(addr.value.ipv6);
				return subObj;
			}
			case NSAP: {
				NSAPDestTnaAddr_SubObj * subObj;
				subObj = new NSAPDestTnaAddr_SubObj
					(addr.value.nsap);
				return subObj;
			}
			case UNNUMBERED:
			default:
				throw(std::string("G2.ADDR type not valid for "
						  "Destination Tna Address"));
				break;
		}

		return 0;
	}

	GuniSubObj * createGuniSpcLblSubObj(const g2mpls_addr_t & data_link,
					    const label_id_t &    label)
	{
		SpcLabel_SubObj * subObj = 0;
		Label_Class *     lbl    = 0;

		try {
			lbl << label;
		} catch (...) {
			G2RSVPTE_ERR("Impossible to transform G2MPLS label "
				     "type into G2RSVPTE Label Class Obj");
			return 0;
		}

		switch (data_link.type) {
			case IPv4: {
				subObj = new SpcLabel_SubObj
					(data_link.value.ipv4.s_addr, lbl);
				break;
			}
			case UNNUMBERED: {
				subObj = new SpcLabel_SubObj
					(data_link.value.unnum, lbl);
				break;
			}
			case NSAP:
			case IPv6:
			default:
				G2RSVPTE_ERR("G2.ADDR type not valid for Data "
					     "Link Address in SPC label Obj");
				break;
		}

		return subObj;
	}

	GuniSubObj * createGuniEgrLblSubObj(const g2mpls_addr_t & data_link,
					    const label_id_t &    label)
	{
		EgressLabel_SubObj * subObj = 0;
		Label_Class *        lbl    = 0;

		try {
			lbl << label;
		} catch (...) {
			G2RSVPTE_ERR("Impossible to transform G2MPLS label "
				     "type into G2RSVPTE Label Class Obj");
			return 0;
		}

		switch (data_link.type) {
			case IPv4: {
				subObj = new EgressLabel_SubObj
					(data_link.value.ipv4.s_addr, lbl);
				break;
			}
			case UNNUMBERED: {
				subObj = new EgressLabel_SubObj
					(data_link.value.unnum, lbl);
				break;
			}
			case NSAP:
			case IPv6:
			default:
				G2RSVPTE_ERR("G2.ADDR type not valid for Data "
					     "Link Address in Egress label Obj");
				break;
		}

		return subObj;
	}

	bool createSrcG2mplsAddr(GuniSubObj & guni, g2mpls_addr_t & addr)
	{
		memset(&addr, 0, sizeof(addr));

		if (guni.type() != TYPE_SRC_TNA) {
			G2RSVPTE_ERR("GenUniSubObj is not a SrcTna Addr");
			return false;
		}

		switch (guni.subType()) {
			case SUBTYPE_IPv4_SRC_TNA: {
				addr.type       = IPv4;
				addr.preflen    = 32;
				addr.value.ipv4 =
					(static_cast<IPv4SourceTnaAddr_SubObj &>
					 (guni)).get().ipv4_addr;
				break;
			}
			case SUBTYPE_IPv6_SRC_TNA: {
				addr.type       = IPv6;
				addr.preflen    = 128;
				addr.value.ipv6 =
					(static_cast<IPv6SourceTnaAddr_SubObj &>
					 (guni)).get().ipv6_addr;
			}
			case SUBTYPE_NSAP_SRC_TNA: {
				addr.type       = NSAP;
				addr.preflen    = 160;

				int size;
				size = (static_cast<NSAPSourceTnaAddr_SubObj &>
					(guni)).get().nsap_len;

				char * data;
				data = (static_cast<NSAPSourceTnaAddr_SubObj &>
					(guni)).get().nsap_addr;

				if (!data || size > 20) {
					G2RSVPTE_ERR("NSAPGenUniSubObj has"
						     " an invalid len/data");
					return false;
				}
				for (int i = 0; i < size; i++) {
					addr.value.nsap.nsap_addr8[i] = data[i];
				}
				break;
			}
			default:
				G2RSVPTE_ERR("GenUniSubObj is not a valid "
					     "SrcTna Addr type");
				return false;;
		}

		return true;;
	}

	bool createDestG2mplsAddr(GuniSubObj & guni, g2mpls_addr_t & addr)
	{
		memset(&addr, 0, sizeof(addr));

		if (guni.type() != TYPE_DST_TNA) {
			G2RSVPTE_ERR("GenUniSubObj is not a DstTna Addr");
			return false;
		}

		switch (guni.subType()) {
			case SUBTYPE_IPv4_DST_TNA: {
				addr.type       = IPv4;
				addr.preflen    = 32;
				addr.value.ipv4 =
					(static_cast<IPv4DestTnaAddr_SubObj &>
					 (guni)).get().ipv4_addr;
				break;
			}
			case SUBTYPE_IPv6_DST_TNA: {
				addr.type       = IPv6;
				addr.preflen    = 128;
				addr.value.ipv6 =
					(static_cast<IPv6DestTnaAddr_SubObj &>
					 (guni)).get().ipv6_addr;
			}
			case SUBTYPE_NSAP_DST_TNA: {
				addr.type       = NSAP;
				addr.preflen    = 160;

				int size;
				size = (static_cast<NSAPDestTnaAddr_SubObj &>
					(guni)).get().nsap_len;

				char * data;
				data = (static_cast<NSAPDestTnaAddr_SubObj &>
					(guni)).get().nsap_addr;

				if (!data || size > 20) {
					G2RSVPTE_ERR("NSAPGenUniSubObj has"
						     " an invalid len/data");
					return false;
				}
				for (int i = 0; i < size; i++) {
					addr.value.nsap.nsap_addr8[i] = data[i];
				}
				break;
			}
			default:
				G2RSVPTE_ERR("GenUniSubObj is not a valid "
					     "DstTna Addr type");
				return false;
		}

		return true;
	}

	bool createDlLabel(SpcLabel_SubObj * spcLbl,
			   g2mpls_addr_t &   dl,
			   label_id_t &      lbl)
	{
		if (!spcLbl) {
			G2RSVPTE_ERR("Invalid SpcLabel_SubObj");
			return false;
		}

		try {
			lbl << spcLbl->get().spc_label;
		} catch(std::string e) {
			G2RSVPTE_ERR("Impossible to transform SPC Label: %s",
				     e.c_str());
			return false;
		}

		//TMP!!!! for now we transform logical port id into IPv4
		dl.type              = IPv4;
		dl.preflen           = 32;
		dl.value.ipv4.s_addr = spcLbl->get().logical_port_id;

		return true;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, GuniSubObj * & obj)
	{
		// Check Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(SubObject::subObjHdrLength - 1));
		assert(mcObjHdr.remainingSize() >= SubObject::subObjHdrLength);

		uint16_t len;
		uint8_t  type;
		uint8_t  subType;

		mcObjHdr >> len >> type >> subType;
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(len - SubObject::subObjHdrLength);

		GUNI_SUBOBJ_STREAM2OBJ(mcObj, obj, type, subType);

		assert(len == obj->length());
		return mc;
	}

	//
	// GuniSubObjects
	//

	//IPv4SourceTnaAddr_SubObj
	IPV4_ADDR_OPERATOR_IMP(IPv4SourceTnaAddr_SubObj);

	//IPv6SourceTnaAddr_SubObj
	IPV6_ADDR_OPERATOR_IMP(IPv6SourceTnaAddr_SubObj);

	//NSAPSourceTnaAddr_SubObj
	NSAP_ADDR_OPERATOR_IMP(NSAPSourceTnaAddr_SubObj);

	//IPv4DestTnaAddr_SubObj
	IPV4_ADDR_OPERATOR_IMP(IPv4DestTnaAddr_SubObj);

	//IPv6DestTnaAddr_SubObj
	IPV6_ADDR_OPERATOR_IMP(IPv6DestTnaAddr_SubObj);

	//NSAPDestTnaAddr_SubObj
	NSAP_ADDR_OPERATOR_IMP(NSAPDestTnaAddr_SubObj);

	// Diversity_SubObj
	std::ostream & operator << (std::ostream &           os,
				    const Diversity_SubObj & obj)
	{
		os << "  Diversity_SubObj"              << std::endl
		   << "   diversity: " << obj.value_.DS << std::endl;
		if (obj.value_.uni_ipv4_session) {
			os << *(static_cast<Session_Class *>
				(obj.value_.uni_ipv4_session));
		}
		if (obj.value_.lsp_tun_ipv4_send_temp) {
			os << *(static_cast<SenderTemplate_Class *>
				(obj.value_.lsp_tun_ipv4_send_temp));
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &           mc,
				    const Diversity_SubObj & obj)
	{
		uint32_t ds_resvd;
		int length = sizeof(ds_resvd);

		MemoryCursor mc2(mc);
		mc2.resize(mc.start(), mc.start() + (length - 1));
		MemoryCursor mcObj(mc2);

		ds_resvd  =  obj.value_.reserved & 0x0fffffff;
		ds_resvd |= (obj.value_.DS & 0x0f) << 28;
		mc2 << ds_resvd;
		assert(mc2.remainingSize() == 0);

		if (obj.value_.uni_ipv4_session) {
			mcObj = mcObj.neighbor(obj.value_.
					       uni_ipv4_session->length());
			mcObj << *(static_cast<Session_Class *>
				   (obj.value_.uni_ipv4_session));
		}

		if (obj.value_.lsp_tun_ipv4_send_temp) {
			mcObj = mcObj.neighbor(obj.value_.
				       lsp_tun_ipv4_send_temp->length());
			mcObj << *(static_cast<SenderTemplate_Class *>
				   (obj.value_.lsp_tun_ipv4_send_temp));
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Diversity_SubObj & obj)
	{
		uint32_t ds_resvd;
		int length = sizeof(ds_resvd);

		MemoryCursor mc2(mc);
		mc2.resize(mc.start(), mc.start() + (length - 1));

		mc2 >> ds_resvd;
		assert(mc2.remainingSize() == 0);
		obj.value_.DS       = (ds_resvd & 0xf0000000) >> 28;
		obj.value_.reserved =  ds_resvd & 0x0fffffff;

		length = mc.remainingSize() - length;

		uint8_t  cNum;
		uint8_t  cType;
		uint16_t len;

		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc2.stop() + 1,
				mc2.stop() + 1 + (objHeader::objHdrLength-1));

		while (length > 0) {

			mcObjHdr >> len >> cNum >> cType;
			assert(mcObjHdr.remainingSize() == 0);

			MemoryCursor mcObj(mc);
			mcObj.resize(mcObjHdr.start(),
				     mcObjHdr.start() + (len - 1));

			if (cNum  == G2RSVP_CLASS_SESSION  &&
			    cType == CTYPE_SESSION_UNI_IPV4) {

				assert(!obj.value_.uni_ipv4_session);
				Session_Class * sess;
				mcObj >> sess;
				obj.value_.uni_ipv4_session = static_cast
					<UNIIPv4Session_Object *>(sess);
				assert(obj.value_.uni_ipv4_session);

			} else if (cNum  == G2RSVP_CLASS_SENDER_TEMPLATE &&
				   cType == CTYPE_LSP_IPV4_SENDER_TEMPLATE) {
				assert(!obj.value_.lsp_tun_ipv4_send_temp);
				SenderTemplate_Class * st;
				mcObj >> st;
				obj.value_.lsp_tun_ipv4_send_temp = static_cast
					<IPV4LSPSenderTemplate_Object *>(st);
				assert(obj.value_.lsp_tun_ipv4_send_temp);

			} else {
				throw("Invalid Diversity_SubObj ...malformed");
			}

			length -= len;

			if (length > 0) {
				mcObjHdr = mcObj.
					neighbor(objHeader::objHdrLength);
			}
		}

		return mc;
	}

	// EgressLabel_SubObj
	std::ostream & operator << (std::ostream &             os,
				    const EgressLabel_SubObj & obj)
	{
		os << "  EgressLabel_SubObj"                       << std::endl
		   << "   Upstream:     " << obj.value_.U          << std::endl
		   << "   c-type:       " << obj.value_.label_type << std::endl
		   << "   logical port: " << obj.value_.logical_port_id
		   << std::endl;

		if (obj.value_.egress_label) {
			Label_Class_OBJ2STREAM(os, (*obj.value_.egress_label));
		}

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const EgressLabel_SubObj & obj)
	{
		int len = sizeof(uint32_t) +       // upstream/resvd/label_type
			sizeof(obj.value_.logical_port_id);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t up_res_type;
		up_res_type  = obj.value_.label_type & 0xff;
		up_res_type |= ((uint32_t)0 & 0x007fffff) << 8;
		up_res_type |= (obj.value_.U & 0x01) << 31;
		mcHdr << up_res_type << obj.value_.logical_port_id;
		assert(mcHdr.remainingSize() == 0);

		if (obj.value_.egress_label) {
			mcObj= mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop());

			Label_Class_OBJ2STREAM(mcObj,
					       (*obj.value_.egress_label));
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    EgressLabel_SubObj & obj)
	{
		int len = sizeof(uint32_t) +       // upstream/resvd/label_type
			sizeof(obj.value_.logical_port_id);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t up_res_type;
		mcHdr >> up_res_type >> obj.value_.logical_port_id;
		assert(mcHdr.remainingSize() == 0);

		obj.value_.U          = (up_res_type & 0x80000000) >> 31;
		obj.value_.reserved   = (up_res_type & 0x7fffff00) >> 8;
		obj.value_.label_type =  up_res_type & 0x000000ff;

		len = mc.remainingSize() - len;

		if (len > 0) {
			//PUT the length of egress_label
			mcObj = mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop());

			Label_Class * lbl = 0;
			Label_Class_STREAM2OBJ(mcObj,
					       lbl,
					       obj.value_.label_type);
			obj.value_.egress_label = lbl;

			len -= (lbl->length() - objHeader::objHdrLength);
		}
		assert(len == 0);

		return mc;
	}

	// SpcLabel_SubObj
	std::ostream & operator << (std::ostream &          os,
				    const SpcLabel_SubObj & obj)
	{
		os << "  SpcLabel_SubObj"                          << std::endl
		   << "   Upstream:     " << obj.value_.U          << std::endl
		   << "   c-type:       " << obj.value_.label_type << std::endl
		   << "   logical port: " << obj.value_.logical_port_id
		   << std::endl;

		if (obj.value_.spc_label) {
			Label_Class_OBJ2STREAM(os, (*obj.value_.spc_label));
		}

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const SpcLabel_SubObj & obj)
	{
		int len = sizeof(uint32_t) +       // upstream/resvd/label_type
			sizeof(obj.value_.logical_port_id);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t up_res_type;
		up_res_type  = obj.value_.label_type & 0xff;
		up_res_type |= ((uint32_t)0 & 0x007fffff) << 8;
		up_res_type |= (obj.value_.U & 0x01) << 31;
		mcHdr << up_res_type << obj.value_.logical_port_id;
		assert(mcHdr.remainingSize() == 0);

		if (obj.value_.spc_label) {
			mcObj= mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop());

			Label_Class_OBJ2STREAM(mcObj,
					       (*obj.value_.spc_label));
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, SpcLabel_SubObj & obj)
	{
		int len = sizeof(uint32_t) +       // upstream/resvd/label_type
			sizeof(obj.value_.logical_port_id);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t up_res_type;
		mcHdr >> up_res_type >> obj.value_.logical_port_id;
		assert(mcHdr.remainingSize() == 0);

		obj.value_.U          = (up_res_type & 0x80000000) >> 31;
		obj.value_.reserved   = (up_res_type & 0x7fffff00) >> 8;
		obj.value_.label_type =  up_res_type & 0x000000ff;

		len = mc.remainingSize() - len;

		if (len > 0) {
			//PUT the length of spc_label
			mcObj = mcObj.neighbor(mc.stop() - mcHdr.stop());
			assert(mcObj.stop() == mc.stop());

			Label_Class * lbl = 0;
			Label_Class_STREAM2OBJ(mcObj,
					       lbl,
					       obj.value_.label_type);
			obj.value_.spc_label = lbl;

			len -= (lbl->length() - objHeader::objHdrLength);
		}
		assert(len == 0);

		return mc;
	}

	// ServiceLevel_SubObj
	std::ostream & operator << (std::ostream &              os,
				    const ServiceLevel_SubObj & obj)
	{
		os << "  ServiceLevel_SubObj"                     << std::endl
		   << "   Serv Lvl: " << obj.value_.service_level << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &              mc,
				    const ServiceLevel_SubObj & obj)
	{
		mc << obj.value_.service_level
		   << (uint8_t) 0   // reserved
		   << (uint16_t)0;  // reserved
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &        mc,
				    ServiceLevel_SubObj & obj)
	{
		uint8_t  resvd8;
		uint16_t resvd16;

		mc >> obj.value_.service_level
		   >> resvd8        // reserved
		   >> resvd16;      // reserved
		assert(mc.remainingSize() == 0);
		return mc;
	}
}

///////////////////////////////////////////////////////////////////////////////
//                           GNS SubObject Class                             //
///////////////////////////////////////////////////////////////////////////////

namespace RSVP {

	//
	// (hdr) GnsSubObj
	//
	std::ostream & operator << (std::ostream & os, const GnsSubObj & obj)
	{
		os << " GnsUniSubObj"                 << std::endl
		   << "  length:   " << obj.length()  << std::endl
		   << "  type:     " << obj.type()    << std::endl
		   << "  sub type: " << obj.subType() << std::endl;
		GNS_SUBOBJ_OBJ2STREAM(os, obj);
		return os;
	}

	MemoryCursor & operator << (MemoryCursor & mc, const GnsSubObj & obj)
	{
		// Writes Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(SubObject::subObjHdrLength - 1));

		mcObjHdr << obj.length() << obj.type() << obj.subType();
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(mc.stop() - mc.start() -
				 (SubObject::subObjHdrLength - 1));

		// Writes Object
		GNS_SUBOBJ_OBJ2STREAM(mcObj, obj);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, GnsSubObj * & obj)
	{
		// Check Object Header
		MemoryCursor mcObjHdr(mc);
		mcObjHdr.resize(mc.start(), mc.start() +
				(SubObject::subObjHdrLength - 1));
		assert(mcObjHdr.remainingSize() >= SubObject::subObjHdrLength);

		uint16_t len;
		uint8_t  type;
		uint8_t  subType;

		mcObjHdr >> len >> type >> subType;
		assert(mcObjHdr.remainingSize() == 0);

		MemoryCursor mcObj = mcObjHdr.
			neighbor(len - SubObject::subObjHdrLength);

		GNS_SUBOBJ_STREAM2OBJ(mcObj, obj, type, subType);

		assert(len == obj->length());
		return mc;
	}

	//
	// GnsSubObjects
	//

	// Application_SubObject
	std::ostream & operator << (std::ostream &             os,
				    const Application_SubObj & obj)
	{
		os << "  Application_SubObj"                    << std::endl
		   << "   App Type: " << obj.value_.app_type    << std::endl
		   << "   App Vers: " << obj.value_.app_version << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const Application_SubObj & obj)
	{
		mc << obj.value_.app_type
		   << obj.value_.app_version;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    Application_SubObj & obj)
	{
		mc >> obj.value_.app_type
		   >> obj.value_.app_version;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	// IPv4CandidateHost_SubObj
	IPV4_ADDR_OPERATOR_IMP(IPv4CandidateHost_SubObj);

	// IPv6CandidateHost_SubObj
	IPV6_ADDR_OPERATOR_IMP(IPv6CandidateHost_SubObj);

	// NSAPCandidateHost_SubObj
	NSAP_ADDR_OPERATOR_IMP(NsapCandidateHost_SubObj);

	// FsResources_SubObj
	std::ostream & operator << (std::ostream &             os,
				    const FsResources_SubObj & obj)
	{
		os << "  FsResources_SubObj"       << std::endl
		   << "   FS name:                "
		   << obj.value_.fs_name           << std::endl
		   << "   FS type:                "
		   << obj.value_.fs_type	   << std::endl
		   << "   Range flags:            "
		   << obj.value_.range_flags	   << std::endl
		   << "   Disk space lower bound: "
		   << obj.value_.disk_space_lb	   << std::endl
		   << "   Disk space upper bound: "
		   << obj.value_.disk_space_ub	   << std::endl
		   << "   Mount point len:        "
		   << obj.value_.mount_point_len   << std::endl
		   << "   Mount point:            "
		   << obj.value_.mount_point	   << std::endl
		   << "   Mount source len:       "
		   << obj.value_.mount_source_len  << std::endl
		   << "   Mount source:           "
		   << obj.value_.mount_source      << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const FsResources_SubObj & obj)
	{
		uint32_t i;

		mc << obj.value_.fs_name
		   << obj.value_.fs_type
		   << obj.value_.range_flags
		   << obj.value_.disk_space_lb
		   << obj.value_.disk_space_ub;

		mc << obj.value_.mount_point_len;
		for (i = 0; i < obj.value_.mount_point_len; i++) {
			mc << ((uint8_t *)obj.value_.mount_point)[i];
		}

		mc << obj.value_.mount_source_len;
		for (i = 0; i < obj.value_.mount_source_len; i++) {
			mc << ((uint8_t *)obj.value_.mount_source)[i];
		}

		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    FsResources_SubObj & obj)
	{
		uint32_t i;

		mc >> obj.value_.fs_name
		   >> obj.value_.fs_type
		   >> obj.value_.range_flags
		   >> obj.value_.disk_space_lb
		   >> obj.value_.disk_space_ub;

		obj.value_.mount_point =
			new char[obj.value_.mount_point_len + 1];
		assert(obj.value_.mount_point);

		mc >> obj.value_.mount_point_len;
		for (i = 0; i < obj.value_.mount_point_len; i++) {
			mc >> ((uint8_t *)obj.value_.mount_point)[i];
		}
		((uint8_t *)obj.value_.mount_point)[i] = '\0';

		obj.value_.mount_source =
			new char[obj.value_.mount_source_len + 1];
		assert(obj.value_.mount_source);

		mc >> obj.value_.mount_source_len;
		for (i = 0; i < obj.value_.mount_source_len; i++) {
			mc >> ((uint8_t *)obj.value_.mount_source)[i];
		}
		((uint8_t *)obj.value_.mount_source)[i] = '\0';

		assert(mc.remainingSize() == 0);
		return mc;
	}

	// SystemCaps_SubObj
	std::ostream & operator << (std::ostream &            os,
				    const SystemCaps_SubObj & obj)
	{
		os << "  SystemCaps_SubObj"                      << std::endl
		   << "   OS type:    " << obj.value_.os_type    << std::endl
		   << "   OS version: " << obj.value_.os_version << std::endl
		   << "   CPU_arch:   " << obj.value_.cpu_arch   << std::endl
		   << "   E:          " << obj.value_.E          << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const SystemCaps_SubObj & obj)
	{
		uint8_t cpu_and_flag;

		cpu_and_flag = ((obj.value_.cpu_arch & 0x7F) << 1 |
				(obj.value_.E        & 0x01));
		mc << obj.value_.os_type
		   << obj.value_.os_version
		   << cpu_and_flag
		   << (uint8_t) 0   /* reserved */
		   << (uint16_t)0;  /* reserved */;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &      mc,
				    SystemCaps_SubObj & obj)
	{
		uint8_t  cpu_and_flag;
		uint8_t  resvd8;
		uint16_t resvd16;

		mc >> obj.value_.os_type
		   >> obj.value_.os_version
		   >> cpu_and_flag
		   >> resvd8
		   >> resvd16;

		obj.value_.cpu_arch = (cpu_and_flag & 0xFE) >> 1;
		obj.value_.E        = (cpu_and_flag & 0x01);

		assert(mc.remainingSize() == 0);
		return mc;
	}

// Range Obj Utils
#define RANGE_OPERATOR_IMP(OBJ_NAME)					     \
	std::ostream & operator << (std::ostream &   os,		     \
				    const OBJ_NAME & obj)		     \
	{								     \
		os << "  " #OBJ_NAME         << std::endl;		     \
		os << "   Lower Bound:     "				     \
		   << obj.value_.lower_bound << std::endl;		     \
		os << "   Upper Bound:     "				     \
		   << obj.value_.upper_bound << std::endl;		     \
		os << "   Inclusion Flags: "				     \
		   << obj.value_.SF          << std::endl;		     \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor &   mc,		     \
				    const OBJ_NAME & obj)		     \
	{								     \
		uint32_t inclusion_flags;				     \
									     \
		inclusion_flags = ((uint32_t)obj.value_.SF & 0x3) << 30;     \
									     \
		mc << obj.value_.lower_bound				     \
		   << obj.value_.upper_bound				     \
		   << inclusion_flags;				     \
									     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc,			     \
				    OBJ_NAME &     obj)			     \
	{								     \
		uint32_t inclusion_flags;				     \
									     \
		mc >> obj.value_.lower_bound				     \
		   >> obj.value_.upper_bound				     \
		   >> inclusion_flags;				     \
									     \
		obj.value_.SF = ((inclusion_flags & 0xc0000000) >> 30);	     \
									     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}

	// IndvCpuSpeed_SubObj
	RANGE_OPERATOR_IMP(IndvCpuSpeed_SubObj);

	// IndvCpuTime_SubObj
	RANGE_OPERATOR_IMP(IndvCpuTime_SubObj);

	// IndvCpuCount_SubObj
	RANGE_OPERATOR_IMP(IndvCpuCount_SubObj);

	// IndvNetBW_SubObj
	RANGE_OPERATOR_IMP(IndvNetBw_SubObj);

	// IndvPhyMem_SubObj_SubObj
	RANGE_OPERATOR_IMP(IndvPhyMem_SubObj);

	// IndvVirMem_SubObj_SubObj
	RANGE_OPERATOR_IMP(IndvVirMem_SubObj);

	// IndvDiskSpace_SubObj
	RANGE_OPERATOR_IMP(IndvDiskSpace_SubObj);

	// TotCpuTime_SubObj
	RANGE_OPERATOR_IMP(TotCpuTime_SubObj);

	// TotCpuCount_SubObj
	RANGE_OPERATOR_IMP(TotCpuCount_SubObj);

	// TotPhyMem_SubObj_SubObj
	RANGE_OPERATOR_IMP(TotPhyMem_SubObj);

	// TotVirMem_SubObj_SubObj
	RANGE_OPERATOR_IMP(TotVirMem_SubObj);

	// TotDiskSpace_SubObj
	RANGE_OPERATOR_IMP(TotDiskSpace_SubObj);

	// TotResCount_SubObj
	RANGE_OPERATOR_IMP(TotResCount_SubObj);

	// DataStaging_SubObj
	std::ostream & operator << (std::ostream &             os,
				    const DataStaging_SubObj & obj)
	{
		os << "  DataStaging_SubObj"           << std::endl
		   << "   FSname:                "
		   << obj.value_.FSname                << std::endl
		   << "   Creation flag:         "
		   << obj.value_.creation_flag         << std::endl
		   << "   Delete on termination: "
		   << obj.value_.delete_on_termination << std::endl
		   << "   File name length:      "
		   << obj.value_.file_name_len         << std::endl
		   << "   File name:             "
		   << obj.value_.file_name             << std::endl
		   << "   Source length:         "
		   << obj.value_.source_len            << std::endl
		   << "   Source:                "
		   << obj.value_.source                << std::endl
		   << "   Target length:         "
		   << obj.value_.target_len            << std::endl
		   << "   Target:                "
		   << obj.value_.target                << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const DataStaging_SubObj & obj)
	{
		uint8_t  flags;
		uint32_t i;

		flags  = (obj.value_.creation_flag & 0x7) << 5;
		flags |= (obj.value_.delete_on_termination & 0x1) << 4;

		mc << obj.value_.FSname
		   << flags;

		mc << obj.value_.file_name_len;
		for (i = 0; i < obj.value_.file_name_len; i++) {
			mc << ((uint8_t *)obj.value_.file_name)[i];
		}

		mc << obj.value_.source_len;
		for (i = 0; i < obj.value_.source_len; i++) {
			mc << ((uint8_t *)obj.value_.source)[i];
		}

		mc << obj.value_.target_len;
		for (i = 0; i < obj.value_.target_len; i++) {
			mc << ((uint8_t *)obj.value_.target)[i];
		}

		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    DataStaging_SubObj & obj)
	{
		uint8_t  flags;
		uint32_t i;

		mc >> obj.value_.FSname
		   >> flags;

		obj.value_.creation_flag         = (flags & 0xE0) >> 5;
		obj.value_.delete_on_termination = (flags & 0x10) >> 4;

		obj.value_.file_name =
			new char[obj.value_.file_name_len + 1];
		assert(obj.value_.file_name);

		mc >> obj.value_.file_name_len;
		for (i = 0; i < obj.value_.file_name_len; i++) {
			mc >> ((uint8_t *)obj.value_.file_name)[i];
		}
		((uint8_t *)obj.value_.file_name)[i] = '\0';

		obj.value_.source =
			new char[obj.value_.source_len + 1];
		assert(obj.value_.source);

		mc >> obj.value_.source_len;
		for (i = 0; i < obj.value_.source_len; i++) {
			mc >> ((uint8_t *)obj.value_.source)[i];
		}
		((uint8_t *)obj.value_.source)[i] = '\0';

		obj.value_.target =
			new char[obj.value_.target_len + 1];
		assert(obj.value_.target);

		mc >> obj.value_.target_len;
		for (i = 0; i < obj.value_.target_len; i++) {
			mc >> ((uint8_t *)obj.value_.target)[i];
		}
		((uint8_t *)obj.value_.target)[i] = '\0';

		assert(mc.remainingSize() == 0);
		return mc;
	}
}

///////////////////////////////////////////////////////////////////////////////
//                             Object Classes                                //
///////////////////////////////////////////////////////////////////////////////

namespace RSVP {

	//
	// Session Class
	//

	// IPv4Session_Object
	std::ostream & operator << (std::ostream &             os,
				    const IPv4Session_Object & obj)
	{
		os << "IPv4Session_Object"                 << std::endl
		   << " IPv4:   " << obj.value_.ipv4_addr  << std::endl
		   << " ProtId: " << obj.value_.protoId    << std::endl
		   << " flags:  " << obj.value_.flags      << std::endl
		   << " port:   " << obj.value_.port       << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const IPv4Session_Object & obj)
	{
		mc << obj.value_.ipv4_addr << obj.value_.protoId
		   << obj.value_.flags     << obj.value_.port;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    IPv4Session_Object & obj)
	{
		mc >> obj.value_.ipv4_addr >> obj.value_.protoId
		   >> obj.value_.flags     >> obj.value_.port;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// IPv6Session_Object
	std::ostream & operator << (std::ostream &             os,
				    const IPv6Session_Object & obj)
	{
		os << "IPv6Session_Object"                  << std::endl
		   << " IPv6:    " << obj.value_.ipv6_addr  << std::endl
		   << " ProtId:  " << obj.value_.protoId    << std::endl
		   << " flags:   " << obj.value_.flags      << std::endl
		   << " port:    " << obj.value_.port       << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const IPv6Session_Object & obj)
	{
		mc << obj.value_.ipv6_addr //.in6_u.u6_addr32[0]
		   << obj.value_.protoId
		   << obj.value_.flags
		   << obj.value_.port;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    IPv6Session_Object & obj)
	{
		mc >> obj.value_.ipv6_addr //.in6_u.u6_addr32[0]
		   >> obj.value_.protoId
		   >> obj.value_.flags
		   >> obj.value_.port;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// LSPTunnelIPv4Session_Object
	std::ostream & operator << (std::ostream &                      os,
				    const LSPTunnelIPv4Session_Object & obj)
	{
		os << "LSPTunnelIPv4Session_Object"                << std::endl
		   << " IPv4:     " << obj.value_.ipv4_addr        << std::endl
		   << " TunId:    " << obj.value_.tunnelID         << std::endl
		   << " ExtTunId: " << obj.value_.extendedTunnelID << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                      mc,
				    const LSPTunnelIPv4Session_Object & obj)
	{
		mc << obj.value_.ipv4_addr
		   << obj.value_.zero_field
		   << obj.value_.tunnelID
		   << obj.value_.extendedTunnelID;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &                mc,
				    LSPTunnelIPv4Session_Object & obj)
	{
		mc >> obj.value_.ipv4_addr
		   >> obj.value_.zero_field
		   >> obj.value_.tunnelID
		   >> obj.value_.extendedTunnelID;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// LSPTunnelIPv6Session_Object
	std::ostream & operator << (std::ostream &                      os,
				    const LSPTunnelIPv6Session_Object & obj)
	{
		// ...
		throw("operator<< for LSPTunnelIPv6Session_Object "
		      "MUST BE IMPLEMENTED");
	}

	MemoryCursor & operator << (MemoryCursor &                      mc,
				    const LSPTunnelIPv6Session_Object & obj)
	{
		// ...
		throw("operator<< for LSPTunnelIPv6Session_Object "
		      "MUST BE IMPLEMENTED");
	}

	MemoryCursor & operator >> (MemoryCursor &                mc,
				    LSPTunnelIPv6Session_Object & obj)
	{
		// ...
		throw("operator>> for LSPTunnelIPv6Session_Object "
		      "MUST BE IMPLEMENTED");
	}

	// UNIIPv4Session_Object
	std::ostream & operator << (std::ostream &                os,
				    const UNIIPv4Session_Object & obj)
	{
		os << "UNIIPv4Session_Object"                    << std::endl
		   << " IPv4:      " << obj.value_.ipv4_addr     << std::endl
		   << " Tunnel Id: " << obj.value_.tunnel_id     << std::endl
		   << " ExtIPv4:   " << obj.value_.ext_IPv4_addr << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                mc,
				    const UNIIPv4Session_Object & obj)
	{
		mc << obj.value_.ipv4_addr
		   << obj.value_.zero_field
		   << obj.value_.tunnel_id
		   << obj.value_.ext_IPv4_addr;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &          mc,
				    UNIIPv4Session_Object & obj)
	{
		mc >> obj.value_.ipv4_addr
		   >> obj.value_.zero_field
		   >> obj.value_.tunnel_id
		   >> obj.value_.ext_IPv4_addr;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// Label Class
	//

	// LabelMpls_Object
	std::ostream & operator << (std::ostream &           os,
				    const LabelMpls_Object & obj)
	{
		os << "LabelMpls_Object"               << std::endl
		   << " label:   " << obj.value_.label << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &           mc,
				    const LabelMpls_Object & obj)
	{
		mc << obj.value_.label;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &     mc,
				    LabelMpls_Object & obj)
	{
		mc >> obj.value_.label;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// GLabel_Object
	GLABEL_OBJ_OPERATOR_IMP(GLabel_Object);

	// WBSLabel_Object
	std::ostream & operator << (std::ostream &          os,
				    const WBSLabel_Object & obj)
	{
		os << "WBSLabel_Object"                       << std::endl
		   << " wbId:     " << obj.value_.waveband_id << std::endl
		   << " startLbl: " << obj.value_.start_label << std::endl
		   << " endLbl:   " << obj.value_.end_label   << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const WBSLabel_Object & obj)
	{
		mc << obj.value_.waveband_id
		   << obj.value_.start_label
		   << obj.value_.end_label;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &    mc,
				    WBSLabel_Object & obj)
	{
		mc >> obj.value_.waveband_id
		   >> obj.value_.start_label
		   >> obj.value_.end_label;

		assert(mc.remainingSize() == 0);

		return mc;
	}

#ifdef CRANKBACK_ON
	//
	// LSP Attributes Class
	//

	// LSPAttr_Object
	std::ostream & operator << (std::ostream &          os,
				    const LSPAttr_Object & obj)
	{
		os << "LSPAttr_Object"                   << std::endl
		   << " TLVs       : "                   << std::endl;

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				LSP_ATTR_TLV * tlv;
				tlv = static_cast<LSP_ATTR_TLV *> (*it);
				os << *tlv;
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &         mc,
				    const LSPAttr_Object & obj)
	{
		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				LSP_ATTR_TLV * tlv;
				tlv = static_cast<LSP_ATTR_TLV *> (*it);
				mc << *tlv;
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &   mc,
				    LSPAttr_Object & obj)
	{
		LSP_ATTR_TLV * tlv = 0;
		mc >> tlv;

		obj.value_.tlvs.push_back(tlv);

		return mc;
	}
#endif // CRANKBACK_ON

	//
	// Session Attribute Class
	//

	// LSPTunnelSessionAttr_Object
	std::ostream & operator << (std::ostream &                      os,
				    const LSPTunnelSessionAttr_Object & obj)
	{
		os << "LSPTunnelSessionAttr_Object"           << std::endl
		   << " SetupPrio: " << obj.value_.setup_prio << std::endl
		   << " HoldPrio:  " << obj.value_.hold_prio  << std::endl
		   << " Flags:     " << obj.value_.flags      << std::endl
		   << " Sname len: " << obj.value_.name_len   << std::endl
		   << " Sess name: " << obj.value_.name       << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                      mc,
				    const LSPTunnelSessionAttr_Object & obj)
	{
		mc << obj.value_.setup_prio << obj.value_.hold_prio
		   << obj.value_.flags      << obj.value_.name_len;
		for (uint32_t i = 0; i < obj.value_.name_len; i++) {
			mc << ((uint8_t *)obj.value_.name)[i];
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &                mc,
				    LSPTunnelSessionAttr_Object & obj)
	{
		mc >> obj.value_.setup_prio >> obj.value_.hold_prio
		   >> obj.value_.flags      >> obj.value_.name_len;

		obj.value_.name = new char[obj.value_.name_len + 1];
		assert(obj.value_.name);

		uint32_t i;
		for (i = 0; i < obj.value_.name_len; i++) {
			mc >> ((uint8_t *)obj.value_.name)[i];
		}
		((uint8_t *)obj.value_.name)[i] = '\0'; // needed ???

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// LSPTunnelRASessionAttr_Object
	std::ostream & operator << (std::ostream &                        os,
				    const LSPTunnelRASessionAttr_Object & obj)
	{
		os << "LSPTunnelRASessionAttr_Object"           << std::endl
		   << " excludeAny: " << obj.value_.exclude_any << std::endl
		   << " includeAny: " << obj.value_.include_any << std::endl
		   << " includeAll: " << obj.value_.include_all << std::endl
		   << " SetupPrio:  " << obj.value_.setup_prio  << std::endl
		   << " HoldPrio:   " << obj.value_.hold_prio   << std::endl
		   << " Flags:      " << obj.value_.flags       << std::endl
		   << " Sname len:  " << obj.value_.name_len    << std::endl
		   << " Sess name:  " << obj.value_.name        << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                        mc,
				    const LSPTunnelRASessionAttr_Object & obj)
	{
		mc << obj.value_.exclude_any << obj.value_.include_any
		   << obj.value_.include_all << obj.value_.setup_prio
		   << obj.value_.hold_prio   << obj.value_.flags
		   << obj.value_.name_len;
		for (uint32_t i = 0; i < obj.value_.name_len; i++) {
			mc << ((uint8_t *)obj.value_.name)[i];
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &                  mc,
				    LSPTunnelRASessionAttr_Object & obj)
	{
		mc >> obj.value_.exclude_any >> obj.value_.include_any
		   >> obj.value_.include_all >> obj.value_.setup_prio
		   >> obj.value_.hold_prio   >> obj.value_.flags
		   >> obj.value_.name_len;

		obj.value_.name = new char[obj.value_.name_len + 1];
		assert(obj.value_.name);

		uint32_t i;
		for (i = 0; i < obj.value_.name_len; i++) {
			mc >> ((uint8_t *)obj.value_.name)[i];
		}
		((uint8_t *)obj.value_.name)[i] = '\0'; // needed ???

		assert(mc.remainingSize() == 0);

		return mc;
	}



	//
	// Suggested Label Class
	//

	// SuggestedLabel_Object
	GLABEL_OBJ_OPERATOR_IMP(SuggestedLabel_Object);

	//
	// Upstream Label Class
	//

	// UpstreamLabel_Object
	GLABEL_OBJ_OPERATOR_IMP(UpstreamLabel_Object);

	//
	// Recovery Label Class
	//

	// RecoveryLabel_Object
	GLABEL_OBJ_OPERATOR_IMP(RecoveryLabel_Object);

	//
	// LabelRequest Class
	//

	// GLabelRequest_Object
	std::ostream & operator << (std::ostream &               os,
				    const GLabelRequest_Object & obj)
	{
		os << "GLabelRequest_Object"                    << std::endl
		   << " encType:    " << obj.value_.enc_type    << std::endl
		   << " switchType: " << obj.value_.switch_type << std::endl
		   << " gPid:       " << obj.value_.g_pid       << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &               mc,
				    const GLabelRequest_Object & obj)
	{
		mc << obj.value_.enc_type
		   << obj.value_.switch_type
		   << obj.value_.g_pid;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &         mc,
				    GLabelRequest_Object & obj)
	{
		mc >> obj.value_.enc_type
		   >> obj.value_.switch_type
		   >> obj.value_.g_pid;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// LabelSet_Object
	std::ostream & operator << (std::ostream &          os,
				    const LabelSet_Object & obj)
	{
		os << "LabelSet_Object"                       << std::endl
		   << " action:    " << obj.value_.action     << std::endl
		   << " reserved:  " << obj.value_.reserved   << std::endl
		   << " labelType: " << obj.value_.label_type << std::endl
		   << " Labels:    "                          << std::endl;

		std::list<Label_Class *>::const_iterator it;
		for (it = obj.value_.labels.begin();
		     it != obj.value_.labels.end();
		     it++) {
			if (*it) {
				Label_Class_OBJ2STREAM(os, (*(*it)));
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &          mc,
				    const LabelSet_Object & obj)
	{
		int len = sizeof(uint32_t); // act_res_label

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t act_res_label;
		act_res_label  = (obj.value_.action & 0x000000ff) << 24;
		act_res_label |= obj.value_.label_type & 0x00003fff;
		mcHdr << act_res_label;
		assert(mcHdr.remainingSize() == 0);

		std::list<Label_Class *>::const_iterator it;
		for (it = obj.value_.labels.begin();
		     it != obj.value_.labels.end();
		     it++) {
			if (*it) {
				mcObj= mcObj.neighbor((*it)->length() -
						      objHeader::objHdrLength);
				Label_Class_OBJ2STREAM(mcObj, (*(*it)));
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, LabelSet_Object & obj)
	{
		int len = sizeof(uint32_t); // act_res_label

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t act_res_label;
		mcHdr >> act_res_label;
		assert(mcHdr.remainingSize() == 0);

		obj.value_.action     = (act_res_label & 0xff000000) >> 24;
		obj.value_.reserved   = (act_res_label & 0x00fffc00) >> 10;
		obj.value_.label_type =  act_res_label & 0x00003fff;

		len = mc.remainingSize() - len;

		while (len > 0) {

			// TODO fix!!! PUT the real length of subChannels
			mcObj = mcObj.neighbor(sizeof(uint32_t));

			Label_Class * lbl = 0;
			Label_Class_STREAM2OBJ(mcObj,
					       lbl,
					       obj.value_.label_type);
			obj.value_.labels.push_back(lbl);

			len -= (lbl->length() - objHeader::objHdrLength);
		}

		return mc;
	}

	// AcceptableLabelSet_Object
	std::ostream & operator << (std::ostream &                    os,
				    const AcceptableLabelSet_Object & obj)
	{
		os << "AcceptableLabelSet_Object"             << std::endl
		   << " action:    " << obj.value_.action     << std::endl
		   << " reserved:  " << obj.value_.reserved   << std::endl
		   << " labelType: " << obj.value_.label_type << std::endl
		   << " Labels:    "                          << std::endl;

		std::list<Label_Class *>::const_iterator it;
		for (it = obj.value_.labels.begin();
		     it != obj.value_.labels.end();
		     it++) {
			if (*it) {
				Label_Class_OBJ2STREAM(os, (*(*it)));
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                    mc,
				    const AcceptableLabelSet_Object & obj)
	{
		int len = sizeof(uint32_t); // act_res_label

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t act_res_label;
		act_res_label  = (obj.value_.action & 0x000000ff) << 24;
		act_res_label |= obj.value_.label_type & 0x00003fff;
		mcHdr << act_res_label;
		assert(mcHdr.remainingSize() == 0);

		std::list<Label_Class *>::const_iterator it;
		for (it = obj.value_.labels.begin();
		     it != obj.value_.labels.end();
		     it++) {
			if (*it) {
				mcObj= mcObj.neighbor((*it)->length() -
						      objHeader::objHdrLength);
				Label_Class_OBJ2STREAM(mcObj, (*(*it)));
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &              mc,
				    AcceptableLabelSet_Object & obj)
	{
		int len = sizeof(uint32_t); // act_res_label

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));
		MemoryCursor mcObj(mcHdr);

		uint32_t act_res_label;
		mcHdr >> act_res_label;
		assert(mcHdr.remainingSize() == 0);

		obj.value_.action     = (act_res_label & 0xff000000) >> 24;
		obj.value_.reserved   = (act_res_label & 0x00fffc00) >> 10;
		obj.value_.label_type =  act_res_label & 0x00003fff;

		len = mc.remainingSize() - len;

		while (len > 0) {

			// TODO fix!!! PUT the real length of subChannels
			mcObj = mcObj.neighbor(sizeof(uint32_t));

			Label_Class * lbl = 0;
			Label_Class_STREAM2OBJ(mcObj,
					       lbl,
					       obj.value_.label_type);
			obj.value_.labels.push_back(lbl);

			len -= (lbl->length() - objHeader::objHdrLength);
		}

		return mc;
	}

	//
	// TimeValues Class
	//

	// TimeValues_Object
	std::ostream & operator << (std::ostream &            os,
				    const TimeValues_Object & obj)
	{
		os << "TimeValues_Object" << std::endl
		   << " refresh period: " << obj.value_.refresh_period
		   << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const TimeValues_Object & obj)
	{
		mc << obj.value_.refresh_period;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &      mc,
				    TimeValues_Object & obj)
	{
		mc >> obj.value_.refresh_period;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// Rsvp_Hop Class
	//

	// IPv4RsvpHop_Object
	std::ostream & operator << (std::ostream &             os,
				    const IPv4RsvpHop_Object & obj)
	{
		os << "IPv4RsvpHop_Object"                       << std::endl
		   << " IPv4:      " << obj.value_.ipv4_next_hop << std::endl
		   << " IF handle: " << obj.value_.if_handle     << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const IPv4RsvpHop_Object & obj)
	{
		mc << obj.value_.ipv4_next_hop
		   << obj.value_.if_handle;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    IPv4RsvpHop_Object & obj)
	{
		mc >> obj.value_.ipv4_next_hop
		   >> obj.value_.if_handle;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// IPv6RsvpHop_Object
	std::ostream & operator << (std::ostream &             os,
				    const IPv6RsvpHop_Object & obj)
	{
		os << "IPv6RsvpHop_Object"                       << std::endl
		   << " IPv6:      " << obj.value_.ipv6_next_hop << std::endl
		   << " IF handle: " << obj.value_.if_handle     << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const IPv6RsvpHop_Object & obj)
	{
		mc << obj.value_.ipv6_next_hop
		   << obj.value_.if_handle;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    IPv6RsvpHop_Object & obj)
	{
		mc >> obj.value_.ipv6_next_hop
		   >> obj.value_.if_handle;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// IFIDIPv4RsvpHop_Object
	std::ostream & operator << (std::ostream &                 os,
				    const IFIDIPv4RsvpHop_Object & obj)
	{
		os << "IFIDIPv4RsvpHop_Object" << std::endl
		   << " next prev IPv4: "      << obj.value_.next_prev_ipv4_hop
		   << std::endl
		   << " log IF handle:  "      << obj.value_.logical_if_hdl
		   << std::endl
		   << " TLVs:           "      << std::endl;

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				os << *(*it);
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                 mc,
				    const IFIDIPv4RsvpHop_Object & obj)
	{
		int len = sizeof(obj.value_.next_prev_ipv4_hop) +
			sizeof(obj.value_.logical_if_hdl);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		MemoryCursor mcObj(mcHdr); //NB pos. here! (because of current)
		mcHdr << obj.value_.next_prev_ipv4_hop
		      << obj.value_.logical_if_hdl;
		assert(mcHdr.remainingSize() == 0);

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				mcObj = mcObj.neighbor((*it)->length());
				mcObj << *(*it);
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &           mc,
				    IFIDIPv4RsvpHop_Object & obj)
	{
		int len = sizeof(obj.value_.next_prev_ipv4_hop) +
			sizeof(obj.value_.logical_if_hdl);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		mcHdr >> obj.value_.next_prev_ipv4_hop
		      >> obj.value_.logical_if_hdl;
		assert(mcHdr.remainingSize() == 0);

		len = mc.remainingSize() - len;

		uint16_t tlv_type;
		uint16_t tlv_len;

		MemoryCursor mcTlvHdr(mcHdr);
		MemoryCursor mcTlv(mcTlvHdr);

		while (len > 0) {

			mcTlvHdr = mcTlv.neighbor(TLV::tlvHdrLength);

			mcTlvHdr >> tlv_type >> tlv_len;
			assert(mcTlvHdr.remainingSize() == 0);

			mcTlv = mc;
			mcTlv.resize(mcTlvHdr.start(),
				     mcTlvHdr.start() + (tlv_len - 1));

			TLV * tlv = 0;
			mcTlv >> tlv;

			obj.value_.tlvs.push_back(tlv);
			len -= tlv->length();
		}

		return mc;
	}

	// IFIDIPv6RsvpHop_Object
	std::ostream & operator << (std::ostream &                 os,
				    const IFIDIPv6RsvpHop_Object & obj)
	{
		os << "IFIDIPv6RsvpHop_Object" << std::endl
		   << " next prev IPv6: "      << obj.value_.next_prev_ipv6_hop
		   << std::endl
		   << " log IF handle:  "      << obj.value_.logical_if_hdl
		   << std::endl
		   << " TLVs:           "      << std::endl;

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				os << *(*it);
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                 mc,
				    const IFIDIPv6RsvpHop_Object & obj)
	{
		int len = sizeof(obj.value_.next_prev_ipv6_hop) +
			sizeof(obj.value_.logical_if_hdl);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		MemoryCursor mcObj(mcHdr); //NB pos. here! (because of current)
		mcHdr << obj.value_.next_prev_ipv6_hop
		      << obj.value_.logical_if_hdl;
		assert(mcHdr.remainingSize() == 0);

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				mcObj = mcObj.neighbor((*it)->length());
				mcObj << *(*it);
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &           mc,
				    IFIDIPv6RsvpHop_Object & obj)
	{
		int len = sizeof(obj.value_.next_prev_ipv6_hop) +
			sizeof(obj.value_.logical_if_hdl);

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		mcHdr >> obj.value_.next_prev_ipv6_hop
		      >> obj.value_.logical_if_hdl;
		assert(mcHdr.remainingSize() == 0);

		len = mc.remainingSize() - len;

		uint16_t tlv_type;
		uint16_t tlv_len;

		MemoryCursor mcTlvHdr(mcHdr);
		MemoryCursor mcTlv(mcTlvHdr);

		while (len > 0) {

			mcTlvHdr = mcTlv.neighbor(TLV::tlvHdrLength);

			mcTlvHdr >> tlv_type >> tlv_len;
			assert(mcTlvHdr.remainingSize() == 0);

			mcTlv = mc;
			mcTlv.resize(mcTlvHdr.start(),
				     mcTlvHdr.start() + (tlv_len - 1));

			TLV * tlv = 0;
			mcTlv >> tlv;

			obj.value_.tlvs.push_back(tlv);
			len -= tlv->length();
		}

		return mc;
	}

	//
	// ExplicitRoute Class
	//

	// ExplicitRoute_Object
	ERO_RRO_OBJ_OPERATOR_IMP(ExplicitRoute_Object,
				 EroSubObject,
				 EroSubObject::eroSubObjHdrLength);

	//
	// RecordedRoute Class
	//

	// RecordedRoute_Object
	ERO_RRO_OBJ_OPERATOR_IMP(RecordedRoute_Object,
				 RroSubObject,
				 RroSubObject::rroSubObjHdrLength);

	//
	// FilterSpec Class
	//

	//IPV6FlowFilterSpec_Object
	IPV6FLOW_OBJ_OPERATOR_IMP(IPV6FlowFilterSpec_Object);

	//IPV4FilterSpec_Object
	IPV4_PORT_OBJ_OPERATOR_IMP(IPV4FilterSpec_Object);

	//IPV6FilterSpec_Object
	IPV6_PORT_OBJ_OPERATOR_IMP(IPV6FilterSpec_Object);

	//IPV4LSPFilterSpec_Object
	IPV4LSP_OBJ_OPERATOR_IMP(IPV4LSPFilterSpec_Object);

	//IPV6LSPFilterSpec_Object
	IPV6LSP_OBJ_OPERATOR_IMP(IPV6LSPFilterSpec_Object);

	//
	// SenderTemplate Class
	//

	//IPV6FlowSenderTemplate_Object
	IPV6FLOW_OBJ_OPERATOR_IMP(IPV6FlowSenderTemplate_Object);

	//IPV4SenderTemplate_Object
	IPV4_PORT_OBJ_OPERATOR_IMP(IPV4SenderTemplate_Object);

	//IPV6SenderTemplate_Object
	IPV6_PORT_OBJ_OPERATOR_IMP(IPV6SenderTemplate_Object);

	//IPV4LSPSenderTemplate_Object
	IPV4LSP_OBJ_OPERATOR_IMP(IPV4LSPSenderTemplate_Object);

	//IPV6LSPSenderTemplate_Object
	IPV6LSP_OBJ_OPERATOR_IMP(IPV6LSPSenderTemplate_Object);

	//
	// FlowSpec Class
	//

	//IntServFlowSpec_Object
	std::ostream & operator << (std::ostream &                 os,
				    const IntServFlowSpec_Object & obj)
	{
		os << "IntServFlowSpec_Object"            << std::endl
		   << " msg_version:         "
		   << obj.value_.data.msg_version         << std::endl
		   << " length:              "
		   << obj.value_.data.length              << std::endl
		   << " service_header:      "
		   << obj.value_.data.service_header      << std::endl
		   << " service_data_length: "
		   << obj.value_.data.service_data_length << std::endl
		   << " param_id_127:        "
		   << obj.value_.data.param_id_127        << std::endl
		   << " param127_flags:      "
		   << obj.value_.data.param127_flags      << std::endl
		   << " param127_length:     "
		   << obj.value_.data.param127_length     << std::endl
		   << " token_bucket_rate:   "
		   << obj.value_.data.token_bucket_rate   << std::endl
		   << " token_bucket_size:   "
		   << obj.value_.data.token_bucket_size   << std::endl
		   << " peak_data_rate:      "
		   << obj.value_.data.peak_data_rate      << std::endl
		   << " min_policed_unit:    "
		   << obj.value_.data.min_policed_unit    << std::endl
		   << " max_policed_unit:    "
		   << obj.value_.data.max_policed_unit    << std::endl;

		if (obj.value_.data.service_header == GUARANTEED_SERVICE) {
			if (obj.value_.param_id_130) {
				os << " param_id_130:        "
				   << *obj.value_.param_id_130 << std::endl;
			}
			if (obj.value_.param130_flags) {
				os << " param130_flags:      "
				   << *obj.value_.param130_flags << std::endl;
			}
			if (obj.value_.param130_length) {
				os << " param130_length:     "
				   << *obj.value_.param130_length << std::endl;
			}
			if (obj.value_.rate) {
				os << " rate:                "
				   << *obj.value_.rate << std::endl;
			}
			if (obj.value_.slack_term) {
				os << " slack_term:          "
				   << *obj.value_.slack_term << std::endl;
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                 mc,
				    const IntServFlowSpec_Object & obj)
	{
		uint16_t vers_res;
		vers_res  = (uint16_t)0 & 0x0fff; // reserverd12 ... useless!!!
		vers_res |= (obj.value_.data.msg_version & 0x0f) << 12;

		mc << vers_res
		   << obj.value_.data.length
		   << obj.value_.data.service_header
		   << (uint8_t)0                  // zero_bit & reserved7;
		   << obj.value_.data.service_data_length
		   << obj.value_.data.param_id_127
		   << obj.value_.data.param127_flags
		   << obj.value_.data.param127_length
		   << obj.value_.data.token_bucket_rate
		   << obj.value_.data.token_bucket_size
		   << obj.value_.data.peak_data_rate
		   << obj.value_.data.min_policed_unit
		   << obj.value_.data.max_policed_unit;

		if (obj.value_.data.service_header == GUARANTEED_SERVICE) {
			if (obj.value_.param_id_130) {
				mc << *obj.value_.param_id_130;
			}
			if (obj.value_.param130_flags) {
				mc << *obj.value_.param130_flags;
			}
			if (obj.value_.param130_length) {
				mc << *obj.value_.param130_length;
			}
			if (obj.value_.rate) {
				mc << *obj.value_.rate;
			}
			if (obj.value_.slack_term) {
				mc << *obj.value_.slack_term;
			}
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &           mc,
				    IntServFlowSpec_Object & obj)
	{
		uint16_t vers_res;
		uint8_t  zero_res;

		mc >> vers_res
		   >> obj.value_.data.length
		   >> obj.value_.data.service_header
		   >> zero_res                         // zero_bit & reserved7;
		   >> obj.value_.data.service_data_length
		   >> obj.value_.data.param_id_127
		   >> obj.value_.data.param127_flags
		   >> obj.value_.data.param127_length
		   >> obj.value_.data.token_bucket_rate
		   >> obj.value_.data.token_bucket_size
		   >> obj.value_.data.peak_data_rate
		   >> obj.value_.data.min_policed_unit
		   >> obj.value_.data.max_policed_unit;

		obj.value_.data.msg_version = (vers_res & 0xf000) >> 12;
		obj.value_.data.reserved12  =  vers_res & 0x0fff;

		obj.value_.data.zero_bit  = (zero_res & 0x80) >> 7;
		obj.value_.data.reserved7 = zero_res & 0x7f;

		if (obj.value_.data.service_header == GUARANTEED_SERVICE) {
			obj.value_.param_id_130 = new uint8_t;
			assert(obj.value_.param_id_130);
			mc >> *obj.value_.param_id_130;

			obj.value_.param130_flags = new uint8_t;
			assert(obj.value_.param130_flags);
			mc >> *obj.value_.param130_flags;

			obj.value_.param130_length = new uint16_t;
			assert(obj.value_.param130_length);
			mc >> *obj.value_.param130_length;

			obj.value_.rate = new uint32_t;
			assert(obj.value_.rate);
			mc >> *obj.value_.rate;

			obj.value_.slack_term = new uint32_t;
			assert(obj.value_.slack_term);
			mc >> *obj.value_.slack_term;
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// SenderTSpec Class
	//

	//SenderTSpec_Object
	std::ostream & operator << (std::ostream &             os,
				    const SenderTSpec_Object & obj)
	{
		os << "SenderTSpec_Object"           << std::endl
		   << " msg_version:         "
		   << obj.value_.msg_version         << std::endl
		   << " length:              "
		   << obj.value_.length              << std::endl
		   << " service_header:      "
		   << obj.value_.service_header      << std::endl
		   << " service_data_length: "
		   << obj.value_.service_data_length << std::endl
		   << " param_id_127:        "
		   << obj.value_.param_id_127        << std::endl
		   << " param127_flags:      "
		   << obj.value_.param127_flags      << std::endl
		   << " param127_length:     "
		   << obj.value_.param127_length     << std::endl
		   << " token_bucket_rate:   "
		   << obj.value_.token_bucket_rate   << std::endl
		   << " token_bucket_size:   "
		   << obj.value_.token_bucket_size   << std::endl
		   << " peak_data_rate:      "
		   << obj.value_.peak_data_rate      << std::endl
		   << " min_policed_unit:    "
		   << obj.value_.min_policed_unit    << std::endl
		   << " max_policed_unit:    "
		   << obj.value_.max_policed_unit    << std::endl;

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const SenderTSpec_Object & obj)
	{
		uint16_t vers_res;
		vers_res  = (uint16_t)0 & 0x0fff; // reserverd12 ... useless!!!
		vers_res |= (obj.value_.msg_version & 0x0f) << 12;

		mc << vers_res
		   << obj.value_.length
		   << obj.value_.service_header
		   << (uint8_t)0                  // zero_bit & reserved7;
		   << obj.value_.service_data_length
		   << obj.value_.param_id_127
		   << obj.value_.param127_flags
		   << obj.value_.param127_length
		   << obj.value_.token_bucket_rate
		   << obj.value_.token_bucket_size
		   << obj.value_.peak_data_rate
		   << obj.value_.min_policed_unit
		   << obj.value_.max_policed_unit;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    SenderTSpec_Object & obj)
	{
		uint16_t vers_res;
		uint8_t  zero_res;

		mc >> vers_res
		   >> obj.value_.length
		   >> obj.value_.service_header
		   >> zero_res                         // zero_bit & reserved7;
		   >> obj.value_.service_data_length
		   >> obj.value_.param_id_127
		   >> obj.value_.param127_flags
		   >> obj.value_.param127_length
		   >> obj.value_.token_bucket_rate
		   >> obj.value_.token_bucket_size
		   >> obj.value_.peak_data_rate
		   >> obj.value_.min_policed_unit
		   >> obj.value_.max_policed_unit;

		obj.value_.msg_version = (vers_res & 0xf000) >> 12;
		obj.value_.reserved12  =  vers_res & 0x0fff;

		obj.value_.zero_bit  = (zero_res & 0x80) >> 7;
		obj.value_.reserved7 = zero_res & 0x7f;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// Protection Class
	//

	// Protection_Object
	std::ostream & operator << (std::ostream &            os,
				    const Protection_Object & obj)
	{
		os << "Protection_Object"                      << std::endl
		   << " secondary:  " << obj.value_.secondary  << std::endl
		   << " link flags: " << obj.value_.link_flags << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const Protection_Object & obj)
	{
		uint32_t sec_res_flags;
		sec_res_flags  =  obj.value_.link_flags & 0x3f;
		sec_res_flags |= (obj.value_.reserved & 0x01ffffff) << 6;
		sec_res_flags |= (obj.value_.secondary & 0x01) << 31;

		mc << sec_res_flags;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &      mc,
				    Protection_Object & obj)
	{
		uint32_t sec_res_flags;
		mc >> sec_res_flags;

		obj.value_.link_flags =  sec_res_flags & 0x0000003f;
		obj.value_.reserved   = (sec_res_flags & 0x7fffffc0) >> 6;
		obj.value_.secondary  = (sec_res_flags & 0x80000000) >> 31;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// AdminStatus Class
	//

	// AdminStatus_Object
	std::ostream & operator << (std::ostream &             os,
				    const AdminStatus_Object & obj)
	{
		os << "AdminStatus_Object"             << std::endl
		   << " reflect:    " << obj.value_.R  << std::endl
		   << " testing:    " << obj.value_.T  << std::endl
		   << " admin down: " << obj.value_.A  << std::endl
		   << " deletion:   " << obj.value_.D  << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const AdminStatus_Object & obj)
	{
		uint32_t r_res_t_a_d;
		r_res_t_a_d  =  obj.value_.D & 0x01;
		r_res_t_a_d |= (obj.value_.A & 0x01) << 1;
		r_res_t_a_d |= (obj.value_.T & 0x01) << 2;
		r_res_t_a_d |= (obj.value_.reserved & 0x0fffffff) << 3;
		r_res_t_a_d |= (obj.value_.R & 0x01) << 31;

		mc << r_res_t_a_d;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    AdminStatus_Object & obj)
	{
		uint32_t r_res_t_a_d;
		mc >> r_res_t_a_d;

		obj.value_.D        =  r_res_t_a_d & 0x00000001;
		obj.value_.A        = (r_res_t_a_d & 0x00000002) >> 1;
		obj.value_.T        = (r_res_t_a_d & 0x00000004) >> 2;
		obj.value_.reserved = (r_res_t_a_d & 0x7ffffff8) >> 3;
		obj.value_.R        = (r_res_t_a_d & 0x80000000) >> 31;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// GnsCallExt Class
	//

	// GnsCallExt_Object
	std::ostream & operator << (std::ostream &            os,
				    const GnsCallExt_Object & obj)
	{
		os << "GnsCallExt_Object"                          << std::endl
		   << " trans id:     " << obj.value_.gns_trans_id << std::endl
		   << " call type:    " << obj.value_.call_type    << std::endl
		   << " start time:   " << obj.value_.start_time   << std::endl
		   << " duration:     " << obj.value_.duration     << std::endl
		   << " job name len: " << obj.value_.job_name_len << std::endl
		   << " job name:     " << obj.value_.job_name     << std::endl
		   << " job proj len: " << obj.value_.job_proj_len << std::endl
		   << " job project:  " << obj.value_.job_project  << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const GnsCallExt_Object & obj)
	{
		mc << obj.value_.gns_trans_id
		   << obj.value_.call_type
		   << obj.value_.start_time
		   << obj.value_.duration
		   << obj.value_.job_name_len;

		uint32_t i;
		for (i = 0; i < obj.value_.job_name_len; i++) {
			mc << ((uint8_t *)obj.value_.job_name)[i];
		}

		mc << obj.value_.job_proj_len;

		for (i = 0; i < obj.value_.job_proj_len; i++) {
			mc << ((uint8_t *)obj.value_.job_project)[i];
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, GnsCallExt_Object & obj)
	{
		mc >> obj.value_.gns_trans_id
		   >> obj.value_.call_type
		   >> obj.value_.start_time
		   >> obj.value_.duration
		   >> obj.value_.job_name_len;

		obj.value_.job_name = new char[obj.value_.job_name_len + 1];
		assert(obj.value_.job_name);

		uint32_t i;
		for (i = 0; i < obj.value_.job_name_len; i++) {
			mc >> ((uint8_t *)obj.value_.job_name)[i];
		}
		((uint8_t *)obj.value_.job_name)[i] = '\0'; // needed ???

		mc >> obj.value_.job_proj_len;

		obj.value_.job_project = new char[obj.value_.job_proj_len + 1];
		assert(obj.value_.job_project);

		for (i = 0; i < obj.value_.job_proj_len; i++) {
			mc >> ((uint8_t *)obj.value_.job_project)[i];
		}
		((uint8_t *)obj.value_.job_project)[i] = '\0'; // needed ???

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// Integrity Class
	//

	// Integrity_Object
	std::ostream & operator << (std::ostream &           os,
				    const Integrity_Object & obj)
	{
		os << "Integrity_Object"                       << std::endl
		   << " flags:      " << obj.value_.flags      << std::endl
		   << " key Id:     " << obj.value_.key_id     << std::endl
		   << " seq number: " << obj.value_.seq_number << std::endl
		   << " digest:     " << std::hex;

		uint32_t i;
		for (i = 0; i < MSG_DIGEST_LEN; i++) {
			os << std::setw(2) << std::setfill('0')
			   << ((uint8_t *)obj.value_.digest)[i];
		}
		os << std::dec << std::endl;

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &           mc,
				    const Integrity_Object & obj)
	{
		uint16_t key_first16;
		uint32_t key_second32;

		key_first16  = (obj.value_.key_id >> 32) & 0x0000ffff;
		key_second32 = obj.value_.key_id & 0xffffffff;

		mc << obj.value_.flags
		   << (uint8_t)0
		   << key_first16
		   << key_second32
		   << obj.value_.seq_number;

		uint32_t i;
		for (i = 0; i < MSG_DIGEST_LEN; i+=sizeof(uint8_t)) {
			mc << ((uint8_t *)obj.value_.digest)[i];
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Integrity_Object & obj)
	{
		uint16_t key_first16;
		uint32_t key_second32;

		mc >> obj.value_.flags
		   >> obj.value_.reserved
		   >> key_first16
		   >> key_second32
		   >> obj.value_.seq_number;

		obj.value_.key_id  = key_second32;
		obj.value_.key_id |= ((uint64_t)key_first16 << 32);

		uint32_t i;
		for (i = 0; i < MSG_DIGEST_LEN; i+=sizeof(uint8_t)) {
			mc >> ((uint8_t *)obj.value_.digest)[i];
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// GeneralisedUni Class
	//

	// GeneralisedUni_Object
	GUNI_GNS_OBJ_OPERATOR_IMP(GeneralisedUni_Object,
				  GuniSubObj,
				  SubObject::subObjHdrLength);

	//
	// GnsUni Class
	//

	// GnsUni_Object
	GUNI_GNS_OBJ_OPERATOR_IMP(GnsUni_Object,
				  GnsSubObj,
				  SubObject::subObjHdrLength);


	//
	// CallOptions_Class
	//

	// CallOptions_Object
	std::ostream & operator << (std::ostream &             os,
				    const CallOptions_Object & obj)
	{
		os << "CallOptions_Object"      << std::endl
		   << " call ops:     "
		   << obj.value_.call_ops_flags << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &             mc,
				    const CallOptions_Object & obj)
	{
		uint32_t call_ops_flags;

		call_ops_flags  = 0;
		call_ops_flags |= (obj.value_.call_ops_flags & 0xFF);

		mc << call_ops_flags;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &       mc,
				    CallOptions_Object & obj)
	{
		uint32_t call_ops_flags;

		mc >> call_ops_flags;

		obj.value_.call_ops_flags  = (call_ops_flags & 0xFF);

		assert(mc.remainingSize() == 0);

		return mc;
	}


	//
	// CallId_Class
	//

	// CallIdOpSpec_Object
	std::ostream & operator << (std::ostream &              os,
				    const CallIdOpSpec_Object & obj)
	{
		os << "CallIdOpSpec"                                 << std::endl
		   << " Addr type:     " << obj.value_.addr_type     << std::endl
		   << " src LSR addr:  " << obj.value_.src_lsr_addr  << std::endl
		   << " Call local id: " << obj.value_.local_id      << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &              mc,
				    const CallIdOpSpec_Object & obj)
	{
		mc << obj.value_.addr_type
		   << (uint8_t)  0
		   << (uint16_t) 0;

		uint32_t i;
		for (i = 0; i < obj.value_.src_lsr_addr_len; i++) {
			mc << ((uint8_t *)obj.value_.src_lsr_addr)[i];
		}

		mc << obj.value_.local_id;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &        mc,
				    CallIdOpSpec_Object & obj)
	{
		uint8_t   reserved8;
		uint16_t  reserved16;


		mc >> obj.value_.addr_type
		   >> reserved8
		   >> reserved16;

		// length is inferred
		// len = remaining size of the buffer - call id length
		obj.value_.src_lsr_addr_len = mc.remainingSize() - 8;

		obj.value_.src_lsr_addr =
			new char[obj.value_.src_lsr_addr_len + 1];
		assert(obj.value_.src_lsr_addr);

		uint32_t i;
		for (i = 0; i < obj.value_.src_lsr_addr_len; i++) {
			mc >> ((uint8_t *)obj.value_.src_lsr_addr)[i];
		}
		((uint8_t *)obj.value_.src_lsr_addr)[i] = '\0';

		mc >> obj.value_.local_id;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// CallIdGlobal_Object
	std::ostream & operator << (std::ostream &              os,
				    const CallIdGlobal_Object & obj)
	{
		os << "CallIdGlobal"                 << std::endl
		   << " Addr type:           "
		   << obj.value_.addr_type           << std::endl
		   << " ITU country code:    "
		   << obj.value_.itu_country_code    << std::endl
		   << " ITU carrier code:    "
		   << obj.value_.itu_carrier_code    << std::endl
		   << " Unique access point: "
		   << obj.value_.unique_access_point << std::endl
		   << " src LSR addr:        "
		   << obj.value_.src_lsr_addr        << std::endl
		   << " Call local id:       "
		   << obj.value_.local_id            << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &              mc,
				    const CallIdGlobal_Object & obj)
	{
		uint32_t type_is;
		uint32_t type_ns1;
		uint32_t type_ns2;
		uint32_t type_ns3;

		type_is   = (obj.value_.addr_type        & 0xFF)       << 24;
		type_is  |= (obj.value_.itu_country_code & 0x00FFFFFF) << 0;

		type_ns1  = (uint32_t)(((obj.value_.itu_carrier_code &
					 0x0000FFFFFFFF0000ull) >> 16) << 0);

		type_ns2  = (uint32_t)((obj.value_.itu_carrier_code &
					0x000000000000FFFFull) >> 0) << 16;

		type_ns2 |= (uint32_t)((obj.value_.unique_access_point &
					0x0000FFFF00000000ull) >> 32) << 0;

		type_ns3  = (uint32_t)((obj.value_.unique_access_point &
					0x00000000FFFFFFFFull) >> 0) << 0;

		mc << type_is << type_ns1 << type_ns2 << type_ns3;

		uint32_t i;
		for (i = 0; i < obj.value_.src_lsr_addr_len; i++) {
			mc << ((uint8_t *)obj.value_.src_lsr_addr)[i];
		}

		mc << obj.value_.local_id;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &        mc,
				    CallIdGlobal_Object & obj)
	{
		uint32_t type_is;
		uint32_t type_ns1;
		uint32_t type_ns2;
		uint32_t type_ns3;

		mc >> type_is >> type_ns1 >> type_ns2 >> type_ns3;

		obj.value_.addr_type         = (type_is & 0xFF000000) >> 24;
		obj.value_.itu_country_code  = (type_is & 0x00FFFFFF) >> 0;

		obj.value_.itu_carrier_code  = (uint64_t) 0;
		obj.value_.itu_carrier_code |= (uint64_t)((type_ns1 &
						      0xFFFFFFFF) >> 0)  << 16;
		obj.value_.itu_carrier_code |= (uint64_t)((type_ns2 &
						      0xFFFF0000) >> 16) << 0;

		obj.value_.unique_access_point  = (uint64_t) 0;
		obj.value_.unique_access_point |= (uint64_t)((type_ns2 &
						      0x0000FFFF) >> 0) << 32;
		obj.value_.unique_access_point |= (uint64_t)((type_ns3 &
						      0xFFFFFFFF) >> 0) << 0;

		// length is inferred
		// len = remaining size of the buffer - call id length
		obj.value_.src_lsr_addr_len = mc.remainingSize() - 8;

		obj.value_.src_lsr_addr =
			new char[obj.value_.src_lsr_addr_len + 1];
		assert(obj.value_.src_lsr_addr);

		uint32_t i;
		for (i = 0; i < obj.value_.src_lsr_addr_len; i++) {
			mc >> ((uint8_t *)obj.value_.src_lsr_addr)[i];
		}
		((uint8_t *)obj.value_.src_lsr_addr)[i] = '\0';

		mc >> obj.value_.local_id;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// CallIdNull_Object
	std::ostream & operator << (std::ostream &            os,
				    const CallIdNull_Object & obj)
	{
		os << "CallIdNull"                                   << std::endl
		   << " Addr type:     " << obj.value_.addr_type     << std::endl
		   << " src LSR addr:  " << obj.value_.src_lsr_addr  << std::endl
		   << " Call local id: " << obj.value_.local_id      << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &              mc,
				    const CallIdNull_Object & obj)
	{
		mc << obj.value_.addr_type
		   << (uint8_t)  0
		   << (uint16_t) 0;

		uint32_t i;
		for (i = 0; i < obj.value_.src_lsr_addr_len; i++) {
			mc << ((uint8_t *)obj.value_.src_lsr_addr)[i];
		}

		mc << obj.value_.local_id;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &      mc,
				    CallIdNull_Object & obj)
	{
		uint8_t   reserved8;
		uint16_t  reserved16;


		mc >> obj.value_.addr_type
		   >> reserved8
		   >> reserved16;

		// length is inferred
		// len = remaining size of the buffer - call id length
		obj.value_.src_lsr_addr_len = mc.remainingSize() - 8;

		obj.value_.src_lsr_addr =
			new char[obj.value_.src_lsr_addr_len + 1];
		assert(obj.value_.src_lsr_addr);

		uint32_t i;
		for (i = 0; i < obj.value_.src_lsr_addr_len; i++) {
			mc >> ((uint8_t *)obj.value_.src_lsr_addr)[i];
		}
		((uint8_t *)obj.value_.src_lsr_addr)[i] = '\0';

		mc >> obj.value_.local_id;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	//
	// NotifyRequest Class
	//

	// IPV4NotifyRequest_Object
	IPV4_ADDR_OPERATOR_IMP(IPV4NotifyRequest_Object);

	// IPV6NotifyRequest_Object
	IPV6_ADDR_OPERATOR_IMP(IPV6NotifyRequest_Object);

#define MSGID_OPERATOR_IMP(OBJ_NAME)					     \
	std::ostream & operator << (std::ostream & os, const OBJ_NAME & obj) \
	{								     \
		os << "  " #OBJ_NAME        << std::endl		     \
		   << "   flags:      "					     \
		   << obj.value_.flags      << std::endl		     \
		   << "   epoch:      "					     \
		   << obj.value_.epoch      << std::endl		     \
		   << "   message_id: "					     \
		   << obj.value_.message_id << std::endl;		     \
		return os;						     \
	}								     \
									     \
	MemoryCursor & operator << (MemoryCursor & mc, const OBJ_NAME & obj) \
	{								     \
		uint32_t  flags_epoch;					     \
									     \
		flags_epoch  = (obj.value_.flags & 0xFF)       << 24;	     \
		flags_epoch |= (obj.value_.epoch & 0x00FFFFFF) << 0;	     \
									     \
		mc << flags_epoch << obj.value_.message_id;		     \
									     \
		assert(mc.remainingSize() == 0);			     \
		return mc;						     \
	}								     \
									     \
	MemoryCursor & operator >> (MemoryCursor & mc, OBJ_NAME & obj)	     \
	{								     \
		uint32_t  flags_epoch;					     \
									     \
		mc >> flags_epoch >> obj.value_.message_id;		     \
									     \
		obj.value_.flags = (flags_epoch & 0xFF000000) >> 24;	     \
		obj.value_.epoch = (flags_epoch & 0x00FFFFFF) << 0;	     \
									     \
		return mc;						     \
	}

	//
	// MessageId Class
	//

	// MessageId_Object
	MSGID_OPERATOR_IMP(MessageId_Object);

	//
	// MessageIdAckNack Class
	//

	// MessageIdAck_Object
	MSGID_OPERATOR_IMP(MessageIdAck_Object);

	// MessageIdNack_Object
	MSGID_OPERATOR_IMP(MessageIdNack_Object);

	//
	// Scope Class
	//

	// IPv4Scope_Object
	ADDR_LIST_OPERATOR_IMP(IPv4Scope_Object, ipv4_t);

	// IPv4Scope_Object
	ADDR_LIST_OPERATOR_IMP(IPv6Scope_Object, ipv6_t);

	//
	// Policy Class
	//

	// PolicyInfo
	PolicyInfo::PolicyInfo(uint16_t     obj_type,
			       uint16_t     data_len,
			       const char * data)
	{
		type            = obj_type;
		opaque_data_len = data_len;

		opaque_data     = new char[data_len];
		assert(opaque_data);

		uint32_t i;
		for (i = 0; i < data_len; i++) {
			((uint8_t *)opaque_data)[i] = ((uint8_t *)data)[i];
		}
	}

	PolicyInfo::PolicyInfo(void):
		type(0), opaque_data_len(0), opaque_data(0)
	{
	}

	PolicyInfo::~PolicyInfo(void)
	{
		DELETE_ARR_OBJ(opaque_data, opaque_data_len);
	}

	bool PolicyInfo::isConsistent(void) const
	{
		if (!type || !opaque_data_len || !opaque_data) {
			return false;
		}
		return true;
	}

	uint16_t PolicyInfo::length(void) const
	{
		return pInfoHdrLength + opaque_data_len;
		//sizeof(*opaque_data) * (opaque_data_len);
	}

	PolicyInfo * PolicyInfo::clone(void) const
	{
		PolicyInfo * obj = new PolicyInfo(type,
						  opaque_data_len,
						  opaque_data);
		return obj;
	}

	std::ostream & operator << (std::ostream &     os,
				    const PolicyInfo & obj)
	{
		os << " Policy Element Information"            << std::endl
		   << "  P-type:      " << obj.type            << std::endl
		   << "  length:      " << obj.opaque_data_len +
			PolicyInfo::pInfoHdrLength << std::endl
		   << "  opaque_data: " << obj.opaque_data     << std::endl;

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &     mc,
				    const PolicyInfo & obj)
	{
		mc << uint16_t(obj.opaque_data_len + PolicyInfo::pInfoHdrLength)
		   << obj.type;

		for (uint32_t i = 0; i < obj.opaque_data_len; i++) {
			mc << ((uint8_t *)obj.opaque_data)[i];
		}

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc,
				    PolicyInfo   & obj)
	{
		uint16_t len;
		mc >> len >> obj.type;
		assert(len > PolicyInfo::pInfoHdrLength);

		obj.opaque_data_len = len - PolicyInfo::pInfoHdrLength;

		obj.opaque_data = new char[obj.opaque_data_len]; // + 1];
		assert(obj.opaque_data);
		assert(mc.remainingSize() == obj.opaque_data_len);

		uint32_t i;
		for (i = 0; i < obj.opaque_data_len; i++) {
			mc >> ((uint8_t *)obj.opaque_data)[i];
		}
		//((uint8_t *)obj.opaque_data)[i] = '\0'; // needed ???

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// Policy_Object
	std::ostream & operator << (std::ostream &        os,
				    const Policy_Object & obj)
	{
		os << "  Policy_Object"                       << std::endl
		   << "   data_offset: " << obj.data_offset() << std::endl;

		if (obj.value_.scope) {
			os << *obj.value_.scope;
		}

		if (obj.value_.orig_rsvp_hop) {
			os << *obj.value_.orig_rsvp_hop;
		}

		if (obj.value_.dest_rsvp_hop) {
			os << *obj.value_.dest_rsvp_hop;
		}

		if (obj.value_.integrity) {
			os << *obj.value_.integrity;
		}

		if (obj.value_.time_values) {
			os << *obj.value_.time_values;
		}

		std::list<PolicyInfo *>::const_iterator it;
		for (it = obj.value_.policy_elems.begin();
		     it != obj.value_.policy_elems.end();
		     it++) {
			if (*it) {
				os << *(*it);
			}
		}

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &        mc,
				    const Policy_Object & obj)
	{
		uint32_t dataoffset_resvd;
		uint16_t length = sizeof(dataoffset_resvd);

		MemoryCursor mc2(mc);
		mc2.resize(mc.start(), mc.start() + (length - 1));
		MemoryCursor mcObj(mc2);

		dataoffset_resvd  = (obj.data_offset() & 0x0000FFFF) << 16;
		dataoffset_resvd |= (uint16_t) 0                     << 0;

		mc2 << dataoffset_resvd;
		assert(mc2.remainingSize() == 0);

		if (obj.value_.scope) {
			mcObj = mcObj.neighbor(obj.value_.scope->length());
			mcObj << *obj.value_.scope;
		}

		if (obj.value_.orig_rsvp_hop) {
			mcObj = mcObj.neighbor(obj.value_.
					       orig_rsvp_hop->length());
			mcObj << *obj.value_.orig_rsvp_hop;
		}

		if (obj.value_.dest_rsvp_hop) {
			mcObj = mcObj.neighbor(obj.value_.
					       dest_rsvp_hop->length());
			mcObj << *obj.value_.dest_rsvp_hop;
		}

		if (obj.value_.integrity) {
			mcObj = mcObj.neighbor(obj.value_.integrity->length());
			mcObj << *obj.value_.integrity;
		}

		if (obj.value_.time_values) {
			mcObj = mcObj.neighbor(obj.value_.
					       time_values->length());
			mcObj << *obj.value_.time_values;
		}

		std::list<PolicyInfo *>::const_iterator it;
		for (it = obj.value_.policy_elems.begin();
		     it != obj.value_.policy_elems.end();
		     it++) {
			if (*it) {
				mcObj = mcObj.neighbor((*it)->length());
				mcObj << *(*it);
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Policy_Object & obj)
	{
		uint32_t dataoffset_resvd;
		uint16_t length = sizeof(dataoffset_resvd);

		MemoryCursor mc2(mc);
		mc2.resize(mc.start(), mc.start() + (length - 1));
		MemoryCursor mcObj(mc);

		mc2 >> dataoffset_resvd;
		obj.value_.data_offset_ = (dataoffset_resvd & 0xFFFF0000)>> 16;

		assert(mc2.remainingSize() == 0);

		//NOTE: If data_offset includes also data_offest and reserved
		//      field we need to decrease length variable
		length = obj.value_.data_offset_ - length;

		uint8_t  cNum;
		uint8_t  cType;
		uint16_t len;
		bool     first;

		MemoryCursor mcObjHdr(mc);
		if (length > 0) {
			mcObjHdr.resize(mc2.stop() + 1,
					mc2.stop() + 1 +
					(objHeader::objHdrLength-1));
		} else {
			mcObj = mc2;
		}

		first = true;

		while (length > 0) {

			mcObjHdr >> len >> cNum >> cType;
			assert(mcObjHdr.remainingSize() == 0);

			mcObj = mc;
			mcObj.resize(mcObjHdr.start(),
				     mcObjHdr.start() + (len - 1));

			switch (cNum) {
				case G2RSVP_CLASS_SCOPE:
					assert(!obj.value_.scope);
					mcObj >> obj.value_.scope;
					assert(obj.value_.scope);
					break;
				case G2RSVP_CLASS_RSVP_HOP:
					if (first) {
						assert(!obj.value_.orig_rsvp_hop);
						mcObj >> obj.value_.orig_rsvp_hop;
						assert(obj.value_.orig_rsvp_hop);
						first = false;
					} else {
						assert(!obj.value_.dest_rsvp_hop);
						mcObj >> obj.value_.dest_rsvp_hop;
						assert(obj.value_.dest_rsvp_hop);
					}
					break;
				case G2RSVP_CLASS_INTEGRITY:
					assert(!obj.value_.integrity);
					mcObj >> obj.value_.integrity;
					assert(obj.value_.integrity);
					break;
				case G2RSVP_CLASS_TIME_VALUES:
					assert(!obj.value_.time_values);
					mcObj >> obj.value_.time_values;
					assert(obj.value_.time_values);
					break;
				default:
					throw(std::string("Invalid class num "
							  "for Policy Obj"));
			}

			length -= len;

			if (length > 0) {
				mcObjHdr = mcObj.
					neighbor(objHeader::objHdrLength);
			}
		}

		assert(length == 0);

		MemoryCursor mc3(mc);
		if (mcObj.stop() < mc.stop()) {
			mc3.resize(mcObj.stop() + 1,
				   mcObj.stop() + 1 +
				   (PolicyInfo::pInfoHdrLength - 1));
		}
		uint16_t pType;

		length = mc.remainingSize() - obj.value_.data_offset_;

		while (length > 0) {

			mc3 >> len >> pType;
			assert(len > PolicyInfo::pInfoHdrLength);
			assert(mc3.remainingSize() == 0);

			MemoryCursor mcObj(mc);
			mcObj.resize(mc3.start(), mc3.start() + (len - 1));

			PolicyInfo * pInfo = 0;
			pInfo = new PolicyInfo();
			assert(pInfo);
			mcObj >> *pInfo;
			obj.value_.policy_elems.push_back(pInfo);

			length -= pInfo->length();

			if (length > 0) {
				mc3=mcObj.neighbor(PolicyInfo::pInfoHdrLength);
			}
		}

		// safety check
		assert(obj.value_.data_offset_ == obj.data_offset());

		return mc;
	}

	//
	// ResvConfirmIPv4 Class
	//

	// ResvConfirmIPv4_Object
	IPV4_ADDR_OPERATOR_IMP(ResvConfirmIPv4_Object);

	// ResvConfirmIPv6_Object
	IPV6_ADDR_OPERATOR_IMP(ResvConfirmIPv6_Object);

	//
	// Style Class
	//

	// Style_Object
	std::ostream & operator << (std::ostream &       os,
				    const Style_Object & obj)
	{
		os << "  Style_Object"            << std::endl
		   << "   Flags           : "
		   << obj.value_.flags            << std::endl
		   << "   Sharing Control : "
		   << obj.value_.sharing          << std::endl
		   << "   Sender Selection: "
		   << obj.value_.sender_selection << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &       mc,
				    const Style_Object & obj)
	{
		uint32_t field;

		field  = ((uint32_t)(obj.value_.flags            & 0xFF)) << 24;
		field |= ((uint32_t)(obj.value_.sharing          & 0x03)) <<  3;
		field |= ((uint32_t)(obj.value_.sender_selection & 0x07)) <<  0;

		mc << field;
		assert(mc.remainingSize() == 0);
		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc,
				    Style_Object & obj)
	{
		uint32_t field;

		mc >> field;

		obj.value_.flags            = (uint8_t) ((field &
							  0xFF000000) >> 24);
		obj.value_.sharing          = (uint8_t) ((field &
							  0x00000018) >> 3);
		obj.value_.sender_selection = (uint8_t) ((field &
							  0x00000007) >> 0);

		assert(mc.remainingSize() == 0);
		return mc;
	}

	//
	// ErrorSpec Class
	//

	// IPv4ErrorSpec_Object
	std::ostream & operator << (std::ostream &               os,
				    const IPv4ErrorSpec_Object & obj)
	{
		os << "IPv4ErrorSpec_Object"                       << std::endl
		   << " IPv4       : " << obj.value_.ipv4_err_node << std::endl
		   << " Flags      : " << obj.value_.flags         << std::endl
		   << " Error Code : " << obj.value_.error_code    << std::endl
		   << " Error Value: " << obj.value_.error_value   << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &               mc,
				    const IPv4ErrorSpec_Object & obj)
	{
		mc << obj.value_.ipv4_err_node
		   << obj.value_.flags
		   << obj.value_.error_code
		   << obj.value_.error_value;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &         mc,
				    IPv4ErrorSpec_Object & obj)
	{
		mc >> obj.value_.ipv4_err_node
		   >> obj.value_.flags
		   >> obj.value_.error_code
		   >> obj.value_.error_value;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// IPv6ErrorSpec_Object
	std::ostream & operator << (std::ostream &               os,
				    const IPv6ErrorSpec_Object & obj)
	{
		os << "IPv6ErrorSpec_Object"                       << std::endl
		   << " IPv6       : " << obj.value_.ipv6_err_node << std::endl
		   << " Flags      : " << obj.value_.flags         << std::endl
		   << " Error Code : " << obj.value_.error_code    << std::endl
		   << " Error Value: " << obj.value_.error_value   << std::endl;
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &               mc,
				    const IPv6ErrorSpec_Object & obj)
	{
		mc << obj.value_.ipv6_err_node
		   << obj.value_.flags
		   << obj.value_.error_code
		   << obj.value_.error_value;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &         mc,
				    IPv6ErrorSpec_Object & obj)
	{
		mc >> obj.value_.ipv6_err_node
		   >> obj.value_.flags
		   >> obj.value_.error_code
		   >> obj.value_.error_value;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	// IFIDIPv4ErrorSpec_Object
	std::ostream & operator << (std::ostream &                   os,
				    const IFIDIPv4ErrorSpec_Object & obj)
	{
		os << "IFIDIPv4ErrorSpec_Object"                   << std::endl
		   << " IPv4       : " << obj.value_.ipv4_err_node << std::endl
		   << " Flags      : " << obj.value_.flags         << std::endl
		   << " Error Code : " << obj.value_.error_code    << std::endl
		   << " Error Value: " << obj.value_.error_value   << std::endl
		   << " TLVs       : "                             << std::endl;

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				os << *(*it);
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                   mc,
				    const IFIDIPv4ErrorSpec_Object & obj)
	{
		int len = (sizeof(obj.value_.ipv4_err_node) +
			   sizeof(obj.value_.flags)         +
			   sizeof(obj.value_.error_code)    +
			   sizeof(obj.value_.error_value));

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		MemoryCursor mcObj(mcHdr); //NB pos. here! (because of current)
		mcHdr << obj.value_.ipv4_err_node
		      << obj.value_.flags
		      << obj.value_.error_code
		      << obj.value_.error_value;

		assert(mcHdr.remainingSize() == 0);

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				mcObj = mcObj.neighbor((*it)->length());
				mcObj << *(*it);
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &             mc,
				    IFIDIPv4ErrorSpec_Object & obj)
	{
		int len = (sizeof(obj.value_.ipv4_err_node) +
			   sizeof(obj.value_.flags)         +
			   sizeof(obj.value_.error_code)    +
			   sizeof(obj.value_.error_value));

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		mcHdr >> obj.value_.ipv4_err_node
		      >> obj.value_.flags
		      >> obj.value_.error_code
		      >> obj.value_.error_value;

		assert(mcHdr.remainingSize() == 0);

		len = mc.remainingSize() - len;

		uint16_t tlv_type;
		uint16_t tlv_len;

		MemoryCursor mcTlvHdr(mcHdr);
		MemoryCursor mcTlv(mcTlvHdr);

		while (len > 0) {

			mcTlvHdr = mcTlv.neighbor(TLV::tlvHdrLength);

			mcTlvHdr >> tlv_type >> tlv_len;
			assert(mcTlvHdr.remainingSize() == 0);

			mcTlv = mc;
			mcTlv.resize(mcTlvHdr.start(),
				     mcTlvHdr.start() + (tlv_len - 1));

			TLV * tlv = 0;
			mcTlv >> tlv;

			obj.value_.tlvs.push_back(tlv);
			len -= tlv->length();
		}

		return mc;
	}

	// IFIDIPv6ErrorSpec_Object
	std::ostream & operator << (std::ostream &                   os,
				    const IFIDIPv6ErrorSpec_Object & obj)
	{
		os << "IFIDIPv6ErrorSpec_Object"                   << std::endl
		   << " IPv6       : " << obj.value_.ipv6_err_node << std::endl
		   << " Flags      : " << obj.value_.flags         << std::endl
		   << " Error Code : " << obj.value_.error_code    << std::endl
		   << " Error Value: " << obj.value_.error_value   << std::endl
		   << " TLVs       : "                             << std::endl;

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				os << *(*it);
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor &                   mc,
				    const IFIDIPv6ErrorSpec_Object & obj)
	{
		int len = (sizeof(obj.value_.ipv6_err_node) +
			   sizeof(obj.value_.flags)         +
			   sizeof(obj.value_.error_code)    +
			   sizeof(obj.value_.error_value));

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		MemoryCursor mcObj(mcHdr); //NB pos. here! (because of current)
		mcHdr << obj.value_.ipv6_err_node
		      << obj.value_.flags
		      << obj.value_.error_code
		      << obj.value_.error_value;

		assert(mcHdr.remainingSize() == 0);

		std::list<TLV *>::const_iterator it;
		for (it = obj.value_.tlvs.begin();
		     it != obj.value_.tlvs.end();
		     it++) {
			if (*it) {
				mcObj = mcObj.neighbor((*it)->length());
				mcObj << *(*it);
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &             mc,
				    IFIDIPv6ErrorSpec_Object & obj)
	{
		int len = (sizeof(obj.value_.ipv6_err_node) +
			   sizeof(obj.value_.flags)         +
			   sizeof(obj.value_.error_code)    +
			   sizeof(obj.value_.error_value));

		MemoryCursor mcHdr(mc);
		mcHdr.resize(mc.start(), mc.start() + (len - 1));

		mcHdr >> obj.value_.ipv6_err_node
		      >> obj.value_.flags
		      >> obj.value_.error_code
		      >> obj.value_.error_value;

		assert(mcHdr.remainingSize() == 0);

		len = mc.remainingSize() - len;

		uint16_t tlv_type;
		uint16_t tlv_len;

		MemoryCursor mcTlvHdr(mcHdr);
		MemoryCursor mcTlv(mcTlvHdr);

		while (len > 0) {

			mcTlvHdr = mcTlv.neighbor(TLV::tlvHdrLength);

			mcTlvHdr >> tlv_type >> tlv_len;
			assert(mcTlvHdr.remainingSize() == 0);

			mcTlv = mc;
			mcTlv.resize(mcTlvHdr.start(),
				     mcTlvHdr.start() + (tlv_len - 1));

			TLV * tlv = 0;
			mcTlv >> tlv;

			obj.value_.tlvs.push_back(tlv);
			len -= tlv->length();
		}

		return mc;
	}

	//
	// Flow_Desc_List Object
	//

	Flow_Desc_List::Flow_Desc_List(FF_FlowDesc_t * ff)
	{
		ff_desc = ff;
		se_desc = 0;
	}

	Flow_Desc_List::Flow_Desc_List(SE_FlowDesc_t * se)
	{
		ff_desc = 0;
		se_desc = se;
	}

	Flow_Desc_List::Flow_Desc_List(void)
	{
		ff_desc = 0;
		se_desc = 0;
	}

	Flow_Desc_List::~Flow_Desc_List(void)
	{
		DELETE_OBJ(ff_desc);
		DELETE_OBJ(se_desc);
	}

	Flow_Desc_List * Flow_Desc_List::clone(void) const
	{
		assert(isConsistent());

		Flow_Desc_List * desc = new Flow_Desc_List();
		assert(desc);
		if (ff_desc) {
			desc->ff_desc = ff_desc->clone();
		}
		if (se_desc) {
			// Temporary, only FF desc is supported for now
			//desc->se_desc = se_desc->clone();
		}

		return desc;
	}

	bool Flow_Desc_List::isConsistent(void) const
	{
		if ((ff_desc && se_desc) || (!ff_desc && !se_desc)) {
			return false;
		}
		return true;
	}

	uint16_t Flow_Desc_List::length(void) const
	{
		assert(isConsistent());

		if (ff_desc) {
			return ff_desc->length();
		}
		if (se_desc) {
			// Temporary, only FF desc is supported for now
			//return se_desc->length();
		}

		return 0;
	}

	bool Flow_Desc_List::setLabelObj(Label_Class * label)
	{
		assert(isConsistent());

		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		SETOBJ_INTO_FLOW_DESC(label);

		return true;
	}

	bool Flow_Desc_List::setRROobj(RecordedRoute_Class * rro)
	{
		assert(isConsistent());

		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		SETOBJ_INTO_FLOW_DESC(rro);

		return true;
	}

	bool Flow_Desc_List::setFlowSpecObj(FlowSpec_Class * flow_spec)
	{
		assert(isConsistent());

		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		SETOBJ_INTO_FLOW_DESC(flow_spec);

		return true;
	}

	bool Flow_Desc_List::setFilterSpecObj(FilterSpec_Class * filter_spec)
	{
		assert(isConsistent());

		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		SETOBJ_INTO_FLOW_DESC(filter_spec);

		return true;
	}

	Label_Class * Flow_Desc_List::getLabelObj(void)
	{
		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc || ff_desc->filters.size() != 1) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		ff_flow_item_desc_t * tmp = ff_desc->filters.front();
		if (!tmp) {
			G2RSVPTE_ERR("Invalid first element from FF "
				     "flow description list!");
			return 0;
		}

		return tmp->label;
	}

	RecordedRoute_Class * Flow_Desc_List::getRROobj(void)
	{
		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc || ff_desc->filters.size() != 1) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		ff_flow_item_desc_t * tmp = ff_desc->filters.front();
		if (!tmp) {
			G2RSVPTE_ERR("Invalid first element from FF "
				     "flow description list!");
			return 0;
		}

		return tmp->rro;
	}

	FlowSpec_Class * Flow_Desc_List::getFlowSpecObj(void)
	{
		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc || ff_desc->filters.size() != 1) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		ff_flow_item_desc_t * tmp = ff_desc->filters.front();
		if (!tmp) {
			G2RSVPTE_ERR("Invalid first element from FF "
				     "flow description list!");
			return 0;
		}

		return tmp->flow_spec;
	}

	FilterSpec_Class * Flow_Desc_List::getFilterSpecObj(void)
	{
		// TMP TMP TMP
		// For now only FF style (with only 1 element)
		// is supported!!!
		if (!ff_desc || ff_desc->filters.size() != 1) {
			G2RSVPTE_ERR("For now only FF style (with only"
				     " 1 element) is supported!");
			return 0;
		}

		ff_flow_item_desc_t * tmp = ff_desc->filters.front();
		if (!tmp) {
			G2RSVPTE_ERR("Invalid first element from FF "
				     "flow description list!");
			return 0;
		}

		return tmp->filter_spec;
	}

	std::ostream & operator << (std::ostream &         os,
				    const Flow_Desc_List & obj)
	{
		os << "Flow_Desc_List"  << std::endl;

		assert(obj.isConsistent());

		if (obj.ff_desc) {
			os << *obj.ff_desc;
		}
		if (obj.se_desc) {
			throw(std::string("Only FFdesc is supported for now"));
			//os << *obj.se_desc;
		}

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &         mc,
				    const Flow_Desc_List & obj)
	{
		assert(obj.isConsistent());

		if (obj.ff_desc) {
			mc << *obj.ff_desc;
		}
		if (obj.se_desc) {
			throw(std::string("Only FFdesc is supported for now"));
			//mc << *obj.se_desc;
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, Flow_Desc_List & obj)
	{
		assert(obj.isConsistent());

		if (obj.ff_desc) {
			mc >> *obj.ff_desc;
		}
		if (obj.se_desc) {
			throw(std::string("Only FFdesc is supported for now"));
			//mc >> obj.se_desc;
		}

		return mc;
	}

	// ff_flow_item_desc
	std::ostream & operator << (std::ostream &            os,
				    const ff_flow_item_desc & obj)
	{
		os << "  ff_flow_item_desc:"  << std::endl;

		OBJ2STREAM(obj.flow_spec, os);
		OBJ2STREAM(obj.filter_spec, os);
		OBJ2STREAM(obj.label, os);
		OBJ2STREAM(obj.rro, os);

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const ff_flow_item_desc & obj)
	{
		MemoryCursor mcObj(mc);
		size_t start = mc.start();

		OBJ2CURSOR2(obj.flow_spec, mcObj, mc, start);
		OBJ2CURSOR2(obj.filter_spec, mcObj, mc, start);
		OBJ2CURSOR2(obj.label, mcObj, mc, start);
		OBJ2CURSOR2(obj.rro, mcObj, mc, start);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, ff_flow_item_desc & obj)
	{
		throw(std::string("Useless operator...it should not be used"));
	}

	// FF_FlowDesc
	std::ostream & operator << (std::ostream & os, const FF_FlowDesc & obj)
	{
		os << " FF_FlowDesc:"  << std::endl;

		std::list<ff_flow_item_desc_t *>::const_iterator it;
		for (it = obj.filters.begin(); it != obj.filters.end(); it++) {
			ff_flow_item_desc_t * tmp;
			tmp = *it;
			if (tmp) {
				os << *tmp;
			}
		}
		return os;
	}

	MemoryCursor & operator << (MemoryCursor & mc, const FF_FlowDesc & obj)
	{
		MemoryCursor mcObj(mc);
		std::list<ff_flow_item_desc_t *>::const_iterator it;
		for (it = obj.filters.begin(); it != obj.filters.end(); it++) {
			ff_flow_item_desc_t * tmp;
			tmp = *it;
			if (tmp) {
				if (it == obj.filters.begin()) {
					mcObj.resize(mc.start(), mc.start() +
						     (tmp->length()-1));
				} else {
					mcObj = mcObj.
						neighbor(tmp->length());
				}
				mcObj << *tmp;
			}
		}

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor & mc, FF_FlowDesc & obj)
	{
		throw(std::string("Useless operator... it should not be used"));
	}

	//
	// Times Class
	//

	// Times_Object
	std::ostream & operator << (std::ostream &            os,
				    const Times_Object &      obj)
	{
		os << "Times_Object"  << std::endl
		   << " start time: " << obj.value_.start_time << std::endl
		   << " end   time: " << obj.value_.end_time << std::endl;

		return os;
	}

	MemoryCursor & operator << (MemoryCursor &            mc,
				    const Times_Object &      obj)
	{
		mc << obj.value_.start_time;
		mc << obj.value_.end_time;

		assert(mc.remainingSize() == 0);

		return mc;
	}

	MemoryCursor & operator >> (MemoryCursor &      mc,
				    Times_Object &      obj)
	{
		mc >> obj.value_.start_time;
		mc >> obj.value_.end_time;

		assert(mc.remainingSize() == 0);

		return mc;
	}
}
