//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//

#include "g2rsvpted/g2rsvpte_persistency_py.h"
#include "g2rsvpted/g2rsvpte_lrm_api.h"
#include "g2rsvpted/g2rsvpte_config.h"
#include "g2rsvpted/g2rsvpte_trace.h"
#include "g2rsvpted/g2rsvpte_message_objects.h"
#include "g2rsvpted/g2rsvpte_corba.h"
#include "g2rsvpted/g2rsvpte_utils.h"

#include "g2mpls_corba_utils.h"


bool PersistencyPy::initDatabase(adj_type_t adjType)
{
#ifdef G2RSVPTE2PC_DEBUG

	G2RSVPTE_DBG("Fake initDatabase to Persistency Controller...debugging!");
	return true;
#else
	try {
		PersistencyController::StorageTypes type;
		std::string                         dbname;
		bool                                res;

		G2RSVPTE_DBG("Going to initialize Database to Persistency Controller");

		switch (adjType) {
			case INNI:
				type   = PersistencyController::STORAGETYPE_INNIRSVPLSP;
				dbname = "INNIRSVP DB";
				break;
			case UNI:
				type   = PersistencyController::STORAGETYPE_UNIRSVPLSP;
				dbname = "UNIRSVP DB";
				break;
			case ENNI:
				type   = PersistencyController::STORAGETYPE_ENNIRSVPLSP;
				dbname = "ENNIRSVP DB";
				break;
		}

		res = pc_mgmt_proxy->initDb(type, dbname.c_str());

		if (!res) {
			G2RSVPTE_ERR("Cannot initialize Database");
			return false;
		}
	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Cannot initialize Database: internal problems");
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Cannot initialize Database: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Cannot initialize Database: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Cannot initialize Database: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot initialize Database: unknown reason");
		return false;
	}

	G2RSVPTE_DBG("Database initializated");

	return true;

#endif // G2RSVPTE2PC_DEBUG
}

bool PersistencyPy::closeDatabase(adj_type_t adjType)
{
#ifdef G2RSVPTE2PC_DEBUG

	G2RSVPTE_DBG("Fake closeDatabase to Persistency Controller...debugging!");
	return true;
#else
	try {
		PersistencyController::StorageTypes type;
		bool                                res;

		G2RSVPTE_DBG("Going to close Database to Persistency Controller");

		switch (adjType) {
			case INNI:
				type   = PersistencyController::STORAGETYPE_INNIRSVPLSP;
				break;
			case UNI:
				type   = PersistencyController::STORAGETYPE_UNIRSVPLSP;
				break;
			case ENNI:
				type   = PersistencyController::STORAGETYPE_ENNIRSVPLSP;
				break;
		}

		res = pc_mgmt_proxy->closeDb(type);

		if (!res) {
			G2RSVPTE_ERR("Cannot close Database");
			return false;
		}
	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Cannot close Database: internal problems");
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Cannot close Database: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Cannot close Database: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Cannot close Database: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot close Database: unknown reason");
		return false;
	}

	G2RSVPTE_DBG("Database closed");

	return true;

#endif // G2RSVPTE2PC_DEBUG
}

bool PersistencyPy::flushDatabase(adj_type_t adjType)
{
#ifdef G2RSVPTE2PC_DEBUG

	G2RSVPTE_DBG("Fake flushDatabase to Persistency Controller...debugging!");
	return true;
#else
	try {
		PersistencyController::StorageTypes type;
		bool                                res;

		G2RSVPTE_DBG("Going to flush initialize Database to Persistency Controller");

		switch (adjType) {
			case INNI:
				type   = PersistencyController::STORAGETYPE_INNIRSVPLSP;
				break;
			case UNI:
				type   = PersistencyController::STORAGETYPE_UNIRSVPLSP;
				break;
			case ENNI:
				type   = PersistencyController::STORAGETYPE_ENNIRSVPLSP;
				break;
		}

		res = pc_mgmt_proxy->flushDb(type);

		if (!res) {
			G2RSVPTE_ERR("Cannot flush Database");
			return false;
		}
	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Cannot flush Database: internal problems");
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Cannot flush Database: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Cannot flush Database: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Cannot flush Database: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot flush Database: unknown reason");
		return false;
	}

	G2RSVPTE_DBG("Database flushed");

	return true;

#endif // G2RSVPTE2PC_DEBUG
}

bool PersistencyPy::storeLsp(RSVP::LSP * lsp)
{
	assert(lsp);
	assert(lsp->getInstance());
	assert(lsp->psb());
	assert(lsp->psb()->pathIn());

#ifdef G2RSVPTE2PC_DEBUG

	G2RSVPTE_DBG("Fake store LSP to Persistency Controller...debugging!");
	return true;
#else
	try {
		g2mplsTypes::lspIdent_var                                lspIdent;
		lsp_ident_t                                              lspIdentTmp;
		PersistencyController::MiscDataTypes::StoredLspData_var  lspData;
		g2mplsTypes::lspParams_var                               lspInfo;
		lsp_info_t                                               lspInfoTmp;
		g2mplsTypes::TELinkId_var                                psbLocTelId;
		g2mplsTypes::TELinkId_var                                psbRemTelId;
		g2mplsTypes::DLinkId_var                                 psbLocDlId;
		g2mplsTypes::DLinkId_var                                 psbRemDlId;
		g2mplsTypes::TELinkId_var                                rsbLocTelId;
		g2mplsTypes::TELinkId_var                                rsbRemTelId;
		g2mplsTypes::DLinkId_var                                 rsbLocDlId;
		g2mplsTypes::DLinkId_var                                 rsbRemDlId;
		net_res_spec_t                                           iTnaResTmp;
		g2mplsTypes::tnaResource_var                             iTnaRes;
		net_res_spec_t                                           eTnaResTmp;
		g2mplsTypes::tnaResource_var                             eTnaRes;
		call_ident_t                                             callIdTmp;
		g2mplsTypes::callIdent_var                               callId;
		call_info_t                                              callInfoTmp;
		g2mplsTypes::callParams_var                              callParams;
		recovery_info_t                                          recoveryInfoTmp;
		g2mplsTypes::recoveryParams_var                          recoveryParms;
		grid_res_spec_t                                          iGnsTnaTmp;
		g2mplsTypes::gridParams_var                              iGnsTna;
		grid_res_spec_t                                          eGnsTnaTmp;
		g2mplsTypes::gridParams_var                              eGnsTna;
		std::string                                              authz;
		g2mplsTypes::aaaParams_var                               aaaInfo;
		std::list<lsp_ero_sobj_t>                                ero;
		std::list<lsp_ero_sobj_t>::iterator                      it;
		bool                                                     res;

		G2RSVPTE_DBG("Going to store LSP to Persistency Controller");

		//get lsp ident
		lspIdentTmp   = lsp->getIdent();
		lspIdent     << lspIdentTmp;

		lspData = new PersistencyController::MiscDataTypes::StoredLspData;
		lspData->mask_.iGnsTna    = false;
		lspData->mask_.eGnsTna    = false;
		lspData->mask_.aaaInfo    = false;

		//get lsp parameters
		memset(&lspInfoTmp, 0, sizeof(lspInfoTmp));
		if (!lsp->getInfo(lspInfoTmp)) {
			G2RSVPTE_ERR("Impossible to retrieve "
				     "LSP info parameters");
			return false;
		}
		lspInfo << lspInfoTmp;

		lspData->lspInfo = lspInfo;

		// save PSB and RSB infos
		lspData->psbInfo.nodeId = lsp->psb()->nodeId();

		if (lsp->psb()->interface()) {
			psbLocTelId << lsp->psb()->interface()->localId();
			psbRemTelId << lsp->psb()->interface()->remoteId();
		} else {
			g2mpls_addr_t psbTelTmp;
			psbTelTmp.type = IPv4;
			psbTelTmp.preflen = 32;
			psbTelTmp.value.ipv4.s_addr = 0;
			psbLocTelId << psbTelTmp;
			psbRemTelId << psbTelTmp;
		}
		lspData->psbInfo.locTelId = psbLocTelId;
		lspData->psbInfo.remTelId = psbRemTelId;

		psbLocDlId << lsp->psb()->localDL();
		lspData->psbInfo.locDlId = psbLocDlId;

		psbRemDlId << lsp->psb()->remoteDL();
		lspData->psbInfo.remDlId = psbRemDlId;

		lspData->psbInfo.labelId.label32(lsp->psb()->txLabel());
		//
		lspData->rsbInfo.nodeId = lsp->rsb()->nodeId();

		if (lsp->rsb()->interface()) {
			rsbLocTelId << lsp->rsb()->interface()->localId();
			rsbRemTelId << lsp->rsb()->interface()->remoteId();
		} else {
			g2mpls_addr_t rsbTelTmp;
			rsbTelTmp.type = IPv4;
			rsbTelTmp.preflen = 32;
			rsbTelTmp.value.ipv4.s_addr = 0;
			rsbLocTelId << rsbTelTmp;
			rsbRemTelId << rsbTelTmp;
		}
		lspData->rsbInfo.locTelId = rsbLocTelId;
		lspData->rsbInfo.remTelId = rsbRemTelId;

		rsbLocDlId << lsp->rsb()->localDL();
		lspData->rsbInfo.locDlId = rsbLocDlId;

		rsbRemDlId << lsp->rsb()->remoteDL();
		lspData->rsbInfo.remDlId = rsbRemDlId;

		lspData->rsbInfo.labelId.label32(lsp->rsb()->txLabel());

		// save callId
		callIdTmp = lsp->callId();
		if (isCallIdentValid(callIdTmp, lsp->getInstance()->adjType()) <= 0) {
			G2RSVPTE_ERR("Impossible to retrieve call Id");
			return false;
		}
		callId   << callIdTmp;

		lspData->callId = callId;

		// save TNAs
		iTnaResTmp = lsp->iRes();
		iTnaRes   << iTnaResTmp;

		lspData->iTna = iTnaRes;

		eTnaResTmp = lsp->eRes();
		eTnaRes   << eTnaResTmp;

		lspData->eTna = eTnaRes;

		memset(&callInfoTmp, 0, sizeof(callInfoTmp));
		memset(&iGnsTnaTmp, 0, sizeof(iGnsTnaTmp));
		memset(&eGnsTnaTmp, 0, sizeof(eGnsTnaTmp));
		memset(&recoveryInfoTmp, 0, sizeof(recoveryInfoTmp));
		authz = "";
		if (lsp->getInstance()->adjType() == UNI ||
		    lsp->getInstance()->adjType() == ENNI) {
			// save callInfo
			if (!lsp->getCallInfo(callInfoTmp)) {
				G2RSVPTE_ERR("Impossible to retrieve "
					     "call Info parameters");
				return false;
			}
			// save GnsTNAs
			if (lsp->checkGnsTna(true)) {
				if (!lsp->getGnsTna(true, iGnsTnaTmp)) {
					G2RSVPTE_ERR("Impossible to retrieve "
						     "ingress GNS Tna parameters");
					return false;
				}
				lspData->mask_.iGnsTna = true;
			}

			if (lsp->checkGnsTna(false)) {
				if (!lsp->getGnsTna(false, eGnsTnaTmp)) {
					G2RSVPTE_ERR("Impossible to retrieve "
						     "egress GNS Tna parameters");
					return false;
				}
				lspData->mask_.eGnsTna = true;
			}

			// save AAAinfo
			if (lsp->checkPolicy()) {
				if (!lsp->getPolicy(authz)) {
					G2RSVPTE_ERR("Impossible to retrieve "
						     "Policy info parameters");
					return false;
				}
				lspData->mask_.aaaInfo = true;
			}
		}
		callParams         << callInfoTmp;
		lspData->callParms  = callParams;

		//TMP save recovery params
		recoveryParms         << recoveryInfoTmp;
		lspData->recoveryParms = recoveryParms;

		iGnsTna         << iGnsTnaTmp;
		lspData->iGnsTna = iGnsTna;

		eGnsTna         << eGnsTnaTmp;
		lspData->eGnsTna = eGnsTna;

		aaaInfo = new g2mplsTypes::aaaParams;
		aaaInfo->authzToken = authz.c_str();
		lspData->aaaInfo    = aaaInfo;

		// save ERO
		ero.clear();
		if (lsp->checkEro()) {
			if (!lsp->psb()->pathOut()->getEro(ero)) {
				G2RSVPTE_ERR("Impossible to get ERO");
				return false;
			}
		}
		g2mplsTypes::eroSeq_var eroItem;
		{
			g2mplsTypes::eroSeq * tmp;

			tmp = new g2mplsTypes::eroSeq(ero.size());
			if (!tmp) {
				G2RSVPTE_ERR("Impossible to allocate "
					     "a new eroSeq sequence");
				return false;
			}
			eroItem = tmp;
		}
		eroItem->length(ero.size());
		int i = 0;
		lsp_ero_sobj_t sObj;
		for (it = ero.begin(); it != ero.end(); it++) {
			memset(&sObj, 0, sizeof(sObj));
			sObj = *it;
			eroItem[i] = getEroItem(sObj);
			i++;
		}
		lspData->ero = eroItem;

		////TMP-FIXME
		lspData->states.opState  = g2mplsTypes::OPERSTATE_UP;
		lspData->states.admState = g2mplsTypes::ADMINSTATE_ENABLED;
		//////////////

		switch (lsp->getInstance()->adjType()) {
			case INNI:
				res = pc_innirsvplsp_proxy->storeLsp(lspIdent, lspData);
				break;
			case UNI:
				res = pc_unirsvplsp_proxy->storeLsp(lspIdent, lspData);
				break;
			case ENNI:
				res = pc_ennirsvplsp_proxy->storeLsp(lspIdent, lspData);
				break;
		}

		if (!res) {
			G2RSVPTE_ERR("Cannot store LSP in Database");
			return false;
		}
	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Cannot store LSP in Database: internal problems");
		return false;
	} catch (std::out_of_range & e) {
		G2RSVPTE_ERR("Cannot store LSP in Database: %s",
			     e.what());
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Cannot store LSP in Database: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Cannot store LSP in Database: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Cannot store LSP in Database: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot store LSP in Database: unknown reason");
		return false;
	}

	G2RSVPTE_DBG("LSP successfully stored");

	return true;

#endif // G2RSVPTE2PC_DEBUG
}

bool PersistencyPy::unstoreLsp(RSVP::LSP * lsp)
{
	assert(lsp);

#ifdef G2RSVPTE2PC_DEBUG

	G2RSVPTE_DBG("Fake unstoreLsp to Persistency Controller...debugging!");
	return true;
#else
	try {
		g2mplsTypes::lspIdent_var   lspIdent;
		lsp_ident_t                 lspIdentTmp;
		bool                        res;

		G2RSVPTE_DBG("Going to unstore LSP to Persistency Controller");

		//get lsp ident
		lspIdentTmp   = lsp->getIdent();
		lspIdent     << lspIdentTmp;

		switch (lsp->getInstance()->adjType()) {
			case INNI:
				res = pc_innirsvplsp_proxy->unstoreLsp(lspIdent);
				break;
			case UNI:
				res = pc_unirsvplsp_proxy->unstoreLsp(lspIdent);
				break;
			case ENNI:
				res = pc_ennirsvplsp_proxy->unstoreLsp(lspIdent);
				break;
		}

		if (!res) {
			G2RSVPTE_ERR("Cannot unstore LSP in Database");
			return false;
		}
	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Cannot unstore LSP in Database: internal problems");
		return false;
	} catch (Types::CannotFetch) {
		G2RSVPTE_ERR("Cannot unstore LSP in Database: cannot fetch LSP");
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Cannot unstore LSP in Database: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Cannot unstore LSP in Database: "
			     "CORBA::Exception");
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Cannot unstore LSP in Database: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot unstore LSP in Database: unknown reason");
		return false;
	}

	G2RSVPTE_DBG("LSP successfully unstored");

	return true;

#endif // G2RSVPTE2PC_DEBUG
}

bool PersistencyPy::fetchLsp(const lsp_ident_t           ident,
			     adj_type_t                  adjType,
			     lsp_ident_t  &              nextIdent,
			     lsp_info_t &                lspInfo,
			     storedLspStateBlockInfo_t & psbInfo,
			     storedLspStateBlockInfo_t & rsbInfo,
			     net_res_spec_t &            iTnaRes,
			     net_res_spec_t &            eTnaRes,
			     call_ident_t &              callId,
			     call_info_t &               callInfo,
			     storedLspDataMask_t &       mask,
			     grid_res_spec_t &           iGnsTna,
			     grid_res_spec_t &           eGnsTna,
			     std::string &               authz,
			     std::list<lsp_ero_sobj_t> & ero,
			     std::string &               resp)
{
#ifdef G2RSVPTE2PC_DEBUG

	G2RSVPTE_DBG("Fake fetchLsp to Persistency Controller...debugging!");
	return false;
#else
	try {
		g2mplsTypes::lspIdent_var                                lspIdent;
		g2mplsTypes::lspIdent_var                                lspNextIdent;
		lsp_ident_t                                              lspIdentTmp;
		PersistencyController::MiscDataTypes::StoredLspData_var  lspData;
		g2mplsTypes::lspParams_var                               lspParams;
		g2mplsTypes::TELinkId_var                                psbLocTelId;
		g2mplsTypes::TELinkId_var                                psbRemTelId;
		g2mplsTypes::DLinkId_var                                 psbLocDlId;
		g2mplsTypes::DLinkId_var                                 psbRemDlId;
		g2mplsTypes::TELinkId_var                                rsbLocTelId;
		g2mplsTypes::TELinkId_var                                rsbRemTelId;
		g2mplsTypes::DLinkId_var                                 rsbLocDlId;
		g2mplsTypes::DLinkId_var                                 rsbRemDlId;
		g2mplsTypes::tnaResource_var                             iTnaResTmp;
		g2mplsTypes::tnaResource_var                             eTnaResTmp;
		g2mplsTypes::callIdent_var                               callIdTmp;
		g2mplsTypes::callParams_var                              callParamsTmp;
		g2mplsTypes::gridParams_var                              iGnsTnaTmp;
		g2mplsTypes::gridParams_var                              eGnsTnaTmp;
		g2mplsTypes::aaaParams_var                               aaaInfo;
		g2mplsTypes::eroItem_var                                 eroItTmp;
		g2mplsTypes::eroSeq_var                                  eroItem;
		bool                                                     res;

		//get lsp ident
		lspIdentTmp   = ident;
		lspIdent     << lspIdentTmp;

		switch (adjType) {
			case INNI:
				res = pc_innirsvplsp_proxy->fetchNextLsp(lspIdent,
									 lspNextIdent,
									 lspData);
				break;
			case UNI:
				res = pc_unirsvplsp_proxy->fetchNextLsp(lspIdent,
									lspNextIdent,
									lspData);
				break;
			case ENNI:
				res = pc_ennirsvplsp_proxy->fetchNextLsp(lspIdent,
									 lspNextIdent,
									 lspData);
				break;
		}

		if (!res) {
			G2RSVPTE_ERR("Cannot fetch LSP in Database");
			return false;
		}

		//get next ident
		nextIdent << lspNextIdent;

		//get lsp info
		lspParams = lspData->lspInfo;
		lspInfo  << lspParams;

		//get PSB and RSB infos
		psbInfo.nodeId = lspData->psbInfo.nodeId;

		psbLocTelId = lspData->psbInfo.locTelId;
		psbInfo.locTelId << psbLocTelId;

		psbLocDlId = lspData->psbInfo.locDlId;
		psbInfo.locDlId << psbLocDlId;

		psbRemDlId = lspData->psbInfo.remDlId;
		psbInfo.remDlId << psbRemDlId;

		psbInfo.labelId = lspData->psbInfo.labelId.label32();
		//
		rsbInfo.nodeId = lspData->rsbInfo.nodeId;

		rsbLocTelId = lspData->rsbInfo.locTelId;
		rsbInfo.locTelId << rsbLocTelId;

		rsbLocDlId = lspData->rsbInfo.locDlId;
		rsbInfo.locDlId << rsbLocDlId;

		rsbRemDlId = lspData->rsbInfo.remDlId;
		rsbInfo.remDlId << rsbRemDlId;

		rsbInfo.labelId = lspData->rsbInfo.labelId.label32();

		//get TNAs
		iTnaResTmp = lspData->iTna;
		iTnaRes   << iTnaResTmp;

		eTnaResTmp = lspData->eTna;
		eTnaRes   << eTnaResTmp;

		//getCallId
		callIdTmp = lspData->callId;
		callId   << callIdTmp;

		if (adjType == UNI ||
		    adjType == ENNI) {
			//getCallParams
			callParamsTmp = new g2mplsTypes::callParams(lspData->callParms);
			callInfo << callParamsTmp;

			//get GnsTnas
			if (lspData->mask_.iGnsTna == true) {
				iGnsTnaTmp = new g2mplsTypes::gridParams(lspData->iGnsTna);
				iGnsTna   << iGnsTnaTmp;
				mask.iGnsTna = 1;
			}
			if (lspData->mask_.eGnsTna == true) {
				eGnsTnaTmp = new g2mplsTypes::gridParams(lspData->eGnsTna);
				eGnsTna   << eGnsTnaTmp;
				mask.eGnsTna = 1;
			}

			//get AAAinfo
			if (lspData->mask_.aaaInfo == true) {
				aaaInfo = new g2mplsTypes::aaaParams(lspData->aaaInfo);
				authz   = aaaInfo->authzToken;
				mask.authz = 1;
			}
		}

		//get ERO
		//eroItem = lspData->ero;
		ero.clear();
		for (size_t i = 0; i < (lspData->ero).length(); i++) {
			//eroItTmp   = eroItem[i];
			eroItTmp   = (lspData->ero)[i];
			eroItemProcess(eroItTmp, ero);
		}

	} catch (Types::InternalProblems) {
		G2RSVPTE_ERR("Cannot fetch LSP in Database: internal problems");
		resp = "internal problems";
		return false;
	} catch (Types::CannotFetch) {
		G2RSVPTE_ERR("Cannot fetch LSP in Database: LSP not present");
		resp = "LSP not stored in Persistency Controller";
		return false;
	} catch (PersistencyController::EndOfRecords & e) {
		G2RSVPTE_DBG("Cannot fetch LSP in Database: end of records");
		resp = "end of records";
		return false;
	} catch (CORBA::SystemException & e) {
		G2RSVPTE_ERR("Cannot fetch LSP in Database: "
			     "CORBA::SystemException (0x%x)", (uint32_t) e.minor());
		resp = "CORBA System Exception";
		return false;
	} catch (CORBA::Exception & e) {
		G2RSVPTE_ERR("Cannot fetch LSP in Database: "
			     "CORBA::Exception");
		resp = "CORBA exception";
		return false;
	} catch (omniORB::fatalException & e) {
		G2RSVPTE_ERR("Cannot fetch LSP in Database: "
			     "omniORB::fatalException:");
		G2RSVPTE_ERR("  file: %s\n", e.file());
		G2RSVPTE_ERR("  line: %d\n", e.line());
		G2RSVPTE_ERR("  mesg: %s\n", e.errmsg());
		resp = "CORBA fatal exception";
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot fetch LSP in Database: unknown reason");
		return false;
	}

	G2RSVPTE_DBG("This LSP has been fetched:");
	SHOW_LSPIDENT(nextIdent);

	return true;

#endif // G2RSVPTE2PC_DEBUG
}
