//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "lib/corba.h"
#include "g2mpls_corba_utils.h"
#include "g2rsvpte_corba.h"

#include "g2rsvpted/g2rsvpte_tnrc_api.h"

// return true if label is found
bool check_label(g2mpls_addr_t   localDL,
		 uint32_t        label,
		 label_state_t & labelStatus,
		 opstate_t &     operStatus)
{
#ifdef G2RSVPTE2TNRC_DEBUG
	G2RSVPTE_DBG("CheckLabel on DL ... debugging");
	labelStatus = LABEL_FREE;
	operStatus  = UP;

	return true;
#endif

	bool res = false;
#if HAVE_OMNIORB
	try {
		g2mplsTypes::DLinkId_var locDL;
		locDL << localDL;

		//START-TMP
		label_id_t lab;
		lab.type             = LABEL_32BIT;
		lab.value.label32.id = label;
		//END-TMP
		g2mplsTypes::labelId_var lbl;
		lbl << lab;

		g2mplsTypes::operState  opState;
		g2mplsTypes::labelState lblState;

		res = tnrc_proxy->getLabelStatus(locDL,
						 lbl,
						 lblState,
						 opState);
		if (res) {
			labelStatus << lblState;
			operStatus  << opState;
		}

	} catch (TNRC::CannotFetch) {
		G2RSVPTE_ERR("CannotFetch ... cannot fetch label from local "
			     "DLink (TNRC)");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot get label from local DLink (TNRC)");
		return false;
	}
#endif
	return res;
}

// return true if the resource setting has been done
bool setResource(RSVP::LSP * lsp, bool isVirtual)
{
	if (!lsp) {
		G2RSVPTE_ERR("Invalid LSP pointer!");
		return false;
	}

#ifdef G2RSVPTE2TNRC_DEBUG
	G2RSVPTE_DBG("SetResource ... debugging");
	return true;
#endif

#if HAVE_OMNIORB

	G2RSVPTE_DBG("SetResource: requesting makeXC to TNRC");

	assert(lsp->psb());
	assert(lsp->rsb());

	try {
		g2mplsTypes::DLinkId_var dlinkIn;
		g2mplsTypes::DLinkId_var dlinkOut;
		g2mplsTypes::labelId_var lblIn;
		g2mplsTypes::labelId_var lblOut;
		label_id_t               labelIn;
		label_id_t               labelOut;
		Types::uint32            cookie;
		g2mplsTypes::xcDirection dir;
		CORBA::Long              responseCtxt;
		CORBA::Long              asyncCtxt;
		bool                     res;

		dlinkIn  << lsp->psb()->localDL();
		dlinkOut << lsp->rsb()->localDL();

		//TMP - TODO - only LABEL_32BIT supported
		labelIn.type              = LABEL_32BIT;
		labelIn.value.label32.id  = lsp->psb()->rxLabel();
		labelOut.type             = LABEL_32BIT;
		labelOut.value.label32.id = lsp->rsb()->txLabel();

		lblIn  << labelIn;
		lblOut << labelOut;

		cookie       = 0;
		//responseCtxt = (CORBA::Long) lsp;  //damian TODO: 64bit OS compliancy
		//asyncCtxt    = (CORBA::Long) lsp;  //damian TODO: 64bit OS compliancy
		responseCtxt = (CORBA::Long) lsp->context();
		asyncCtxt    = (CORBA::Long) lsp->context();

		dir        = g2mplsTypes::XCDIR_BIDIRECTIONAL;

		if (lsp->times().start_time && lsp->times().end_time) {
			// ADVANCE RESERVATION
			Types::uint32  advanceRsrv;
			CORBA::Long    startTime;
			CORBA::Long    endTime;
			//TMP - TODO - retrieve this information
			advanceRsrv = 1;
			startTime   = lsp->times().start_time;
			endTime     = lsp->times().end_time;

			res = tnrc_proxy->reserveXC(cookie,
						    dlinkIn,
						    lblIn,
						    dlinkOut,
						    lblOut,
						    isVirtual,
						    dir,
						    advanceRsrv,
						    startTime,
						    endTime,
						    responseCtxt);
		} else {
			Types::uint32  activate;
			Types::uint32  rsrvCookie;
			//TMP - TODO - retrieve this information
			activate   = 0;
			rsrvCookie = 0;

			res = tnrc_proxy->makeXC(cookie,
						 dlinkIn,
						 lblIn,
						 dlinkOut,
						 lblOut,
						 isVirtual,
						 dir,
						 activate,
						 rsrvCookie,
						 responseCtxt,
						 asyncCtxt);
		}
		if (res) {
			lsp->tnrcCookie(cookie);
			G2RSVPTE_DBG("SetResource: TNRC preliminary "
				     "response OK");
			return true;
		} else {
			G2RSVPTE_DBG("SetResource: TNRC preliminary "
				     "response failed");
			return false;
		}
	} catch (TNRC::ParamError) {
		G2RSVPTE_ERR("Cannot setResource: Parameters Error");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot setResource: unknown error");
		return false;
	}
#endif
	return true;
}

// return true if the resource releasing has been done
bool releaseResource(RSVP::LSP * lsp)
{
	if (!lsp) {
		G2RSVPTE_ERR("Invalid LSP pointer!");
		return false;
	}

#ifdef G2RSVPTE2TNRC_DEBUG
	G2RSVPTE_DBG("releaseResource ... debugging");
	return true;
#endif

#if HAVE_OMNIORB

	G2RSVPTE_DBG("releaseResource: requesting destroyXC to TNRC");

	try {
		Types::uint32            cookie;
		Types::uint32            deactivate;
		CORBA::Long              responseCtxt;
		bool                     res;

		cookie       = lsp->tnrcCookie();
		responseCtxt = (CORBA::Long) lsp->context(); //damian TODO: 64bit OS compliancy

		//TMP - TODO - retrieve this information
		deactivate   = 0;

		if (lsp->times().start_time && lsp->times().end_time) {
			// ADVANCE RESERVATION
			res = tnrc_proxy->unreserveXC(cookie,
						      responseCtxt);
		} else {
			res = tnrc_proxy->destroyXC(cookie,
						    deactivate,
						    responseCtxt);
		}
		if (res) {
			G2RSVPTE_DBG("releaseResource: TNRC preliminary "
				     "response OK");
			return true;
		} else {
			G2RSVPTE_DBG("releaseResource: TNRC preliminary "
				     "response failed");
			return false;
		}
	} catch (TNRC::ParamError) {
		G2RSVPTE_ERR("Cannot releaseResource: Parameters Error");
		return false;
	} catch (...) {
		G2RSVPTE_ERR("Cannot releaseResource: unknown error");
		return false;
	}
#endif
	return true;
}
