//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "g2rsvpted/g2rsvpte_utils.h"
#include "g2rsvpted/g2rsvpte_trace.h"
#include "g2rsvpted/g2rsvpte_message_objects.h"
#include "g2mpls_corba_utils.h"
//
// host2net/net2host utils
//
uint64_t htonll(uint64_t val)
{
	return( ( ((uint64_t)htonl((uint32_t)val)) << 32 ) +
		htonl( (uint32_t)(val >> 32) ) );
}

uint64_t ntohll(uint64_t val)
{
	return( ( ((uint64_t)ntohl((uint32_t)val)) << 32 ) +
		ntohl( (uint32_t)(val >> 32) ) );
}

//
// Timer utils
//
uint32_t generateRefreshInterval(uint32_t interval)
{
	// we should pick up a random value in the [0.5 * R; 1.5 * R] interval
	uint32_t refr;

	//srand (time(NULL)) ;
	refr = (uint32_t)(interval * 0.5 +
			  ((interval * (float)rand()) / (float)RAND_MAX_INT));

	// useless if rand() works well
	if (refr > (uint32_t)(1.5 * interval)) {
		G2RSVPTE_ERR("Bad random value for refresh interval "
			     "(%d > 1.5 * %d)!!!!\n", refr, interval);
	}

	return refr;
}

//
// ERO utils
//
bool fill_ero(uint32_t                          src,
	      uint32_t                          dst,
	      std::list<RSVP::EroSubObject *> & subObjs)
{
	if (!subObjs.empty()) {
		G2RSVPTE_WRN("ERO sub objs list is not empty!");
		return false;
	}

	if (!src || !dst) {
		G2RSVPTE_ERR("src end/or dst node id are invalid");
		return false;
	}

#ifdef G2RSVPTE2PC_DEBUG
	RSVP::IPv4Pref_SubObj * ipv4;
	ipv4_t                  tmpId;

#if 0 // ERO without iTNA
	// Source node
	ipv4 = new RSVP::IPv4Pref_SubObj;
	if (!ipv4) {
		G2RSVPTE_ERR("Impossible to allocate memory for src ERO sObj");
		return false;
	}
	ipv4->get().preflen          = 32;
	//ipv4->get().ipv4_addr.s_addr = src;

	memset(&tmpId, 0, sizeof(tmpId));
	if (strchr(ITNA, '.') != 0) {
		// matched "A.B.C.D"
		if (!inet_aton(ITNA, &(tmpId))) {
			G2RSVPTE_ERR("Invalid IP address (%s) for local id",
				     ITNA);
			return false;
		}
		ipv4->get().ipv4_addr = tmpId;
	} else {
		G2RSVPTE_ERR("Invalid ITNA");
	}

	G2RSVPTE_DBG("Adding node %s to ERO sObj",
		     numb_htoa(ntohl(ipv4->get().ipv4_addr.s_addr)));
	subObjs.push_back(ipv4);
#endif

	// Destination node
	ipv4 = new RSVP::IPv4Pref_SubObj;
	if (!ipv4) {
		G2RSVPTE_ERR("Impossible to allocate memory for src ERO sObj");
		return false;
	}
	ipv4->get().preflen          = 32;
	//ipv4->get().ipv4_addr.s_addr = dst;

	memset(&tmpId, 0, sizeof(tmpId));
	if (strchr(REMOTEID2, '.') != 0) {
		// matched "A.B.C.D"
		if (!inet_aton(REMOTEID2, &(tmpId))) {
			G2RSVPTE_ERR("Invalid IP address (%s) for local id",
				     REMOTEID2);
			return false;
		}
		ipv4->get().ipv4_addr = tmpId;
	} else {
		G2RSVPTE_ERR("Invalid REMOTEID2");
	}

	G2RSVPTE_DBG("Adding node %s to ERO sObj",
		     numb_htoa(ntohl(ipv4->get().ipv4_addr.s_addr)));
	subObjs.push_back(ipv4);

	return true;
#endif

	//XXX TODO FIXME: to be implemented (connection with Path Computation)
	return false;
}

#if HAVE_OMNIORB
g2mplsTypes::eroItem_var createNullEroItem(void)
{
	g2mplsTypes::eroItem_var eroIt;

	eroIt = new g2mplsTypes::eroItem;

	eroIt->node = 0;
	eroIt->teLink.ipv4(0);
	eroIt->upstreamDataLink.ipv4(0);
	eroIt->downstreamDataLink.ipv4(0);
	eroIt->upstreamLabel.label32(0);
	eroIt->downstreamLabel.label32(0);
	eroIt->loose = false;

	return eroIt;
}

g2mplsTypes::eroItem getEroItem(const lsp_ero_sobj_t & sObj)
{
	g2mplsTypes::eroItem_var eroIt;

	eroIt = createNullEroItem();

	switch (sObj.type) {
		case ERO_SUBOBJ_IPV4:
			eroIt->teLink.ipv4(ntohl(sObj.hop.ipv4.addr.s_addr));
			break;
		case ERO_SUBOBJ_IPV6:
			throw("ERO subobject of type ERO_SUBOBJ_IPV6 is not yet supported");
		case ERO_SUBOBJ_LABEL:
			if (sObj.hop.label.upstream == LABEL_UPSTREAM) {
				eroIt->upstreamLabel.label32(sObj.hop.label.value);
			} else {
				eroIt->downstreamLabel.label32(sObj.hop.label.value);
			}
			break;
		case ERO_SUBOBJ_UNNUM:
			eroIt->node = sObj.hop.unnum.router_id;
			eroIt->teLink.unnum(sObj.hop.unnum.intf_id);
			break;
		case ERO_SUBOBJ_NODE:
			eroIt->node = sObj.hop.node.node_id;
			break;
		case ERO_SUBOBJ_AS:
			throw("ERO subobject of type ERO_SUBOBJ_AS is not yet supported");
			break;
		case ERO_SUBOBJ_UNKNOWN:
			throw("ERO subobject of type ERO_SUBOBJ_UNKNOWN");
			break;
	}

	eroIt->loose = (sObj.loose == ERO_SUBOBJ_LOOSE) ? true : false;

	return eroIt._retn();
}

void
eroItemProcess(const g2mplsTypes::eroItem_var & eroItem,
	       std::list<lsp_ero_sobj_t> &      ero)
{
	lsp_ero_sobj_t eroSubObj;

	linkId2eroSubObj(eroItem->node, eroItem->teLink, eroItem->loose, eroSubObj);
	if (eroSubobjIsConsistent(eroSubObj)) {
		//insert ERO Sub Obj in the list
		ero.push_back(eroSubObj);
	}

	label2eroSubObj(eroItem->upstreamLabel, true, eroItem->loose, eroSubObj);
	if (eroSubobjIsConsistent(eroSubObj)) {
		//insert ERO Sub Obj in the list
		ero.push_back(eroSubObj);
	}

	label2eroSubObj(eroItem->downstreamLabel, false, eroItem->loose, eroSubObj);
	if (eroSubobjIsConsistent(eroSubObj)) {
		//insert ERO Sub Obj in the list
		ero.push_back(eroSubObj);
	}
}

void
linkId2eroSubObj(const g2mplsTypes::nodeId & nodeId,
		 const g2mplsTypes::linkId & linkId,
		 bool                        loose,
		 lsp_ero_sobj_t &            eroSubObj)
{
	memset(&eroSubObj, 0, sizeof(eroSubObj));

	g2mpls_addr_t link;

	try {
		g2mplsTypes::linkId_var tmp;
		tmp = linkId;
		link << tmp;
	} catch (...){
		G2RSVPTE_ERR("Problems in converting linkId");
		return;
	}

	if (nodeId &&
	    (linkId._d() != g2mplsTypes::LINKIDTYPE_UNNUM) &&
	    is_addr_null(link) != 0) {
		eroSubObj.type             = ERO_SUBOBJ_NODE;
		eroSubObj.hop.node.node_id = nodeId;
		if (loose) {
			eroSubObj.loose = ERO_SUBOBJ_LOOSE;
		} else {
			eroSubObj.loose = ERO_SUBOBJ_STRICT;
		}
		return;
	}

	switch (linkId._d()) {
		case g2mplsTypes::LINKIDTYPE_IPV4: {
			eroSubObj.type                 = ERO_SUBOBJ_IPV4;
			eroSubObj.hop.ipv4.addr.s_addr = htonl(linkId.ipv4());
			eroSubObj.hop.ipv4.prefix      = 32;
			break;
		}
		case g2mplsTypes::LINKIDTYPE_IPV6: {
			int    i;

			eroSubObj.type    = ERO_SUBOBJ_IPV6;
			for (i = 0; i < 4; i++) {
				eroSubObj.hop.ipv6.addr.s6_addr32[i] = linkId.ipv6()[i];
			}
			eroSubObj.hop.ipv6.prefix      = 128;
			break;
		}
		case g2mplsTypes::LINKIDTYPE_UNNUM: {
		        eroSubObj.type                = ERO_SUBOBJ_UNNUM;
			eroSubObj.hop.unnum.router_id = nodeId;
			eroSubObj.hop.unnum.intf_id   = linkId.unnum();
			break;
		}
		default:
			eroSubObj.type = ERO_SUBOBJ_UNKNOWN;
			break;
	}

	if (loose) {
		eroSubObj.loose = ERO_SUBOBJ_LOOSE;
	} else {
		eroSubObj.loose = ERO_SUBOBJ_STRICT;
	}
}

void
label2eroSubObj(const g2mplsTypes::labelId & labelId,
		bool                         upstream,
		bool                         loose,
		lsp_ero_sobj_t &             eroSubObj)
{
	memset(&eroSubObj, 0, sizeof(eroSubObj));

	switch (labelId._d()) {
		case g2mplsTypes:: LABELTYPE_L32: {
			eroSubObj.type = ERO_SUBOBJ_LABEL;
			eroSubObj.hop.label.value = labelId.label32();
			if (upstream) {
				eroSubObj.hop.label.upstream = LABEL_UPSTREAM;
			} else {
				eroSubObj.hop.label.upstream = LABEL_DOWNSTREAM;
			}
			break;
		}
		case g2mplsTypes:: LABELTYPE_L60: {
			// TODO FIXME
			// not yet supported in G2.RSVP-TE ERO
			//
			eroSubObj.type = ERO_SUBOBJ_UNKNOWN;
			break;
		}
		default:
			eroSubObj.type = ERO_SUBOBJ_UNKNOWN;
			break;
	}

	if (loose) {
		eroSubObj.loose = ERO_SUBOBJ_LOOSE;
	} else {
		eroSubObj.loose = ERO_SUBOBJ_STRICT;
	}
}

bool
eroSubobjIsConsistent(const lsp_ero_sobj_t & eroSubObj)
{
	switch (eroSubObj.type) {
		case ERO_SUBOBJ_UNKNOWN:
			break;
		case ERO_SUBOBJ_IPV4:
			if (eroSubObj.hop.ipv4.addr.s_addr) {
				return true;
			}
			break;
		case ERO_SUBOBJ_IPV6:
			if (eroSubObj.hop.ipv6.addr.s6_addr32[0] &&
			    eroSubObj.hop.ipv6.addr.s6_addr32[1] &&
			    eroSubObj.hop.ipv6.addr.s6_addr32[2] &&
			    eroSubObj.hop.ipv6.addr.s6_addr32[3]) {
				return true;
			}
			break;
		case ERO_SUBOBJ_UNNUM:
			if (eroSubObj.hop.unnum.router_id &&
			    eroSubObj.hop.unnum.intf_id) {
				return true;
			}
			break;
		case ERO_SUBOBJ_NODE:
			if (eroSubObj.hop.node.node_id) {
				return true;
			}
			break;
		case ERO_SUBOBJ_LABEL:
			if (eroSubObj.hop.label.value) {
				return true;
			}
			break;
		default:
			break;
	}

	return false;
}

g2mplsTypes::eroSeq_var
getExclEroSeq(std::list<lsp_ero_sobj_t> exclEro)
{
	lsp_ero_sobj_t                      sObj;
	std::list<lsp_ero_sobj_t>::iterator it;
	size_t                              exclEroSize = 0;

	//get exclEro size
	for (it  = exclEro.begin();
	     it != exclEro.end();
	     it++) {
		memset(&sObj, 0, sizeof(sObj));
		sObj = *it;
		if (sObj.type == ERO_SUBOBJ_IPV4) {
			exclEroSize++;
		}
	}

	g2mplsTypes::eroSeq_var exclEroSeq;
	{
		g2mplsTypes::eroSeq * tmp;

		tmp = new g2mplsTypes::eroSeq(exclEroSize);
		if (!tmp) {
			G2RSVPTE_ERR("Impossible to allocate "
				     "a new eroSeq sequence");
			return false;
		}
		exclEroSeq = tmp;
	}
	exclEroSeq->length(exclEroSize);

	g2mplsTypes::eroItem_var eroIt;
	uint32_t                 errNode;
	int i = 0;

	for (it  = exclEro.begin();
	     it != exclEro.end();
	     it++) {
		memset(&sObj, 0, sizeof(sObj));
		sObj = *it;

		switch (sObj.type) {
			case ERO_SUBOBJ_NODE:
				errNode = sObj.hop.node.node_id;
				break;
			case ERO_SUBOBJ_IPV4:
				eroIt = createNullEroItem();
				eroIt->node = errNode;
				eroIt->teLink.ipv4(sObj.hop.ipv4.addr.s_addr);
				eroIt->loose = (sObj.loose == ERO_SUBOBJ_LOOSE) ? true : false;
				exclEroSeq[i] = eroIt;
				i++;
				break;
			case ERO_SUBOBJ_LABEL:
				if (sObj.hop.label.upstream == LABEL_UPSTREAM) {
					exclEroSeq[i-1].upstreamLabel.label32(sObj.hop.label.value);
				} else {
					exclEroSeq[i-1].downstreamLabel.label32(sObj.hop.label.value);
				}
				break;
			default:
				throw("ERO subobject type not valid");
				break;
		}
	}

	return exclEroSeq;
}

#endif // HAVE_OMNIORB
