/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#ifndef G2RSVPTED_H
#define G2RSVPTED_H

#include <list>

#include "g2rsvpted/g2rsvpte_utils.h"
#include "g2rsvpted/g2rsvpte_dm.h"


// RSVP  master for system wide configuration
class RSVP_Master {
 public:
	static bool                        init(void);
	static bool                        destroy(void);
	static RSVP_Master &               instance(void);

	void                               init_vty(void);
	struct thread_master *             getMaster();

	static std::list<RSVP::G2RSVPTE *> getProtoList(void);
	static int                         getProtoListSize(void);
	static RSVP::G2RSVPTE *            getProtoInstance(adj_type_t adjType);
	static RSVP::G2RSVPTE *            getProtoInstanceNoTun(void);
	static RSVP::G2RSVPTE *            getProtoFromScngwFd(int fd);
	static bool                        attach(RSVP::G2RSVPTE * prot);
	static bool                        detach(RSVP::G2RSVPTE * prot);
	static char *                      getIorsDir(void);
	static void                        setIorsDir(char * dir);

	static void                        manageMsg(bool tunneledPackets,
						     g2mpls_addr_t   addr,
						     RSVP::Message * msg);

 protected:
	RSVP_Master& operator=(const RSVP_Master& j);
	RSVP_Master(const RSVP_Master & j);
	RSVP_Master(void);
	~RSVP_Master(void);

 private:
	static RSVP_Master *               instance_;
	static struct thread_master *      master_;     // thread master
	static std::list<RSVP::G2RSVPTE *> g2rsvptes_;  // instances list
	static time_t                      start_time_; // start time
	static char *                      iorsDir_;    // IOR directory
};

#define RSVP_MASTER    (RSVP_Master::instance())

#endif // G2RSVPTED_H
