//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "config.h"

#include "g2rsvpte_buffer.h"
#include "g2rsvpte_scn_socket.h"
#include "g2rsvpte_trace.h"
#include "g2rsvpted.h"
#include "g2rsvpte_messages.h"

#include "sockopt.h"
#include "sigevent.h"
#include "privs.h"

#include <errno.h>
#include <sstream>

// DUMMY ... for testing rsvp messages
struct thread_master * master = NULL;
struct zebra_privs_t   g2rsvpted_privs;
zebra_capabilities_t _caps_p [] = {
	ZCAP_NET_RAW,
	ZCAP_BIND,
	ZCAP_NET_ADMIN,
};



// SIG handlers
static void sighup (void)
{
	G2RSVPTE_INF("SIGHUP received");
}

static int terminate_execution = 0;

struct quagga_signal_t g2rsvpte_signals[4];
struct thread   thread;

static void sigint (void)
{
	G2RSVPTE_NOT("Terminating on SIGINT");

	// rsvp_terminate();
	terminate_execution = 1;
}

// SIGUSR1 handler.
static void sigusr1 (void)
{
	zlog_rotate (NULL);
}



using namespace RSVP;

#define TEST_PATH
//#define TEST_RESV
//#define TEST_PATH_ERR
//#define TEST_RESV_ERR
//#define TEST_PATH_TEAR
//#define TEST_RESV_TEAR
//#define TEST_RESV_CONF

#define SESSION_OBJ
// #define SESSION_ATTRIBUTE_OBJ
// #define SUGGESTED_LABEL_OBJ
// #define UPSTREAM_LABEL_OBJ
// #define RECOVERY_LABEL_OBJ
// #define LABEL_REQUEST_OBJ
// #define LABEL_SET_OBJ
 #define TIME_VALUES_OBJ
 #define RSVP_HOP_OBJ
// #define ERO_OBJ
 #define RRO_OBJ
#define SENDER_TEMPLATE_OBJ
// #define SENDER_TSPEC_OBJ
//#define PROTECTION_OBJ
// #define ADMIN_STATUS_OBJ
// #define GNS_CALL_EXT_OBJ
// #define INTEGRITY_OBJ
//#define GUNI_OBJ
// #define GNS_OBJ
// #define CALL_OPS        //NOT SUPPORTED BY WIRESHARK
// #define CALL_ID
// //#define   CALL_ID_GBL
// #define   CALL_ID_OPSPEC
// #define NOTIFY_REQ
// #define MESSAGE_ID
// #define MESSAGE_ID_ACK
// #define MESSAGE_ID_NACK
// #define POLICY_DATA       //NOT SUPPORTED BY WIRESHARK (need TIME & RSVP_HOP)
// #define RESV_CONFIRM
// #define STYLE
// #define FF_FLOW_DESC
// #define FILTER_SPEC

int main(int argc, char ** argv)
{
	G2RSVPTE_WRN("---------- TEST START ----------");

	int               count;
	PathMessage *     path;
	ResvMessage *     resv;
	PathErrMessage *  pathErr;
	ResvErrMessage *  resvErr;
	PathTearMessage * pathTear;
	ResvTearMessage * resvTear;
	ResvConfMessage * resvConf;
	struct in_addr    addr;

	count = 0;

	path = new PathMessage();
	assert(path);

	resv = new ResvMessage();
	assert(resv);

	pathErr = new PathErrMessage();
	assert(pathErr);

	resvErr = new ResvErrMessage();
	assert(resvErr);

	pathTear = new PathTearMessage();
	assert(pathTear);

	resvTear = new ResvTearMessage();
	assert(resvTear);

	resvConf = new ResvConfMessage();
	assert(resvConf);

	// DUMMY ... for testing rsvp messages
	// DUMMY ... for testing rsvp messages
	assert(RSVP_Master::init());
	master = RSVP_MASTER.getMaster();
	g2rsvpted_privs.caps_p    = _caps_p;
	g2rsvpted_privs.cap_num_p = sizeof(_caps_p)/sizeof(_caps_p[0]);
	g2rsvpted_privs.cap_num_i = 0;
	zprivs_init(&g2rsvpted_privs);

	g2rsvpte_signals[0].signal = SIGHUP;
	g2rsvpte_signals[0].handler = &sighup;
	g2rsvpte_signals[1].signal = SIGUSR1;
	g2rsvpte_signals[1].handler = &sigusr1;
	g2rsvpte_signals[2].signal = SIGINT;
	g2rsvpte_signals[2].handler = &sigint;
	g2rsvpte_signals[3].signal = SIGTERM;
	g2rsvpte_signals[3].handler = &sigint;
	signal_init(RSVP_MASTER.getMaster(),
		    Q_SIGC(g2rsvpte_signals),
		    g2rsvpte_signals);


	G2RSVPTE_WRN("---------- INITIALIZE PATH ----------");

	///////////
	GLabel_Object * l;
	l = new GLabel_Object();
	assert(l);
	l->get().label = new uint32_t[2];
	l->get().label[0] = 1212;
	l->get().label[1] = 1213;
	l->get().size  = sizeof(uint32_t) * 2;
	///////////

#ifdef SESSION_OBJ
	// OBJ 1
	IPv4Session_Object * obj = new IPv4Session_Object();
	assert(obj);
	inet_aton("127.0.0.1", &(obj->get().ipv4_addr));
	obj->get().protoId = 22;
	obj->get().flags = 0;
	obj->get().port = 2048;

	path->session_ = obj;//uni_obj; // obj;
	assert(obj);
	//CLONE
	resv->session_ = obj->clone(); //uni_obj;/obj;
#endif

#ifdef SESSION_ATTRIBUTE_OBJ
	// OBJ 2
	LSPTunnelRASessionAttr_Object * obj2;
	obj2 = new LSPTunnelRASessionAttr_Object();
	assert(obj2);
	obj2->get().exclude_any = 0x5e9;
	obj2->get().include_any = 0x02020101;
	obj2->get().include_all = 0x02030101;
	obj2->get().setup_prio = 10;
	obj2->get().hold_prio = 12;
	obj2->get().flags = 0;
	obj2->get().name = new char[sizeof("testXX")];
	memcpy(obj2->get().name, (void *)"testXX", sizeof("testXX"));
	obj2->get().name_len = strlen(obj2->get().name);
	//printf("size name %d \n", obj2->get().name_len);
	//printf("name %s\n", obj2->get().name);
	path->session_attribute_ = obj2;
#endif

#ifdef SUGGESTED_LABEL_OBJ
	// OBJ 3
	SuggestedLabel_Object * obj3;
	obj3 = new SuggestedLabel_Object();
	assert(obj3);
	//LabelMpls_Object * lbl = new LabelMpls_Object();
	//assert(lbl);
#define DEFINE_2LABEL
#ifdef DEFINE_2LABEL
	obj3->get().label = new uint32_t[2];
	obj3->get().label[0] = 12345678;
	obj3->get().label[1] = 11223344;
	count = 2;
#else
	obj3->get().label = new uint32_t(12345);
	count = 1;
#endif
	assert(obj3->get().label);
	obj3->get().size  = sizeof(*obj3->get().label) * count;
	//obj3->get().label = lbl;
	path->suggested_label_ = obj3;
#endif

#ifdef UPSTREAM_LABEL_OBJ
	// OBJ 4
	UpstreamLabel_Object * obj4;
	obj4 = new UpstreamLabel_Object();
	assert(obj4);
#ifdef DEFINE_2LABEL
	obj4->get().label = new uint32_t[2];
	obj4->get().label[0] = 87654321;
	obj4->get().label[1] = 44332211;
	count = 2;
#else
	obj4->get().label = new uint32_t(54321);
	count = 1;
#endif
	assert(obj4->get().label);
	obj4->get().size  = sizeof(*obj4->get().label) * count;
	path->upstream_label_ = obj4;
#endif

#ifdef RECOVERY_LABEL_OBJ
	// OBJ 5
	RecoveryLabel_Object * obj5;
	obj5 = new RecoveryLabel_Object();
	assert(obj5);
#ifdef DEFINE_2LABEL
	obj5->get().label = new uint32_t[2];
	obj5->get().label[0] = 123123;
	obj5->get().label[1] = 112233;
	count = 2;
#else
	obj5->get().label = new uint32_t(1221);
	count = 1;
#endif
	assert(obj5->get().label);
	obj5->get().size  = sizeof(*obj5->get().label) * count;
	path->recovery_label_ = obj5;
#endif

#ifdef LABEL_REQUEST_OBJ
	// OBJ 6
	GLabelRequest_Object * obj6 = new GLabelRequest_Object();
	assert(obj6);
	obj6->get().enc_type    = 1;
	obj6->get().switch_type = 2;
	obj6->get().g_pid       = 25;
	path->label_request_    = obj6;
#endif

#ifdef LABEL_SET_OBJ
	// OBJ 7
	LabelSet_Object * obj7;
	obj7 = new LabelSet_Object();
	assert(obj7);
	// label
	GLabel_Object * lbl = new GLabel_Object();
	assert(lbl);
	lbl->get().label = new uint32_t(98765);
	lbl->get().size  = sizeof(uint32_t);
	obj7->get().labels.push_back(lbl);
#ifdef DEFINE_2LABEL
	GLabel_Object * lbl2 = new GLabel_Object();
	assert(lbl2);
	lbl2->get().label = new uint32_t[2];
	lbl2->get().label[0] = 1212;
	lbl2->get().label[1] = 1313;
	lbl2->get().size  = sizeof(uint32_t) * 2;
	obj7->get().labels.push_back(lbl2);
#endif
	obj7->get().action     = 1;
	obj7->get().label_type = CTYPE_GENERALISED_LABEL; //2
	path->label_set_ = obj7;
#endif

#ifdef TIME_VALUES_OBJ
	// OBJ 8
	TimeValues_Object * obj8 = new TimeValues_Object();
	assert(obj8);
	obj8->get().refresh_period = 1000;
	path->time_values_ = obj8;
#endif

#ifdef RSVP_HOP_OBJ
	// OBJ 9
#if 0
	IPv4RsvpHop_Object * obj9 = new IPv4RsvpHop_Object();
	assert(obj9);
	inet_aton("192.168.1.1", &(obj9->get().ipv4_next_hop));
	obj9->get().if_handle = 1010;
#else
	IFIDIPv4RsvpHop_Object * obj9 = new IFIDIPv4RsvpHop_Object();
	assert(obj9);
	inet_aton("192.168.1.1", &(obj9->get().next_prev_ipv4_hop));
	obj9->get().logical_if_hdl = 1010;
	// TLV1: IF_IDX_TLV
	IF_IDX_TLV * tlv = new IF_IDX_TLV();
	assert(tlv);
	inet_aton("192.168.1.2", &(addr));
	tlv->get().ip_address = ntohl(addr.s_addr);
	tlv->get().interface_id = 4660; //0x1234
	obj9->get().tlvs.push_back(tlv);
	// TLV2: IPv4_TLV
	IPv4_TLV * tlv2 = new IPv4_TLV();
	assert(tlv2);
	inet_aton("192.168.1.3", &(tlv2->get().ipv4_addr));
	obj9->get().tlvs.push_back(tlv2);
	// TLV3
	COMP_IF_DOWN_TLV * tlv3 = new COMP_IF_DOWN_TLV();
	assert(tlv3);
	inet_aton("192.168.1.4", &(addr));
	tlv3->get().ip_address = ntohl(addr.s_addr);
	tlv3->get().interface_id = 4662; //0x1236
	obj9->get().tlvs.push_back(tlv3);
#endif
	path->rsvp_hop_ = obj9;
#endif

#ifdef ERO_OBJ
	// OBJ 10
	ExplicitRoute_Object * obj10 = new ExplicitRoute_Object();
	assert(obj10);
	// ERO_SUBOBJ1: IPv4Pref_SubObj
	IPv4Pref_SubObj * sObj = new IPv4Pref_SubObj();
	assert(sObj);
	inet_aton("192.168.1.10", &(sObj->get().ipv4_addr));
	sObj->get().preflen = 32;
	sObj->looseFlag(1);
	obj10->get().subObjs.push_back(sObj);
	// ERO_SUBOBJ2: ASNum_SubObj
	ASNum_SubObj * sObj2 = new ASNum_SubObj();
	assert(sObj2);
	sObj2->get().as_number = 0x1234;
	sObj2->looseFlag(1);
	obj10->get().subObjs.push_back(sObj2);
	// ERO_SUBOBJ3: Unnum_SubObj
	Unnum_SubObj * sObj3 = new Unnum_SubObj();
	assert(sObj3);
	sObj3->get().router_id    = 12345;
	sObj3->get().interface_id = 12346;
	sObj3->looseFlag(1);
	obj10->get().subObjs.push_back(sObj3);

	// ERO_SUBOBJ4: LabelERO_SubObj
	LabelERO_SubObj * sObj4 = new LabelERO_SubObj();
	assert(sObj4);
	sObj4->get().upstream = 1;
	///////////
	//LabelMpls_Object * l = new LabelMpls_Object();
	//assert(l);
	//l->get().label = 9988;
	sObj4->get().c_type = l->cType();
	sObj4->get().label = l->clone();
	sObj4->looseFlag(1);
	obj10->get().subObjs.push_back(sObj4);
	//
	path->ero_ = obj10;
#endif

#ifdef RRO_OBJ
	// OBJ 11
	RecordedRoute_Object * obj11 = new RecordedRoute_Object();
	assert(obj11);
	// RRO_SUBOBJ1: IPv4Addr_SubObj
	IPv4Addr_SubObj * rroSubObj = new IPv4Addr_SubObj();
	assert(rroSubObj);
	inet_aton("192.168.11.10", &(rroSubObj->get().ipv4_addr));
	rroSubObj->get().preflen = 32;
	rroSubObj->get().flags = 1;
	obj11->get().subObjs.push_back(rroSubObj);
	// RRO_SUBOBJ3: Unnumbered_SubObj
	Unnumbered_SubObj * rroSubObj3 = new Unnumbered_SubObj();
	assert(rroSubObj3);
	rroSubObj3->get().flags        = 2;
	rroSubObj3->get().router_id    = 12345;
	rroSubObj3->get().interface_id = 12346;
	obj11->get().subObjs.push_back(rroSubObj3);
	// RRO_SUBOBJ4: Label_SubObj
	Label_SubObj * rroSubObj4 = new Label_SubObj();
	assert(rroSubObj4);
	rroSubObj4->get().flags = 1;
	///////////
	//l = new LabelMpls_Object();
	//assert(l);
	//l->get().label = 9988;
	///////////
	rroSubObj4->get().flags  = 2;
	rroSubObj4->get().c_type = l->cType();
	rroSubObj4->get().label  = l->clone();
	obj11->get().subObjs.push_back(rroSubObj4);
	//
	path->rro_ = obj11;
#endif

#ifdef SENDER_TEMPLATE_OBJ
	// OBJ 12
	IPV4LSPSenderTemplate_Object * obj12 = new IPV4LSPSenderTemplate_Object();
	assert(obj12);
	inet_aton("192.168.33.212", &(obj12->get().ipv4_addr));
	obj12->get().lsp_id = 222;
	path->sender_template_ = obj12;
#endif

#ifdef SENDER_TSPEC_OBJ
	// OBJ 13
	SenderTSpec_Object * obj13 = new SenderTSpec_Object();
	assert(obj13);
	obj13->get().msg_version         = 0;
	obj13->get().reserved12          = 0;
	obj13->get().length              = 7;
	obj13->get().service_header      = 1;
	obj13->get().zero_bit            = 0;
	obj13->get().reserved7           = 0;
	obj13->get().service_data_length = 6;
	obj13->get().param_id_127        = 127;
	obj13->get().param127_flags      = 0;
	obj13->get().param127_length     = 5;
	obj13->get().token_bucket_rate   = 123;
	obj13->get().token_bucket_size   = 1234;
	obj13->get().peak_data_rate      = 12345;
	obj13->get().min_policed_unit    = 123456;
	obj13->get().max_policed_unit    = 1234567;
	path->sender_tspec_ = obj13;
#endif

#ifdef PROTECTION_OBJ
	// OBJ 14
	Protection_Object * obj14 = new Protection_Object();
	assert(obj14);
	obj14->get().secondary  = 1;
	obj14->get().reserved   = 0x00000000;
	obj14->get().link_flags = 0xf0; //0xffff;
	path->protection_ = obj14;
#endif

#ifdef ADMIN_STATUS_OBJ
	// OBJ 15
	AdminStatus_Object * obj15 = new AdminStatus_Object();
	assert(obj15);
	obj15->get().R          = 1;
	obj15->get().reserved   = 0xfff00fff;
	obj15->get().T          = 0;
	obj15->get().A          = 1;
	obj15->get().D          = 1;
	path->admin_status_ = obj15;
#endif

#ifdef GNS_CALL_EXT_OBJ
	// OBJ 16
	GnsCallExt_Object * obj16 = new GnsCallExt_Object();
	assert(obj16);
	obj16->get().gns_trans_id = 0x07070707;
	obj16->get().gns_trans_id = obj16->get().gns_trans_id << 32 | 0x06060606;
	obj16->get().start_time   = 123;
	obj16->get().duration     = 333;
	obj16->get().job_name     = new char[sizeof("job_name")];
	memcpy(obj16->get().job_name, (void *)"job_name", sizeof("job_name"));
	obj16->get().job_name_len = strlen(obj16->get().job_name);
	obj16->get().job_project  = new char[sizeof("job_project")];
	memcpy(obj16->get().job_project,
	       (void *)"job_project",
	       sizeof("job_project"));
	obj16->get().job_proj_len = strlen(obj16->get().job_project);
	path->gns_call_ext_ = obj16;
	resv->gns_call_ext_ = obj16->clone();
#endif

#ifdef INTEGRITY_OBJ
	// OBJ 17
	Integrity_Object * obj17 = new Integrity_Object();
	assert(obj17);
	obj17->get().flags  = 0x80;

	obj17->get().key_id = 0x00000707;
	obj17->get().key_id = obj17->get().key_id << 32 | 0x06060606;

	obj17->get().seq_number = 0x01010101;
	obj17->get().seq_number = obj17->get().seq_number << 32 | 0x02020202;
	for(int i = 0; i < MSG_DIGEST_LEN; i++)
		obj17->get().digest[i] = i;
	path->integrity_ = obj17;
#endif

#ifdef GUNI_OBJ
	// OBJ 18
	GeneralisedUni_Object * obj18 = new GeneralisedUni_Object();
	assert(obj18);

	// GUNI_SUBOBJ1: Application_SubObject
	IPv4SourceTnaAddr_SubObj * tna = new IPv4SourceTnaAddr_SubObj();
	assert(tna);
	inet_aton("192.168.2.2", &(tna->get().ipv4_addr));
	obj18->get().subObjs.push_back(tna);

	// GUNI_SUBOBJ2: NSAPSourceTnaAddr_SubObj
	NSAPSourceTnaAddr_SubObj * nsap = new NSAPSourceTnaAddr_SubObj();
	assert(nsap);
	nsap->get().nsap_addr = new char[sizeof("10.10.1.1")];
	memcpy(nsap->get().nsap_addr, (void *)"10.10.1.1", sizeof("10.10.1.1"));
	nsap->get().nsap_len  = strlen(nsap->get().nsap_addr);
	obj18->get().subObjs.push_back(nsap);

	// GUNI_SUBOBJ3: ServiceLevel_SubObj
	ServiceLevel_SubObj * serv = new ServiceLevel_SubObj();
	assert(serv);
	serv->get().service_level = 0x02;
	obj18->get().subObjs.push_back(serv);

	UNIIPv4Session_Object * uni_obj = 0;
	{
		uni_obj = new UNIIPv4Session_Object();
		assert(uni_obj);
		inet_aton("192.168.1.1", &(uni_obj->get().ipv4_addr));
		uni_obj->get().tunnel_id     = 22;
		uni_obj->get().ext_IPv4_addr = 2048;
	}

	// GUNI_SUBOBJ4: Diversity_SubObj
	Diversity_SubObj * div = new Diversity_SubObj();
	div->get().DS                     = 0x01;
	div->get().uni_ipv4_session       = uni_obj;
	div->get().lsp_tun_ipv4_send_temp = static_cast
		<IPV4LSPSenderTemplate_Object *>(obj12->clone());
	obj18->get().subObjs.push_back(div);

	// GUNI_SUBOBJ5: EgressLabel_SubObj
	EgressLabel_SubObj * egress = new EgressLabel_SubObj();
	egress->get().U          = 0x00;
	GLabel_Object * l2;
	l2 = new GLabel_Object();
	assert(l2);
	l2->get().label = new uint32_t(291);
	l2->get().size  = sizeof(uint32_t);
	egress->get().label_type      = l2->cType();
	egress->get().logical_port_id = 0xbeefface;
	egress->get().egress_label    = l2;
	obj18->get().subObjs.push_back(egress);

	// GUNI_SUBOBJ6: SpcLabel_SubObj
	SpcLabel_SubObj * spc = new SpcLabel_SubObj();
	spc->get().U          = 0x01;
	GLabel_Object * l3 = new GLabel_Object();
	assert(l3);
	l3->get().label = new uint32_t(293);
	l3->get().size  = sizeof(uint32_t);
	spc->get().label_type      = l3->cType();
	spc->get().logical_port_id = 0xbeefface;
	spc->get().spc_label       = l3;
	obj18->get().subObjs.push_back(spc);

	path->gen_uni_ = obj18;
#endif

#ifdef GNS_OBJ
	// OBJ 19
	GnsUni_Object * obj19 = new GnsUni_Object();
	assert(obj19);

	// GNS_UNI_SUBOBJ1: Application_SubObj
	Application_SubObj * app = new Application_SubObj();
	assert(app);
	app->get().app_type    = 0x01;
	app->get().app_version = 0x02;
	obj19->get().subObjs.push_back(app);

	// GNS_UNI_SUBOBJ2/1: IPv4CandidateHost_SubObj
	IPv4CandidateHost_SubObj * ipv4Host = new IPv4CandidateHost_SubObj();
	assert(ipv4Host);
	inet_aton("254.253.252.251", &(ipv4Host->get().ipv4_addr));
	obj19->get().subObjs.push_back(ipv4Host);
	// GNS_UNI_SUBOBJ2/3: NsapCandidateHost_SubObj
	NsapCandidateHost_SubObj * nsapHost = new NsapCandidateHost_SubObj();
	assert(nsapHost);
	nsapHost->get().nsap_addr = new char[sizeof("10.10.10.10.10")];
	memcpy(nsapHost->get().nsap_addr,
	       (void *)"10.10.10.10.10",
	       sizeof("10.10.10.10.10"));
	nsapHost->get().nsap_len  = strlen(nsapHost->get().nsap_addr);
	obj19->get().subObjs.push_back(nsapHost);

	// GNS_UNI_SUBOBJ3: FsResources_SubObj
	FsResources_SubObj * fsRes = new FsResources_SubObj();
	assert(fsRes);
	fsRes->get().fs_name          = 0x3;
	fsRes->get().fs_type          = 0x4;
	fsRes->get().range_flags      = 0x11;
	fsRes->get().disk_space_lb    = 1234;
	fsRes->get().disk_space_ub    = 9876;
	fsRes->get().mount_point      = new char[sizeof("/home/grid")];
	memcpy(fsRes->get().mount_point,
	       (void*)"/home/grid",
	       sizeof("/home/grid"));
	fsRes->get().mount_point_len  = strlen(fsRes->get().mount_point);
	fsRes->get().mount_source     = new char[sizeof("nfs://1.1.1.1/pub/")];
	memcpy(fsRes->get().mount_source,
	       (void*)"nfs://1.1.1.1/pub/",
	       sizeof("nfs://1.1.1.1/pub/"));
	fsRes->get().mount_source_len = strlen(fsRes->get().mount_source);
	obj19->get().subObjs.push_back(fsRes);

	// GNS_UNI_SUBOBJ4: SystemCaps_SubObj
	SystemCaps_SubObj * sysCap = new SystemCaps_SubObj();
	assert(sysCap);
	sysCap->get().os_version    = 0x1111;
	sysCap->get().os_type       = 0x2222;
	sysCap->get().cpu_arch      = 0x33;
	sysCap->get().E             = 1;
	obj19->get().subObjs.push_back(sysCap);

	// GNS_UNI_SUBOBJ5: IndvCpuSpeed_SubObj
	IndvCpuSpeed_SubObj * ics = new IndvCpuSpeed_SubObj();
	assert(ics);
	ics->get().lower_bound   = 0x11111111;
	ics->get().upper_bound   = 0x99999999;
	ics->get().SF            = 0x3;
	obj19->get().subObjs.push_back(ics);

	// GNS_UNI_SUBOBJ6: IndvCpuTime_SubObj
	IndvCpuTime_SubObj * ict = new IndvCpuTime_SubObj();
	assert(ict);
	ict->get().lower_bound   = 0x11111111;
	ict->get().upper_bound   = 0x99999999;
	ict->get().SF            = 0x3;
	obj19->get().subObjs.push_back(ict);

	// GNS_UNI_SUBOBJ7: IndvCpuCount_SubObj
	IndvCpuCount_SubObj * icc = new IndvCpuCount_SubObj();
	assert(icc);
	icc->get().lower_bound   = 0x11111111;
	icc->get().upper_bound   = 0x99999999;
	icc->get().SF            = 0x3;
	obj19->get().subObjs.push_back(icc);

	// GNS_UNI_SUBOBJ8: IndvNetBw_SubObj
	IndvNetBw_SubObj * nbw = new IndvNetBw_SubObj();
	assert(nbw);
	nbw->get().lower_bound   = 0x11111111;
	nbw->get().upper_bound   = 0x99999999;
	nbw->get().SF            = 0x3;
	obj19->get().subObjs.push_back(nbw);

	// GNS_UNI_SUBOBJ9: IndvPhyMem_SubObj
	IndvPhyMem_SubObj * ipm = new IndvPhyMem_SubObj();
	assert(ipm);
	ipm->get().lower_bound   = 0x11111111;
	ipm->get().upper_bound   = 0x99999999;
	ipm->get().SF            = 0x3;
	obj19->get().subObjs.push_back(ipm);

	// GNS_UNI_SUBOBJ10: IndvVirMem_SubObj
	IndvVirMem_SubObj * ivm = new IndvVirMem_SubObj();
	assert(ivm);
	ivm->get().lower_bound   = 0x11111111;
	ivm->get().upper_bound   = 0x99999999;
	ivm->get().SF            = 0x3;
	obj19->get().subObjs.push_back(ivm);

	// GNS_UNI_SUBOBJ11: IndvDiskSpace_SubObj
	IndvDiskSpace_SubObj * ids = new IndvDiskSpace_SubObj();
	assert(ids);
	ids->get().lower_bound   = 0x11111111;
	ids->get().upper_bound   = 0x99999999;
	ids->get().SF            = 0x3;
	obj19->get().subObjs.push_back(ids);

	// GNS_UNI_SUBOBJ12: TotCpuTime_SubObj
	TotCpuTime_SubObj * tct = new TotCpuTime_SubObj();
	assert(tct);
	tct->get().lower_bound   = 0x11111111;
	tct->get().upper_bound   = 0x99999999;
	tct->get().SF            = 0x3;
	obj19->get().subObjs.push_back(tct);

	// GNS_UNI_SUBOBJ13: TotCpuCount_SubObj
	TotCpuCount_SubObj * tcc = new TotCpuCount_SubObj();
	assert(tcc);
	tcc->get().lower_bound   = 0x11111111;
	tcc->get().upper_bound   = 0x99999999;
	tcc->get().SF            = 0x3;
	obj19->get().subObjs.push_back(tcc);

	// GNS_UNI_SUBOBJ14: TotPhyMem_SubObj
	TotPhyMem_SubObj * tpm = new TotPhyMem_SubObj();
	assert(tpm);
	tpm->get().lower_bound   = 0x11111111;
	tpm->get().upper_bound   = 0x99999999;
	tpm->get().SF            = 0x3;
	obj19->get().subObjs.push_back(tpm);

	// GNS_UNI_SUBOBJ15: TotVirMem_SubObj
	TotVirMem_SubObj * tvm = new TotVirMem_SubObj();
	assert(tvm);
	tvm->get().lower_bound   = 0x11111111;
	tvm->get().upper_bound   = 0x99999999;
	tvm->get().SF            = 0x3;
	obj19->get().subObjs.push_back(tvm);

	// GNS_UNI_SUBOBJ16: TotDiskSpace_SubObj
	TotDiskSpace_SubObj * tds = new TotDiskSpace_SubObj();
	assert(tds);
	tds->get().lower_bound   = 0x11111111;
	tds->get().upper_bound   = 0x99999999;
	tds->get().SF            = 0x3;
	obj19->get().subObjs.push_back(tds);

	// GNS_UNI_SUBOBJ17:TotResCount_SubObj
	TotResCount_SubObj * trc = new TotResCount_SubObj();
	assert(trc);
	trc->get().lower_bound   = 0x11111111;
	trc->get().upper_bound   = 0x99999999;
	trc->get().SF            = 0x3;
	obj19->get().subObjs.push_back(trc);

	// GNS_UNI_SUBOBJ18:DataStaging_SubObj
	DataStaging_SubObj * ds = new DataStaging_SubObj();
	assert(ds);
	ds->get().FSname                = 0x12;
	ds->get().creation_flag         = 0x6;
	ds->get().delete_on_termination = 0x1;
	ds->get().file_name             = new char[sizeof("phosphorus-g2mpls")];
	memcpy(ds->get().file_name,
	       (void*)"phosphorus-g2mpls",
	       sizeof("phosphorus-g2mpls"));
	ds->get().file_name_len         = strlen(ds->get().file_name);
	ds->get().source                = new char[sizeof("pisa")];
	memcpy(ds->get().source, (void*)"pisa", sizeof("pisa"));
	ds->get().source_len            = strlen(ds->get().source);
	ds->get().target                = new char[sizeof("poznan")];
	memcpy(ds->get().target, (void*)"poznan", sizeof("poznan"));
	ds->get().target_len            = strlen(ds->get().target);
	obj19->get().subObjs.push_back(ds);

	path->gns_uni_eg_ = obj19;
#endif

#ifdef CALL_OPS
	// OBJ 20
	CallOptions_Object * obj20 = new CallOptions_Object();
	assert(obj20);
	obj20->get().call_ops_flags = 0x3;
	path->call_ops_ = obj20;
#endif

#ifdef CALL_ID
#ifdef CALL_ID_OPSPEC
	// OBJ 21
	CallIdOpSpec_Object * obj21 = new CallIdOpSpec_Object();
	assert(obj21);
	obj21->get().addr_type = 0x1;
	obj21->get().src_lsr_addr = new char[sizeof("beef")];
	memcpy(obj21->get().src_lsr_addr, (void *)"beef", sizeof("beef"));
	obj21->get().src_lsr_addr_len  = strlen(obj21->get().src_lsr_addr);
	obj21->get().local_id = 0x1122334455667788ull;
	path->call_id_ = obj21;
#else
#ifdef CALL_ID_GBL
	// OBJ 21
	CallIdGlobal_Object * obj22 = new CallIdGlobal_Object();
	assert(obj22);
	obj22->get().addr_type = 0x7f;
	obj22->get().itu_country_code = 0x123;
	obj22->get().itu_carrier_code = 0x0000112233445566ull;
	obj22->get().unique_access_point = 0x0000665544332211ull;
	obj22->get().src_lsr_addr = new char[sizeof("69.69.69.1")];
	memcpy(obj22->get().src_lsr_addr,
	       (void *)"69.69.69.1",
	       sizeof("69.69.69.1"));
	obj22->get().src_lsr_addr_len = strlen(obj22->get().src_lsr_addr);
	obj22->get().local_id= 0x1122334455667788ull;
	path->call_id_ = obj22;
#endif
#endif
#endif

#ifdef NOTIFY_REQ
	// IPV4NotifyRequest_Object
	IPV4NotifyRequest_Object* obj23 = new IPV4NotifyRequest_Object();
	assert(obj23);
	inet_aton("254.253.252.251", &(obj23->get().ipv4_addr));
	path->notify_request_ = obj23;
#endif

#ifdef MESSAGE_ID
	MessageId_Object * obj24 = new MessageId_Object();
	assert(obj24);
	obj24->get().flags = 0x33;
	obj24->get().epoch = 0x112233;
	obj24->get().message_id = 0xBEEFFACE;
	path->message_id_ = obj24;
#endif

#ifdef MESSAGE_ID_ACK
	MessageIdAck_Object * obj25 = new MessageIdAck_Object();
	assert(obj25);
	obj25->get().flags = 0x33;
	obj25->get().epoch = 0x112233;
	obj25->get().message_id = 0xBEEFFACE;
	path->message_id_ack_nack_ = obj25;
#else
#ifdef MESSAGE_ID_NACK
	MessageIdNack_Object* obj26 = new MessageIdNack_Object();
	assert(obj26);
	obj26->get().flags = 0x33;
	obj26->get().epoch = 0x112233;
	obj26->get().message_id = 0xBEEFFACE;
	path->message_id_ack_nack_ = obj26;
#endif
#endif

#ifdef POLICY_DATA
	IPv4Scope_Object * scp = new IPv4Scope_Object();
	assert(scp);
	ipv4_t * ipv4 = new ipv4_t();
	inet_aton("1.2.3.4", ipv4);
	scp->get().subObjs.push_back(ipv4);
	//scp->get().subObjs.push_back(ipv4);
	//scp->get().subObjs.push_back(ipv4);
	Policy_Object * obj27 = new Policy_Object();
	assert(obj27);
	obj27->get().scope         = scp;
	obj27->get().orig_rsvp_hop = obj9->clone();
	obj27->get().dest_rsvp_hop = obj9->clone();
	obj27->get().time_values   = obj8->clone();
	obj27->get().integrity     = 0;
	char * data = new char[sizeof("cane")];
	memcpy(data, (void *)"cane", sizeof("cane"));
	PolicyInfo * pInfo = new PolicyInfo(1, strlen(data), data);
	delete [] data;
	assert(pInfo);
	char * data2 = new char[sizeof("dimerda")];
	memcpy(data2, (void *)"dimerda", sizeof("dimerda"));
	PolicyInfo * pInfo2 = new PolicyInfo(2, strlen(data2), data2);
	delete [] data2;
	assert(pInfo2);
	obj27->get().policy_elems.push_back(pInfo);
	obj27->get().policy_elems.push_back(pInfo2);
	path->policy_data_ = obj27;
#endif

#ifdef RESV_CONFIRM
	ResvConfirmIPv4_Object * obj28 = new ResvConfirmIPv4_Object();
	assert(obj28);
	inet_aton("192.168.1.1", &(obj28->get().ipv4_addr));
	resv->resv_confirm_ = obj28;
#endif

#ifdef STYLE
	Style_Object * obj29 = new Style_Object();
	assert(obj29);
	obj29->get().flags = 0x12;
	obj29->get().sharing = G2RSVPTE_SHARING_CTRL_DISTINCT_RESV;
	obj29->get().sender_selection = G2RSVPTE_SENDER_SELECTION_CTRL_EXPLICIT;
	resv->style_ = obj29;
#endif

#ifdef FILTER_SPEC
	IPV4FilterSpec_Object * obj31 = new IPV4FilterSpec_Object();
	assert(obj31);
	inet_aton("192.168.1.111", &(obj31->get().ipv4_addr));
	obj31->get().src_port = 1212;
#endif

#ifdef FF_FLOW_DESC
	FF_FlowDesc_t * ff = new FF_FlowDesc_t();
	assert(ff);
	ff_flow_item_desc_t * item = new ff_flow_item_desc_t();
	assert(item);
	item->label       = l->clone();
	item->rro         = obj11->clone();
	item->filter_spec = obj31;
	item->flow_spec   = 0;
	ff->filters.push_back(item);

	//ff_flow_item_desc_t * item2 = new ff_flow_item_desc_t();
	//assert(item2);
	//item2->label       = 0;
	//item2->rro         = obj11;
	//item2->filter_spec = 0;
	//item2->flow_spec   = 0;
	//ff->filters.push_back(item2);

	Flow_Desc_List * obj30 = new Flow_Desc_List(ff);
	assert(obj30);
	resv->flow_desc_ = obj30;
	//printf("-----------------LEN %d\n\n", obj30->length());
#endif


	MemoryStream ms;
	SCNSocket    scn;
	Message *    msg;
	Interface *  intf = 0;
	G2RSVPTE *   p = 0;

	msg = 0;

	try {

		g2mpls_addr_t local;
		local.type    = IPv4;
		local.preflen = 32;
		inet_aton("192.168.1.20", &local.value.ipv4);
		intf = new Interface(p, local,        local,
				     0, (opstate_t)0, (admstate_t)0);

#ifdef TEST_PATH
		G2RSVPTE_WRN("---------- SEND PATH ----------");
		ms << *path;   // equ to ms << *(static_cast<Message *>(path));
		std::cout << ms;
		std::cout << *static_cast<Message *>(path);
		G2RSVPTE_DBG("Sending path msg!!!");
		scn.write(path, intf, 0);

//		G2RSVPTE_WRN("---------- RECEIVE PATH ----------");
//		std::cout << ms;
//
//		ms >> msg;
//		std::cout << *msg;
#endif


		G2RSVPTE_WRN("---------- INITIALIZE RESV ----------");
		// ....

		MemoryStream ms_resv;
#ifdef TEST_RESV
		G2RSVPTE_WRN("---------- SEND RESV ----------");
		ms_resv << *resv;
		std::cout << ms_resv;
		std::cout << *static_cast<Message *>(resv);

		G2RSVPTE_DBG("Sending resv msg!!!");
		scn.write(resv, intf, 0);

//		G2RSVPTE_WRN("---------- RECEIVE RESV ----------");
//		std::cout << ms_resv;
//
//		ms_resv >> msg;
//		std::cout << *msg;
#endif



#ifdef TEST_PATH_ERR
		G2RSVPTE_WRN("---------- INITIALIZE PATH_ERR ----------");
		pathErr->session_ = obj->clone();
		pathErr->call_ops_ = obj20->clone();
#ifdef CALL_ID_OPSPEC
		pathErr->call_id_ = obj21->clone();
#else
#ifdef CALL_ID_GBL
		pathErr->call_id_ = obj22->clone();
#endif
#endif
		pathErr->gns_call_ext_ = obj16->clone();
		pathErr->integrity_ = obj17->clone();

		pathErr->policy_data_ = obj27->clone();
		pathErr->sender_template_ = obj12->clone();
		pathErr->sender_tspec_ = obj13->clone();
		pathErr->rro_ = obj11->clone();
		pathErr->suggested_label_ = obj3->clone();
		pathErr->recovery_label_ = obj5->clone();
		pathErr->upstream_label_ = obj4->clone();


		MemoryStream ms_path_err;
		G2RSVPTE_WRN("---------- SEND PATH_ERR ----------");
		ms_path_err << *pathErr;
		std::cout << ms_path_err;
		std::cout << *static_cast<Message *>(pathErr);

		G2RSVPTE_DBG("Sending resv msg!!!");
		//SCNSocket scn;
		scn << ms_path_err;

		G2RSVPTE_WRN("---------- RECEIVE PATH_ERR ----------");
		std::cout << ms_path_err;

		ms_path_err >> msg;
		std::cout << *msg;
#endif


#ifdef TEST_RESV_ERR
		G2RSVPTE_WRN("---------- INITIALIZE RESV_ERR ----------");
		resvErr->session_ = obj->clone();
		resvErr->integrity_ = obj17->clone();
		resvErr->policy_data_ = obj27->clone();

		MemoryStream ms_resv_err;
		G2RSVPTE_WRN("---------- SEND RESV_ERR ----------");
		ms_resv_err << *resvErr;
		std::cout << ms_resv_err;
		std::cout << *static_cast<Message *>(resvErr);

		G2RSVPTE_DBG("Sending resv msg!!!");
		//SCNSocket scn;
		scn << ms_resv_err;

		G2RSVPTE_WRN("---------- RECEIVE RESV_ERR ----------");
		std::cout << ms_resv_err;

		ms_resv_err >> msg;
		std::cout << *msg;
#endif

#ifdef TEST_PATH_TEAR
		G2RSVPTE_WRN("---------- INITIALIZE PATH_TEAR ----------");
		pathTear->session_ = obj->clone();
		pathTear->call_ops_ = obj20->clone();
#ifdef CALL_ID_OPSPEC
		pathTear->call_id_ = obj21->clone();
#else
#ifdef CALL_ID_GBL
		pathTear->call_id_ = obj22->clone();
#endif
#endif
		pathTear->gns_call_ext_ = obj16->clone();
		pathTear->integrity_ = obj17->clone();

		pathTear->sender_template_ = obj12->clone();
		pathTear->sender_tspec_ = obj13->clone();
		pathTear->rro_ = obj11->clone();
		pathTear->suggested_label_ = obj3->clone();
		pathTear->recovery_label_ = obj5->clone();
		pathTear->upstream_label_ = obj4->clone();

		MemoryStream ms_path_tear;
		G2RSVPTE_WRN("---------- SEND PATH_TEAR ----------");
		ms_path_tear << *pathTear;
		std::cout << ms_path_tear;
		std::cout << *static_cast<Message *>(pathTear);

		G2RSVPTE_DBG("Sending resv msg!!!");
		//SCNSocket scn;
		scn << ms_path_tear;

		G2RSVPTE_WRN("---------- RECEIVE PATH_TEAR ----------");
		std::cout << ms_path_tear;

		ms_path_tear >> msg;
		std::cout << *msg;
#endif

#ifdef TEST_RESV_TEAR
		G2RSVPTE_WRN("---------- INITIALIZE RESV_TEAR ----------");
		resvTear->session_ = obj->clone();
		resvTear->integrity_ = obj17->clone();
		resvTear->scope_ = scp->clone();

		MemoryStream ms_resv_tear;
		G2RSVPTE_WRN("---------- SEND RESV_TEAR ----------");
		ms_resv_tear << *resvTear;
		std::cout << ms_resv_tear;
		std::cout << *static_cast<Message *>(resvTear);

		G2RSVPTE_DBG("Sending resv msg!!!");
		//SCNSocket scn;
		scn << ms_resv_tear;

		G2RSVPTE_WRN("---------- RECEIVE RESV_TEAR ----------");
		std::cout << ms_resv_tear;

		ms_resv_tear >> msg;
		std::cout << *msg;
#endif

#ifdef TEST_RESV_CONF
		G2RSVPTE_WRN("---------- INITIALIZE RESV_CONF ----------");
		resvConf->session_ = obj->clone();
		resvConf->integrity_ = obj17->clone();
		resvConf->style_ = obj29->clone();
		resvConf->resv_confirm_ = obj28->clone();

		MemoryStream ms_resv_conf;
		G2RSVPTE_WRN("---------- SEND RESV_CONF ----------");
		ms_resv_conf << *resvConf;
		std::cout << ms_resv_conf;
		std::cout << *static_cast<Message *>(resvConf);

		G2RSVPTE_DBG("Sending resv msg!!!");
		//SCNSocket scn;
		scn << ms_resv_conf;

		G2RSVPTE_WRN("---------- RECEIVE RESV_CONF ----------");
		std::cout << ms_resv_conf;

		ms_resv_conf >> msg;
		std::cout << *msg;
#endif

		G2RSVPTE_WRN("---------- TEST END ----------");

		//std::ostringstream str;
		//str << *sObj; //*obj10;
		//printf("\n\n\nERO subObj: %s", str.str().c_str());

	} catch (std::string e) {
		G2RSVPTE_ERR("Catch exception: %s", e.c_str());
	} catch (...) {
		G2RSVPTE_ERR("Catch UNKNOWN exception!");
	}

	while (!terminate_execution) {
		if (thread_fetch(RSVP_MASTER.getMaster(), &thread)) {
			thread_call(&thread);
		}
	}

	RSVP_Master::destroy();

	//if (msg) delete msg;
	delete path;
	delete resv;
	delete pathErr;
	delete resvErr;
	delete pathTear;
	delete resvTear;
	delete resvConf;
	delete l;

	return 0;
}
