
#include "soapStub.h"

//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#include "BESFactoryBindingServer.hh"
#include "g2mpls_types.h"


#define BES_CA  1
#define BES_GAS 2
#define BES_TA   3
#define BES_GAD  4
#define BES_GFAD 5


int
BES_thread()
{
    BESFactoryBindingService bes;
    
    GUNIGW_DBG("Starting BES Service in Port 8080");

    if (bes.run(8080))
    {
        GUNIGW_ERR("Error starting BES process");
        printf("ERROR\n");
        bes.soap_stream_fault(std::cerr);
        exit(-1);
    }
    
    GUNIGW_DBG("Exiting BES_thread...");
    
    return BES_OK;

}

int
init_BESFactory()
{
    pthread_t tid;
                
    if (!pthread_create(&tid, NULL, (void*(*)(void*))BES_thread, NULL))
    {
        GUNIGW_ERR("Error creating BES thread");
        return BES_ERR;
    }
    
    return BES_OK;
}

static int
call_map(struct soap soap, char *CC, call_data_t *data, int call_type)
{
    switch (call_type)
    {
        case BES_CA:
            net_res_spec_t      nres;
	    float fl_bw;
	    char *temp;
	    temp = new char[255];
                                
            memset(&data->cdata, 0, sizeof(call_info_t));
            if (((bes__CreateActivityType *)CC)->bes__ActivityDocument)
            {
                if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition)
                {
                    memset(&data->cid, 0, sizeof(call_ident_t));
                    data->cid.type                       = CALLID_OPSPEC;
                    data->cid.local_id                   = atoi(JOB_ID((bes__CreateActivityType *)CC));
		    GUNIGW_DBG("Job_ID: %s",((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->id);
                    data->cid.src_addr.type              = IPv4;
                    data->cid.src_addr.preflen           = 32;
                    data->cid.src_addr.value.ipv4.s_addr = htonl(GUNIGW_MASTER.routerId());
		    data->cdata.call_type    		 = CALL_TYPE_aUGWzUGW;
                }
                if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription)
                {
                    if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__JobIdentification)
                    {
                        // Job Name
                        BITMASK_BITSET(data->cdata.mask_, job_name);
                        data->cdata.job_name = new std::string(JOB_NAME((bes__CreateActivityType *)CC));            
                        // Job Project
                        BITMASK_BITSET(data->cdata.mask_, job_project);
                        data->cdata.job_project = new std::string(JOB_PROJECT((bes__CreateActivityType *)CC));
                    }
                    if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Application)
                    {
			GUNIGW_DBG("ApplicationName: %s",((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Application->jsdl__ApplicationName);
                    }

                    if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes)
                    {
                        // iTNA res data
                        memset(&nres, 0, sizeof(nres));
                        BITMASK_BITSET(data->cdata.mask_, iTNA_res);
                        BITMASK_BITSET(data->cdata.iTNA_res.mask_, net);
                        nres.tna.type       = IPv4;
                        nres.tna.preflen    = 32;
                        inet_aton(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->IngresTNA, &(nres.tna.value.ipv4));
                        GUNIGW_DBG("IngressTNA: %s", ((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->IngresTNA);
                        BITMASK_BITSET(nres.mask_, tna);
                        memcpy(&(data->cdata.iTNA_res.net), &nres, sizeof(nres));

                        if(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->AddressingType == net__NetworkAddressingType__unicast)
			{
                            GUNIGW_DBG("Unicast call");
			    // eTNA res data
                            memset(&nres, 0, sizeof(nres));
                            BITMASK_BITSET(data->cdata.mask_, eTNA_res);
                            // eTNA res data (net)
                            BITMASK_BITSET(data->cdata.eTNA_res.mask_, net);
                            nres.tna.type       = IPv4;
                            nres.tna.preflen    = 32;
                            inet_aton(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->EgresTNA, &(nres.tna.value.ipv4));
                            GUNIGW_DBG("EgressTNA: %s", ((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->EgresTNA);
                            BITMASK_BITSET(nres.mask_, tna);
                            memcpy(&(data->cdata.eTNA_res.net), &nres, sizeof(nres));
			}
			else
			{
                            GUNIGW_DBG("Anycast call");
			}
               
                        // Bandwidth
			GUNIGW_DBG("Bandwidth: %d", ((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->Bandwidth);
                        //data->ldata.bw      = ((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->Bandwidth;i
		    }
		    if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Resources)
		    {
            		grid_res_spec_t     gres;
                        memset(&gres, 0, sizeof(gres));
		    	if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Resources->jsdl__TotalCPUCount)
		    	{
                            BITMASK_BITSET(data->cdata.mask_, eTNA_res);
		    	    BITMASK_BITSET(data->cdata.eTNA_res.mask_, grid);
			    gres.tot_cpu_count.lb_included = 1;
			    gres.tot_cpu_count.ub_included = 1;
		   	    gres.tot_cpu_count.lower_bound = atoi(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Resources->jsdl__TotalCPUCount->LowerBoundedRange);
		   	    gres.tot_cpu_count.upper_bound = atoi(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Resources->jsdl__TotalCPUCount->UpperBoundedRange);
			    GUNIGW_DBG("TotalCPUCount: LB=%d", atoi(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Resources->jsdl__TotalCPUCount->LowerBoundedRange));
			    GUNIGW_DBG("TotalCPUCount: UB=%d", atoi(((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->jsdl__Resources->jsdl__TotalCPUCount->UpperBoundedRange));
			    BITMASK_BITSET(gres.mask_, tot_cpu_count);
			    memcpy(&(data->cdata.eTNA_res.grid), &gres, sizeof(gres));
		    	}
		    }
                    
                }
                //if (((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->ar__ARAttributes->ar__StartTime)
                //{
                    // Advance Reservations
                    //BITMASK_BITSET(data->cdata.mask_, times);
                    //data->cdata.times.start_time = 0;
                    //data->cdata.times.end_time   = 0;
                //}
            }
           // recovery infor data
            memset(&data->rdata, 0, sizeof(recovery_info_t));

            // lsp info data
            memset(&data->ldata, 0, sizeof(lsp_info_t));
            BITMASK_BITSET(data->ldata.mask_, sw_cap);
            data->ldata.sw_cap          = SWCAP_FSC;
            BITMASK_BITSET(data->ldata.mask_, enc_type);
            data->ldata.enc_type        = ENCT_FIBER;
            BITMASK_BITSET(data->ldata.mask_, gpid);
            data->ldata.gpid            = GPID_ETHERNET;
            BITMASK_BITSET(data->ldata.mask_, link_prot_mask);
            data->ldata.link_prot_mask  = PROTTYPE_NONE;
            BITMASK_BITSET(data->ldata.mask_, crankback);
            data->ldata.crankback       = CB_NONE;
            BITMASK_BITSET(data->ldata.mask_, type);
            data->ldata.type            = LSP_TYPE_SC;
            BITMASK_BITSET(data->ldata.mask_, role);
            data->ldata.role            = LSP_ROLE_PRIMARY;
            BITMASK_BITSET(data->ldata.mask_, action);
            data->ldata.action          = RESOURCE_XCONNECT;
            BITMASK_BITSET(data->ldata.mask_, rro_mode);
            data->ldata.rro_mode        = LSP_RRO_OFF;
                        
	    BITMASK_BITSET(data->ldata.mask_, bw);
            data->ldata.bw      =  ((bes__CreateActivityType *)CC)->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->Bandwidth;
	    GUNIGW_DBG("Bandwidth2: %d", data->ldata.bw);

            return BES_OK;
            break;
        case BES_GAS:
            // callGetDetails
            memset(&data->cid, 0, sizeof(call_ident_t));
            data->cid.type                       = CALLID_OPSPEC;
            temp = strtok(((bes__TerminateActivitiesType *)CC)->ActivityIdentifier[0]->ReferenceParameters->__any[0], ">");
            temp = strtok(NULL, "<");
            data->cid.local_id                   = atoi(temp);
            data->cid.src_addr.type              = IPv4;
            data->cid.src_addr.preflen           = 32;
            data->cid.src_addr.value.ipv4.s_addr = htonl(GUNIGW_MASTER.routerId());
	    data->cdata.call_type    		 = CALL_TYPE_aUGWzUGW;
            break;
        case BES_TA:
            // callDestroy
            memset(&data->cid, 0, sizeof(call_ident_t));
            data->cid.type                       = CALLID_OPSPEC;
	    temp = strtok(((bes__TerminateActivitiesType *)CC)->ActivityIdentifier[0]->ReferenceParameters->__any[0], ">");
	    temp = strtok(NULL, "<");
	    data->cid.local_id                   = atoi(temp);
            data->cid.src_addr.type              = IPv4;
            data->cid.src_addr.preflen           = 32;
            data->cid.src_addr.value.ipv4.s_addr = htonl(GUNIGW_MASTER.routerId());
	    data->cdata.call_type    		 = CALL_TYPE_aUGWzUGW;
            break;
        case BES_GAD:
            // callGetDetails
            break;
        case BES_GFAD:
            break;
            
    }
    
    return BES_OK;
}

int
BESFactoryBindingService::CreateActivity(bes__CreateActivityType *CA,
        bes__CreateActivityResponseType *CAR)
{ 
    struct soap soap;
    call_data_t     *data = new call_data_t;
    std::string     err;
    struct timeval  timeout;
    struct timeval  time_tmp;

    GUNIGW_DBG("WSAG Call: CreateActivity");
    
    if(header)
        GUNIGW_DBG("Assertion: %s", header->Assertion);

//GUNIGW_DBG("AddressingType: %s",CA->bes__ActivityDocument->jsdl__JobDefinition->jsdl__JobDescription->net__NetworkAttributes->net__AddressingType);

//GUNIGW_DBG("StartTime: %s",CA->bes__ActivityDocument->jsdl__JobDefinition->ar__ARAttributes->ar__StartTime);
//data->cdata.times.start_time = 0;
//data->cdata.times.end_time   = 0;
            
    if (!call_map(soap, (char *)CA, data, BES_CA))
    {
        GUNIGW_ERR("Error mapping call data");
        return SOAP_OK;
    }
    
    GUNIGW_DBG("Going to create a GUNIGW CALL with localID %d", (int)data->cid.local_id);

#if HAVE_OMNIORB
    CallController::Mgmt_var         nccMgmtProxy;

    nccMgmtProxy = GUNIGW_MASTER.getNccProxy();
 
    if (CORBA::is_nil(nccMgmtProxy)){
        GUNIGW_ERR("Cannot get Corba client object "
            "previously set up");
	assert(0);
    }

    g2mplsTypes::callIdent_var id;

    try {
	id << data->cid;
    } catch (std::out_of_range & e) {
	err  = "Exception occurred during callId translation (";
	err += e.what();
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "Unknown exception during callId translation";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }

    g2mplsTypes::callParams_var callInfo;

    try {
	callInfo << data->cdata;
    } catch (std::out_of_range & e) {
        err  = "Exception occurred during callInfo translation (";
	err += e.what();
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "Unknown exception during callInfo translation";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }

    g2mplsTypes::recoveryParams_var recoveryInfo;

    try {
	recoveryInfo << data->rdata;
    } catch (std::out_of_range & e) {
	err  = "Exception occurred during recoveryInfo translation (";
	err += e.what();
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "Unknown exception during recoveryInfo translation";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }

    g2mplsTypes::lspParams_var lspInfo;

    try {
    	lspInfo << data->ldata;
    } catch (std::out_of_range & e) {
        err  = "Exception occurred during lspInfo translation (";
	err += e.what();
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "Unknown exception during lspInfo translation";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }

    g2mplsTypes::aaaParams_var aaaInfo;
    aaaInfo = new g2mplsTypes::aaaParams;
    if(header)
    {
	aaaInfo->authzToken = CORBA::string_dup(header->Assertion);
    }
    else
	aaaInfo->authzToken = "";

    g2mplsTypes::actorInfo_var actor;
    actor = new g2mplsTypes::actorInfo;

    actor->issuer       = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
    actor->forceCommand = false;

    try {
	nccMgmtProxy->callCreate(id          ,
				 callInfo    ,
				 recoveryInfo,
				 lspInfo     ,
				 aaaInfo     ,
				 actor       );
    } catch (Types::InternalProblems & e) {
        err += "callCreate through CORBA Mgmt ";
	err += "interface failed for internal problems (reason= ";
	err += e.what;
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (Types::CannotFetch & e) {
        err += "callCreate through CORBA Mgmt ";
        err += "interface failed for fetching problems (reason= ";
        err += e.what;
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "callCreate through CORBA Mgmt ";
        err += "interface failed for unknown reason";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }
    GUNIGW_DBG("Sent callCreate through CORBA Mgmt interface");

    GUNIGW_DBG("Created call with identity [%s]",
              ((id->idType == g2mplsTypes::CALLIDTYPE_GLOBUNIQ) ?
               "GLOBALLY UNIQUE" : "OPERATOR SPECIFIC"));
    GUNIGW_DBG("\t - srcAddr             : %s",
              addr_ntoa(data->cid.src_addr));
    GUNIGW_DBG("\t - localId             : %s",
              key2string(data->cid.local_id).c_str());

#if NCC_GLOBAL_CALLID
    if (id->idType == g2mplsTypes::CALLIDTYPE_GLOBUNIQ) {
            GUNIGW_DBG("\t - Globally unique segments");
            GUNIGW_DBG("\t     - international : 0x%02x%02x%02x",
                      id->segs.intlSeg[0],
                      id->segs.intlSeg[1],
                      id->segs.intlSeg[2]);
            GUNIGW_DBG("\t     - national      : 0x%08x%08x%08x",
                      id->segs.natlSeg[0],
                      id->segs.natlSeg[1],
                      id->segs.natlSeg[2]);
    }
#endif

    if (BITMASK_BITTEST(data->cdata.mask_, iTNA_res))
    {
        g2mplsTypes::resourcePosition  pos;
        pos = g2mplsTypes::RESOURCEPOSITION_INGRESS;
        if (BITMASK_BITTEST(data->cdata.iTNA_res.mask_, net))
        {
            g2mplsTypes::tnaResource_var  tnaRes;
            try {
                tnaRes << data->cdata.iTNA_res.net;
            } catch (std::out_of_range & e) {
                err  = "Exception occurred during ";
                err += "translation of iTNA_res ";
                err += "network params(";
                err += e.what();
                err += ")";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            }
            try {
                nccMgmtProxy->callSetTna(id    ,
                                         pos   ,
                                         tnaRes,
                                         actor );
            } catch (Types::InternalProblems & e) {
                err += "callSetTna through CORBA Mgmt ";
                err += "interface failed for internal ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            } catch (Types::CannotFetch & e) {
                    err += "callSetTna through CORBA Mgmt ";
                err += "interface failed for fetching ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            } catch (...) {
                err += "callSetTna through CORBA Mgmt ";
                err += "interface failed for unknown reason";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            }

            GUNIGW_DBG("Sent callSetTna for iTNA "
                      "through CORBA Mgmt interface");
        }

        //gunigw_lock(true, 0);
        //gettimeofday(&time_tmp, NULL);

        
        if (BITMASK_BITTEST(data->cdata.iTNA_res.mask_, grid)) {
            g2mplsTypes::gridParams_var   gnsTnaRes;
            
            try {
                gnsTnaRes << data->cdata.iTNA_res.grid;
            } catch (std::out_of_range & e) {
                err  = "Exception occurred during ";
                err += "translation of iTNA_res ";
                err += "grid params(";
                err += e.what();
                err += ")";
        	return SOAP_OK;
            }
            try {
                nccMgmtProxy->callSetGnsTna(id       ,
                                            pos      ,
                                            gnsTnaRes,
                                            actor    );
            } catch (Types::InternalProblems & e) {
                err += "callSetGnsTna through CORBA Mgmt ";
                err += "interface failed for internal ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	return SOAP_OK;
            } catch (Types::CannotFetch & e) {
                err += "callSetGnsTna through CORBA Mgmt ";
                err += "interface failed for fetching ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	return SOAP_OK;
            } catch (...) {
                err += "callSetGnsTna through CORBA Mgmt ";
                err += "interface failed for unknown reason";
        	return SOAP_OK;
            }

            GUNIGW_DBG("Sent callSeGnstTna for iTNA "
                       "through CORBA Mgmt interface");
        }
    }

    if (BITMASK_BITTEST(data->cdata.mask_, eTNA_res))
    {
        g2mplsTypes::resourcePosition  pos;

        pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;

        if (BITMASK_BITTEST(data->cdata.eTNA_res.mask_, net)) {
            g2mplsTypes::tnaResource_var  tnaRes;

            try {
                tnaRes << data->cdata.eTNA_res.net;
            } catch (std::out_of_range & e) {
                err  = "Exception occurred during ";
                err += "translation of eTNA_res ";
                err += "network params(";
                err += e.what();
                err += ")";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            }

            try {
                nccMgmtProxy->callSetTna(id    ,
                                         pos   ,
                                         tnaRes,
                                         actor );
            } catch (Types::InternalProblems & e) {
                err += "callSetTna through CORBA Mgmt ";
                err += "interface failed for internal ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            } catch (Types::CannotFetch & e) {
                err += "callSetTna through CORBA Mgmt ";
                err += "interface failed for fetching ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            } catch (...) {
                err += "callSetTna through CORBA Mgmt ";
                err += "interface failed for unknown reason";
        	GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
            }

            GUNIGW_DBG("Sent callSetTna for eTNA "
                       "through CORBA Mgmt interface");
        }

        if (BITMASK_BITTEST(data->cdata.eTNA_res.mask_, grid)) {
            g2mplsTypes::gridParams_var   gnsTnaRes;

            try {
                gnsTnaRes << data->cdata.eTNA_res.grid;
            } catch (std::out_of_range & e) {
                err  = "Exception occurred during ";
                err += "translation of iTNA_res ";
                err += "grid params(";
                err += e.what();
                err += ")";
        	return SOAP_OK;
            }

            try {
                nccMgmtProxy->callSetGnsTna(id       ,
                                            pos      ,
                                            gnsTnaRes,
                                            actor    );
            } catch (Types::InternalProblems & e) {
                err += "callSetGnsTna through CORBA Mgmt ";
                err += "interface failed for internal ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	return SOAP_OK;
            } catch (Types::CannotFetch & e) {
                err += "callSetGnsTna through CORBA Mgmt ";
                err += "interface failed for fetching ";
                err += "problems (reason= ";
                err += e.what;
                err += ")";
        	return SOAP_OK;
            } catch (...) {
                err += "callSetGnsTna through CORBA Mgmt ";
                err += "interface failed for unknown reason";
        	return SOAP_OK;
            }
            GUNIGW_DBG("Sent callSetGnsTna for eTNA "
                       "through CORBA Mgmt interface");
        }
    
        try {
                nccMgmtProxy->callSetUp(id, actor);
        } catch (Types::InternalProblems & e) {
                err += "callSetUp through CORBA Mgmt ";
                err += "interface failed for internal problems ";
                err += "(reason= ";
                err += e.what;
                err += ")";
                GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
        } catch (Types::CannotFetch & e) {
                err += "callSetUp through CORBA Mgmt ";
                err += "interface failed for fetching problems ";
                err += "(reason= ";
                err += e.what;
                err += ")";
                GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
        } catch (...) {
                err += "callSetUp through CORBA Mgmt ";
                err += "interface failed for unknown reason";
                GUNIGW_ERR("%s", err.c_str());
        	return SOAP_OK;
        }

        GUNIGW_DBG("Sent callSetUp through "
                   "CORBA Mgmt interface");
    }

    // EastWest Interface
    // Wait for Event
    //gettimeofday(&timeout, NULL);
    //while(is_gunigw_locked(0) && (timeout.tv_sec - time_tmp.tv_sec) < 20)
    //{
    //	gettimeofday(&timeout, NULL);
    //}

    //if (!is_gunigw_locked(0))
    //{
        // Setup Indication received
    //}
    //else
    //{
    //	GUNIGW_DBG("Timeout! (SetupIndication expected)", err.c_str());
    //  gunigw_lock(false, 0);
    //}
#endif // HAVE_OMNIORB

    
    
    CAR->ActivityIdentifier = new wsa__EndpointReferenceType;
    CAR->ActivityIdentifier->Address = new wsa__AttributedURIType;
    CAR->ActivityIdentifier->Address->__item = new char[255];
    //sprintf(CAR->ActivityIdentifier->Address->__item, "http://%s:8080", hostsrv_ipv4_addr());
    char hostname[255];
    gethostname (hostname, sizeof(hostname));
    sprintf(CAR->ActivityIdentifier->Address->__item, "http://%s:8080", hostname);
    CAR->ActivityIdentifier->ReferenceParameters  = new wsa__ReferenceParametersType;
    CAR->ActivityIdentifier->ReferenceParameters->__size = 1;
    CAR->ActivityIdentifier->ReferenceParameters->__any = new char*;
    CAR->ActivityIdentifier->ReferenceParameters->__any[0] = new char[255];
    sprintf(CAR->ActivityIdentifier->ReferenceParameters->__any[0], 
            "<ph:ID xmlns:ph=\"http://schemas.phosphorus.eu\">%d</ph:ID>", (int)data->cid.local_id);
    
    GUNIGW_DBG("WSAG Call: Returning CreateActivity");
    
    return SOAP_OK;
}

int
BESFactoryBindingService::GetActivityStatuses(bes__GetActivityStatusesType *GAS,
        bes__GetActivityStatusesResponseType *GASR)
{
    //Call IDL
    struct soap soap;
    GUNIGW_DBG("WSAG Call: GetActivityStatuses");
    soap_init(&soap);
    
    call_data_t     *data;
    std::string     err;
    int		    bes_err = 0;
    
    data = new call_data_t;
    
    if (!call_map(soap, (char *)GAS, data, BES_GAS))
    {
        GUNIGW_ERR("Error mapping call data");
        return SOAP_OK;
    }

#if HAVE_OMNIORB
    GUNIGW_DBG("Requesting details for Call with localID %d "
                                "and src LSR %s",
                                (int)data->cid.local_id,
                                addr_ntoa(data->cid.src_addr));

    CallController::Mgmt_var  nccMgmtProxy;

    nccMgmtProxy = GUNIGW_MASTER.getNccProxy();

    if (CORBA::is_nil(nccMgmtProxy))
    {
            GUNIGW_ERR("Cannot get Corba nccMgmtProxy client object "
                      "previously set up");
            assert(0);
    }

    g2mplsTypes::callIdent_var id;
    //	id = new g2mplsTypes::callIdent;

    try {
            id << data->cid;
    } catch (std::out_of_range & e) {
    	err  = "Exception occurred during callId translation (";
	err += e.what();
	err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "Unknown exception during callId translation";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }

    g2mplsTypes::callParams_var                          cinfo;
    g2mplsTypes::recoveryParams_var                      rinfo;
    g2mplsTypes::lspParams_var                           linfo;
    g2mplsTypes::statesBundle_var                        states;
    CallController::Mgmt::recoBundleIdentSeq_var recoBundles;
   
    g2mplsTypes::actorInfo_var actor;
    actor = new g2mplsTypes::actorInfo;

    actor->issuer = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
    actor->forceCommand = true;

    try {
            nccMgmtProxy->callGetDetails(id,
                                         cinfo,
                                         rinfo,
                                         linfo,
                                         actor,
                                         states,
                                         recoBundles);
    } catch (Types::InternalProblems & e) {
            err += "callGetDetails through CORBA Mgmt ";
            err += "interface failed for internal problems ";
            err += "(reason= ";
            err += e.what;
            err += ")";
            GUNIGW_ERR("%s", err.c_str());
            return SOAP_OK;
    } catch (Types::CannotFetch & e) {
            err += "callGetDetails through CORBA Mgmt ";
            err += "interface failed for fetching problems ";
            err += "(reason= ";
            err += e.what;
            err += ")";
            GUNIGW_ERR("%s", err.c_str());
            //return SOAP_OK;
	    bes_err = 1;
    } catch (...) {
            err += "callGetDetails through CORBA Mgmt ";
            err += "interface failed for unknown reason";
            GUNIGW_ERR("%s", err.c_str());
            return SOAP_OK;
    }
    
    GUNIGW_DBG("Sent callGetDetails through "
                   "CORBA Mgmt interface");

    g2mplsTypes::resourcePosition            pos;
    g2mplsTypes::gridParams_var              gnsTna;

    pos = g2mplsTypes::RESOURCEPOSITION_EGRESS;

    try {
            nccMgmtProxy->callGetGnsTna(id,
                                     pos,
                                     gnsTna);
    } catch (Types::InternalProblems & e) {
            err += "callGetTna through CORBA Mgmt ";
            err += "interface failed for internal problems ";
            err += "(reason= ";
            err += e.what;
            err += ")";
            GUNIGW_ERR("%s", err.c_str());
            return SOAP_OK;
    } catch (Types::CannotFetch & e) {
            err += "callGetTna through CORBA Mgmt ";
            err += "interface failed for fetching problems ";
            err += "(reason= ";
            err += e.what;
            err += ")";
            GUNIGW_ERR("%s", err.c_str());
            //return SOAP_OK;
	    if (!bes_err) bes_err = 2;
    } catch (...) {
            err += "callGetTna through CORBA Mgmt ";
            err += "interface failed for unknown reason";
            GUNIGW_ERR("%s", err.c_str());
            return SOAP_OK;
    }

    GUNIGW_DBG("Sent callGetTna through "
                   "CORBA Mgmt interface");


#endif // HAVE_OMNIORB

    GASR->__sizeResponse = 1;
    GASR->Response = new bes__GetActivityStatusResponseType*;
    GASR->Response[0] = new bes__GetActivityStatusResponseType;
    GASR->Response[0]->bes__ActivityStatus = new bes__ActivityStatusType;
    if(bes_err == 1)
    {
        GASR->Response[0]->bes__ActivityStatus->state = bes__ActivityStateEnumeration__Failed;
    }
    else
    {
    	if (states->opState == g2mplsTypes::OPERSTATE_UP)
            GASR->Response[0]->bes__ActivityStatus->state = bes__ActivityStateEnumeration__Running;
	else if (states->opState == g2mplsTypes::OPERSTATE_DOWN)
    	    GASR->Response[0]->bes__ActivityStatus->state = bes__ActivityStateEnumeration__Pending;
	GUNIGW_DBG("operstate: %s", states->opState == g2mplsTypes::OPERSTATE_UP ? "UP" : "DOWN");
    }
    
    if (!bes_err)
    {
    	GASR->Response[0]->ActivityIdentifier = new wsa__EndpointReferenceType;
	grid_res_spec_t tnaG;
	tnaG << gnsTna;
	GUNIGW_DBG("CandHost: %s", inet_ntoa(tnaG.cand_host.value.ipv4));
	//sprintf(GASR->Response[0]->ActivityIdentifier->Address->__item, "http://%s:8080", hostsrv_ipv4_addr());
	//char hostname[255];
	//gethostname (hostname, sizeof(hostname));
	//sprintf(GASR->Response[0]->ActivityIdentifier->Address->__item, "http://%s:8080", hostname);
	GASR->Response[0]->ActivityIdentifier->__size = 1;
	GASR->Response[0]->ActivityIdentifier->__any = new char*;
	GASR->Response[0]->ActivityIdentifier->__any[0] = new char [255];
	sprintf(GASR->Response[0]->ActivityIdentifier->__any[0], "<serverId>%s</serverId>", inet_ntoa(tnaG.cand_host.value.ipv4));
    }
    
    GUNIGW_DBG("WSAG Call: Returning GetActivityStatuses");
    return SOAP_OK;
}

int
BESFactoryBindingService::TerminateActivities(bes__TerminateActivitiesType *TA,
        bes__TerminateActivitiesResponseType *TAR)
{
    //Call IDL
    struct soap soap;
    GUNIGW_DBG("WSAG Call: TerminateActivities");
    soap_init(&soap);

    call_data_t     *data;
    std::string     err;
    uint32_t	    callId;

    data = new call_data_t;
        
    if (!call_map(soap, (char *)TA, data, BES_TA))
    {
        GUNIGW_ERR("Error mapping call data");
        return SOAP_OK;
    }
    
    callId = data->cid.local_id;
    GUNIGW_DBG("Going to destroy a GUNIGW CALL with localID %d", (int)data->cid.local_id);
 
#if HAVE_OMNIORB
    
    CallController::Mgmt_var nccMgmtProxy;
    
    nccMgmtProxy = GUNIGW_MASTER.getNccProxy();

    if (CORBA::is_nil(nccMgmtProxy)) {
            GUNIGW_ERR("Cannot get Corba nccMgmtProxy client "
                      "object previously set up");
            assert(0);
    }

    g2mplsTypes::callIdent_var id;
    // id = new g2mplsTypes::callIdent;
    try {
        id << data->cid;
    } catch (std::out_of_range & e) {
        err  = "Exception occurred during "
               "callId translation (";
        err += e.what();
        err += ")";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "Unknown exception during callId translation";
        GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }

    g2mplsTypes::actorInfo_var actor;
    actor = new g2mplsTypes::actorInfo;

    actor->issuer = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
    actor->forceCommand = true;

    try {
        nccMgmtProxy->callSetDown(id, actor);
    } catch (Types::InternalProblems & e) {
        err += "callSetDown through CORBA Mgmt ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
       	GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (Types::CannotFetch & e) {
        err += "callSetDown through CORBA Mgmt ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
       	GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    } catch (...) {
        err += "callSetDown through CORBA Mgmt ";
        err += "interface failed for unknown reason";
       	GUNIGW_ERR("%s", err.c_str());
        return SOAP_OK;
    }
#endif // HAVE_OMNIORB

    GUNIGW_DBG("Sent callDestroy through CORBA Mgmt interface");
    
    //std::string RefPar;
    TAR->__sizeResponse = 1;
    //TAR->__size = 0;
    TAR->Response = new bes__TerminateActivityResponseType*;
    TAR->Response[0] = new bes__TerminateActivityResponseType;
    TAR->Response[0]->Cancelled = true;
    //TAR->Response[0]->ActivityIdentifier->ReferenceParameters  = new wsa__ReferenceParametersType;
    //TAR->Response[0]->ActivityIdentifier->ReferenceParameters->__size = 1;
    //TAR->Response[0]->ActivityIdentifier->ReferenceParameters->__any = new char*;
    //TAR->Response[0]->ActivityIdentifier->ReferenceParameters->__any[0] = new char[255];
    //sprintf(TAR->Response[0]->ActivityIdentifier->ReferenceParameters->__any[0],
    //	"<ph:ID xmlns:ph=\"http://schemas.phosphorus.eu\">%d</ph:ID>", callId);

    //char hostname[255];
    //gethostname (hostname, sizeof(hostname));
    //sprintf(TAR->Response[0]->ActivityIdentifier->Address->__item, "http://%s:8080", hostname);


    GUNIGW_DBG("WSAG Call: Returning TerminateActivities");
                
    return SOAP_OK;
}

int
BESFactoryBindingService::GetActivityDocuments(bes__GetActivityDocumentsType *GAD,
        bes__GetActivityDocumentsResponseType *GADR)
{
    //Call IDL
    struct soap soap;
    GUNIGW_DBG("WSAG Call: GetActivityDocuments\n");
    soap_init(&soap);
    GADR = new bes__GetActivityDocumentsResponseType;
    //GADR->ActivityIdentifier.Address = new char;
    //strcpy(CAR->ActivityIdentifier.Address, "http://127.0.0.1:8080");
    return SOAP_OK;
}

int
BESFactoryBindingService::GetFactoryAttributesDocument(bes__GetFactoryAttributesDocumentType *GFAD,
        bes__GetFactoryAttributesDocumentResponseType *GFADR)
{
    //Call IDL
    struct soap soap;
    GUNIGW_DBG("WSAG Call: GetFactoryAttributesDocument\n");
    soap_init(&soap);
    //callController(BES_GFAD, (char *)GFAD);
    GFADR = new bes__GetFactoryAttributesDocumentResponseType;
    //GADR->ActivityIdentifier.Address = new char;
    //strcpy(CAR->ActivityIdentifier.Address, "http://127.0.0.1:8080");
    return SOAP_OK;
}


