//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#ifndef GUNIGW_H
#define GUNIGW_H

#include <zebra.h>
#include <list>

#include "thread.h"
#include "vty.h"
#include "command.h"
#include "linklist.h"
#include "prefix.h"
#include "table.h"
#include "memory.h"
#include "stream.h"
#include "sockunion.h"
#include "network.h"

#include "gunigw_trace.hh"

#if HAVE_OMNIORB
#include "gunigw_corba.hh"
#endif // HAVE_OMNIORB


// GUNIGW  master for system wide configuration
class GUNIGW_Master {
 public:
	static bool                     init(void);
	static bool                     destroy(void);
	static GUNIGW_Master &          instance(void);
			                  
	void                            init_vty(void);
	struct thread_master *          getMaster();
	bool				dryrun(void);
	void				dryrun(bool value);
	uint32_t			routerId(void);
	void				routerId(uint32_t value);
	uint32_t			APserverId(void);
	void				APserverId(uint32_t value);

#if HAVE_OMNIORB
	CallController::Mgmt_var &	getNccProxy(void);
	CallController::EastWest_var &  getNccEWProxy(void);
	g2mplsTopology::Info_var &	getospfProxy(void);
	LRM::Info_var &			getLrmProxy(void);
#endif // HAVE_OMNIORB

 protected:
	GUNIGW_Master& operator=(const GUNIGW_Master& j);
	GUNIGW_Master(const GUNIGW_Master & j);
	GUNIGW_Master(void);
	~GUNIGW_Master(void);

 private:	
	static GUNIGW_Master *              instance_;
	static struct thread_master *       master_;     // thread master
	static time_t                       startTime_;  // start time
        static bool                         dryrun_;
	static uint32_t                     routerId_;
	static uint32_t                     APserverId_;
#if HAVE_OMNIORB
	static CallController::Mgmt_var	    nccMgmtProxy_;
	static CallController::EastWest_var nccEWProxy_;
 	static g2mplsTopology::Info_var	    ospfProxy_;
	static LRM::Info_var 		    lrmProxy_;
#endif // HAVE_OMNIORB
};

#define GUNIGW_MASTER    (GUNIGW_Master::instance())

#endif // GUNIGW_H

