//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//  Copyright (C) 2006, 2007, 2008, 2009 University of Essex.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//  Eduard Escalona       (Univ. of Essex)   <eescal_at_essex.ac.uk>
//



#include "zebra.h"

#include "lib/corba.h"
#include "log.h"
#include "hg2gw_trace.hh"
#include "hg2gw_corba.hh"
#include "../idl/lrm.hh"
#include "../idl/CallController.hh"
#include "../idl/RecoveryController.hh"
#include "../idl/g2mplsTopology.hh"
#include "../idl/lrm.hh"
#include <iostream>
#include <string>
#include <stdlib.h>

#if HAVE_OMNIORB

const char server[] = "localhost:8080";
//const char server[] = "http://bender.phosphorus.i2cat.cat:8080/harmony-idb-1";
networkReservationBindingProxy nres;
_rsv__createReservation *CR = new _rsv__createReservation;
_rsv__createReservationResponse *CRR = new _rsv__createReservationResponse;

using namespace std;

//
// Topology servant implementation
//

class HG2GW_i : public POA_CallController::EastWest,
                public PortableServer::RefCountServantBase
{
public:
    inline HG2GW_i();
    virtual ~HG2GW_i();

    CORBA::Boolean
    callCreate(g2mplsTypes::callIdent&,
               const g2mplsTypes::callParams&,
               const g2mplsTypes::recoveryParams&,
               const g2mplsTypes::lspParams&,
               const g2mplsTypes::aaaParams&,
               const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callSetTna(const g2mplsTypes::callIdent&,
               g2mplsTypes::resourcePosition,
               const g2mplsTypes::tnaResource&,
               const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callSetGnsTna(const g2mplsTypes::callIdent&,
                  g2mplsTypes::resourcePosition,
                  const g2mplsTypes::gridParams&,
                  const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callDestroy(const g2mplsTypes::callIdent&,
                const g2mplsTypes::actorInfo&);

    CORBA::Boolean
    callRaiseMsgEvent(const g2mplsTypes::callIdent& id,
                      g2mplsTypes::callMessageType msgType,
                      const char* reason,
                      const g2mplsTypes::actorInfo& actor);
};

HG2GW_i::HG2GW_i()
{
}

HG2GW_i::~HG2GW_i()
{
}

HG2GW_i*     servant      = 0;

CORBA::Boolean
HG2GW_i::callCreate(g2mplsTypes::callIdent& id,
           const g2mplsTypes::callParams& callPar,
           const g2mplsTypes::recoveryParams& recPar,
           const g2mplsTypes::lspParams& lspPar,
           const g2mplsTypes::aaaParams& aaaPar,
           const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();
    
    CR->createReservation = new rsv__CreateReservationEncapsulated;
    CR->createReservation->createReservation = new rsv__CreateReservationType;
    CR->createReservation->createReservation->__sizeService = 1;
    CR->createReservation->createReservation->Service = new rsv__ServiceConstraintType*;
    CR->createReservation->createReservation->Service[0]->ServiceID = id.localId;
    CR->createReservation->createReservation->Service[0]->__sizeConnections = 1;
    CR->createReservation->createReservation->Service[0]->Connections = new rsv__ConnectionConstraintType*;
    CR->createReservation->createReservation->Service[0]->Connections[0]->MinBW = lspPar.bw;
    CR->createReservation->createReservation->Service[0]->union_ServiceConstraintType.FixedReservationConstraints->StartTime = lspPar.times.startTime;
    CR->createReservation->createReservation->Service[0]->union_ServiceConstraintType.FixedReservationConstraints->Duration = lspPar.times.endTime - lspPar.times.startTime;
    
    STACK_UNLOCK();
}

CORBA::Boolean
HG2GW_i::callSetTna(const g2mplsTypes::callIdent& id,
           g2mplsTypes::resourcePosition resPos,
           const g2mplsTypes::tnaResource& tnaRes,
           const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();

    net_res_spec_t  tna;
    g2mplsTypes::tnaResource_var tnaResTmp;
    std::string     err;

    try {
        tnaResTmp = tnaRes;
        tna << tnaResTmp;
    } catch (std::out_of_range & e) {
        err  = "TNA not set: parameters error (";
        err += e.what();
        err += ")";
        HG2GW_ERR("%s", err.c_str());
        STACK_UNLOCK();
        return false;
    } catch (...) {
        err += "Unknown exception during tnaResource translation";
        HG2GW_ERR("%s", err.c_str());
        STACK_UNLOCK();
        return false;
    }

    switch (resPos) {
        case g2mplsTypes::RESOURCEPOSITION_INGRESS:
            CR->createReservation->createReservation->Service[0]->Connections[0]->Source = new nsp__EndpointType;
            CR->createReservation->createReservation->Service[0]->Connections[0]->Source->EndpointId = new char[255];
            inet_aton(CR->createReservation->createReservation->Service[0]->Connections[0]->Source->EndpointId, &(tna.tna.value.ipv4));
            break;
        case g2mplsTypes::RESOURCEPOSITION_EGRESS:
            CR->createReservation->createReservation->Service[0]->Connections[0]->__sizeTarget = 1;
            CR->createReservation->createReservation->Service[0]->Connections[0]->Target = new nsp__EndpointType*;
            CR->createReservation->createReservation->Service[0]->Connections[0]->Target[0]->EndpointId = new char[255];
            inet_aton(CR->createReservation->createReservation->Service[0]->Connections[0]->Target[0]->EndpointId, &(tna.tna.value.ipv4));
            break;
        default:
            break;
    }

    STACK_UNLOCK();
}

CORBA::Boolean
HG2GW_i::callSetGnsTna(const g2mplsTypes::callIdent&,
              g2mplsTypes::resourcePosition,
              const g2mplsTypes::gridParams&,
              const g2mplsTypes::actorInfo&)
{
    STACK_LOCK();
    STACK_UNLOCK();
}

CORBA::Boolean
HG2GW_i::callDestroy(const g2mplsTypes::callIdent&,
            const g2mplsTypes::actorInfo&)
{
    STACK_LOCK();
    STACK_UNLOCK();
}

CORBA::Boolean
HG2GW_i::callRaiseMsgEvent(const g2mplsTypes::callIdent& id,
                           g2mplsTypes::callMessageType msgType,
                           const char* reason,
                           const g2mplsTypes::actorInfo& actor)
{
    STACK_LOCK();

    std::string     err;
    g2mplsTypes::callIdent_var idtemp;
    call_ident_t id2;

    idtemp = id;
    id2 << idtemp;

    call_ident_t * id3 = new call_ident_t;
    *id3 = id2;

    switch (msgType) {
        case g2mplsTypes::CALLMESSAGETYPE_SETUPINDICATION:
            hg2gw_lock(false);
	    HG2GW_DBG("Received callRaiseMsgEvent(SetupIndication) through "
	                           "CORBA EW interface");
            thread_add_event(HG2GW_MASTER.getMaster(),
                             sendSetupConfirm,
                             id3,
                             0);
            break;
        case g2mplsTypes::CALLMESSAGETYPE_SETUPREQUEST:
            hg2gw_lock(false);
            HG2GW_DBG("Received callRaiseMsgEvent(SetupRequest) through "
                       "CORBA EW interface");

            thread_add_event(HG2GW_MASTER.getMaster(),
                             sendSetupIndication,
                             id3,
                             0);
            break;
        case g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM:
            break;
        default:
            break;
    }

    STACK_UNLOCK();
    return true;
}

bool corba_client_setup(CallController::Mgmt_var & l)
{
    CORBA::ORB_var orb;
    orb = corba_orb();

    if (CORBA::is_nil(orb)) {
        throw string("Cannot get ORB");
        HG2GW_ERR("Cannot get ORB");
        return false;
    }

    string ior;
    if (!corba_fetch_ior(CORBA_SERVANT_NETWORK_CALLCONTROLLER, ior)) {
        throw string("Cannot fetch IOR");
        HG2GW_ERR("Cannot fetch IOR");
        return false;
    }

    try {
        CORBA::Object_var obj;
        obj = orb->string_to_object(ior.c_str());
        if (CORBA::is_nil(obj)) {
            throw string("Cannot get object");
            HG2GW_ERR("Cannot get object");
            return false;
        }

        l = CallController::Mgmt::_narrow(obj);
        if (CORBA::is_nil(l)) {
            throw string("Cannot invoke on a nil object reference");
            HG2GW_ERR("cannot invoke on a nil object reference");
            return false;
        }

    } catch (CORBA::SystemException & e) {
        HG2GW_ERR("Caught CORBA::SystemException");
        return false;
    } catch (CORBA::Exception & e) {
        HG2GW_ERR("Caught CORBA::Exception");
        return false;
    } catch (...) {
        HG2GW_ERR("CLIENT_SETUP: caught unknown exception");
        return false;
    }

    return true;
}

bool corba_client_setup(CallController::EastWest_var & l)
{
    CORBA::ORB_var orb;
    orb = corba_orb();

    if (CORBA::is_nil(orb)) {
        throw string("Cannot get ORB");
        HG2GW_ERR("Cannot get ORB");
        return false;
    }

    string ior;
    if (!corba_fetch_ior(CORBA_SERVANT_NETWORK_CALLCONTROLLER_EW, ior)) {
        throw string("Cannot fetch IOR");
        HG2GW_ERR("Cannot fetch IOR");
        return false;
    }

    try {
        CORBA::Object_var obj;
        obj = orb->string_to_object(ior.c_str());
        if (CORBA::is_nil(obj)) {
            throw string("Cannot get object");
            HG2GW_ERR("Cannot get object");
            return false;
        }

        l = CallController::EastWest::_narrow(obj);
        if (CORBA::is_nil(l)) {
            throw string("Cannot invoke on a nil object reference");
            HG2GW_ERR("cannot invoke on a nil object reference");
            return false;
        }

    } catch (CORBA::SystemException & e) {
        HG2GW_ERR("Caught CORBA::SystemException");
        return false;
    } catch (CORBA::Exception & e) {
        HG2GW_ERR("Caught CORBA::Exception");
        return false;
    } catch (...) {
        HG2GW_ERR("CLIENT_SETUP: caught unknown exception");
        return false;
    }

    return true;
}

bool corba_client_setup(g2mplsTopology::Info_var & l)
{
    CORBA::ORB_var orb;
    orb = corba_orb();

    if (CORBA::is_nil(orb)) {
        throw string("Cannot get ORB");
        HG2GW_ERR("Cannot get ORB");
        return false;
    }

    string ior;
    if (!corba_fetch_ior(CORBA_SERVANT_G2TOPOLOGY, ior)) {
        throw string("Cannot fetch IOR");
        HG2GW_ERR("Cannot fetch IOR");
        return false;
    }

    try {
        CORBA::Object_var obj;
        obj = orb->string_to_object(ior.c_str());
        if (CORBA::is_nil(obj)) {
            throw string("Cannot get object");
            HG2GW_ERR("Cannot get object");
            return false;
        }

        l = g2mplsTopology::Info::_narrow(obj);
        if (CORBA::is_nil(l)) {
            throw string("Cannot invoke on a nil object reference");
            HG2GW_ERR("cannot invoke on a nil object reference");
            return false;
        }
    } catch (CORBA::SystemException & e) {
        HG2GW_ERR("Caught CORBA::SystemException");
        return false;
    } catch (CORBA::Exception & e) {
        HG2GW_ERR("Caught CORBA::Exception");
        return false;
    } catch (...) {
        HG2GW_ERR("CLIENT_SETUP: caught unknown exception");
        return false;
    }

    return true;
}

bool corba_client_setup(LRM::Info_var & l)
{
    CORBA::ORB_var orb;
    orb = corba_orb();

    if (CORBA::is_nil(orb)) {
        throw string("Cannot get ORB");
        HG2GW_ERR("Cannot get ORB");
        return false;
    }

    string ior;
    if (!corba_fetch_ior(CORBA_SERVANT_LRM, ior)) {
        throw string("Cannot fetch IOR");
        HG2GW_ERR("Cannot fetch IOR");
        return false;
    }

    try {
	CORBA::Object_var obj;
    	obj = orb->string_to_object(ior.c_str());
    	if (CORBA::is_nil(obj)) {
		    throw string("Cannot get object");
		    HG2GW_ERR("Cannot get object");
		    return false;
    	}

	l = LRM::Info::_narrow(obj);
	if (CORBA::is_nil(l)) {
	    throw string("Cannot invoke on a nil object reference");
	    HG2GW_ERR("cannot invoke on a nil object reference");
	    return false;
	}
    } catch (CORBA::SystemException & e) {
        HG2GW_ERR("Caught CORBA::SystemException");
        return false;
    } catch (CORBA::Exception & e) {
        HG2GW_ERR("Caught CORBA::Exception");
        return false;
    } catch (...) {
        HG2GW_ERR("CLIENT_SETUP: caught unknown exception");
        return false;
    }

    return true;
}


bool corba_client_shutdown(corba_servant_t s)
{
    return 1;
}

bool corba_server_setup(void)
{
    HG2GW_DBG("Setting up CORBA server side");

    try {
        servant = new HG2GW_i();
        if (!servant) {
            HG2GW_ERR("Cannot create servant");
            throw string("Cannot create servant");
        }

        PortableServer::POA_var poa;
        poa = corba_poa();
        if (CORBA::is_nil(poa)) {
            HG2GW_ERR("Cannot get POA");
            throw string("Cannot get POA");
        }

        PortableServer::ObjectId_var servant_id;
        servant_id = poa->activate_object(servant);

        CORBA::Object_var obj;
        obj = servant->_this();
        if (CORBA::is_nil(obj)) {
            HG2GW_ERR("Cannot get object");
            throw string("Cannot get object");
        }

        CORBA::ORB_var orb;
        orb = corba_orb();
        if (CORBA::is_nil(orb)) {
            HG2GW_ERR("Cannot get ORB");
            throw string("Cannot get ORB");
        }

        CORBA::String_var sior(orb->object_to_string(obj));
        zlog_debug("IOR = '%s'", (char*) sior);

        if (!corba_dump_ior(CORBA_SERVANT_HG2GW, string(sior))) {
            HG2GW_ERR("Cannot dump IOR");
            throw string("Cannot dump IOR");
        }

        servant->_remove_ref();

        PortableServer::POAManager_var poa_manager;
        poa_manager = corba_poa_manager();
        if (CORBA::is_nil(poa_manager)) {
            HG2GW_ERR("Cannot get POA Manager");
            throw string("Cannot get POA Manager");
        }

        poa_manager->activate();
    } catch (CORBA::SystemException & e) {
        HG2GW_ERR("Caught CORBA::SystemException");
        fprintf(stderr, "Caught CORBA::SystemException\n");
        return 0;
    } catch (CORBA::Exception & e) {
        HG2GW_ERR("Caught CORBA::Exception");
        fprintf(stderr, "Caught CORBA::Exception\n");
        return 0;
    } catch (omniORB::fatalException & e) {
        fprintf(stderr, "Caught omniORB::fatalException:\n");
        fprintf(stderr, "  file: %s\n", e.file());
        fprintf(stderr, "  line: %d\n", e.line());
        fprintf(stderr, "  mesg: %s\n", e.errmsg());
        return 0;
    } catch (string & e) {
        fprintf(stderr, "Caught exception: %s\n", e.c_str());
        return 0;
    } catch (...) {
        fprintf(stderr, "Caught unknown exception\n");
        return 0;
    }
	return 1;
}

bool corba_server_shutdown(void)
{
    try {
        zlog_debug("Shutting down CORBA server side");

        if (!corba_remove_ior(CORBA_SERVANT_HG2GW)) {
            throw string("Cannot remove IOR");
        }
    } catch (...) {
            zlog_err("Caught unknown exception");
            return 0;
    }
    return 1;
}

int
sendSetupIndication(struct thread *thread)
{
    call_ident_t * id3;
    call_ident_t id2;

    g2mplsTypes::callIdent_var id;
    std::string     err;

    CallController::EastWest_var         nccEastWestProxy;
    nccEastWestProxy = HG2GW_MASTER.getNccEWProxy();

    if (CORBA::is_nil(nccEastWestProxy)){
        HG2GW_ERR("Cannot get Corba client object "
                   "previously set up");
        assert(0);
    }

    g2mplsTypes::actorInfo_var actor2;
    actor2 = new g2mplsTypes::actorInfo;
    actor2->issuer       = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
    actor2->forceCommand = false;

    id3 = (call_ident_t *) THREAD_ARG(thread);
    id2 = *id3;
    id << id2;

    try {
        nccEastWestProxy->callRaiseMsgEvent(id,
        	g2mplsTypes::CALLMESSAGETYPE_SETUPINDICATION,
            	"",
            	actor2);
    } catch (Types::InternalProblems & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for unknown reason";
        HG2GW_ERR("%s", err.c_str());
        return false;
    }

    HG2GW_DBG("Sent callRaiseMsgEvent(SetupIndication) through "
               "CORBA EW interface");

    delete id3;
    return 0;
}

int
sendSetupConfirm(struct thread *thread)
{
    call_ident_t * id3;
    call_ident_t id2;

    g2mplsTypes::callIdent_var id;
    std::string     err;

    CallController::EastWest_var         nccEastWestProxy;
    nccEastWestProxy = HG2GW_MASTER.getNccEWProxy();

    if (CORBA::is_nil(nccEastWestProxy)){
        HG2GW_ERR("Cannot get Corba client object "
                   "previously set up");
        assert(0);
    }

    g2mplsTypes::actorInfo_var actor2;
    actor2 = new g2mplsTypes::actorInfo;
    actor2->issuer       = g2mplsTypes::ISSUERTYPE_MANAGEMENT_IF;
    actor2->forceCommand = false;

    id3 = (call_ident_t *) THREAD_ARG(thread);
    id2 = *id3;
    id << id2;

    try {
        nccEastWestProxy->callRaiseMsgEvent(id,
                g2mplsTypes::CALLMESSAGETYPE_SETUPCONFIRM,
                "",
                actor2);
    } catch (Types::InternalProblems & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for internal problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (Types::CannotFetch & e) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for fetching problems ";
        err += "(reason= ";
        err += e.what;
        err += ")";
        HG2GW_ERR("%s", err.c_str());
        return false;
    } catch (...) {
        err += "callRaiseMsgEvent through CORBA EastWest ";
        err += "interface failed for unknown reason";
        HG2GW_ERR("%s", err.c_str());
        return false;
    }

    HG2GW_DBG("Sent callRaiseMsgEvent(SetupConfirm) through "
               "CORBA EW interface");

    delete id3;
    return 0;
}

#endif // HAVE_OMNIORB
