/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *  Copyright (C) 2006, 2007, 2008, 2009 PSNC
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 *  Adam Kaliszan         (PSNC)             <kaliszan_at_man.poznan.pl>
 */


#include <zebra.h>
#include "thread.h"
#include "vty.h"
#include "command.h"
#include "linklist.h"
#include "prefix.h"
#include "table.h"
#include "if.h"
#include "memory.h"
#include "network.h"
#include "stream.h"
#include "log.h"
#include "sockunion.h"
#include "zclient.h"
#include "lrmd.h"
#include "lrm_zebra.h"
#include "zserv.h"
#include "lrm_core.h"
#if HAVE_OMNIORB
#include "lrm_corba.h"
#endif /* HAVE_OMNIORB */

int                serverFd;
int                connFd;
struct sockaddr_in client_sock;

typedef struct {
	int x;
	int y;
} coord;

/* LRM process wide configuration. */
static struct lrm_master lrm_master;

/* LRM process wide configuration pointer to export. */
struct lrm_master * lrm_m;

/* Structure that contains interfaces of LRM */
struct zlist * lrmiflist;

struct lrm_scn_init lrm_scn;


#if 0
int
lrm_sleep_thread (struct thread *thread)
{

	sleep(0.1);
	thread_add_read(master, lrm_sleep_thread, NULL, 0);

	return 1;
}
#endif

static int
lrm_conn_thread (struct thread *thread)
{
	struct sockaddr_in * addr;
	int                  con_fd = 0;
	//zlog_debug ("dummy thread: I got executed!!");
	//connFD = accept(serverFd, (struct s_addr *)NULL, NULL);
	int                  ret;
	coord                pos;
	int                  res = 0;

	addr = THREAD_ARG(thread);
	con_fd = THREAD_FD(thread);

	zlog_debug("conn thread:I got executed\n");

	errno = 0;

	if ((ret = recv(con_fd, &pos, sizeof(pos), 0)) !=  0) {
		zlog_debug("recv %s\n", strerror(errno));
		thread_add_read(master, lrm_conn_thread, addr, con_fd);
		if (ret > 0) {
			res = pos.x + pos.y;
			zlog_debug("received %d bytes from client: %d,%d\n",
				   ret, pos.x, pos.y);
			send(con_fd, &res, sizeof(res), 0);
		}
	}
	else{
		/*
		 * the client properly closed the connection we shutdown
		 * the socket and do not add the thread back again
		 *  shutdown(con_fd, SHUT_RDWR);
		 */
		zlog_debug("connection closed with the client\n");
	}

	return 1;
}


static int
lrm_serv_thread (struct thread *thread)
{
	static int           nb_conn = 0;
	uint32_t             sz = sizeof(struct sockaddr_in);
	struct sockaddr_in * c_sock;
	int                  connfd;

	c_sock = malloc(sizeof(struct sockaddr_in));
	bzero(c_sock, sizeof(struct sockaddr_in));

	zlog_debug ("serv thread: I got executed!!");

	if ((connfd = accept(serverFd,
			     (struct sockaddr *)c_sock, &sz)) == -1) {
		zlog_debug("accept error:%s\n", strerror(errno));
		return 0;
	}

	nb_conn++;

	set_nonblocking(connfd);

	zlog_debug("accepted connection from %s\n",
		   inet_ntoa(c_sock->sin_addr));

	thread_add_read(master, lrm_conn_thread, c_sock, connfd);
	thread_add_read(master, lrm_serv_thread, NULL, serverFd);

	return 1;
}


void
lrm_init (void)
{
	//thread_add_read (master, &lrm_sleep_thread, 0, 0);
	//thread_add_read(master, lrm_serv_thread, 0, serverFd);
}

void
lrm_terminate(void)
{
  lrm_t           * lrm;
  struct zlistnode * node;
  struct zlistnode * nnode;

  /* exit immediately if LRM not actually running */
  if (listcount(lrm_m->lrm) == 0) {
	  exit(0);
  }

  for (ALL_LIST_ELEMENTS(lrm_m->lrm, node, nnode, lrm)) {
	  lrm_finish(lrm);
  }
}


void
lrm_if_init ()
{
	/*
	 * Initialize LRM interface data structure.
	 * (similar to Zebra interface structure)
	 */

	lrmiflist = list_new();

	if_init();

	if (lrmiflist) {
	        lrmiflist->cmp = (int (*)(void *, void *))if_cmp_func;
		return;
	}

	lrm_m->iflist = iflist;
}


void
lrm_master_init ()
{
	memset(&lrm_master, 0, sizeof(struct lrm_master));

	lrm_m             = &lrm_master;
	lrm_m->lrm        = list_new();
	lrm_m->master     = thread_master_create();
	lrm_m->start_time = quagga_time(NULL);
	bzero(&lrm_scn, sizeof(struct lrm_scn_init));
	lrm_scn.status = SYNC;
}


void
lrm_serv_init()
{
	struct sockaddr_in servaddr;

	serverFd = socket(AF_INET, SOCK_STREAM, 0);

	memset(&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port = htons(2611);

	bind(serverFd, (struct sockaddr *)&servaddr, sizeof(servaddr));

	listen(serverFd, 5);
}


/*
 * This taken is partially from
 * if_create: lib/if.c
 * */
struct interface *
lrm_if_create(char *name, int name_len)
{

	struct interface * ifp;

	ifp          = XCALLOC (MTYPE_IF, sizeof(struct interface));
	ifp->ifindex = IFINDEX_INTERNAL;

	ASSERT(name);
	/* ASSERT (namelen <= INTERFACE_NAMSIZ); */

	strncpy (ifp->name, name, name_len);

	ifp->name[name_len] = '\0';

	/*
	if (if_lookup_by_name(ifp->name) == NULL) {
		listnode_add_sort (iflist, ifp);
	} else {
	  zlog_err("if_create(%s): corruption detected -- "
		   "interface with this name exists already!",
		   ifp->name);
	}
	*/

	ifp->connected      = list_new ();
	ifp->connected->del = (void (*) (void *)) connected_free;

	return ifp;
}


#if 0
void
lrm_interface_list(void)
{
	struct zlist     * te_list = get_list_ptr(TEL_LIST);
	te_link_t       * data_tel;
	struct zlistnode * node;

	/* loop on the TEL list and create an interface out of each TEL */

	for (ALL_LIST_ELEMENTS_RO(te_list, node, data_tel)) {

		struct interface * ifp;

		ifp = lrm_if_create(data_tel->tel_name,
				    strlen(data_tel->tel_name));
		ifp->ifindex = IFINDEX_INTERNAL;
		ifp->flags   = 0;
		ifp->mtu     = 65636;
		ifp->metric  = 1;

		listnode_add(iflist, ifp);

		/* Now we add the address of the TEL to this "interface" */
		{
			struct connected    * ifc  = connected_new();
			struct prefix_ipv4  * p    = prefix_ipv4_new ();
			struct  prefix_ipv4 * peer = prefix_ipv4_new ();

			p->family = AF_INET;
			/* Here we put the local TE link id ipv4 */
			p->prefix = data_tel->lcl_id.ipv4;
			p->prefixlen = prefixlen;
			ifc->address = (struct prefix *) p;

			ifc->flags = ZEBRA_IFA_PEER;
			ifc->conf = ZEBRA_IFC_CONFIGURED;

			peer->family = AF_INET;
			/* And here the remote TE Link id ipv4 */
			peer->prefix = data_tel->rem_id.ipv4;
			peer->prefixlen = prefixlen;
			ifc->destination = (struct prefix *) peer;
			ifc->ifp = ifp;
			listnode_add (ifp->connected, ifc);
		}

	}
}
#endif


/*
 * Creates a zebra/ospf interface out of a LRM TE Link
 */
struct interface *
TEL2interface(te_link_t * tel)
{
	struct interface   *ifp;
	struct zlistnode   *node;
	struct zlistnode   *nnode;
	u_int32_t          *data, *new_data;

        ifp = if_get_by_name_len(tel->tel_name, strnlen(tel->tel_name, 20));
	/*ifp->ifindex = IFINDEX_INTERNAL;*/
	ifp->ifindex = tel->tel_key;

	/*flags of the interface*/
	ifp->flags  = 4163; //IFF_UP and IFF_RUNNING should be enough
	/*
	 * We explicitly put MTU to maximum 2^16 to avoid pkts frag by ospfd
	 */
	ifp->mtu    = 65536;
	ifp->metric = 1;
	ifp->status = ZEBRA_INTERFACE_ACTIVE;
	struct connected    * ifc  = connected_new();
	struct prefix_ipv4  * p    = prefix_ipv4_new ();
	struct  prefix_ipv4 * peer = prefix_ipv4_new ();

	p->family    = AF_INET;
	p->prefix    = tel->lcl_id.value.ipv4;	/* local TE link id ipv4 */
	p->prefixlen = tel->lcl_id.preflen;

	ifc->address = (struct prefix *) p;
	ifc->flags   = ZEBRA_IFA_PEER;
	ifc->conf    = ZEBRA_IFC_CONFIGURED;

	peer->family     = AF_INET;
	peer->prefix     = tel->rem_id.value.ipv4; /* remote TE Link id ipv4 */
	peer->prefixlen  = tel->rem_id.preflen;
	ifc->destination = (struct prefix *) peer;
	ifc->ifp         = ifp;

	listnode_add (ifp->connected, ifc);

	/*now we send the te link TE params to ospf*/
	zlog_debug("Begin Wrinting TE parameters into interface %s", tel->tel_name);
	ifp->adj_type       = tel->adj_type;
	//TODO FIXME add command that add interface (with no TE-link) to ospf enni instance
	if (ifp->adj_type == ENNI)
	{
		ifp->ospf_instance = INNI;
		if (tel->rem_rc_id.type == IPv4)
			ifp->rem_rc_id = tel->rem_rc_id.value.ipv4.s_addr;
		else
			zlog_err("rc_id supported only IPv4");
	}
	else
	{
		ifp->ospf_instance = ifp->adj_type;
		ifp->rem_rc_id = 0;
	}
	ifp->te_local_id    = tel->lcl_id.value.ipv4.s_addr;
	ifp->te_remote_id   = tel->rem_id.value.ipv4.s_addr;
	ifp->te_remote_node_id = tel->rem_node_id;   // local node ID is found in ospf as active ospf instance Router ID
	ifp->te_metric         = tel->te_metric;
	ifp->te_link_color     = tel->link_color;
	ifp->te_swcap          = tel->swcap;
	ifp->te_swcap_options  = 0;                  // TODO provide approciate data
	ifp->te_enctype        = tel->enctype;
	ifp->te_max_bw         = tel->max_bw;
	ifp->te_max_res_bw     = tel->max_res_bw;
	memcpy(ifp->te_avail_bw_per_prio, tel->avail_bw_per_prio, MAX_BW_PRIORITIES * sizeof(u_int32_t));
	memcpy(ifp->te_max_LSP_bw, tel->max_LSP_bw, MAX_BW_PRIORITIES * sizeof(u_int32_t));
	ifp->te_min_LSP_bw     = tel->min_LSP_bw;
	ifp->te_protection_type= tel->protection_type;
	/*copy the advance reservation calendar of events*/
	adv_rsrv_calendar_copy(ifp->adv_rsrv_calendar, tel->calendar);
	/*copy the lambdas bitmap*/
	if (ifp->te_swcap == SWCAP_LSC) {
		lambdas_bitmap_copy(&(ifp->lambdas_bitmap), &(tel->lambdas_bitmap));
	}
	/*create the new srgl ids list*/

        if (ifp->te_SRLG_ids != NULL)
		list_delete_all_node(ifp->te_SRLG_ids);
	else
	{
		ifp->te_SRLG_ids = list_new();
		//ifp->te_SRLG_ids->del = del_te_SRLG_id;
	}

	for(ALL_LIST_ELEMENTS(tel->SRLG_ids, node, nnode, data))
	{
		new_data = (uint32_t *)malloc(sizeof(uint32_t));
		*new_data = *data;
		listnode_add(ifp->te_SRLG_ids, new_data);
	}

	/* zeroing */
	ifp->te_TNA_address_type  = IPv4;
	ifp->te_TNA_prefix_length = 0;
	ifp->te_TNA_address = (u_int32_t *) malloc(4);
	memset(ifp->te_TNA_address, 0, 4);

	if ( ifp->adj_type == UNI  ||
	    (ifp->adj_type == ENNI && !is_addr_null(tel->tna_id))) {
		ifp->te_TNA_address_type  = tel->tna_id.type;
		ifp->te_TNA_prefix_length = tel->tna_id.preflen;
		switch (ifp->te_TNA_address_type)
			{
				case IPv4:
					ifp->te_TNA_address = (u_int32_t *) malloc(4);
					memcpy(ifp->te_TNA_address , &tel->tna_id.value.ipv4, 4);
					break;
				case IPv6:
					ifp->te_TNA_address = (u_int32_t *) malloc(16);
					memcpy(ifp->te_TNA_address , &tel->tna_id.value.ipv6, 16);
					break;
				case NSAP:
					ifp->te_TNA_address = (u_int32_t *) malloc(20);
					memcpy(ifp->te_TNA_address , &tel->tna_id.value.nsap, 20);
					break;
				default:
					zlog_err("Bad TNA type %d", ifp->te_TNA_address_type);
					ifp->te_TNA_address = 0;
					assert(0);
			}
	}
	zlog_debug("End Wrinting TE parameters into interface %s", tel->tel_name);
	/*END of TE-link parameters*/

	return ifp;
}

/*
 * Notifies clients (ospfd mainly) that a new router ID has been added
 */
void
lrm_router_id_notify (u_int32_t rid, adj_type_t adj)
{
	struct zlistnode  * node;
	struct zserv     * client;
	struct prefix      p;

	p.u.prefix4.s_addr = 0;
	p.family           = AF_INET;
	p.prefixlen        = 32;

	if (rid == 0) {
		/* Nothing to do with null ROUTER ID */
		zlog_err("Nothing to notify..Router ID %s is nonsense",
			 numb_htoa(rid));
		return;
	}

	p.u.prefix4.s_addr = htonl(rid);

	zlog_debug("Going to notify new Router ID %s to attached clients",
		   numb_htoa(rid));

	for (ALL_LIST_ELEMENTS_RO(lrm_m->client_list, node, client)) {
		zsend_router_id_update(client, &p, adj);
		zlog_debug("Notify sent for new Router ID %s "
			   "to attached clients (sock %d)",
			   numb_htoa(rid), client->sock);

	}
}

/*
 * Notifies clients (ospfd mainly) that a new interface has been added
 */
int
lrm_interface_add_notify (struct interface * ifp)
{
	struct zlistnode  * cnode;
	struct zlistnode  * cnnode;
	struct connected * c;
	struct zserv     * cl;
	struct zlistnode  * clnode;
	struct zlistnode  * clnnode;

	/* Interface information is needed. */
	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if (zsend_interface_add (cl, ifp) < 0) {
			return -1;
		}

		for (ALL_LIST_ELEMENTS(ifp->connected, cnode, cnnode, c)) {

			if (zsend_interface_address(ZEBRA_INTERFACE_ADDRESS_ADD,
						    cl,
						    ifp,
						    c) < 0) {
				return -1;
			}
			/*
			zlog_debug("connected address %s",
				   inet_ntoa (c->address->u.prefix4));
			zlog_debug("connected address %s",
				   inet_ntoa (c->destination->u.prefix4));
			*/
		}
	}

	return 0;
}

/*
 * Notifies clients (ospfd mainly) that an interface has been deleted
 */
int
lrm_interface_delete_notify(struct interface * ifp)
{
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if (zsend_interface_delete (cl, ifp) < 0) {
			return -1;
		}
	}

	return 0;
}

/*
 * This function notifies SCNGWS that a new TE link has been added, brought up
 * and attached to a control channel.
 */
int
lrm_tel_add_notify(te_link_t * tel)
{
	lrm_scn.tel_number++;
	lrm_scn.tel_counter = lrm_scn.tel_number;
	//update the counter: obviously we also just add 1
	lrm_scn.tot_counter = lrm_scn.tel_number + lrm_scn.cc_number + lrm_scn.scnif_number;
#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if (zsend_lrm_add_tel(cl, tel) < 0) {
			return -1;
		}
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_tel_cc_assoc_add(tel) == 0) {
			return -1;
		}
	}
#endif /* !HAVE_OMNIORB */

	return 0;
}

/*
 * This function notifies SCNGWS that a new control channel
 * has been added and enabled
 */
int
lrm_cc_add_notify(cc_t * cc)
{
	lrm_scn.cc_number++;
	lrm_scn.cc_counter = lrm_scn.cc_number;
	lrm_scn.tot_counter = lrm_scn.tel_number + lrm_scn.cc_number + lrm_scn.scnif_number;
#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if(zsend_lrm_add_cc(cl, cc) < 0)
			return -1;
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_cc_scnif_assoc_add(cc) == 0) {
			return -1;
		}
		}
#endif /* !HAVE_OMNIORB */
	return 0;
}




int
lrm_scnif_add_notify(ctrl_intf_t *scnif)
{
	lrm_scn.scnif_number++;
	lrm_scn.scnif_counter = lrm_scn.scnif_number;
	lrm_scn.tot_counter = lrm_scn.tel_number + lrm_scn.cc_number + lrm_scn.scnif_number;
#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if(zsend_lrm_add_scnif(cl, scnif) < 0)
			return -1;
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_scnif_add(scnif) == 0) {
			return -1;
		}
	}
#endif /* !HAVE_OMNIORB */
	return 0;
}


int
lrm_scnif_del_notify(ctrl_intf_t *scnif)
{
	lrm_scn.scnif_number--;
	lrm_scn.scnif_counter = lrm_scn.scnif_number;
	lrm_scn.tot_counter = lrm_scn.tel_number + lrm_scn.cc_number + lrm_scn.scnif_number;
#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if(zsend_lrm_del_scnif(cl, scnif) < 0)
			return -1;
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_scnif_delete(scnif) == 0) {
			return -1;
		}
	}
#endif /* !HAVE_OMNIORB */
	return 0;
}



int
lrm_cc_del_notify(cc_t * cc)
{
	lrm_scn.cc_number--;
	lrm_scn.cc_counter = lrm_scn.cc_number;
	lrm_scn.tot_counter = lrm_scn.tel_number + lrm_scn.cc_number + lrm_scn.scnif_number;

#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if(zsend_lrm_del_cc(cl, cc->cc_id) < 0)
			return -1;
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_cc_scnif_assoc_delete(cc) == 0) {
			return -1;
		}
	}
#endif /* !HAVE_OMNIORB */
	return 0;
}


/*
 * This function notifies SCNGWS that a cc has been added of removed
 */
int
lrm_tel_update_notify(te_link_t * tel, int op, u_int32_t cc_id)
{
#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if (zsend_lrm_update_tel(cl, tel, op, cc_id) < 0) {
			return -1;
		}
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_tel_cc_assoc_update(tel, op, cc_id) == 0) {
			return -1;
		}
	}
#endif /* !HAVE_OMNIORB */

	return 0;
}
/*
 * This function notifies that a te-link has been deleted from lrmd
 * */
int
lrm_tel_del_notify(te_link_t * tel)
{
	/*update the lrm_scn structure*/
	lrm_scn.tel_number--;
	lrm_scn.tel_counter = lrm_scn.tel_number;
	lrm_scn.tot_counter = lrm_scn.tel_number + lrm_scn.cc_number + lrm_scn.scnif_number;
#if !HAVE_OMNIORB
	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;

	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if (zsend_lrm_del_tel(cl, tel->tel_key) < 0) {
			return -1;
		}
	}
#else
	if (lrm_scn.status == UP) {
		if(scngw_tel_cc_assoc_delete(tel) == 0) {
			return -1;
		}
	}
#endif /* !HAVE_OMNIORB */

	return 0;
}


/*
 * when SCN disconnects puts the lrm_scn.status to SYNC and every cc and tel
 * of telink_list and cc_list to NOT_SENT
 *
 */
int reset_lrm_scn()
{
	lrm_t           * lrm;
	struct zlistnode * node;
	struct zlistnode * nnode;
	te_link_t       * tel   = NULL;
	cc_t            * cc    = NULL;
	ctrl_intf_t     * scnif = NULL;

	lrm = lrm_lookup();
	for (ALL_LIST_ELEMENTS(lrm->telink_list, node, nnode, tel)) {
		ASSERT(tel);
		tel->sync_status = NOT_SENT;
	}
	for (ALL_LIST_ELEMENTS(lrm->cc_list, node, nnode, cc)) {
		ASSERT(cc);
		cc->sync_status = NOT_SENT;
	}
	for (ALL_LIST_ELEMENTS(lrm->scn_if_list, node, nnode, scnif)) {
		ASSERT(cc);
		scnif->sync_status = NOT_SENT;
	}

	lrm_scn.status = SYNC;
	lrm_scn.scnif_counter = lrm_scn.scnif_number;
	lrm_scn.cc_counter = lrm_scn.cc_number;
	lrm_scn.tel_counter = lrm_scn.tel_number;
	lrm_scn.tot_counter = lrm_scn.tel_counter + lrm_scn.cc_counter + lrm_scn.scnif_counter;

	return 0;
}



int lrm_interface_update_notify(struct interface *ifp, int up_type){


	struct zlistnode * clnode;
	struct zlistnode * clnnode;
	struct zserv    * cl;
	zlog_debug("updating interface %s", ifp->name);
	for (ALL_LIST_ELEMENTS(lrm_m->client_list,clnode, clnnode, cl)) {
		if (zsend_te_interface_update (cl, ifp, up_type) < 0) {
			return -1;
		}
	}
	return 0;
}
