/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */



#ifndef LRMD_H
#define LRMD_H

#ifdef __cplusplus
extern "C" {
#endif

#include "memory.h"
#include "thread.h"
#include "linklist.h"
#include "lrm_core.h"


#define LRMD_DEFAULT_CONFIG "/usr/local/etc/lrmd.conf"

//#define PATH_LRMD_PID "/var/run/quagga/lrmd.pid"

#define LRMD_VTY_PORT 2610    /* telnet port to login to the lrmd vty */

//#define LRM_VTYSH_PATH "/var/run/quagga/lrmd.vty"

#define LRM_CODE_VERSION "0.1"


/* unix socket to communicate with the vtysh */

/* LRM  master for system wide configuration and variables. */
struct lrm_master {

	/* LRM instance */
	struct zlist          * lrm;

	/* LRM thread master */
	struct thread_master * master;

	/* Zebra client list. */
	struct zlist          * client_list;

	/* Zebra interface list. */
	struct zlist          * iflist;

	/* LRM start time. */
	time_t start_time;

};

struct  lrm_scn_init {
	int scnif_number;
	int scnif_counter;
	int cc_number;	/*the number of cc assoc to send when SCN connects for the first time*/
	int cc_counter;
	int tel_number;	/*the number of tel assoc to send when SCN connects for the first time*/
	int tel_counter;
	int tot_counter;	/*counter of the number of all associations sent so far*/
#define SYNC	0
#define UP	1
	int status; /*the status of the synchronisation*/
	int err_counter;
};


/* Extern variables. */
extern struct lrm_master    * lrm_m;
extern struct thread_master * master;
extern struct lrm_scn_init  lrm_scn;


/* Function prototypes. */
void               lrm_if_init (void);
void               lrm_init(void);
void               lrm_terminate(void);
void               lrm_master_init (void);
void               lrm_serv_init(void);
struct interface * lrm_if_create(char *name, int name_len);
struct interface * TEL2interface(te_link_t *tel);
void               lrm_router_id_notify (u_int32_t rid, adj_type_t adj);
void               lrm_router_enni_id_notify (u_int32_t rid);
void               lrm_router_uni_id_notify (u_int32_t rid);
int                lrm_interface_add_notify (struct interface * ifp);
int                lrm_interface_delete_notify(struct interface *ifp);
int                lrm_interface_update_notify(struct interface *ifp, int up_type);
int                lrm_scnif_add_notify(ctrl_intf_t *scnif);
int                lrm_scnif_del_notify(ctrl_intf_t *scnif);
int                lrm_tel_cc_assoc_all_notify (te_link_t *tel);
int                lrm_tel_add_notify(te_link_t *tel);
int                lrm_tel_del_notify(te_link_t *tel);
int                lrm_tel_update_notify(te_link_t *tel, int op, u_int32_t cc_id);
int                lrm_cc_add_notify(cc_t * cc);
int                lrm_cc_del_notify(cc_t * cc);
#if HAVE_OMNIORB
int		   reset_lrm_scn(void);
#endif /* HAVE_OMNIORB */

#ifdef __cplusplus
}
#endif

#endif /* LRMD_H */
