/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
 *  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
 *  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
 *  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
 *  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
 */


#ifndef __ZSERV_H__
#define __ZSERV_H__

#define SCN_PORT	9999

#include "zebra/rib.h"
#include "if.h"
#include "workqueue.h"


/* Default port information. */
#define ZEBRA_VTY_PORT                2601

/* Default configuration filename. */
#define DEFAULT_CONFIG_FILE "zebra.conf"

/* Client structure. */
struct zserv
{
  /* Client file descriptor. */
  int sock;

  /* Input/output buffer to the client. */
  struct stream *ibuf;
  struct stream *obuf;

  /* Buffer of data waiting to be written to client. */
  struct buffer *wb;

  /* Threads for read/write. */
  struct thread *t_read;
  struct thread *t_write;

  /* Thread for delayed close. */
  struct thread *t_suicide;

  /* default routing table this client munges */
  int rtm_table;

  /* This client's redistribute flag. */
  u_char redist[ZEBRA_ROUTE_MAX];

  /* Redistribute default route flag. */
  u_char redist_default;

  /* Interface information. */
  u_char ifinfo;

  /* Router-id information. */
  u_char ridinfo;
#if !HAVE_OMNIORB
#define SCN_CLIENT	1
#define OTHER_CLIENT	0
  int client_type;
#endif /* !HAVE_OMNIORB */
};

/* Zebra instance */
struct zebra_t
{
  /* Thread master */
  struct thread_master *master;
  struct zlist *client_list;

  /* default table */
  int rtm_table_default;

  /* rib work queue */
  struct work_queue *ribq;
};

/* Count prefix size from mask length */
#define PSIZE(a) (((a) + 7) / (8))

/* Prototypes. */
extern void zebra_init (void);
extern void zebra_if_init (void);
extern void hostinfo_get (void);
extern void rib_init (void);
extern void interface_list (void);
extern void kernel_init (void);
extern void route_read (void);
extern void zebra_snmp_init (void);
extern void zebra_vty_init (void);

#if !HAVE_OMNIORB
void zread_lrm_all_assoc_add(struct zserv *client);
//te-link functions
int zsend_lrm_del_tel(struct zserv *client, int tel_key);
int zsend_lrm_add_tel(struct zserv *client, te_link_t *tel);
int zsend_lrm_update_tel(struct zserv *cl, te_link_t *tel, int op, u_int32_t cc_id);
//cc functions
int zsend_lrm_add_cc(struct zserv *client, cc_t *cc);
int zsend_lrm_update_cc(struct zserv *client, cc_t *cc);
int zsend_lrm_del_cc(struct zserv *client, int cc_id);
//scn-if functions
int zsend_lrm_add_scnif(struct zserv *client, ctrl_intf_t *scnif);
int zsend_lrm_del_scnif(struct zserv *client, ctrl_intf_t *scnif);

void write_tel_cc_assoc(te_link_t *tel, struct stream *s);
void write_cc_ctrlif_assoc(cc_t *cc, struct stream *s);

int szend_lrm_scn_fatal_err(struct zserv *client);
void zsend_lrm_scn_end_sync(struct zserv *client);
int zread_lrm_scn_up(struct zserv *client);

void zread_lrm_scn_end_ack(struct zserv *client);
#endif /* !HAVE_OMNIORB */


int zsend_te_interface_update(struct zserv *client, struct interface *ifp, int up_type);
int zsend_interface_add (struct zserv *, struct interface *);
int zsend_interface_delete (struct zserv *, struct interface *);
int zsend_interface_address (int, struct zserv *, struct interface *,
                             struct connected *);
int zsend_interface_update (int, struct zserv *, struct interface *);
int zsend_route_multipath (int, struct zserv *, struct prefix *,
			   struct rib *);
int zsend_router_id_update(struct zserv *, struct prefix *, adj_type_t adj);

extern pid_t pid;
extern pid_t old_pid;

#endif /* __ZSERV_H__ */
