//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "zebra.h"

#include "lib/corba.h"
#include "log.h"
#include "ncc_trace.hh"
#include "ncc_corba.hh"
#include "../idl/lrm.hh"
#include "../idl/CallController.hh"
#include "../idl/RecoveryController.hh"
#include "../idl/g2mplsTopology.hh"
#include <iostream>
#include <string>
#include <stdlib.h>



#if HAVE_OMNIORB

using namespace std;

bool corba_client_setup(CallController::Mgmt_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();

	if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		NCC_ERROR("Cannot get ORB");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_NETWORK_CALLCONTROLLER, ior)) {
		throw string("Cannot fetch IOR");
		NCC_ERROR("Cannot fetch IOR");
		return false;
	}

	CORBA::Object_var obj;
	obj = orb->string_to_object(ior.c_str());
	if (CORBA::is_nil(obj)) {
		throw string("Cannot get object");
		NCC_ERROR("Cannot get object");
		return false;
	}

	l = CallController::Mgmt::_narrow(obj);
	if (CORBA::is_nil(l)) {
		throw string("Cannot invoke on a nil object reference");
		NCC_ERROR("cannot invoke on a nil object reference");
		return false;
	}

	return true;
}

bool corba_client_setup(RecoveryController::NorthBound_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();

	if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		NCC_ERROR("Cannot get ORB");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_RC_NBI, ior)) {
		throw string("Cannot fetch IOR");
		NCC_ERROR("Cannot fetch IOR");
		return false;
	}

	CORBA::Object_var obj;
	obj = orb->string_to_object(ior.c_str());
	if (CORBA::is_nil(obj)) {
		throw string("Cannot get object");
		NCC_ERROR("Cannot get object");
		return false;
	}

	l = RecoveryController::NorthBound::_narrow(obj);
	if (CORBA::is_nil(l)) {
		throw string("Cannot invoke on a nil object reference");
		NCC_ERROR("cannot invoke on a nil object reference");
		return false;
	}

	return true;
}

bool corba_client_setup(g2mplsTopology::Info_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();

	if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		NCC_ERROR("Cannot get ORB");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_G2MPLS_TOPOLOGY, ior)) {
		throw string("Cannot fetch IOR");
		NCC_ERROR("Cannot fetch IOR");
		return false;
	}

	CORBA::Object_var obj;
	obj = orb->string_to_object(ior.c_str());
	if (CORBA::is_nil(obj)) {
		throw string("Cannot get object");
		NCC_ERROR("Cannot get object");
		return false;
	}

	l = g2mplsTopology::Info::_narrow(obj);
	if (CORBA::is_nil(l)) {
		throw string("Cannot invoke on a nil object reference");
		NCC_ERROR("cannot invoke on a nil object reference");
		return false;
	}

	return true;
}

bool corba_client_setup(LRM::Info_var & l)
{
	CORBA::ORB_var orb;
	orb = corba_orb();

	if (CORBA::is_nil(orb)) {
		throw string("Cannot get ORB");
		NCC_ERROR("Cannot get ORB");
		return false;
	}

	string ior;
	if (!corba_fetch_ior(CORBA_SERVANT_LRM, ior)) {
		throw string("Cannot fetch IOR");
		NCC_ERROR("Cannot fetch IOR");
		return false;
	}

	CORBA::Object_var obj;
	obj = orb->string_to_object(ior.c_str());
	if (CORBA::is_nil(obj)) {
		throw string("Cannot get object");
		NCC_ERROR("Cannot get object");
		return false;
	}

	l = LRM::Info::_narrow(obj);
	if (CORBA::is_nil(l)) {
		throw string("Cannot invoke on a nil object reference");
		NCC_ERROR("cannot invoke on a nil object reference");
		return false;
	}

	return true;
}

bool corba_client_shutdown(corba_servant_t s)
{
	return 1;
}

#endif // HAVE_OMNIORB
