//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include <zebra.h>

#include "thread.h"
#include "vty.h"
#include "command.h"
#include "linklist.h"
#include "prefix.h"
#include "table.h"
#include "memory.h"
#include "stream.h"
#include "log.h"
#include "sockunion.h"
#include "network.h"

#include "nccd.hh"
#include "nccd/ncc_config.hh"
#include "nccd/ncc_trace.hh"

//
// Statics initialization
//
NCC_Master *			NCC_Master::instance_     = 0;
struct thread_master *		NCC_Master::master_       = 0; // thread master
time_t				NCC_Master::startTime_    = 0; // start time
bool				NCC_Master::dryrun_       = false;
uint32_t			NCC_Master::routerId_     = 0;
#if HAVE_OMNIORB
CallController::Mgmt_var		NCC_Master::nccMgmtProxy_ = 0;
RecoveryController::NorthBound_var	NCC_Master::rcNBIProxy_   = 0;
LRM::Info_var					NCC_Master::lrmProxy_     = 0;
#endif // HAVE_OMNIORB

//
// CLASS NCC_Master implementation
//
NCC_Master::NCC_Master(void)
{
	METIN();

	NCC_WARNING("Called NCC_Master constructor");

	METOUT();
}

NCC_Master::~NCC_Master(void)
{
	METIN();

	NCC_WARNING("Called NCC_Master destructor");

	METOUT();
}

bool
NCC_Master::init(void)
{
	METIN();

	NCC_DEBUG("Initialising NCC_Master");

	instance_ = new NCC_Master;
	if (!instance_) {
		NCC_ERROR("Cannot allocate enough memory ...");
		METOUT();
		assert(0);
	}

	master_ = thread_master_create();
	if (!master_) {
		NCC_ERROR("Cannot create ZEBRA thread master ");
		assert(0);
	}

	startTime_ = quagga_time(NULL);
	dryrun_    = false;
	routerId_  = 0xBEEFFACE;

	METOUT();

	return true;
}

bool
NCC_Master::destroy(void)
{
	METIN();

	NCC_DEBUG("Finalising NCC_Master");

	startTime_    = 0;

#if HAVE_OMNIORB
	nccMgmtProxy_ = 0;
	rcNBIProxy_   = 0;
	lrmProxy_ = 0;
#endif // HAVE_OMNIORB

	delete instance_;
	instance_ = 0;

	thread_master_free(master_);
	master_ = 0;

	METOUT();

	return true;
}

NCC_Master &
NCC_Master::instance(void)
{
	METIN();

	if (instance_ == 0) {
		NCC_DEBUG("NCC_Master singleton instance is empty\n");

		instance_ = new NCC_Master;
		if (!instance_) {
			NCC_ERROR("Cannot allocate enough memory ...");
			METOUT();
			assert(0);
		}
	}

	METOUT();

	return * instance_;
}

struct thread_master *
NCC_Master::getMaster(void)
{
	METIN();

	//NCC_DEBUG("returning NCC master of threads");
	METOUT();

	return master_;
}

#if HAVE_OMNIORB
CallController::Mgmt_var &
NCC_Master::getNccProxy(void)
{
	METIN();

	METOUT();

	return nccMgmtProxy_;
}

RecoveryController::NorthBound_var &
NCC_Master::getRcNBIProxy(void)
{
	METIN();

	METOUT();

	return rcNBIProxy_;
}

LRM::Info_var &
NCC_Master::getLrmProxy(void)
{
	METIN();

	METOUT();

	return lrmProxy_;
}
#endif // HAVE_OMNIORB

bool
NCC_Master::dryrun(void)
{
	METIN();

	METOUT();

	return dryrun_;
}

void
NCC_Master::dryrun(bool value)
{
	METIN();

	dryrun_ = value;

	METOUT();
}

uint32_t
NCC_Master::routerId(void)
{
	METIN();

	METOUT();

	return routerId_;
}

void
NCC_Master::routerId(uint32_t value)
{
	METIN();

	routerId_ = value;

	METOUT();
}
