/*
 *  This file is part of phosphorus-g2mpls.
 *
 *  Copyright (C) 2006, 2007, 2008, 2009 PSNC
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Authors:
 *
 *  Adam Kaliszan         (PSNC)             <kaliszan_at_man.poznan.pl>
 *  Damian Parniewicz     (PSNC)             <damianp_at_man.poznan.pl>
 *  Lukasz Lopatowski     (PSNC)             <llopat_at_man.poznan.pl>
 *  Jakub Gutkowski       (PSNC)             <jgutkow_at_man.poznan.pl>
 */


#ifndef _OSPF_CORBA_H_
#define _OSPF_CORBA_H_

#if HAVE_OMNIORB

typedef enum {
  NTYPE_UNKNOWN,
  NTYPE_NETWORK,
  NTYPE_GRID
} node_type_t;

typedef enum {
  GRIDSUBNTYPE_UNKNOWN,
  GRIDSUBNTYPE_SERVICE,
  GRIDSUBNTYPE_COMPUTINGELEMENT,
  GRIDSUBNTYPE_SUBCLUSTER,
  GRIDSUBNTYPE_STORAGEELEMENT
} grid_s_node_type_t;

typedef enum {
  LINKT_UNKNOWN,
  LINKT_TE,
  LINKT_TE_SDHSONET,
  LINKT_TE_G709,
  LINKT_TE_WDM
} link_type_t;

typedef enum {
  LINKM_UNKNOWN         ,
  LINKM_P2P_UNNUMBERED  ,
  LINKM_P2P_NUMBERED    ,
  LINKM_MULTIACCESS     ,
  LINKM_ENNI_INTERDOMAIN,
  LINKM_ENNI_INTRADOMAIN
} link_mode_t;

typedef struct te_link_if_sw_cap
{
  u_char       switching_cap;     /** Switching Capability */
  u_char       encoding;          /** Encoding */
  float        maxLSPbw[8];       /** Max LSP Bandwidth */

  float        min_lsp_bw;        /** Minimul LSP Bandwidth (TDM & PSC) */
  u_char       indication;        /** Indication            (TDM)       */
  uint16_t     mtu;               /** Interface MTU         (PSC)       */
} te_link_if_sw_cap_t;

#ifdef __cplusplus
extern "C" {
#endif

void grid_subnode_del(uint32_t siteId, uint32_t id, grid_s_node_type_t type);
int corba_pce_client_setup(void);
int corba_unigw_client_setup(void);

/***** Corba update NODES *****/
void node_add                                    (uint8_t server,
                                                  int id,
                                                  node_type_t type);
void node_del                                    (uint8_t server,
                                                  uint32_t id,
                                                  node_type_t type);
void update_net_node                             (int id,
                                                  uint8_t isDomain);

/***** Corba update TE LINK *****/
void init_link_tmp_values                        (void);
void init_link_ident                             (void);

typedef enum {
  INTERDOM_TEL,
  INTRADOM_TEL
} telink_type_t;

void set_link_ident                              (telink_type_t type);

uint8_t link_update                              (uint8_t option, telink_type_t type);
void link_update_com                             (void);
void link_update_tdm                             (void);
void link_update_lscG709                         (void);
void link_update_lscwdm                          (void);
void link_update_states                          (void);
void link_update_genbw                           (void);
void link_update_tdmbw                           (void);
void link_update_lscwdm_bw                       (void);

void corba_update_advertising_router             (struct in_addr value);
void corba_update_te_link_type                   (uint8_t type);
void corba_update_te_link_id                     (struct in_addr id);
void corba_update_te_link_lclif_ipaddr           (struct in_addr value);
void corba_update_te_link_rmtif_ipaddr           (struct in_addr value);
void corba_update_te_link_metric                 (uint32_t teMetric);
void corba_update_te_link_max_bw                 (float teMaxBw);
void corba_update_te_link_max_res_bw             (float teMaxResvBw);
void corba_update_te_link_unrsv_bw               (float avBand[]);
void corba_update_te_link_rsc_clsclr             (uint32_t teColorMask);
void corba_update_te_link_lcl_rmt_ids            (uint32_t localId,
                                                  uint32_t remoteId);
void corba_update_te_link_protect_type           (uint8_t teProtectionTypeMask);
void corba_update_te_link_if_sw_cap_desc_pscisc  (uint8_t switching_cap,
                                                  uint8_t encoding,
                                                  float maxLSPbw[],
                                                  float minLSPbw,
                                                  uint16_t interfaceMTU);
void corba_update_te_link_if_sw_cap_desc_tdmisc  (uint8_t switching_cap,
                                                  uint8_t encoding,
                                                  float maxLSPbw[],
                                                  float minLSPbw,
                                                  uint8_t indication);
void corba_update_te_link_if_sw_cap_desc_genisc  (uint8_t switching_cap,
                                                  uint8_t encoding,
                                                  float maxLSPbw[]);
void corba_update_te_link_if_sw_cap_desc         (void);
void init_grid_TELink_Iscs                       (void);
void corba_update_te_link_shared_risk_link_grp   (uint32_t *srlg,
                                                  uint16_t len);
void corba_update_te_link_srlg                   (void);
void corba_update_te_link_lcl_node_id            (struct in_addr localNodeId);
void corba_update_te_link_rmt_node_id            (struct in_addr remoteNodeId);
void corba_update_te_link_ssdh_if_sw_cap_desc    (struct zlist* freeTS);
void corba_update_te_link_anc_rc_id              (struct in_addr value);
void corba_update_te_link_band_account           (uint32_t *band_account,
                                                  uint32_t list_len);
void corba_update_te_link_span_length            (uint32_t spanLength);
void corba_update_te_link_d_pdm                  (uint32_t dispersionPMD);
void corba_update_te_link_amp_list               (struct amp_par *amplist,
                                                  uint16_t len);
void corba_update_te_link_av_wave_mask           (u_int16_t num_wavelengths,
                                                  u_int32_t label_set_desc,
                                                  uint32_t *bitmap,
                                                  uint16_t bitmap_len);
void corba_update_te_link_callendar              (struct te_link_calendar *te_calendar,
                                                  uint16_t te_calendar_len);
void corba_update_te_link_tecal                  (void);

/***** Corba update TE TNA *****/
void init_tna_ident                              (void);
void corba_update_te_tna_addr                    (g2mpls_addr_t tna);
void corba_update_te_tna_node                    (struct in_addr node_id);
void corba_update_te_tna_id                      (int server,
                                                  uint8_t option,
                                                  g2mpls_addr_t tna_addr);
void corba_update_te_tna_anc_rc_id               (struct in_addr value);
void tna_ids_update                              (uint8_t option);

/***** Corba update TE NA *****/
void corba_update_te_na_router_id                (uint32_t router_id);
void corba_update_te_na_aa_id                    (uint32_t area_id);
void corba_update_te_na_ipv4_lcl_pref            (struct prefix_ip4 *pref_list,
                                                  uint16_t list_len);
void corba_update_te_na_ipv6_lcl_pref            (struct prefix_ip6 *pref_list,
                                                  uint16_t list_len);

/***** Corba update TE RA *****/
void corba_update_te_ra_router_addr              (struct in_addr address);
void corba_update_te_ra_aa_id                    (uint32_t area_id);

/***** Corba update GRID SITE *****/
void init_grid_GridSite(void);
void corba_update_gn_GridSite                    (int server);
void corba_update_gn_GridSite_ID                 (uint32_t id);
void corba_update_gn_GridSite_Name               (const char* name);
void corba_update_gn_GridSite_Latitude           (uint8_t *latitude);
void corba_update_gn_GridSite_Longitude          (uint8_t *latitude);
void corba_update_gn_GridSite_PERouter_ID        (struct in_addr ipv4Id);

/***** Corba update GRID SERVICE *****/
void init_grid_GridService(void);
void corba_update_gn_GridService                 (int server);
void corba_update_gn_GridService_ID              (uint32_t id);
void corba_update_gn_GridService_ParentSite_ID   (uint32_t id);
void corba_update_gn_GridService_ServiceInfo     (uint16_t type,
                                                  uint16_t version);
void corba_update_gn_GridService_Status          (char status);
void corba_update_gn_GridService_AddressLength   (u_char addressLength);
void corba_update_gn_GridService_IPv4Endpoint    (struct in_addr ipv4Endp);
void corba_update_gn_GridService_IPv6Endpoint    (struct in6_addr ipv6Endp);
void corba_update_gn_GridService_NsapEndpoint    (uint32_t nsapEndp[]);

/***** Corba update GRID COMPUTING ELEMENT *****/
void init_grid_GridComputingElement(void);
void corba_update_gn_GCE                         (int server);
void corba_update_gn_GCE_ID                      (uint32_t id);
void corba_update_gn_GCE_ParentSiteID            (uint32_t Id);
void corba_update_gn_GCE_LrmsInfo                (uint16_t lrmsType,
                                                  uint16_t lrmsVersion);
void corba_update_gn_GCE_AddressLength           (u_char addrLength);
void corba_update_gn_GCE_IPv4HostName            (struct in_addr ipv4HostNam);
void corba_update_gn_GCE_IPv6HostName            (struct in6_addr ipv6HostNam);
void corba_update_gn_GCE_NsapHostName            (uint32_t nsapHostNam[]);
void corba_update_gn_GCE_GatekeeperPort          (uint32_t gateKPort);
void corba_update_gn_GCE_JobManager              (const char* jman);
void corba_update_gn_GCE_DataDir                 (const char* datd);
void corba_update_gn_GCE_DefaultStorageElement   (uint32_t defaultSelement);
void corba_update_gn_GCE_JobsStates              (uint16_t freeJobSlots,
                                                  char status);
void corba_update_gn_GCE_JobsStats               (uint32_t runningJobs,
                                                  uint32_t waitingJobs,
                                                  uint32_t totalJobs);
void corba_update_gn_GCE_JobsTimePerformances    (uint32_t estRespTime,
                                                  uint32_t worstRespTime);
void corba_update_gn_GCE_JobsTimePolicy          (uint32_t maxWcTime,
                                                  uint32_t maxObtWcTime,
                                                  uint32_t maxCpuTime,
                                                  uint32_t maxObtCpuTime);
void corba_update_gn_GCE_JobsLoadPolicy          (uint32_t maxTotalJobs,
                                                  uint32_t maxRunJobs,
                                                  uint32_t maxWaitJobs,
                                                  uint16_t assignJobSlots,
                                                  uint16_t maxSlotsPerJob,
                                                  char priorityPreemptionFlag);
void corba_update_gn_GCE_CeCalendar              (struct zlist *ceCalList);
void corba_update_gn_GCE_Name                    (const char* name);

/***** Corba update GRID SUBCLUSTER *****/
void init_grid_GridSubCluster(void);
void corba_update_gn_GSCluster                   (int server);
void corba_update_gn_GSCluster_ID                (uint32_t id);
void corba_update_gn_GSCluster_ParentSiteID      (uint32_t parSiteId);
void corba_update_gn_GSCluster_CpuInfo           (uint32_t physicalCpus,
                                                  uint32_t logicalCpus,
                                                  char cpuArch);
void corba_update_gn_GSCluster_OsInfo            (uint16_t osType,
                                                  uint16_t osVersion);
void corba_update_gn_GSCluster_MemoryInfo        (uint32_t ramSize,
                                                  uint32_t virtualMemorySize);
void corba_update_gn_GSCluster_NewSoftwarePackage(uint16_t softType,
                                                  uint16_t softVersion,
                                                  char *env);
void corba_update_gn_GSCluster_SoftwarePackages  (void);
void corba_update_gn_GSCluster_SubClusterCalendar(struct zlist *scCalList);
void corba_update_gn_GSCluster_Name              (const char* name);

/***** Corba update GRID STORAGE ELEMENT *****/
void init_grid_GridStorage(void);
void corba_update_gn_GridStorage                 (int server);
void corba_update_gn_GridStorage_ID              (uint32_t id);
void corba_update_gn_GridStorage_ParentSiteID    (uint32_t parSiteId);
void corba_update_gn_GridStorage_StorageInfo     (uint32_t storInfo);
void corba_update_gn_GridStorage_OnlineSize      (uint32_t totalSize,
                                                  uint32_t usedSize);
void corba_update_gn_GridStorage_NearlineSize    (uint32_t totalSize,
                                                  uint32_t usedSize);
void corba_update_gn_GridStorage_NewStorageArea  (char *name,
                                                  char *path,
                                                  uint32_t totalOnlineSize,
                                                  uint32_t freeOnlineSize,
                                                  uint32_t resTotalOnlineSize,
                                                  uint32_t totalNearlineSize,
                                                  uint32_t freeNearlineSize,
                                                  uint32_t resNearlineSize,
                                                  char retPolAccLat,
                                                  char expirationMode);
void corba_update_gn_GridStorage_StorageAreas    (void);
void corba_update_gn_GridStorage_SeCalendar      (struct zlist *seCalList);
void corba_update_gn_GridStorage_Name            (const char* name);
void init_grid_TELink_Iscs                       ();

#ifdef __cplusplus
}
#endif

#endif // HAVE_OMNIORB

#endif // _OSPF_CORBA_H_
