//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 PSNC
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Adam Kaliszan         (PSNC)             <kaliszan_at_man.poznan.pl>
//  Damian Parniewicz     (PSNC)             <damianp_at_man.poznan.pl>
//  Lukasz Lopatowski     (PSNC)             <llopat_at_man.poznan.pl>
//  Jakub Gutkowski       (PSNC)             <jgutkow_at_man.poznan.pl>
//

#include "g2mpls_corba_utils.h"
#include "ospfd/ospf_corba_utils.h"
#include "idl/g2mplsTypes.hh"


void OSPF_CORBA_DEBUG(ocd_type_t type, ocd_action_t action, ocd_server_t server, string msgStr,
                      g2mplsTypes::tnaIdent tnaIdent, g2mplsTypes::teLinkIdent linkIdent)
{
  string actionStr = "";
  switch (action)
  {
    case OCD_ACTION_SENT:    actionStr = "Sent";    break;
    case OCD_ACTION_ABORTED: actionStr = "Aborted sending"; break;
  }

  string serverStr = "";
  switch (server)
  {
    case OCD_SERV_G2PCERA: serverStr = "G2PCERA"; break;
    case OCD_SERV_GUNIGW:  serverStr = "GUNIGW";  break;
  }

  zlog_debug("[DBG] CORBA: %s %s message to %s", actionStr.c_str(), msgStr.c_str(), serverStr.c_str());

  g2mplsTypes::tnaId_var tmpTna;
  g2mplsTypes::TELinkId_var tmpLink;
  std::string tnaIdStr;
  std::string localIdStr;
  std::string remoteIdStr;
  std::string lclNodeIdStr;
  std::string rmtNodeIdStr;

  switch (type)
  {
    case OCD_TYPE_GRID:
      break;

    case OCD_TYPE_TNA:
      struct in_addr nodeId;
      nodeId.s_addr = htonl(tnaIdent.node);
      tmpTna = tnaIdent.tna;
      tnaIdStr << tmpTna;
      zlog_debug("[DBG]        Node id: %s, TNA id: %s", inet_ntoa(nodeId), tnaIdStr.c_str());
      break;

    case OCD_TYPE_TELINK:
      struct in_addr lclNodeId, rmtNodeId;
      lclNodeId.s_addr = htonl(linkIdent.localNodeId);
      rmtNodeId.s_addr = htonl(linkIdent.remoteNodeId);
      lclNodeIdStr = inet_ntoa(lclNodeId);
      rmtNodeIdStr = inet_ntoa(rmtNodeId);

      tmpLink = linkIdent.localId;
      localIdStr << tmpLink;
      tmpLink = linkIdent.remoteId;
      remoteIdStr << tmpLink;

      zlog_debug("[DBG]        Lcl node id: %s,  Rmt node id: %s", lclNodeIdStr.c_str(), rmtNodeIdStr.c_str());
      zlog_debug("[DBG]        Lcl id: %s,  Rmt id: %s", localIdStr.c_str(), remoteIdStr.c_str());
      break;
  }
  return;
}

g2mplsTypes::nodeType
nodeType_from_node_type_t(node_type_t type)
{
  g2mplsTypes::nodeType ntype;
  switch (type)
  {
    case NTYPE_UNKNOWN: ntype = g2mplsTypes::NODETYPE_UNKNOWN; break;
    case NTYPE_NETWORK: ntype = g2mplsTypes::NODETYPE_NETWORK; break;
    case NTYPE_GRID: ntype = g2mplsTypes::NODETYPE_GRID; break;
  }
  return ntype;
}

g2mplsTypes::gridSubNodeType
gridSubNodeType_from_grid_s_node_type_t(grid_s_node_type_t type)
{
  g2mplsTypes::gridSubNodeType gntype;
  switch (type)
  {
    case GRIDSUBNTYPE_UNKNOWN:          gntype = g2mplsTypes::GRIDSUBNODETYPE_UNKNOWN; break;
    case GRIDSUBNTYPE_SERVICE:          gntype = g2mplsTypes::GRIDSUBNODETYPE_SERVICE; break;
    case GRIDSUBNTYPE_COMPUTINGELEMENT: gntype = g2mplsTypes::GRIDSUBNODETYPE_COMPUTINGELEMENT; break;
    case GRIDSUBNTYPE_SUBCLUSTER:       gntype = g2mplsTypes::GRIDSUBNODETYPE_SUBCLUSTER; break;
    case GRIDSUBNTYPE_STORAGEELEMENT:   gntype = g2mplsTypes::GRIDSUBNODETYPE_STORAGEELEMENT; break;
  }
  return gntype;
}

g2mplsTypes::gridServiceType
service_type_from_uint(uint16_t value)
{
  g2mplsTypes::gridServiceType type;
  switch (value)
  {
    case 0x0000: type = g2mplsTypes::SERVICE_UNKNOWN; break;
    case 0x0001: type = g2mplsTypes::ORG_GLITE_WMS; break;
    case 0x0002: type = g2mplsTypes::ORG_GLITE_RGMA_LATESTPRODUCER; break;
    case 0x0003: type = g2mplsTypes::ORG_GLITE_RGMA_STREAMPRODUCER; break;
    case 0x0004: type = g2mplsTypes::ORG_GLITE_RGMA_DBPRODUCER; break;
    case 0x0005: type = g2mplsTypes::ORG_GLITE_RGMA_CANONICALPRODUCER; break;
    case 0x0006: type = g2mplsTypes::ORG_GLITE_RGMA_ARCHIVER; break;
    case 0x0007: type = g2mplsTypes::ORG_GLITE_RGMA_CONSUMER; break;
    case 0x0008: type = g2mplsTypes::ORG_GLITE_RGMA_REGISTRY; break;
    case 0x0009: type = g2mplsTypes::ORG_GLITE_RGMA_SCHEMA; break;
    case 0x000A: type = g2mplsTypes::ORG_GLITE_RGMA_BROWSER; break;
    case 0x000B: type = g2mplsTypes::ORG_GLITE_RGMA_PRIMARYPRODUCER; break;
    case 0x000C: type = g2mplsTypes::ORG_GLITE_RGMA_SECONDARYPRODUCER; break;
    case 0x000D: type = g2mplsTypes::ORG_GLITE_RGMA_ONDEMANDPRODUCER; break;
    case 0x000E: type = g2mplsTypes::ORG_GLITE_VOMS; break;
    case 0x000F: type = g2mplsTypes::ORG_GLITE_FIREMANCATALOG; break;
    case 0x0010: type = g2mplsTypes::ORG_GLITE_SEINDEX; break;
    case 0x0011: type = g2mplsTypes::ORG_GLITE_METADATA; break;
    case 0x0012: type = g2mplsTypes::ORG_GLITE_CHANNELMANAGEMENT; break;
    case 0x0013: type = g2mplsTypes::ORG_GLITE_FILETRANSFER; break;
    case 0x0014: type = g2mplsTypes::ORG_GLITE_FILETRANSFERSTATS; break;
    case 0x0015: type = g2mplsTypes::ORG_GLITE_CHANNELAGENT; break;
    case 0x0016: type = g2mplsTypes::ORG_GLITE_KEYSTORE; break;
    case 0x0017: type = g2mplsTypes::ORG_GLITE_FAS; break;
    case 0x0018: type = g2mplsTypes::ORG_GLITE_GLITEIO; break;
    case 0x0100: type = g2mplsTypes::SRM; break;
    case 0x0200: type = g2mplsTypes::GSIFTP; break;
    case 0x0300: type = g2mplsTypes::ORG_EDG_LOCAL_REPLICA_CATALOG; break;
    case 0x0301: type = g2mplsTypes::ORG_EDG_REPLICA_METADATA_CATALOG; break;
    case 0x0302: type = g2mplsTypes::ORG_EDG_SE; break;
    case 0x0400: type = g2mplsTypes::IT_INFN_GRIDICE; break;
    case 0x0500: type = g2mplsTypes::MYPROXY; break;
    case 0x0600: type = g2mplsTypes::GUMS; break;
    case 0x0700: type = g2mplsTypes::GRIDCAT; break;
    case 0x0800: type = g2mplsTypes::EDU_CALTECH_CACR_MONALISA; break;
    case 0x0900: type = g2mplsTypes::OPENSSH; break;
    case 0x0A00: type = g2mplsTypes::MDS_GIIS; break;
    case 0x0B00: type = g2mplsTypes::BDII; break;
    case 0x0C00: type = g2mplsTypes::RLS; break;
    case 0x0D00: type = g2mplsTypes::DATA_LOCATION_INTERFACE; break;
    case 0x0E00: type = g2mplsTypes::PBS_TORQUE_SERVER; break;
    case 0x0E01: type = g2mplsTypes::PBS_TORQUE_MAUI; break;
    case 0x0F01: type = g2mplsTypes::UNICORE_CORE_TARGETSYSTEMFACTORY; break;
    case 0x0F02: type = g2mplsTypes::UNICORE_CORE_TARGETSYSTEM; break;
    case 0x0F03: type = g2mplsTypes::UNICORE_CORE_STORAGEMANAGEMENT; break;
    case 0x0F04: type = g2mplsTypes::UNICORE_CORE_FILETRANSFER; break;
    case 0x0F05: type = g2mplsTypes::UNICORE_CORE_JOBMANAGEMENT; break;
    case 0x0F06: type = g2mplsTypes::UNICORE_CORE_REGISTRY; break;
    case 0x0F07: type = g2mplsTypes::UNICORE_WORKFLOW_WORKFLOWFACTORY; break;
    case 0x0F08: type = g2mplsTypes::UNICORE_WORKFLOW_WORKFLOWMANAGEMENT; break;
    case 0x0F09: type = g2mplsTypes::UNICORE_WORKFLOW_SERVICEORCHESTRATOR; break;
    case 0x0F0A: type = g2mplsTypes::UNICORE_WORKFLOW_GRIDRESOURCEINFORMATIONSERVICE; break;
    case 0x0F0B: type = g2mplsTypes::UNICORE_CISINFORMATIONPROVIDER; break;
    case 0xFF00: type = g2mplsTypes::SERVICE_OTHER; break;
  }
  return type;
}

g2mplsTypes::gridServiceState
service_state_from_uint(uint8_t value)
{
  g2mplsTypes::gridServiceState state;
  switch (value)
  {
    case 0x0: state = g2mplsTypes::GRIDSERVICESTATE_UNKNOWN; break;
    case 0x1: state = g2mplsTypes::GRIDSERVICESTATE_OK; break;
    case 0x2: state = g2mplsTypes::GRIDSERVICESTATE_WARNING; break;
    case 0x3: state = g2mplsTypes::GRIDSERVICESTATE_CRITICAL; break;
    case 0xF: state = g2mplsTypes::GRIDSERVICESTATE_OTHER; break;
  }
  return state;
}

g2mplsTypes::gridLrmsType
lrms_type_from_uint(uint16_t value)
{
  g2mplsTypes::gridLrmsType type;
  switch (value)
  {
    case 0x0000: type = g2mplsTypes::GRIDLRMSTYPE_UNKNOWN; break;
    case 0x0001: type = g2mplsTypes::GRIDLRMSTYPE_OPENPBS; break;
    case 0x0002: type = g2mplsTypes::GRIDLRMSTYPE_LSF; break;
    case 0x0003: type = g2mplsTypes::GRIDLRMSTYPE_CONDOR; break;
    case 0x0004: type = g2mplsTypes::GRIDLRMSTYPE_BQS; break;
    case 0x0005: type = g2mplsTypes::GRIDLRMSTYPE_CONDORG; break;
    case 0x0006: type = g2mplsTypes::GRIDLRMSTYPE_FBSNG; break;
    case 0x0007: type = g2mplsTypes::GRIDLRMSTYPE_TORQUE; break;
    case 0x0008: type = g2mplsTypes::GRIDLRMSTYPE_PBSPRO; break;
    case 0x0009: type = g2mplsTypes::GRIDLRMSTYPE_SGE; break;
    case 0x000A: type = g2mplsTypes::GRIDLRMSTYPE_NQE; break;
    case 0x000B: type = g2mplsTypes::GRIDLRMSTYPE_FORK; break;
    case 0xFFFF: type = g2mplsTypes::GRIDLRMSTYPE_OTHER; break;
  }
  return type;
}

g2mplsTypes::gridCeSeState
cese_state_from_uint(uint8_t value)
{
  g2mplsTypes::gridCeSeState state;
  switch (value)
  {
    case 0x00: state = g2mplsTypes::GRIDCESESTATE_UNKNOWN; break;
    case 0x01: state = g2mplsTypes::GRIDCESESTATE_QUEUING; break;
    case 0x02: state = g2mplsTypes::GRIDCESESTATE_PRODUCTION; break;
    case 0x03: state = g2mplsTypes::GRIDCESESTATE_CLOSED; break;
    case 0x04: state = g2mplsTypes::GRIDCESESTATE_DRAINING; break;
  }
  return state;
}

g2mplsTypes::gridCpuArch
cpu_arch_from_uint(uint8_t value)
{
  g2mplsTypes::gridCpuArch arch;
  switch (value)
  {
    case 0x00: arch = g2mplsTypes::GRIDCPUARCH_UNKNOWN; break;
    case 0x01: arch = g2mplsTypes::GRIDCPUARCH_SPARC; break;
    case 0x02: arch = g2mplsTypes::GRIDCPUARCH_POWERPC; break;
    case 0x03: arch = g2mplsTypes::GRIDCPUARCH_X86; break;
    case 0x04: arch = g2mplsTypes::GRIDCPUARCH_X86_32; break;
    case 0x05: arch = g2mplsTypes::GRIDCPUARCH_X86_64; break;
    case 0x06: arch = g2mplsTypes::GRIDCPUARCH_PARISC; break;
    case 0x07: arch = g2mplsTypes::GRIDCPUARCH_MIPS; break;
    case 0x08: arch = g2mplsTypes::GRIDCPUARCH_IA64; break;
    case 0x09: arch = g2mplsTypes::GRIDCPUARCH_ARM; break;
    case 0xFF: arch = g2mplsTypes::GRIDCPUARCH_OTHER; break;
  }
  return arch;
}

g2mplsTypes::gridOsType
os_type_from_uint(uint16_t value)
{
  g2mplsTypes::gridOsType type;
  switch (value)
  {
    case 0x0000: type = g2mplsTypes::GRIDOSTYPE_UNKNOWN; break;
    case 0x0001: type = g2mplsTypes::GRIDOSTYPE_MACOS; break;
    case 0x0002: type = g2mplsTypes::GRIDOSTYPE_ATTUNIX; break;
    case 0x0003: type = g2mplsTypes::GRIDOSTYPE_DGUX; break;
    case 0x0004: type = g2mplsTypes::GRIDOSTYPE_DECNT; break;
    case 0x0005: type = g2mplsTypes::GRIDOSTYPE_TRU64_UNIX; break;
    case 0x0006: type = g2mplsTypes::GRIDOSTYPE_OPENVMS; break;
    case 0x0007: type = g2mplsTypes::GRIDOSTYPE_HPUX; break;
    case 0x0008: type = g2mplsTypes::GRIDOSTYPE_AIX; break;
    case 0x0009: type = g2mplsTypes::GRIDOSTYPE_MVS; break;
    case 0x000A: type = g2mplsTypes::GRIDOSTYPE_OS400; break;
    case 0x000B: type = g2mplsTypes::GRIDOSTYPE_OS_2; break;
    case 0x000C: type = g2mplsTypes::GRIDOSTYPE_JAVAVM; break;
    case 0x000D: type = g2mplsTypes::GRIDOSTYPE_MSDOS; break;
    case 0x000E: type = g2mplsTypes::GRIDOSTYPE_WIN3X; break;
    case 0x000F: type = g2mplsTypes::GRIDOSTYPE_WIN95; break;
    case 0x0010: type = g2mplsTypes::GRIDOSTYPE_WIN98; break;
    case 0x0011: type = g2mplsTypes::GRIDOSTYPE_WINNT; break;
    case 0x0012: type = g2mplsTypes::GRIDOSTYPE_WINCE; break;
    case 0x0013: type = g2mplsTypes::GRIDOSTYPE_NCR3000; break;
    case 0x0014: type = g2mplsTypes::GRIDOSTYPE_NETWARE; break;
    case 0x0015: type = g2mplsTypes::GRIDOSTYPE_OSF; break;
    case 0x0016: type = g2mplsTypes::GRIDOSTYPE_DC_OS; break;
    case 0x0017: type = g2mplsTypes::GRIDOSTYPE_RELIANT_UNIX; break;
    case 0x0018: type = g2mplsTypes::GRIDOSTYPE_SCO_UNIXWARE; break;
    case 0x0019: type = g2mplsTypes::GRIDOSTYPE_SCO_OPENSERVER; break;
    case 0x001A: type = g2mplsTypes::GRIDOSTYPE_SEQUENT; break;
    case 0x001B: type = g2mplsTypes::GRIDOSTYPE_IRIX; break;
    case 0x001C: type = g2mplsTypes::GRIDOSTYPE_SOLARIS; break;
    case 0x001D: type = g2mplsTypes::GRIDOSTYPE_SUNOS; break;
    case 0x001E: type = g2mplsTypes::GRIDOSTYPE_U6000; break;
    case 0x001F: type = g2mplsTypes::GRIDOSTYPE_ASERIES; break;
    case 0x0020: type = g2mplsTypes::GRIDOSTYPE_TANDEMNSK; break;
    case 0x0021: type = g2mplsTypes::GRIDOSTYPE_TANDEMNT; break;
    case 0x0022: type = g2mplsTypes::GRIDOSTYPE_BS2000; break;
    case 0x0023: type = g2mplsTypes::GRIDOSTYPE_LINUX; break;
    case 0x0024: type = g2mplsTypes::GRIDOSTYPE_LYNX; break;
    case 0x0025: type = g2mplsTypes::GRIDOSTYPE_XENIX; break;
    case 0x0026: type = g2mplsTypes::GRIDOSTYPE_VM; break;
    case 0x0027: type = g2mplsTypes::GRIDOSTYPE_INTERACTIVE_UNIX; break;
    case 0x0028: type = g2mplsTypes::GRIDOSTYPE_BSDUNIX; break;
    case 0x0029: type = g2mplsTypes::GRIDOSTYPE_FREEBSD; break;
    case 0x002A: type = g2mplsTypes::GRIDOSTYPE_NETBSD; break;
    case 0x002B: type = g2mplsTypes::GRIDOSTYPE_GNU_HURD; break;
    case 0x002C: type = g2mplsTypes::GRIDOSTYPE_OS9; break;
    case 0x002D: type = g2mplsTypes::GRIDOSTYPE_MACH_KERNEL; break;
    case 0x002E: type = g2mplsTypes::GRIDOSTYPE_INFERNO; break;
    case 0x002F: type = g2mplsTypes::GRIDOSTYPE_QNX; break;
    case 0x0030: type = g2mplsTypes::GRIDOSTYPE_EPOC; break;
    case 0x0031: type = g2mplsTypes::GRIDOSTYPE_IXWORKS; break;
    case 0x0032: type = g2mplsTypes::GRIDOSTYPE_VXWORKS; break;
    case 0x0033: type = g2mplsTypes::GRIDOSTYPE_MINT; break;
    case 0x0034: type = g2mplsTypes::GRIDOSTYPE_BEOS; break;
    case 0x0035: type = g2mplsTypes::GRIDOSTYPE_HP_MPE; break;
    case 0x0036: type = g2mplsTypes::GRIDOSTYPE_NEXTSTEP; break;
    case 0x0037: type = g2mplsTypes::GRIDOSTYPE_PALMPILOT; break;
    case 0x0038: type = g2mplsTypes::GRIDOSTYPE_RHAPSODY; break;
    case 0x0039: type = g2mplsTypes::GRIDOSTYPE_WINDOWS_2000; break;
    case 0x003A: type = g2mplsTypes::GRIDOSTYPE_DEDICATED; break;
    case 0x003B: type = g2mplsTypes::GRIDOSTYPE_OS_390; break;
    case 0x003C: type = g2mplsTypes::GRIDOSTYPE_VSE; break;
    case 0x003D: type = g2mplsTypes::GRIDOSTYPE_TPF; break;
    case 0x003E: type = g2mplsTypes::GRIDOSTYPE_WINDOWS_R_ME; break;
    case 0x003F: type = g2mplsTypes::GRIDOSTYPE_CALDERA_OPEN_UNIX; break;
    case 0x0040: type = g2mplsTypes::GRIDOSTYPE_OPENBSD; break;
    case 0x0042: type = g2mplsTypes::GRIDOSTYPE_WINDOWS_XP; break;
    case 0x0043: type = g2mplsTypes::GRIDOSTYPE_Z_OS; break;
    case 0xFFFF: type = g2mplsTypes::GRIDOSTYPE_OTHER; break;
  }
  return type;
}

g2mplsTypes::gridApplicationType
appl_type_from_uint(uint16_t value)
{
  g2mplsTypes::gridApplicationType type;
  switch (value)
  {
    case 0x0000: type = g2mplsTypes::GRIDAPPLICATIONTYPE_UNKNOWN; break;
    case 0x0001: type = g2mplsTypes::GRIDAPPLICATIONTYPE_WISDOM; break;
    case 0x0002: type = g2mplsTypes::GRIDAPPLICATIONTYPE_KODAVIS; break;
    case 0x0003: type = g2mplsTypes::GRIDAPPLICATIONTYPE_TOPS; break;
    case 0x0004: type = g2mplsTypes::GRIDAPPLICATIONTYPE_DDSS; break;
    case 0x0005: type = g2mplsTypes::GRIDAPPLICATIONTYPE_INCA; break;
    case 0xFFFF: type = g2mplsTypes::GRIDAPPLICATIONTYPE_OTHER; break;
  }
  return type;
}

g2mplsTypes::gridStorageArch
storage_arch_from_uint(uint8_t value)
{
  g2mplsTypes::gridStorageArch arch;
  switch (value)
  {
    case 0x0: arch = g2mplsTypes::GRIDSTORAGEARCH_UNKNOWN; break;
    case 0x1: arch = g2mplsTypes::GRIDSTORAGEARCH_DISK; break;
    case 0x2: arch = g2mplsTypes::GRIDSTORAGEARCH_TAPE; break;
    case 0x3: arch = g2mplsTypes::GRIDSTORAGEARCH_MULTIDISK; break;
    case 0xF: arch = g2mplsTypes::GRIDSTORAGEARCH_OTHER; break;
  }
  return arch;
}

g2mplsTypes::gridStorageRetentionPolicy
storage_retpol_from_uint(uint8_t value)
{
  g2mplsTypes::gridStorageRetentionPolicy ret;
  switch (value)
  {
    case 0x0: ret = g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_UNKNOWN; break;
    case 0x1: ret = g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_CUSTODIAL; break;
    case 0x2: ret = g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_OUTPUT; break;
    case 0x3: ret = g2mplsTypes::GRIDSTORAGERETENTIONPOLICY_REPLICA; break;
  }
  return ret;
}

g2mplsTypes::gridStorageAccessLatency
storage_acclat_from_uint(uint8_t value)
{
  g2mplsTypes::gridStorageAccessLatency lat;
  switch (value)
  {
    case 0x0: lat = g2mplsTypes::GRIDSTORAGEACCESSLATENCY_UNKNOWN; break;
    case 0x1: lat = g2mplsTypes::GRIDSTORAGEACCESSLATENCY_ONLINE; break;
    case 0x2: lat = g2mplsTypes::GRIDSTORAGEACCESSLATENCY_NEARLINE; break;
    case 0x3: lat = g2mplsTypes::GRIDSTORAGEACCESSLATENCY_OFFLINE; break;
  }
  return lat;
}

g2mplsTypes::gridStorageExpirationMode
storage_expmode_from_uint(uint8_t value)
{
  g2mplsTypes::gridStorageExpirationMode mode;
  switch (value)
  {
    case 0x0: mode = g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_UNKNOWN; break;
    case 0x1: mode = g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_NEVER_EXPIRE; break;
    case 0x2: mode = g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_WARN_WHEN_EXPIRED; break;
    case 0x3: mode = g2mplsTypes::GRIDSTORAGEEXPIRATIONMODE_RELEASE_WHEN_EXPIRED; break;
  }
  return mode;
}

g2mplsTypes::linkMode
linkMode_from_link_mode_t(link_mode_t mode)
{
  g2mplsTypes::linkMode lmode;
  switch (mode)
  {
    case LINKM_UNKNOWN:          lmode = g2mplsTypes::LINKMODE_UNKNOWN;          break;
    case LINKM_P2P_UNNUMBERED:   lmode = g2mplsTypes::LINKMODE_P2P_UNNUMBERED;   break;
    case LINKM_P2P_NUMBERED:     lmode = g2mplsTypes::LINKMODE_P2P_NUMBERED;     break;
    case LINKM_MULTIACCESS:      lmode = g2mplsTypes::LINKMODE_MULTIACCESS;      break;
    case LINKM_ENNI_INTERDOMAIN: lmode = g2mplsTypes::LINKMODE_ENNI_INTERDOMAIN; break;
    case LINKM_ENNI_INTRADOMAIN: lmode = g2mplsTypes::LINKMODE_ENNI_INTRADOMAIN; break;
  }
  return lmode;
}

g2mplsTypes::switchingCap
switching_cap_from_uchar(uint8_t value)
{
  g2mplsTypes::switchingCap cap;
  switch (value)
  {
    case 1:   cap = g2mplsTypes::SWITCHINGCAP_PSC_1; break;
    case 2:   cap = g2mplsTypes::SWITCHINGCAP_PSC_2; break;
    case 3:   cap = g2mplsTypes::SWITCHINGCAP_PSC_3; break;
    case 4:   cap = g2mplsTypes::SWITCHINGCAP_PSC_4; break;
    case 51:  cap = g2mplsTypes::SWITCHINGCAP_L2SC;  break;
    case 100: cap = g2mplsTypes::SWITCHINGCAP_TDM;   break;
    case 150: cap = g2mplsTypes::SWITCHINGCAP_LSC;   break;
    case 200: cap = g2mplsTypes::SWITCHINGCAP_FSC;   break;
  }
  return cap;
}

uint8_t
switching_cap_from_enum(g2mplsTypes::switchingCap swCap)
{
  switch (swCap)
  {
    case g2mplsTypes::SWITCHINGCAP_PSC_1: return 1;
    case g2mplsTypes::SWITCHINGCAP_PSC_2: return 2;
    case g2mplsTypes::SWITCHINGCAP_PSC_3: return 3;
    case g2mplsTypes::SWITCHINGCAP_PSC_4: return 4;
    case g2mplsTypes::SWITCHINGCAP_L2SC:  return 51;
    case g2mplsTypes::SWITCHINGCAP_TDM:   return 100;
    case g2mplsTypes::SWITCHINGCAP_LSC:   return 150;
    case g2mplsTypes::SWITCHINGCAP_FSC:   return 200;
  }
  return 0;
}

g2mplsTypes::encodingType
encoding_type_from_uchar(uint8_t value)
{
  g2mplsTypes::encodingType type;
  switch (value)
  {
    case 1:  type = g2mplsTypes::ENCODINGTYPE_PACKET;          break;
    case 2:  type = g2mplsTypes::ENCODINGTYPE_ETHERNET;        break;
    case 3:  type = g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH;   break;
    case 4:  type = g2mplsTypes::ENCODINGTYPE_RESERVED_1;      break;
    case 5:  type = g2mplsTypes::ENCODINGTYPE_SDH_SONET;       break;
    case 6:  type = g2mplsTypes::ENCODINGTYPE_RESERVED_2;      break;
    case 7:  type = g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER; break;
    case 8:  type = g2mplsTypes::ENCODINGTYPE_LAMBDA;          break;
    case 9:  type = g2mplsTypes::ENCODINGTYPE_FIBER;           break;
    case 10: type = g2mplsTypes::ENCODINGTYPE_RESERVED_3;      break;
    case 11: type = g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL;    break;
    case 12: type = g2mplsTypes::ENCODINGTYPE_G709_ODU;        break;
    case 13: type = g2mplsTypes::ENCODINGTYPE_G709_OC;         break;
  }
  return type;
}

uint8_t
encoding_type_from_enum(g2mplsTypes::encodingType encType)
{
  switch (encType)
  {
    case g2mplsTypes::ENCODINGTYPE_PACKET:          return 1;
    case g2mplsTypes::ENCODINGTYPE_ETHERNET:        return 2;
    case g2mplsTypes::ENCODINGTYPE_ANSI_ETSI_PDH:   return 3;
    case g2mplsTypes::ENCODINGTYPE_RESERVED_1:      return 4;
    case g2mplsTypes::ENCODINGTYPE_SDH_SONET:       return 5;
    case g2mplsTypes::ENCODINGTYPE_RESERVED_2:      return 6;
    case g2mplsTypes::ENCODINGTYPE_DIGITAL_WRAPPER: return 7;
    case g2mplsTypes::ENCODINGTYPE_LAMBDA:          return 8;
    case g2mplsTypes::ENCODINGTYPE_FIBER:           return 9;
    case g2mplsTypes::ENCODINGTYPE_RESERVED_3:     return 10;
    case g2mplsTypes::ENCODINGTYPE_FIBERCHANNEL:   return 11;
    case g2mplsTypes::ENCODINGTYPE_G709_ODU:       return 12;
    case g2mplsTypes::ENCODINGTYPE_G709_OC:        return 13;
  }
  return 0;
}

bool isNodeInSeq(g2mplsTypes::nodeIdent nodeIdent, g2mplsTypes::nodeIdentSeq* nodeSeq)
{
  g2mplsTypes::nodeIdent tmp;
  for (int i =0; i<nodeSeq->length(); i++) {
    tmp = (*nodeSeq)[i];
    if ((tmp.type == nodeIdent.type) && (tmp.id == nodeIdent.id))
      return true;
  }
  return false;
}

bool isSubNodeInSeq(g2mplsTypes::gridSubNodeIdent subNodeIdent, g2mplsTypes::gridSubNodeIdentSeq* snodes)
{
  g2mplsTypes::gridSubNodeIdent tmp;
  for (int i =0; i<snodes->length(); i++) {
    tmp = (*snodes)[i];
    if ((tmp.type == subNodeIdent.type) && (tmp.id == subNodeIdent.id))
      return true;
  }
  return false;
}

bool isTnaIdinSeq(g2mplsTypes::tnaIdent tna, g2mplsTypes::tnaIdentSeq* tnaSeq)
{
  g2mplsTypes::tnaIdent tmp;

  g2mpls_addr_t addr1;
  g2mpls_addr_t addr2;

  g2mplsTypes::tnaId_var tna1;
  g2mplsTypes::tnaId_var tna2;
  tna1 = tna.tna;
  addr1 << tna1;

  for (int i =0; i<tnaSeq->length(); i++) {
    tmp = (*tnaSeq)[i];
    tna2 = tmp.tna;
    addr2 << tna2;
    if ((tmp.rc == tna.rc) && (tmp.node == tna.node) && addr_equal(addr1, addr2) && (tmp.prefix == tna.prefix))
      return true;
  }
  return false;
}

bool isTELinkInSeq(g2mplsTypes::teLinkIdent ident, g2mplsTypes::teLinkIdentSeq* linkSeq)
{
  g2mplsTypes::teLinkIdent tmp;

  g2mpls_addr_t lclAddr1, rmtAddr1;
  g2mpls_addr_t lclAddr2, rmtAddr2;

  g2mplsTypes::TELinkId_var lclTeLinkId1, rmtTeLinkId1; 
  g2mplsTypes::TELinkId_var lclTeLinkId2, rmtTeLinkId2;

  lclTeLinkId1 = ident.localId;
  lclAddr1 << lclTeLinkId1;
  rmtTeLinkId1 = ident.remoteId;
  rmtAddr1 << rmtTeLinkId1;

  for (int i =0; i<linkSeq->length(); i++) {
    tmp = (*linkSeq)[i];
    lclTeLinkId2 = tmp.localId;
    lclAddr2 << lclTeLinkId2;
    rmtTeLinkId2 = tmp.remoteId;
    rmtAddr2 << rmtTeLinkId2;

    if ((tmp.localNodeId == ident.localNodeId) 
      && (tmp.remoteNodeId == ident.remoteNodeId)
      && addr_equal(lclAddr1, lclAddr2)
      && addr_equal(rmtAddr1, rmtAddr2)
      && (tmp.mode == ident.mode)
      && (tmp.localRcId == ident.localRcId)
      && (tmp.remoteRcId == ident.remoteRcId))
      return true;
  }
  return false;
}

bool equalTeLinks(const g2mplsTypes::teLinkIdent* linkParams1, g2mplsTypes::teLinkIdent* linkParams2)
{
  g2mpls_addr_t lclAddr1, rmtAddr1;
  g2mpls_addr_t lclAddr2, rmtAddr2;

  g2mplsTypes::TELinkId_var lclTeLinkId1, rmtTeLinkId1; 
  g2mplsTypes::TELinkId_var lclTeLinkId2, rmtTeLinkId2;

  lclTeLinkId1 = linkParams1->localId;
  lclAddr1 << lclTeLinkId1;
  rmtTeLinkId1 = linkParams1->remoteId;
  rmtAddr1 << rmtTeLinkId1;

  lclTeLinkId2 = linkParams2->localId;
  lclAddr2 << lclTeLinkId2;
  rmtTeLinkId2 = linkParams2->remoteId;
  rmtAddr2 << rmtTeLinkId2;

  if (linkParams1->localNodeId != linkParams2->localNodeId)
    return false;
  if (linkParams1->remoteNodeId != linkParams2->remoteNodeId)
    return false;
  if (!addr_equal(lclAddr1, lclAddr2))
    return false;
  if (!addr_equal(rmtAddr1, rmtAddr2))
    return false;
  if (linkParams1->mode != linkParams2->mode)
    return false;
  if (linkParams1->localRcId != linkParams2->localRcId)
    return false;
  if (linkParams1->remoteRcId != linkParams2->remoteRcId)
    return false;

  return true;
}

bool equalNetNodes(g2mplsTypes::netNodeParams* nodeParams1, g2mplsTypes::netNodeParams* nodeParams2)
{
  if (nodeParams1->isDomain != nodeParams2->isDomain)
    return false;
  if (nodeParams1->state.opState != nodeParams2->state.opState)
    return false;
  if (nodeParams1->state.admState != nodeParams2->state.admState)
    return false;
  if (nodeParams1->colors != nodeParams2->colors)
    return false;
  if (nodeParams1->areas.length() != nodeParams2->areas.length())
    return false;
  for (int i =0; i<nodeParams1->areas.length(); i++) {
    if (nodeParams1->areas[i] != nodeParams2->areas[i])
      return false; 
  }
  return true;
}

bool equalGridSites(g2mplsTypes::gridSiteParams* siteInfo1, g2mplsTypes::gridSiteParams* siteInfo2)
{
  char* siteName1 = (char *) (CORBA::String_var) siteInfo1->name;
  char* siteName2 = (char *) (CORBA::String_var) siteInfo2->name;

  if (strcmp(siteName1, siteName2) != 0)
    return false;
  if (siteInfo1->location.latResolution != siteInfo2->location.latResolution)
    return false;
  if (siteInfo1->location.latitute != siteInfo2->location.latitute)
    return false;
  if (siteInfo1->location.lonResolution != siteInfo2->location.lonResolution)
    return false;
  if (siteInfo1->location.longitude != siteInfo2->location.longitude)
    return false;
  if (siteInfo1->peRouterId != siteInfo2->peRouterId)
    return false;

  return true;
}

bool equalGridServices(g2mplsTypes::gridServiceParams* serviceInfo1, g2mplsTypes::gridServiceParams* serviceInfo2)
{
  if (serviceInfo1->data.type != serviceInfo2->data.type)
    return false;
  if (serviceInfo1->data.mjrRev != serviceInfo2->data.mjrRev)
    return false;
  if (serviceInfo1->data.mnrRev != serviceInfo2->data.mnrRev)
    return false;
  if (serviceInfo1->data.bldFix != serviceInfo2->data.bldFix)
    return false;
  if (serviceInfo1->state != serviceInfo2->state)
    return false;

  g2mpls_addr_t endPointAddr1;
  g2mpls_addr_t endPointAddr2;
  g2mplsTypes::gridHostId_var endPId1;
  g2mplsTypes::gridHostId_var endPId2;
  endPId1 = serviceInfo1->endPointAddr;
  endPId2 = serviceInfo2->endPointAddr;

  endPointAddr1 << endPId1;
  endPointAddr2 << endPId2;

  if (addr_equal(endPointAddr1, endPointAddr2) == 0)
    return false;

  return true;
}

bool equalGridCompElems(g2mplsTypes::gridCEParams* compInfo1, g2mplsTypes::gridCEParams* compInfo2)
{
  char* str1 = (char *) (CORBA::String_var) compInfo1->name;
  char* str2 = (char *) (CORBA::String_var) compInfo2->name;

  if (strcmp(str1, str2) != 0)
    return false;
  if (compInfo1->lrmsInfo.type != compInfo2->lrmsInfo.type)
    return false;
  if (compInfo1->lrmsInfo.mjrRev != compInfo2->lrmsInfo.mjrRev)
    return false;
  if (compInfo1->lrmsInfo.mnrRev != compInfo1->lrmsInfo.mnrRev)
    return false;
  if (compInfo1->lrmsInfo.bldFix != compInfo1->lrmsInfo.bldFix)
    return false;

  g2mpls_addr_t hostAddr1;
  g2mpls_addr_t hostAddr2;
  g2mplsTypes::gridHostId_var hostId1;
  g2mplsTypes::gridHostId_var hostId2;
  hostId1 = compInfo1->hostAddr;
  hostId2 = compInfo2->hostAddr;

  hostAddr1 << hostId1;
  hostAddr2 << hostId2;

  if (addr_equal(hostAddr1, hostAddr2) == 0)
    return false;
  if (compInfo1->gatekeeperPort != compInfo2->gatekeeperPort)
    return false;

  str1 = (char *) (CORBA::String_var) compInfo1->jobManager;
  str2 = (char *) (CORBA::String_var) compInfo2->jobManager;

  if (strcmp(str1, str2) != 0)
    return false;

  str1 = (char *) (CORBA::String_var) compInfo1->dataDir;
  str2 = (char *) (CORBA::String_var) compInfo2->dataDir;

  if (strcmp(str1, str2) != 0)
    return false;
  if (compInfo1->defaultStorageElemId != compInfo2->defaultStorageElemId)
    return false;
  if (compInfo1->jobsState.freeJobSlots != compInfo2->jobsState.freeJobSlots)
    return false;
  if (compInfo1->jobsState.state != compInfo2->jobsState.state)
    return false;
  if (compInfo1->jobsStats.runningJobs != compInfo2->jobsStats.runningJobs)
    return false;
  if (compInfo1->jobsStats.waitingJobs != compInfo2->jobsStats.waitingJobs)
    return false;
  if (compInfo1->jobsStats.totalJobs != compInfo2->jobsStats.totalJobs)
    return false;
  if (compInfo1->jobsTimePerf.estimatedResponseTime != compInfo2->jobsTimePerf.estimatedResponseTime)
    return false;
  if (compInfo1->jobsTimePerf.worstResponseTime != compInfo2->jobsTimePerf.worstResponseTime)
    return false;
  if (compInfo1->jobsTimePolicy.maxWallclockTime != compInfo2->jobsTimePolicy.maxWallclockTime)
    return false;
  if (compInfo1->jobsTimePolicy.maxObtainableWallclockTime != compInfo2->jobsTimePolicy.maxObtainableWallclockTime)
    return false;
  if (compInfo1->jobsTimePolicy.maxCpuTime != compInfo2->jobsTimePolicy.maxCpuTime)
    return false;
  if (compInfo1->jobsTimePolicy.maxObtainableCpuTime != compInfo2->jobsTimePolicy.maxObtainableCpuTime)
    return false;
  if (compInfo1->jobsLoadPolicy.maxTotalJobs != compInfo2->jobsLoadPolicy.maxTotalJobs)
    return false;
  if (compInfo1->jobsLoadPolicy.maxRunningJobs != compInfo2->jobsLoadPolicy.maxRunningJobs)
    return false;
  if (compInfo1->jobsLoadPolicy.maxWaitingJobs != compInfo2->jobsLoadPolicy.maxWaitingJobs)
    return false;
  if (compInfo1->jobsLoadPolicy.assignedJobSlots != compInfo2->jobsLoadPolicy.assignedJobSlots)
    return false;
  if (compInfo1->jobsLoadPolicy.maxSlotsPerJob != compInfo2->jobsLoadPolicy.maxSlotsPerJob)
    return false;
  if (compInfo1->jobsLoadPolicy.priority != compInfo2->jobsLoadPolicy.priority)
    return false;
  if (compInfo1->jobsLoadPolicy.preemptionFlag != compInfo2->jobsLoadPolicy.preemptionFlag)
    return false;
  if (compInfo1->freeJobSlotsCalendar.length() != compInfo2->freeJobSlotsCalendar.length())
    return false;

  for (int i =0; i<compInfo1->freeJobSlotsCalendar.length(); i++) {
    if (compInfo1->freeJobSlotsCalendar[i].unixTime != compInfo2->freeJobSlotsCalendar[i].unixTime)
      return false; 
    if (compInfo1->freeJobSlotsCalendar[i].JobSlots != compInfo2->freeJobSlotsCalendar[i].JobSlots)
      return false;
  }

  return true;
}

bool equalGridSubClusters(g2mplsTypes::gridSubClusterParams* subClInfo1, g2mplsTypes::gridSubClusterParams* subClInfo2)
{
  char* str1 = (char *) (CORBA::String_var) subClInfo1->name;
  char* str2 = (char *) (CORBA::String_var) subClInfo2->name;

  if (strcmp(str1, str2) != 0)
    return false;
  if (subClInfo1->cpu.cpuCounts.physical != subClInfo2->cpu.cpuCounts.physical)
    return false;
  if (subClInfo1->cpu.cpuCounts.logical != subClInfo2->cpu.cpuCounts.logical)
    return false;
  if (subClInfo1->cpu.cpuArch != subClInfo2->cpu.cpuArch)
    return false;
  if (subClInfo1->os.type != subClInfo2->os.type)
    return false;
  if (subClInfo1->os.mjrRev != subClInfo2->os.mjrRev)
    return false;
  if (subClInfo1->os.mnrRev != subClInfo2->os.mnrRev)
    return false;
  if (subClInfo1->os.bldFix != subClInfo2->os.bldFix)
    return false;
  if (subClInfo1->memory.ramSize != subClInfo2->memory.ramSize)
    return false;
  if (subClInfo1->memory.virtualMemorySize != subClInfo2->memory.virtualMemorySize)
    return false;
  if (subClInfo1->softwarePackages.length() != subClInfo2->softwarePackages.length())
    return false;

  for (int i =0; i< subClInfo1->softwarePackages.length(); i++) {
    if (subClInfo1->softwarePackages[i].software.type != subClInfo2->softwarePackages[i].software.type)
      return false;
    if (subClInfo1->softwarePackages[i].software.mjrRev != subClInfo2->softwarePackages[i].software.mjrRev)
      return false;
    if (subClInfo1->softwarePackages[i].software.mnrRev != subClInfo2->softwarePackages[i].software.mnrRev)
      return false;
    if (subClInfo1->softwarePackages[i].software.bldFix != subClInfo2->softwarePackages[i].software.bldFix)
      return false;

    str1 = (char *) (CORBA::String_var) subClInfo1->softwarePackages[i].softwareEnvironmentSetup;
    str2 = (char *) (CORBA::String_var) subClInfo2->softwarePackages[i].softwareEnvironmentSetup;

    if (strcmp(str1, str2) != 0)
      return false;
  }

  if (subClInfo1->subClusterCalendar.length() != subClInfo2->subClusterCalendar.length())
    return false;

  for (int i =0; i< subClInfo1->subClusterCalendar.length(); i++) {
    if (subClInfo1->subClusterCalendar[i].unixTime != subClInfo2->subClusterCalendar[i].unixTime)
      return false; 
    if (subClInfo1->subClusterCalendar[i].cpuCount.physical != subClInfo2->subClusterCalendar[i].cpuCount.physical)
      return false;
    if (subClInfo1->subClusterCalendar[i].cpuCount.logical != subClInfo2->subClusterCalendar[i].cpuCount.logical)
      return false;
  }
  return true;
}

bool equalGridStorages(g2mplsTypes::gridSEParams* storageInfo1, g2mplsTypes::gridSEParams* storageInfo2)
{
  char* str1 = (char *) (CORBA::String_var) storageInfo1->name;
  char* str2 = (char *) (CORBA::String_var) storageInfo2->name;

  if (strcmp(str1, str2) != 0)
    return false;
  if (storageInfo1->storageInfo.arch != storageInfo2->storageInfo.arch)
    return false;
  if (storageInfo1->storageInfo.state != storageInfo2->storageInfo.state)
    return false;
  if (storageInfo1->storageInfo.accessProtocolsMask != storageInfo2->storageInfo.accessProtocolsMask)
    return false;
  if (storageInfo1->storageInfo.controlProtocolsMask != storageInfo2->storageInfo.controlProtocolsMask)
    return false;
  if (storageInfo1->onlineSize.total != storageInfo2->onlineSize.total)
    return false;
  if (storageInfo1->onlineSize.used != storageInfo2->onlineSize.used)
    return false;
  if (storageInfo1->nearlineSize.total != storageInfo2->nearlineSize.total)
    return false;
  if (storageInfo1->nearlineSize.used != storageInfo2->nearlineSize.used)
    return false;
  if (storageInfo1->storageAreas.length() != storageInfo2->storageAreas.length())
    return false;

  for (int i =0; i< storageInfo1->storageAreas.length(); i++) {

  str1 = (char *) (CORBA::String_var) storageInfo1->storageAreas[i].storageAreaName;
  str2 = (char *) (CORBA::String_var) storageInfo2->storageAreas[i].storageAreaName;

  if (strcmp(str1, str2) != 0)
    return false;

  str1 = (char *) (CORBA::String_var) storageInfo1->storageAreas[i].storageAreaPath;
  str2 = (char *) (CORBA::String_var) storageInfo2->storageAreas[i].storageAreaPath;

  if (strcmp(str1, str2) != 0)
    return false;

    if (storageInfo1->storageAreas[i].storageAreaInfo.totalOnlineSize != storageInfo2->storageAreas[i].storageAreaInfo.totalOnlineSize)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.freeOnlineSize != storageInfo2->storageAreas[i].storageAreaInfo.freeOnlineSize)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.reservedTotalOnlineSize != storageInfo2->storageAreas[i].storageAreaInfo.reservedTotalOnlineSize)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.totalNearlineSize != storageInfo2->storageAreas[i].storageAreaInfo.totalNearlineSize)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.freeNearlineSize != storageInfo2->storageAreas[i].storageAreaInfo.freeNearlineSize)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.reservedNearlineSize != storageInfo2->storageAreas[i].storageAreaInfo.reservedNearlineSize)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.retentionPolicy != storageInfo2->storageAreas[i].storageAreaInfo.retentionPolicy)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.accessLatency != storageInfo2->storageAreas[i].storageAreaInfo.accessLatency)
      return false;
    if (storageInfo1->storageAreas[i].storageAreaInfo.expirationMode != storageInfo2->storageAreas[i].storageAreaInfo.expirationMode)
      return false;
  }

  if (storageInfo1->seCalendar.length() != storageInfo2->seCalendar.length())
    return false;

  for (int i =0; i< storageInfo1->seCalendar.length(); i++) {
    if (storageInfo1->seCalendar[i].unixTime != storageInfo2->seCalendar[i].unixTime)
      return false; 
    if (storageInfo1->seCalendar[i].storageCount.freeOnlineSize != storageInfo2->seCalendar[i].storageCount.freeOnlineSize)
      return false;
    if (storageInfo1->seCalendar[i].storageCount.logicalCpus != storageInfo2->seCalendar[i].storageCount.logicalCpus)
      return false;
  }
  return true;
}

bool equalComParams(g2mplsTypes::teLinkComParams* comParam1, g2mplsTypes::teLinkComParams* comParam2)
{
  if (comParam1->adminMetric != comParam2->adminMetric)
    return false;
  if (comParam1->teMetric != comParam2->teMetric)
    return false;
  if (comParam1->teColorMask != comParam2->teColorMask)
    return false;
  if (comParam1->teProtectionTypeMask != comParam2->teProtectionTypeMask)
    return false;
  if (comParam1->teMaxBw != comParam2->teMaxBw)
    return false;
  if (comParam1->teMaxResvBw != comParam2->teMaxResvBw)
    return false;

  return true;
}

bool equalTdmParams(g2mplsTypes::teLinkTdmParams* tdmParam1, g2mplsTypes::teLinkTdmParams* tdmParam2)
{
  if (tdmParam1->hoMuxCapMask != tdmParam2->hoMuxCapMask)
    return false;
  if (tdmParam1->loMuxCapMask != tdmParam2->loMuxCapMask)
    return false;
  if (tdmParam1->transparencyMask != tdmParam2->transparencyMask)
    return false;
  if (tdmParam1->blsrRingId != tdmParam2->blsrRingId)
    return false;

  return true;
}

bool equalLscG709Params(g2mplsTypes::teLinkLscG709Params* lscG709Params1, g2mplsTypes::teLinkLscG709Params* lscG709Params2)
{
  //TODO

  return false;
}

bool equalLscWdmParams(g2mplsTypes::teLinkLscWdmParams lscWdmParam1, g2mplsTypes::teLinkLscWdmParams lscWdmParam2)
{
  if (lscWdmParam1.dispersionPMD != lscWdmParam2.dispersionPMD)
    return false;
  if (lscWdmParam1.spanLength != lscWdmParam2.spanLength)
    return false;
  if (lscWdmParam1.amplifiers.length() != lscWdmParam2.amplifiers.length())
    return false;

  for (int i =0; i< lscWdmParam1.amplifiers.length(); i++) {
    if (lscWdmParam1.amplifiers[i].gain != lscWdmParam2.amplifiers[i].gain)
      return false; 
    if (lscWdmParam1.amplifiers[i].noiseFigure != lscWdmParam2.amplifiers[i].noiseFigure)
      return false;
  }
  return true;
}

bool equalLinkStates(g2mplsTypes::statesBundle* states1, g2mplsTypes::statesBundle* states2)
{
  if (states1->opState != states2->opState)
    return false;
  if (states1->admState != states2->admState)
    return false;

  return true;
}

bool equalGenBw(g2mplsTypes::availBwPerPrio bw1, g2mplsTypes::availBwPerPrio bw2)
{
  for (int i=0; i<8; i++)
    if (bw1[i] != bw2[i])
      return false;

  return true;
}

bool equalTdmBw(g2mplsTypes::freeCTPSeq freeCTP1, g2mplsTypes::freeCTPSeq freeCTP2)
{
  if (freeCTP1.length() != freeCTP2.length())
    return false;

  for (int i =0; i< freeCTP1.length(); i++) {
    if (freeCTP1[i].sigType != freeCTP2[i].sigType)
      return false; 
    if (freeCTP1[i].ctps != freeCTP2[i].ctps)
      return false;
  }
  return true;
}

bool equalLscG709Bw(g2mplsTypes::freeCTPSeq freeODUk1, g2mplsTypes::freeCTPSeq freeOCh1,
                    g2mplsTypes::freeCTPSeq freeODUk2, g2mplsTypes::freeCTPSeq freeOCh2)
{
  if (freeODUk1.length() != freeODUk2.length())
    return false;
  if (freeOCh1.length() != freeOCh2.length())
    return false;

  for (int i =0; i< freeODUk1.length(); i++) {
    if (freeODUk1[i].sigType != freeODUk2[i].sigType)
      return false; 
    if (freeODUk1[i].ctps != freeODUk2[i].ctps)
      return false;
  }

  for (int i =0; i< freeOCh1.length(); i++) {
    if (freeOCh1[i].sigType != freeOCh2[i].sigType)
      return false; 
    if (freeOCh1[i].ctps != freeOCh2[i].ctps)
      return false;
  }
  return true;
}

bool equalLscWdmBw(g2mplsTypes::teLinkWdmLambdasBitmap bitmap1, g2mplsTypes::teLinkWdmLambdasBitmap bitmap2)
{
  if (bitmap1.baseLambda != bitmap2.baseLambda)
    return false;
  if (bitmap1.numLambdas != bitmap2.numLambdas)
    return false;
  if (bitmap1.bitmap.length() != bitmap2.bitmap.length())
    return false;

  for (int i=0; i< bitmap1.bitmap.length(); i++)
    if (bitmap1.bitmap[i] != bitmap2.bitmap[i])
      return false;

  return true;
}

g2mplsTypes::srlgSeq diffSrlgs(g2mplsTypes::srlgSeq srlgs1, g2mplsTypes::srlgSeq srlgs2)
{
  g2mplsTypes::srlgSeq_var seq;
  g2mplsTypes::srlgSeq * tmp;
  bool *ind = new bool[srlgs1.length()];
  for (int m=0; m<srlgs1.length(); m++)
    ind[m] = false;

  int count = srlgs1.length();

  for (int i=0; i< srlgs1.length(); i++)
    for (int j=0; j< srlgs2.length(); j++)
  {
      if (srlgs1[i] == srlgs2[j])
      {
        count--;
        ind[i] = true;
      }
  }

  tmp = new g2mplsTypes::srlgSeq(count);
  seq = tmp;
  seq->length(count);

  int index = 0;
  for (int m=0; m<srlgs1.length(); m++)
    if (ind[m] == false)
  {
    seq[index] = srlgs1[m];
    index++;
  }

  return seq;
}

g2mplsTypes::teLinkCalendarSeq diffCalendars(g2mplsTypes::teLinkCalendarSeq cal1, g2mplsTypes::teLinkCalendarSeq cal2)
{
  g2mplsTypes::teLinkCalendarSeq_var seq;
  g2mplsTypes::teLinkCalendarSeq * tmp;
  bool *ind = new bool[cal1.length()];
  for (int m=0; m<cal1.length(); m++)
    ind[m] = false;

  int count = cal1.length();
  bool equal;

  for (int i=0; i< cal1.length(); i++)
    for (int j=0; j< cal2.length(); j++)
  {
    if (cal1[i].unixTime == cal2[j].unixTime)
    {
      equal = true;
      for (int k=0; k<8; k++)
        if (cal1[i].availBw[k] != cal2[j].availBw[k])
          equal = false;

      if (equal) {
        count--;
        ind[i] = true;
      }
    }
  }

  tmp = new g2mplsTypes::teLinkCalendarSeq(count);
  seq = tmp;
  seq->length(count);

  int index = 0;
  for (int m=0; m<cal1.length(); m++)
    if (ind[m] == false)
  {
    seq[index] = cal1[m];
    index++;
  }

  return seq;
}

g2mplsTypes::iscSeq diffIscs(g2mplsTypes::iscSeq iscs1, g2mplsTypes::iscSeq iscs2)
{
  g2mplsTypes::isc_var isc1;
  g2mplsTypes::isc_var isc2;
  g2mplsTypes::iscParamsTdm_var tdm1;
  g2mplsTypes::iscParamsTdm_var tdm2;
  g2mplsTypes::iscParamsPsc_var psc1;
  g2mplsTypes::iscParamsPsc_var psc2;
  g2mplsTypes::iscParamsGen_var gen1;
  g2mplsTypes::iscParamsGen_var gen2;

  g2mplsTypes::iscSeq_var seq;
  g2mplsTypes::iscSeq * tmp;
  bool *ind = new bool[iscs1.length()];
  for (int m=0; m<iscs1.length(); m++)
    ind[m] = false;

  int count = iscs1.length();

  bool equal;
  for (int i=0; i< iscs1.length(); i++)
    for (int j=0; j< iscs2.length(); j++)
  {
    isc1 = iscs1[i];
    isc2 = iscs2[j];

    if (isc1->_d() == isc2->_d())
    {
      switch (isc1->_d())
      {
        case g2mplsTypes::SWITCHINGCAP_PSC_1:
        case g2mplsTypes::SWITCHINGCAP_PSC_2:
        case g2mplsTypes::SWITCHINGCAP_PSC_3:
        case g2mplsTypes::SWITCHINGCAP_PSC_4:

          psc1 = isc1->psc();
          psc2 = isc2->psc();

          equal = true;

          if (psc1->swCap != psc2->swCap)
            equal = false;
          if (psc1->encType != psc2->encType)
            equal = false;
          if (psc1->minLSPbandwidth != psc2->minLSPbandwidth)
            equal = false;
          if (psc1->interfaceMTU != psc2->interfaceMTU)
            equal = false;

          for (int k=0; k<8; k++)
            if (psc1->maxLSPbandwidth[k] != psc2->maxLSPbandwidth[k])
              equal = false;

          if (equal) {
            count--;
            ind[i] = true;
          }

          break;

        case g2mplsTypes::SWITCHINGCAP_TDM  :

          tdm1 = isc1->tdm();
          tdm2 = isc2->tdm();

          equal = true;

          if (tdm1->swCap != tdm2->swCap)
            equal = false;
          if (tdm1->encType != tdm2->encType)
            equal = false;
          if (tdm1->indication != tdm2->indication)
            equal = false;

          for (int k=0; k<8; k++)
            if (tdm1->maxLSPbandwidth[k] != tdm2->maxLSPbandwidth[k])
              equal = false;

          if (equal) {
            count--;
            ind[i] = true;
          }

          break;

        case g2mplsTypes::SWITCHINGCAP_L2SC :
        case g2mplsTypes::SWITCHINGCAP_LSC  :
        case g2mplsTypes::SWITCHINGCAP_FSC  :

          gen1 = isc1->gen();
          gen2 = isc2->gen();

          equal = true;

          if (gen1->swCap != gen2->swCap)
            equal = false;
          if (gen1->encType != gen2->encType)
            equal = false;

          for (int k=0; k<8; k++)
            if (gen1->maxLSPbandwidth[k] != gen2->maxLSPbandwidth[k])
              equal = false;

          if (equal) {
            count--;
            ind[i] = true;
          }

          break;
        default :

          zlog_debug("[ERR] CORBA: method diffIscs (value out of range !!!)");
          break;
      }
    }
  }

  tmp = new g2mplsTypes::iscSeq(count);
  seq = tmp;
  seq->length(count);

  int index = 0;
  for (int m=0; m<iscs1.length(); m++)
    if (ind[m] == false)
  {
    seq[index] = iscs1[m];
    index++;
  }

  return seq;
}
