import os, re
from popen2 import popen2


call_ero_patt = re.compile('calleroitem: N=(?P<node>[^;]*)')
lsp_ero_patt = re.compile('lsperoitem: N=(?P<node>[^;]*)')
iTna_patt = re.compile('iTnaRes: TNA=ipv4#(?P<iTna>[^;]*)')
eTna_patt = re.compile('eTnaRes: TNA=ipv4#(?P<eTna>[^;]*)')

addresses = [
    '192.168.100.8',
    '192.168.100.9',
    '192.168.100.10',
    '192.168.100.11',
    '192.168.100.137',
    '192.168.100.138',
    '192.168.100.139',
]

def parse_single_path(lines):
    path = {'iTna':None, 'eTna':None, 'callEro':[], 'lspEro':[]}
    SEARCH_STATE = 'search_iTNA'
    
    for line in lines:
        if SEARCH_STATE == 'search_iTNA':
            match = iTna_patt.search(line)
            if match is None:
                continue
            path.update(match.groupdict())
            SEARCH_STATE = 'search_eTNA'
        
        elif SEARCH_STATE == 'search_eTNA':
            match = eTna_patt.search(line)
            if match is None:
                continue
            path.update(match.groupdict())
            SEARCH_STATE = 'search_callOrLspEro'
            
        elif SEARCH_STATE == 'search_callOrLspEro':
            match_call = call_ero_patt.search(line)
            if match_call:
                path['callEro'].append(match_call.groupdict()['node'])
                
            match_lsp = lsp_ero_patt.search(line)
            if match_lsp:
                path['lspEro'].append(match_lsp.groupdict()['node'])
    path['lspEro'] = [path['callEro'][0]] + path['lspEro']
    print ' - Path found:', path
    return path
    
def ispresent(path, pathes):
    for p in pathes:
        if p == path:
            return True
    return False

def generate_path_infos(info):
    pathes = []
    line_number = 0
    prev_start_line = -1
    lines = info.splitlines()
    for line in lines:
        # next call found or last line
        if (' call: CALLIDTYPE_' in line) or (line_number == len(lines)-1):
            if prev_start_line != -1: # skip first call header, trigger on the next
                #print 'Parsing next call'
                path = parse_single_path(lines[prev_start_line:line_number+1])

                if ispresent(path, pathes) is False:
                    pathes.append(path)
                    yield path
                
            prev_start_line = line_number
        line_number += 1

def generate_pathes():
    pathes = []
    raw_text = ""
    for ip_addr in addresses:
        print '\n', '-'*20, '\nCtrl Address', ip_addr, '\n'
        try:
            dire = "/home/user01/phosphorus-g2mpls/src"
            iors_dir = '/mnt/g2iors'
            stdin, stdout = os.popen2("python %s/pyg2mpls/g2utils/g2info.py -t %sg2mpls_modules/testsuite/G2MPLS-DEMOS/192.168.100.139.ADVA_3/routes.py %s %s" % (dire, dire, ip_addr, iors_dir))
            text = stdout.read()
            print text
            raw_text += text
            for path in generate_path_infos(text):
               if ispresent(path, pathes) is False:
                   pathes.append(path)
               else: 
	           pass
                   #print ' = Path already found, skip it:', path
        except Exception, e:
            from traceback import print_exc
            print_exc()

    return pathes, raw_text

class IO:
    def __init__(self):
        self.input = ''
    def write(self, data):
        self.input += data

def pathes_info(pathes):
    import pprint
    text_buffor = 'There are following pathes installed in G2MPLS Control Plane:\n'
    io = IO()
    for path in pathes:
        text_buffor += ' Path:\n'
        pprint.pprint(path, io)
        text_buffor += io.input
        
    print text_buffor

def send_pathes_info(addr, ncc_text, lrm_text):
    import httplib, urllib
    try:
    	params = urllib.urlencode({'pathes': ncc_text, 'telinks': lrm_text})
    	headers = {"Content-type": "application/x-www-form-urlencoded",
            	   "Accept": "text/plain"}
    	conn = httplib.HTTPConnection("%s:80" % addr)
    	conn.request("POST", "/testbed/phosphorus_testbed/fff/test3.php", params, headers)
    	response = conn.getresponse()
    	data = response.read()
    	#print 'http response is', response, data
    	conn.close()
    except Exception, e:
        from traceback import print_exc
        print_exc()


if __name__ == '__main__':
    import time
    from lrm_monitor import get_link_statuses
 
    while True:
    	pathes, ncc_text = generate_pathes()
	print ncc_text
    	#print 'There were found %i pathes' % len(pathes)

        lrm_text = " "  # = get_link_statuses() 

        send_pathes_info('150.254.171.133', ncc_text, lrm_text)

    	#pathes_info(pathes)
        time.sleep(1)

