//
//  This file is part of phosphorus-g2mpls.
//
//  Copyright (C) 2006, 2007, 2008, 2009 Nextworks s.r.l.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
//  Authors:
//
//  Giacomo Bernini       (Nextworks s.r.l.) <g.bernini_at_nextworks.it>
//  Gino Carrozzo         (Nextworks s.r.l.) <g.carrozzo_at_nextworks.it>
//  Nicola Ciulli         (Nextworks s.r.l.) <n.ciulli_at_nextworks.it>
//  Giodi Giorgi          (Nextworks s.r.l.) <g.giorgi_at_nextworks.it>
//  Francesco Salvestrini (Nextworks s.r.l.) <f.salvestrini_at_nextworks.it>
//



#include "log.h"
#include "zebra.h"

#if HAVE_OMNIORB

#include "lib/corba.h"
#include "g2pcera_trace.hh"
#include "g2pcera_corba_servant.hh"
#include "g2pcera_corba_utils.hh"
#include "g2pcera_api.hh"
#include "g2pcera_dumper.hh"

#include <iostream>
#include <string>
#include <stdlib.h>
#include <stdexcept>

using namespace std;


#include "g2mpls_corba_utils.h"
//
//
// g2mplsTopology_i
//
//
g2mplsTopology_i::~g2mplsTopology_i()
{
}

CORBA::Boolean
g2mplsTopology_i::nodeAdd(const g2mplsTypes::nodeIdent& id)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var     myId;
	std::string                    myIdString;
	pceraErrorCode_t               ret;
	node_ident_t                   ident;
	std::string                    err;

	try {
		myId = id;
		myIdString << myId;
		ident << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to add node %s into default topology "
		      "through CORBA g2mplsTopology interface",
		      myIdString.c_str());

	ret = PCERA_TOPO_API::nodeAdd(G2PCERA_DEFAULT_INSTANCE, ident, err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s added into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      myIdString.c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_ALREADY_EXISTS: {
			G2PCERA_DEBUG("Cannot add node %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      myIdString.c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::NodeAlreadyExists(myId,
								 err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot add node %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      myIdString.c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}


CORBA::Boolean
g2mplsTopology_i::nodeDel(const g2mplsTypes::nodeIdent& id)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var     myId;
	std::string                    myIdString;
	pceraErrorCode_t               ret;
	node_ident_t                   ident;
	std::string                    err;

	try {
		myId = id;
		myIdString << myId;
		ident << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to delete node %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      myIdString.c_str());

	ret = PCERA_TOPO_API::nodeDel(G2PCERA_DEFAULT_INSTANCE, ident, err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s deleted from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      myIdString.c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot delete node %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      myIdString.c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot delete node %s from topology in "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      myIdString.c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

g2mplsTypes::nodeIdentSeq*
g2mplsTopology_i::nodeGetAll()
{
	STACK_LOCK();
	METIN();
	size_t                  count;
	std::list<node_ident_t> tmp;
	std::string             err;

	G2PCERA_DEBUG("Someone is trying to get all the node identities "
		      "from default topology "
		      "through CORBA g2mplsTopology interface");

	tmp = PCERA_TOPO_API::nodeGetAll(G2PCERA_DEFAULT_INSTANCE, err);

	count = tmp.size();

	g2mplsTypes::nodeIdentSeq_var seq;
	{
		g2mplsTypes::nodeIdentSeq * tmp;
		tmp = new g2mplsTypes::nodeIdentSeq(count);
		if (!tmp) {
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       InternalProblems("Out of memory for "
						"g2mplsTypes::"
						"nodeIdentSeq"));
		}
		seq = tmp;
	}
	seq->length(count);

	if (count) {
		std::list<node_ident_t>::const_iterator iter;
		g2mplsTypes::nodeIdent_var              id;
		size_t                                  i;

		for (iter  = tmp.begin(), i = 0;
		     iter != tmp.end();
		     iter++, i++) {
			try {
				id << *iter;
			} catch(...) {
				G2PCERA_ERROR("Error on node_ident_t "
					      "conversions for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on "
							 "node_ident_t "
							 "conversions");
			}
			seq[i] = id;
		}
	}

	G2PCERA_DEBUG("Got all the node identities "
		      "from default topology "
		      "through CORBA g2mplsTopology interface");

	METOUT();
	STACK_UNLOCK();
	return seq._retn();
}

CORBA::Boolean
g2mplsTopology_i::netNodeUpdate(g2mplsTypes::nodeId                id,
				const g2mplsTypes::netNodeParams&  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var     myId;
	pceraErrorCode_t               ret;
	node_ident_t                   ident;
	net_node_data_t                data;
	std::string                    err;

	ident.type = TOPO_NODE_NETWORK;
	ident.id   = id;

	try {
		myId << ident;
	} catch (...) {
		G2PCERA_ERROR("Error on param conversions (id) for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on param "
						       "conversions (id)");
	}

	try {
		g2mplsTypes::netNodeParams_var tmp;
		tmp = new g2mplsTypes::netNodeParams(info);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update node %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      nodeIdent2string(ident).c_str());

	ret = PCERA_TOPO_API::netNodeUpdate(G2PCERA_DEFAULT_INSTANCE,
					    ident.id,
					    data,
					    err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      nodeIdent2string(ident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update node %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_NODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update node %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       NodeParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update node %s  into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::netNodeGet(g2mplsTypes::nodeId             id,
			     g2mplsTypes::netNodeParams_out  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var     myId;
	pceraErrorCode_t               ret;
	node_ident_t                   ident;
	net_node_data_t                data;
	std::string                    err;

	ident.type = TOPO_NODE_NETWORK;
	ident.id   = id;

	try {
		myId << ident;
	} catch (...) {
		G2PCERA_ERROR("Error on param conversions (id) for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on param "
						       "conversions (id)");
	}

	G2PCERA_DEBUG("Someone is trying to get params for node %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      nodeIdent2string(ident).c_str());

	ret = PCERA_TOPO_API::netNodeGet(G2PCERA_DEFAULT_INSTANCE,
					 ident.id,
					 data,
					 err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::netNodeParams_var tmp;
				tmp << data;
				info = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(info)");
			}
			G2PCERA_DEBUG("Got params for node %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      nodeIdent2string(ident).c_str());

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get params for node %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_NODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get params for node %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       NodeParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get params for node %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridSiteUpdate(g2mplsTypes::nodeId                 id,
				 const g2mplsTypes::gridSiteParams&  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var     myId;
	pceraErrorCode_t               ret;
	node_ident_t                   ident;
	grid_site_data_t               data;
	std::string                    err;

	ident.type = TOPO_NODE_GRID;
	ident.id   = id;

	try {
		myId << ident;
	} catch (...) {
		G2PCERA_ERROR("Error on param conversions (id) for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on param "
						       "conversions (id)");
	}

	try {
		g2mplsTypes::gridSiteParams_var tmp;
		tmp = new g2mplsTypes::gridSiteParams(info);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update node %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      nodeIdent2string(ident).c_str());

	ret = PCERA_TOPO_API::gridSiteUpdate(G2PCERA_DEFAULT_INSTANCE,
					     ident.id,
					     data,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      nodeIdent2string(ident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update node %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_NODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update node %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       NodeParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update node %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridSiteGet(g2mplsTypes::nodeId              id,
			      g2mplsTypes::gridSiteParams_out  info,
			      g2mplsTypes::gridSubNodes_out    snodes)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var     myId;
	pceraErrorCode_t               ret;
	node_ident_t                   ident;
	grid_site_data_t               data;
	grid_subnodes_t                subnodes;
	std::string                    err;

	ident.type = TOPO_NODE_GRID;
	ident.id   = id;

	try {
		myId << ident;
	} catch (...) {
		G2PCERA_ERROR("Error on param conversions (id) for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on param "
						       "conversions (id)");
	}

	G2PCERA_DEBUG("Someone is trying to get params for node %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      nodeIdent2string(ident).c_str());

	ret = PCERA_TOPO_API::gridSiteGet(G2PCERA_DEFAULT_INSTANCE,
					  ident.id,
					  data,
					  subnodes,
					  err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::gridSiteParams_var tmp;
				tmp << data;
				info = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(info)");
			}

			try {
				g2mplsTypes::gridSubNodes_var tmp;
				tmp << subnodes;
				snodes = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (snodes) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(snodes)");
			}
			G2PCERA_DEBUG("Got params for node %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      nodeIdent2string(ident).c_str());

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get params for node %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_NODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get params for node %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      nodeIdent2string(ident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       NodeParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get params for node %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridSubNodeDel(g2mplsTypes::nodeId                    siteId,
				 const g2mplsTypes::gridSubNodeIdent&   id)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	std::string                        err;

	ident.type = TOPO_NODE_GRID;
	ident.id   = siteId;

	try {
		myId << ident;
	} catch (...) {
		G2PCERA_ERROR("Error on param conversions (id) for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on param "
						       "conversions (id)");
	}

	try {
		mySId = id;
		sident << mySId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (id) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(id)");
	}

	G2PCERA_DEBUG("Someone is trying to delete subnode %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridSubNodeDel(G2PCERA_DEFAULT_INSTANCE,
					     ident.id,
					     sident,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Subnode %s deleted from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot delete subnode %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot delete subnode %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot delete %s subnode from topology in "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      subnodeIdent2string(sident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridServiceUpdate(g2mplsTypes::nodeId                 siteId,
				    g2mplsTypes::gridSubNodeId            id,
				    const g2mplsTypes::gridServiceParams& info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_service_data_t                data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_SERVICE;
	sident.id   = id;

	try {
		g2mplsTypes::gridServiceParams_var tmp;
		tmp = info;
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridServiceUpdate(G2PCERA_DEFAULT_INSTANCE,
						ident.id,
						sident.id,
						data,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Subnode %s updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridServiceGet(g2mplsTypes::nodeId              siteId,
				 g2mplsTypes::gridSubNodeId       id,
				 g2mplsTypes::gridServiceParams&  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_service_data_t                data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_SERVICE;
	sident.id   = id;

	G2PCERA_DEBUG("Someone is trying to get params for subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridServiceGet(G2PCERA_DEFAULT_INSTANCE,
					     ident.id,
					     sident.id,
					     data,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::gridServiceParams_var tmp;
				tmp << data;
				info = tmp;
			} catch (...) {
				G2PCERA_ERROR("Error on input param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("Got params for subnode %s into "
				      "topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot  get params for subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get params for subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridCompElemUpdate(g2mplsTypes::nodeId                siteId,
				     g2mplsTypes::gridSubNodeId         id,
				     const g2mplsTypes::gridCEParams&   info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_ce_data_t                     data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_COMPUTINGELEMENT;
	sident.id   = id;

	try {
		g2mplsTypes::gridCEParams_var tmp;
		tmp = new g2mplsTypes::gridCEParams(info);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridCompElemUpdate(G2PCERA_DEFAULT_INSTANCE,
						 ident.id,
						 sident.id,
						 data,
						 err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Subnode %s updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridCompElemGet(g2mplsTypes::nodeId           siteId,
				  g2mplsTypes::gridSubNodeId    id,
				  g2mplsTypes::gridCEParams_out info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_ce_data_t                     data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_COMPUTINGELEMENT;
	sident.id   = id;

	G2PCERA_DEBUG("Someone is trying to get params for subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridCompElemGet(G2PCERA_DEFAULT_INSTANCE,
					      ident.id,
					      sident.id,
					      data,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::gridCEParams_var tmp;
				tmp << data;
				info = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on input param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("Got params for subnode %s into "
				      "topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot  get params for subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get params for subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridSubClusterUpdate(g2mplsTypes::nodeId              siteId,
				       g2mplsTypes::gridSubNodeId       id,
				       const g2mplsTypes::
				       gridSubClusterParams&            info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_subcluster_data_t             data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_SUBCLUSTER;
	sident.id   = id;

	try {
		g2mplsTypes::gridSubClusterParams_var tmp;
		tmp = new g2mplsTypes::gridSubClusterParams(info);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridSubClusterUpdate(G2PCERA_DEFAULT_INSTANCE,
						   ident.id,
						   sident.id,
						   data,
						   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Subnode %s updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridSubClusterGet(g2mplsTypes::nodeId                 siteId,
				    g2mplsTypes::gridSubNodeId          id,
				    g2mplsTypes::gridSubClusterParams_out info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_subcluster_data_t             data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_SUBCLUSTER;
	sident.id   = id;

	G2PCERA_DEBUG("Someone is trying to get params for subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridSubClusterGet(G2PCERA_DEFAULT_INSTANCE,
						ident.id,
						sident.id,
						data,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::gridSubClusterParams_var tmp;
				tmp << data;
				info = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on input param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("Got params for subnode %s into "
				      "topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot  get params for subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get params for subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridStorageElemUpdate(g2mplsTypes::nodeId             siteId,
					g2mplsTypes::gridSubNodeId      id,
					const g2mplsTypes::gridSEParams& info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_se_data_t                     data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_STORAGEELEMENT;
	sident.id   = id;

	try {
		g2mplsTypes::gridSEParams_var tmp;
		tmp = new g2mplsTypes::gridSEParams(info);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridStorageElemUpdate(G2PCERA_DEFAULT_INSTANCE,
						    ident.id,
						    sident.id,
						    data,
						    err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Subnode %s updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update subnode %s into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::gridStorageElemGet(g2mplsTypes::nodeId            siteId,
				     g2mplsTypes::gridSubNodeId     id,
				     g2mplsTypes::gridSEParams_out  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var         myId;
	g2mplsTypes::gridSubNodeIdent_var  mySId;
	pceraErrorCode_t                   ret;
	node_ident_t                       ident;
	grid_subnode_ident_t               sident;
	grid_se_data_t                     data;
	std::string                        err;

	ident.type  = TOPO_NODE_GRID;
	ident.id    = siteId;

	sident.type = GRIDSUBNODE_STORAGEELEMENT;
	sident.id   = id;

	G2PCERA_DEBUG("Someone is trying to get params for subnode %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      subnodeIdent2string(sident).c_str());

	ret = PCERA_TOPO_API::gridStorageElemGet(G2PCERA_DEFAULT_INSTANCE,
						 ident.id,
						 sident.id,
						 data,
						 err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::gridSEParams_var tmp;
				tmp << data;
				info = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on input param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on input "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("Got params for subnode %s into "
				      "topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      subnodeIdent2string(sident).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_NODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchNode(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot  get params for subnode %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       CannotFetchSubNode(myId,
						  mySId,
						  err.c_str()));
		}
			break;
		case PCERA_ERROR_SUBNODE_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get params for subnode %s "
				      "into topology "
				      "in G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      subnodeIdent2string(sident).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       SubNodeParamsMismatch(myId,
						     mySId,
						     err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get params for subnode %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      nodeIdent2string(ident).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::tnaIdAdd(const g2mplsTypes::tnaIdent&  ident)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::tnaIdent_var     myId;
	pceraErrorCode_t              ret;
	tna_ident_t                   tna;
	std::string                   err;

	try {
		myId = ident;
		tna << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to add TNA %s into default topology "
		      "through CORBA g2mplsTopology interface",
		      tnaIdent2string(tna).c_str());

	ret = PCERA_TOPO_API::tnaAdd(G2PCERA_DEFAULT_INSTANCE, tna, err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("TNA %s added into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      tnaIdent2string(tna).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_TNA_ALREADY_EXISTS: {
			G2PCERA_DEBUG("Cannot add TNA %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      tnaIdent2string(tna).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::TnaAlreadyExists(myId,
								err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot add TNA %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      tnaIdent2string(tna).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::tnaIdDel(const g2mplsTypes::tnaIdent&  ident)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::tnaIdent_var     myId;
	pceraErrorCode_t              ret;
	tna_ident_t                   tna;
	std::string                   err;

	try {
		myId = ident;
		tna << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to delete TNA %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      tnaIdent2string(tna).c_str());

	ret = PCERA_TOPO_API::tnaDel(G2PCERA_DEFAULT_INSTANCE, tna, err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("TNA %s deleted from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      tnaIdent2string(tna).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_TNA_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot delete TNA %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      tnaIdent2string(tna).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchTna(myId,
							      err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot delete TNA %s from topology in "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      tnaIdent2string(tna).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

g2mplsTypes::tnaIdentSeq *
g2mplsTopology_i::tnaIdsGetAllFromNode(g2mplsTypes::nodeId  nodeId,
				       CORBA::Boolean       isDomain)
{
	STACK_LOCK();
	METIN();

	size_t                  count;
	node_ident_t            ident;
	std::list<tna_ident_t>  tmp;
	std::string             err;

	ident.type = TOPO_NODE_NETWORK;
	ident.id   = nodeId;

	G2PCERA_DEBUG("Someone is trying to get all the node identities "
		      "from default topology "
		      "through CORBA g2mplsTopology interface");

	tmp = PCERA_TOPO_API::tnaGetAllFromNode(G2PCERA_DEFAULT_INSTANCE,
						nodeId,
						isDomain,
						err);

	count = tmp.size();

	g2mplsTypes::tnaIdentSeq_var seq;
	{
		g2mplsTypes::tnaIdentSeq * tmp;
		tmp = new g2mplsTypes::tnaIdentSeq(count);
		if (!tmp) {
			METOUT();
			STACK_UNLOCK();
			throw(g2mplsTopology::
			      InternalProblems("Out of memory for "
					       "g2mplsTypes::"
					       "tnaIdentSeq"));
		}
		seq = tmp;
	}
	seq->length(count);

	if (count) {
		std::list<tna_ident_t>::const_iterator iter;
		g2mplsTypes::tnaIdent_var              id;
		size_t                                 i;

		for (iter  = tmp.begin(), i = 0;
		     iter != tmp.end();
		     iter++, i++) {
			try {
				id << *iter;
			} catch(...) {
				G2PCERA_ERROR("Error on tna_ident_t "
					      "conversions for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw (g2mplsTopology::
				       InternalProblems("Error on tna_ident_t "
							"conversions for"));
			}
			seq[i] = id;
		}
	}

	G2PCERA_DEBUG("Got all the TNA identities from node %s "
		      "in default topology "
		      "through CORBA g2mplsTopology interface",
		      nodeIdent2string(ident).c_str());

	METOUT();
	STACK_UNLOCK();
	return seq._retn();
}

CORBA::Boolean
g2mplsTopology_i::linkAdd(const g2mplsTypes::teLinkIdent&   ident)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var  myId;
	pceraErrorCode_t              ret;
	telink_ident_t                tel;
	std::string                   err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to add TE-LINK %s into "
		      "default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::linkAdd(G2PCERA_DEFAULT_INSTANCE, tel, err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("TE-LINK %s added into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_ALREADY_EXISTS: {
			G2PCERA_DEBUG("Cannot add TE-LINK %s into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::LinkAlreadyExists(myId,
								 err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot add TE-LINK %s into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::linkDel(const g2mplsTypes::teLinkIdent&   ident)
{
	STACK_LOCK();
	METIN();
	g2mplsTypes::teLinkIdent_var  myId;
	pceraErrorCode_t              ret;
	telink_ident_t                tel;
	std::string                   err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to delete TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::linkDel(G2PCERA_DEFAULT_INSTANCE, tel, err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("TE-LINK %s deleted from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot delete TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot delete TE-LINK %s from topology in "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

g2mplsTypes::teLinkIdentSeq*
g2mplsTopology_i::teLinkGetAllFromNode(const g2mplsTypes::nodeIdent&  ident)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeIdent_var myId;
	size_t                     count;
	node_ident_t               id;
	std::list<telink_ident_t>  tmp;
	std::string                err;

	try {
		myId = ident;
		id << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions");
	}

	G2PCERA_DEBUG("Someone is trying to get all the node identities "
		      "from default topology "
		      "through CORBA g2mplsTopology interface");

	tmp = PCERA_TOPO_API::teLinkGetAllFromNode(G2PCERA_DEFAULT_INSTANCE,
						   id.id,
						   err);

	count = tmp.size();

	g2mplsTypes::teLinkIdentSeq_var seq;
	{
		g2mplsTypes::teLinkIdentSeq * tmp;
		tmp = new g2mplsTypes::teLinkIdentSeq(count);
		if (!tmp) {
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       InternalProblems("Out of memory for "
						"g2mplsTypes::"
						"teLinkIdentSeq"));
		}
		seq = tmp;
	}
	seq->length(count);

	if (count) {
		std::list<telink_ident_t>::const_iterator iter;
		g2mplsTypes::teLinkIdent_var              id;
		size_t                                    i;

		for (iter  = tmp.begin(), i = 0;
		     iter != tmp.end();
		     iter++, i++) {
			try {
				id << *iter;
			} catch(...) {
				G2PCERA_ERROR("Error on telink_ident_t "
					      "conversions for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on "
							 "telink_ident_t "
							 "conversions");
			}
			seq[i] = id;
		}
	}

	G2PCERA_DEBUG("Got all the TE-LINK identities from node %s "
		      "in default topology "
		      "through CORBA g2mplsTopology interface",
		      nodeIdent2string(id).c_str());

	METOUT();
	STACK_UNLOCK();
	return seq._retn();
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateCom(const g2mplsTypes::teLinkIdent&      ident,
				  const g2mplsTypes::teLinkComParams&  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::teLinkComParams_var myInfo;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	telink_com_data_t                data;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		myInfo = info;
		data << myInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update COM params of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateCom(G2PCERA_DEFAULT_INSTANCE,
					      tel,
					      data,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("COM params of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update COM params of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update COM params of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update COM params of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetCom(const g2mplsTypes::teLinkIdent&  ident,
			       g2mplsTypes::teLinkComParams&    info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::teLinkComParams_var myInfo;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	telink_com_data_t                data;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get COM params of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetCom(G2PCERA_DEFAULT_INSTANCE,
					   tel,
					   data,
					   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				myInfo << data;
				info = myInfo;
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("COM params of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get COM params of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get COM params of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get COM params of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateTdm(const g2mplsTypes::teLinkIdent&      ident,
				  const g2mplsTypes::teLinkTdmParams&  info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::teLinkTdmParams_var myInfo;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	telink_tdm_data_t                data;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		myInfo = info;
		data << myInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update TDM params of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateTdm(G2PCERA_DEFAULT_INSTANCE,
					      tel,
					      data,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("TDM params of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update TDM params of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update TDM params of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update TDM params of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetTdm(const g2mplsTypes::teLinkIdent&  ident,
			       g2mplsTypes::teLinkTdmParams&    info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::teLinkTdmParams_var myInfo;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	telink_tdm_data_t                data;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get TDM params of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetTdm(G2PCERA_DEFAULT_INSTANCE,
					   tel,
					   data,
					   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				myInfo << data;
				info = myInfo;
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("TDM params of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get TDM params of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get TDM params of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get TDM params of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateLscG709(const g2mplsTypes::teLinkIdent&   ident,
				      const g2mplsTypes::
				      teLinkLscG709Params&              info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var         myId;
	g2mplsTypes::teLinkLscG709Params_var myInfo;
	pceraErrorCode_t                     ret;
	telink_ident_t                       tel;
	telink_lscg709_data_t                data;
	std::string                          err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		myInfo = info;
		data << myInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update LSC-G709 params "
		      "of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateLscG709(G2PCERA_DEFAULT_INSTANCE,
						  tel,
						  data,
						  err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("LSC-G709 params of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update LSC-G709 params "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update LSC-G709 params "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update LSC-G709 params of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetLscG709(const g2mplsTypes::teLinkIdent&      ident,
				   g2mplsTypes::teLinkLscG709Params_out info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var         myId;
	g2mplsTypes::teLinkLscG709Params_var myInfo;
	pceraErrorCode_t                     ret;
	telink_ident_t                       tel;
	telink_lscg709_data_t                data;
	std::string                          err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get LSC-G709 params of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetLscG709(G2PCERA_DEFAULT_INSTANCE,
					       tel,
					       data,
					       err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				myInfo << data;
				info = myInfo;
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("LSC-G709 params of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get LSC-G709 params "
				      "of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get LSC-G709 params "
				      "of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get LSC-G709 params of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateLscWdm(const g2mplsTypes::teLinkIdent&    ident,
				     const g2mplsTypes::
				     teLinkLscWdmParams&                info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var         myId;
	pceraErrorCode_t                     ret;
	telink_ident_t                       tel;
	telink_lscwdm_data_t                 data;
	std::string                          err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		g2mplsTypes::teLinkLscWdmParams_var tmp;
		tmp = new g2mplsTypes::teLinkLscWdmParams(info);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (info) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(info)");
	}

	G2PCERA_DEBUG("Someone is trying to update LSC-WDM params "
		      "of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateLscWdm(G2PCERA_DEFAULT_INSTANCE,
						 tel,
						 data,
						 err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("LSC-WDM params of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update LSC-WDM params "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update LSC-WDM params "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update LSC-WDM params of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetLscWdm(const g2mplsTypes::teLinkIdent&       ident,
				  g2mplsTypes::teLinkLscWdmParams_out   info)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var         myId;
	pceraErrorCode_t                     ret;
	telink_ident_t                       tel;
	telink_lscwdm_data_t                data;
	std::string                          err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get LSC-WDM params of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetLscWdm(G2PCERA_DEFAULT_INSTANCE,
					      tel,
					      data,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::teLinkLscWdmParams_var tmp;
				tmp << data;
				info = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("LSC-WDM params of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get LSC-WDM params "
				      "of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get LSC-WDM params "
				      "of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get LSC-WDM params of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateStates(const g2mplsTypes::teLinkIdent&    ident,
				     const g2mplsTypes::statesBundle&   states)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::statesBundle_var    myStates;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	opstate_t                        opState;
	admstate_t                       admState;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		myStates = states;
		opState  << myStates->opState;
		admState << myStates->admState;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (states) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(states)");
	}

	G2PCERA_DEBUG("Someone is trying to update states of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateStates(G2PCERA_DEFAULT_INSTANCE,
						 tel,
						 opState,
						 admState,
						 err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("States of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update states of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update states of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update states of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetStates(const g2mplsTypes::teLinkIdent&  ident,
				  g2mplsTypes::statesBundle&       states)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::statesBundle_var    myStates;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	opstate_t                        opState;
	admstate_t                       admState;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get states of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetStates(G2PCERA_DEFAULT_INSTANCE,
					      tel,
					      opState,
					      admState,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				myStates->opState  << opState;
				myStates->admState << admState;
				states = myStates;
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("states of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get states of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get states of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get states of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateGenBw(const g2mplsTypes::teLinkIdent&     ident,
				    const g2mplsTypes::availBwPerPrio   bw)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	bw_per_prio_t                    abw;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		size_t i;

		for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
			abw.bw[i] = bw[i];
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (bw) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(bw)");
	}

	G2PCERA_DEBUG("Someone is trying to update gen. bw of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateGenBw(G2PCERA_DEFAULT_INSTANCE,
						tel,
						abw,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Gen. Bw of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update gen. bw of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update gen. bw of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update gen. bw of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetGenBw(const g2mplsTypes::teLinkIdent&   ident,
				 g2mplsTypes::availBwPerPrio       bw)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	g2mplsTypes::availBwPerPrio_var  myBw;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	bw_per_prio_t                    abw;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get gen. bw of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetGenBw(G2PCERA_DEFAULT_INSTANCE,
					     tel,
					     abw,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				size_t i;

				for (i = 0; i < GMPLS_BW_PRIORITIES; i++) {
					bw[i] = abw.bw[i];
				}

			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (bw) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(bw)");
			}

			G2PCERA_DEBUG("Gen. bw of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get gen. bw of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get gen. bw of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get gen. bw of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateTdmBw(const g2mplsTypes::teLinkIdent&  ident,
				    const g2mplsTypes::freeCTPSeq&   freeTS)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<uint32_t>              freeCTP;
	std::string                      err;

	freeCTP.clear();

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		size_t i;

		for (i = 0; i < freeTS.length(); i++) {
			uint32_t ftsEntry = 0;
			ftsEntry |= ((freeTS[i].sigType & 0x000000FF) << 24);
			ftsEntry |= ((freeTS[i].ctps    & 0x00FFFFFF) <<  0);

			freeCTP.push_back(ftsEntry);
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (freeTS) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(freeTS)");
	}

	G2PCERA_DEBUG("Someone is trying to update TDM bw of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateTdmBw(G2PCERA_DEFAULT_INSTANCE,
						tel,
						freeCTP,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("TDM bw of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update TDM bw of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update TDM bw of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update TDM bw of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetTdmBw(const g2mplsTypes::teLinkIdent&  ident,
				 g2mplsTypes::freeCTPSeq_out      freeTS)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<uint32_t>              freeCTP;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get TDM bw of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetTdmBw(G2PCERA_DEFAULT_INSTANCE,
					     tel,
					     freeCTP,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::freeCTPSeq *   tmp;
				size_t                      size;

				size = freeCTP.size();

				tmp = new g2mplsTypes::freeCTPSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw g2mplsTopology::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "freeCTPSeq");
				}
				freeTS = tmp;
				freeTS->length(size);

				if (size) {
					std::list<uint32_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = freeCTP.begin(), i = 0;
					     iter != freeCTP.end();
					     iter++, i++) {
						freeTS[i].sigType =
							((*iter & 0xFF000000)>>
							 24);
						freeTS[i].ctps =
							((*iter & 0x00FFFFFF)>>
							 0);
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (freeTS) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(freeCTP)");
			}

			G2PCERA_DEBUG("TDM bw of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get TDM bw of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get TDM bw of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get TDM bw of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateLscG709Bw(const g2mplsTypes::teLinkIdent& ident,
					const g2mplsTypes::
					freeCTPSeq&                   freeODUk,
					const g2mplsTypes::
					freeCTPSeq&                   freeOCh)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<uint32_t>              free_oduk;
	std::list<uint32_t>              free_och;
	std::string                      err;

	free_oduk.clear();
	free_och.clear();

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		size_t i;

		for (i = 0; i < freeODUk.length(); i++) {
			uint32_t ftsEntry;
			ftsEntry |= ((freeODUk[i].sigType & 0x000000FF) << 24);
			ftsEntry |= ((freeODUk[i].ctps    & 0x00FFFFFF) <<  0);

			free_oduk.push_back(ftsEntry);
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (freeODUk) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(freeODUk)");
	}

	try {
		size_t i;

		for (i = 0; i < freeOCh.length(); i++) {
			uint32_t ftsEntry;
			ftsEntry |= ((freeOCh[i].sigType & 0x000000FF) << 24);
			ftsEntry |= ((freeOCh[i].ctps    & 0x00FFFFFF) <<  0);

			free_och.push_back(ftsEntry);
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (freeOCh) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(freeOCh)");
	}

	G2PCERA_DEBUG("Someone is trying to update G709 bw of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateLscG709Bw(G2PCERA_DEFAULT_INSTANCE,
						    tel,
						    free_oduk,
						    free_och,
						    err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("G709 bw of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update G709 bw of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update G709 bw of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update G709 bw of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetLscG709Bw(const g2mplsTypes::teLinkIdent&  ident,
				     g2mplsTypes::freeCTPSeq_out      freeODUk,
				     g2mplsTypes::freeCTPSeq_out      freeOCh)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<uint32_t>              free_oduk;
	std::list<uint32_t>              free_och;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get G709 bw of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetLscG709Bw(G2PCERA_DEFAULT_INSTANCE,
						 tel,
						 free_oduk,
						 free_och,
						 err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::freeCTPSeq *   tmp;
				size_t                      size;

				size = free_oduk.size();

				tmp = new g2mplsTypes::freeCTPSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw g2mplsTopology::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "freeCTPSeq");
				}
				freeODUk = tmp;
				freeODUk->length(size);

				if (size) {
					std::list<uint32_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = free_oduk.begin(), i = 0;
					     iter != free_oduk.end();
					     iter++, i++) {
						freeODUk[i].sigType =
							((*iter & 0xFF000000)>>
							 24);
						freeODUk[i].ctps =
							((*iter & 0x00FFFFFF)>>
							 0);
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (freeODUk) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(freeODUk)");
			}

			try {
				g2mplsTypes::freeCTPSeq *   tmp;
				size_t                      size;

				size = free_och.size();

				tmp = new g2mplsTypes::freeCTPSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw g2mplsTopology::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "freeCTPSeq");
				}
				freeOCh = tmp;
				freeOCh->length(size);

				if (size) {
					std::list<uint32_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = free_och.begin(), i = 0;
					     iter != free_och.end();
					     iter++, i++) {
						freeOCh[i].sigType =
							((*iter & 0xFF000000)>>
							 24);
						freeOCh[i].ctps =
							((*iter & 0x00FFFFFF)>>
							 0);
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (freeOCh) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(freeOCh)");
			}

			G2PCERA_DEBUG("G709 bw of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get G709 bw of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get G709 bw of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get G709 bw of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkUpdateLscWdmBw(const g2mplsTypes::teLinkIdent&  ident,
				       const g2mplsTypes::
				             teLinkWdmLambdasBitmap&    bm)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var            myId;
	pceraErrorCode_t                        ret;
	telink_ident_t                          tel;
	wdm_link_lambdas_bitmap_t               data;
	std::string                             err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		g2mplsTypes::teLinkWdmLambdasBitmap_var tmp;
		tmp = new  g2mplsTypes::teLinkWdmLambdasBitmap(bm);
		data << tmp;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (bm) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(bm)");
	}

	G2PCERA_DEBUG("Someone is trying to update WDM lambda bitmap "
		      "of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkUpdateLscWdmBw(G2PCERA_DEFAULT_INSTANCE,
						   tel,
						   data,
						   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("WDM lambda bitmap of TE-LINK %s "
				      "updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update WDM lambda bitmap "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update WDM lambda bitmap "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update WDM lambda bitmap of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetLscWdmBw(const g2mplsTypes::teLinkIdent&     ident,
				    g2mplsTypes::teLinkWdmLambdasBitmap_out bm)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	wdm_link_lambdas_bitmap_t        data;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get WDM lambda bitmap "
		      "of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetLscWdmBw(G2PCERA_DEFAULT_INSTANCE,
						tel,
						data,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::wdmLambdasBitmap_var tmp;
				tmp << data;
				bm = tmp._retn();
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (info) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(info)");
			}

			G2PCERA_DEBUG("WDM lambda bitmap "
				      "of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get WDM lambda bitmap "
				      "of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get WDM lambda bitmap "
				      "of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get WDM lambda bitmap "
		      "of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkAppendSrlgs(const g2mplsTypes::teLinkIdent&  ident,
				    const g2mplsTypes::srlgSeq&      srlgs)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<uint32_t>              isrlgs;
	std::string                      err;

	isrlgs.clear();

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		size_t i;

		for (i = 0; i < srlgs.length(); i++) {
			isrlgs.push_back(srlgs[i]);
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (srlgs) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(srlgs)");
	}

	G2PCERA_DEBUG("Someone is trying to update SRLGs of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkAppendSrlgs(G2PCERA_DEFAULT_INSTANCE,
						tel,
						isrlgs,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("SRLGs of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update SRLGs of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update SRLGs of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update SRLGs of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetSrlgs(const g2mplsTypes::teLinkIdent&   ident,
				 g2mplsTypes::srlgSeq_out          srlgs)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<uint32_t>              isrlgs;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get SRLGs of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetSrlgs(G2PCERA_DEFAULT_INSTANCE,
					     tel,
					     isrlgs,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::srlgSeq *   tmp;
				size_t                   size;

				size = isrlgs.size();

				tmp = new g2mplsTypes::srlgSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw g2mplsTopology::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "srlgSeq");
				}
				srlgs = tmp;
				srlgs->length(size);

				if (size) {
					std::list<uint32_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = isrlgs.begin(), i = 0;
					     iter != isrlgs.end();
					     iter++, i++) {
						srlgs[i] = *iter;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (srlgs) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(srlgs)");
			}

			G2PCERA_DEBUG("SRLGs of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get SRLGs of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get SRLGs of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get SRLGs of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkAppendCalendar(const g2mplsTypes::teLinkIdent&  ident,
				       const g2mplsTypes::
				       teLinkCalendarSeq&               cal)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::map<uint32_t,bw_per_prio_t> ical;
	std::string                      err;

	ical.clear();

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		size_t i;

		for (i = 0; i < cal.length(); i++) {
			uint32_t        unixTime;
			bw_per_prio_t   abw;
			size_t          j;

			unixTime = cal[i].unixTime;

			for (j = 0; j < GMPLS_BW_PRIORITIES; j++) {
				abw.bw[j] = cal[i].availBw[j];
			}
			ical[unixTime] = abw;
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (cal) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(cal)");
	}

	G2PCERA_DEBUG("Someone is trying to update calendar of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkAppendCalendar(G2PCERA_DEFAULT_INSTANCE,
						   tel,
						   ical,
						   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Calendar of TE-LINK %s updated "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update calendar of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot update calendar of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update calendar of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetCalendar(const g2mplsTypes::teLinkIdent&     ident,
				    g2mplsTypes::teLinkCalendarSeq_out  cal)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::map<uint32_t,bw_per_prio_t> ical;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get calendar of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetCalendar(G2PCERA_DEFAULT_INSTANCE,
						tel,
						ical,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::teLinkCalendarSeq * tmp;
				size_t                           size;

				size = ical.size();

				tmp = new g2mplsTypes::teLinkCalendarSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw g2mplsTopology::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "teLink"
								 "CalendarSeq");
				}
				cal = tmp;
				cal->length(size);

				if (size) {
					std::map<uint32_t,bw_per_prio_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = ical.begin(), i = 0;
					     iter != ical.end();
					     iter++, i++) {
						cal[i].unixTime =
							(*iter).first;
						size_t  j;
						for (j = 0;
						     j < GMPLS_BW_PRIORITIES;
						     j++) {
							cal[i].availBw[j] =
								(*iter).
								second.bw[j];
						}
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (cal) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(cal)");
			}

			G2PCERA_DEBUG("Calendar of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get calendar of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get calendar of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get calendar of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkAppendIsc(const g2mplsTypes::teLinkIdent&  ident,
				  const g2mplsTypes::iscSeq&       iscs)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var  myId;
	pceraErrorCode_t              ret;
	telink_ident_t                tel;
	std::list<isc_t>              data;
	std::string                   err;

	data.clear();

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	try {
		size_t i;

		for (i = 0; i < iscs.length(); i++) {
			isc_t                tmp;
			g2mplsTypes::isc_var tmp1;
			tmp1 = iscs[i];
			tmp << tmp1;
			data.push_back(tmp);
		}
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions "
			      "(iscs) for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(iscs)");
	}

	G2PCERA_DEBUG("Someone is trying to update ISC list "
		      "of TE-LINK %s "
		      "into default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkAppendIsc(G2PCERA_DEFAULT_INSTANCE,
					      tel,
					      data,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("ISC list of TE-LINK %s "
				      "updated into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot update ISC list "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot updateISC list "
				      "of TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot update ISC list of TE-LINK %s "
		      "into topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}

CORBA::Boolean
g2mplsTopology_i::teLinkGetIsc(const g2mplsTypes::teLinkIdent&  ident,
			       g2mplsTypes::iscSeq_out          iscs)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::teLinkIdent_var     myId;
	pceraErrorCode_t                 ret;
	telink_ident_t                   tel;
	std::list<isc_t>                 data;
	std::string                      err;

	try {
		myId = ident;
		tel << myId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (ident) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw g2mplsTopology::InternalProblems("Error on input "
						       "param conversions "
						       "(ident)");
	}

	G2PCERA_DEBUG("Someone is trying to get ISC list of TE-LINK %s "
		      "from default topology "
		      "through CORBA g2mplsTopology interface",
		      teLinkIdent2string(tel).c_str());

	ret = PCERA_TOPO_API::teLinkGetIsc(G2PCERA_DEFAULT_INSTANCE,
					   tel,
					   data,
					   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::iscSeq *   tmp;
				size_t                  size;

				size = data.size();

				tmp = new g2mplsTypes::iscSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw g2mplsTopology::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "iscSeq");
				}
				iscs = tmp;
				iscs->length(size);

				if (size) {
					std::list<isc_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = data.begin(), i = 0;
					     iter != data.end();
					     iter++, i++) {
						g2mplsTypes::isc_var tmp;
						tmp << *iter;
						iscs[i] = tmp._retn();
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (iscs) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw g2mplsTopology::
					InternalProblems("Error on output "
							 "param conversions "
							 "(iscs)");
			}

			G2PCERA_DEBUG("ISC list of TE-LINK %s got "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA g2mplsTopology interface",
				      teLinkIdent2string(tel).c_str());
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get ISC list of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::CannotFetchLink(myId,
							       err.c_str()));
		}
			break;
		case PCERA_ERROR_LINK_PARAMS_MISMATCH: {
			G2PCERA_DEBUG("Cannot get ISC list of TE-LINK %s "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      teLinkIdent2string(tel).c_str(),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (g2mplsTopology::
			       LinkParamsMismatch(myId,
						  err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get ISC list of TE-LINK %s "
		      "from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      teLinkIdent2string(tel).c_str(),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (g2mplsTopology::InternalProblems(err.c_str()));
}


//
//
//  G2PCERA_i
//
//

G2PCERA_i::~G2PCERA_i(void)
{
}

CORBA::Boolean
G2PCERA_i::nodeGetFromTnaRes(const g2mplsTypes::tnaResource&    tnaRes,
			     g2mplsTypes::nodeId&               node)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::tnaResource_var  myTnaRes;
	pceraErrorCode_t              ret;
	net_res_spec_t                tna_res;
	uint32_t                      rId;
	std::string                   err;

	node = 0;

	try {
		myTnaRes = tnaRes;
		tna_res << myTnaRes;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (tnaRes) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(tnaRes)");
	}

	G2PCERA_DEBUG("Someone is trying to get node from a TNA Res "
		      "into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::nodeGetFromTnaRes(G2PCERA_DEFAULT_INSTANCE,
						tna_res,
						rId,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s got from TNA res "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface",
				      numb_htoa(rId));
			node = rId;
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_TNA_NOT_FOUND: {
			g2mplsTypes::tnaId_var myId;

			try {
				myId << tna_res.tna;
			} catch (...) {
				G2PCERA_ERROR("Error on input param "
					      "conversions (tna) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					InternalProblems("Error on input "
							 "param conversions "
							 "(tna)");
			}
			G2PCERA_DEBUG("Cannot get node from TNA res "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchTna(myId,
						       err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get node from TNA res from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

g2mplsTypes::nodeId
G2PCERA_i::nodeGetFromTna(const g2mplsTypes::tnaId& tna)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeId     tmp;
	g2mplsTypes::tnaId_var  myTna;
	pceraErrorCode_t        ret;
	g2mpls_addr_t           itna;
	uint32_t                rId;
	std::string             err;

	tmp = 0;

	try {
		myTna = tna;
		itna << myTna;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (tna) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(tna)");
	}

	G2PCERA_DEBUG("Someone is trying to get node from a TNA %s "
		      "into default topology "
		      "through CORBA G2PCERA interface",
		      addr_ntoa(itna));

	ret = PCERA_CALC_API::nodeGetFromTna(G2PCERA_DEFAULT_INSTANCE,
					     itna,
					     rId,
					     err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s got from TNA "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface",
				      numb_htoa(rId));
			tmp = rId;
			METOUT();
			STACK_UNLOCK();
			return tmp;
		}
			break;
		case PCERA_ERROR_TNA_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get node from TNA %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      addr_ntoa(itna),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchTna(myTna,
						       err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get node from TNA %s from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      addr_ntoa(itna),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::nodeGetFromGnsTna(const g2mplsTypes::gridParams&     tnaGnsRes,
			     const g2mplsTypes::nodeIdentSeq&   excludeSet,
			     g2mplsTypes::nodeId&               netNodeId,
			     g2mplsTypes::nodeId&               gridSiteId)
{
	STACK_LOCK();
	METIN();

	pceraErrorCode_t              ret;
	grid_res_spec_t               gns_res;
	std::list<uint32_t>           exclude_set;
	uint32_t                      rId;
	uint32_t                      gridId;
	std::string                   err;

	exclude_set.clear();

	netNodeId  = 0;
	gridSiteId = 0;

	try {
		gns_res << tnaGnsRes;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (tnaGnsRes) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(tnaGnsRes)");
	}

	try {
		size_t i;

		for (i = 0; i < excludeSet.length(); i++) {
			exclude_set.push_back(excludeSet[i].id);
		}
	} catch(...) {
		G2PCERA_ERROR("Error on input param conversions (excludeSet) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(excludeSet)");
	}

	G2PCERA_DEBUG("Someone is trying to get node from a GNS TNA "
		      "into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::nodeGetFromGnsTna(G2PCERA_DEFAULT_INSTANCE,
						gns_res,
						exclude_set,
						rId,
						gridId,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s got from GNS TNA "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface",
				      numb_htoa(rId));
			netNodeId  = rId;
			gridSiteId = gridId;

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get node from GNS TNA from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::nodeGetFromTELink(const g2mplsTypes::TELinkId& teLink,
			     g2mplsTypes::nodeId&         node,
			     g2mplsTypes::nodeId&         adjNode)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::nodeId        tmp;
	g2mplsTypes::TELinkId_var  myTel;
	pceraErrorCode_t           ret;
	g2mpls_addr_t              tel;
	uint32_t                   rId;
	uint32_t                   adjRid;
	std::string                err;

	node    = 0;
	adjNode = 0;

	try {
		myTel = teLink;
		tel << myTel;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (teLink) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(teLink)");
	}

	G2PCERA_DEBUG("Someone is trying to get node from TE-LINK %s "
		      "into default topology "
		      "through CORBA G2PCERA interface",
		      addr_ntoa(tel));

	ret = PCERA_CALC_API::nodeGetFromTeLink(G2PCERA_DEFAULT_INSTANCE,
						tel,
						rId,
						adjRid,
						err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Node %s got from TE-LINK "
				      "from topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface",
				      numb_htoa(rId));
			node    = rId;
			adjNode = adjRid;

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_LINK_NOT_FOUND: {
			G2PCERA_DEBUG("Cannot get node from TE-LINK %s "
				      "into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      addr_ntoa(tel),
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchLink(myTel,
							err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Cannot get node from TE-LINK %s from topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      addr_ntoa(tel),
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::callRoute(const g2mplsTypes::tnaResource&      src,
		     const g2mplsTypes::tnaResource&      dst,
		     const g2mplsTypes::callIdent&        callId,
		     const g2mplsTypes::callParams&       callInfo,
		     const g2mplsTypes::recoveryParams&   recInfo,
		     const g2mplsTypes::lspParams&        lspInfo,
		     g2mplsTypes::eroSeq_out              wEro)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::tnaResource_var      myTnaSrc;
	g2mplsTypes::tnaResource_var      myTnaDst;
	g2mplsTypes::callIdent_var        myCallId;
	g2mplsTypes::recoveryParams_var   myRecInfo;
	g2mplsTypes::lspParams_var        myLspInfo;

	pceraErrorCode_t                  ret;
	net_res_spec_t                    isrc;
	net_res_spec_t                    idst;
	call_ident_t                      icall_id;
	call_info_t                       icall_info;
	recovery_info_t                   irec_info;
	lsp_info_t                        ilsp_info;
	std::list<ero_hop_t>              iwEro;
	std::string                       err;

	try {
		myTnaSrc = src;
		isrc << myTnaSrc;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (src) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(src)");
	}
	try {
		myTnaDst = dst;
		idst << myTnaDst;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (dst) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(dst)");
	}
	try {
		myCallId = callId;
		icall_id << myCallId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callId) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callId)");
	}
	try {
		icall_info << callInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callInfo)");
	}
	try {
		myRecInfo = recInfo;
		irec_info << myRecInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (recInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(recInfo)");
	}
	try {
		myLspInfo = lspInfo;
		ilsp_info << myLspInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (lspInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(lspInfo)");
	}

	G2PCERA_DEBUG("Called callRoute into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::callRoute(G2PCERA_DEFAULT_INSTANCE,
					isrc,
					idst,
					icall_id,
					icall_info,
					irec_info,
					ilsp_info,
					iwEro,
					err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::eroSeq *   tmp;
				size_t                  size;

				size = iwEro.size();

				tmp = new g2mplsTypes::eroSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw G2PCERA::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "eroSeq");
				}
				wEro = tmp;
				wEro->length(size);

				if (size) {
					std::list<ero_hop_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = iwEro.begin(), i = 0;
					     iter != iwEro.end();
					     iter++, i++) {
						g2mplsTypes::eroItem_var hop;

						hop << *iter;
						wEro[i] = hop;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (wEro) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					ConnectionEroMismatch(myCallId,
							      "Error on "
							      "output "
							      "param "
							      "conversions "
							      "(wEro)");
			}

			G2PCERA_DEBUG("Successful callRoute in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface");
			G2PCERA_DEBUG("Returned ERO");
			G2PCERA_DEBUG(DUMP_BAR1);

			std::string prefix0 = DUMP_TAB;
			std::string prefix1 = prefix0 + DUMP_TAB;

			std::list<ero_hop_t>::const_iterator iter;

			G2PCERA_DEBUG("%s- ERO Hops",
				      prefix0.c_str());
			for (iter =  iwEro.begin();
			     iter != iwEro.end();
			     iter++) {
				logDump(prefix1, *iter);
				G2PCERA_DEBUG("%s-------------------"
					      "------------",
					      prefix1.c_str());
			}
			G2PCERA_DEBUG(DUMP_BAR2);

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_TNA_NOT_FOUND: {
			g2mplsTypes::tnaId_var myId;
			myId = new g2mplsTypes::tnaId;
			myId->ipv4(0);

			G2PCERA_DEBUG("Cannot get src or dst TNA for a "
				      "callRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchTna(myId, err.c_str()));
		}
			break;
		case PCERA_ERROR_CALL_PARAMS_MISMATCH: {

			G2PCERA_DEBUG("Problems with call params in a "
				      "callRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CallParamsMismatch(myCallId,
							   callInfo,
							   myRecInfo,
							   myLspInfo,
							   err.c_str()));
		}
			break;
		case PCERA_ERROR_NO_ROUTE: {

			G2PCERA_DEBUG("No route for issued "
				      "callRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::NoRoute(myCallId,
						err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Problems in callRoute in topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::callRouteAny(const g2mplsTypes::tnaResource&     src,
			const g2mplsTypes::gridParams&      dst,
			const g2mplsTypes::callIdent&       callId,
			const g2mplsTypes::callParams&      callInfo,
			const g2mplsTypes::recoveryParams&  recInfo,
			const g2mplsTypes::lspParams&       lspInfo,
			g2mplsTypes::eroSeq_out             wEro,
			g2mplsTypes::tnaResource &          eTna)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::tnaResource_var      myTnaSrc;
	g2mplsTypes::callIdent_var        myCallId;
	g2mplsTypes::recoveryParams_var   myRecInfo;
	g2mplsTypes::lspParams_var        myLspInfo;

	pceraErrorCode_t                  ret;
	net_res_spec_t                    isrc;
	grid_res_spec_t                   idst;
	call_ident_t                      icall_id;
	call_info_t                       icall_info;
	recovery_info_t                   irec_info;
	lsp_info_t                        ilsp_info;
	std::list<ero_hop_t>              iwEro;
	net_res_spec_t                    e_tna;
	std::string                       err;

	try {
		myTnaSrc = src;
		isrc << myTnaSrc;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (src) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(src)");
	}
	try {
		idst << dst;
	} catch (std::out_of_range & e) {
                G2PCERA_ERROR("Error on input param conversion (out_of_range): %s",
		              e.what());
                METOUT();
                STACK_UNLOCK();
                throw G2PCERA::InternalProblems("Error on input "
                                                "param conversions "
                                                "(dst)");
        } catch (std::invalid_argument & e) {
                G2PCERA_ERROR("Error on input param conversion (invalid_argument): %s",
                              e.what());
                METOUT();
                STACK_UNLOCK();
                throw G2PCERA::InternalProblems("Error on input "
                                                "param conversions "
                                                "(dst)");
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (dst) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(dst)");
	}
	try {
		myCallId = callId;
		icall_id << myCallId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callId) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callId)");
	}
	try {
		icall_info << callInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callInfo)");
	}
	try {
		myRecInfo = recInfo;
		irec_info << myRecInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (recInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(recInfo)");
	}
	try {
		myLspInfo = lspInfo;
		ilsp_info << myLspInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (lspInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(lspInfo)");
	}

	G2PCERA_DEBUG("Called callRouteAny into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::callRouteAny(G2PCERA_DEFAULT_INSTANCE,
					   isrc,
					   idst,
					   icall_id,
					   icall_info,
					   irec_info,
					   ilsp_info,
					   iwEro,
					   e_tna,
					   err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::eroSeq *   tmp;
				size_t                  size;

				size = iwEro.size();

				tmp = new g2mplsTypes::eroSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw G2PCERA::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "eroSeq");
				}
				wEro = tmp;
				wEro->length(size);

				if (size) {
					std::list<ero_hop_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = iwEro.begin(), i = 0;
					     iter != iwEro.end();
					     iter++, i++) {
						g2mplsTypes::eroItem_var hop;

						hop << *iter;
						wEro[i] = hop;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (wEro) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					ConnectionEroMismatch(myCallId,
							      "Error on "
							      "output "
							      "param "
							      "conversions "
							      "(wEro)");
			}
			try {
				g2mplsTypes::tnaResource_var tmp;
				tmp << e_tna;
				eTna = tmp;
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (eTna) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::InternalProblems("Error on "
								"output "
								"param "
								"conversions "
								"eTna)");
			}

			G2PCERA_DEBUG("Successful callRouteAny in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface");

			G2PCERA_DEBUG("Returned ERO");
			G2PCERA_DEBUG(DUMP_BAR1);

			std::string prefix0 = DUMP_TAB;
			std::string prefix1 = prefix0 + DUMP_TAB;

			std::list<ero_hop_t>::const_iterator iter;

			G2PCERA_DEBUG("%s- ERO Hops",
				      prefix0.c_str());
			for (iter  = iwEro.begin();
			     iter != iwEro.end();
			     iter++) {
				logDump(prefix1, *iter);
				G2PCERA_DEBUG("%s-------------------"
					      "------------",
					      prefix1.c_str());
			}
			G2PCERA_DEBUG(DUMP_BAR2);

			G2PCERA_DEBUG("Returned eTna");
			G2PCERA_DEBUG(DUMP_BAR1);
			logDump(prefix0, e_tna);
			G2PCERA_DEBUG(DUMP_BAR2);

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_TNA_NOT_FOUND: {
			g2mplsTypes::tnaId_var myId;
			myId = new g2mplsTypes::tnaId;
			myId->ipv4(0);

			G2PCERA_DEBUG("Cannot get src or dst TNA for a "
				      "callRouteAny into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchTna(myId, err.c_str()));
		}
			break;
		case PCERA_ERROR_CALL_PARAMS_MISMATCH: {

			G2PCERA_DEBUG("Problems with call params in a "
				      "callRouteAny into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CallParamsMismatch(myCallId,
							   callInfo,
							   myRecInfo,
							   myLspInfo,
							   err.c_str()));
		}
			break;
		case PCERA_ERROR_NO_ROUTE: {

			G2PCERA_DEBUG("No route for issued "
				      "callRouteAny into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::NoRoute(myCallId,
						err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Problems in callRouteAny in topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::callFlush(const g2mplsTypes::callIdent&  callId)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::callIdent_var        myCallId;

	pceraErrorCode_t                  ret;
	call_ident_t                      icall_id;
	std::string                       err;

	try {
		myCallId = callId;
		icall_id << myCallId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callId) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callId)");
	}

	G2PCERA_DEBUG("Called callFlush into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::callFlush(G2PCERA_DEFAULT_INSTANCE,
					icall_id,
					err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Successful callFlush in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface");
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_CALL_NOT_FOUND: {

			G2PCERA_DEBUG("Cannot flush not existing call with "
				      "callFlush into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchCall(myCallId,
							err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Problems in callFlush in topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::callConfirm(const g2mplsTypes::callIdent&  callId)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::callIdent_var        myCallId;

	pceraErrorCode_t                  ret;
	call_ident_t                      icall_id;
	std::string                       err;

	try {
		myCallId = callId;
		icall_id << myCallId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callId) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callId)");
	}

	G2PCERA_DEBUG("Called callConfirm into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::callConfirm(G2PCERA_DEFAULT_INSTANCE,
					  icall_id,
					  err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			G2PCERA_DEBUG("Successful callConfirm in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface");
			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_CALL_NOT_FOUND: {

			G2PCERA_DEBUG("Cannot confirm not existing call with "
				      "callConfirm into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::CannotFetchCall(myCallId,
							err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Problems in callConfirm in topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::connectionRoute(const g2mplsTypes::connEndPoint& src,
			   const g2mplsTypes::connEndPoint& dst,
			   const g2mplsTypes::callIdent&    callId,
			   const g2mplsTypes::lspParams&    lspInfo,
			   g2mplsTypes::eroSeq_out          wEro,
			   g2mplsTypes::eroSeq_out          pEro)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::connEndPoint_var     myCEPSrc;
	g2mplsTypes::connEndPoint_var     myCEPDst;
	g2mplsTypes::callIdent_var        myCallId;
	g2mplsTypes::lspParams_var        myLspInfo;

	pceraErrorCode_t             ret;
	ero_hop_t                    isrc;
	ero_hop_t                    idst;
	call_ident_t                 icall_id;
	lsp_info_t                   ilsp_info;
	std::list<ero_hop_t>         iwEro;
	std::list<ero_hop_t>         ipEro;
	std::string                  err;

	iwEro.clear();
	ipEro.clear();

	try {
		myCEPSrc = src;
		isrc << myCEPSrc;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (src) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(src)");
	}
	try {
		myCEPDst = dst;
		idst << myCEPDst;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (dst) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(dst)");
	}
	try {
		myCallId = callId;
		icall_id << myCallId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callId) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callId)");
	}
	try {
		myLspInfo = lspInfo;
		ilsp_info << myLspInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (lspInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(lspInfo)");
	}

	G2PCERA_DEBUG("Called connectionRoute into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::connectionRoute(G2PCERA_DEFAULT_INSTANCE,
					      isrc,
					      idst,
					      icall_id,
					      ilsp_info,
					      iwEro,
					      ipEro,
					      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::eroSeq *   tmp;
				size_t                  size;

				size = iwEro.size();

				tmp = new g2mplsTypes::eroSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw G2PCERA::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "eroSeq");
				}
				wEro = tmp;
				wEro->length(size);

				if (size) {
					std::list<ero_hop_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = iwEro.begin(), i = 0;
					     iter != iwEro.end();
					     iter++, i++) {
						g2mplsTypes::eroItem_var hop;

						hop << *iter;
						wEro[i] = hop;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (wEro) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					ConnectionEroMismatch(myCallId,
							      "Error on "
							      "output "
							      "param "
							      "conversions "
							      "(wEro)");
			}

			try {
				g2mplsTypes::eroSeq *   tmp;
				size_t                  size;

				size = ipEro.size();

				tmp = new g2mplsTypes::eroSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw G2PCERA::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "eroSeq");
				}
				pEro = tmp;
				pEro->length(size);

				if (size) {
					std::list<ero_hop_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = ipEro.begin(), i = 0;
					     iter != ipEro.end();
					     iter++, i++) {
						g2mplsTypes::eroItem_var hop;

						hop << *iter;
						pEro[i] = hop;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (pEro) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					ConnectionEroMismatch(myCallId,
							      "Error on "
							      "output "
							      "param "
							      "conversions "
							      "(pEro)");
			}

			G2PCERA_DEBUG("Successful connectionRoute in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface");

			G2PCERA_DEBUG("Returned primary ERO");
			G2PCERA_DEBUG(DUMP_BAR1);

			std::string prefix0 = DUMP_TAB;
			std::string prefix1 = prefix0 + DUMP_TAB;

			std::list<ero_hop_t>::const_iterator iter;

			G2PCERA_DEBUG("%s- ERO Hops",
				      prefix0.c_str());
			for (iter  = iwEro.begin();
			     iter != iwEro.end();
			     iter++) {
				logDump(prefix1, *iter);
				G2PCERA_DEBUG("%s-------------------"
					      "------------",
					      prefix1.c_str());
			}
			G2PCERA_DEBUG(DUMP_BAR2);

			G2PCERA_DEBUG("Returned backup ERO");
			G2PCERA_DEBUG(DUMP_BAR1);

			G2PCERA_DEBUG("%s- ERO Hops",
				      prefix0.c_str());
			for (iter  = ipEro.begin();
			     iter != ipEro.end();
			     iter++) {
				logDump(prefix1, *iter);
				G2PCERA_DEBUG("%s-------------------"
					      "------------",
					      prefix1.c_str());
			}
			G2PCERA_DEBUG(DUMP_BAR2);

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_CALL_PARAMS_MISMATCH:
		case PCERA_ERROR_CONN_PARAMS_MISMATCH: {

			G2PCERA_DEBUG("Problems with connection params in a "
				      "connectionRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::ConnectionParamsMismatch(myCallId,
								 myLspInfo,
								 err.c_str()));
		}
			break;
		case PCERA_ERROR_NO_ROUTE: {

			G2PCERA_DEBUG("No route for issued "
				      "connectionRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::NoRoute(myCallId,
						err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Problems in connectionRoute in topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}

CORBA::Boolean
G2PCERA_i::connectionRouteDisjoint(const g2mplsTypes::connEndPoint& src,
				   const g2mplsTypes::connEndPoint& dst,
				   const g2mplsTypes::callIdent& callId,
				   const g2mplsTypes::lspParams& lspInfo,
				   const g2mplsTypes::multiEroSeq& multiXro,
				   g2mplsTypes::eroSeq_out wEro,
				   g2mplsTypes::eroSeq_out pEro)
{
	STACK_LOCK();
	METIN();

	g2mplsTypes::connEndPoint_var     myCEPSrc;
	g2mplsTypes::connEndPoint_var     myCEPDst;
	g2mplsTypes::callIdent_var        myCallId;
	g2mplsTypes::lspParams_var        myLspInfo;

	pceraErrorCode_t             ret;
	ero_hop_t                    isrc;
	ero_hop_t                    idst;
	call_ident_t                 icall_id;
	lsp_info_t                   ilsp_info;
	std::list<ero_hop_t>         iwEro;
	std::list<ero_hop_t>         ipEro;
	std::string                  err;

	std::list< std::list<ero_hop_t> > iMXro;

	iMXro.clear();
	iwEro.clear();
	ipEro.clear();

	try {
		myCEPSrc = src;
		isrc << myCEPSrc;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (src) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(src)");
	}
	try {
		myCEPDst = dst;
		idst << myCEPDst;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (dst) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(dst)");
	}
	try {
		myCallId = callId;
		icall_id << myCallId;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (callId) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(callId)");
	}
	try {
		myLspInfo = lspInfo;
		ilsp_info << myLspInfo;
	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (lspInfo) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::InternalProblems("Error on input "
						"param conversions "
						"(lspInfo)");
	}
	try {
		size_t i;

		for (i = 0; i < multiXro.length(); i++) {
			size_t               j;
			std::list<ero_hop_t> iXro;

			iXro.clear();

			for (j = 0; j < multiXro[i].length(); j++) {

				ero_hop_t                ihop;
				g2mplsTypes::eroItem_var hop;

				hop = multiXro[i][j];
				ihop << hop;
				iXro.push_back(ihop);
			}

			iMXro.push_back(iXro);
		}

	} catch (...) {
		G2PCERA_ERROR("Error on input param conversions (multiXro) "
			      "for %s....",
			      __PRETTY_FUNCTION__);
		METOUT();
		STACK_UNLOCK();
		throw G2PCERA::ConnectionEroMismatch(myCallId,
						     "Error on input "
						     "param conversions "
						     "(excludeEro)");
	}

	G2PCERA_DEBUG("Called connectionRoute into default topology "
		      "through CORBA G2PCERA interface");

	ret = PCERA_CALC_API::connectionRouteDisjoint(G2PCERA_DEFAULT_INSTANCE,
						      isrc,
						      idst,
						      icall_id,
						      ilsp_info,
						      iMXro,
						      iwEro,
						      ipEro,
						      err);

	switch (ret) {
		case PCERA_ERROR_NONE: {
			try {
				g2mplsTypes::eroSeq *   tmp;
				size_t                  size;

				size = iwEro.size();

				tmp = new g2mplsTypes::eroSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw G2PCERA::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "eroSeq");
				}
				wEro = tmp;
				wEro->length(size);

				if (size) {
					std::list<ero_hop_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = iwEro.begin(), i = 0;
					     iter != iwEro.end();
					     iter++, i++) {
						g2mplsTypes::eroItem_var hop;

						hop << *iter;
						wEro[i] = hop;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (wEro) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					ConnectionEroMismatch(myCallId,
							      "Error on "
							      "output "
							      "param "
							      "conversions "
							      "(wEro)");
			}

			try {
				g2mplsTypes::eroSeq *   tmp;
				size_t                  size;

				size = ipEro.size();

				tmp = new g2mplsTypes::eroSeq(size);
				if (!tmp) {
					METOUT();
					STACK_UNLOCK();
					throw G2PCERA::
						InternalProblems("Cannot "
								 "allocate "
								 "g2mplsTypes::"
								 "eroSeq");
				}
				pEro = tmp;
				pEro->length(size);

				if (size) {
					std::list<ero_hop_t>::
						const_iterator  iter;
					size_t                  i;

					for (iter  = ipEro.begin(), i = 0;
					     iter != ipEro.end();
					     iter++, i++) {
						g2mplsTypes::eroItem_var hop;

						hop << *iter;
						pEro[i] = hop;
					}
				}
			} catch (...) {
				G2PCERA_ERROR("Error on output param "
					      "conversions (pEro) "
					      "for %s....",
					      __PRETTY_FUNCTION__);
				METOUT();
				STACK_UNLOCK();
				throw G2PCERA::
					ConnectionEroMismatch(myCallId,
							      "Error on "
							      "output "
							      "param "
							      "conversions "
							      "(pEro)");
			}

			G2PCERA_DEBUG("Successful connectionRoute in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "through CORBA G2PCERA interface");

			G2PCERA_DEBUG("Returned primary ERO");
			G2PCERA_DEBUG(DUMP_BAR1);

			std::string prefix0 = DUMP_TAB;
			std::string prefix1 = prefix0 + DUMP_TAB;

			std::list<ero_hop_t>::const_iterator iter;

			G2PCERA_DEBUG("%s- ERO Hops",
				      prefix0.c_str());
			for (iter  = iwEro.begin();
			     iter != iwEro.end();
			     iter++) {
				logDump(prefix1, *iter);
				G2PCERA_DEBUG("%s-------------------"
					      "------------",
					      prefix1.c_str());
			}
			G2PCERA_DEBUG(DUMP_BAR2);

			G2PCERA_DEBUG("Returned backup ERO");
			G2PCERA_DEBUG(DUMP_BAR1);

			G2PCERA_DEBUG("%s- ERO Hops",
				      prefix0.c_str());
			for (iter  = ipEro.begin();
			     iter != ipEro.end();
			     iter++) {
				logDump(prefix1, *iter);
				G2PCERA_DEBUG("%s-------------------"
					      "------------",
					      prefix1.c_str());
			}
			G2PCERA_DEBUG(DUMP_BAR2);

			METOUT();
			STACK_UNLOCK();
			return true;
		}
			break;
		case PCERA_ERROR_CALL_PARAMS_MISMATCH:
		case PCERA_ERROR_CONN_PARAMS_MISMATCH: {

			G2PCERA_DEBUG("Problems with connection params in a "
				      "connectionRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::ConnectionParamsMismatch(myCallId,
								 myLspInfo,
								 err.c_str()));
		}
			break;
		case PCERA_ERROR_NO_ROUTE: {

			G2PCERA_DEBUG("No route for issued "
				      "connectionRoute into topology in "
				      "G2.PCE-RA DEFAULT INSTANCE "
				      "(err: code=%s, reason='%s')",
				      PCERA_ERR_TO_STRING(ret),
				      err.c_str());
			METOUT();
			STACK_UNLOCK();
			throw (G2PCERA::NoRoute(myCallId,
						err.c_str()));
		}
			break;
		default:
			// nothing special to do... exit with InternalProblems
			break;
	}

	G2PCERA_DEBUG("Problems in connectionRoute in topology for "
		      "G2.PCE-RA DEFAULT INSTANCE "
		      "(err: code=%s, reason='%s')",
		      PCERA_ERR_TO_STRING(ret),
		      err.c_str());
	METOUT();
	STACK_UNLOCK();
	throw (G2PCERA::InternalProblems(err.c_str()));
}


#endif // HAVE_OMNIORB
